# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople

a1              = 0.000423193      # +/- 2.88e-06     (0.6804%)
a3              = 0.00238104       # +/- 1.166e-05    (0.4898%)
a5              = 0.00932382       # +/- 3.74e-05     (0.4011%)
a7              = 0.0294491        # +/- 0.0001004    (0.341%)
a9              = 0.0781823        # +/- 0.0002244    (0.287%)
a11             = 0.172174         # +/- 0.0003875    (0.2251%)
a13             = 0.293805         # +/- 0.000427     (0.1453%)
a15             = 0.335432         # +/- 0.0003756    (0.112%)
a17             = 0.193157         # +/- 0.0005909    (0.3059%)
a19             = 0.0306509        # +/- 0.0002106    (0.6871%)

a2              = 272.598          # +/- 1.686        (0.6185%)
a4              = 49.9921          # +/- 0.2078       (0.4156%)
a6              = 13.9982          # +/- 0.04687      (0.3348%)
a8              = 4.82202          # +/- 0.01388      (0.2879%)
a10             = 1.89086          # +/- 0.004839     (0.2559%)
a12             = 0.811918         # +/- 0.001881     (0.2316%)
a14             = 0.373021         # +/- 0.0007885    (0.2114%)
a16             = 0.180375         # +/- 0.0003476    (0.1927%)
a18             = 0.0902968        # +/- 0.0001569    (0.1737%)
a20             = 0.0452878        # +/- 6.913e-05    (0.1527%)


f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)
f8(x)   =  a15 * N * ( a16 )**(0.75) * x * exp(-a16 * x**2)
f9(x)   =  a17 * N * ( a18 )**(0.75) * x * exp(-a18 * x**2)
f10(x)  =  a19 * N * ( a20 )**(0.75) * x * exp(-a20 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x) + f8(x) + f9(x) + f10(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a16, a18, a20, a1, a3, a5, a7, a9, a11, a13, a15, a17, a19
