# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople

a2              = 86.3188          # +/- 0.3661       (0.4241%)
a4              = 24.1714          # +/- 0.08261      (0.3418%)
a6              = 8.3272           # +/- 0.02448      (0.294%)
a8              = 3.26592          # +/- 0.008546     (0.2617%)
a10             = 1.4029           # +/- 0.003332     (0.2375%)
a12             = 0.645159         # +/- 0.001406     (0.2179%)
a14             = 0.312839         # +/- 0.0006279    (0.2007%)
a16             = 0.158031         # +/- 0.0002912    (0.1842%)
a18             = 0.0820132        # +/- 0.000137     (0.167%)
a20             = 0.0423967        # +/- 6.258e-05    (0.1476%)
a22             = 470.652          # +/- 2.97         (0.6311%)
a1              = 0.00120548       # +/- 6.035e-06    (0.5006%)
a3              = 0.00474563       # +/- 1.954e-05    (0.4118%)
a5              = 0.0152023        # +/- 5.384e-05    (0.3542%)
a7              = 0.041733         # +/- 0.0001281    (0.307%)
a9              = 0.0988778        # +/- 0.0002555    (0.2584%)
a11             = 0.196123         # +/- 0.0003885    (0.1981%)
a13             = 0.301069         # +/- 0.0003687    (0.1225%)
a15             = 0.307092         # +/- 0.0003925    (0.1278%)
a17             = 0.156783         # +/- 0.0005315    (0.339%)
a19             = 0.0219433        # +/- 0.0001579    (0.7194%)
a21             = 0.000213901      # +/- 1.486e-06    (0.6945%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)
f8(x)   =  a15 * N * ( a16 )**(0.75) * x * exp(-a16 * x**2)
f9(x)   =  a17 * N * ( a18 )**(0.75) * x * exp(-a18 * x**2)
f10(x)  =  a19 * N * ( a20 )**(0.75) * x * exp(-a20 * x**2)
f11(x)  =  a21 * N * ( a22 )**(0.75) * x * exp(-a22 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x) + f8(x) + f9(x) + f10(x) + f11(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a16, a18, a20, a22, a1, a3, a5, a7, a9, a11, a13, a15, a17, a19, a21
