# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople

a2              = 145.909          # +/- 0.6303       (0.432%)
a4              = 40.8612          # +/- 0.1423       (0.3481%)
a6              = 14.078           # +/- 0.04218      (0.2996%)
a8              = 5.52199          # +/- 0.01473      (0.2668%)
a10             = 2.37248          # +/- 0.005753     (0.2425%)
a12             = 1.09152          # +/- 0.002436     (0.2232%)
a14             = 0.529845         # +/- 0.001096     (0.2068%)
a16             = 0.268463         # +/- 0.000515     (0.1918%)
a18             = 0.140649         # +/- 0.000249     (0.177%)
a20             = 0.0752618        # +/- 0.0001213    (0.1612%)
a22             = 0.0399377        # +/- 5.718e-05    (0.1432%)
a24             = 795.454          # +/- 5.113        (0.6427%)
a1              = 0.000626147      # +/- 3.196e-06    (0.5104%)
a3              = 0.00247224       # +/- 1.04e-05     (0.4208%)
a5              = 0.0079826        # +/- 2.908e-05    (0.3643%)
a7              = 0.0223295        # +/- 7.163e-05    (0.3208%)
a9              = 0.0551259        # +/- 0.0001544    (0.2801%)
a11             = 0.118993         # +/- 0.0002785    (0.234%)
a13             = 0.215639         # +/- 0.000376     (0.1744%)
a15             = 0.301258         # +/- 0.0003176    (0.1054%)
a17             = 0.277557         # +/- 0.0004114    (0.1482%)
a19             = 0.127044         # +/- 0.0004699    (0.3699%)
a21             = 0.015857         # +/- 0.0001188    (0.7491%)
a23             = 0.00011101       # +/- 7.854e-07    (0.7075%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)
f8(x)   =  a15 * N * ( a16 )**(0.75) * x * exp(-a16 * x**2)
f9(x)   =  a17 * N * ( a18 )**(0.75) * x * exp(-a18 * x**2)
f10(x)  =  a19 * N * ( a20 )**(0.75) * x * exp(-a20 * x**2)
f11(x)  =  a21 * N * ( a22 )**(0.75) * x * exp(-a22 * x**2)
f12(x)  =  a23 * N * ( a24 )**(0.75) * x * exp(-a24 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x) + f8(x) + f9(x) + f10(x) + f11(x) + f12(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a16, a18, a20, a22, a24, a1, a3, a5, a7, a9, a11, a13, a15, a17, a19, a21, a23
