# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople

a2              = 1318.64          # +/- 8.61         (0.653%)
a4              = 241.966          # +/- 1.062        (0.4391%)
a6              = 67.7724          # +/- 0.2399       (0.3539%)
a8              = 23.3524          # +/- 0.07114      (0.3046%)
a10             = 9.16077          # +/- 0.02486      (0.2714%)
a12             = 3.93641          # +/- 0.009718     (0.2469%)
a14             = 1.81147          # +/- 0.004123     (0.2276%)
a16             = 0.87975          # +/- 0.001861     (0.2116%)
a18             = 0.446279         # +/- 0.0008812    (0.1975%)
a20             = 0.23457          # +/- 0.0004319    (0.1841%)
a22             = 0.126763         # +/- 0.0002163    (0.1706%)
a24             = 0.0696489        # +/- 0.0001087    (0.156%)
a26             = 0.0378161        # +/- 5.261e-05    (0.1391%)
a1              = 5.90049e-05      # +/- 4.243e-07    (0.7191%)
a3              = 0.000332902      # +/- 1.728e-06    (0.5191%)
a5              = 0.0013165        # +/- 5.643e-06    (0.4286%)
a7              = 0.00426997       # +/- 1.59e-05     (0.3724%)
a9              = 0.0120736        # +/- 3.995e-05    (0.3309%)
a11             = 0.0305191        # +/- 8.997e-05    (0.2948%)
a13             = 0.0691624        # +/- 0.0001781    (0.2576%)
a15             = 0.137887         # +/- 0.000293     (0.2125%)
a17             = 0.230654         # +/- 0.0003537    (0.1533%)
a19             = 0.295879         # +/- 0.0002806    (0.09484%)
a21             = 0.248409         # +/- 0.0004231    (0.1703%)
a23             = 0.102878         # +/- 0.0004102    (0.3987%)
a25             = 0.0115571        # +/- 8.972e-05    (0.7764%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)
f8(x)   =  a15 * N * ( a16 )**(0.75) * x * exp(-a16 * x**2)
f9(x)   =  a17 * N * ( a18 )**(0.75) * x * exp(-a18 * x**2)
f10(x)  =  a19 * N * ( a20 )**(0.75) * x * exp(-a20 * x**2)
f11(x)  =  a21 * N * ( a22 )**(0.75) * x * exp(-a22 * x**2)
f12(x)  =  a23 * N * ( a24 )**(0.75) * x * exp(-a24 * x**2)
f13(x)  =  a25 * N * ( a26 )**(0.75) * x * exp(-a26 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x) + f8(x) + f9(x) + f10(x) + f11(x) + f12(x) + f13(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a16, a18, a20, a22, a24, a26, a1, a3, a5, a7, a9, a11, a13, a15, a17, a19, a21, a23, a25
