# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople
a2              = 23.103           # +/- 0.1285       (0.5562%)
a4              = 4.23592          # +/- 0.01582      (0.3735%)
a6              = 1.18508          # +/- 0.00355      (0.2995%)
a8              = 0.407118         # +/- 0.001034     (0.2539%)
a10             = 0.158098         # +/- 0.0003449    (0.2182%)
a12             = 0.0651133        # +/- 0.00012      (0.1843%)

# sto-6g linear parameters - pople
a1              = 0.00916361       # +/- 5.564e-05    (0.6072%)
a3              = 0.0493611        # +/- 0.0002082    (0.4218%)
a5              = 0.168531         # +/- 0.0005158    (0.306%)
a7              = 0.370543         # +/- 0.0006658    (0.1797%)
a9              = 0.416497         # +/- 0.0006579    (0.158%)
a11             = 0.130355         # +/- 0.0006842    (0.5249%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a1, a3, a5, a7, a9, a11

plot [7:]"sto-1s.dat" using 1:2 w l, fit_gto(x)

pause -1
