# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople
a2              = 45.0597          # +/- 0.2584       (0.5734%)
a4              = 8.26444          # +/- 0.03186      (0.3855%)
a6              = 2.3137           # +/- 0.007174     (0.3101%)
a8              = 0.796312         # +/- 0.002111     (0.2651%)
a10             = 0.311341         # +/- 0.0007222    (0.232%)
a12             = 0.132305         # +/- 0.0002687    (0.2031%)
a14             = 0.0582444        # +/- 0.0001013    (0.174%)

# sto-6g linear parameters - pople
a1              = 0.00399632       # +/- 2.513e-05    (0.6287%)
a3              = 0.0220246        # +/- 9.813e-05    (0.4455%)
a5              = 0.0808699        # +/- 0.0002796    (0.3458%)
a7              = 0.215572         # +/- 0.0005413    (0.2511%)
a9              = 0.383323         # +/- 0.0005313    (0.1386%)
a11             = 0.349934         # +/- 0.0006828    (0.1951%)
a13             = 0.0890468        # +/- 0.00051      (0.5727%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a1, a3, a5, a7, a9, a11, a13
