# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople
a2              = 84.5601          # +/- 0.498        (0.5889%)
a4              = 15.5117          # +/- 0.06144      (0.3961%)
a6              = 4.34408          # +/- 0.01386      (0.3191%)
a8              = 1.49623          # +/- 0.004098     (0.2739%)
a10             = 0.586234         # +/- 0.001419     (0.242%)
a12             = 0.250993         # +/- 0.0005419    (0.2159%)
a14             = 0.114128         # +/- 0.0002186    (0.1915%)
a16             = 0.052947         # +/- 8.78e-05     (0.1658%)

# sto-6g linear parameters - pople
a1              = 0.00182408       # +/- 1.18e-05     (0.6471%)
a3              = 0.0101664        # +/- 4.707e-05    (0.463%)
a5              = 0.0386862        # +/- 0.0001434    (0.3708%)
a7              = 0.113363         # +/- 0.0003349    (0.2954%)
a9              = 0.252219         # +/- 0.0005264    (0.2087%)
a11             = 0.377702         # +/- 0.0004298    (0.1138%)
a13             = 0.289314         # +/- 0.0006774    (0.2341%)
a15             = 0.0616196        # +/- 0.0003795    (0.6158%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)
f8(x)   =  a15 * N * ( a16 )**(0.75) * x * exp(-a16 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x) + f8(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a16, a1, a3, a5, a7, a9, a11, a13, a15
