# constants
FIT_LIMIT   = 1e-10              # fitting tolerance
FIT_MAXITER = 0                  # no limit for number of iterations
pi          = 3.141592654        # value of pi
N           = (2.0/pi)**(0.75)   # 1s normalization constant

# sto-6g exponent parameters - pople

a2              = 153.699          # +/- 0.9272       (0.6033%)
a4              = 28.1964          # +/- 0.1144       (0.4058%)
a6              = 7.89755          # +/- 0.02584      (0.3273%)
a8              = 2.72103          # +/- 0.007658     (0.2814%)
a10             = 1.06697          # +/- 0.002665     (0.2498%)
a12             = 0.457887         # +/- 0.001031     (0.2251%)
a14             = 0.209887         # +/- 0.0004269    (0.2034%)
a16             = 0.100649         # +/- 0.0001833    (0.1821%)
a18             = 0.0487262        # +/- 7.747e-05    (0.159%)
a1              = 0.000865426      # +/- 5.743e-06    (0.6636%)
a3              = 0.00485149       # +/- 2.315e-05    (0.4771%)
a5              = 0.0188031        # +/- 7.297e-05    (0.3881%)
a7              = 0.0578439        # +/- 0.0001868    (0.323%)
a9              = 0.144379         # +/- 0.0003712    (0.2571%)
a11             = 0.27804          # +/- 0.0004843    (0.1742%)
a13             = 0.360054         # +/- 0.0003807    (0.1057%)
a15             = 0.236939         # +/- 0.0006438    (0.2717%)
a17             = 0.0431761        # +/- 0.0002826    (0.6545%)

f1(x)   =  a1  * N * ( a2  )**(0.75) * x * exp(-a2  * x**2)
f2(x)   =  a3  * N * ( a4  )**(0.75) * x * exp(-a4  * x**2)
f3(x)   =  a5  * N * ( a6  )**(0.75) * x * exp(-a6  * x**2)
f4(x)   =  a7  * N * ( a8  )**(0.75) * x * exp(-a8  * x**2)
f5(x)   =  a9  * N * ( a10 )**(0.75) * x * exp(-a10 * x**2)
f6(x)   =  a11 * N * ( a12 )**(0.75) * x * exp(-a12 * x**2)
f7(x)   =  a13 * N * ( a14 )**(0.75) * x * exp(-a14 * x**2)
f8(x)   =  a15 * N * ( a16 )**(0.75) * x * exp(-a16 * x**2)
f9(x)   =  a17 * N * ( a18 )**(0.75) * x * exp(-a18 * x**2)

fit_gto(x) = f1(x) + f2(x) + f3(x) + f4(x) + f5(x) + f6(x) + f7(x) + f8(x) + f9(x)

fit fit_gto(x) "sto-1s.dat" using 1:2 via a2, a4, a6, a8, a10, a12, a14, a16, a18, a1, a3, a5, a7, a9, a11, a13, a15, a17
