!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module address_tools_mod

  use control_mod

  integer(dpi), external :: address  

! Description : exe_by_address_n  C routine to execute a Fortran routine 
! Description :                   ( with n arguments n <= routine_arguments_max_number)
! Description :                   from its address 

  external exe_by_address_0, exe_by_address_1, exe_by_address_2, &
           exe_by_address_3, exe_by_address_4, exe_by_address_5, &
           exe_by_address_6, exe_by_address_7, exe_by_address_8

! Owner   : R. Assaraf ra@lct.jussieu.fr
 
  contains

! ==========================================================================
  subroutine exe_sub_byadr (subroutine_address, addresses_list, n_arg)
! --------------------------------------------------------------------------
!
! Name           :  exe_sub_byadr
!
! Argument (in ) : subroutine_adres
! Argument (out) : addresses_list
! Description    : call exe_sub_byadr(address(sub), (/address(ob1),.../))
! Description    : is equivalent to call sub(ob1,ob2,...)
! Description    : the number of arguments ob1, ob2 can be variable. 
!
! Authors: R. Assaraf
! Date   : 15 Dec 2001
! --------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)          :: subroutine_address
  integer(dpi)          :: n_arg
  integer(dpi)          :: addresses_list(n_arg)

! local:
! integer(dpi), save :: address_by_nb_arg(0:8)=-1
! integer(dpi), save  :: nb_arg_max=8 
! integer(dpi) :: nb_arg

! begin:
! first realization:

!    if (address_by_nb_arg (0) == -1) then  ! address_by_nb_arg not initialized
!     address_by_nb_arg (0) = address (exe_by_address_0)
!      address_by_nb_arg (1) = address (exe_by_address_1)
!      address_by_nb_arg (2) = address (exe_by_address_2)
!      address_by_nb_arg (3) = address (exe_by_address_3)
!      address_by_nb_arg (4) = address (exe_by_address_4)
!      address_by_nb_arg (5) = address (exe_by_address_5)
!      address_by_nb_arg (6) = address (exe_by_address_6)
!      address_by_nb_arg (7) = address (exe_by_address_7)
!      address_by_nb_arg (8) = address (exe_by_address_8)
!      write (logf, *) 'initialisation des addresses exe_by_address_'
!   endif
!      nb_arg = size (addresses_list)
!     if (nb_arg>nb_arg_max.or.nb_arg<0) then
!       write (logf, *) 'exe_on_sub_adr_arg_adr: nb_arg = ', nb_arg,'should be in [0, 8]'
!       stop
!    endif
!     call exe(address_by_nb_arg (n_arg) , subroutine_address,addresses_list)

! this realization is more efficient
    if (n_arg < 4) then
       if (n_arg < 2) then
          if (n_arg == 0) then
             call exe_by_address_0 (subroutine_address)
          else
             call exe_by_address_1 (subroutine_address, addresses_list)
          endif
       else if (n_arg == 2) then 
          call exe_by_address_2 (subroutine_address, addresses_list)
       else
          call exe_by_address_3 (subroutine_address, addresses_list)
       endif
    else if (n_arg <6 ) then
       if (n_arg == 4 ) then 
          call exe_by_address_4 (subroutine_address, addresses_list)
       else
          call exe_by_address_5 (subroutine_address, addresses_list)
       endif
    else if (n_arg == 6) then
       call exe_by_address_6 (subroutine_address, addresses_list)
    else if (n_arg == 7) then
       call exe_by_address_7 (subroutine_address, addresses_list)
    else if (n_arg == 8) then
       call exe_by_address_8 (subroutine_address, addresses_list)
    else 
      write (logf, *) 'exe_sub_byadr-e: nb of arguments ', n_arg, &
              ' exceeds routine_arguments_max_number = ', routine_arguments_max_number
      stop 'exe_sub_byadr-e: routine_arguments_max_number exceeded'
    endif
 
  end subroutine exe_sub_byadr

! ==========================================================================
  subroutine exe_on_adr_list(addresses_list,n_arg)
! --------------------------------------------------------------------------
!
! Name           :  exe_on_adr_list
!
! Argument (in ) : addresses_list
! Argument (in ) : n_arg
! Description    : call exe_on_adr_list(/address(ob1),.../))
! Description    : is equivalent to call ob1(ob2,ob3,...)
! Description    : n_arg must be the dimension of addresses_list
! Description    : and ob_1 must be a subroutine.
!
! Authors: R. Assaraf
! Date   : 25 Jan 2001
! --------------------------------------------------------------------------
  implicit none
! i/o:

  integer(dpi)          :: n_arg
  integer(dpi)          :: addresses_list(n_arg)

! local:

! begin:
    if (n_arg<5) then
       if (n_arg<3) then
          if (n_arg == 1) then
             call exe_by_address_0(addresses_list (1) )
          else
             call exe_by_address_1(addresses_list(1) , addresses_list (2:2) )
          endif
       else if (n_arg == 3) then 
          call exe_by_address_2(addresses_list(1) , addresses_list (2:3) )
       else
          call exe_by_address_3(addresses_list(1) , addresses_list (2:4 ) )
       endif
    else if (n_arg<7) then
       if (n_arg == 5) then 
          call exe_by_address_4(addresses_list(1) , addresses_list (2:5) )
       else
          call exe_by_address_5(addresses_list(1) , addresses_list (2:6) )
       endif
    else if (n_arg == 7) then
       call exe_by_address_6(addresses_list(1) , addresses_list (2:7) )
    else if (n_arg == 7) then
       call exe_by_address_7(addresses_list(1) , addresses_list (2:8) )
    else if (n_arg == 9) then
       call exe_by_address_8(addresses_list(1) , addresses_list (2:9) )
    else 
      write (logf, *) 'n_arg out of range'
    endif
 
  end subroutine exe_on_adr_list

end module address_tools_mod
