! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module aos_bywlk_mod

  use basic_types_tools_mod
  use mos_mod
  use elec_xpfs_rexp_bywlk_mod
  use mo_ao_s_coeffs_mod
  real(dp), pointer :: aos_radial_bywlk (:,:,:,:) => NULL ()
contains 

! ==========================================================================
  subroutine aos_radial_bywlk_bld
! --------------------------------------------------------------------------
!
! Name     : aos_radial_bywlk_bld
!
! Creates :  aos_radial_bywlk (elec, mo, ao, walk)
!
! Needs   : elec_xpfs_rexp_bywlk
! Author: R. Assaraf
! Date  : 09 May 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: electrons_nb,  walkers_nb, xpfs_c_nb, aos_nb, mos_nb
  integer(dpi) :: walk_i, elec_i, ao_i, xpf_c_i, mo_i
  integer(dpi) :: xp1
! header:

  if (header_exe) then
 
     l_here = 'aos_radial_bywlk_bld'
     call object_create ('aos_radial_bywlk')
  

     call object_needed ('mos_to_xpfs_c')
     call object_needed ('elec_xpfs_rexp_bywlk')
     call object_needed ('ao_first_in_xpfs_c')
     call object_needed ('xpfs_c')
     return
  endif

! dimensions 
  electrons_nb = size (elec_xpfs_rexp_bywlk, dim = 1)
  xpfs_c_nb    = size (elec_xpfs_rexp_bywlk, dim = 2)
  walkers_nb   = size (elec_xpfs_rexp_bywlk, dim =3)
  aos_nb       = size (ao_first_in_xpfs_c)
  mos_nb       = size (mos_to_xpfs_c, dim=1)

  if (.not.associated (aos_radial_bywlk)) then 
     call alloc ('aos_radial_bywlk',aos_radial_bywlk, electrons_nb, &
          mos_nb, aos_nb, walkers_nb)
  endif

  aos_radial_bywlk = 0.d0
  do ao_i = 1, aos_nb
     xp1 = ao_first_in_xpfs_c (ao_i)
     do xpf_c_i =1, xpfs_c_nb
        if ( xpfs_c (xpf_c_i)%center ==  xpfs_c (xp1)%center  &
             .and. xpfs_c (xpf_c_i)%x_power == xpfs_c (xp1)%x_power &
             .and. xpfs_c (xpf_c_i)%y_power == xpfs_c (xp1)%y_power &
             .and. xpfs_c (xpf_c_i)%z_power == xpfs_c (xp1)%z_power )  then

           
           do walk_i = 1, walkers_nb
              do mo_i = 1, mos_nb
                 do elec_i = 1, electrons_nb
                    aos_radial_bywlk ( elec_i, mo_i, ao_i, walk_i) = &
                         aos_radial_bywlk ( elec_i, mo_i, ao_i, walk_i) + &
                         mos_to_xpfs_c (mo_i,xpf_c_i)* &
                         elec_xpfs_rexp_bywlk (elec_i, xpf_c_i,walk_i)

                 enddo ! do elec
              enddo ! do mo_i
           enddo ! do walk_i

        endif
    enddo ! do xpf_c_i
 enddo ! do ao_i


end subroutine aos_radial_bywlk_bld
end module aos_bywlk_mod
