! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module arrays_mod 

  use control_mod
  use strings_mod
  use tools_mod

  implicit none

  interface printarray
     module procedure        &
       array_string_print,   &
       array_integer_print,  &
       array_boolean_print,  &
       array_double_print,   &
       matrix_integer_print, &
       matrix_double_print
  end interface 

  contains

! ========================================================================

  subroutine array_string_print (rout_name, array, arrayname)
  implicit none
! i/o:
  character(len=*)           :: rout_name
  character(len=*)           :: arrayname
  character(len=*), pointer  :: array(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: dimen, i

! begin:
  l_here ='array_string_print'
  dimen   = size(array)
  message = l_here*' array '*arrayname*' is empty'
  if(dimen .eq. 0)  then
    call warning (l_here, message) 
  else
   do i = 1, dimen
    write (logf,'(1x,4a,i3,2a)') &
    trim(rout_name),': array ',trim(arrayname),' elem ',i,' name ',trim(array(i))
   end do
  end if
  
  end subroutine array_string_print

  subroutine array_integer_print (rout_name, array, arrayname)
  implicit none
! i/o:
  character(len=*)           :: rout_name
  character(len=*)                           :: arrayname
  integer(dpi), pointer       :: array (:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: dimen, i

! begin:
  l_here ='array_integer_print'
  dimen = size(array)
  message = routines_2*' array '*arrayname*' is empty'

  if(dimen .eq. 0)  then
    call warning (l_here, message) 
  else
    do i = 1, dimen
     write (logf,'(1x,3a,i5,a,i10)') &
           trim(rout_name),': ',trim(arrayname),i,' = ',array(i)
    end do
  end if
 
  end subroutine array_integer_print

  subroutine array_boolean_print (rout_name, array, arrayname)
  implicit none
! i/o:
  character(len=*)                     :: rout_name
  character(len=*)                     :: arrayname
    logical(bool), pointer :: array (:)

! local:
! character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: dimen, i

! begin:
! l_here='array_boolean_print'
  dimen = size(array)
  message = routines_2*' array '*arrayname*' is empty'
  if(dimen .eq. 0)  then
    call warning (rout_name, message) 
  else
  do i = 1, dimen
   write (logf,'(1x,3a,i5,a,l1)') &
         trim(rout_name),': ',trim(arrayname),i,' = ',array(i)
  end do
  end if
  
  end subroutine array_boolean_print

  subroutine array_double_print (rout_name, array, arrayname)
  implicit none
! i/o:
  character(len=*)           :: rout_name
  character(len=*)                         :: arrayname
  real(dp), pointer :: array (:)

! local:

  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: dimen, i

! begin:
  l_here ='array_double_print'
  dimen = size(array)
  message = routines_2*' array '*arrayname*' is empty'
  if(dimen .eq. 0)  then
    call warning (l_here, message) 
  else
   do i = 1, dimen
    write (logf,'(1x,3a,i5,a,g25.15)') &
          trim(rout_name),': ',trim(arrayname),i,' = ',array(i)
   end do
  end if
  
  end subroutine array_double_print

  subroutine matrix_integer_print (rout_name, matrix, matrixname)
  implicit none
! i/o:
  character(len=*)           :: rout_name
  character(len=*)                         :: matrixname
  integer(dpi), pointer :: matrix (:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: nb_lin, i

! begin:
  l_here ='matrix_integer_print'
  nb_lin = size(matrix,1)
  message = routines_2*' array '*matrixname*' is empty'
  if(nb_lin .eq. 0)  then
    call warning (l_here, message) 
  else
  do i = 1, nb_lin
   write (logf,'(1x,4a,i3,a,/(t2,10i5))') &
   trim(rout_name),': matrix ',trim(matrixname),' line ',i,' values ',matrix(i,:)
  end do
  end if
  
  end subroutine matrix_integer_print

  subroutine matrix_double_print (rout_name, matrix, matrixname)
  implicit none

! i/o:
  character(len=*)           :: rout_name
  character(len=*)                         :: matrixname
  real(dp), pointer :: matrix (:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: nb_lin, i

! begin:
  l_here ='matrix_double_print'
  nb_lin = size(matrix,1)

  message = routines_2*' array '*matrixname*' is empty'
  if(nb_lin .eq. 0)  then
    call warning (l_here, message) 
  else
    do i = 1, nb_lin
     write (logf,'(1x,4a,i3,a,/(t2,5g25.15))') &
     trim(rout_name),': matrix ',trim(matrixname),' line ',i,' values ',matrix(i,:)
    end do
  end if
  
  end subroutine matrix_double_print

! ===================================================================================
  subroutine string_array_resize (array_name, array, new_size)
! -----------------------------------------------------------------------------------
!
! Name           : string_array_resize
!
! Argument (in ) : array_name
! Argument (in ) : array
! Argument (in ) : new_size
!
! Description    : set array to new_size
!
! Authors: F. Colonna from R. Assaraf
! Date   : 25 Dec 2002
! -----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)               :: array_name  
  character(len=*), pointer      :: array (:)
  integer(dpi)                   :: new_size

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here  
  character(len=len(array)), pointer :: array_temp (:)
  integer(dpi)                   :: min_size 
  integer(dpi)                   :: old_size 

! begin:
  l_here='string_array_resize'
  call enter (l_here, 5)

  old_size = size (array)

  if (debug) then
   write (logf,*) trim(l_here),'-d: array >',trim(array_name), &
                  '< resized from ',old_size,' to ',new_size
  end if

  if (.not.associated (array)) then 

    message = 'array >'+array_name+'< not associated. could not be resized'
    call die (routines_21, message)

  else if (size (array) /= new_size) then 
       
    allocate (array_temp (new_size))
    min_size = min (new_size, size (array))
    array_temp (1:min_size) = array (1:min_size)
    deallocate (array)
    array => array_temp

    write (logf,*) trim(l_here),': array >',trim(array_name), &
                  '< resized from ',old_size,' to ',new_size

  else
    write (logf,*) trim(l_here),': array >',trim(array_name), &
    '< not resized. old size ',old_size,' = new size ',new_size

  endif

  call exit (l_here, 5)

  end subroutine string_array_resize

! ======================================================================================
  function string_array_compress (sarray) result (result)
! --------------------------------------------------------------------------------------
!
! Name        : string_array_compress
!
! Description : take out blanks strings from sarray
! Description : blanks all remaining strings
! Description : returns number of non-blank elements
!
! Authors:F. Colonna
! Date  :15 Dec 2002
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: sarray (:)

! result:
  integer(dpi) :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: wrd_i, wrd_nb

! begin:
  l_here ='string_array_compress'
  call enter (l_here, 3)

  wrd_nb = size(sarray)
  if(wrd_nb == 0) then
    message ='in routine'*l_here*' nb of elements in strings array ='+wrd_nb
    call require (message,  wrd_nb > 0)
  end if

  if(debug) then
    write (logf,*)trim(routines_21),'-d: nb of elements in strings array =',wrd_nb
  end if

  result = 0
  do wrd_i = 1, wrd_nb

    if (sarray (wrd_i) /= " " ) then

      result = result + 1
      sarray (result) = sarray(wrd_i)

    end if

  end do 

  if(result < wrd_nb) then
    sarray (result+1: wrd_nb ) = blanks
  end if

  if(debug) then 
    do wrd_i = 1, result 
      write (logf,*)trim(l_here),'-d: compressed word # ',wrd_i,' >',trim(sarray(wrd_i)),'<'
    end do
  end if

  call exit (l_here, 3)

  end function string_array_compress

end module arrays_mod 
