! -*- f90 -*-
! =========================================================================
  subroutine _name_from_type_(_allocation) (name, obj _dims_arguments_)
! -------------------------------------------------------------------------
!
! Name           : _name_from_type_(_allocation)
!
! Argument       : name
! Argument       : obj
! Argument       : dim
!
! Description    : allocate  and catalog obj
!
! Authors: R. Assaraf
! Date   : 17 May 2002
! -------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: name
  _obj_declaration_(pointer,obj)
  _dims_declaration_

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)      :: obj_i, create_index

! begin:
  l_here='_name_from_type_(_allocation)'

  if(header_exe) then

     return
  endif

  call enter(l_here, 10)
  
  if (debug) write (logf,*) trim(l_here), "-d: allocation of >"+name+"< cur_create=", cur_create
 
  cur_create = cur_create + 1	

  create_index = cur_create_ind (cur_create)

  call alias_byaddr_bld (obj, objects_addr (create_index))

  if (associated(obj)) then
 
     if (trace_depend) write (logf,*) trim (l_here), ': ', trim (name), ' left in memory'
 
  else 

     if (trace_depend) write (logf,*) trim (l_here), ': allocate (', trim (name), &
          '(' _dims_print_ ,') )'
     
     obj_i = string_in_array_locate (name, objects_names)

     if (obj_i /= create_index.and. cur_nod <= routines_nb) then
        call one_node_write (cur_nod, logf, l_here)
        write (logf,*) trim(l_here),": check routine >"+routines_name(nodes_rtn(cur_nod))+"<"
        write (logf,*) trim(l_here),": cur_create=",cur_create
        call die (l_here, "allocated >"+objects_names(obj_i)+"< different from created >"+objects_names(create_index)+"<")
     endif

     if(obj_i == 0) then 
        message='obj '*name*' does not exist '
        call die (l_here, message)
     endif

     call _type_allocation_ (create_index _dims_arguments_)
     call alias_byaddr_bld (obj, objects_addr (create_index))
     
  endif

  call exit (l_here, 10)

end subroutine _name_from_type_(_allocation)

! =========================================================================
  subroutine _name_from_type_(_is_needed) (obj)
! -------------------------------------------------------------------------
!
! Name           : _name_from_type_(_is_needed)
!
! Argument       : name
! Argument       : obj
!
! Description    : 
!
! Authors: R. Assaraf
! Date   : 17 May 2002
! -------------------------------------------------------------------------
  implicit none

! i/o:
!  character(len=*) :: name
  _obj_declaration_(pointer,obj)

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)      :: obj_i, needed_index

! begin:
!  l_here='_name_from_type_(_is_needed)'

!  call enter(l_here, 10)
  
!  if (header_exe) then 

!     call object_needed ("name")

!  else 
     cur_needed = cur_needed + 1
     
     needed_index = cur_needed_ind (cur_needed)
     
     call alias_byaddr_bld (obj, objects_addr (needed_index))
!  endif

!  call exit (l_here, 10)

end subroutine _name_from_type_(_is_needed)

!----------------------------------------------------------------------
  subroutine _name_from_type_(_type_allocation) (obj_i _dims_arguments_)
! -------------------------------------------------------------------------
!
! Name           : _name_from_type_(_type_allocation) 
!
! Argument       : obj_i
! Argument : dim_1
! Argument : 
!
! Description    : defines the type of obj_i to be _name_from_type_()
! Description    : and allocate an object of type  _name_from_type_()
!
! Authors: R. Assaraf
! Date   : 20 May 2002
! -------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: obj_i
 _dims_declaration_

! local:
  character(len=max_string_len_routine_name), save :: l_here 
  integer(dpi), save :: cur_type = NOT_INIT_I 

! begin:
  l_here='_name_from_type_(_type_allocation)'

  call enter(l_here, 10)

    if (cur_type == NOT_INIT_I) &
    cur_type = string_in_array_locate_or_die ('_name_from_type_()', types_name)

  if ( obj_i <= 0) then 
    message=' object obj_i='+obj_i+' does not exist '
    call die (l_here, message)
  endif

  allocate ( _name_from_type_(_type_alias))
  allocate ( _name_from_type_(_type_alias)%p ( _dims_1_n_ ) ) 
       
  objects_addr ( obj_i) = address (_name_from_type_(_type_alias))
  objects_type ( obj_i) = cur_type

  nullify (_name_from_type_(_type_alias))

  call exit (l_here, 10)

  end subroutine _name_from_type_(_type_allocation)
 
 
!----------------------------------------------------------------------
  subroutine _name_from_type_(_byaddr_bld) (obj, addr_obj)
! -------------------------------------------------------------------------
!
! Name           : integer_1_byadd_bld
!
! Argument (in ) : addr_obj
! Argument (out) : integer_1_type_obj
!
! Description    : builds the object _name_from_type_(_type_obj) from the address addr_obj
! Description    : _name_from_type_(_type_obj) => _name_from_type_(_integer_1)_type_alias%p
!
! Authors: R. Assaraf
! Date  : 17 May 2001
! -------------------------------------------------------------------------
  implicit none
 
! i/o:
  _obj_declaration_(pointer,obj)
  integer(dpi)      :: addr_obj
 
! local:
  character(len=max_string_len_routine_name), save :: l_here 

! begin:
  l_here = '_name_from_type_(_byaddr_bld)'
  call enter(l_here, 10)

  call exe_by_address_1 (address (_name_from_type_(_type_alias_bld)), addr_obj)
 
  if (associated (_name_from_type_(_type_alias))) then
     obj =>  _name_from_type_(_type_alias%p)
  else
     obj =>  NULL ()
  endif

  call exit(l_here, 10)

  end subroutine _name_from_type_(_byaddr_bld)
 

! =========================================================================
  subroutine _name_from_type_(_catalog) (name, obj)
! -------------------------------------------------------------------------
!
! Name           : _name_from_type_(_catalog)
!
! Argument       : name
! Argument       : obj
!
! Description    : catalog obj
!
! Authors: R. Assaraf
! Date   : 20 Jul 2002
! -------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: name
  _obj_declaration_(pointer,obj)

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)      :: index
  integer(dpi)      :: cur_type = NOT_INIT_I

! begin:
  l_here='_name_from_type_(_catalog)'
  call enter(l_here, 10)

  if (cur_type == NOT_INIT_I) cur_type = string_in_array_locate_or_die ('_name_from_type_', types_name)

  call objects_name_add_once (trim (name))

  index = string_in_array_locate (name, objects_names)


  if (objects_addr (index)/= 0) &
       call die (l_here, 'obj '*name*' has already an address')

  allocate (_name_from_type_(_type_alias))

  _name_from_type_(_type_alias)%p =>  obj
  
  objects_addr (index) = address (_name_from_type_(_type_alias))
  objects_type (index) = cur_type

  nullify (_name_from_type_(_type_alias))

  call exit (l_here, 10)

end subroutine _name_from_type_(_catalog)


! =========================================================================
  subroutine _name_from_type_(_type_alias_bld) (obj_target)
! -------------------------------------------------------------------------
!
! Name           : _name_from_type_(_type_alias_bld)
!
! Argument (in ) : obj_target
!
! Description    : _name_from_type_(_type_alias) => obj_target
!
! Authors: R. Assaraf
! Date   : 07 May 2001
! -------------------------------------------------------------------------
  implicit none
! i/o:
  type(_name_from_type_(_type)) , target :: obj_target
  
! local:
  character(len=max_string_len_routine_name) :: l_here 
 
! begin:
  l_here='_name_from_type_(_type_alias_bld)'
  call enter(l_here, 10)

  _name_from_type_(_type_alias) =>  obj_target

  call exit(l_here, 10)

  end subroutine _name_from_type_(_type_alias_bld)


!----------------------------------------------------------------------
  subroutine _name_from_type_(_type_alias_byaddr_bld) (alias, addr_obj)
! -------------------------------------------------------------------------
!
! Name           : _name_from_type_(_type_alias_byaddr_bld)
!
! Argument (in ) : addr_obj
! Argument (out) : alias
!
! Description    : builds the object alias from the address addr_obj
! Description    : alias =>  _name_from_type_(_type_alias)
!
! Authors: R. Assaraf
! Date  : 17 May 2001
! -------------------------------------------------------------------------
  implicit none
 
! i/o:
  type(_name_from_type_(_type)), pointer :: alias
  integer(dpi)         :: addr_obj
 
! local:
  character(len=max_string_len_routine_name), save :: l_here 

! begin:
  l_here = '_name_from_type_(_type_alias_byaddr_bld)'
  call enter(l_here, 10)

  call exe_by_address_1 (address (_name_from_type_(_type_alias_bld)), addr_obj)
  alias => _name_from_type_(_type_alias)

  call exit(l_here, 10)

  end subroutine  _name_from_type_(_type_alias_byaddr_bld)
 
