! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module basis_menu_mod

  use xpfs_mod
  use aos_mod
  use files_menu_mod

  contains

! ========================================================================
  subroutine basis_menu
! ------------------------------------------------------------------------
!
! Name           : basis_menu
!
! Description    : collects basis-set information from menu
!
! Authors:X. Krokidis, F. Colonna
! Date  :26 Sep 2000
! Revision : 08 Dec 2000 file =?
! Revision : 07 Oct 2002 F. Colonna
! ------------------------------------------------------------------------
  implicit none

! local:
  logical(bool)                              :: done
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: what_to_print

! begin:
  l_here ='basis_menu'
  call enter (l_here)

! initialize:
  what_to_print = NOT_INIT_S

  done = .false.
  call next_command_get ('basis')
  do while (.not.done)
   call next_word_get ('basis')
   if(cur_word(1:4) .eq. 'help') then

    write (logf,*) 'basis: basis definitions'
    write (logf,*) 'basis: syntax:'
    write (logf,*) 'basis: basis'
    write (logf,*) 'basis:  help'
    write (logf,*) 'basis:  fil{e}   [command]'
    write (logf,*) 'basis:  rea{d}   [command]'
    write (logf,*) 'basis:  pri{nt} = [string] (norepeat)'
    write (logf,*) 'basis:     [string] may be one among:'
    write (logf,*) 'basis:     xpfs_c{art}'
    write (logf,*) 'basis:     xpfs_s{pherical}'
    write (logf,*) 'basis:     xpfs_c_o{verlap}'
    write (logf,*) 'basis:     aos_sphe{rical}'
    write (logf,*) 'basis:     aos_cart{esian}'
    write (logf,*) 'basis:     aos_cart_n{ormed}'
    write (logf,*) 'basis:  end_bas'
    write (logf,*) 'basis: example'
    write (logf,*) 'basis: basis'
    write (logf,*) 'basis:  file ... end_file'
    write (logf,*) 'basis:  read ... end_read'
    write (logf,*) 'basis:  print = epfs'
    write (logf,*) 'basis: end_bas'

   else if(cur_word(1:3) .eq. 'fil') then
    call files_menu

   else if(cur_word(1:3) .eq. 'rea') then
    call epfs_read_menu

   else if(token_is ('pri', what_to_print)) then
     
   else
    call menu_exit (l_here, done)
   end if

  end do

! print 
  if (what_to_print(1:1) .eq. '?') then

  else if (what_to_print(1:4) .eq. 'epfs') then
    call epfs_print

  else if (what_to_print(1:6) .eq. 'xpfs_c') then
    call xpfs_c_print

  else if (what_to_print(1:6) .eq. 'xpfs_s') then
    call xpfs_s_print

  else if (what_to_print(1:8) .eq. 'xpfs_c_o') then
    call xpfs_overlap_print ("xpfs_c")

  else if (what_to_print(1:10) == "aos_cart_n") then
    call aos_c_nrmd_print

  else if (what_to_print(1:8)  == "aos_cart") then
    call aos_c_print

  else if (what_to_print(1:8)  == "aos_sphe") then
    call aos_s_print

  else
   message = 'illegal object to be printed >'+what_to_print+'<'
   call die (l_here, message)
  end if

  call exit (l_here)

  end subroutine basis_menu

end module basis_menu_mod
