! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module c_routines_mod

  use control_mod
  use strings_mod

! Owner   : F. Colonna colonna@lct.jussieu.fr
! Revision: 13 Fev 2001: stemp, string => local

  contains

! ========================================================================================
  function exesys (command) 
! ----------------------------------------------------------------------------------------
!
! Name           : exesys
!
! Argument (in ) : command
!
! Description    : executes a system command
! Description    : returns code 
!
! Authors: G. Jansen, F. Colonna
! Date  :10 Oct 2000
! Revision: 25 Dec 2003 by R. Assaraf, require: ret_code.ne.0 -> ret_code.eq.0
! ----------------------------------------------------------------------------------------
  implicit none

! function:
  integer(dpi) exesys

! i/o:
  character(len=*), intent(in) :: command

! local function:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string 
  integer(dpi) :: system

! local:
  integer(dpi) :: ret_code

! begin:
  l_here ='exesys'
  call enter (l_here)

! !!! char(0) is necessary to mark the end of the string correctly
  string = trim(command)//char(0)
 
  write (logf,'(4a)') trim(l_here),': >',trim(string),'<'

  ret_code = system (string)

  message ='in routine'*l_here*' return code is '*ret_code
  call require (message, ret_code .eq. 0)

  exesys = ret_code
  
  call exit (l_here)

  end function exesys

! ========================================================================================
  function architecture_get () 
! ----------------------------------------------------------------------------------------
!
! Name           : architecture_get
!
! Description    : returns architecture name
!
! Authors: F. Colonna
! Date  :10 Oct 2000
! ----------------------------------------------------------------------------------------
  implicit none

! function 
  character(len=max_len_arch_name) :: architecture_get

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)    :: string 
 
! begin:
  l_here ='architecture_get'
  call enter (l_here,3)

  string = blanks
  call string_from_env_f (string, 'QMCMOL_UNAME_a')
  
  arch_name        = string_head_get (string, ' ')
  architecture_get = arch_name
  if(debug) write (logf,*)trim(l_here),'-d: architecture name >',trim(arch_name),'<'

  call exit (l_here,3)

  end function architecture_get

! ========================================================================================
  function machine_name_get () 
! ----------------------------------------------------------------------------------------
!
! Name           : machine_name_get
!
!
! Description    : returns machine name
!
! Comments   : uname -a is
! Comments   : Linux quantum 2.2.16 #2 SMP Tue Jul 25 10:54:16 FRA Ete 2000 i686 unknown
! Comments   : AIX gargantua 3 4 002019739400
! Comments   : SunOS liliput 4.1.3_U1 1 sun4m
! Comments   : IRIX surya 6.5 07201606 IP20
!
! Authors: F. Colonna
! Date  :01 Nov 2000
! ----------------------------------------------------------------------------------------
  implicit none

! function
  character(len=max_len_mach_name) :: machine_name_get

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)    :: stemp  

! begin:
  l_here ='machine_name_get'
  call enter (l_here,3)

  mach_name = blanks
  call string_from_env_f (mach_name, 'QMCMOL_UNAME_a')
 
  stemp       = string_tail (mach_name, ' ')
  mach_name   = string_head_get (stemp, ' ')
  
  machine_name_get = mach_name

  if(debug) write (logf,*)trim(l_here),'-d: machine name >',trim(mach_name),'<'

  call exit (l_here,3)
  
  end function machine_name_get

! ========================================================================================
  function directory_get () 
! ----------------------------------------------------------------------------------------
! Name           : directory_get
!
!
! Description    : returns directory name
!
! Authors: F. Colonna
! Date  :10 Oct 2000
! ----------------------------------------------------------------------------------------
  implicit none

! function
  character(len=max_string_len_file_name)    :: directory_get

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='directory_get'
  call enter (l_here,3)

  dir_cur_name = blanks
  call string_from_env_f (dir_cur_name, 'PWD')

  directory_get = dir_cur_name
  if(debug) write (logf,*)trim(l_here),'-d: current directory name >',trim(dir_cur_name),'<'

  call exit (l_here,3)

  end function directory_get

! ========================================================================================
  function username_get () 
! ----------------------------------------------------------------------------------------
!
! Name           : username_get
!
!
! Description    : returns user_name name
!
! Authors: F. Colonna
! Date  :10 Oct 2000
! ----------------------------------------------------------------------------------------
  implicit none

! function
  character(len=max_len_user_name)           :: username_get

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='username_get'
  call enter (l_here,3)

  user_name = blanks
  call string_from_env_f (user_name, 'USER')

  username_get = user_name
  if(debug) write (logf,*)trim(l_here),'-d: user name >',trim(user_name),'<'

  call exit (l_here,3)

  end function username_get

! ========================================================================================
  subroutine string_from_env_f (string, env)
! ========================================================================================
!
! Name           : string_from_env_f
!
! Argument (out) : string
! Argument (in ) : env
!
! Description    : "string" = content of environment variable "env"
!
! Authors:R. Assaraf, F. Colonna
! Date   : 9 Oct 2000
! ----------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: string
  character(len=*)                           :: env

! local:
  integer :: length
  character(len=max_string_len_routine_name) :: l_here   
  character(len=max_string_len)              :: l_str  ! necessary to avoid side effects

! begin:
  l_here='string_from_env_f'
  call enter(l_here,10)

  call string_from_env_c (l_str, trim(env)//char(0), length)
! keep this, it is necessary to avoid side effects from C
  string = l_str
  string(length+1:len(string)) = repeat (" ",len(string))

  if(length == 0) &
  call die (l_here,'environment variable >'+env+'< = >'+l_str+' not set')

  if(debug) then
    write (logf,*)trim(l_here),'-d: env variable >',trim(env),'< is >',trim(string),'<'
  end if

  call exit(l_here,10)

  end subroutine string_from_env_f

end module c_routines_mod
