! -*- f90 -*-
!-----------------------------------------------------------------------------
!     This file is part of QmcMol A GNU QMC program for molecules.
!    Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!     R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!     P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!            http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software Foundation; either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
! for more details. You should have received a copy of the GNU General
! Public License along with QmcMol; see the file COPYING. If not,
! write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


module ci_mod

 use basic_types_tools_mod
 use jastrow_factor_bywlk_mod
 use det_sum_bywlk_mod
 use psi_bywlk_mod
 use e_loc_vne_bywlk_mod
 use e_loc_vee_bywlk_mod
 use e_loc_vnn_mod
 use ecp_bywlk_mod


 implicit none

 real(dp), pointer     :: configuration_over_psi_bywlk (:,:) => NULL()
 real(dp), pointer     :: configuration_lapl_over_psi_bywlk (:,:) => NULL()

 real(dp), pointer     :: overlap_matrix      (:,:) => NULL()
 real(dp), pointer     :: overlap_matrix_sq   (:,:) => NULL()
 real(dp), pointer     :: overlap_matrix_var  (:,:) => NULL()
 real(dp), pointer     :: overlap_matrix_err  (:,:) => NULL()

 real(dp), pointer     :: ci_matrix           (:,:) => NULL()
 real(dp), pointer     :: ci_matrix_sq        (:,:) => NULL()
 real(dp), pointer     :: ci_matrix_var       (:,:) => NULL()
 real(dp), pointer     :: ci_matrix_err       (:,:) => NULL()

 real(dp), pointer     :: ci_eigenvectors     (:,:) => NULL()
 real(dp), pointer     :: ci_eigenvalues      (:)   => NULL()

 real(dp),save                 :: ci_count

! Needs          : jastrow_factor_bywlk (conf, walk)
! Needs          : jastrow_factor_grad_bywlk (elec, conf, dim, walk)
! Needs          : jastrow_factor_lapl_bywlk (elec, conf, walk)

! Needs          : det_sum_bywlk (conf, walk)
! Needs          : det_sum_grad_bywlk (elec, conf, dim, walk)
! Needs          : det_sum_lapl_bywlk (elec, conf, walk)

! Needs          : psi_bywlk (walk)

! Author: A. Scemama
! Date: 22 May 2006
!
! Description  :
!  ci_mod : Computes the CI matrix and the overlap matrix

! Work arrays
 real(dp), pointer     :: S(:,:) => NULL()
 real(dp), pointer     :: H(:,:) => NULL()
 real(dp), pointer     :: work(:,:) => NULL()

 contains

! =========================================================================
  subroutine configuration_over_psi_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : configuration_over_psi_bywlk_bld
! Needs          : det_sum_bywlk
! Needs          : jastrow_factor_bywlk
! Needs          : psi_bywlk
! 
! Description    : Builds Phi_i.J_i / Psi
! 
! Authors:A. Scemama
! Date  :22 May 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  character(len=max_len_conf_type) :: l_jastr_type

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i

  logical(bool)                 :: no_jastrow

! header:

  if (header_exe) then

    l_here = 'configuration_over_psi_bywlk_bld'
    rname = 'configuration_over_psi_bywlk'
    call object_create (rname)

    call object_needed ('psi_bywlk')
    call object_needed ('jastrow_factor_bywlk')
    call object_needed ('det_sum_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_configurations.inc'
  include 'chksiz_det_sum_bywlk.inc'

  l_jastr_type = configurations(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_bywlk.inc'
  end if

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
     write (logf,*)trim(l_here),'-d: no_jastrow        =',no_jastrow         
     write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'
  end if        

! allocations:
  call alloc ('configuration_over_psi_bywlk', configuration_over_psi_bywlk, &
    configurations_nb, walkers_nb)
! end allocations

  configuration_over_psi_bywlk(:,:) = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

       if( .not. no_jastrow) then
          configuration_over_psi_bywlk (conf_i, walk_i) = &
                jastrow_factor_bywlk (conf_i, walk_i)  &
               * det_sum_bywlk (conf_i, walk_i) &
               / psi_bywlk (walk_i)
       else 
          configuration_over_psi_bywlk (conf_i, walk_i) = &
                 det_sum_bywlk (conf_i, walk_i) &
               / psi_bywlk (walk_i)
       end if

    end do ! do conf_i 
    
 end do ! do walk_i 

  include 'debug_psi_bywlk.inc'
!   include 'ensnb_psi_bywlk.inc'

  call exit(l_here)

  end subroutine configuration_over_psi_bywlk_bld

! =========================================================================
  subroutine overlap_matrix_bld
! -------------------------------------------------------------------------
! 
! Name           : overlap_matrix_bld
! Needs          : configuration_over_psi_bywlk
! 
! Description    : Builds the overlap matrix < Phi_i.J_i | Phi_j.J_j >
! 
! Authors:A. Scemama
! Date  :22 May 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  character(len=max_len_conf_type) :: l_jastr_type

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, conf_j
  integer(dpi)                  :: walk_i


! header:

  if (header_exe) then

    l_here = 'overlap_matrix_bld'

    rname = 'overlap_matrix'
    call object_create (rname)

    call object_needed ('configuration_over_psi_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if        

! allocations:
  if (.not.associated(overlap_matrix_sq)) then
    call alloc ('overlap_matrix', overlap_matrix, &
      configurations_nb, configurations_nb)
    call alloc ('overlap_matrix_var', overlap_matrix_var, &
      configurations_nb, configurations_nb)
    call alloc ('overlap_matrix_err', overlap_matrix_err, &
      configurations_nb, configurations_nb)
    allocate(overlap_matrix_sq(configurations_nb, configurations_nb))
    ci_count = 0.
  endif
! end allocations

  overlap_matrix    (:,:) = overlap_matrix    (:,:)*ci_count
  overlap_matrix_sq (:,:) = overlap_matrix_sq (:,:)*ci_count

  do walk_i = 1, walkers_nb
    ci_count = ci_count + 1.
    do conf_i = 1, configurations_nb
      do conf_j = 1, configurations_nb

        overlap_matrix (conf_i, conf_j) =                           &
             overlap_matrix (conf_i, conf_j)                        &
           +(configuration_over_psi_bywlk (conf_j, walk_i)          &
           * configuration_over_psi_bywlk (conf_i, walk_i))

        overlap_matrix_sq (conf_i, conf_j) =                        &
             overlap_matrix_sq (conf_i, conf_j)                     &
           +(configuration_over_psi_bywlk (conf_j, walk_i)          &
           * configuration_over_psi_bywlk (conf_i, walk_i) )**2

      end do ! do conf_i
    end do ! do conf_j
 end do ! do walk_i 
 
 overlap_matrix    (:,:) = overlap_matrix    (:,:)/ci_count
 overlap_matrix_sq (:,:) = overlap_matrix_sq (:,:)/ci_count

  call exit(l_here)

  end subroutine overlap_matrix_bld

! =========================================================================
  subroutine overlap_matrix_var_bld
! -------------------------------------------------------------------------
! 
! Name           : overlap_matrix_var_bld
! Needs          : overlap_matrix
! 
! Description    : Builds the variance of the overlap matrix
! 
! Authors:A. Scemama
! Date  : 2 Jun 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, conf_j
  integer(dpi)                  :: walk_i


! header:
! header:

  if (header_exe) then

    l_here = 'overlap_matrix_var_bld'

    rname = 'overlap_matrix_var'
    call object_create (rname)

    call object_needed ('overlap_matrix')

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if        

! allocations:
  if (.not.associated(overlap_matrix_sq)) then
    call alloc ('overlap_matrix_var', overlap_matrix_var, &
      configurations_nb, configurations_nb)
  endif
! end allocations

 overlap_matrix_var(:,:) = overlap_matrix_sq (:,:) - overlap_matrix(:,:)**2

  call exit(l_here)

  end subroutine overlap_matrix_var_bld
! =========================================================================
  subroutine overlap_matrix_err_bld
! -------------------------------------------------------------------------
! 
! Name           : overlap_matrix_err_bld
! Needs          : overlap_matrix_var
! 
! Description    : Builds the error of the overlap matrix
! 
! Authors:A. Scemama
! Date  :02 Jun 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, conf_j
  integer(dpi)                  :: walk_i


! header:

  if (header_exe) then

    l_here = 'overlap_matrix_err_bld'

    rname = 'overlap_matrix_err'
    call object_create (rname)
    call object_needed ('overlap_matrix_var')

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if        

! allocations:
  if (.not.associated(overlap_matrix_sq)) then
    call alloc ('overlap_matrix_err', overlap_matrix_err, &
      configurations_nb, configurations_nb)
  endif
! end allocations

 overlap_matrix_err(:,:) = sqrt(overlap_matrix_var(:,:))/(ci_count-1)

  call exit(l_here)

  end subroutine overlap_matrix_err_bld

! =========================================================================
  subroutine configuration_lapl_over_psi_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : configuration_lapl_over_psi_bywlk_bld
! Needs          : det_sum_bywlk
! Needs          : det_sum_grad_bywlk
! Needs          : det_sum_lapl_bywlk
! Needs          : jastrow_factor_bywlk
! Needs          : jastrow_factor_grad_bywlk
! Needs          : jastrow_factor_lapl_bywlk
! Needs          : psi_bywlk
! 
! Description    : Builds Phi_i.J_i / Psi
! 
! Authors:A. Scemama
! Date  :22 May 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  character(len=max_len_conf_type) :: l_jastr_type

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i

  real(dp)                      :: dot_prod

  logical(bool)                 :: no_jastrow

! header:

  if (header_exe) then

    l_here = 'configuration_lapl_over_psi_bywlk_bld'
    rname = 'configuration_lapl_over_psi_bywlk'
    call object_create (rname)

    call object_needed ('det_sum_bywlk')
    call object_needed ('det_sum_grad_bywlk')
    call object_needed ('det_sum_lapl_bywlk')
    call object_needed ('jastrow_factor_bywlk')
    call object_needed ('jastrow_factor_grad_bywlk')
    call object_needed ('jastrow_factor_lapl_bywlk')
    call object_needed ('psi_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_configurations.inc'
  include 'chksiz_det_sum_bywlk.inc'

  l_jastr_type = configurations(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_bywlk.inc'
  end if

! end requirements

! dimensions:
  electrons_nb      = size (det_sum_grad_bywlk, 1)
  configurations_nb = size (det_sum_grad_bywlk, 2)
  dimensions_nb     = size (det_sum_grad_bywlk, 3)
  walkers_nb        = size (det_sum_grad_bywlk, 4)

! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
     write (logf,*)trim(l_here),'-d: no_jastrow        =',no_jastrow         
     write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'
  end if        

! allocations:
  call alloc ('configuration_lapl_over_psi_bywlk', configuration_lapl_over_psi_bywlk, &
    configurations_nb, walkers_nb)
! end allocations

  configuration_lapl_over_psi_bywlk(:,:) = zero

  if( .not. no_jastrow) then

     do walk_i = 1, walkers_nb

        do conf_i = 1, configurations_nb

           do elec_i = 1, electrons_nb

              dot_prod = zero

              do dim_i = 1, dimensions_nb

                 dot_prod = dot_prod &
                      +  jastrow_factor_grad_bywlk (elec_i, conf_i, dim_i, walk_i) &
                      *  det_sum_grad_bywlk (elec_i, conf_i, dim_i, walk_i)
              end do ! do dim_i

              configuration_lapl_over_psi_bywlk (conf_i, walk_i)          &
                   = configuration_lapl_over_psi_bywlk (conf_i, walk_i)   &
                   + jastrow_factor_lapl_bywlk (elec_i, conf_i, walk_i)   &
                   * det_sum_bywlk (conf_i, walk_i)                       &
                   + jastrow_factor_bywlk (conf_i, walk_i)                &
                   * det_sum_lapl_bywlk (elec_i, conf_i, walk_i)          &
                   + two * dot_prod

           end do ! do elec_i

        end do ! do conf_i

     end do ! do walk_i

  else

     do walk_i = 1, walkers_nb

        do conf_i = 1, configurations_nb

           do elec_i = 1, electrons_nb

               configuration_lapl_over_psi_bywlk (conf_i, walk_i)        &
                   =  configuration_lapl_over_psi_bywlk (conf_i, walk_i) &
                   + det_sum_lapl_bywlk (elec_i, conf_i, walk_i)

           end do ! do elec_i

        end do ! do conf_i

     end do ! do walk_i

  end if


 do conf_i = 1, configurations_nb
   configuration_lapl_over_psi_bywlk (conf_i, :) = &
     configuration_lapl_over_psi_bywlk (conf_i, :) / psi_bywlk (:)
 end do ! do conf_i

  include 'debug_psi_bywlk.inc'
!   include 'ensnb_psi_bywlk.inc'

  call exit(l_here)

  end subroutine configuration_lapl_over_psi_bywlk_bld

! =========================================================================
  subroutine ci_matrix_bld
! -------------------------------------------------------------------------
! 
! Name           : ci_matrix_bld
! Needs          : configuration_over_psi_bywlk_bld
! Needs          : e_loc_kin_bywlk (walk)
! Needs          : e_loc_vne_bywlk (walk)
! Needs          : e_loc_vee_bywlk (walk)
! Needs          : e_loc_vnn ()
! Needs          : ecp_bywlk (walk)
! 
! Description    : Builds the CI matrix < Phi_i.J_i |H| Phi_j.J_j >
! 
! Authors:A. Scemama
! Date  :22 May 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, conf_j
  integer(dpi)                  :: walk_i
  real(dp)                      :: pot

! header:

  if (header_exe) then

    l_here = 'ci_matrix_bld'

    rname = 'ci_matrix'
    call object_create (rname)

    call object_needed ('configuration_over_psi_bywlk')
    call object_needed ('configuration_lapl_over_psi_bywlk')
    call object_needed ('e_loc_vne_bywlk')
    call object_needed ('e_loc_vee_bywlk')
    call object_needed ('ecp_bywlk')
    call object_needed ('e_loc_vnn')

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if        

! allocations:
  if (.not.associated(ci_matrix_sq)) then
    call alloc ('ci_matrix', ci_matrix, &
      configurations_nb, configurations_nb)
    call alloc ('ci_matrix_var', ci_matrix_var, &
      configurations_nb, configurations_nb)
    call alloc ('ci_matrix_err', ci_matrix_err, &
      configurations_nb, configurations_nb)
    allocate(ci_matrix_sq(configurations_nb, configurations_nb))
    ci_count = 0.
  endif
! end allocations

  ci_matrix    (:,:) = ci_matrix    (:,:)*ci_count
  ci_matrix_sq (:,:) = ci_matrix_sq (:,:)*ci_count

  do walk_i = 1, walkers_nb
    ci_count = ci_count + 1.
    pot = e_loc_vne_bywlk(walk_i) + e_loc_vee_bywlk(walk_i) + e_loc_vnn + ecp_bywlk(walk_i)
    do conf_i = 1, configurations_nb
      do conf_j = 1, configurations_nb

        ci_matrix (conf_i, conf_j) =                           &
             ci_matrix (conf_i, conf_j)                        &
           +configuration_over_psi_bywlk (conf_j, walk_i)          &
           *(-half*configuration_lapl_over_psi_bywlk (conf_i, walk_i) &
           + pot*configuration_over_psi_bywlk (conf_i, walk_i) ) 

        ci_matrix_sq (conf_i, conf_j) =                        &
             ci_matrix_sq (conf_i, conf_j)                     &
           +(configuration_over_psi_bywlk (conf_j, walk_i)          &
           *(-half*configuration_lapl_over_psi_bywlk (conf_i, walk_i) & 
           + pot*configuration_over_psi_bywlk (conf_i, walk_i) ) )**2

      end do ! do conf_i
    end do ! do conf_j
 end do ! do walk_i 
 
 do conf_i = 1, configurations_nb
    do conf_j = 1, configurations_nb

        ci_matrix (conf_i, conf_j) =  0.5d0 *                  &
           ( ci_matrix (conf_i, conf_j)                        &
           + ci_matrix (conf_j, conf_i) )

        ci_matrix_sq (conf_i, conf_j) = 0.5d0 *                &
           ( ci_matrix_sq (conf_i, conf_j)                     &
           + ci_matrix_sq (conf_j, conf_i)  )

    end do ! do conf_i
 end do ! do conf_j

 ci_matrix    (:,:) = ci_matrix    (:,:)/ci_count
 ci_matrix_sq (:,:) = ci_matrix_sq (:,:)/ci_count

  call exit(l_here)

  end subroutine ci_matrix_bld


! =========================================================================
  subroutine ci_matrix_var_bld
! -------------------------------------------------------------------------
! 
! Name           : ci_matrix_var_bld
! Needs          : ci_matrix
! 
! Description    : Builds the variance of the CI matrix elements
! 
! Authors:A. Scemama
! Date  :06 Jun 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, conf_j
  integer(dpi)                  :: walk_i
  real(dp)                      :: pot

! header:

  if (header_exe) then

    l_here = 'ci_matrix_var_bld'

    rname = 'ci_matrix_var'
    call object_create (rname)

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
  walkers_nb        = size (det_sum_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if        

! allocations:
  if (.not.associated(ci_matrix_sq)) then
    call alloc ('ci_matrix_var', ci_matrix_var, &
      configurations_nb, configurations_nb)
  endif
! end allocations

 ci_matrix_var(:,:) = ci_matrix_sq (:,:) - ci_matrix(:,:)**2

  call exit(l_here)

  end subroutine ci_matrix_var_bld


! =========================================================================
  subroutine ci_matrix_err_bld
! -------------------------------------------------------------------------
! 
! Name           : ci_matrix_err_bld
! Needs          : ci_matrix_var
! 
! Description    : Builds the error on the CI matrix
! 
! Authors:A. Scemama
! Date  :06 Jun 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: rname

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, conf_j
  integer(dpi)                  :: walk_i
  real(dp)                      :: pot

! header:

  if (header_exe) then

    l_here = 'ci_matrix_err_bld'

    rname = 'ci_matrix_err'
    call object_create (rname)

    call object_needed ('ci_matrix_var')

    return

  end if

! begin:
  call enter(l_here)

 ci_matrix_err(:,:) = sqrt(ci_matrix_var(:,:))/(ci_count-1)

  call exit(l_here)

  end subroutine ci_matrix_err_bld


! =========================================================================
  subroutine ci_eigenvectors_bld
! -------------------------------------------------------------------------
! 
! Name           : ci_eigenvectors_bld
! Needs          : ci_matrix
! Needs          : overlap_matrix
! 
! Description    : Calculates the eigenvectors of the CI matrix
! 
! Authors:A. Scemama
! Date  :22 May 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)                                        :: det_S
  integer(dpi)                                    :: configurations_nb
  integer(dpi)                                    :: i,j,k
  integer(dpi)                                    :: nrot 
  integer(dpi), pointer                           :: iorder(:) => NULL()
  logical     , pointer                           :: done  (:) => NULL()
  real(dp), pointer                        :: ci_vec_tmp (:,:) => NULL()
  real(dp), pointer                        :: ci_val_tmp (:)   => NULL()

! header:

  if (header_exe) then

    l_here = 'ci_eigenvectors_bld'

    call object_create ('ci_eigenvectors')

    call object_needed ('ci_matrix')
    call object_needed ('overlap_matrix')

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

! dimensions:
  configurations_nb = size (det_sum_bywlk, 1)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
  end if        

! allocations:
  if (.not.associated(ci_eigenvectors)) then
    call alloc ('ci_eigenvectors', ci_eigenvectors, &
      configurations_nb, configurations_nb)
    call alloc ('ci_eigenvalues', ci_eigenvalues, &
      configurations_nb)
  endif
  if (.not.associated(H))    allocate (    H(configurations_nb,configurations_nb) )
  if (.not.associated(S))    allocate (    S(configurations_nb,configurations_nb) )
  if (.not.associated(work)) allocate ( work(configurations_nb,configurations_nb) )

  allocate (iorder(configurations_nb))
  allocate (done  (configurations_nb))
  allocate (ci_vec_tmp(configurations_nb,configurations_nb))
  allocate (ci_val_tmp(configurations_nb))

! end allocations

  S (:,:) = overlap_matrix(:,:)
  call invert_mat (S, configurations_nb, det_S, work)

  H (:,:) = 0.
  do i=1,configurations_nb
    do j=1,configurations_nb
      do k=1,configurations_nb
        H(i,j) = H(i,j) + S(k,i)*ci_matrix(j,k)
      enddo
    enddo
  enddo

 call jacobi(H,configurations_nb,configurations_nb,ci_val_tmp,ci_vec_tmp,nrot)
!call svdcmp(H,configurations_nb,configurations_nb,configurations_nb,configurations_nb,&
! ci_val_tmp,ci_vec_tmp)


! Sort the eigenvalues and eigenvectors

  do i=1,configurations_nb
    iorder(i) = i
  enddo

  ci_val_tmp(configurations_nb) = 0.
  do i=1,configurations_nb
    done(i) = .False.
    ci_vec_tmp(i,configurations_nb) = 0.
    ci_vec_tmp(configurations_nb,i) = 0.
  enddo
  
  do i=1,configurations_nb
    iorder(i) = configurations_nb
    do j=1,configurations_nb
      if (abs(ci_val_tmp(j)) > abs(ci_val_tmp(iorder(i)))) then
        if (.not. done(j)) then
          iorder(i)=j
        endif
      endif
    enddo
    done(iorder(i)) = .True.
  enddo

  do i=1,configurations_nb
    do j=1,configurations_nb
      ci_eigenvectors(i,j) = ci_vec_tmp(i,iorder(j))
    enddo
    ci_eigenvalues(i) = ci_val_tmp(iorder(i))
  enddo

  deallocate (iorder)
  deallocate (done)
  deallocate (ci_vec_tmp)
  deallocate (ci_val_tmp)

  call exit(l_here)

  end subroutine ci_eigenvectors_bld

  subroutine ci_eigenvalues_bld
! -------------------------------------------------------------------------
! 
! Name           : ci_eigenvalues_bld
! Needs          : ci_eigenvectors
! 
! Description    : Calculates the eigenvalues of the CI matrix
! 
! Authors:A. Scemama
! Date  :06 Jun 2006
! -------------------------------------------------------------------------
  implicit none 
  
! i/o

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                                    :: configurations_nb
  integer(dpi)                                    :: i,j,k
  integer(dpi)                                    :: nrot 

  real(dp)                                        :: det_S

! header:

  if (header_exe) then

    l_here = 'ci_eigenvectors_bld'

    call object_create ('ci_eigenvalues')
    call object_needed ('ci_eigenvectors')

    return

  end if

! begin:
  call enter(l_here)

! requirements:

! end requirements

  call exit(l_here)

  end subroutine ci_eigenvalues_bld


end module ci_mod

