! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module configurations_guide_mod

! Descriptions: command name "config"
! Revision: 13 Fev 2001: stemp, string => local

  use basic_types_tools_mod
  use objects_paths_mod
  use determinants_guide_mod
  use files_menu_mod
  use jastrow_mod
  use parser_mod  
  use nuclei_mod
  use system_data_mod

  implicit none
 
  integer(dpi), parameter           :: max_len_conf_type_guide =  8
  integer(dpi), parameter           :: max_len_conf_name_guide = 16
  
 
! The definition of a determinant is in determinants_guide_mod
! type determinant
!  integer(dpi), pointer :: alfa_orbitals(:) => NULL()
!  integer(dpi), pointer :: beta_orbitals(:) => NULL()
!  real(dp)              :: weight
! end type determinant

  type configuration_guide
!   type (jastrow)                   :: conf_jastr
    type (determinant_guide), pointer      :: conf_deter(:)  => NULL ()
    character(len=max_len_conf_name_guide) :: name 
    character(len=max_len_conf_type_guide) :: jastr_type  = "?" ! NOT_INIT_S 
    real(dp)                         :: weight ! = ONE
    integer(dpi)                     :: n_determinant     = NOT_INIT_I
    real(dp), pointer                :: deter_weight (:) => NULL()
  end type configuration_guide

  real(dp),pointer :: configurations_guide_weight (:) => NULL ()

! Improve: what a piece of shift ... no global scalars F. Colonna
! integer(dpi) :: configurations_guide_nb = NOT_INIT_I
  integer(dpi) :: cur_conf        = NOT_INIT_I

  type(configuration_guide), pointer :: configurations_guide(:)  !     => NULL ()

 integer(dpi)            :: jastr_pow_max_guide = 4 
 real(dp), pointer       :: conf_jastr_a_ee_guide (:,:,:)  => NULL ()
 real(dp), pointer       :: conf_jastr_beta_ee_guide (:,:) => NULL ()
 real(dp), pointer       :: conf_jastr_a_en_guide (:,:,:)  => NULL ()
 real(dp), pointer       :: conf_jastr_beta_en_guide (:,:) => NULL ()
 real(dp), pointer       :: conf_jastr_a_een_guide (:,:,:) => NULL ()

contains

! ===============================================================================
  subroutine jastrow_arrays_guide_bld
! -------------------------------------------------------------------------------
!
! Name           : jastrow_arrays_guide_bld
!
! Authors: R. Assaraf
! Date  : 10 Aug 2002
! -------------------------------------------------------------------------------
  use epfs_mod 

  implicit none
!i:o:
  
!local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: atoms_nb, configurations_guide_nb
  l_here = 'jastrow_arrays_guide_bld'


  if (header_exe) then 
 
    call object_create ('conf_jastr_a_ee_guide')
    call object_create ('conf_jastr_a_en_guide')
    call object_create ('conf_jastr_a_een_guide')
    call object_create ('conf_jastr_beta_ee_guide')
    call object_create ('conf_jastr_beta_en_guide')

    call object_needed ('atoms_charges')
    call object_needed ('configurations_guide')
   return

  endif

  call enter (l_here)

  configurations_guide_nb = size (configurations_guide)

  atoms_nb = size (atoms_charges)

  if (configurations_guide_nb == 0) call die (l_here, 'configurations_guide_nb=0')
  
  if (.not.associated (conf_jastr_a_ee_guide)) call alloc ('conf_jastr_a_ee_guide', &
       conf_jastr_a_ee_guide, configurations_guide_nb, 2, jastr_pow_max_guide)

  if (.not.associated (conf_jastr_beta_ee_guide)) call alloc ('conf_jastr_beta_ee_guide', &
       conf_jastr_beta_ee_guide, configurations_guide_nb, 2)

  if (.not.associated (conf_jastr_a_en_guide)) call alloc ('conf_jastr_a_en_guide', &
       conf_jastr_a_en_guide, configurations_guide_nb, jastr_pow_max_guide , atoms_nb)

  if (.not.associated (conf_jastr_beta_en_guide)) call alloc ('conf_jastr_beta_en_guide',&
       conf_jastr_beta_en_guide, configurations_guide_nb, atoms_nb)

  if (.not.associated (conf_jastr_a_een_guide)) call alloc ('conf_jastr_a_een_guide',&
       conf_jastr_a_een_guide, configurations_guide_nb, 2, atoms_nb)


  select case (configurations_guide(1)%jastr_type(1:3))
  
  case ('REA') 
     
     call conf_jastr_guide_read

  case ('ONE') 

    conf_jastr_a_en_guide = 0.d0
    conf_jastr_a_ee_guide = 0.d0
    conf_jastr_a_een_guide   = 0.d0
    conf_jastr_beta_en_guide = 1.d0    
    conf_jastr_beta_ee_guide = 1.d0

  case ('DEF') 
   
     call die (l_here, 'Jastrow DEFAULT: not implemented')

  case ('NON') 
 
     call die (l_here, 'Jastrow NONE: not implemented')
    
  case default
   
     write (logf,*) trim (l_here), '-e: jastrow >'  &
          +configurations_guide(1)%jastr_type  &
          +'<should be either read, one, none or default'
     call die (l_here, 'invalid option for jastrow')

  end select
 
  call exit (l_here)

  end subroutine jastrow_arrays_guide_bld

! =============================================================================
  subroutine conf_jastr_guide_read
! -----------------------------------------------------------------------------
!
! Name           : conf_jastr_guide_read
!
! Description    : read all conf_jastr file
!
! OverWrites     : conf_jastr
!
! Authors:X. Krokidis, F. Colonna
! Date  :28 Oct 2000
! -----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action 
  character(len=max_string_len_file_name)    :: f_name   
  integer(dpi)                               :: unit
  integer(dpi)                               :: conf_i, conf_read
  integer(dpi)                               :: configurations_guide_nb

! begin:
  l_here ='conf_jastr_guide_read'
  
  if (header_exe) then

    l_here = "conf_jastr_guide_read"
 
    return

  end if
  
  call enter (l_here) 

! check how many configurations_guide in input file: configurations_guide_nb
  call check_asso_conf_guide &
  ('configurations_guide', configurations_guide, 'configurations_guide_init', configurations_guide_init)

  configurations_guide_nb = size(configurations_guide)
  message = 'in '*l_here*' no configurations_guide found in input file'
  call require(message, configurations_guide_nb /= 0)

  include 'chksiz_configurations_guide.inc'

! open the jastrow_guide-in file
  f_type = 'jastrow_guide-in'//blanks
  unit = file_open_and_cat_from_type (f_type)

  if ( unit == 0) then
    message = 'in routine'*l_here*' jastrow_guide unit not found'
    call require (message, unit /= 0)
  end if

  f_name = files_names(unit)
  f_action = files_actions(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: jastrow file unit   =',unit    
   write (logf,*)trim(l_here),'-d: jastrow file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: jastrow file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: jastrow file action >',trim(f_action),'<'
   write (logf,*)trim(l_here),'-d: configurations_guide_nb   =',configurations_guide_nb
  end if

! allocate each jastrow array
!   call cur_conf_jastr_guide_alloc (configurations_guide_nb)
  
  do conf_i = 1, configurations_guide_nb

     conf_read = conf_i
     
! read each jastrow array
     call cur_conf_jastr_guide_guide_read (conf_i, unit)
  end do

  write (logf,*)trim(l_here),':  ',configurations_guide_nb,' configurations_guide read from:'
  write (logf,*)trim(l_here),': jastrow_guide file name   >',trim(f_name),'<'

  if(debug) then
   call conf_jastr_guide_print
  end if

  message ='in routine'*l_here* &
  ' nb of configurations_guide read ='*conf_read* &
  ' nb of configurations_guide expected='*configurations_guide_nb
  call require (message, conf_read == configurations_guide_nb)

  call file_close_by_unit (unit)

  call touch_bynam   ('conf_jastr_a_ee_guide')
  call touch_bynam   ('conf_jastr_a_en_guide')
  call touch_bynam   ('conf_jastr_a_een_guide')
  call touch_bynam   ('conf_jastr_beta_ee_guide')
  call touch_bynam   ('conf_jastr_beta_en_guide')

  call exit (l_here)

  end subroutine conf_jastr_guide_read

! ===============================================================================
  subroutine cur_conf_jastr_guide_guide_read (conf_i, unit)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_guide_read
!
! Argument (in ) : conf_i
! Argument (in ) : unit   
!
! Fonctionality  : Utility  
! 
! Description    : read one configuration jastrow parameters 
! Description    : from file of type "jastrow" (unit)
!
! Fills          : conf_jastr_beta_ee_guide (conf,lik)      
! Fills          : conf_jastr_beta_en_guide (conf,atom) 
! Fills          : conf_jastr_a_ee_guide (conf,lik, deg)
! Fills          : conf_jastr_a_en_guide (conf, deg, atom)
! Fills          : conf_jastr_a_een_guide (conf, lik, atom)
!
! OverWrites     : arrays conf_jastr for configuration conf_i
!
! Authors:X. Krokidis, F. Colonna
! Date  :28 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! Revision: 21 Jul 2002 by F. Colonna utility routine out of header
! Revision: 21 Au  2002 by R. Assaraf jastrow out of structure configurations_guide
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: conf_i
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here 
  character(len=max_string_len_file_name)    :: f_name   
  character(len=max_len_conf_name_guide)           :: c_name, cur_name
  character(len=max_len_atom_name)           :: a_name
  character(len=max_string_len)              :: stemp  
  character(len=max_string_len)              :: string 
  integer(dpi)                               :: l_atom, j, l_record
  integer(dpi)                               :: atoms_nb
  logical(bool)                               :: done
  integer(dpi)                               :: i

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi), pointer                        :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len

! header:
! if (header_exe) then

!   return

! end if

! begin:
  l_here='cur_conf_jastr_guide_guide_read'
  call enter (l_here)

  rewind (unit)

  f_name   = files_names(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: conf_i            =',conf_i  
   write (logf,*)trim(l_here),'-d: jastrow file unit =',unit    
   write (logf,*)trim(l_here),'-d: jastrow file name >',trim(f_name),'<'
  end if

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 

! fetch configuration conf_i:
  cur_name = configurations_guide(conf_i)%name

  if(debug) write (logf,*)trim(l_here),'-d: fetching current configuration # ', &
            conf_i,' named >',trim(cur_name),'<'

  if(cur_name(1:1) == '?') then 
   write (logf,*)trim(l_here), & 
   '-d: fetching not initialized current configuration # ', &
   conf_i,' named >',trim(cur_name),'< '
   call die (l_here, 'current fetched configuration not initialized')
  end if

  l_record = file_fetch_word (cur_name, 3, unit)
  c_name = cur_name

  if(debug) write (logf,*)trim(l_here),'-d: fetched record # ',l_record  

  l_atom = 0

  done = .false.
  do while (.not.done)

   stemp = blanks
   string = blanks
   read (unit,'(a)',iostat=iostat) stemp 

   if(iostat < 0) then

! eof
    message = 'end of file reading file >'*trim(files_names(unit))*'<'* &
    ' iostat='*iostat
    call die (l_here, message)
   
   else if(iostat /= 0) then

! i/o error
    message ='i/o error while reading file >'*trim(files_names(unit))*'<'* &
    ' iostat='*iostat
    call die (l_here, message)

   end if

   if(debug) write (logf,*)trim(l_here),'-d: reading stemp=',trim(stemp)  
   if(len_trim(stemp) == 0 ) cycle 

   string = string_to_upper (stemp)
   stemp  = adjustl (string)
! Improve
! split input record

   number_of_words = string_to_words_count (string,' ', word_max_len)
   call split_string (string, ' ', words, lenwrd, number_of_words, len(words))

   if(debug) write (logf,*)trim(l_here), '-d: string=',trim(string)  
   string = stemp
   if(debug) write (logf,*)trim(l_here), '-d: configuration # ',conf_i,' >',trim(c_name),'<'

   if(trim(words(1)) == 'BETA_EE') then

      conf_jastr_beta_ee_guide (conf_i,1) = string_to_double (words(3))

      conf_jastr_beta_ee_guide (conf_i,2) = string_to_double (words(4))
    
   else if(trim(words(1)) == 'BETA_EN') then
 
      conf_jastr_beta_en_guide (conf_i, l_atom)  = string_to_double (words(3))

   else if(trim(words(1)) == 'A_EE_LIKE') then
   i = 1
   do j = 1,jastr_pow_max_guide
         conf_jastr_a_ee_guide (conf_i,i,j) = string_to_double (words(j+2))
   enddo
   
   else if(trim(words(1)) == 'A_EE_UNLIKE') then
   i = 2
   do j = 1,jastr_pow_max_guide
      conf_jastr_a_ee_guide (conf_i,i,j) = string_to_double (words(j+2))
   enddo

   else if(trim(words(1)) == 'A_EN') then

    do j = 1, jastr_pow_max_guide
       conf_jastr_a_en_guide (conf_i, j, l_atom) = string_to_double (words(j+2))
    end do

   else if(trim(words(1)) == 'A_EEN') then

      conf_jastr_a_een_guide (conf_i, 1, l_atom) = string_to_double (words(3))

      conf_jastr_a_een_guide (conf_i, 2, l_atom)  = string_to_double (words(4))

   else if(trim(words(1)) == 'ATOM_NAME') then

    l_atom = l_atom + 1
    a_name = words(3)

    if(print_level > 1) write (logf,'(2a,i2,3a)')trim(l_here),  &
    ': atom  # ',l_atom,' >',trim(a_name),'< '

   else if(trim(words(1)) == 'END') then

    if(print_level > 0) write (logf,'(2a,i2,3a)')trim(l_here),  &
    ': configuration # ',conf_i,' >',trim(c_name),'< '

    message ='in '*l_here* &
    ' nb of atoms found='*l_atom* &
    ' nb of atoms expected='*atoms_nb
    call require (message, l_atom == atoms_nb)
   
    exit 

   else
   end if

  end do

   write (logf,*)trim(l_here),': configuration # ',conf_i,' read from:'
   write (logf,*)trim(l_here),': jastrow_guide file name >',trim(f_name),'<'

  if(debug) then
   call cur_conf_jastr_guide_print (conf_i)
  end if

! deallocate (words)

  call exit (l_here)

  end subroutine cur_conf_jastr_guide_guide_read

! ===============================================================================
  subroutine conf_jastr_guide_write
! -------------------------------------------------------------------------------
!
! Name           : conf_jastr_guide_write
!
! Description    : writes on file all jastrow's parameters (all configurations_guide)
!
! Authors:F.Colonna
! Date  :28 Oct 2000
! Revision: 16 Dec 2000 FCC possiblity to use same jastrow file
! Revision: 16 Dec 2000 FCC to read AND write
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: c
  integer(dpi)                               :: unit
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: configurations_guide_nb
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_title)        :: f_title
  character(len=max_string_len_file_type)    :: f_type

! begin:
  l_here = "conf_jastr_guide_write"
  call enter (l_here)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 

  if  (.not.associated(configurations_guide)) call die (l_here, 'configurations_guide not associated')

!  call check_asso_conf_guide &
!  ('configurations_guide', configurations_guide, 'configurations_guide_init', configurations_guide_init)


  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
   'configurations_guide', size (configurations_guide, 1), 1)

  configurations_guide_nb = system%configurations_guide_nb

  if(debug) write (logf,*)trim(l_here),' configurations_guide_nb=',configurations_guide_nb

! look at the jastrow-out file
  f_type = 'jastrow_guide-out'//blanks
  unit   = unit_from_type (f_type)

  if(unit .eq. 0) then
! look at the jastrow-inout file
   f_type = 'jastrow_guide-inout'//blanks
   unit   = unit_from_type (f_type)
  end if

  if(debug)write (logf,*)trim(l_here),'-d: jastrow file unit =',unit
  if(unit .eq. 0) then 
    call files_show
    message = 'in routine'*l_here*' jastrow_guide type not found'
    call die (l_here, message)
  end if

! open
  unit = file_open_and_cat_from_type (f_type)

  f_name   = files_names(unit)
  f_title  = files_titles(unit)
  f_action = files_actions(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: jastrow file unit   =',unit
   write (logf,*)trim(l_here),'-d: jastrow file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: jastrow file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: jastrow file action >',trim(f_action),'<'
   write (logf,*)trim(l_here),'-d: jastrow file title >',trim(f_title),'<'
  end if

  call file_header_write (unit)

  write(unit,'(a,i5)')'number of nuclei         = ',atoms_nb
  write(unit,'(a,i5)')'number of configurations_guide = ',configurations_guide_nb
  write(unit,'(a)')    'end_of_header '
  
  do c = 1, configurations_guide_nb
   call cur_conf_jastr_guide_write (c, unit)
  end do

  write(unit,'(a)')'eof'

  call file_close_by_unit (unit)

  write (logf,'(2a,i2,a,/,3a)')trim(l_here),': jastrow_guide parameters for ', &
  configurations_guide_nb,' configurations_guide have been written on file:',    &
  '>',trim(f_name),'<'

  call exit (l_here)

 end subroutine conf_jastr_guide_write 

! ===============================================================================
  subroutine cur_conf_jastr_guide_write (conf_i, unit)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_write
!
! Argument (in ) : conf_i
! Argument (in ) : unit
!
! Description    : writes current configuration jastrows on unit
!
! Authors:F.Colonna
! Date  :28 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: conf_i
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: a
  integer(dpi)                               :: atoms_nb

! begin:
  l_here = "cur_conf_jastr_guide_write"
  call enter (l_here)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 

!  call check_asso_cur_conf_jastr_guide (conf_i)

  if (.not.associated (conf_jastr_a_ee_guide)) &
      call provide_bynam('conf_jastr_a_ee_guide')

  if(configurations_guide(conf_i)%name == '?') then
    write (logf,*)trim(l_here),'-d: configuration # ',conf_i,' not initialized'
    call die (l_here, 'configuration not initialized')
  end if

  write(unit,'(2a)')'CONFIGURATION = ',trim(configurations_guide(conf_i)%name)
  write(unit,'(a)')' '

  write(unit,'(a,(2e25.15))')         &
  'BETA_EE     = ', conf_jastr_beta_ee_guide (conf_i,:)

  write(unit,'(a,(4e25.15))')         &
  'A_EE_LIKE   = ', conf_jastr_a_ee_guide (conf_i,1,:)

  write(unit,'(a,(4e25.15))')         &
  'A_EE_UNLIKE = ', conf_jastr_a_ee_guide (conf_i,2,:)
 
  do a = 1,atoms_nb

    write(unit,'(2a)')'ATOM_NAME   = ',trim(atoms(a)%name)
    write(unit,'(a,(e25.15))')         &
    'BETA_EN     = ', conf_jastr_beta_en_guide (conf_i,a) 

    write(unit,'(a,(4e25.15))')        &
    'A_EN        = ', conf_jastr_a_en_guide (conf_i,:,a)

    write(unit,'(a,(2e25.15))')         &
    'A_EEN       = ', conf_jastr_a_een_guide (conf_i,:,a)

  enddo

  write(unit,'(a)')' '
  write(unit,'(a)')'END'
  write(unit,'(a)')' '

  call exit (l_here)

 end subroutine cur_conf_jastr_guide_write 

! ===============================================================================
  subroutine conf_jastr_guide_print_res (text)
! -------------------------------------------------------------------------------
!
! Name           : conf_jastr_guide_print_res
!
! Argument (in ) : text  
!
! Description    : prints out current configuration jastrows    
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: text   

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string
  integer(dpi)                               :: conf_i, configurations_guide_nb
  integer(dpi)                               :: atom_i, atoms_nb
  integer(dpi)                               :: debut, fin

! begin:
  l_here = "conf_jastr_guide_print_res"
  call enter (l_here)

  debut  = 40 - (len_trim(text)/2)
  fin    = 40 + (len_trim(text)/2)

  if(debug) then
    write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
    write (logf,*) trim(l_here),'-d: debut=', debut,' fin=',fin
  end if

  atoms_nb = size(atoms)
  configurations_guide_nb = size(configurations_guide)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
    write (logf,*)trim(l_here),'-d: configuration =',configurations_guide_nb
  end if

  string                 = dashes
  string (debut-1:debut) = " "
  string (debut:)        = trim(text)//' '//dashes

! message to log file
  write (logf,'(1x,2a,a80)') trim(l_here),': ',string

  write (res,'(a)') ' '
  write (res,'(a80)') string
  write (res,'(a)') ' '
!  write (res,'(t1,a,       t21,a,      t64,a)') &
!              'conf # name', 'type','beta_ee','aa_ee_like','a_ee_unlike'
!  write (res,'(a)') ' '

 do conf_i = 1, configurations_guide_nb

  write(res,'(2a)')'CONFIGURATION = ',trim(configurations_guide(conf_i)%name)
  write(res,'(a)')' '

  write(res,'(a,(2e25.15))')         &
  'BETA_EE     = ', conf_jastr_beta_ee_guide (conf_i,:)

  write(res,'(a,(4e25.15))')         &
  'A_EE_LIKE   = ', conf_jastr_a_ee_guide (conf_i,1,:)

  write(res,'(a,(4e25.15))')         &
  'A_EE_UNLIKE = ', conf_jastr_a_ee_guide (conf_i,2,:)
 
  do atom_i = 1,atoms_nb

    write(res,'(2a)')'ATOM_NAME   = ',trim(atoms(atom_i)%name)
    write(res,'(a,(e25.15))')         &
    'BETA_EN     = ', conf_jastr_beta_en_guide (conf_i,atom_i) 

    write(res,'(a,(4e25.15))')        &
    'A_EN        = ', conf_jastr_a_en_guide (conf_i,:,atom_i)

    write(res,'(a,(2e25.15))')         &
    'A_EEN       = ', conf_jastr_a_een_guide (conf_i,:,atom_i)

  enddo ! do atom_i
 
enddo! do conf_i

  call exit (l_here)

  end subroutine conf_jastr_guide_print_res 

! ===============================================================================
  subroutine configuration_guide_menu
! -------------------------------------------------------------------------------
!
! Name           : configuration_guide_menu
!
! Description    : collects configuration information from menu
!
! -- Creates     : configurations_guide(conf)%conf_deter (det)
! -- Creates     : configurations_guide(conf)%conf_deter(det)%alfa_orbitals (elec_a)
! -- Creates     : configurations_guide(conf)%conf_deter(det)%beta_orbitals (elec_b)
!
! Authors:X. Krokidis, F. Colonna
! Date  :18 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool)                    :: done    =.false.
  logical(bool)                    :: l_print =.false.
 
  character(len=max_len_conf_name_guide) :: l_conf_name
  integer(dpi)                     :: n_det
  character(len=max_len_conf_type_guide) :: l_jastr_type 
! real(dp)                         :: dp_value
  real(dp), pointer                :: weight => NULL ()
! begin:
  l_here='configuration_guide_menu'
  call enter (l_here)

! initialize:
  l_jastr_type = blanks
  l_conf_name  = blanks

  call check_asso_conf_guide &
       ('configurations_guide', configurations_guide, 'configurations_guide_init', configurations_guide_init)
  if (.not.associated (weight)) allocate (weight)
  if (.not.associated (configurations_guide_weight)) then
     allocate (configurations_guide_weight (size (configurations_guide)))
     call catalog ('configurations_guide_weight', configurations_guide_weight)
  endif

! count:
  cur_conf = cur_conf + 1

  if(debug) write (logf,*)trim(l_here),'-d: cur_conf=',cur_conf  

  done = .false.
  call next_command_get ('config')
  loop_while: do while (.not.done)
   call next_word_get ('config')
   if(debug)write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
   if(cur_word(1:4) .eq. 'help') then

    write (logf,'(a)') ' config-h: configuration setup'
    write (logf,'(a)') ' config-h: all Jastrow fields for this current'
    write (logf,'(a)') ' config-h: configurations_guide should be filled l_here'
    write (logf,'(a)') ' config-h: syntax:'
    write (logf,'(a)') ' config-h: con{figuration}'
    write (logf,'(a)') ' config-h:   help'
    write (logf,'(a)') ' config-h:   det{erminant}   [command] [repeat]'
    write (logf,'(a)') ' config-h:   jas{trow}     = [string] non{e}:def{ault}:rea{d}:one'
    write (logf,'(a)') ' config-h:   pow{er_jastrow} = [real]'
    write (logf,'(a)') ' config-h:   nam{e}        = [string_with_blanks]'
    write (logf,'(a)') ' config-h:   pri{nt}       = [boolean]'
    write (logf,'(a)') ' config-h:   wei{ght}      = [real]'
    write (logf,'(a)') ' config-h: end'
    write (logf,'(a)') ' config-h: example'
    write (logf,'(a)') ' config-h: configuration'
    write (logf,'(a)') ' config-h:   determinant ... end'
    write (logf,'(a)') ' config-h:   jastrow = read'
    write (logf,'(a)') ' config-h:   name = "GROUND_STATE"'
    write (logf,'(a)') ' config-h:   weight = 1.1         '
    write (logf,'(a)') ' config-h:  end_con'

  else if(token_is ('jas', l_jastr_type)) then
    configurations_guide(cur_conf)%jastr_type = string_to_upper (l_jastr_type)

  else if(token_is ('pow',jastr_pow_max_guide)) then
  
  else if(cur_word(1:3) .eq. 'det') then
    call determinant_guide_menu
    
! read current determinant info:
! Improve : should be hidden

    n_det = size(determinants_guide)

    configurations_guide(cur_conf)%n_determinant = n_det  
    if(debug) write (logf,*) trim(l_here),': n_determinant=',  &
             configurations_guide(cur_conf)%n_determinant

    if(.not.associated(configurations_guide(cur_conf)%conf_deter)) then

      write (logf,*)trim(l_here), &
      ': configurations_guide(',cur_conf,')%conf_deter not yet associated'
      write (logf,*) trim(l_here),': allocate (configurations_guide(', &
               cur_conf,')%conf_deter(',n_det,'))'
      
      allocate (configurations_guide(cur_conf)%conf_deter(n_det), stat = all_err)
      
      if(all_err /= 0)  &
           call die (l_here, 'allocate (configurations_guide(cur_conf)%conf_deter(n_det))')

   end if

   configurations_guide(cur_conf)%conf_deter = determinants_guide

   if (.not.associated (configurations_guide(cur_conf)%deter_weight)) then 
      call objects_name_add_once('det_weights_'+cur_conf)
      call alloc ('det_weights_'+cur_conf, configurations_guide(cur_conf)%deter_weight, n_det)
   endif
   configurations_guide(cur_conf)%deter_weight = determinants_guide_weights
  
   else if(token_is ('nam', l_conf_name)) then
   configurations_guide(cur_conf)%name = string_to_upper (l_conf_name)

   else if(token_is ('pri', l_print)) then

   else if(token_is ('wei',  weight)) then
    configurations_guide_weight (cur_conf) = weight
    if(debug) write (logf,*) trim(l_here),'-d: conf_weight=',configurations_guide_weight (cur_conf)

   else
    call menu_exit (l_here, done)

   end if

  end do loop_while

! if no name is given. build one like 'CONF_n':
  if(configurations_guide(cur_conf)%name(1:1) == '?') then

    configurations_guide(cur_conf)%name = cur_conf_jastr_guide_name_bld (cur_conf)
 
    write (logf,*)trim(l_here),': configurations_guide (',cur_conf,')%name ',  &
          ' re-set as >',trim(configurations_guide(cur_conf)%name),'<'
  end if

    write (logf,*)trim(l_here),': conf # ',cur_conf, &
    ' named >',trim(configurations_guide(cur_conf)%name),'<', &
    ' jastr_type >',trim(configurations_guide(cur_conf)%jastr_type),'<'

  if(debug) call cur_conf_jastr_guide_print (cur_conf)
  if(l_print) then 
    call cur_conf_jastr_guide_print (cur_conf)
    call cur_conf_deter_guide_print (cur_conf)
  end if

! now temporarily allocated determinants_guide object must be deleted

  if(associated(determinants_guide)) then 
    deallocate (determinants_guide)
    write (logf,*)trim(l_here),': local object >determinants_guide< deallocated'
  end if

  call exit (l_here)

  end subroutine configuration_guide_menu

! ===============================================================================
  subroutine cur_conf_deter_guide_print (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_deter_guide_print
!
! Argument (in ) : conf_i
!
! Description    : prints out current configuration determinants_guide    
!
! Authors: F.Colonna
! Date   : 23 Nov 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: conf_i 

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: a, n_deter

! begin:
  l_here = "cur_conf_deter_guide_print"
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: configuration = ',conf_i
  
  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
   'configurations_guide', size (configurations_guide, 1), 1)

! call check_asso_cur_conf_deter_guide (conf_i)

! Improve end

  if(configurations_guide(conf_i)%name == '?') then 
    write (logf,*)trim(l_here),'-d: configuration # ',conf_i,' not initialized'
    call die (l_here, 'configuration not initialized')
  end if

  n_deter = size(configurations_guide(conf_i)%conf_deter)

  write (logf,'(2a,i3,3a)')            &
  trim(l_here),': configuration # ',conf_i, &
   ' name "',trim(configurations_guide(conf_i)%name),'"'

  write (logf,'(2a,i3,a,i3)')           &
  trim(l_here),': configuration # ',conf_i,' nb of determinants_guide =',n_deter

   do a = 1,n_deter
    call determinant_guide_print (configurations_guide(conf_i)%conf_deter(a))
   enddo

  call exit (l_here)

  end subroutine cur_conf_deter_guide_print 

! ===============================================================================
  subroutine determinant_guide_print (deter)
! -------------------------------------------------------------------------------
!
! Name           : determinant_guide_print
!
! Argument (in ) : deter
!
! Description    : prints out current determinant    
!
! Authors: F.Colonna
! Date   : 23 Nov 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  type (determinant_guide)   :: deter 

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here = "determinant_guide_print"
  call enter (l_here)

! call check_asso_cur_conf_deter_guide (conf_i)

  message=' in routine'*routines_321* &
          ' object "deter%alfa_orbitals" probably not associated'
  call require (message, associated(deter%alfa_orbitals) .or. &
                         associated(deter%beta_orbitals))

  message = l_here+': determinant_weight ='+deter%weight
  write (logf,'(a)') trim(message)
  call printarray (l_here, deter%alfa_orbitals, 'alfa_orbitals')
  call printarray (l_here, deter%beta_orbitals, 'beta_orbitals')

  call exit (l_here)

  end subroutine determinant_guide_print 

! ===============================================================================
  function cur_conf_jastr_guide_name_bld (conf_i) 
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_name_bld
!
! Argument (in ) : conf_i
!
! Description    : returns a string like 'CONF_l'     
!
! Authors:F.Colonna
! Date  :27 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! function:
  character(len=max_len_conf_name_guide) :: cur_conf_jastr_guide_name_bld

! i/o:
  integer(dpi)      :: conf_i 

! local:  
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string 

! begin:
  l_here ='cur_conf_jastr_guide_name_bld' 
  call enter (l_here)

  string = 'CONF_'//adjustl(encode(conf_i))

  cur_conf_jastr_guide_name_bld = trim(string)  

  if(debug) write (logf,*)trim(l_here),'-d: cur_conf # ',conf_i
  if(debug) write (logf,*)trim(l_here),'-d: cur_conf name >',trim(string),'<'

  call exit (l_here)

  end function cur_conf_jastr_guide_name_bld

! ===============================================================================
  subroutine configurations_guide_init
! -------------------------------------------------------------------------------
!
! Name           : configurations_guide_init
!
! Description    : initialize configuration information
!
! Creates        : configurations_guide (conf)
!
! Authors:X. Krokidis, F. Colonna
! Date  :18 Oct 2000
! Revision :27 Dec 2000 FCC string_head_inside_cmd_count
! -------------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: configurations_guide_nb

! header:

  if (header_exe) then

    l_here = "configurations_guide_init"
!    call object_create ('configurations_guide')
    return

  end if

! begin:
  call enter (l_here)

  if(associated(configurations_guide)) then
   call exit(l_here)
   return ! Improve
  endif

  if(debug) write (logf,*)trim(l_here),'-d: cur_command >',trim(cur_command),'<'
  if(debug) write (logf,*)trim(l_here),'-d: cur_cmd_nest_lev=',cur_cmd_nest_lev
  if(debug) write (logf,*)trim(l_here),'-d: command_pos=',command_pos

  if(cur_cmd_nest_lev < 1) then
  message ='in routines '*routines_21*CR*' cur_cmd_nest_lev='* &
           cur_cmd_nest_lev* &
           ' probably "call next_command_get" is missing'*CR &
           *' run "perl check_new_command.pl" on all fortran'
  call require (message, cur_cmd_nest_lev .ge. 1)
  end if

  if(debug) write (logf,*)trim(l_here),'-d: cur_command_name_stack "',&
     trim(cur_command_name_stack(cur_cmd_nest_lev)),'"'

! count how many "configuration" words are in input_words delimited
! by the "gui" command whose position is defined by the stack

  configurations_guide_nb = string_head_inside_cmd_count ('con','gui','command')
  system%configurations_guide_nb = configurations_guide_nb

  if(debug) write (logf,*)trim(l_here),'-d: configurations_guide_nb=',configurations_guide_nb

  if(configurations_guide_nb <= 0) then
  call require ('in routine '//trim(l_here)//' configurations_guide_nb = 0', &
                configurations_guide_nb .ne. 0)
  end if

! allocations:

  if(.not.associated(configurations_guide)) then
    if(trace_depend) then
      write (logf,*)trim(l_here),': configurations_guide not yet associated'
      write (logf,*)trim(l_here),': allocate (configurations_guide(',configurations_guide_nb,'))'
    end if
      allocate (configurations_guide(configurations_guide_nb), stat = all_err)
      if(all_err /= 0)  &
      call die (l_here, 'allocate (configurations_guide(configurations_guide_nb))')
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': configurations_guide left in memory'
  end if

! end allocations

! initialize:
  configurations_guide%name = NOT_INIT_S
  cur_conf = 0

! check that atoms are defined
  call check_asso_atom ('atoms', atoms, 'atoms_init', atoms_init)

! deallocate (l_cmd_names)

  call exit (l_here)

  end subroutine configurations_guide_init

! ===============================================================================
  subroutine check_asso_conf_guide (object_name, object, routine_name, routine)
! -------------------------------------------------------------------------------
!
! Name           : check_asso_conf_guide
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :18 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*),   intent(in) :: object_name, routine_name
  type (configuration_guide), pointer  :: object(:)
  external :: routine

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='check_asso_conf_guide'
  call enter (l_here, 3)

  if(debug) then
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'
  end if

  if(.not.associated(object)) then
    if(trace_depend) then
      write (logf,*)trim(l_here),': ',trim(object_name),' not associated'
      write (logf,*)trim(l_here),': calling routine ',trim(routine_name)
    end if
    call routine 
  else 
    if(trace_depend) write (logf,*)trim(l_here),': ',trim(object_name),' already associated'
  end if

  call exit (l_here, 3)

  end subroutine check_asso_conf_guide


! ===============================================================================
  subroutine check_asso_conf_guide_all
! -------------------------------------------------------------------------------
!
! Name           : check_asso_conf_guide_all
!
!
! Description    : check association for all configurations_guide 
! Description    : and configurations_guide jastrows    
!
! Authors: F.Colonna
! Date   : 21 Nov 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
!  integer(dpi)                               :: c
  integer(dpi)                               :: configurations_guide_nb

! begin:
  l_here = "check_asso_conf_guide_all"
  call enter (l_here)

  call check_asso_conf_guide &
  ('configurations_guide', configurations_guide, 'configurations_guide_init', configurations_guide_init)
  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
   'configurations_guide', size (configurations_guide, 1), 1)

  configurations_guide_nb = system%configurations_guide_nb

  if(debug) write (logf,*)trim(l_here),' configurations_guide_nb=',configurations_guide_nb

  call exit (l_here)

  end subroutine check_asso_conf_guide_all 

! ===============================================================================
  subroutine configuration_all_print_print
! -------------------------------------------------------------------------------
!
! Name           : configuration_all_print_print (deter)
!
! Description    : prints out all configurations_guide
!
! Authors: F.Colonna
! Date   : 27 Dec 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)  :: c

! begin:
  l_here = "configuration_all_print_print"
  call enter (l_here)

  write (logf,*)trim(l_here),': total number of configurations_guide= ',size(configurations_guide)

  do c = 1, size(configurations_guide)
   write (logf,*)trim(l_here),': configuration # ',c
   write (logf,*)trim(l_here),': name >',trim(configurations_guide(c)%name),'<'
   write (logf,*)trim(l_here),': weight =',configurations_guide(c)%weight
   write (logf,*)trim(l_here),': jastr_type >',trim(configurations_guide(c)%jastr_type),'<'
   call cur_conf_jastr_guide_print (c)
   write (logf,*)trim(l_here),': determinants_guide:'
   call cur_conf_deter_guide_print (c)
  end do

  call exit (l_here)

  end subroutine configuration_all_print_print


! ===============================================================================
  subroutine alfa_orbitals_guide_init_2 (orbitals, n_orb)
! -------------------------------------------------------------------------------
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: n_orb
  integer(dpi), pointer :: orbitals(:)  

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here = 'alfa_orbitals_guide_init_2'
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: n_orb=',n_orb     

  call require ('in routine'//trim(l_here)//' n_orb = 0', n_orb .ne. 0)

  allocate (orbitals(n_orb), stat = all_err)
  if(all_err /= 0) call die (l_here,'allocate (alfa_orbitals(n_orb))')

  call message_out &
  (l_here, trim(encode(n_orb))//' orbitals allocated')

  call exit (l_here)

  end subroutine alfa_orbitals_guide_init_2

! ===============================================================================
  subroutine beta_orbitals_guide_init_2 (orbitals, n_orb)
! -------------------------------------------------------------------------------
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: n_orb
  integer(dpi), pointer :: orbitals(:)  

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here = 'beta_orbitals_guide_init_2'
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: n_orb=',n_orb     

  call require ('in routine'//trim(l_here)//' n_orb = 0', n_orb .ne. 0)

  allocate (orbitals(n_orb), stat = all_err)
  if(all_err /= 0) call die (l_here,'allocate (beta_orbitals(n_orb))')

  call message_out &
  (l_here, trim(encode(n_orb))//' orbitals allocated')

  call exit (l_here)

  end subroutine beta_orbitals_guide_init_2

! all these routines are garbage

! ===============================================================================
!  subroutine cur_conf_jastr_guide_alloc (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_alloc
!
! Fonctionality  : Utility  
!
! Argument (in ) : conf_i
!
! Allocates      : configurations_guide(conf)%conf_jastr%beta_ee (lik)      
! Allocates      : configurations_guide(conf)%conf_jastr%beta_en (atom) 
! Allocates      : configurations_guide(conf)%conf_jastr%a_ee (lik, deg)
! Allocates      : configurations_guide(conf)%conf_jastr%a_en (deg, atom)
! Allocates      : configurations_guide(conf)%conf_jastr%a_een (lik, atom)
!
! Authors:X. Krokidis, F. Colonna
! Date  :25 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! Revision: 21 Jul 2002 by F. Colonna utility routine out of header
! -------------------------------------------------------------------------------
!  implicit none 

! i/o:
!  integer(dpi), intent(in)                   :: conf_i

! local:
!  character(len=max_string_len_routine_name) :: l_here = "?"
!  integer(dpi)                               :: atoms_nb
 
! header:
!  call header_die

! begin:
!  l_here='cur_conf_jastr_guide_alloc'
!  call enter (l_here)

! check atoms array is allocated and filled
!  call check_asso_atom ('atoms', atoms, 'atoms_init', atoms_init)

!  atoms_nb = size(atoms)

!  call require ('in routine'*trim(l_here)*' conf_i = 0', conf_i /= 0)

!  if(debug) then
!   write (logf,*)trim(l_here),'-d: conf_i=', conf_i
!   write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb
!  end if

!  call require ('in routine'//trim(l_here)//' atoms_nb = 0', &
!                atoms_nb /= 0)

! allocate: 
!  call allocation ('configurations_guide(conf_i)%conf_jastr%BETA_EE',  &
!                   configurations_guide(conf_i)%conf_jastr%BETA_EE, 2)

!  call allocation ('configurations_guide(conf_i)%conf_jastr%BETA_EN',  &
!                   configurations_guide(conf_i)%conf_jastr%BETA_EN, atoms_nb)

!  call allocation ('configurations_guide(conf_i)%conf_jastr%A_EE',  &
!                   configurations_guide(conf_i)%conf_jastr%A_EE, 2,4)

!  call allocation ('configurations_guide(conf_i)%conf_jastr%A_EN',  &
!                   configurations_guide(conf_i)%conf_jastr%A_EN, 4, atoms_nb)

!  call allocation ('configurations_guide(conf_i)%conf_jastr%A_EEN',  &
!                   configurations_guide(conf_i)%conf_jastr%A_EEN, 2, atoms_nb)


!   call allocation ('conf_jastr_beta_ee_guide', conf_jastr_beta_ee_guide, conf_i, 2)
!   call allocation ('conf_jastr_beta_en_guide', conf_jastr_beta_en_guide, conf_i, atoms_nb)
!   call allocation ('conf_jastr_a_ee_guide', conf_jastr_a_ee_guide, conf_i, 2, 4)

!   call allocation ('conf_jastr_a_en_guide', conf_jastr_a_en_guide, conf_i,  4, atoms_nb)

!   call allocation ('conf_jastr_a_een_guide', conf_jastr_a_een_guide, conf_i, 2, atoms_nb)
!  call exit (l_here)

!  end subroutine cur_conf_jastr_guide_alloc

! ===============================================================================
  subroutine conf_jastr_guide_print
! -------------------------------------------------------------------------------
!
! Name           : conf_jastr_guide_print
!
! Description    : prints out all configurations_guide jastrows    
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: c
  integer(dpi)                               :: configurations_guide_nb

! begin:
  l_here = "conf_jastr_guide_print"
  call enter (l_here)

 ! call check_asso_conf_guide_all 

  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
   'configurations_guide', size (configurations_guide, 1), 1)

  configurations_guide_nb = system%configurations_guide_nb

  if(debug) write (logf,*)trim(l_here),'-d: configurations_guide_nb=',configurations_guide_nb

  do c = 1, configurations_guide_nb
   call cur_conf_jastr_guide_print (c)
  end do

  call exit (l_here)

  end subroutine conf_jastr_guide_print 

! ===============================================================================
  subroutine cur_conf_jastr_guide_print (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_print
!
! Argument (in ) : conf_i
!
! Description    : prints out current configuration jastrows    
!
! Authors:F.Colonna
! Date  :05 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: conf_i 

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: a
  integer(dpi)                               :: atoms_nb

! begin:
  l_here = "cur_conf_jastr_guide_print"
  call enter (l_here)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
    write (logf,*)trim(l_here),'-d: configuration = ',conf_i
  end if

  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
   'configurations_guide', size (configurations_guide, 1), 1)

!  call check_asso_cur_conf_jastr_guide (conf_i)

! Improve end

  if(configurations_guide(conf_i)%name == '?') then 
    write (logf,*)trim(l_here),'-d: configuration # ',conf_i,' not initialized'
    call die (l_here, 'configuration not initialized')
  end if

  write (logf,'(2a,i3,3a)')            &
  trim(l_here),': configuration # ',conf_i, &
   ' name "',trim(configurations_guide(conf_i)%name),'"'

  write (logf,'(2a,i3,3a)')            &
  trim(l_here),': configuration # ',conf_i, &
   ' jastr_type "',trim(configurations_guide(conf_i)%jastr_type),'"'

  write (logf,'(2a,(2e25.15))')         &
   trim(l_here),':     BETA_EE ',conf_jastr_beta_ee_guide (conf_i,:)

  write (logf,'(2a,(4e25.15))')         &
  trim(l_here),':   A_EE_LIKE ',conf_jastr_a_ee_guide(conf_i,1,:) 
  write (logf,'(2a,(4e25.15))')         &
  trim(l_here),': A_EE_UNLIKE ',conf_jastr_a_ee_guide (conf_i,2,:) 

   do a = 1,atoms_nb

    write (logf,'(3a)') trim(l_here),': atom >',trim(atoms(a)%name),'<'

    write (logf,'(2a,(e25.15))')         &
    trim(l_here),':     BETA_EN ',conf_jastr_beta_en_guide(conf_i,a)

    write (logf,'(2a,(4e25.15))')        &
    trim(l_here),':        A_EN ',conf_jastr_a_en_guide(conf_i,:,a)

    write (logf,'(2a,(2e25.15))')        &
    trim(l_here),':       A_EEN ',conf_jastr_a_een_guide(conf_i,:,a)

   enddo

  call exit (l_here)

  end subroutine cur_conf_jastr_guide_print 

! ===============================================================================
  subroutine cur_conf_jastr_guide_bld (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_bld
!
! Improve here should be all allocations
!
! Description    : build jastrow for configuration conf_i
! Description    : according to configurations_guide(conf)%jastr_type
! Description    : it is read, initialized or skipped
!
! Authors:X. Krokidis, F. Colonna
! Date  :25 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: conf_i

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action 
  character(len=max_len_conf_type_guide)           :: l_jastr_type 

! begin:
  l_here ='cur_conf_jastr_guide_bld'
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: configuration # ',conf_i,' (conf_i)'

  l_jastr_type = string_to_upper(configurations_guide(conf_i)%jastr_type)
  if(debug)write (logf,*)trim(l_here), &
  '-d: configuration jastr_type >',trim(l_jastr_type),'<'

  select case (l_jastr_type(1:4) )
  case ("NONE")

!   call cur_conf_jastr_guide_none_bld (conf_i) ! Improve skip this. no more used

  case ("DEFA")

!   call cur_conf_jastr_guide_def_bld (conf_i)

  case ("READ")

! Improve => read header
! open the jastrow-in file

    f_type = 'jastrow_guide-in'//blanks
    unit = file_open_and_cat_from_type (f_type)

    message = 'in '*l_here*' jastrow_guide unit not found'
    call require (message, unit /= 0)

    f_name = files_names(unit)
    f_action = files_actions(unit)

    if(debug) then
     write (logf,*)trim(l_here),'-d: jastrow file unit   =',unit
     write (logf,*)trim(l_here),'-d: jastrow file name   >',trim(f_name),'<'
     write (logf,*)trim(l_here),'-d: jastrow file type   >',trim(f_type),'<'
     write (logf,*)trim(l_here),'-d: jastrow file action >',trim(f_action),'<'
    end if

! end Improve

     call cur_conf_jastr_guide_guide_read (conf_i, unit)

  case default

    message = ' in input Jastrow type => '*l_jastr_type*'< it must be:' &
    *' one of Jastrow = :READ:NONE:DEFA: '
    call die(l_here, message)

  end select

  call exit (l_here)

  end subroutine cur_conf_jastr_guide_bld

! ===============================================================================
!  subroutine cur_conf_jastr_guide_def_bld (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_def_bld
!
! Argument (in ) : conf_i
!
! Description    : build jastrow default for one configuration conf_i
!
! Authors:X. Krokidis
! Date  :25 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! -------------------------------------------------------------------------------
!  implicit none

! i/o:
!  integer(dpi)                               :: conf_i

! local:
!  character(len=max_string_len_routine_name) :: l_here
!  character(len=max_string_len)              :: l_string
!  integer(dpi)                               :: a
!  integer(dpi)                               :: atoms_nb
!  integer(dpi)                               :: configurations_guide_nb

! begin:
!  l_here ='cur_conf_jastr_guide_def_bld'
!  call enter (l_here)

!  call check_asso_conf_guide &
!  ('configurations_guide', configurations_guide, 'configurations_guide_init', configurations_guide_init)

!  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
!   'configurations_guide', size (configurations_guide, 1), 1)

!  configurations_guide_nb = system%configurations_guide_nb

! check atoms array is allocated and filled
!  call check_asso_atom ('atoms', atoms, 'die', die) 
!  atoms_nb = size(atoms)
!  if(debug) write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 

!  if(debug)write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
!  if(debug)write (logf,*)trim(l_here),'-d: conf_i = ',conf_i
!  if(debug)write (logf,*)trim(l_here),'-d: jastr_type =',configurations_guide(conf_i)%jastr_type 

!  l_string = 'configurations_guide('+conf_i+')%conf_jastr%BETA_EE'
!  call allocation (l_string, configurations_guide(conf_i)%conf_jastr%BETA_EE, 2)

!  l_string = 'configurations_guide('+conf_i+')%conf_jastr%BETA_EN'
!  call allocation (l_string, configurations_guide(conf_i)%conf_jastr%BETA_EN, atoms_nb)

!  l_string = 'configurations_guide('+conf_i+')%conf_jastr%A_EE'
!  call allocation (l_string, configurations_guide(conf_i)%conf_jastr%A_EE,2,4)
 
!  l_string = 'configurations_guide('+conf_i+')%conf_jastr%A_EN'
!  call allocation (l_string, configurations_guide(conf_i)%conf_jastr%A_EN, 4, atoms_nb)

!  l_string = 'configurations_guide('+conf_i+')%conf_jastr%A_EEN'
!  call allocation (l_string, configurations_guide(conf_i)%conf_jastr%A_EEN, 2, atoms_nb)

!  configurations_guide(conf_i)%conf_jastr%BETA_EE = ONE
!  configurations_guide(conf_i)%conf_jastr%A_EE    = ONE

!  do a = 1,atoms_nb
!   configurations_guide(conf_i)%conf_jastr%BETA_EN(a) = ONE
!   configurations_guide(conf_i)%conf_jastr%A_EN(:,a)  = ONE
!   configurations_guide(conf_i)%conf_jastr%A_EEN(:,a) = ONE
!  end do

!  call exit (l_here)

!  end subroutine cur_conf_jastr_guide_def_bld

! ===============================================================================
  subroutine cur_conf_jastr_guide_none_bld (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : cur_conf_jastr_guide_none_bld
!
! Argument (in ) : conf_i
!
! Description    : build NO jastrow for each configuration
!
! Authors:X. Krokidis, F. Colonna
! Date  :25 Oct 2000
! Revision: 15 Sep 2001 by F. Colonna check_asso_atom 
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                   :: conf_i

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: atoms_nb
  integer(dpi)                               :: configurations_guide_nb
  integer(dpi)                               :: a

! begin:
  l_here ='cur_conf_jastr_guide_none_bld'
  call enter (l_here)

! check atoms array is allocated and filled
  call check_asso_atom ('atoms', atoms, 'die', die) 
  atoms_nb = size(atoms)
  if(debug) write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 

  call check_asso_conf_guide &
  ('configurations_guide', configurations_guide, 'configurations_guide_init', configurations_guide_init)

  call  check_size (l_here, 'system%configurations_guide_nb', system%configurations_guide_nb, &
   'configurations_guide', size (configurations_guide, 1), 1)

  configurations_guide_nb = system%configurations_guide_nb

  if(debug)write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
  if(debug)write (logf,*)trim(l_here),'-d: conf_i = ',conf_i

  conf_jastr_beta_ee_guide (conf_i,:) = ONE
  conf_jastr_a_ee_guide    = ZERO

  do a = 1,atoms_nb
   conf_jastr_beta_en_guide(conf_i,a) = ONE
   conf_jastr_a_en_guide(conf_i,:,a)  = ZERO
   conf_jastr_a_een_guide(conf_i,:,a) = ZERO
  end do

  call exit (l_here)

  end subroutine cur_conf_jastr_guide_none_bld

! ===============================================================================
!  subroutine check_asso_cur_conf_jastr_guide (conf_i)
! -------------------------------------------------------------------------------
!
! Name           : check_asso_cur_conf_jastr_guide
!
! Argument (in ) : conf_i
!
! Description    : builds conf_jastr for configuration conf_i
!
! Authors:F.Colonna
! Date  :26 Oct 2000
! -------------------------------------------------------------------------------
!  implicit none

! i/o
!  integer(dpi)  :: conf_i

! local:
!  character(len=max_string_len_routine_name) :: l_here

! begin:
!  l_here ='check_asso_cur_conf_jastr_guide'
!  call enter (l_here,2)

!  call check_association ('configurations_guide(conf_i)%conf_jastr%BETA_EE', &
!                   configurations_guide(conf_i)%conf_jastr%BETA_EE,          &
!                   'cur_conf_jastr_guide_bld', cur_conf_jastr_guide_bld, conf_i)

!  call check_association ('configurations_guide(conf_i)%conf_jastr%BETA_EN', &
!                   configurations_guide(conf_i)%conf_jastr%BETA_EN,          &
!                   'cur_conf_jastr_guide_bld', cur_conf_jastr_guide_bld, conf_i)

!  call check_association ('configurations_guide(conf_i)%conf_jastr%A_EE', &
!                   configurations_guide(conf_i)%conf_jastr%A_EE,          &
!                   'cur_conf_jastr_guide_bld', cur_conf_jastr_guide_bld, conf_i)

!  call check_association ('configurations_guide(conf_i)%conf_jastr%A_EN',  &
!                   configurations_guide(conf_i)%conf_jastr%A_EN,           &
!                   'cur_conf_jastr_guide_bld', cur_conf_jastr_guide_bld, conf_i)

!  call check_association ('configurations_guide(conf_i)%conf_jastr%A_EEN', &
!                   configurations_guide(conf_i)%conf_jastr%A_EEN,          &
!                   'cur_conf_jastr_guide_bld', cur_conf_jastr_guide_bld, conf_i)

!  call exit (l_here,2)

!  end subroutine check_asso_cur_conf_jastr_guide


! ===============================================================================
!  subroutine configurations_guide_deep_copy (conf_left, conf_right)
! -------------------------------------------------------------------------------
!
! Name       :    configurations_guide_deep_copy
! Description    : prints out all configurations_guide
!
! Authors: X. Krokidis, R. Assaraf
! Date   : 25 Apr
! -------------------------------------------------------------------------------
!  implicit none

! i/o:
!  type(configuration), pointer :: conf_left(:),conf_right(:)
 
! local:
!  character(len=max_string_len_routine_name) :: l_here
!  integer(dpi)  :: c, j

! begin:
!  l_here = "configurations_guide_deep_copy"
!  call enter (l_here)

!  if (debug) &
!  write (logf,*)trim(l_here),'-d: total number of configurations_guide= ',size(configurations_guide)

!  do c = 1, size(configurations_guide)

!  if (debug) write (logf,*)trim(l_here),': configuration # ',c
!  conf_left(c)%name = conf_right(c)%name
!  conf_left(c)%weight = conf_right(c)%weight
!  conf_left(c)%jastr_type = conf_right(c)%jastr_type

!  call copy (conf_left(c)%conf_jastr%A_EE,conf_right(c)%conf_jastr%A_EE)
!  call copy (conf_left(c)%conf_jastr%BETA_EE,conf_right(c)%conf_jastr%BETA_EE) 
!  call copy (conf_left(c)%conf_jastr%A_EN,conf_right(c)%conf_jastr%A_EN)
!  call copy (conf_left(c)%conf_jastr%BETA_EN,conf_right(c)%conf_jastr%BETA_EN)
!  call copy (conf_left(c)%conf_jastr%A_EEN,conf_right(c)%conf_jastr%A_EEN)
!  conf_left(c)%n_determinant = conf_right(c)%n_determinant

!  allocate(conf_left(c)%conf_deter(size(conf_right(c)%conf_deter)))

!  do j = 1,size(conf_right(c)%conf_deter)

!   conf_left(c)%conf_deter(j)%weight = conf_right(c)%conf_deter(j)%weight

!    message='in subroutine '+l_here+'conf_left('+c+')%conf_deter('&
!           +j+')%alfa_orbitals should not be associated'
!    call require(message,&
!                .not.associated(conf_left(c)%conf_deter(j)%alfa_orbitals))

!   call copy (conf_left(c)%conf_deter(j)%alfa_orbitals,   &
!              conf_right(c)%conf_deter(j)%alfa_orbitals)
!   call copy (conf_left(c)%conf_deter(j)%beta_orbitals,   &
!              conf_right(c)%conf_deter(j)%beta_orbitals)
!  end do ! over j loop

!  enddo

!  call exit (l_here)

!  end subroutine configurations_guide_deep_copy


end module configurations_guide_mod
