! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module control_mod

  use kinds_parameters_mod
  use system_data_mod      

! --------------------------------------------------------------------------------
! Revision: 11 Feb 2001 F. Colonna  TAB added
! Revision: 13 Fev 2001: stemp, string => local
! Revision: 01 Apr 2002 strjunk added to be used for ensnb...inc
! Revision: 10 Apr 2002 FCC routine_names => routine_stack_names 
! Revision: 02 Aug 2002 Roland Assaraf 
! Revision: comment of lines related to system_data_mod
! Revision: walkers_data_mod 
! Revision: 23 Dec 2002 Rvalunk added to be used for empty...inc
! Revision: 05 Feb 2003 put some constants to kinds_parameters_mod

! Description : default file units
  integer(dpi), parameter     :: err                 =   2 ! stderr
  integer(dpi), parameter     :: logf                =   6 ! log file
  integer(dpi), parameter     :: in                  =   5 ! stdin
  integer(dpi), parameter     :: out                 =   6 ! stdout
  integer(dpi), parameter     :: rtn                 =   7 ! routines_names
  integer(dpi), parameter     :: dpn                 =   8 ! dependence_list
  integer(dpi), parameter     :: res                 =   9 ! results files

! string lengths
  integer(dpi), parameter :: max_string_len_debug_list   = 2048
! integer(dpi), parameter :: max_string_len              = 512 ! pbs with nulls
  integer(dpi), parameter :: max_string_len              = 256

! parser
  integer(dpi), parameter :: max_string_len_input         = 256
  integer(dpi), parameter :: max_string_len_command       =  32
  integer(dpi), parameter :: max_string_len_fortok_name   =  31 ! fortran variables 
  integer(dpi), parameter :: max_string_len_symbol_name   =  31 
  integer(dpi), parameter :: max_string_len_symbol_expres = 256 
  integer(dpi), parameter :: max_string_len_expr_type     =   8

! geometry, basis, wavefunction
  integer(dpi), parameter :: max_len_pseudo_name         =   8
  integer(dpi), parameter :: max_len_atom_name           =   8
  integer(dpi), parameter :: max_len_atom_symbol         =   2 ! mendeleiev table

! system
  integer(dpi), parameter :: max_len_user_name           =  32
  integer(dpi), parameter :: max_len_arch_name           =  32
  integer(dpi), parameter :: max_len_mach_name           =  32
  integer(dpi), parameter :: max_len_exec_name           =  255
  integer(dpi), parameter :: max_len_vers_name           =  8
  integer(dpi), parameter :: max_string_len_job_name     =  80
  integer(dpi), parameter :: max_string_len_run_type     =  16

! for max_len_epf_name (see epfs_mod)
! for max_len_ao_name (see aos_mod)

! file management
  integer(dpi), parameter :: max_files                   = 100

  integer(dpi), parameter :: max_string_len_file_name    =  256 ! 512
  integer(dpi), parameter :: max_string_len_file_type    =  32
  integer(dpi), parameter :: max_string_len_format       =  80
  integer(dpi), parameter :: max_string_len_action       =  16
  integer(dpi), parameter :: max_string_len_position     =  16
  integer(dpi), parameter :: max_string_len_program      =  16
  integer(dpi), parameter :: max_string_len_status       =  16

  integer(dpi), parameter :: max_string_len_title        =  255
  integer(dpi), parameter :: max_string_len_object_name  =  79 ! improve
  integer(dpi), parameter :: max_string_len_object_dim_names =  32
!JT  integer(dpi), parameter :: max_string_len_routine_name =  31 
  integer(dpi), parameter :: max_string_len_routine_name =  50 !JT

! echo tabulation
  integer(dpi)            :: echo_tab = 50

! routines messages
  integer(dpi)   :: debug_level      = 0
  integer(dpi)   :: die_level        = 0
  integer(dpi)   :: print_level      = 0
  integer(dpi)   :: trace_level      = 0
  integer(dpi)   :: warning_level    = 0
  logical(bool)  :: trace_depend     = .false.
  logical(bool)  :: fast             = .false.
  logical(bool)  :: debug            = .false.
  logical(bool)  :: cpu              = .false.
  logical(bool)  :: echo             = .true.   ! echoes input command lines
  logical(bool)  :: show             = .true.   ! show set variables

! routines management:
  integer(dpi)            :: routine_stack_level    = 0
  integer(dpi), parameter :: routine_max_number     = 300
  integer(dpi), parameter :: routine_arguments_max_number = 8

! strings defined in enter routine. Used for error messages.
  character(len=max_string_len) :: routines_1    ! the current routine name
  character(len=max_string_len) :: routines_2    ! the previous routine name
  character(len=max_string_len) :: routines_3    ! the penultimate routine name
  character(len=max_string_len) :: routines_21   ! previous-current
  character(len=max_string_len) :: routines_32   ! penultimate-previous
  character(len=max_string_len) :: routines_321  ! penultimate-previous-current

! Description : routine_stack_names (rtn)
! Description : the routine stack array. stacks called routines. used in error message

  character(len=max_string_len_routine_name), pointer :: routine_stack_names(:) => NULL ()

! Description : debug_routines_list 
! Description : list of routines to be debugged string. defined in setup_menu routine.

  character(len=max_string_len_debug_list) :: debug_routines_list    

! Description : some useful things. used locally everytime.
! Description : character(len=max_string_len) :: blanks! juste blanks "       .. "
! Description : character(len=max_string_len) :: dashes    ! just dashes -------------
! Description : character(len=max_string_len) :: points    ! just points .............
! Description : character(len=max_string_len) :: message   ! to write a message locally
! Description : character(len=max_string_len) :: strjunk   ! to store a string locally
! Description : real(dp)                      :: valjunk   ! to store a value locally
! Description : integer(dpi)                  :: all_err   ! error code for allocate
! Description : integer(dpi)                  :: iostat    ! error code file management

  character(len=max_string_len) :: blanks
  character(len=max_string_len) :: dashes    
  character(len=max_string_len) :: message
  character(len=max_string_len) :: description = NOT_INIT_S  
  character(len=max_string_len) :: points    
  character(len=max_string_len) :: strjunk   
  integer(dpi)                  :: all_err   
  integer(dpi)                  :: iostat   
  real(dp)                      :: valjunk   

! Description : cpu
! Description : real(sp)                        :: actual_cpu,  start_cpu
! Description : character(len=max_string_len)   :: cpu_routines_list 
! Description : real(sp), pointer :: cpu_start_routines(:) 

  real(sp)                        :: actual_cpu,  start_cpu
  character(len=max_string_len)   :: cpu_routines_list      = ' '
  real(sp), pointer :: cpu_start_routines(:) => NULL ()

! Description : Date and time

  character(len= 16) :: cur_date
  character(len= 16) :: cur_time
  character(len= 16) :: cur_cpu
  character(len= 3)  :: cur_month
  character(len= 3)  :: cur_day_name

  character(len= 3)  :: monthes (12) = &
  (/ 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', &
  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'/)

! Description : memory in octets
  integer (big) :: memory_min = 0         
  integer (big) :: memory_max = 0

! Description : system

  character(len=max_string_len_program)   :: interfaced_program = ' '
  character(len=max_len_user_name)        :: user_name
  character(len=max_len_arch_name)        :: arch_name
  character(len=max_len_mach_name)        :: mach_name
  character(len=max_len_exec_name)        :: exec_name ! executable
  character(len=max_len_vers_name)        :: version_name ! version    
  character(len=max_string_len_file_name) :: dir_cur_name
  character(len=max_string_len_file_name) :: dir_exe_name
  character(len=max_string_len_title)     :: title = ' '
  character(len=max_string_len_job_name)  :: job_name

  contains

! ================================================================================
  subroutine init      
! --------------------------------------------------------------------------------
!
! Name           : init
!
! Description    : initialize program 
!
! Authors: F. Colonna
! Date  :26 Sep 2000
! --------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug =.false.

! begin:
  l_here ='init'

! initialize "blanks" 
  blanks = repeat(' ',len(blanks)) 
  message    = repeat(' ',len(message))

! initialize "dashes" 
  dashes = repeat('-',len(dashes)) 

! initialize "points" for cpu and trace
  points = repeat('.',len(points)) 

! initialize "title" 
! title = repeat('.',len(title))

! debug_routines_list. 
  debug_routines_list = repeat(' ',len(debug_routines_list))

  interfaced_program = 'QMCMOL'//blanks

  job_name       = blanks
  version_name   = '1.00 01-Feb-2002'
  version_name   = '1.01 08-Oct-2002'
! new input form as defined by Michel, Peter, Roland and Francois
  version_name   = '1.02 14-Nov-2002'
! object management and expressions work
  version_name   = '1.10' !  18-Dec-2002'

  memory_min = 0
  memory_max = 0

  cpu = .false.
  print_level = -1 ! then is set to 0 when the log file is opened (see files_init)
  trace_level = 0

  call routine_stack_names_init

  allocate (cpu_start_routines (routine_max_number), stat = all_err)
 
  start_cpu = zero
  call cpu_now 
  start_cpu = actual_cpu

  call date_time

  call system_data_init


  if (l_debug) then 
   write (logf,*)trim(l_here),'-d: start_cpu =',start_cpu    
   write (logf,*)trim(l_here),'-d: Date =',trim(cur_date)
   write (logf,*)trim(l_here),'-d: time =',trim(cur_time)
   write (logf,*)trim(l_here),'-d: version =',trim(version_name)
   write (logf,*)trim(l_here),'-d: job_name =',trim(job_name)
   write (logf,*)trim(l_here),'-d: interfaced_program =',trim(interfaced_program)
  end if

! catalog routines' addresses
! call routines_catalog_init

  end subroutine init      

! ================================================================================
  subroutine finish      
! --------------------------------------------------------------------------------
!
! Name           : finish
!
! Description    : ends program 
!
! Authors: F. Colonna
! Date  :26 Sep 2000
! --------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string 
  logical(bool)                              :: l_debug =.false.
  real(sp)                                   :: dif_cpu

! begin:
  l_here ='finish'

  if(l_debug) write (logf,*)trim(l_here),'program ended normally' 

  write (logf,'(/a)')'program QMCMOL ended normally '

! Date and time
  call date_time
  write (logf,'(4a)')'at ',trim(cur_time),' on ',trim(cur_date)   

! cpu
  call cpu_now  ! gives actual_cpu
  dif_cpu = actual_cpu - start_cpu
  string  = cpu_to_string (dif_cpu)
  write (logf,'(2a)') 'total cpu ',trim(string)

! memory allocated
  string = memory_to_string (memory_min)
  write (logf,'(2a)') 'min memory used ',trim(string)

  string = memory_to_string (memory_max)
  write (logf,'(2a)') 'max memory used ',trim(string)

  write(res,'(a80,/)') dashes

  end subroutine finish      

! ================================================================================
  subroutine routine_stack_names_init
! --------------------------------------------------------------------------------
!
! Name           : routine_stack_names_init
!
! Description    : initialize  array routine_stack_names
!
! Authors: F. Colonna
! Date  :11 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.
  integer (big)                               :: memory_siz

! begin:
  l_here ='routine_stack_names_init'

  if(associated(routine_stack_names)) then
   message ='object >routine_stack_names< already associated. will be deallocated'
   call warning (l_here, message)
   memory_siz = kind(routine_stack_names)*size(routine_stack_names)
   memory_min = memory_min - memory_siz
   deallocate (routine_stack_names)
  end if

  allocate (routine_stack_names(routine_max_number), stat = all_err)

  if (all_err /= 0) then
   write (logf,'(2a)') trim(l_here), &
   '-e: allocation failed for object >routine_stack_names<'
   write (logf,'(2a,i10)') trim(l_here), &
   '-e: dimensions are ',routine_max_number
   call die (l_here, 'allocation failed')
  end if

  memory_siz = len(routine_stack_names)*size(routine_stack_names)
  memory_min = memory_min + memory_siz
  memory_max = memory_max + memory_siz

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: allocation done for object >routine_stack_names<'
  end if

  end subroutine routine_stack_names_init

! ================================================================================
  subroutine die (routine, text)
! --------------------------------------------------------------------------------
!
! Name           : die 
!
! Argument (in ) : routine
! Argument (in ) : text
!
! Description    : output message and list the stack of called routines
! Description    : output text with '_error: '
!
! Authors: F. Colonna
! Date  :11 Oct 2000
! --------------------------------------------------------------------------------

! i/o:
  character(len=*),intent(in)                :: routine
  character(len=*),intent(in)                :: text

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug
  integer :: r

! begin:
  l_here ='die'

  l_debug = .false.
  if(l_debug) then
   write (logf,*) trim(l_here),'-d: routine >',trim(routine),'<'
   write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
  end if

  write(err,'(1x,3a)') trim(routine),'-die: '
  write(err,'(1x,3a)') trim(routine),'-die: ',trim(text) 
  write(err,'(1x,2a)') trim(routine),'-die: called routines stack:'

  write (logf,'(1x,3a)') trim(routine),'-die: '
  write (logf,'(1x,3a)') trim(routine),'-die: ',trim(text) 
  write (logf,'(1x,2a)') trim(routine),'-die: called routines stack:'

  do r = 1, routine_stack_level   
   write(err,'(3a)')  ' >',trim(routine_stack_names(r)),'<'
   write (logf,'(3a)') ' >',trim(routine_stack_names(r)),'<'
  end do

  write (logf,'(a)')' '

  stop 'by die'

  end subroutine die

! ================================================================================
  subroutine require (message, condition)
! --------------------------------------------------------------------------------
!
! Name           : require
!
! Argument (in ) : message 
! Argument (in ) : condition 
!
! Description    : stops by calling die if condition is false
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: message
  logical(bool),    intent(in)  :: condition

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='require'

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: message >',trim(message),'<'
   write (logf,*)trim(l_here),'-d: condition =',condition
  end if

  if(.not.condition)  then
   write (logf,*)trim(l_here), ': the program was stopped because ',trim(message)
   call die (l_here,'condition raised')
  end if

  end subroutine require

! ================================================================================
  subroutine check_lbound (where, &
                         name_reference, reference, &
                         name_array, lbound_array, dim) 
! --------------------------------------------------------------------------------
!
! Name           : subroutine check_lbound (where, &
!
! Argument (in)  : where           routine name where check is done
! Argument (in)  : name_array
! Argument (in)  : lbound_array   
! Argument (in)  : dim
! Argument (in)  : name_reference
! Argument (in)  : reference
!
! Description    : checks whether lbound (array, dim) == reference
!
! Authors: F. Colonna
! Date  :03 Feb 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in) :: where, name_array, name_reference
  integer(dpi), intent(in)     :: dim, reference
  integer(dpi), intent(in)     :: lbound_array

! local:

! begin:
  if(lbound_array /= reference) then
    write(message,'(3a,i1,a,i10,3a,i10)') &
    ' lbound (',trim(name_array),', ',dim,') = ', lbound_array,  &
    ' /= ',trim(name_reference),' = ', reference
    call die (where, message)
  end if

  end subroutine check_lbound

! ================================================================================
  subroutine check_ubound (where, &
                         name_reference, reference, &
                         name_array, ubound_array, dim) 
! --------------------------------------------------------------------------------
!
! Name           : subroutine check_ubound (where, &
!
! Argument (in)  : where           routine name where check is done
! Argument (in)  : name_array
! Argument (in)  : ubound_array   
! Argument (in)  : dim
! Argument (in)  : name_reference
! Argument (in)  : reference
!
! Description    : checks whether ubound (array, dim) == reference
!
! Authors: F. Colonna
! Date  :03 Feb 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in) :: where, name_array, name_reference
  integer(dpi), intent(in)     :: dim, reference
  integer(dpi), intent(in)     :: ubound_array

! local:

! begin:
  if(ubound_array /= reference) then
    write(message,'(3a,i1,a,i10,3a,i10)') &
    ' ubound (',trim(name_array),', ',dim,') = ', ubound_array,  &
    ' /= ',trim(name_reference),' = ', reference
    call die (where, message)
  end if

  end subroutine check_ubound

! ================================================================================
  subroutine check_size (where, &
                         name_reference, reference, &
                         name_array, size_array, dim) 
! --------------------------------------------------------------------------------
!
! Name           : subroutine check_size (where, &
!
! Argument (in)  : where           routine name where check is done
! Argument (in)  : name_array
! Argument (in)  : size_array   
! Argument (in)  : dim
! Argument (in)  : name_reference
! Argument (in)  : reference
!
! Description    : checks whether size (array, dim) == reference
!
! Authors: F. Colonna
! Date  :03 Feb 2002
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in) :: where, name_array, name_reference
  integer(dpi), intent(in)     :: dim, reference
  integer(dpi), intent(in)     :: size_array

! local:

! begin:
  if(size_array /= reference) then
    write(message,'(3a,i1,a,i10,3a,i10)') &
    ' size (',trim(name_array),', ',dim,') = ', size_array,  &
    ' /= ',trim(name_reference),' = ', reference
    call die (where, message)
  end if

  if(reference <= 0) then
     write (logf,*)trim(where),'-w: ',trim(name_reference),' = ',reference
     call die (trim(where)//'-check_size', 'bad '//trim(name_reference) )
  end if

  end subroutine check_size

! ================================================================================
  subroutine check_same_size (where, &
  name_array_1, size_array_1, dim_1, name_array_2, size_array_2, dim_2)
! --------------------------------------------------------------------------------
!
! Name       : subroutine check_same_size (where, &
!
! Arguments  : (where, name_array_1, size_array_1, dim_1
! Arguments  :  name_array_2, size_array_2, dim_2)
!
! Argument (in)  : where           routine name where check is done
! Argument (in)  : name_array_1
! Argument (in)  : size_array_1   
! Argument (in)  : dim_1
! Argument (in)  : name_array_2
! Argument (in)  : size_array_2
! Argument (in)  : dim_2
!
! Description    : checks whether size (array_1, dim_1) == size (array_2, dim_2)
!
! Authors: F. Colonna
! Date  :13 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in) :: where, name_array_1, name_array_2
  integer(dpi), intent(in)     :: dim_1, dim_2
  integer(dpi), intent(in)     :: size_array_1, size_array_2

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='check_same_size'

  write(message,'(2a,3a,i1,a,i10,3a,i1,a,i10)') 'in ',trim(where), &
  ' size(', trim(name_array_1), ', ', dim_1, ') = ', size_array_1,  &
  ' size(', trim(name_array_2), ', ', dim_2, ') = ', size_array_2

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: where >',trim(where),'<'
   write (logf,*)trim(l_here),'-d: message >',trim(message),'<'
  end if

  call require (message, size_array_1 == size_array_2)

  end subroutine check_same_size

! ================================================================================
  subroutine ensure (message, condition)
! --------------------------------------------------------------------------------
!
! Name           : ensure
!
! Argument (in ) : message 
! Argument (in ) : condition 
!
! Description    : stops by calling die if condition is false
!
! Authors: F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: message
  logical(bool),    intent(in)  :: condition

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug = .false.

! begin:
  l_here ='ensure'

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: message >',trim(message),'<'
   write (logf,*)trim(l_here),'-d: condition =',condition
  end if

  if(.not.condition)  then
   write (logf,*)trim(l_here), ': the program was stopped because ',trim(message)
   call die (l_here,'condition raised')
  end if

  end subroutine ensure

! ================================================================================
  subroutine message_out (routine, text)
! --------------------------------------------------------------------------------
!
! Name          : message_out
! Argument (in) : routine 
! Argument (in) : text 
!
! Description    : outputs trim(routine),': ',trim(text)
!
! Authors:F. Colonna
! Date  :11 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*),intent(in) :: routine, text

! local:
! character(len=max_string_len_routine_name) :: l_here

! begin:
! l_here='message_out'

  write (logf,'(1x,3a)') trim(routine),': ',trim(text)

  end subroutine message_out

! ================================================================================
  subroutine warning (routine, text)
! --------------------------------------------------------------------------------
!
! Name          : warning 
! Argument (in) : routine 
! Argument (in) : text 
!
! Description    : outputs trim(routine),'-w: ',trim(text)
!
! Authors: F. Colonna
! Date  :11 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! local:
! character(len=max_string_len_routine_name) :: l_here
  character(len=*),intent(in) :: routine, text

! begin:
! l_here='warning'

  write (logf,'(1x,3a)') trim(routine),'-w: ',trim(text)

  end subroutine warning

! ================================================================================
  function cpu_to_string (secs) 
! --------------------------------------------------------------------------------
!
! Name          : cpu_to_string
!
! Argument (in) : secs
! 
! Description    : converts "secs" into a string "..h..m..s..c"
!
! Authors:F.Colonna
! Date  :06 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! function:
  character(len= 20) :: cpu_to_string

! i/o:
  real(sp)          :: secs

! local:  
! character(len=max_string_len_routine_name) :: l_here
  integer hcpu, mcpu, scpu, ccpu

! local:
  character(len=max_string_len) :: string 
  real(sp)                      :: rsecs, hours, minutes, cents

! begin:
! l_here='cpu_to_string' 

   rsecs    = real(secs)
   hcpu     = ifix(rsecs/3600.)
   hours    = float(hcpu)
   mcpu     = ifix((rsecs-3600.*hours)/60.)
   minutes  = float(mcpu)
   scpu     = ifix(rsecs-3600.*hours-60.*minutes)
   cents    = 100.d0*secs -100.d0*ifix(rsecs)
   ccpu     = nint(cents)

   write(string,'(i2.2,a,i2.2,a,i2.2,a,i2.2,a)') &
                hcpu,'h',mcpu,'m',scpu,'s',ccpu,'c'

   cpu_to_string = string 

  end function cpu_to_string

! ================================================================================
  function memory_to_string (size) result(result) 
! --------------------------------------------------------------------------------
!
! Name           : memory_to_string
!
! Argument (in ) : size increment of memory 
!
! Description    : converts "size" octets into a string "mm kk oo Octects"
! Description    : may be zero when object already allocated
!
! Authors:F.Colonna
! Date  :06 Oct 2000
! --------------------------------------------------------------------------------
  implicit none

! result:
  character(len= 31) :: result

! i/o:
  integer (big)      :: size

! local:  
! character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len) :: string 
  integer (big)      :: gigas, megas, kilos, octets

! begin:
! l_here='memory_to_string' 

  if(size .lt. 0) then
    write (logf,*)'memory_to_string-e: non positive array size =',size
    call die ('memory_to_string', 'non positive array size')
  end if

  gigas = size/1000000000  
  megas = (size - (1000000000)*gigas)/1000000 
  kilos = (size - (1000000000)*gigas  - (1000000)*megas)/1000
  octets = size - (1000000000)*gigas  - (1000000)*megas -(1000)*kilos

!  write(string,'(i3.3,a,i3.3,a,i3.3,a,i3.3,a)')  &
!        gigas,' ',megas,' ', kilos,' ',octets,'-O'

  write(string,'(i3.3,a,i3.3,a,i3.3,a,i3.3,a)')  &
        gigas,'-Go ',megas,'-Mo ', kilos,'-Ko ',octets,'-Octets'

  result = string 

  end function memory_to_string

! ================================================================================
  subroutine cpu_now 
! --------------------------------------------------------------------------------
!
! Name       : cpu_now 
!
! Description    : calls cpu_time
! Description    : fills cpu_real
! Description    : calculates elapsed cpu cpu_diff = cpu_real-start_cpu
!
! Authors: F.Colonna
! Date  : 21 Sep 2000
! --------------------------------------------------------------------------------
  implicit none
  real(sp) :: cpu_real
  real(dp) :: cpu_diff 

  call cpu_time (cpu_real)

  actual_cpu = cpu_real
  cpu_diff   = dble (cpu_real-start_cpu)
  write(cur_cpu,'(f16.2)') cpu_diff

  end subroutine cpu_now   

! ================================================================================
  subroutine date_time 
! --------------------------------------------------------------------------------
!
! Name       : date_time
! Description    : returns "cur_date" and "cur_time" strings 
!
! Authors: F.Colonna
! Date  : 21 Sep 2000
! --------------------------------------------------------------------------------
  implicit none

! i/o
  integer(dpi) :: values(8), year, month, day, hour, minute, second
  character(len= 10) :: time_string, zone
  character(len=8)  :: date_string

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)       :: l_debug = .false.
 
! begin:
  l_here ='date_time'
  call date_and_time (date_string, time_string, zone, values)

  year   = values(1)
  month  = values(2)
  day    = values(3)
  hour   = values(5)
  minute = values(6)
  second = values(7)

  if(l_debug) then
   write (logf,'(2a)') trim(l_here),': date_string   =',date_string
   write (logf,'(2a)') trim(l_here),': time_string   =',time_string
   write (logf,'(3a)') trim(l_here),': zone >',trim(zone),'<'
   write (logf,'(a,i10)') trim(l_here),': year   =',year
   write (logf,'(a,i10)') trim(l_here),': month  =',month
   write (logf,'(a,i10)') trim(l_here),': day    =',day
   write (logf,'(a,i10)') trim(l_here),': hour   =',hour
   write (logf,'(a,i10)') trim(l_here),': minute =',minute
   write (logf,'(a,i10)') trim(l_here),': second =',second
  end if

  cur_month        = monthes(month)

  if(l_debug) then
   write (logf,'(3a)') trim(l_here),': cur_month  =',cur_month
  end if

  write(cur_date,'(i2.2,1x,a,1x,i4.4)') day,cur_month,year  
  write(cur_time,'(i2.2,a,i2.2,a,i2.2,a)') hour,'h:',minute,'m:',second,'s'

  end subroutine date_time

end module control_mod

