!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module density_grid_menu_mod

! ------------------------------------------------------------------------
! 
! Author: A. Scemama
! Date  : 22 Dec 2005
! Revision: 
! ------------------------------------------------------------------------
 
  use system_data_mod
  use parser_mod
  use expressions_mod
  use density_grid_mod

  implicit none

 logical(bool)         :: grid_write
 logical(bool)         :: grid_read

  contains
 
! ========================================================================
  subroutine density_grid_menu
! ------------------------------------------------------------------------
! Name           : density_grid_menu
 
! Description    : input density_grid parameters
 
! Author: A. Scemama
! Date  : 22 Dec 2005
! ------------------------------------------------------------------------
  implicit none
    
! local:
  logical(bool)       :: done
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='density_grid_menu'
  call enter(l_here, 1)
  objects_desc ( object_index('density_grid') ) = &
 'Values of the density at the points defined by grid_coor'
! initialize:
  done = .false.
  call next_command_get ('density_grid')
  do while (.not.done)
     call next_word_get ('density_grid')
     if(debug) write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
     if(trim(cur_word) .eq. 'help') then
      write (logf,*) 'density_grid-h: menu'
      write (logf,*) 'density_grid-h: syntax'
      write (logf,*) 'density_grid-h: den{sity_grid}'
      write (logf,*) 'density_grid-h:   help'
      write (logf,*) 'density_grid-h:   estimator    = [string]'
      write (logf,*) 'density_grid-h:   ionization   = [double]'
      write (logf,*) 'density_grid-h:   end_den'
      write (logf,*) 'density_grid-h: example'
      write (logf,*) 'density_grid-h: density_grid'
      write (logf,*) 'density_grid-h:   estimator = Long_range'
      write (logf,*) 'density_grid-h:   ionization= -0.95'
      write (logf,*) 'density_grid-h: end_den'

  else if(token_is ('est', estimator)) then

  else if(token_is ('ionization', ionization)) then
      if ( ionization.gt.0.d0) then
        call die(l_here,'The input ionization potential is >0, check your input.')
      endif
 
      if (estimator.eq.'Poisson') then
        write (logf,*) 'density_grid-h:Poisson estimator is selected'
        write (logf,*) 'density_grid-h:the ionization potential will not be used.'

      else if (estimator.eq.'Long_range') then
        write (logf,*) 'density_grid-h:Long range estimator is selected'

      else if (estimator.eq.'Long_short_range') then
        write (logf,*) 'density_grid-h:Long and short range estimator is selected'

      endif

  else
    call menu_exit (l_here, done)
  end if
     
  end do
    
  call exit (l_here, 1)
    
  end subroutine density_grid_menu
 
end module density_grid_menu_mod

