!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module density_guide_bywlk_mod

  use basic_types_tools_mod
  use psi_guide_bywlk_mod

  implicit none

! Creates            : density_guide_bywlk (walk)
  real(dp), pointer :: density_guide_bywlk (:) => NULL()

! Needs              : psi_guide_bywlk (walk)

  contains

! ========================================================================
  subroutine density_guide_bywlk_bld 
! ------------------------------------------------------------------------
! 
! Name           : density_guide_bywlk_bld
! 
! Description    : density_guide_bywlk (walk)     
!
! Creates        : density_guide_bywlk (walk)
!
! Needs          : psi_guide_bywlk (walk)
!
! Author: J. Toulouse
! Date  : 25 Jul 2005
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: walk_i

! header:

  if (header_exe) then

    l_here = "density_guide_bywlk_bld"

!    call object_create ('density_guide_bywlk')

     call object_needed ('psi_guide_bywlk')
     call object_needed ('elec_coor_bywlk')
    return

  end if

! begin:
  call enter(l_here)

! requirements:

!   include 'chksiz_psi_guide_bywlk.inc'
    include 'chksiz_elec_coor_bywlk.inc'
! end requirements
! dimensions:

!  walkers_nb = size (psi_guide_bywlk, 1)
   walkers_nb = size (elec_coor_bywlk,3)

  if(debug) then
     write (logf,*)trim(l_here),'-d: walkers_nb  = ',walkers_nb
  end if 

! end dimensions

! allocations:

  if(.not.associated(density_guide_bywlk)) then
    write (logf,*)trim(l_here),': density_guide_bywlk not associated'
    call alloc ('density_guide_bywlk', density_guide_bywlk, walkers_nb)
  else
    if (trace_depend) write (logf,*)trim(l_here),': density_guide_bywlk left in memory'
  end if

  if(debug) then 
    do walk_i = 1, walkers_nb
       write (logf,*)trim(l_here),'-d: psi (',walk_i,') =', psi_guide_bywlk (walk_i)
    end do
  end if

  do walk_i = 1, walkers_nb
    density_guide_bywlk (walk_i) = psi_guide_bywlk (walk_i) * psi_guide_bywlk (walk_i)
  end do

  if(debug) then 
    do walk_i = 1, walkers_nb
      write (logf,*)trim(l_here),'-d: '
      write (logf,*)trim(l_here),'-d: density_guide_bywlk (walk)'
      write (logf,*)trim(l_here),'-d: density_guide_bywlk (',walk_i,') =', density_guide_bywlk (walk_i)
    end do
  end if

  call exit(l_here)

  end subroutine density_guide_bywlk_bld 

end module density_guide_bywlk_mod
