!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

! =======================================================================================
  subroutine det_mat (a, n, det)
! ---------------------------------------------------------------------------------------
! Description    :
! Description    : determinant of a matrix a:
! Description    : construct triangular matrix, multiply the diagonal elements
! Description    :
! Description    : the matrix is destroyed!
! Description    :
! Description    : diagonal elements may not be zero!
! Description    : 
! Description    : the algorithm is standard, the real source of the routine is unknown 
!
! Author: P.Reinhardt (translator...)
! Date  : 5 Dec 2000
! ---------------------------------------------------------------------------------------
  use strings_mod
  implicit none
  integer(dpi) :: n
  real(dp)     :: a(n,n)
  real(dp)     :: det
!
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: nm1,kp1,k,j,jj,i
  real(dp)     :: work

! begin:
  l_here ='det_mat'
  call enter (l_here,5) 
!
! determinant of a matrix:
! construct triangular matrix, multiply the diagonal elements
!
! the matrix is destroyed!
!
! diagonal elements may not be zero!
! 
  nm1 = n - 1
  do k = 1, nm1
   kp1 = k + 1
!
!           compute multipliers
!
   work = -1.D0/a(k,k)
   do j = kp1,n
    a(j,k) = work*a(j,k)
   end do
!
!           row elimination with column indexing
!
   do j = kp1, n
    do jj = kp1,n
     a(jj,j) = a(jj,j)+a(k,j)*a(jj,k)
    end do
   end do
! close loop over k
  end do
!
  det = a(1,1)
  do i = 2,n
   det = det*a(i,i)
  end do
!
  call exit (l_here,5) 

  end subroutine det_mat
