! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module determinants_mod

  use basic_types_tools_mod
  use parser_mod  
  use arrays_mod  
  use system_data_mod  

  implicit none
 
  type determinant 
  integer(dpi), pointer :: alfa_orbitals(:) => NULL()
  integer(dpi), pointer :: beta_orbitals(:) => NULL()
  real(dp)              :: weight
  end type determinant 

  real (dp), pointer :: determinants_weights (:) => NULL ()
 
  integer(dpi) :: n_determinants = -1
  integer(dpi) :: det_alfa_nb     = -1
  integer(dpi) :: det_beta_nb     = -1
  integer(dpi) :: cur_det        = -1
  integer(dpi) :: weight_cur_det = -1

  type (determinant), pointer :: determinants(:) => NULL ()

  contains

! ==========================================================================
  subroutine determinant_menu
! --------------------------------------------------------------------------
!
! Name           : determinant_menu
!
! Description    : collects determinant information from menu
!
! Authors:X. Krokidis, F. Colonna
! Date  :26 Sep 2000
! Revision: 08 Feb 2002 by F. Colonna  (system%max_determinants_nb)
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool) :: done

! begin:
  l_here ='determinant_menu'
  call enter (l_here)

! initialize:
!  call check_asso_det &
!  ('determinants', determinants, 'determinants_init', determinants_init)
   if (.not.associated (determinants)) call determinants_init
! count:
  cur_det = cur_det + 1
  system%max_determinants_nb = max (system%max_determinants_nb, cur_det)

  if(debug) write (logf,*)trim(l_here),'-d: cur_det =',cur_det

  done = .false.
  call next_command_get ('determinant')
  do while (.not.done)
   call next_word_get ('determinant')
   if(cur_word(1:4) .eq. 'help') then

    write (logf,*) 'determinant: determinant definitions'
    write (logf,*) 'determinant: syntax:'
    write (logf,*) 'determinant: det{erminant}'
    write (logf,*) 'determinant:   help'
    write (logf,*) 'determinant:   alf{a_orbitals} = [list_of_integers]'
    write (logf,*) 'determinant:   bet{a_orbitals} = [list_of_integers]'
    write (logf,*) 'determinant:   wei{ght}        = [double]'
    write (logf,*) 'determinant:   end_det'
    write (logf,*) 'determinant: example'
    write (logf,*) 'determinant: det{erminant'
    write (logf,*) 'determinant:   alf{a_orbitals}  = (1, 2,3, 4 ) '
    write (logf,*) 'determinant:   bet{a_orbitals}  = ( 1,5 ,7, 8) '
    write (logf,*) 'determinant:   wei{ght}  = 0.03'
    write (logf,*) 'determinant:   alf{a_orbitals}  = (10, 2,6, 9 ) '
    write (logf,*) 'determinant:   bet{a_orbitals}  = ( 5, 19, 6, 11)'
    write (logf,*) 'determinant:   wei{ght}  = 0.3 '
    write (logf,*) 'determinant: end'

   else if(cur_word(1:3) .eq. 'alf') then
     
     det_alfa_nb = det_alfa_nb +1
     call next_list_integer_get ()
     call alfa_orbitals_init (n_list_elements)
     determinants(det_alfa_nb)%alfa_orbitals = list_of_integer
   
     write (logf,*)trim(l_here),': alfa determinant # ',det_alfa_nb, &
     ' with ',n_list_elements,' alfa_orbitals :',list_of_integer
  
   else if(cur_word(1:3) .eq. 'bet') then
     det_beta_nb = det_beta_nb +1
     call next_list_integer_get ()
     call beta_orbitals_init (n_list_elements)
     determinants(det_beta_nb)%beta_orbitals = list_of_integer
     write (logf,*)trim(l_here),': beta determinant # ',det_beta_nb, &
     ' with ',n_list_elements,' beta_orbitals :',list_of_integer

   else if(cur_word(1:3) .eq. 'wei') then
     weight_cur_det = weight_cur_det +1
     call next_value_get ('weight', determinants(weight_cur_det)%weight)
     determinants_weights (weight_cur_det) = determinants(weight_cur_det)%weight
   else
     call menu_exit (l_here, done)
   end if

  end do

   write (logf,*)trim(l_here),': maximum number of determinants =', system%max_determinants_nb
  if(debug) then
   write (logf,*)trim(l_here),'-d: determinants_weight', determinants(cur_det)%weight
   write (logf,*)trim(l_here),'-d: cur_det=',cur_det 
   call printarray (l_here, determinants(cur_det)%alfa_orbitals, 'alfa_orbitals')
   call printarray (l_here, determinants(cur_det)%beta_orbitals, 'beta_orbitals')
  end if

  call exit (l_here)

  end subroutine determinant_menu

! ==========================================================================
  subroutine determinants_init
! --------------------------------------------------------------------------
!
! Name           : determinants_init
!
! Description    : initialize counters
!
! Creates        : determinants (det)
!
! Authors:X. Krokidis, F. Colonna
! Date  :26 Sep 2000
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here

! header:

  if (header_exe) then

    l_here = "determinants_init"

    call object_create ('determinants')

    return

  end if

! begin:
  call enter (l_here)

! count how many "weight" words are in input_words
  n_determinants = string_head_inside_cmd_count ('deter','config','command')

  if(debug) write (logf,*)trim(l_here),'-d: n_determinants=',n_determinants

  call require (' in '//trim(l_here)//' n_determinants = 0', &
                n_determinants .ne. 0)

  allocate (determinants(n_determinants), stat = all_err)
  write (logf,*)trim(l_here),': allocate (determinants(',n_determinants,'))'
  if(all_err /= 0)  &
  call die (l_here, 'allocate (determinants(n_determinants))')
 
  allocate (determinants_weights(n_determinants), stat = all_err)
  write (logf,*)trim(l_here),': allocate (determinants_weights(',n_determinants,'))'
  if(all_err /= 0)  &
       call die (l_here, 'allocate (determinants_weights(n_determinants))')

! initialize counters:
  det_alfa_nb = 0
  det_beta_nb = 0
  weight_cur_det = 0
  cur_det = 0

  call exit (l_here)

  end subroutine determinants_init

! ==========================================================================
  subroutine alfa_orbitals_init (orb_nb)
! --------------------------------------------------------------------------
!
! Name           : alfa_orbitals_init
!
! Argument (in ) : orb_nb
!
! Description    : initialize orb_nb alfa_orbitals
!
! Creates        : determinants(det)%alfa_orbitals (elec_a)
!
! Authors:X. Krokidis, F. Colonna
! Date: 14 Feb 2001
! --------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: orb_nb

! local:
  character(len=max_string_len_routine_name),save :: l_here
  
! header:

  if (header_exe) then

    l_here = "alfa_orbitals_init"

    call object_create ('determinants(det)%alfa_orbitals')

    return

  end if

! begin:
  l_here = "alfa_orbitals_init"
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: orb_nb=',orb_nb     
  if(debug)write (logf,*)trim(l_here),'-d: det_alfa_nb=',det_alfa_nb
  if(.not.associated(determinants)) call die (l_here, "determinants not associated")
!  call require ('in routine'//trim(l_here)//' orb_nb = 0', orb_nb .ne. 0)

  allocate (determinants(det_alfa_nb)%alfa_orbitals(orb_nb)) !, stat = all_err)
  
  if(all_err /= 0) call die &
  (l_here,'allocate (determinants(det_alfa_nb)%alfa_orbitals(orb_nb))')

  if (print_level > 0) then
    message = l_here+': allocate (determinants(det_alfa_nb)%alfa_orbitals('+orb_nb+'))'
    write (logf,'(a)') trim(message)
  end if

  call exit (l_here)

  end subroutine alfa_orbitals_init

! ==========================================================================
  subroutine beta_orbitals_init (orb_nb)
! --------------------------------------------------------------------------
!
! Name           : beta_orbitals_init
!
! Argument (in ) : orb_nb
!
! Description    : initialize orb_nb beta_orbitals
! Description    : orbital = determinants(det_beta_nb)%beta_orbitals(elec)
! Description    : elec ={1..electrons_beta_nb}
!
! Creates        : determinants(det)%beta_orbitals (elec_b)
!
! Authors:X. Krokidis, F. Colonna
! Date: 14 Feb 2001
! --------------------------------------------------------------------------
! --------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: orb_nb

! local:
  character(len=max_string_len_routine_name),save :: l_here
  
! header:

  if (header_exe) then

    l_here = "beta_orbitals_init"

    call object_create ('determinants(det)%beta_orbitals')

    return

  end if

! begin:
  l_here = "alfa_orbitals_init"
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'-d: orb_nb=',orb_nb     
  if(debug)write (logf,*)trim(l_here),'-d: det_alfa_nb=',det_alfa_nb
  if(.not.associated(determinants)) call die (l_here, "determinants not associated")
  
  call require ('in '//trim(l_here)//' orb_nb = 0', orb_nb .ne. 0)

  allocate &
  (determinants(det_beta_nb)%beta_orbitals(orb_nb), stat = all_err)
  if(all_err /= 0) call die &
  (l_here,'allocate (determinants(det_beta_nb)%beta_orbitals(orb_nb))')

  if (trace_level > 0) then
   message = l_here+': allocate (determinants(det_beta_nb)%beta_orbitals('+orb_nb+'))'
   write (logf,'(a)') trim(message)
  end if

  call exit (l_here)

  end subroutine beta_orbitals_init

! ==========================================================================
  subroutine check_asso_det (object_name, object, routine_name, routine)
! --------------------------------------------------------------------------
!
! Name           : check_asso_char_1
!
! Argument (in ) : object_name
! Argument (in ) : object
! Argument (in ) : routine_name
! Argument (in ) : routine
!
! Description    : builds an object if not associated
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! --------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent(in) :: object_name, routine_name
  type (determinant),  pointer :: object(:)
  external :: routine
! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='check_asso_det'

  if(debug) &
  write (logf,*)trim(routines_21),'-d: object_name >',trim(object_name),'<'

  if(.not.associated(object)) then
   if(debug .or. trace_level > 0) then
    write (logf,'(4a)') trim(l_here), &
    ': object >',trim(object_name),'< not created'
    write (logf,'(4a)') trim(l_here), &
    ': calling routine >',trim(routine_name),'<'
   end if
   call routine 
  end if

  end subroutine check_asso_det

end module determinants_mod
