! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module e_loc_tot_bywlk_mod

  use basic_types_tools_mod
  use drift_bywlk_mod
  use e_loc_kin_bywlk_mod
  use e_loc_vne_bywlk_mod
  use e_loc_vee_bywlk_mod
  use e_loc_vnn_mod
  use ecp_bywlk_mod

  implicit none

! Description    : e_loc(w)    = e_loc_kin (w) + e_loc_ee (w) + e_loc_en (w) + e_loc_vnn + ecp_bywlk(w)
! Description    : e_loc_hf(w) = e_loc_hf_kin (w) + e_loc_ee (w) + e_loc_en (w) + e_loc_vnn 

! Creates        : e_loc_tot_bywlk (walk)
! Creates        : e_loc_hf_tot_bywlk (walk)

  real(dp), pointer :: e_loc_tot_bywlk    (:) => NULL ()
  real(dp), pointer :: e_loc_hf_tot_bywlk (:) => NULL ()

! Needs          : e_loc_kin_bywlk (walk)
! Needs          : e_loc_vne_bywlk (walk)
! Needs          : e_loc_vee_bywlk (walk)
! Needs          : e_loc_vnn ()
! Needs          : ecp_bywlk (walk)

  contains

! =========================================================================
  subroutine e_loc_tot_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : e_loc_tot_bywlk_bld
! 
! Creates        : e_loc_tot_bywlk (walk)
!
! Needs          : e_loc_kin_bywlk (walk)
! Needs          : e_loc_vne_bywlk (walk)
! Needs          : e_loc_vee_bywlk (walk)
! Needs          : ecp_bywlk (walk)
! Needs          : e_loc_vnn
!
! Description    : e_loc(w) = e_loc_kin (w) + e_loc_ee (w) + e_loc_en (w) + ecp_bywlk (w)
!
! Author         : F. Colonna
! Date           : 02 Feb 2002
! Revision       : 21 Nov 2005 A.C.Kollias allocation check removed, now inside alloc
! -------------------------------------------------------------------------
  implicit none
 
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)      :: walk_i
  integer(dpi)      :: walkers_nb

! header:

  if (header_exe) then

     l_here = "e_loc_tot_bywlk_bld"

     description = 'local energy  (H psi_T /psi_T) -- e_loc_tot_bywlk (walk)'
     call object_create ('e_loc_tot_bywlk')

     call object_needed ('e_loc_kin_bywlk')
     call object_needed ('e_loc_vne_bywlk')
     call object_needed ('e_loc_vee_bywlk')
     call object_needed ('ecp_bywlk')
     call object_needed ('e_loc_vnn')
    return

  end if

! begin:
  call enter(l_here) 

! requirements:
  include 'chksiz_e_loc_kin_bywlk.inc'
  include 'chksiz_e_loc_vee_bywlk.inc'
  include 'chksiz_e_loc_vne_bywlk.inc'
! end requirements

! dimensions:
  walkers_nb = size(e_loc_kin_bywlk, 1)

! end dimensions

  if(debug)then
    write (logf,*)trim(l_here),'-d: walkers_nb=',walkers_nb 
    do walk_i = 1, walkers_nb
      write (logf,*)trim(l_here),'-d: e_loc_kin_bywlk (',walk_i,') =', &
                                e_loc_kin_bywlk(walk_i)
      write (logf,*)trim(l_here),'-d: e_loc_vee_bywlk (',walk_i,') =', &
                                e_loc_vee_bywlk(walk_i)
      write (logf,*)trim(l_here),'-d: e_loc_vne_bywlk (',walk_i,') =', &
                                e_loc_vne_bywlk(walk_i)
      write (logf,*)trim(l_here),'-d: e_loc_vnn =', e_loc_vnn
    end do
  end if

! allocations:
    call alloc ('e_loc_tot_bywlk',e_loc_tot_bywlk, walkers_nb)

! end allocations:

  e_loc_tot_bywlk = e_loc_kin_bywlk + e_loc_vne_bywlk + e_loc_vee_bywlk + e_loc_vnn + ecp_bywlk

  include 'debug_e_loc_tot_bywlk.inc'

  call exit(l_here)
 
  end subroutine e_loc_tot_bywlk_bld

! =========================================================================
  subroutine e_loc_hf_tot_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : e_loc_hf_tot_bywlk_bld
! 
! Creates        : e_loc_hf_tot_bywlk (walk)
!
! Needs          : e_loc_hf_kin_bywlk (walk)
! Needs          : e_loc_vne_bywlk (walk)
! Needs          : e_loc_vee_bywlk (walk)
! Needs          : e_loc_vnn
!
! Description    : e_loc(w) = e_loc_hf_kin (w) + e_loc_ee (w) + e_loc_en (w)
!
! Author         : A.C. Kollias 
! Date           : 21 Nov 2005
! -------------------------------------------------------------------------
  implicit none
 
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)      :: walk_i
  integer(dpi)      :: walkers_nb

! header:

  if (header_exe) then

     l_here = "e_loc_hf_tot_bywlk_bld"

     description = 'local energy  (H psi_hf /psi_hf) -- e_loc_hf_tot_bywlk (walk)'

     call object_create ('e_loc_hf_tot_bywlk')

     call object_needed ('e_loc_hf_kin_bywlk')
     call object_needed ('e_loc_vne_bywlk')
     call object_needed ('e_loc_vee_bywlk')
     call object_needed ('e_loc_vnn')
    return

  end if

! begin:
  call enter(l_here) 

! requirements:
  include 'chksiz_e_loc_vee_bywlk.inc'
  include 'chksiz_e_loc_vne_bywlk.inc'
! end requirements

! dimensions:
  walkers_nb = size(e_loc_hf_kin_bywlk, 1)

! end dimensions

  if(debug)then
    write (logf,*)trim(l_here),'-d: walkers_nb=',walkers_nb 
    do walk_i = 1, walkers_nb
      write (logf,*)trim(l_here),'-d: e_loc_hf_kin_bywlk (',walk_i,') =', &
                                e_loc_hf_kin_bywlk(walk_i)
      write (logf,*)trim(l_here),'-d: e_loc_vee_bywlk (',walk_i,') =', &
                                e_loc_vee_bywlk(walk_i)
      write (logf,*)trim(l_here),'-d: e_loc_vne_bywlk (',walk_i,') =', &
                                e_loc_vne_bywlk(walk_i)
      write (logf,*)trim(l_here),'-d: e_loc_vnn =', e_loc_vnn
    end do
  end if

! allocations:
    call alloc ('e_loc_hf_tot_bywlk',e_loc_hf_tot_bywlk, walkers_nb)
! end allocations:

  e_loc_hf_tot_bywlk = e_loc_hf_kin_bywlk + e_loc_vne_bywlk + e_loc_vee_bywlk + e_loc_vnn 

  include 'debug_e_loc_tot_bywlk.inc'

  call exit(l_here)
 
  end subroutine e_loc_hf_tot_bywlk_bld

end module e_loc_tot_bywlk_mod



















