! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module e_loc_vee_bywlk_mod

  use basic_types_tools_mod
  use ee_dist_bywlk_mod

  implicit none

! Description   : calculate the electron-electron Coulomb potential
! Creates       : e_loc_vee_bywlk (walk)

  real(dp), pointer :: e_loc_vee_bywlk (:) => NULL()

! Needs         : ee_dist_inv_bywlk (elec, elec, walk)

  contains

! ========================================================================
  subroutine e_loc_vee_bywlk_bld 
! ------------------------------------------------------------------------
!
! Name           : e_loc_vee_bywlk_bld
!
! Creates        : e_loc_vee_bywlk (walk)
! 
! Needs          : ee_dist_inv_bywlk (elec, elec, walk)
!
! Description    : calculate the electron-electron Coulomb potential 
! Description    : for all configuration
 
! Author:   F. Colonna
! Date:     02 Feb 2002
! ------------------------------------------------------------------------
  implicit none
 
! i/o:
 
! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: elec_i, elec_j, walk_i
  integer(dpi)      :: walkers_nb
  integer(dpi)      :: electrons_nb

! header:

  if (header_exe) then

    l_here = "e_loc_vee_bywlk_bld"
    description = 'electron - electron interaction -- e_loc_vee_bywlk (walk)'
    call object_create ('e_loc_vee_bywlk')

    call object_needed ('ee_dist_inv_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements
  include 'chksiz_ee_dist_inv_bywlk.inc'
! end requirements

  walkers_nb = size(ee_dist_inv_bywlk, 3)
  electrons_nb = size(ee_dist_inv_bywlk, 1)

  if(debug)then
    write (logf,*)trim(l_here),'-d: walkers_nb=',walkers_nb 
    write (logf,*)trim(l_here),'-d: electrons_nb=',electrons_nb 
  end if

! allocations:
    call alloc ('e_loc_vee_bywlk', e_loc_vee_bywlk, walkers_nb )

! end allocations:

  e_loc_vee_bywlk = 0.d0
  do walk_i = 1, walkers_nb
    do elec_i = 1, electrons_nb
      do elec_j = elec_i + 1, electrons_nb

        e_loc_vee_bywlk (walk_i) = e_loc_vee_bywlk (walk_i) &
                   + ee_dist_inv_bywlk (elec_i, elec_j, walk_i)

        end do
     end do 
  end do

  if(debug)then
    write (logf,*)trim(l_here),'-d: '
    write (logf,*)trim(l_here),'-d: e_loc_vee_bywlk (walk)'
    do walk_i = 1, walkers_nb 
      write (logf,*)trim(l_here),'-d: e_loc_vee_bywlk (',walk_i,') =', &
                                e_loc_vee_bywlk (walk_i)
    end do
  end if

  call exit(l_here)
 
  end subroutine e_loc_vee_bywlk_bld
 
end module e_loc_vee_bywlk_mod
