!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module e_loc_vnn_mod

  use basic_types_tools_mod
  use nuclei_mod
  use atom_atom_distance_mod

  implicit none

! Description   : calculate the atom-atom Coulomb potential
! Creates       : e_loc_vnn 

  real(dp), pointer :: e_loc_vnn => NULL()

! Needs         : atoms(atom)%charge
! Needs         : atom_atom_dist_inv (atom, atom)

  contains

! ========================================================================
  subroutine e_loc_vnn_bld
! ------------------------------------------------------------------------
! 
! Name           : e_loc_vnn_bld
! 
! Creates        : e_loc_vnn
!
! Description    : calculate the nuclei-nuclei Coulombic potential 
!
! Needs          : atoms(atom)%charge
! Needs          : atom_atom_dist_inv (atom, atom)
!
! Author:F. Colonna
! Date  :26 Mar 2002
! ------------------------------------------------------------------------
  implicit none
 
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: atom_i, atom_j
  integer(dpi)      :: atoms_nb

! header:

  if (header_exe) then

    l_here = "e_loc_vnn_bld"

    call object_create ('e_loc_vnn')

    call object_needed ('atoms_charges')
    call object_needed ('atom_atom_dist_inv')

    return

  end if

! begin:
  call enter(l_here) 

! requirements:

  include 'chksiz_atoms.inc'
  include 'chksiz_atom_atom_dist_inv.inc'

! end requirements

! dimensions:
  atoms_nb = size (atom_atom_dist_inv, 1)
 
! end dimensions
  if(debug)then
    write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
  end if

! allocations:
    call alloc ('e_loc_vnn',e_loc_vnn )

! end allocations:

  e_loc_vnn = zero

  do atom_j = 1,atoms_nb
    do atom_i = atom_j + 1, atoms_nb
        e_loc_vnn = e_loc_vnn &
          + atoms_charges (atom_i)    &
          * atoms_charges (atom_j)    &
          * atom_atom_dist_inv (atom_i, atom_j)
    end do
  end do

  if(debug) write (logf,*)trim(l_here),'-d: e_loc_vnn =',e_loc_vnn

  call exit(l_here)
 
  end subroutine e_loc_vnn_bld
 
end module e_loc_vnn_mod
