!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module e_reference_bywlk_mod

  use basic_types_tools_mod
  use nuclei_mod
  use system_data_mod

  implicit none

! Description        : e_reference_bywlk (walk)
  real(dp), pointer :: e_reference_bywlk (:) => NULL()

  contains

! ========================================================================
  subroutine e_reference_bywlk_bld 
! ------------------------------------------------------------------------
! 
! Name           : e_reference_bywlk_bld
! 
! Description    : e_reference_bywlk (walk)     
!
! Creates        : e_reference_bywlk (walk)
!
! Needs          : 
!
! Author: F. Colonna 
! Date  : 23 Jan 2002
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  real(dp)                      :: e_trial

! header:

  if (header_exe) then

    l_here = "e_reference_bywlk_bld"

    call object_create ('e_reference_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! require_atom: 
  atoms_nb = size(atoms)
! require_atom: 
  if(atoms_nb == 0) call die (l_here,' nb of atoms ='+atoms_nb)

  walkers_nb = system%walkers_nb
  if(walkers_nb == 0) call die (l_here,' nb of walkers ='+walkers_nb)
  
  if(nuclear_potential_energy <= NOT_INIT_DP) then
     call nuclear_potential
  end if

  call alloc ('e_reference_bywlk', e_reference_bywlk, walkers_nb)

  if (debug) write (logf,*)trim(l_here),'-d: nb of walkers = ',walkers_nb
  if (debug) write (logf,*)trim(l_here),'-d: nb of atoms   = ',atoms_nb

  do walk_i = 1, walkers_nb

    e_trial = zero
    do atom_i = 1, size(atoms)
      e_trial = e_trial - 0.52180d+00*atoms(atom_i)%charge**2.3930d+00
    enddo

    e_reference_bywlk (walk_i)  = e_trial + nuclear_potential_energy
    write (logf,'(2a,i5)') trim(l_here),': walker # ',walk_i
    write (logf,'(2a,g15.8)') trim(l_here), &
    ': initial estimated energy (Sum_atom 0.5218 Z^2.393) = ', &
    e_reference_bywlk(walk_i)

    end do
 
  call exit(l_here)

  end subroutine e_reference_bywlk_bld 

end module e_reference_bywlk_mod
