! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module ecp_bywlk_mod

  use basic_types_tools_mod
  use drift_bywlk_mod
  use e_loc_kin_bywlk_mod
  use e_loc_vne_bywlk_mod
  use e_loc_vee_bywlk_mod
  use e_loc_vnn_mod
  use ecp_mod

  use mos_on_grid_mod
  use walkers_bywlk_mod
  use tools_mc_mod
  implicit none

  real(dp), pointer      :: ecp_bywlk (:) => NULL () 
  real(dp), pointer      :: ul_partial_ea_bywlk (:,:,:) => NULL ()
  real(dp), pointer      :: coef_ecp_ea_bywlk (:,:,:) => NULL ()
  real(dp), pointer      :: ecp_ea_bywlk (:,:,:) => NULL () 
  real(dp), pointer      :: elec_ecp_random_rot_bywlk (:,:,:,:,:) => NULL ()
  real(dp), allocatable  :: psi_ylm_elec_ecp_bywlk(:,:,:,:)
   
!  real(dp), allocatable  :: coef_ecp_l(:,:)
   
contains

! =============================================================================
  subroutine  ecp_ea_bywlk_bld
! -----------------------------------------------------------------------------
! 
! Name : ecp_ea_bywlk_bld
! 
! Creates : ecp_ea_bywlk(elec, atom,  walk)
!       
! needs   : psi_ylm_elec_ecp_bywlk 
! needs   : elec_coor_bywlk
! needs   : psi_bywlk
! needs   : atoms_coordinates
! needs   : list_ecp_atoms
! needs   : ea_distance_bywlk
! needs   : ea_dist_xyz_bywlk
!
! Description: ecp_ea_bywlk(elec, atom,  walk)
! Description: ecp value for the electron 'elec', atom 'atom', walker 'walk'
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Dc 2000
! Revision   : 27 Apr 2001 R. Assaraf  deallocate statements 
! Revision   : 01 Feb 2001 => PP : skip atoms vith no ecp
! Revision   : 09 Dec 2003  R. Assaraf 
! Revision   : function ecp_by_atom(coords,psi_func,iatom) -> Osipe form
! -----------------------------------------------------------------------------
  implicit none
   
! i/o:

! local
  integer(dpi)          :: atom_i, elec_i, walk_i
  integer(dpi)          :: l, m, indlmx, ecp_i
  integer(dpi)          :: icoef
  integer(dpi)          :: ecp_atoms_nb
  real(dp)              :: val
  real(dp)              :: xyzr(4)
  real(dp), allocatable, save :: Ylm(:)
  real(dp), allocatable, save :: Ul(:)

  integer(dpi) :: dimensions_nb, atoms_nb, electrons_nb, walkers_nb
  integer(dpi) ::  indlm
 
  character(max_string_len) :: l_here

! header
  if (header_exe) then

     l_here = "ecp_ea_bywlk_bld"

     description = 'ecp_ea_bywlk(elec, atom, walk) contribution to ecp'
     call object_create ('ecp_ea_bywlk')
       
     call object_needed ('psi_ylm_elec_ecp_bywlk')
     call object_needed ('elec_coor_bywlk')
     call object_needed ('psi_bywlk')
     call object_needed ('atoms_coordinates')
     call object_needed ('list_ecp_atoms')
     call object_needed ('ea_distance_bywlk')
     call object_needed ('ea_dist_xyz_bywlk')
    
     return

  end if


! begin
  l_here ='ecp_by_atom_by_electron'
  call enter (l_here,3)
 
  atoms_nb = size (atoms_coordinates, dim =1)
  walkers_nb = size (elec_coor_bywlk, dim=3)
  electrons_nb = size (elec_coor_bywlk, dim=1)
  ecp_atoms_nb = size (list_ecp_atoms)

     call alloc ('ecp_ea_bywlk',ecp_ea_bywlk, &
          electrons_nb, atoms_nb, walkers_nb)

  indlmx=(maxl+1)**2
! local allocations 
  
  if (.not.allocated(ylm)) allocate(ylm(indlmx))
  if (.not.allocated(Ul)) allocate(Ul(0:maxl))
  if (.not.allocated(coef_ecp_l))  allocate(coef_ecp_l(0:maxl,-2:2))
  
  ecp_ea_bywlk = 0.d0 

  do ecp_i = 1, ecp_atoms_nb
     atom_i = list_ecp_atoms (ecp_i)
        
     do walk_i = 1, walkers_nb   
        do elec_i =1, electrons_nb
! calculate relative positions and electon-nuclei coordinates
           xyzr(1:3)= ea_dist_xyz_bywlk (elec_i, atom_i,:, walk_i)
           xyzr(4) = ea_distance_bywlk (elec_i,atom_i,walk_i)
! exit if electron out of ecp range 
! improve
           if(xyzr(4).gt.range(atom_i)) then 
              ecp_ea_bywlk (elec_i, atom_i, walk_i)=0.
              coef_ecp=0.
           endif
 
! Ylm 
           call calc_Ylm(xyzr,Ylm,maxl)

! Ul 
           call calc_Ul_partial(atom_i, xyzr, Ul)
    
! final summation
           val=0.
           do l = 0, lmecp(atom_i)
              do m = -l, l
                 indlm = l*l+l+1+m
                 val =val + ylm(indlm)*&
                      psi_ylm_elec_ecp_bywlk(indlm, elec_i,ecp_i,walk_i)&
                      *Ul(l)
              enddo ! do m
           enddo ! do l
           ecp_ea_bywlk (elec_i, atom_i, walk_i) = &
                val/psi_bywlk (walk_i) + Ul(lmecp(atom_i)+1)

! final summation for partial coefficients

           do icoef = -2, 0
              val=0.
              do l = 0, lmecp(atom_i)
                 do m = -l, l
                    indlm = l*l+l+1+m
                    val =val + ylm(indlm)*&
                         psi_ylm_elec_ecp_bywlk(indlm, elec_i, ecp_i, walk_i)&
                         *coef_ecp_l(l,icoef)
                 enddo ! do m
              enddo ! do  l
              coef_ecp(icoef) = val/psi_bywlk (walk_i)
           enddo ! do icoef
           coef_ecp(-1) = coef_ecp(-1) + Ul(lmecp(atom_i)+1)
           
        enddo ! elec_i
     enddo  ! walk_i
     
  enddo ! do atom_i

  call exit (l_here,3)

end subroutine ecp_ea_bywlk_bld

! =======================================================   
  subroutine calc_Ul_partial (atom_i, xyzr, Ul)
! -------------------------------------------------------
! 
! Arguments  : subroutine calc_Ul_partial(atom_i,xyzr,Ul)
! Description: evaluate SBK Ul function, and returns its
! Description: decomposition in powers of r
! 
! Authors    : P. Pernot
! Date       : 6 mars 2001
! -------------------------------------------------------                     
  implicit none
  
! i/o:
      integer(dpi), intent(in)      :: atom_i
      double precision, intent(in)  :: xyzr(4)
      double precision, intent(out) :: Ul(0:*)
    
! local:
      character(len=max_string_len_routine_name) :: l_here
      integer(dpi)        :: l, iterm, power
      real(dp), parameter :: zero=0.d0
      real(dp)            :: r, contrib, fac
      
! begin
 l_here ='calc_Ul_partial'
 call enter (l_here,3)
    
!      do l=1,size(nUL,dim=1)
!      print*,'nUL(',l,',1,:,1)=',nUL(l,:,1)
!      enddo

!      do l=1,size(cUL,dim=1)
!      print*,'cUL(',l,',1,:,1)=',cUL(l,:,1)
!      enddo
 
!      do l=1,size(zUL,dim=1)
!      print*,'zUL(',l,',1,:,1)=',zUL(l,:,1)
!      enddo
      
  r=xyzr(4)
  fac=1./(r+tiny(r))**2
  coef_ecp_l=zero
  do l= 0, lmecp(atom_i)+1
     Ul(l)=zero
     do iterm= 1, ntUl(atom_i)
        power = nUl(l,iterm,atom_i)
        contrib  = cUl(l,iterm,atom_i)* &
             r**power * &
             exp(-zUl(l,iterm,atom_i)*r*r)* &
             fac
        coef_ecp_l(l,power-2) = coef_ecp_l(l,power-2) + contrib
        Ul(l) = Ul(l) + contrib
     enddo
  enddo
    
  call exit (l_here,3)
    
end subroutine calc_Ul_partial

! =======================================================  
  subroutine psi_ylm_elec_ecp_bywlk_bld 
! -------------------------------------------------------  
! Name : psi_ylm_elec_ecp_bywlk_bld
!
! Create : psi_ylm_elec_ecp_bywlk (indlmx, elec, ecp_i, walk)
!
! Needs : elec_coor_bywlk (elec,dim, walk)
! Needs : atoms_coordinates (atom,dim)
! Needs : ea_distance_bywlk (elec,atom,dim,walk)
! Needs : list_ecp_atoms (ecp_i)
!
! Description:  Ylm*Psi at xyzr point
! 
! Authors    : B. Lvy, P. Pernot
! Date       : 15 Nov 2000
! Revision   : 23 Dec 2003 by R. Assaraf, Osipe form
! ------------------------------------------------------- 
  implicit none
    
! i/o:

! local:
  integer(dpi) :: ecp_atoms_nb, walkers_nb, dimensions_nb, electrons_nb
  integer(dpi) :: atom_i, elec_i, ecp_i, walk_i, dim_i
  integer(dpi)  :: iquad, m, l, i_r
  real(dp), allocatable, save      :: psi_loc (:)
  real(dp), pointer      :: coor_save(:,:,:)     => NULL ()
  real(dp)  :: r_ia
  character(max_string_len) :: l_here
  integer(dpi) :: indlm, indlmx
  real(dp)  :: a(3,3)
  logical(bool) :: first_passage = .true.
! header:

  if (header_exe) then

    l_here = "psi_ylm_elec_ecp_bywlk_bld"

    description = 'Ylm*Psi at quadrature points'
    call object_create ('psi_ylm_elec_ecp_bywlk')
    
    call object_needed ('r_quad_atom_mos_grid') ! grid
    call object_needed ('elec_coor_bywlk')
    call object_needed ('atoms_coordinates')
    call object_needed ('elec_ecp_random_rot_bywlk')  
    call object_needed ('ea_distance_bywlk')
    call object_needed ('list_ecp_atoms')
    call object_needed ('psi_bywlk')
    call object_needed ('quad')
    call object_needed ('wlm')
  
    return

  end if

! begin:
  l_here ='psi_ylm_elec_ecp_bywlk_bld'
  call enter (l_here,3)

! dimensions:
  electrons_nb = size (elec_coor_bywlk, dim=1)
  ecp_atoms_nb = size (list_ecp_atoms)
  dimensions_nb= size (elec_coor_bywlk, dim=2)
  walkers_nb   = size (elec_coor_bywlk, dim=3)
 
! allocations: 
  if (.not.allocated(psi_loc)) then
     allocate(psi_loc(walkers_nb))
  endif

 if (.not.associated(coor_save)) then
     allocate (coor_save (electrons_nb, dimensions_nb, walkers_nb))
  endif
  
  coor_save = elec_coor_bywlk

  indlmx=(maxl+1)**2

  if (.not.allocated(psi_ylm_elec_ecp_bywlk)) &
       allocate(psi_ylm_elec_ecp_bywlk(indlmx, electrons_nb, ecp_atoms_nb, walkers_nb))

! end allocations 
 
  do ecp_i=1, size (list_ecp_atoms)
     atom_i=list_ecp_atoms (ecp_i)

     do elec_i=1, electrons_nb
        psi_ylm_elec_ecp_bywlk(:,elec_i,ecp_i,:) = 0.d0 
        QUAD_LOOP: do iquad = 1, n_quad
       
! absolute coordinates of the quadrature points
 
           do walk_i=1, walkers_nb
              r_ia = ea_distance_bywlk (elec_i,atom_i,walk_i) 
              if (r_ia < range(atom_i)) then

                 select case(ecp_method)

                 case ("randomize")
                    a = elec_ecp_random_rot_bywlk (elec_i, ecp_i, :, :, walk_i)
                    do dim_i = 1, 3
                       elec_coor_bywlk (elec_i, dim_i, walk_i) =         &
                            atoms_coordinates(atom_i, dim_i)  + r_ia * ( &
                            + a(dim_i,1) * quad (iquad,1)                &
                            + a(dim_i,2) * quad (iquad,2)                &
                            + a(dim_i,3) * quad (iquad,3) )
                    enddo ! do dim_i

                 case ("grid")
!quadrature points on grid
                    i_r = int (r_ia/range(ecp_i)*radial_mesh_nb)+1

                    if (i_r<1 .or. i_r > radial_mesh_nb) call die (l_here, "i_r="+i_r)

                    mos_bywlk (elec_i, : ,walk_i) = &
                         r_quad_atom_mos_grid (i_r, iquad, ecp_i, :)

                 case default
                    elec_coor_bywlk(elec_i,1,walk_i) =&
                         r_ia*quad(iquad,1) &
                         + atoms_coordinates (atom_i, 1)
   
                    elec_coor_bywlk(elec_i,2,walk_i) =&
                         r_ia*quad(iquad,2) &
                         + atoms_coordinates (atom_i, 2)

                    elec_coor_bywlk(elec_i,3,walk_i) =&
                         r_ia*quad(iquad,3) &
                         + atoms_coordinates (atom_i, 3)

                    
                 end select
                 elec_valid_bywlk (elec_i,walk_i) = .false.
              endif
           enddo ! do walk_i
                 
           only_one_electron = elec_i
   
           if (ecp_method.eq."grid") then
              call psi_from_mos_coor_calc
           else
              call psi_from_coor_bywlk_calc
           endif

!         call psi_from_mos_coor_calc
!           call psi_from_coor_bywlk_calc
 
          psi_loc = psi_bywlk
           indlmx = (maxl+1)**2    
           do l= 0, lmecp(atom_i)
              do m= -l, l          
                 indlm=l*l+l+1+m
                 do walk_i=1, walkers_nb
                    if (.not.elec_valid_bywlk (elec_i,walk_i)) then
                    psi_ylm_elec_ecp_bywlk(indlm,elec_i,ecp_i,walk_i) = &
                         psi_ylm_elec_ecp_bywlk(indlm,elec_i,ecp_i,walk_i) &
                         + wlm(iquad,indlm) * psi_loc (walk_i)
                  endif
                 enddo ! do walk_i
!           if(debug) write(6,'(1X,A,5i5,e15.8)') trim(l_here)//'-d : ',&
!                       atom_i,iquad,elec_i,l,m,psi_ylm_elec_ecp_bywlk(indlm)
              enddo ! do m
           enddo ! do l
  
        enddo QUAD_LOOP
        elec_coor_bywlk = coor_save
         
        call psi_from_coor_bywlk_calc 
        elec_valid_bywlk = .true.
     enddo ! do elec_i  
  enddo ! do ecp_i
  only_one_electron = 0
  elec_valid_bywlk  = .false.


  call exit (l_here,3)

end subroutine psi_ylm_elec_ecp_bywlk_bld


! =========================================================================
  subroutine elec_ecp_random_rot_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : eas_random_bld
! 
! Creates        : elec_ecp_random_rot (elec, ecp, dim, dim, walk)
!
! Needs          : elec_coor_bywlk
! Needs          : list_ecp_atoms
! 
! Description    : random rotation matrix by electron by atom by walker 
!               
!
! Author:R. Assaraf
! Date  : 9 May 2005
! -------------------------------------------------------------------------
  use ecp_mod
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: walk_i, elec_i, atom_i, ecp_i
  integer(dpi) :: atoms_nb, electrons_nb, walkers_nb, ecp_atoms_nb, dimensions_nb
  real(dp)     :: a(3,3)
  real(dp)     :: psi, teta, phi
  real(dp)     :: cos_psi, sin_psi, cos_teta, sin_teta, cos_phi, sin_phi
! header:

  if (header_exe) then

    l_here = "elec_ecp_ramdom_bywlk_bld"

    description = 'random rotation'
    call object_create ('elec_ecp_random_rot_bywlk')
  
    call object_needed ('elec_coor_bywlk')
    call object_needed ('list_ecp_atoms')
    return

  end if

! begin:
  call enter(l_here) 
  electrons_nb  = size (elec_coor_bywlk,1)
  dimensions_nb = size (elec_coor_bywlk,2)
  walkers_nb = size(elec_coor_bywlk, 3)
  
  ecp_atoms_nb = size (list_ecp_atoms)
  
! end dimensions

! allocations:
  if (.not.associated (elec_ecp_random_rot_bywlk)) then
     call alloc ('elec_ecp_random_rot_bywlk', elec_ecp_random_rot_bywlk, &
          electrons_nb, ecp_atoms_nb, dimensions_nb, dimensions_nb, walkers_nb)
  endif
! end allocations:
   
 
  if (ecp_method.eq."randomize") then
     do walk_i = 1, walkers_nb

        do ecp_i=1, ecp_atoms_nb

           do elec_i=1, electrons_nb
       
              psi   = ranfmc() * twopi
              phi   = ranfmc() * twopi
              teta  = ranfmc() * twopi

              sin_psi = sin(psi)
              cos_psi = cos(psi)
           
              sin_phi = sin (phi)
              cos_phi = cos (phi)
              
              sin_teta = sin (teta)
              cos_teta = cos (teta)
              
              a(1,1) = -sin_psi * sin_phi + cos_teta * cos_phi * cos_psi
              a(1,2) =  sin_psi * cos_phi + cos_teta * sin_phi * cos_psi
              a(1,3) = -cos_psi * sin_teta
              a(2,1) = -cos_psi * sin_phi - cos_teta * cos_phi * sin_psi
              a(2,2) =  cos_psi * cos_phi - cos_teta * sin_phi * sin_psi
              a(2,3) =  sin_psi * sin_teta
              a(3,1) =  sin_teta * cos_phi
              a(3,2) =  sin_teta * sin_phi
              a(3,3) =  cos_teta 

              elec_ecp_random_rot_bywlk (elec_i, ecp_i,:,:,walk_i) = a

           enddo ! do elec_i
           
        enddo ! do ecp_i
        
     enddo ! do walk_i
  endif
  call exit(l_here)
 
end subroutine elec_ecp_random_rot_bywlk_bld

! =========================================================================
  subroutine ecp_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : ecp_bywlk_bld
! 
! Creates        : ecp_bywlk(walk)
!
! Needs          : elec_coor_bywlk
! 
! Description    : pseudo-potential
!
! Author:R. Assaraf
! Date  :03 Dec 2003
! -------------------------------------------------------------------------
  use ecp_mod
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: walk_i, elec_i, atom_i
  real(dp) :: ecp_val
  integer(dpi) :: atoms_nb, electrons_nb, walkers_nb
! header:

  if (header_exe) then

    l_here = "ecp_bywlk_bld"

    description = 'ecp values'
    call object_create ('ecp_bywlk')
  
    call object_needed ('ecp_ea_bywlk')
    call object_needed ('elec_coor_bywlk')
    call object_needed ('psi_bywlk')
    call object_needed ('atoms_coordinates')
    return

  end if

! begin:
  call enter(l_here) 

  atoms_nb = size (atoms_coordinates, dim = 1)
  walkers_nb = size(elec_coor_bywlk, 3)
  electrons_nb = size (elec_coor_bywlk,1)
  
! end dimensions

! allocations:

     call alloc ('ecp_bywlk',ecp_bywlk, walkers_nb)
 
! end allocations:
  ecp_bywlk = 0.d0

  do atom_i=1, atoms_nb
     do elec_i=1, electrons_nb
        ecp_bywlk= ecp_bywlk + ecp_ea_bywlk (elec_i, atom_i, :)
     enddo ! do elec
  enddo ! do atom_i

  call exit(l_here)
 
end subroutine ecp_bywlk_bld


! ===============================================================================
 subroutine psi_from_coor_bywlk_calc 
! -------------------------------------------------------------------------------
!
! Name           : 
!
! Argument (in ) : 
!
! Description    : 
!
! Authors:
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  
! result:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi), pointer :: path_bynod (:) => NULL ()
  integer(dpi), save :: coor_index (1)
! begin:
  l_here ='psi_from_coor_bywlk_calc'
  call enter (l_here,5)
  

  if (.not.associated (path_bynod)) then
     
     call path_bynod_bld (path_bynod, (/object_index('elec_coor_bywlk')/),&
          (/object_index('psi_bywlk')/))
  endif

  is_valid_bynod (path_bynod) = .false.
  call provide_bynam ('psi_bywlk')

!  call path_bynod_exe (path_bynod)
 
  
  call exit (l_here,5)
  
end subroutine psi_from_coor_bywlk_calc
  

! ===============================================================================
 subroutine psi_from_mos_coor_calc 
! -------------------------------------------------------------------------------
!
! Name           : 
!
! Argument (in ) : 
!
! Description    : 
!
! Authors:
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  
! result:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi), pointer :: path_bynod (:) => NULL ()
  integer(dpi), save :: coor_index (1)
! begin:
  l_here ='psi_from_mos_coor_calc'
  call enter (l_here,5)
  

  if (.not.associated (path_bynod)) then
     
     call path_bynod_bld (path_bynod, (/object_index('elec_coor_bywlk'),&
                                        object_index('mos_bywlk')/),      &
          (/object_index('psi_bywlk')/))
  endif
 
  call path_bynod_exe (path_bynod)
 
  call exit (l_here,5)
  
end subroutine psi_from_mos_coor_calc 

! ===============================================================================
 subroutine mos_from_coor_calc 
! -------------------------------------------------------------------------------
!
! Name           : 
!
! Argument (in ) : 
!
! Description    : 
!
! Authors:
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  
! result:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi), pointer :: path_bynod (:) => NULL ()
! begin:
  l_here ='mos_from_coor_calc'
  call enter (l_here,5)
  

  if (.not.associated (path_bynod)) then
     
     call path_bynod_bld (path_bynod, (/object_index('elec_coor_bywlk')/),&
          (/object_index('mos_bywlk')/))
  endif
 
  call path_bynod_exe (path_bynod)
 
  call exit (l_here,5)
  
end subroutine mos_from_coor_calc 




end module ecp_bywlk_mod


















