!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module electrons_menu_mod

! ------------------------------------------------------------------------
! 
! Author: F.Colonna
! Date  : 28 Oct 2002
! Revision: electron part of wavefunction_menu
! ------------------------------------------------------------------------
 
  use system_data_mod
  use parser_mod

  contains
 
! ========================================================================
  subroutine electrons_menu
! ------------------------------------------------------------------------
! Name           : electrons_menu
 
! Description    : input electrons information
 
! Author: F. Colonna
! Date  : 28 Oct 2008
! ------------------------------------------------------------------------
  implicit none
    
! local:
  logical(bool)       :: done
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)        :: charge
  integer(dpi)        :: electrons_nb
  integer(dpi)        :: alfa_electrons_nb
  integer(dpi)        :: beta_electrons_nb
    
! begin:
  l_here ='electrons_menu'
  call enter(l_here)

! initialize:
  alfa_electrons_nb = NOT_INIT_I
  beta_electrons_nb = NOT_INIT_I
  electrons_nb      = NOT_INIT_I
  charge            = NOT_INIT_I
    
  done = .false.
  call next_command_get ('electrons')
  do while (.not.done)
     call next_word_get ('electrons')
     if(debug) write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
     if(trim(cur_word) .eq. 'help') then
      write (logf,*) 'elec-h: menu'
      write (logf,*) 'elec-h: syntax'
      write (logf,*) 'elec-h: ele{ctrons}'
      write (logf,*) 'elec-h:   help'
      write (logf,*) 'elec-h:   alf{a_electrons_nb} = [integer]'
      write (logf,*) 'elec-h:   alp{ha_electrons_nb}= [integer]'
      write (logf,*) 'elec-h:   bet{a_electrons_nb} = [integer]'
      write (logf,*) 'elec-h:   cha{rge}            = [integer] molecular'
      write (logf,*) 'elec-h:   ele{ctrons}         = [integer]'
      write (logf,*) 'elec-h:   end_ele'
      write (logf,*) 'elec-h: example'
      write (logf,*) 'elec-h: ele alfa = 2 beta = 3 end_ele'
      
  else if(token_is ('alf', alfa_electrons_nb)) then

  else if(token_is ('bet', beta_electrons_nb)) then
      
  else if(token_is ('cha', charge)) then
      
  else if(token_is ('ele', electrons_nb)) then
      
  else
    call menu_exit (l_here, done)
  end if
     
  end do
    
! check consistency:
  if(alfa_electrons_nb ==  NOT_INIT_I .and.  &
     electrons_nb      ==  NOT_INIT_I .and.  &
     beta_electrons_nb ==  NOT_INIT_I) then

     write (logf,*)trim(l_here),'-error: '
     write (logf,*)trim(l_here),'-e: alfa_electrons_nb =',alfa_electrons_nb
     write (logf,*)trim(l_here),'-e: beta_electrons_nb =',beta_electrons_nb
     write (logf,*)trim(l_here),'-e: electrons_nb      =',electrons_nb
     call die (l_here, 'enter electron number')

  else if(alfa_electrons_nb ==  NOT_INIT_I .and.  &
          electrons_nb      /=  NOT_INIT_I .and. &
          beta_electrons_nb /=  NOT_INIT_I) then
     alfa_electrons_nb = electrons_nb - beta_electrons_nb

  else if(beta_electrons_nb ==  NOT_INIT_I .and.  &
          electrons_nb      /=  NOT_INIT_I .and.   &
          alfa_electrons_nb /=  NOT_INIT_I) then
     beta_electrons_nb = electrons_nb - alfa_electrons_nb

  else if(electrons_nb      ==  NOT_INIT_I .and.  &
          alfa_electrons_nb /=  NOT_INIT_I .and. &
          beta_electrons_nb /=  NOT_INIT_I) then
     electrons_nb = alfa_electrons_nb + beta_electrons_nb

  else if(electrons_nb /= alfa_electrons_nb + beta_electrons_nb) then

     write (logf,*)trim(l_here),'-error: '
     write (logf,*)trim(l_here),'-e: alfa_electrons_nb =',alfa_electrons_nb
     write (logf,*)trim(l_here),'-e: beta_electrons_nb =',beta_electrons_nb
     write (logf,*)trim(l_here),'-e: electrons_nb      =',electrons_nb
     call die (l_here, 'enter correct electron number')

  end if

! Improve check the charge somewhere
!    n_charge = zero
!    do atom_i=1,size(atoms)
!     n_charge = n_charge + atoms(atom_i)%charge
!    enddo
!    l_charge = n_charge - float(electrons_nb)
!    if(abs(l_charge - charge) > EPSILON(+1.0)) then
!     message='system charge from input = '*charge &
!             +CR*" "*l_here+'-die: sum of nuclei charges ('+n_charge &
!             +') - electrons charge ('+electrons_nb+') = '*l_charge  &
!             +CR*" "*l_here+'-die: please check charge in input'
!     call die (l_here, message)
!    endif

    if(print_level .ge. 0 .or. debug) then
     write (logf,*) trim(l_here), &
          ': charge            =',charge
     
     write (logf,*) trim(l_here), &
          ': alfa_electrons_nb =',alfa_electrons_nb
     
     write (logf,*) trim(l_here), &
          ': beta_electrons_nb =',beta_electrons_nb 
     
     write (logf,*) trim(l_here), &
          ': electrons_nb      =',electrons_nb
     
    end if
    
! define system here:
    system%electrons_nb      = electrons_nb
    system%alfa_electrons_nb = alfa_electrons_nb
    system%beta_electrons_nb = beta_electrons_nb
    system%charge            = charge
    
  call exit(l_here)
    
  end subroutine electrons_menu
 
end module electrons_menu_mod
