! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module electrons_spins_mod

  use basic_types_tools_mod
  use tools_mod
  use system_data_mod
  implicit none

! Description            : electrons_spins (elec) = +1, -1
! Creates                : electrons_spins (elec) 

  integer(dpi), pointer :: electrons_spins(:) => NULL()
  
  contains

! ========================================================================
  subroutine electrons_spins_bld
! ------------------------------------------------------------------------
!
! Name           : electrons_spins_bld 
!
! Description    : electrons_spins (elec) = +1, -1
! Creates        : electrons_spins (elec) 
!
! Authors        : F. Colonna
! Date           : 31 Jan 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: alfa_electrons_nb
  integer(dpi)                  :: beta_electrons_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: elec_i

! header:

  if (header_exe) then

    l_here = "electrons_spins_bld"

    call object_create ('electrons_spins')

    return

  end if

! begin:
  call enter (l_here)

  alfa_electrons_nb = system%alfa_electrons_nb
  beta_electrons_nb = system%beta_electrons_nb
  electrons_nb      = system%electrons_nb

  if(alfa_electrons_nb == 0) call die (l_here,' nb of alfa_electrons_nb ='+alfa_electrons_nb)

  if(beta_electrons_nb == 0) call die (l_here,' nb of beta_electrons_nb ='+beta_electrons_nb)

! allocations:

  if(.not.associated(electrons_spins)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': electrons_spins not yet associated'
      call alloc ('electrons_spins', &
                   electrons_spins,  &
                   electrons_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': electrons_spins left in memory'
  end if

! end allocations

  if(debug) then
    write (logf,*)trim(l_here),'-d: alfa_electrons_nb =',alfa_electrons_nb
    write (logf,*)trim(l_here),'-d: beta_electrons_nb =',beta_electrons_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      =',electrons_nb
  end if

  electrons_spins = 1
  if(beta_electrons_nb > 0) then
    do elec_i = alfa_electrons_nb+1, electrons_nb
      electrons_spins(elec_i) = -1
    end do
  end if

  include 'debug_electrons_spins.inc'
!   include 'ensnb_electrons_spins.inc'

  call exit (l_here)

  end subroutine electrons_spins_bld
  
end module electrons_spins_mod
