! -*- F90 -*-
module expressions_mod

  use parser_mod
  use objects_paths_mod
  use basic_types_tools_mod

! Descriptions : expression is real (........)
! Descriptions :               bool (........)
! Descriptions :               inte (........)
! Descriptions : expressions_first_word (expr) first parenthesis
! Descriptions : expressions_last_word  (expr) last  parenthesis
! Descriptions : expressions_name       (expr)
! Descriptions : expressions_type       (expr) 'real', 'bool', 'inte'
! Descriptions : expression_words       (expr) 'word of the expression'

  character(len=max_string_len_object_name), pointer :: expressions_name  (:) => NULL ()
  character(len=max_string_len_expr_type),   pointer :: expressions_type  (:) => NULL ()
  integer(dpi), pointer                              :: expressions_first_word (:) => NULL ()
  integer(dpi), pointer                              :: expressions_last_word  (:) => NULL ()
  real(dp)    , pointer                              :: real_value
  integer(dpi), private  :: expressions_current_nb
  integer(dpi),parameter :: operand_nb_max = needs_max_nb
  integer(dpi)           :: create_index = NOT_INIT_I
  integer(dpi), pointer  :: expressions_word_pos (:) => NULL ()  

  integer(dpi) :: needs_nb
  integer(dpi) :: needs (operand_nb_max)
  contains

! ==========================================================================
  subroutine expressions_bld
! --------------------------------------------------------------------------
!
! Name           : expressions_bld
!
! Description    : builds expressions defined by 
! Description    : name = type (arithmetic expression on objects)
! Description    : name defines a new object name
! Description    : type is real, bool or inte
! Description    : expressions_first_word (expr) index in input_words of first (
! Description    : expressions_last_word (expr)  index in input_words of last  )
!
! Creates        : expressions_name       (expr)
! Creates        : expressions_type       (expr)
! Creates        : expressions_first_word (expr)
! Creates        : expressions_last_word  (expr)
! Creates        : expressions_current_nb number of current expression in input
!
! Needs          : input_words        (wrd)
! Needs          : input_words_semans (wrd)
!
! Authors:F. Colonna
! Date  :06 Dec 2002
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: expr_i, expr_nb 
  integer(dpi) :: wrd_i, wrd_nb 

! header:

  if (header_exe) then
        
    l_here = "expressions_bld"
    description = 'expressions name are objects defined ' &
              *CR*' as object = real (expression)'    &
              *CR*' as object = bool (expression)'    &
              *CR*' as object = inte (expression)'

    call object_create ('expressions_name')

    description = 'expressions type are real, bool or inte'
    call object_create ('expressions_type')

    description = 'expressions_first_word (expr) index in input_words of first ('
    call object_create ('expressions_first_word')

    description = 'expressions_last_word (expr) index in input_words of last )'
    call object_create ('expressions_last_word')

    call object_create('expressions_word_pos')    
    if (.not.associated(input_words)) call input_words_bld
    
    if (.not.associated(input_words_semans)) call input_words_semans_bld
    
!    call check_association ('input_words', input_words, &
!                            'input_words_bld', input_words_bld)

!    call check_association ('input_words_semans', input_words_semans, &
!                            'input_words_semans_bld', input_words_semans_bld)

    ! call object_needed ('input_words')
    ! call object_needed ('input_words_semans')
 
    return

  end if

! begin:
  call enter (l_here)

  expressions_current_nb = 0

  wrd_nb = size(input_words_semans)
  include 'chksiz_input_words_semans.inc'

! count nb of expressions in input_words_semans.
  expr_nb = string_head_in_array_count ('expression', input_words_semans)

  if(expr_nb == 0) then
   call die (l_here,' no expressions')
  else 

  call alloc ('expressions_name', expressions_name, expr_nb)
  call alloc ('expressions_type', expressions_type, expr_nb)
  call alloc ('expressions_first_word', expressions_first_word, expr_nb)
  call alloc ('expressions_last_word', expressions_last_word, expr_nb)
  call alloc ('expressions_word_pos',expressions_word_pos, expr_nb)
! store object names. and types.
  expr_i = 0
  do wrd_i =  1, wrd_nb
    if(input_words_semans(wrd_i) == "expression") then
      expr_i = expr_i + 1
      expressions_word_pos (expr_i) = wrd_i 
      expressions_name (expr_i) = input_words (wrd_i)
      expressions_type (expr_i) = input_words (wrd_i + 2)
    end if
  end do

  if(expr_nb /= expr_i) then
    message = 'in routine'*l_here*' nb of expressions found ='*expr_i &
              *' while string_head_in_array_count found '*expr_nb
    call ensure (message, expr_nb == expr_i)
  end if

  expressions_first_word = NOT_INIT_I
  expressions_last_word  = NOT_INIT_I

  if(debug) then
    do expr_i =  1, expr_nb
      write (logf,*)trim(l_here),'-d: expression # ',expr_i,' >', &
              trim(expressions_name (expr_i)),'< type >',   &
              trim(expressions_type (expr_i)),'<'
    end do
  end if

  end if ! no expression

  call exit (l_here)

  end subroutine expressions_bld

! ==========================================================================
  subroutine expressions_print
! --------------------------------------------------------------------------
!
! Name           : expressions_print
!
! Description    : builds expressions defined by lists of expressions 
!
! Needs          : expressions (expr_nb)
!
! Authors:F. Colonna
! Date  :06 Dec 2002
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: expr_nb
  integer(dpi) :: expr_i

! header:
  if (header_exe) then
    l_here = "expressions_print"
    call object_needed ('expressions_name')
    call object_needed ('expressions_type')
    call object_needed ('expressions_first_word')
    call object_needed ('expressions_last_word')
    return
  end if

! begin:
  call enter (l_here)

  expr_nb = size (expressions_name, 1)
  if(expr_nb <= 0) then
    message = 'in routine'*l_here*' nb of expressions = '*expr_nb 
    call ensure (message, expr_nb > 0)
  end if

  write (logf,*)trim(l_here),': objects as expressions'
  do expr_i =  1, expr_nb
    write (logf,*)trim(l_here),'-d: expression # ',expr_i,' >', &
            trim(expressions_name (expr_i)),'< type >',   &
            trim(expressions_type (expr_i)),'< starts at word # ',   &
            expressions_first_word (expr_i), &
            ' ends at word # ', expressions_last_word(expr_i)
  end do

  call exit (l_here)

  end subroutine expressions_print

! ==========================================================================
  subroutine next_expression_get
! --------------------------------------------------------------------------
!
! Name           : next_expression_get
!
! Description    : get next expression from menu objects_menu
! Description    : if input_words_semans (cur_word_pos) = "expression"
! Description    : expression = operator (.................)  
!
! Needs          : input_words (wrd) 
! Needs          : input_words_semans (wrd) 
!
! Fills          : expressions_first_word 
! Fills          : expressions_last_word 
!
! Authors:F. Colonna
! Date  :05 Dec 2002
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_object_name)       :: name 
  character(len=max_string_len_expr_type)         :: type 
  character(len=max_string_len)                   :: expression_string
  integer(dpi) :: wrd_i, i, obj_i, obj_j
! header:
  if (header_exe) then
    l_here = "next_expression_get"
    call object_needed ('expressions_name')
    call object_needed ('expressions_type')
    call object_needed ('expressions_first_word')
    call object_needed ('expressions_last_word')
    call object_needed ('expressions_word_pos')
    return
  end if

! begin:
  call enter (l_here)

  name = input_words (cur_word_pos)
  type = input_words_semans (cur_word_pos +2)
  do i =1, size (expressions_word_pos)
     if (expressions_word_pos (i) == cur_word_pos) expressions_current_nb = i
  enddo

  if(expressions_name (expressions_current_nb) /= name) then
    message = 'in routine'*l_here*' expression "'+name+'"' &
    *' is the '+expressions_current_nb+' in menu '   &
    *' while the '+expressions_current_nb+ &
    '-th expression in array expressions_name is "'+name+'"' 
    call require (message, expressions_name (expressions_current_nb) == name)
  end if

  if(input_words_semans (cur_word_pos) /= "expression") then
    message = 'in routine'*l_here*' input_words_semans ('+cur_word_pos+')=' &
    +input_words_semans (cur_word_pos)*' should be "expression"'
    call require (message, input_words_semans (cur_word_pos) == "expression")
  end if

  if(expressions_type (expressions_current_nb) /= type) then
    message = 'in routine'*l_here*' expression "'+name+'"' &
    +' of type "'+expressions_type (expressions_current_nb)+'" has type "'+type+' in menu'
    call require (message, expressions_type (expressions_current_nb) == type)
  end if

  if(debug) then
    write (logf,*)trim(l_here), '-d: name >',trim(name),'<'
    write (logf,*)trim(l_here), '-d: type >',trim(type),'<'
  end if

  call cursor_advance
  
  call cursor_advance ! =
  call cursor_advance
  
! scan expression to find last ")"
  do while (input_words_semans (cur_word_pos)(1:5) == 'expr ')

     if(debug) then
        write (logf,*)trim(l_here), '-d: input_words_semans (',cur_word_pos,') = >', &
             trim(input_words_semans (cur_word_pos)),'<'
     end if

      if      (input_words_semans (cur_word_pos) == 'expr 1 l_parenthesis') then
        expressions_first_word (expressions_current_nb) = cur_word_pos
      else if (input_words_semans (cur_word_pos) == 'expr 1 r_parenthesis') then
        expressions_last_word (expressions_current_nb) = cur_word_pos
      end if

      call cursor_advance

   end do ! while on cursor  : cur_word_pos

! cursor one step behind
   call cursor_back

  if(debug) then
    write (logf,*)trim(l_here),'-d: expressions_first_word =', &
                          expressions_first_word (expressions_current_nb)
    write (logf,*)trim(l_here),'-d: expressions_last_word  =', &
                          expressions_last_word (expressions_current_nb)
    write (logf,*)trim(l_here),'-d: object name >',trim(name),'<'
    write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
    write (logf,*)trim(l_here),'-d: cur_word_pos =',cur_word_pos
  end if
 
! builds the nodes from expression 
  expression_string = BLANKS

  do wrd_i =expressions_first_word (expressions_current_nb)+1, expressions_last_word (expressions_current_nb)-1
 
     expression_string = expression_string*' '*input_words (wrd_i)

  enddo ! do wrd_i

 
  if (type == "val") then
     obj_i = object_index (expressions_name (expressions_current_nb))
     if (objects_addr (obj_i) == 0) call provide_byind (obj_i)

     obj_i = object_from_expression (expression_string)
        
     call provide_byind (obj_i)
     call equal_set_bld (object_index (expressions_name(expressions_current_nb)), &
          obj_i)
 
  else if (type == "def") then
   
     obj_i = string_in_array_locate (expressions_name (expressions_current_nb), objects_names)

     if (obj_i /=0) then
        write (logf,*) trim(l_here)," object >"+expressions_name (expressions_current_nb)+"< already defined, cannot be redefined"
        call die (l_here, "suggestion, use val ( ) instead of def ( )") 
     endif
     
     obj_i = object_from_expression (expression_string)
   
     call provide_byind (obj_i)
     call objects_name_add_once (expressions_name (expressions_current_nb))
     obj_j = object_index(expressions_name (expressions_current_nb))
     
     objects_type(obj_j) = objects_type (obj_i)
     objects_addr(obj_j) = objects_addr (obj_i)

     objects_addr (obj_i) = 0
     
     if (objects_node (obj_i) ==0 .or. objects_node (obj_i) == NOT_INIT_I) then

        call die (l_here, "object >"+objects_names(obj_i)+"<should have a creation node")
     endif
     
     is_valid_bynod (objects_node (obj_i)) = .false.   
 
  else
     call expression_decode (expressions_name (expressions_current_nb), expression_string)
     
  endif

  call exit (l_here)

end subroutine next_expression_get

! ==========================================================================
   subroutine  parenthesis_words_delete (expression_words, first, last)
! --------------------------------------------------------------------------
!
! Name       : parenthesis_words_delete
!
! Argument (in-out) :: expression_words
! Argument (in)     :: first
! Argument (in)     :: last
!
! Description    : (expr) -> expr (first=first+1, last = last-1)
!
! Authors: R. Assaraf
! Date  :08 Dec 2002
! --------------------------------------------------------------------------
  implicit none 
! i/o  : 
  integer(dpi)     :: first, last
  character(len=*) :: expression_words (:)

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) ::  cursor, level
! header:
  call header_die
  
! begin:
  l_here = 'parenthesis_words_delete'
  call enter(l_here)

  level = 0
  do  cursor = first, last
     if (expression_words (cursor) == '(') level = level+1 
     if (expression_words (cursor) == ')') level = level-1
  enddo ! do cursor
  
  if (level /= 0) then 
   call expr_nam_bld (expression_words, message,first, last)
   message='in expression: '+message+' non matching parenthesis'
   call die (l_here, message)
  endif

  do while (expression_words (first)=='('.and.expression_words (last)==')')
     level = 0
     do  cursor = first, last
        if (expression_words (cursor) == '(') level = level+1 
        if (expression_words (cursor) == ')') level = level-1
! cursor put to 0 <=> current word is ')', closes the first parenthesis
        if (level == 0.and.cursor<last)  then
         call exit (l_here)
         return
        endif

     enddo ! do cursor
     first = first+1
     last = last-1
  enddo ! do while

  call exit(l_here)

  end subroutine parenthesis_words_delete

! ==========================================================================
   subroutine  expr_nam_bld (expression_words, expr_nam, first, last)
! --------------------------------------------------------------------------
!
! Name       : expr_nam_bld
!
! Argument (in)   : expression_words
! Argument (in)   : first
! Argument (in)   : last
!
! Description    : builds expr_name as the concatenation of the
! Description    : words between first and last in expression_words
!
! Needs          : expression_words (wrd) 
!
! Authors: R. Assaraf
! Date   : 12 Dec 2002
! --------------------------------------------------------------------------
  implicit none 
! i/o  : 
  character(len=*) :: expression_words (:) 
  character(len=*) :: expr_nam
  integer(dpi)      :: first, last
 
! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) ::  cursor
! header:
!  call header_die
  
! begin:
  l_here = 'expr_nam_bld'
  call enter(l_here)

  expr_nam='' 
  do cursor = first, last
   expr_nam = expr_nam+expression_words(cursor)
  enddo ! do cursor

  call exit(l_here)

  end subroutine expr_nam_bld

! ==========================================================================
  subroutine expression_decode (expr_name, expr)
! --------------------------------------------------------------------------
!
! Name           : expression_decode
!
! Argument  (in/out) : expr_name
! Argument  (in)     : expr
!
! Description    : builds the path too build the object 'expr_name'
! Description    : defined from the expression expr
! Description    : the words in expression_string must be 
! Description    : separated by blanks : to be improved
!
! Authors: R. Assaraf
! Date  :16  Fev 2002
! --------------------------------------------------------------------------
  implicit none 
! i/o  : 

  character(len=*) :: expr_name
  character(len=*) :: expr

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: create 
! header:
  
! begin:
  l_here = 'expression_decode'

  call enter(l_here)
  create  = object_from_expression (expr)

  objects_names (create) =expr_name
  objects_desc (create) = expr

  call exit(l_here)

 end subroutine expression_decode

! ==========================================================================
  recursive  function object_from_expression (expr)   result (index) 
! --------------------------------------------------------------------------
!
! Name    : object_from_expression
!
! Argument (in) : expr
!
! Description   : gives the index of the object described by the expression "expr"
! Descripttion  : If no object is described by expr, define it.
! Description   : Its name, objects_names (index) is automatically built
! Description   : This function may be called in a header 
! Description   : in this case global variables needs, needs_nb, create_index are used
! Description   : (see one example)
!
! Improve   : These three global variables harms the readibility of the code.
!
! Authors: R. Assaraf
! Date   : 16  Fev 2002
! Revision: 26 Dec 2003, R. A., "xxx" -> xxx is a string 
! --------------------------------------------------------------------------
  implicit none 

! result
  integer(dpi) :: index

! i/o  : 

  character(len=*) :: expr

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: operand_nb 
  integer(dpi) :: operand_f(operand_nb_max)
  integer(dpi) :: operand_l (operand_nb_max)
  character(len=max_string_len_object_name) :: operator_name
  character(len=max_string_len_object_name) :: expr_name
  integer(dpi) :: node_i, rtn_i
  integer(dpi) :: i
  integer(dpi) :: f, l
  real(dp), pointer :: real_value
  integer(dpi), pointer :: integer_value
  character(len=max_string_len), pointer :: string_value => NULL ()
  integer(dpi), save :: sv = 0
  integer(dpi) :: create_index_sav
  integer(dpi) :: needs_nb_sav
  integer(dpi) :: needs_sav (operand_nb_max) 
! header:
  
! begin:
  l_here = 'object_from_expression'

! This function can be called from a header
! but has not to call headers
  header_exe = .false.
 
  call enter (l_here)

! global variables this function modifies

  create_index_sav = create_index
  needs_sav = needs 
  needs_nb_sav = needs_nb
  
! inside the parenthesis
  call in_parenthesis_bounds_bld (f, l, expr)

  if (debug) write (logf,*) trim(l_here), '-d: expr >',trim(expr),'<'
  if (debug) write (logf,*) trim(l_here), '-d: expr without uneseful () >',trim(expr(f:l)),'<'


  if (words_number (expr(f:l)) == 1) then
! expr contains one word
     index = string_in_array_locate ( adjustl(trim(expr(f:l))), &
          objects_names (1:objects_nb))
 
     if (index == 0) then
        if (integer_is (trim(adjustl( (expr(f:l)))))) then
           allocate(integer_value)
           call catalog (adjustl(expr(f:l)), integer_value)
           call decode (integer_value, expr(f:l))
           index = object_index (adjustl(trim(expr(f:l))))
        else if (numeric_is(trim (expr(f:l)))) then
           allocate(real_value)
           call catalog (adjustl(expr(f:l)), real_value)
           call decode (real_value, expr(f:l))
           index = object_index (adjustl(trim(expr(f:l))))
        else 
           call die (l_here, 'object >'+expr(f:l)+'< does not exist')
        endif
     endif

 
     if (debug) write (logf,*) trim(l_here), '-d: expr(f:l) >',trim(expr(f:l)),'< index=',index
  
  else   if (expr(f:f)=='"'.and.expr(l:l)=='"') then
! expr is a string
     allocate (string_value)
     string_value=expr(f+1:l-1)
     sv=sv+1
     call catalog ("string_value_"+sv,string_value)
     index = object_index ("string_value_"+sv)
 
  else
! expr contains more than one word
! expr is an operator operator_name applied on the two following operands :
! expr (operands_f(1):operands_l(1)) and expr (operands_f(2): operands_l(2))

     operator_name = NOT_INIT_S
     operand_f = NOT_INIT_I
     operand_l = NOT_INIT_I
     operand_nb = 0
  

     if (is_array_element (expr (f:l), operand_f, operand_l, operand_nb)) then
        operator_name = 'array_element_calc'
        
     else if (is_binary_operation (expr(f:l), '+',operand_f(1:2), operand_l (1:2))) then
        operand_nb = 2 
        operator_name = 'sum' 
 
     else if (is_binary_operation (expr(f:l), '-',operand_f(1:2), operand_l (1:2))) then
        operand_nb = 2 
        operator_name = 'dif' 
 
     else if (is_binary_operation (expr(f:l), '*',operand_f(1:2), operand_l (1:2))) then
        operand_nb = 2 
        operator_name = 'prod' 
  
     else if  (is_binary_operation (expr(f:l), '/',operand_f(1:2), operand_l (1:2))) then
        operand_nb = 2 
        operator_name = 'div' 

     else if (is_binary_operation (expr(f:l), '^',operand_f(1:2), operand_l (1:2))) then
        operand_nb = 2
        operator_name = 'expo'

     else if (is_binary_operation (expr (f:l), ':',operand_f(1:2), operand_l (1:2)))  then
        operand_nb = 2
        operator_name = 'clone'
  
     else if (is_generic_operation (expr(f:l), operator_name, operand_f, operand_l, operand_nb)) then 

     else if (is_unary_operation (expr(f:l), '-', operand_f(1:1), operand_l(1:1))) then
        operand_nb = 1
        operator_name = 'minus_calc'
   
     else
          
        call die (l_here, 'syntax error in expression >'+trim(expr)+'<')
        
     endif

! positions of operands with respect to expr instead of expr(f:l) 
     do i= 1, operand_nb
        operand_f(i)=operand_f(i)  +f-1
        operand_l(i)=operand_l(i)  +f-1
 
        if (operand_f(i) <=0 .or. operand_l(i) > len(trim(expr))) then
           write (logf,*) trim(l_here),' operand_f:', operand_f (i)
           write (logf,*) trim(l_here),' operand_l:', operand_l (i)
           write (logf,*) trim(l_here),' subscripts of expr >'+expr+'<'
           call die (l_here, 'operands subscripts out of range')
        endif
     enddo
     
     if (debug) then
        write (logf,*) trim(l_here),'-d: expr >',trim(expr),' : ' &
             ,trim(operator_name)//'( ', &
             (trim(expr(operand_f(i): operand_l(i)))//', ', i= 1,operand_nb), ' )'
     endif

! needs from operands
     needs_nb = operand_nb
     if (needs_nb > needs_max_nb) call die (l_here, &
          'needs_max_nb='+needs_max_nb&
          +'< needs_nb='+needs_nb)
 
     needs = NOT_INIT_I
     do i= 1, operand_nb

        needs(i) = object_from_expression (expr (operand_f(i):operand_l(i)))

        if (debug) write (logf, *) trim(l_here),'-d: expr >',trim(expr),&
             '< need(',i,')=',needs(i),' >', trim(objects_names (needs(i))),'<'
     enddo ! do i     

! create name
     expr_name = adjustl(trim(expr(f:l))) 

     rtn_i = string_in_array_locate (operator_name, routines_name)
     
     if (rtn_i == 0) call die (l_here, 'operator >'+operator_name+'< not defined, error '&
          *'in expression'+expr)
     
!---------------------
! creation of the node
!---------------------

     
! node built in the header ?
     create_index = 0
     call one_routine_header_exe (rtn_i)
     index = create_index
! index = 0:  node not built in the header
 
     if (index ==0) then
     
        node_i = node_from_operation (operator_name, objects_names(needs(1:operand_nb)) )
    
        if (node_i ==0 ) then
  
           call node_create (operator_name,(/expr_name/), objects_names(needs(1:operand_nb)) )
           
           node_i = node_from_operation (operator_name, objects_names(needs(1:operand_nb)) )
           
        endif
        
        if (debug) write (logf,*) trim(l_here),'-d: object found at node # ', node_i
        if (debug) call one_node_write ( node_i, logf, l_here)

        if (node_i <= 0 .or. node_i > nodes_nb) call die (l_here,'node_i='+node_i)
        if (nodes_create%first(node_i) < 0)  call die (l_here,'nodes_create%first('+node_i+')='+&
             nodes_create%first(node_i))
        
        index = nodes_create%val (nodes_create%first(node_i))
     endif

  endif

  create_index = create_index_sav
  needs = needs_sav
  needs_nb = needs_nb_sav
  
  call exit(l_here)

end function object_from_expression

! ==========================================================================
  function  is_binary_operation (string, op, operand_f, operand_l) result (result)
! --------------------------------------------------------------------------
!
! Name       : is_binary_operation
!
! Argument (in)     :: string
! Argument (in)     :: op
! Argument (out)    :: operand_f, operand_l
! 
! Description  :: check if the string is a binary operation like:  sub_expr1 op sub_expr2
! Description  :: operand_f and operand_l are such that 
! Description  :: string (operand_f (1):operand_l (1)) = sub_expr1
! Description  :: string (operand_f (2):operand_l (2)) = sub_expr2
!
! Authors: R. Assaraf
! Date   : 12 Fev 2003
! --------------------------------------------------------------------------
  implicit none 
! i/o  : 
  logical(bool)    :: result
  character(len=*) :: string
  character(len=*) :: op
  integer(dpi)     :: operand_f (:)
  integer(dpi)     :: operand_l (:)

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: f, l
  integer(dpi) :: level, cur_word
! header:
  call header_die
  
! begin:
  l_here = 'is_binary_operation'
  call enter(l_here, 3)

  if (size(operand_f) /= 2) call die (l_here, 'size(need_f)='+size(operand_f)+'/= 2')
  if (size(operand_l) /= 2) call die (l_here, 'size(need_f)='+size(operand_l)+'/= 2')
  result = .false.
  f=0
  l=0
  level=0
  cur_word = 0
  do while (l /= len(trim(string)))
     call next_word_bounds_bld (f,l, string)
     cur_word=cur_word + 1
     if (string (f:l) == '(') level = level+1
     if (string (f:l) == ')') level = level-1
     if (string(f:l) == trim(op) .and. level ==0.and.cur_word /= 1) then
         operand_f(1)= 1
         operand_l(1)=f-1
         operand_f(2)=l+1
         operand_l(2)=len(trim(string))
         result=.true.
      endif
   enddo ! do while
  
  call exit(l_here, 3)

 end function is_binary_operation

! ==========================================================================
  function  is_unary_operation (string, op, operand_f, operand_l) result (result)
! --------------------------------------------------------------------------
!
! Name       : is_unary_operation
!
! Argument (in)     :: string
! Argument (in)     :: op
! Argument (out)    :: operand_f, operand_l
! 
! Description  :: check if the string is a unary operation like:  op sub_expr
! Description  :: operand_f and operand_l are such that 
! Description  :: string (operand_f (1):operand_l (1)) = sub_expr
! Improve :: control of string is - subexpr not finished 
! Authors: R. Assaraf
! Date   : 18 Fev 2003
! --------------------------------------------------------------------------
  implicit none 
! i/o  : 
  logical(bool)    :: result
  character(len=*) :: string
  character(len=*) :: op
  integer(dpi)     :: operand_f (:)
  integer(dpi)     :: operand_l (:)

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: f, l
  integer(dpi) :: level, cur_word
! header:
  call header_die
  
! begin:
  l_here = 'is_unary_operation'
  call enter(l_here, 3)

  if (size(operand_f) /= 1) call die (l_here, 'size(need_f)='+size(operand_f)+'/= 1')
  if (size(operand_l) /= 1) call die (l_here, 'size(need_f)='+size(operand_l)+'/= 1')
  result = .false.
  f=0
  l=0
  level=0
  cur_word = 0

  do while (l /= len(trim(string)))
     call next_word_bounds_bld (f,l, string)
     cur_word=cur_word+  1
     if (string (f:l) == '(') level = level+1
     if (string (f:l) == ')') level = level-1
 
     if (string(f:l)  == trim(op) .and. cur_word == 1) then
        operand_f(1) =  l+1
        operand_l(1) =  len(trim(string))
        result = .true.
     endif
     
   enddo ! do while
  
  call exit(l_here, 3)
 
end function is_unary_operation

! ======================================================================================
  function  is_generic_operation (string, op,  operand_f, operand_l,   operand_nb) result (result)
! --------------------------------------------------------------------------------------
!
! Name       : is_generic_operation
!
! Argument (in)     :: string
! Argument (out)    :: op
! Argument (out)    :: operand_f (wrd)
! Argument (out)    :: operand_l (wrd)
! Argument (out)    :: operand_nb
! 
! Description  :: check if the string is on the form op (sub_expr1, sub_expr2, ...) 
! Description  :: operand_f and operand_l are such that 
! Description  :: string (operand_f (1):operand_l (1)) = sub_expr1
! Description  :: string (operand_f (2):operand_l (2)) = sub_expr2
! Description  :: ...
!
! Authors: R. Assaraf
! Date   : 12 Fev 2003
! --------------------------------------------------------------------------
  implicit none

! result:
  logical(bool) :: result
 
! i/o  : 
  character(len=*) :: string
  character(len=*) :: op
  integer(dpi)     :: operand_f (:)
  integer(dpi)     :: operand_l (:)
  integer(dpi)     :: operand_nb

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: f, l
  integer(dpi) :: level, cur_word

! header:
  call header_die
  
! begin:
  l_here = 'is_generic_operation'
  call enter(l_here, 3)
 
  if (debug) then 
   write (logf,*) trim(l_here), '-d: string >',trim(string),'<'
  endif
 
  result = .false.
  f=0
  l=0
  level=0
  cur_word = 0
  operand_nb = 0

  do while (l /= len(trim(string)))

     if (debug) write (logf,*) trim(l_here), '-d:f  l, len(trim(string))',f,  l, len(trim(string)) 

     call next_word_bounds_bld (f,l, string)
     cur_word=cur_word + 1


     if (string (f:l) == '(') level = level+1
     if (string (f:l) == ')') level = level-1
  
  if (debug) write(logf,*) trim(l_here), '-d: level cur_word word: ', level, cur_word, string(f:l) 

! if only one word, no operator
     if (cur_word == 1 .and. l == len(trim(string))) then
       result = .false.
       call exit (l_here)
       return
     endif

! first word, the operator name
     if (cur_word == 1) then
        if (string(f:l) == '('.or.string(f:l) == ')') then
           result = .false.
           call exit (l_here)
           return
        else
           op=string(f:l)
        endif
     endif

! second word must be  '(' for result to be true
     if (cur_word== 2 .and. string(f:l) /= '(') then
        result = .false.
        call exit(l_here)
        return
     endif

     if ( cur_word == 3) then 
        operand_f(1) = f
        operand_nb = 1
     endif

! number of commas 
     if (level== 1 .and. string(f:l) == ',') then 
        operand_nb=operand_nb+1
        operand_l(operand_nb-1) = f-1
        operand_f(operand_nb) = l+1
     endif

! The closing parenthesis is not at the end of the string
! => result = false
     if (level==0 .and. l==len(trim(string))) then 
        if (string(f:l)/=')') call die (l_here, 'current string >'+string(f:l)+&
             '< should be >)< since level has been set to 0')
        operand_l (operand_nb)= l-1
        result = .true.
        call exit(l_here)
        return
     endif
  enddo ! do while
 
 call exit(l_here, 3)
 
 end function is_generic_operation

! ======================================================================================
  function  is_array_element (string, operand_f, operand_l,   operand_nb) result (result)
! --------------------------------------------------------------------------------------
!
! Name       : is_array_element
!
! Argument (in)  : string
! Argument (out) : operand_f (wrd)
! Argument (out) : operand_l (wrd)
! Argument (out) : operand_nb
! 
! Description  : check if the string is on the form array_name [sub_expr1, sub_expr2, ...]
! Description  : operand_f and operand_l are such that 
! Description  : string (operand_f (1):operand_l (1)) = sub_expr1
! Description  : string (operand_f (2):operand_l (2)) = sub_expr2
! Description  : ...
! Description  : nearly identical to is_generic_operation
! 
! Authors: R. Assaraf
! Date   : 26  Fev 2003
! --------------------------------------------------------------------------
  implicit none

! result:
  logical(bool) :: result
 
! i/o  : 
  character(len=*) :: string
  integer(dpi)     :: operand_f (:)
  integer(dpi)     :: operand_l (:)
  integer(dpi)     :: operand_nb

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: f, l
  integer(dpi) :: level, cur_word

! header:
  call header_die
  
! begin:
  l_here = 'is_array_element'
  call enter(l_here, 3)
 
  if (debug) then 
   write (logf,*) trim(l_here), '-d: string >',trim(string),'<'
  endif
 
  result = .false.
  f=0
  l=0
  level=0
  cur_word = 0
  operand_nb = 0

  do while (l /= len(trim(string)))

     if (debug) write (logf,*) trim(l_here), '-d:f  l, len(trim(string))',f,  l, len(trim(string)) 

     call next_word_bounds_bld (f,l, string)
     cur_word=cur_word + 1


     if (string (f:l) == '(') level = level+1
     if (string (f:l) == ')') level = level-1
  
  if (debug) write(logf,*) trim(l_here), '-d: level cur_word word: ', level, cur_word, string(f:l) 
  
! if only one word, no array
     if (cur_word == 1 .and. l == len(trim(string))) then
       result = .false.
       call exit (l_here)
       return
     endif

! first word, the array name
     if (cur_word == 1) then
        if (string(f:l) == '('.or.string(f:l) == ')') then
           result = .false.
           call exit (l_here)
           return
        else
           operand_nb= operand_nb+1
           operand_l(operand_nb) = l
           operand_f(operand_nb) = f
        endif
     endif

! second word must be  '[' for result to be true
     if (cur_word== 2 .and. string(f:l) /= '[') then
        result = .false.
        call exit(l_here)
        return
     endif

     if ( cur_word == 3) then 
       operand_nb = operand_nb+1 
       operand_f(operand_nb) = f
      endif

! number of commas 
     if (level==0 .and. string(f:l) == ',') then 
        operand_nb=operand_nb+1
        operand_l(operand_nb-1) = f-1
        operand_f(operand_nb) = l+1
     endif

! The closing parenthesis is not at the end of the string
! => result = false
     if (string(f:l)==']'.and. l==len(trim(string))) then 
        operand_l (operand_nb)= l-1
        result = .true.
        call exit(l_here)
        return
     endif
  enddo ! do while
 
 call exit(l_here, 3)
 
 end function is_array_element

! ==========================================================================
   subroutine  in_parenthesis_bounds_bld (first, last, expr)
! --------------------------------------------------------------------------
!
! Name       : in_parenthesis_bounds_bld 
!
! Argument (in) :: expr
! Argument (out) :: first
! Argument (out) :: last
!
! Description    : expr = (subexpr) -> subexpr=expr (first=first+1, last = last-1)
!
! Authors: R. Assaraf
! Date  :16 Fev 2002
! --------------------------------------------------------------------------
  implicit none 

! i/o  : 
  integer(dpi)     ::  first, last
  character(len=*) ::  expr  

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: wrd_l, wrd_r, cur_wrd
  integer(dpi) :: level
! header:
  call header_die
  
! begin:
  l_here = 'in_parenthesis_bounds_bld'
  call enter(l_here, 3)

  first = 1
  last  = len(trim(expr))
    
  cur_wrd = 0
  wrd_l = 0
  wrd_r = 0
  level = 0

  do while (wrd_r < len(trim(expr)))
     call   next_word_bounds_bld (wrd_l, wrd_r, expr)
     cur_wrd = cur_wrd +1
     if (expr(wrd_l:wrd_r) == '(') level=level+1
     if (expr(wrd_l:wrd_r) == ')') level=level-1


     if (cur_wrd == 1) then
        if (level == 1) then
           first = wrd_l
        else
! if first word is not a parenthesis exit
           call exit(l_here)
           return
        endif
     endif
! if closing parenthesis not at the end exit
     if (level ==0 .and. wrd_r <len(trim(expr))) then 
        call exit (l_here)
        return
     endif
  enddo ! do while

  if (level/=0) call die (l_here, 'non matching parenthesis in expression >'+expr+'<')   

  last=wrd_l-1
  first=first+1

  call exit(l_here, 3)

  end subroutine in_parenthesis_bounds_bld

! ======================================================================================
  subroutine  word_bounds_bynum_bld (wrd_l, wrd_r, level, wrd_num, string) 
! --------------------------------------------------------------------------------------
!
! Name       : word_bounds_bynum_bld
!
! Argument (out) : wrd_l 
! Argument (out) : wrd_r
! Argument (out) : level 
! Argument (in)  : wrd_num
! Argument (in)  : string
!
! Description  : gives the boundaries wrd_l, wrd_r  of the word # wrd_num in string
! Description  : level is the level of parenthesis of the word # wrd_num
!
! Authors: R. Assaraf
! Date   : 16 Fev 2003
! --------------------------------------------------------------------------
   implicit none 
! i/o  : 
   integer(dpi)     :: wrd_l, wrd_r
   integer(dpi)     :: wrd_num
   integer(dpi)     :: level
   character(len=*) :: string
 
! local:
   character(len=max_string_len_routine_name),save :: l_here
   integer(dpi) :: cur_wrd

! header:
   call header_die
  
! begin:
  l_here = 'word_bounds_bynum_bld'
  call enter(l_here, 3)
  if (wrd_num <= 0 .or. wrd_num > words_number (string)) call die (l_here, &
      'wrd_num ='+wrd_num+' should be a valid word number in string >'+string+'<')
 
  level = 0
  cur_wrd = 0
  do while (cur_wrd < wrd_num)
     call next_word_bounds_bld (wrd_l, wrd_r, string)
     if (string (wrd_l:wrd_r) == '(') level = level +1
     if (string (wrd_l:wrd_r) == ')') level = level -1
     cur_wrd = cur_wrd +1    
  enddo ! do while

  if (cur_wrd /= wrd_num) call die (l_here, 'cur_wrd ='+cur_wrd+'/= wrd_num='+wrd_num)
  
  call exit(l_here, 3)
 
 end subroutine word_bounds_bynum_bld

! ======================================================================================
 function  words_number (string) result (result)
! --------------------------------------------------------------------------------------
!
! Name       : words_number
!
! Argument (in):: string
! 
! Description  :: gives the number of words in the expression string
!
! Authors: R. Assaraf
! Date   : 13 Fev 2003
! --------------------------------------------------------------------------
   implicit none 
! i/o  : 
   integer(dpi)     :: result
   character(len=*) :: string
 
! local:
   character(len=max_string_len_routine_name),save :: l_here
   integer(dpi) :: f, l
! header:
   call header_die
  
! begin:
  l_here = 'words_number'
  call enter(l_here, 3)


  result = 0
  f=0
  l=0
 
  do while (l /= len(trim(string)))
     call next_word_bounds_bld (f,l, string)
     result = result +1 
  enddo ! do while
  
  call exit(l_here, 3)
 
 end function words_number

! ==========================================================================
  subroutine  next_word_bounds_bld (first, last, string)
! --------------------------------------------------------------------------
!
! Name       : next_word_bld
!
! Argument (in-out) :: first
! Argument (in)     :: last
! Argument (in)     :: string
!
! Description : With the convention that a word is any substring 
! Description : separated by blanks: 
! Description : On entry -> string(first:last) is the current word
! Description : On exit  -> string(first:last) is  next word.
! Description : last = first= 0  entering string
! Description : last= len(trim(string)) last word
!
! Authors: R. Assaraf
! Date   : 12 Fev 2003
! --------------------------------------------------------------------------
  implicit none 
! i/o  : 
  integer(dpi)     :: first, last
  character(len=*) :: string

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool) :: is_separator
  character(len= 1):: cc, nc, lc
! header:
  call header_die
  
! begin:
  l_here = 'next_word_bounds_bld'
  call enter(l_here, 3)
 
  if (first > last) call die (l_here, 'first='+first+'should be <= last='+last)

   if (last > len(trim(string))) &
      call die (l_here, 'last='+last+' should be <= len(trim(string))='&
      +(0+len(trim(string))))

  if (first >= 1) then
     if (string(first:first) == ' ') &
          call die (l_here, 'string(first='+first+') should not be a blank')
  endif

  if (last <= len(trim(string)).and. last > 0) then
     if (string(last:last) == ' ') &
          call die (l_here, 'string(last='+last+')  should not be a blank')
  endif
 
   if (last == len(trim(string))) then
!  string (first:last) is the last word of string "string"
     call exit(l_here, 3)
   return
  endif
  
  debug_expr: if (debug) then
     write (logf,*) trim(l_here), '-d: string >', trim(string),'<'
     write (logf,*) trim(l_here), '-d: current word:'
     write (logf,*) trim(l_here), '-d: first last:', first, last
     if (last <= len(trim(string)) .and. first>0 .and. last > 0) then
        write (logf,*) trim(l_here),'-d: current word >',string(first:last),'<'
     endif
  endif debug_expr
 
! the first character of the next word, is the non blank character 
! following the character number "last" in string "string"
  first=last+1 
  do while (string(first:first)==" ".and. first < len(trim(string)))
     first=first+1
  enddo ! do while

  last=first

! determination of "last" such that string(first:last) is the next word
! "last" is such that string(last+1) is a separator
  is_separator=.false.
  
  do while (.not. is_separator)

! "cc" is the current character, "nc" the next caracter , "lc" the last character
     cc=string(last:last)

     if (last<len(trim(string))) then
        nc=string(last+1:last+1)
     else
        nc= " "
     endif

     if (last>2) then
        lc=string(last-1:last-1)
     else
        lc= " "
     endif

     if (cc== " ") call die (l_here,'cc=>'+cc+'< should not be a blank')

! separators
     if ( cc == " " .or. nc == " " .or. &
          cc == "," .or. nc == "," .or. &
          cc == "^" .or. nc == "^" .or. &
          cc == ":" .or. nc == ":" .or. &
          cc == "[" .or. nc == "[" .or. &
          cc == "]" .or. nc == "]" .or. &
          cc == "(" .or. nc == "(" .or. &
          cc == ")" .or. nc == ")" .or. &
          cc == "*" .or. nc == "*" .or. &
          cc == "/" .or. nc == "/" .or. &
          cc == "+" .or. nc == "+" .or. &
          cc == "-" .or. nc == "-" ) is_separator = .true.  

! exception + and - are not words when they are not 
! used as operators: expressions like  "4e+06" or "5.0d-05"

     if (nc == "+" .or. nc == "-" ) then
        if (numeric_is(string(first:first)) .and. & 
             (cc == "d" .or. cc == "e")) &
             is_separator = .false.
     endif

     if (cc == "+" .or. cc == "-" ) then
        if (numeric_is(string(first:first)) .and. & 
             (lc == "d" .or. lc == "e")) &
             is_separator = .false.
     endif
 
   
     last=last+1
  enddo ! do while
 
  last=last-1
  
  debug_next_word:  if (debug) then
     write (logf,*) trim(l_here), '-d: string >', trim(string),'<'
     write (logf,*) trim(l_here), '-d: current word:'
     write (logf,*) trim(l_here), '-d: first last:', first, last
     if (last <= len(trim(string)) .and. first>0 .and. last > 0 ) then
        write (logf,*) trim(l_here), '-d: current word >',string(first:last),'<'
     endif
  endif debug_next_word
  
  call exit(l_here, 3)
 
end subroutine next_word_bounds_bld
    
end module expressions_mod
