!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


! ===============================================================================
  subroutine file_header_read (unit)
! -------------------------------------------------------------------------------
!
! Name           : file_header_read 
! 
! Argument (in ) : unit
!
! Description    : read file headers
! Description    : interprets all system items
!
! Authors: X. Krokidis
! Date   : 16 Jan 2001
! -------------------------------------------------------------------------------
  use tools_mod
  use control_mod
  use system_data_mod 
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: line
  character(len= 24)                          :: item 
  integer(dpi)                               :: n_rec
  integer(dpi)                               :: item_value
  logical(bool)                              :: found

! begin:
  l_here ='file_header_read'
  call enter (l_here)

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

  rewind(unit)

  if(debug)write (logf,*)trim(l_here),'-d: unit=',unit

  line = ' '
  n_rec = 0
  found = .false.
  do while (.not.found) 
    n_rec = n_rec + 1
    read(unit,'(a)') line

    write (logf,*)trim(l_here),': header:', trim(line)

    item = line
    if(debug) write (logf,*)trim(l_here),'-d: item >',item,'<'

    select case (item)

    case ("number_of_dimensions")
      item_value = string_next_integer_get (line, item)
      system%dimensions_nb = item_value
      if(debug) write (logf,*)trim(l_here),'-d: system%dimensions_nb =',system%dimensions_nb

      if(system%dimensions_nb <= 0) call die &
      (l_here, ' nb of dimensions ='+system%dimensions_nb)

    case ("number_alfa_electrons")
      item_value = string_next_integer_get (line, item)
      system%alfa_electrons_nb = item_value
      if(debug) write (logf,*)trim(l_here),'-d: system%alfa_electrons_nb =',system%alfa_electrons_nb

    case ("number_beta_electrons")
      item_value = string_next_integer_get (line, item)
      system%beta_electrons_nb = item_value
      if(debug) write (logf,*)trim(l_here),'-d: system%beta_electrons_nb =',system%beta_electrons_nb

    case ("number_electrons")
      item_value = string_next_integer_get (line, item)
      system%electrons_nb = item_value
      if(debug) write (logf,*)trim(l_here),'-d: system%electrons_nb =',system%electrons_nb

      if(system%electrons_nb <= 0) call die &
      (l_here, ' nb of electrons ='+system%electrons_nb)

    case ("number_of_walkers")
      item_value = string_next_integer_get (line, item)
      if (system%walkers_nb == NOT_INIT_I) system%walkers_nb = item_value
      if(debug) write (logf,*)trim(l_here),'-d: system%walkers_nb =',system%walkers_nb
      
      if(system%walkers_nb <= 0) call die &
      (l_here, ' nb of walkers ='+system%walkers_nb)

    case ("end_of_header")             

      found = trim(line) == 'end_of_header'
      if(debug) write (logf,*)trim(l_here),'-d: found =',found

    end select

  enddo

  if(.not.found) then
    rewind(unit)
    if(warning_level > -1) then
      write (logf,*)trim(l_here),'-w: no header lines file >', &
             unit,'< rewinded '
    end if
  end if 

  call exit (l_here)

  end subroutine file_header_read
