! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module files_mod

  use tools_mod
  use c_routines_mod

! ---------------------------------------------------------------
! Revision: 02 Aug 2002 Roland Assaraf 
! Revision: routine files_header_read put in the file files_header_read.f90
! Revision: no more problem of dependencies, files_header_read can read 
! Revision: whatever you want in all the program
! ---------------------------------------------------------------

  integer(dpi) :: nb_files = 0 

  character(len=max_string_len_file_name)          :: files_names_err
  character(len=max_string_len_file_name)          :: files_names_in
  character(len=max_string_len_file_name)          :: files_names_logf
  character(len=max_string_len_file_name)          :: files_names_out
  character(len=max_string_len_file_name)          :: files_names_res

! Descriptions: file catalogue arrays

! character(len=max_string_len_access),    pointer :: files_accesss(:)   => NULL()
! character(len=max_string_len_recl),      pointer :: files_recls(:)     => NULL()

  character(len=max_string_len_action),    pointer :: files_actions(:)   => NULL()
  character(len=max_string_len_file_name), pointer :: files_names(:)     => NULL()
  character(len=max_string_len_file_type), pointer :: files_types(:)     => NULL()

  character(len=max_string_len_format),    pointer :: files_formats(:)   => NULL()
  character(len=max_string_len_file_name), pointer :: files_paths(:)     => NULL()
  character(len=max_string_len_position),  pointer :: files_positions(:) => NULL()
  character(len=max_string_len_program),   pointer :: files_programs(:)  => NULL()
  character(len=max_string_len_title),     pointer :: files_titles(:)    => NULL()
  character(len=max_string_len_status),    pointer :: files_status(:)    => NULL()
  logical(bool),                           pointer :: files_open_is(:)   => NULL()

! Author: F. Colonna
! Date  : 29 Jan 2001
! Revision :19 Dec 2002 F.Colonna files_names

  contains

! ===============================================================================
  subroutine catalogue_print_by_unit (unit)
! -------------------------------------------------------------------------------
!
! Name           : catalogue_print_by_unit
! Argument (in ) : unit
!
! Description    : prints all catalogue-items for a given unit
!
! Authors:F. Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent(in)                   :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='catalogue_print_by_unit'
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(l_here),'-d: current file unit =',unit
   write (logf,*)trim(l_here),'-d: size(files_types)=',size(files_types)
  end if

  message = 'in routine'*l_here+' unit is 0'
  call require (message, unit /= 0)

  message = 'in routine'*l_here+' nb of files catalogued is zero'
  call require (message, size(files_types) /= 0)

  write (logf,'(2a,i2)') trim(l_here),': current file unit =',unit
  write (logf,'(4a)') trim(l_here),': name    >',trim(files_names(unit)),'<'
  write (logf,'(4a)') trim(l_here),': type    >',trim(files_types(unit)),'<'
  write (logf,'(4a)') trim(l_here),': action  >',trim(files_actions(unit)),'<'
  write (logf,'(4a)') trim(l_here),': format  >',trim(files_formats(unit)),'<'
  write (logf,'(4a)') trim(l_here),': position>',trim(files_positions(unit)),'<'
  write (logf,'(4a)') trim(l_here),': interfaced_program >',trim(files_programs(unit)),'<'
  write (logf,'(4a)') trim(l_here),': title   >',trim(files_titles(unit)),'<'
  write (logf,'(4a)') trim(l_here),': status  >',trim(files_status(unit)),'<'
  write (logf,'(2a,l1)') trim(l_here),': is_opened ',files_open_is(unit)

  call exit (l_here)

  end subroutine catalogue_print_by_unit

! ===============================================================================
  function file_catalogue (f_name,     &
                           f_type,     &
                           f_action,   &
                           f_format,   &
                           f_path,     &
                           f_position, &
                           f_program,  &
                           f_title,    &
                           f_status,   &
                           f_is_open) result(unit)
! -------------------------------------------------------------------------------
!
! Name                   : function file_catalogue
!
! Argument (in)          : f_name 
! Argument (in)          : f_type
! Argument (in)          : f_action
! Argument (in) optional : f_format
! Argument (in) optional : f_path
! Argument (in) optional : f_position
! Argument (in) optional : f_program
! Argument (in) optional : f_title
! Argument (in) optional : f_status
! Argument (in) optional : f_is_open
!
! Preamble   : -- does not OPEN a file -- just catalogue it
! Preamble   : the catalogue is a set of string-arrays keeping 
! Preamble   : file-management information (see upper)
!
! Preamble   : 3 items are necessary to catalogue a file:
! Preamble   : the name   "files_names"   string-array
! Preamble   : the type   "files_types"   string-array
! Preamble   : the action "files_actions" string-array
! Preamble   : the others are optional 
!
! Preamble   : the UNIT (entry in the catalogue) is UNIQUELY defined
! Preamble   : by the "file-type" AND "file-action" 
! Preamble   : therefore knowing the type of a file we get the unit
!
! Description    : this function mainly attributes
! Description    : a (new) UNIT number (catalogue entry) from the file type
! Description    : to be used at (defferred) open as logical fortran unit 
! Description    : 
! Description    : file-type and file-action are concatenated by
! Description    : suffixing the type with "-in", "-out", "-inout" 
! Description    : according to the file-action    :
! Description    :    f_action == "append"    f_type => f_type-inout 
! Description    :    f_action == "delete"    f_type => f_type-del 
! Description    :    f_action == "read"      f_type => f_type-in  
! Description    :    f_action == "readwrite" f_type => f_type-inout 
! Description    :    f_action == "write"     f_type => f_type-out 
! Description    : 

! Description    : f_status defaults are given according to    :
! Description    : 
! Description    :    f_action == "append"    f_status = "old"
! Description    :    f_action == "read"      f_status = "old"
! Description    :    f_action == "readwrite" f_status = "old"
! Description    :    f_action == "write"     f_status = "new"
! Description    : 
!
! Description    : if a file of type "f_type" is already catalogued 
! Description    : all new file items are over-written 
!
! Remark         : f_name may correspond to several f_type (ex: basis and mos)
! Remark         : therefore f_type => unit (without equivalence)
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! Revision: 16 Dec 2000 FCC undefined action case corrected
! Revision: 17 Jan 2001 FCC suffixes are required in routines 
! Revision: 17 Jan 2001 FCC (e.q. mos_to_aos_c_nrmd_read)
! Revision: 27 May 2001 FCC many changes due to optional arguments
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)            :: f_name 
  character(len=*), intent(in)            :: f_type
  character(len=*), intent(in)            :: f_action
  character(len=*), optional, intent(in)  :: f_format 
  character(len=*), optional, intent(in)  :: f_path   
  character(len=*), optional, intent(in)  :: f_position
  character(len=*), optional, intent(in)  :: f_program 
  character(len=*), optional, intent(in)  :: f_title  
  character(len=*), optional              :: f_status
  logical(bool),    optional              :: f_is_open 

! result:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: l_type, l_tail
  character(len=max_string_len_action)       :: l_action

  logical(bool)                              :: exist

! begin:
  l_here ='file_catalogue'
  call enter (l_here)

  l_action = f_action
  l_tail = '-'//string_tail (f_type, '-', back = .true.)

  if(debug) then
   write (logf,*)trim(l_here),'-d: action >',trim(f_action),'< '
   write (logf,*)trim(l_here),'-d: type   >',trim(f_type),'< '
   write (logf,*)trim(l_here),'-d: l_tail >',trim(l_tail),'<'
  end if

! add extension "-in", "-out", "-inout"

  l_type = f_type

  select case(l_action)

   case ("append")

    if(trim(l_tail) /= "-out") then
     l_type = string_append (f_type, '-out')
     write (logf,*)trim(l_here),': app-file >',trim(f_name),'<', &
                  trim(l_here),': file-type >',trim(f_type),'< re-set to >',trim(l_type),'< '
    end if

   case ("delete")

    if(trim(l_tail) /= "-del") then
     l_type = string_append (f_type, '-del')
     write (logf,*)trim(l_here),': del-file >',trim(f_name),'<', &
             trim(l_here),': file-type re-set to >',trim(l_type),'< '
    end if

   case ("read")

    if(trim(l_tail) /= "-in") then
     l_type = string_append (f_type, '-in')
     write (logf,*)trim(l_here),': read-file >',trim(f_name),'<'
     write (logf,*)trim(l_here),': file-type re-set to >',trim(l_type),'<'
    end if

   case ("readwrite")
    if(trim(l_tail) /= "-inout") then
     l_type = string_append (f_type, '-inout')
     write (logf,*)trim(l_here),': rw-file >',trim(f_name),'<'
     write (logf,*)trim(l_here),': file-type re-set to >',trim(l_type),'<'
    end if

   case ("write")
    if(trim(l_tail) /= "-out") then
     l_type = string_append (f_type, '-out')
     write (logf,*)trim(l_here),': write-file >',trim(f_name),'<'
     write (logf,*)trim(l_here),': file-type re-set to >',trim(l_type),'<'
    end if

   case default
     message = "undefined action >"+l_action+"< should be one of:" &
     +CR+"     append delete read readwrite write"
     call die (l_here, message)
  end select

  call check_association ('files_types', files_types, 'files_init', files_init)

  if(debug) then
   write (logf,*)trim(l_here),'-d: file-type   >',trim(l_type),'< '
   write (logf,*)trim(l_here),'-d: file-action >',trim(f_action),'< '
   write (logf,*)trim(l_here),'-d: file-status >',trim(f_status),'< '
  end if

! get file unit
  unit = string_in_array_locate (l_type, files_types, 1, size(files_types))
  if(trim(l_type) == "results-out") then
    unit = res
  else if(trim(l_type) == "logfile-out") then
    unit = logf
  else
  end if

  if (unit .ne. 0) then 

   write (logf,*) trim(l_here), &
   ': file-type >',trim(l_type),'< already catalogued unit=',unit

   if (f_name == NOT_INIT_S) then
    message = 'in routine'*l_here*CR &
             *' file-type >'+f_type+'< has undefined file name'
    call require (message, f_name /= NOT_INIT_S) 
   end if

   if(file_inquire_open (f_name, f_path) /= 0) then
    message = 'file >'+f_name+'< already opened'
    call warning (l_here, message)
   end if

   write (logf,*) trim(l_here), &
   ': file-name >',trim(files_names(unit)),'< '

   write (logf,*) trim(l_here), &
   ': file-action >',trim(files_actions(unit)),'< '

   write (logf,*) trim(l_here), &
   ': catalogue information will be over-written'

  else

! find free unit
   unit = string_in_array_locate ('free ', files_types, 10, size(files_types))
   if (unit .eq. 0)  then
    call files_show
    message = 'no free catalogue entry left for file-type >'+l_type+'<'
    call die (l_here, message)
   else
    write (logf,*) trim(l_here),': file >',trim(f_name),'<'
    write (logf,*) trim(l_here),': type >',trim(l_type),'< catalogued unit=',unit
   end if
  end if

  if(.not.present(f_status) .or. len_trim(f_status) == 0) then

   if(trim(f_action) == "append")    f_status = "old"//blanks
   if(trim(f_action) == "read")      f_status = "old"//blanks
   if(trim(f_action) == "readwrite") f_status = "old"//blanks
   if(trim(f_action) == "write")     f_status = "new"//blanks

  else

! avoid some possible contradictions.
! when status is "new" file should not exist.

    exist   = file_exists (f_name, f_path)
    if(debug) write (logf,*)trim(l_here),'-d: file exist =',exist
    message ='in routine'*l_here+' status = "new" ' &
             +CR+' for already existing file:' & 
             +CR+'>'+f_name+'<'
    call ensure (message, .not.(exist .and. (f_status(1:3) == "new")))

! status == "new" and action == "read"
    message ='in routine'*l_here+ &
            ' action = "read" and status = "new" for file:' & 
            +CR+'>'+f_name+'<'
    call ensure (message, &
    .not.((f_status(1:3) == "new") .and. f_action(1:4) == "read") )

! status == "old" and file does not exist and action == "write" 
    if ( .not. exist .and. f_status(1:3) == "old" & 
         .and. f_action(1:5) == "write") then
      message ='status is "old", action is "write" for inexisting file >'+f_name+CR &
               +'< please change status = from "old" to "unknown" in input file'
      call die (l_here, message)
    end if

! status == "replace" and action == "read"
    message ='in routine'*l_here+ &
            ' action = "read" and status = "replace" for file:' & 
            +CR+'>'+f_name+'<'
    call ensure (message, &
    .not.((f_status(1:3) == "replace") .and. f_action(1:4) == "read") )

  end if

  if(present(f_position) .and. len_trim(f_position) /= 0) then
! position == "append" and action == "read"
    message ='in routine'*l_here+ &
            ' action = "read" and position = "append" for file:' & 
            +CR+'>'+f_name+'<'
    call ensure (message, &
    .not.((f_position(1:6) == "append") .and. f_action(1:4) == "read") )
  end if

  files_actions(unit)   =  f_action
  files_formats(unit)   =  f_format
  files_names(unit)     =  f_name
  files_types(unit)     =  l_type 
  files_paths(unit)     =  f_path     
  files_positions(unit) =  f_position 
  files_programs(unit)  =  string_to_upper (f_program )
  files_titles(unit)    =  f_title 
  files_status(unit)    =  f_status 

  if(debug) then 
    write (logf,*)trim(l_here),'-d: file-unit=',unit 
    call catalogue_print_by_unit (unit)
  end if

! check if file already open.
  if(.not.present(f_is_open)) then
    files_open_is(unit)   =  .false. 
  else
    if (f_is_open .and. (f_position(1:6) .ne. "append")) then
     write (logf,*)trim(l_here),': file >',trim(f_name), &
            '< already opened. not in append mode'
    end if
  end if

  call exit (l_here)

  end function file_catalogue

! ===============================================================================
  subroutine file_close_by_name (f_name)
! -------------------------------------------------------------------------------
!
! Name          : file_close_by_name
! Argument (in) : f_name 
!
! Description   : close a file knowing its name
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit
  character(len=max_string_len_file_type)    :: f_type

! begin:
  l_here ='file_close_by_name'
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(routines_21),'-',trim(l_here), &
           ' file-name >',trim(f_name),'<'
  end if

  unit   = unit_from_name (f_name)
  f_type = files_types(unit)

  if(debug .or. print_level  >  0) then
   write (logf,*)trim(l_here),': file-name >',trim(f_name),'< '
   write (logf,*)trim(l_here),': file-type >',trim(f_type),'< '
  end if

  if (unit .eq. 0) then
   write (logf,*)trim(routines_21),'-e: file name >',trim(f_name), &
                             '< not found in catalogue:'
   call files_show
   call die (l_here, 'file name not found in catalogue')
  end if

  call file_close_by_unit (unit)

  call exit (l_here)

  end subroutine file_close_by_name

! ===============================================================================
  subroutine file_close_by_type (f_type)
! -------------------------------------------------------------------------------
!
! Name          : file_close_by_type
!
! Argument (in) : f_type 
!
! Description    : close a file knowing its type
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_type

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit

! begin:
  l_here ='file_close_by_type'
  call enter (l_here)

  unit = string_in_array_locate (f_type, files_types, 1, size(files_types))

  if(debug)write (logf,*)trim(l_here),'-d: unit =',unit

  if (unit .eq. 0) then
   write (logf,*) trim(routines_21),'-e: file type >',trim(f_type), &
                              '< not found in catalogue:'
   call files_show
   call die (l_here, 'file type not found in catalogue')
  end if

  call file_close_by_unit (unit)

  call exit (l_here)

  end subroutine file_close_by_type

! ===============================================================================
  subroutine file_close_and_uncat_by_name (f_name)
! -------------------------------------------------------------------------------
!
! Name          : file_close_and_uncat_by_name
! Argument (in) : f_name 
!
! Description   : close and uncatalogue a file knowing its name
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit
  character(len=max_string_len_file_type)    :: f_type

! begin:
  l_here ='file_close_and_uncat_by_name'
  call enter (l_here)

  if(debug) then
   write (logf,*)trim(routines_21),'-',trim(l_here), &
           ' file-name >',trim(f_name),'<'
  end if

  unit   = unit_from_name (f_name)
  f_type = files_types(unit)

  if(debug .or. print_level  >  0) then
   write (logf,*)trim(l_here),': file-name >',trim(f_name),'< '
   write (logf,*)trim(l_here),': file-type >',trim(f_type),'< '
  end if

  if (unit .eq. 0) then
   write (logf,*)trim(routines_21),'-e: file name >',trim(f_name), &
                             '< not found in catalogue:'
   call files_show
   call die (l_here, 'file name not found in catalogue')
  end if

  call file_close_and_uncat_by_unit (unit)

  call exit (l_here)

  end subroutine file_close_and_uncat_by_name

! ===============================================================================
  subroutine file_close_and_uncat_by_type (f_type)
! -------------------------------------------------------------------------------
!
! Name          : file_close_and_uncat_by_type
!
! Argument (in) : f_type 
!
! Description   : close and uncatalogue a file knowing its type
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_type

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit

! begin:
  l_here ='file_close_and_uncat_by_type'
  call enter (l_here)

  unit = string_in_array_locate (f_type, files_types, 1, size(files_types))

  if(debug)write (logf,*)trim(l_here),'-d: unit =',unit

  if (unit .eq. 0) then
   write (logf,*)trim(routines_21),'-e: file type >',trim(f_type), &
                             '< not found in catalogue:'
   call files_show
   call die (l_here, 'file type not found in catalogue')
  end if

  call file_close_and_uncat_by_unit (unit)

  call exit (l_here)

  end subroutine file_close_and_uncat_by_type

! ===============================================================================
  subroutine file_close_by_unit (unit)
! -------------------------------------------------------------------------------
!
! Name          : file_close_by_unit
!
! Argument (in) : unit 
!
! Description   : close a file knowing its unit
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent(in)                   :: unit  

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  logical(bool)                              :: l_debug
  logical(bool)                              :: l_opened
  integer(dpi)                               :: l_unit  

! begin:
  l_here ='file_close_by_unit'
  call enter (l_here)
  l_debug = debug

  f_name = files_names(unit)
  f_type = files_types(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: unit=',unit
   write (logf,*)trim(l_here),'-d: file >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: type >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: files_open_is =',files_open_is(unit)   
   inquire (file = f_name, number = l_unit, opened = l_opened) 
   write (logf,*)trim(l_here),'-d: inquire => opened ?=',l_opened,' unit=',l_unit
  end if

  if(unit .eq. in) then

    rewind unit
    if(print_level  >  5) write (logf,*)trim(l_here),': unit ',unit,' rewind'

  else if (unit .ne. out) then

   write (logf,*)trim(l_here),': file >',trim(files_names(unit)), &
           '< closed on unit ',unit

   if(files_open_is (unit)) then 

     files_open_is (unit) = .false.
     close (unit = unit)

     if(l_debug .or. print_level  >  0) then
       write (logf,*) trim(l_here),'-w: file >',trim(f_name),  &
                '< closed and kept in catalogue. unit ', &
                unit,' type >', trim(f_type),'<'
     end if

    else

      if(l_debug .or. warning_level  >  0) then
       write (logf,*) trim(l_here),'-w: file >',trim(f_name),             &
                '< not opened not closed kept in catalogue. unit ', &
                 unit,' type >', trim(f_type),'<'
      end if

    end if

  end if

  call exit (l_here)

  end subroutine file_close_by_unit

! ===============================================================================
  subroutine file_close_and_uncat_by_unit (unit)
! -------------------------------------------------------------------------------
!
! Name          : file_close_and_uncat_by_unit
!
! Argument (in) : unit 
!
! Description   : close and uncatalogue a file knowing its unit
!
! Authors:F.Colonna
! Date  :07 Jun 2001
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent(in)                   :: unit  

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type

! begin:
  l_here ='file_close_and_uncat_by_unit'
  call enter (l_here)

  f_name = files_names(unit)
  f_type = files_types(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: unit=',unit
   write (logf,*)trim(l_here),'-d: file >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: type >',trim(f_type),'<'
  end if

  if(unit .eq. in) then
    rewind unit
    if(print_level  >  -1) write (logf,*)trim(l_here),': unit ',unit,' rewind'

  else if (unit .ne. out) then
    write (logf,'(1x,3a,i2)') &
   'file_close_and_uncat_by_unit: file >',trim(files_names(unit)),'< closed on unit ',unit

    call file_uncatalogue (unit)
    close (unit = unit)

    if(debug .or. print_level  >  -1) then
     write (logf,*) trim(l_here),': file >', &
     trim(f_name), '< closed and uncatalogued. unit ',unit,' type >', trim(f_type),'<'
    end if

  end if

  call exit (l_here)

  end subroutine file_close_and_uncat_by_unit

! ===============================================================================
  subroutine file_delete (f_name, f_path)
! -------------------------------------------------------------------------------
!
! Name                    : file_delete
!
! Argument (in)           : f_name
! Argument (optional,in)  : f_path       
!
! Description    : delete file on disk    
! Description    : also uncatalogue the file if catalogued
!
! Authors:F.Colonna
! Date   :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name
  character(len=*), intent(in), optional     :: f_path

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: l_path     
  logical(bool)                              :: exist
  integer(dpi)                               :: unit

! begin:
  l_here ='file_delete'
  call enter (l_here)

  if(debug)write (logf,*)trim(l_here),'d: file >',trim(f_name),'<'
  if(debug)write (logf,*)trim(l_here),'d: path >',trim(f_path),'<'

  unit = unit_from_name (f_name)
  if(debug)write (logf,*)trim(l_here),'d: file >',trim(f_name),'< unit=',unit  

  if(.not.present(f_path)) then
   l_path = './' ! blanks ! intent(in)
  else
   l_path = f_path     ! intent(in)
  end if

  if(debug)write (logf,*)trim(l_here),'d: l_path >',trim(l_path),'<'

  exist = file_exists (f_name, l_path)

  if (debug) write (logf,*)trim(l_here),'-d: file >',trim(f_name),'< exist=',exist

  if (.not. exist) then 
   write (logf,*)trim(l_here),': file >',trim(f_name),'< does not exist. not deleted'

  else

   if( .not. files_open_is(unit)) then 
    call unit_open (unit) 
   end if

   close (unit = unit, status = "delete", iostat = iostat)

   if (iostat /= 0) then

    write (logf,*)trim(l_here),': close-delete failed for file >',trim(f_name), &
                         '< on unit ',unit,' iostat=',iostat
    message = 'close-delete failed while trying to delete file. iostat='+ &
    iostat
    call die (l_here, message)

   else
    write (logf,*) trim(l_here),': file >',trim(f_name),'< unit ',unit,' deleted from disk'
   end if

  end if

  call file_uncatalogue (unit)   

  call exit (l_here)

  end subroutine file_delete

! ===============================================================================
  function file_exists (f_name, f_path) result(result) 
! -------------------------------------------------------------------------------
!
! Name          : file_exists
!
! Argument (in) : f_name
! Argument (in) : f_path 
!
! Description    : returns true or false  
! Description    : f_path is used only when it starts with a SLASH
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name
  character(len=*), intent(in), optional     :: f_path

! result:
  logical(bool)                              :: result

! local:
  character(len=max_string_len_routine_name) :: l_here

  character(len=max_string_len_file_name)    :: file           
  character(len=max_string_len_file_name)    :: l_path     
  logical(bool)                              :: exist

! begin:
  l_here ='file_exists'
  call enter (l_here,5)

  if(.not.present(f_path)) then
   l_path = blanks ! intent(in)
  else
   l_path = f_path     ! intent(in)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d: file-name >',trim(f_name),'<'
   write (logf,*)trim(routines_21),'-d: file-path >',trim(l_path),'<'
  end if

  file = file_name_with_path (f_name, l_path) 
  inquire (file = file, exist = exist)

  result = exist

  if (exist .and. (print_level  >  5)) write (logf,*)trim(l_here), &
     ': file >',trim(file),'< exists'

  if (.not. exist .and. (warning_level  >  1)) write (logf,*)trim(l_here), &
     ': file >',trim(file),'< does not exist'

  if (debug) write (logf,*) &
      trim(l_here),': file >',trim(f_name),'<'

  if (debug) write (logf,*) &
      trim(l_here),': path >',trim(l_path),'<'

  if (debug) write (logf,*) &
      trim(l_here),': file >',trim(file),'< exist=',exist

  call exit (l_here,5)

  end function file_exists

! ===============================================================================
  function file_record_with_word (this_word, word_pos, unit) &
                           result(result)
! -------------------------------------------------------------------------------
!
! Name           : file_record_with_word 
!
! Argument (in ) : this_word
! Argument (in ) : word_pos
! Argument (in ) : unit
!
! Description    : returns the record of unit unit
! Description    : containing "this_word" at the word_pos-th position in record
! Description    : empty string returned if not found
!
! Authors: F. Colonna
! Date  :29 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: this_word     
  integer(dpi)                               :: word_pos
  integer(dpi)                               :: unit

! result:
  character(len=max_string_len)              :: result 

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: done
  integer(dpi)                               :: count        
  character(len=max_string_len)              :: record 

! Improve
! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi), pointer                        :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len

! begin:
  l_here ='file_record_with_word'
  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(l_here),'-d: fectching >',trim(this_word),'<'
    write (logf,*)trim(l_here),'-d: at position =',word_pos
    write (logf,*)trim(l_here),'-d: in file >',trim(files_names(unit)),'<'
  end if

  message = 'in routine'*l_here+' unit is 0'
  call require (message, unit /= 0)

  message = 'in routine'*l_here+' unit is not opened'
  call require (message, files_open_is(unit) )

  count = 0
  done = .false.
  do while (.not.done)

   record = blanks
   read (unit,'(a)',iostat=iostat) record  

   if(iostat < 0) then

! exit at eof
    if(debug .or. print_level > 2) then 
     write (logf,*)trim(l_here),': end-of-file reached for file >', &
                             trim(files_names(unit)),'<'
    end if

    record = "" 
    exit 
    
   else if(iostat /= 0) then
! i/o error
    message ='error in reading file >'+files_names(unit)+'< iostat='+iostat
    call die (l_here, message)

   end if

   count = count + 1
   if(debug) write (logf,*)trim(l_here),'-d: count=',count
   if(debug) write (logf,*)trim(l_here),'-d: fetch loop. reading record >', &
             trim(record),'<'

   if(len_trim(record ) == 0 ) cycle 
   
! split input record
   number_of_words = string_to_words_count (record ,' ', word_max_len)
   call split_string (record , ' ', words, lenwrd, number_of_words, (len(words)+0))

   if(debug) then 
     write (logf,*)trim(l_here),'-d: record  >',trim(record ),'<'
     write (logf,*)trim(l_here),'-d: number_of_words = ',number_of_words
     write (logf,*)trim(l_here),'-d: fetching >',trim(this_word),'<'
   end if

   if(number_of_words >= 1 ) then 
    if(trim(words(word_pos)) == trim(this_word) ) then

     if(debug .or. print_level > 0) then 
       write (logf,*)trim(l_here),': word # ',word_pos, &
       ' found at record # ',count,' >',trim(record ),'<'
     end if
     if(debug) then 
       write (logf,*)trim(l_here),'-d: words(word_pos) >',trim(words(word_pos)),'<'
       write (logf,*)trim(l_here),'-d: in file >',trim(files_names(unit)),'<'
      end if
     exit
   
    end if  
   end if  

  end do  ! fetching loop

  result = record

  if(debug) then 
    write (logf,*)trim(l_here),'-d: fetched word >',trim(this_word),'<', &
            ' was found at record =',count   
  end if

  call exit (l_here, 5)

  end function file_record_with_word

! ===============================================================================
  function file_fetch_word (this_word, word_pos, unit) &
                           result(result)
! -------------------------------------------------------------------------------
!
! Name           : file_fetch_word 
!
! Argument (in ) : this_word
! Argument (in ) : word_pos
! Argument (in ) : unit
!
! Description    : returns the record number of unit unit
! Description    : containing "this_word" at the word_pos-th position in record
! Description    : - count is returned if word not found
! Description    : starts counting at current position  
!
! Authors: F. Colonna
! Date  :29 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: this_word     
  integer(dpi)                               :: word_pos
  integer(dpi)                               :: unit

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: done
  integer(dpi)                               :: count        
  character(len=max_string_len)              :: record 

! Improve
! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi), pointer                        :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len

! begin:
  l_here ='file_fetch_word'
  call enter (l_here, 5)

  if(debug) then
    write (logf,*)trim(l_here),'-d: fectching >',trim(this_word),'<'
    write (logf,*)trim(l_here),'-d: at position =',word_pos
    write (logf,*)trim(l_here),'-d: in file >',trim(files_names(unit)),'<'
  end if

  message = 'in routine'*l_here+' unit is 0'
  call require (message, unit /= 0)

  message = 'in routine'*l_here+' unit is not opened'
  call require (message, files_open_is(unit) )

  count = 0
  done = .false.
  do while (.not.done)

   record = blanks
   read (unit,'(a)',iostat=iostat) record  

! exit at eof
   if(iostat < 0) then

    if(debug .or. print_level > 2) then 
     write (logf,*)trim(l_here),': end-of-file reached for file >', &
                             trim(files_names(unit)),'<'
    end if

    count = -count
    exit 
    
   else if(iostat /= 0) then
! i/o error
    message ='error in reading file >'+files_names(unit)+'< iostat='+iostat
    call die (l_here, message)

   end if

   count = count + 1
   if(debug) write (logf,*)trim(l_here),'-d: count=',count
   if(debug) write (logf,*)trim(l_here),'-d: fetch loop. reading record >', &
             trim(record),'<'

   if(len_trim(record ) == 0 ) cycle 
   
! split input record
   number_of_words = string_to_words_count (record ,' ', word_max_len)
   call split_string (record , ' ', words, lenwrd, number_of_words, (len(words)+0))

   if(debug) then 
     write (logf,*)trim(l_here),'-d: record  >',trim(record ),'<'
     write (logf,*)trim(l_here),'-d: word_pos  = ',word_pos
     write (logf,*)trim(l_here),'-d: number_of_words = ',number_of_words
     write (logf,*)trim(l_here),'-d: fetching >',trim(this_word),'<'
   end if

   if(number_of_words >= 1 .and. number_of_words >= word_pos) then 

    if(trim(words(word_pos)) == trim(this_word) ) then

     if(debug .or. print_level > 0) then 
       write (logf,*)trim(l_here),': word # ',word_pos, &
       ' found at record # ',count,' >',trim(record ),'<'
     end if

     if(debug) then 
       write (logf,*)trim(l_here),'-d: words(word_pos) >',trim(words(word_pos)),'<'
       write (logf,*)trim(l_here),'-d: in file >',trim(files_names(unit)),'<'
      end if

     done = .true.
     exit
   
    end if  ! words(word_pos) == this_word

   end if  ! if(number_of_words 

  end do  ! fetching loop

  result = count

  if(debug) then 
    write (logf,*)trim(l_here),'-d: fetched word >',trim(this_word),'<', &
            ' was found at record =',count   
  end if

  call exit (l_here, 5)

  end function file_fetch_word

! ===============================================================================
  subroutine file_header_write (unit)
! -------------------------------------------------------------------------------
!
! Name          : file_header_write
! 
! Argument (in) : unit
!
! Description   : writes file headers in unit unit
!
! Authors: F. Colonna
! Date  :29 Oct 2000
! Revision :21 Dec 2002 F.Colonna results
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit    

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='file_header_write'
  call enter (l_here, 3)

  if(debug) write (logf,*) trim(l_here),'-d: unit=',unit

  call date_time

  arch_name = architecture_get ()
  mach_name = machine_name_get ()
  user_name = username_get ()
  dir_cur_name = directory_get ()

  write(unit,'(2a)') 'QmcMol version ',trim(version_name)
  write(unit,'(2a)') 'executable ',trim(exec_name)
  write(unit,'(2a)') 'run on machine ',trim(mach_name)
  write(unit,'(2a)') 'under os ',trim(arch_name)

  if(unit /= logf) then
   if (job_name /= " ")  &
    write(unit,'(2a)')'job name  ',trim(job_name)

   if (title /= " ")  &
    write(unit,'(2a)')'job title ',trim(title)
  end if

  if(.not.associated(files_names)) then
   if(trace_depend) then
    write (logf,*) trim(l_here),': object files_names not associated'
    write (logf,*) trim(l_here),': calling routine files_init'
   end if
   call files_init
  end if

  if(unit /= logf .or. unit /= res) then 
    if(files_names(unit) /= " ") &
    write(unit,'(2a)')'file ',trim(files_names(unit))

    if(trim(files_paths(unit)) == './') then
      write(unit,'(2a)')'path ',trim(dir_cur_name)
    else
      write(unit,'(2a)')'path ',trim(files_paths(unit))
    end if
  end if

  write(unit,'(2a)')'done by ',trim(user_name)
  write(unit,'(4a)')'at ',trim(cur_time),' on ',trim(cur_date)

  if ( (unit /= logf .or. unit /= res ) .and. files_titles(unit) /= " ") &
  write(unit,'(2a)')'title ',trim(files_titles(unit))

  write(unit,'(a)')' '

! write(unit,'(a)')'end_of_header'  
! should be set after some permanents data 

  call exit (l_here, 3)

  end subroutine file_header_write

! ==================================================================================
  function file_name_suffix_get (file_name) result (result) 
! ----------------------------------------------------------------------------------
!
! Name           : file_name_suffix_get 
!
! Argument (in ) : file_name
!
! Description    : file_name = path_name/basename.suffix
! Description    : if no "." then suffix = " "                    
!
! Authors:F.Colonna
! Date   :21 Dec 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)     :: file_name

! result:
  character(len=max_string_len)    :: result

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='file_name_suffix_get'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: file_name >',trim(file_name),'<'

  if (len_trim (file_name) == 0) then
    message ='in routines'*routines_21*CR &
            *' file-name is an empty string' 
    call require (message, len_trim (file_name) == 0)
  end if

  if( index(file_name,".") == 0) then
    result = " "
  else
    result = string_tail (file_name, ".", .true.)
  end if

  if(debug) write (logf,*) trim(l_here),'-d: suffix >',trim(result),'<'

  call exit (l_here,5)

  end function file_name_suffix_get

! ==================================================================================
  function file_name_basename (file_name) result (result) 
! ----------------------------------------------------------------------------------
!
! Name           : file_name_basename 
!
! Argument (in ) : file_name
!
! Description    : file_name = path_name/basename.suffix     
!
! Authors:F.Colonna
! Date   :21 Dec 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: file_name

! result:
  character(len=max_string_len) :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len) :: string

! begin:
  l_here ='file_name_basename'
  call enter (l_here,5)

  if(debug) then
    write (logf,*) trim(l_here),'-d: file_name >',trim(file_name),'<'
  end if

  if (len_trim (file_name) == 0) then
    message ='in routines'*routines_21*CR &
            *' file-name is an empty string'
    call require (message, len_trim (file_name) == 0)
  end if

  if (index(file_name, slash) == 0) then
    string = file_name 
  else 
    string = string_tail (file_name, slash, .true.)
  end if

  if (index(string, ".") == 0) then
    result = file_name 
  else 
    result = string_head_get (string, ".", .true.)
  end if

  if(debug) write (logf,*) trim(l_here),'-d: basename >',trim(result),'<'

  if (len_trim (result) == 0) then
    message ='in routines'*routines_21*CR &
            *' file >'+file_name+'< has empty basename'
    call require (message, len_trim (file_name) == 0)
  end if

  call exit (l_here,5)

  end function file_name_basename

! ==================================================================================
  function file_name_path_get (file_name) result (result) 
! ----------------------------------------------------------------------------------
!
! Name           : file_name_path_get 
!
! Argument (in ) : file_name
!
! Description    : file_name = path_name/basename.suffix     
!
! Authors:F.Colonna
! Date   :21 Dec 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)     :: file_name

! result:
  character(len=max_string_len)    :: result

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='file_name_path_get'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: file_name >',trim(file_name),'<'

  if (len_trim (file_name) == 0) then
    message ='in routines'*routines_21*CR &
            *' file-name is an empty string'
    call require (message, len_trim (file_name) == 0)
  end if

  result = string_head_get (file_name, slash, .true.)

  if(debug) write (logf,*) trim(l_here),'-d: path >',trim(result),'<'

  if (len_trim (result) == 0) then
    message ='in routines'*routines_21*CR &
            *' file-path is an empty string'
    call require (message, len_trim (file_name) == 0)
  end if

  call exit (l_here,5)

  end function file_name_path_get

! ===============================================================================
  function file_name_with_path (f_name, f_path) result(result) 
! -------------------------------------------------------------------------------
!
! Name          : file_name_with_path 
!
! Argument (in) : f_name
! Argument (in) : f_path
!
! Description   : concatenates the path to file name 
! Description   : (if not already included in file name)
!
! Authors:F. Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name   
  character(len=*), intent(in), optional     :: f_path   

! result:
  character(len=max_string_len_file_name)    :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: fc_path     
  character(len=max_string_len)              :: stemp  
  integer :: last_char

! begin:
  l_here ='file_name_with_path'
  call enter (l_here,5)

  stemp = blanks

  if(present(f_path)) then 
   last_char = len_trim(f_path)
   fc_path = f_path          
  else
   fc_path = blanks 
   last_char = 0
  end if

  if(debug) then
   write (logf,*) trim(l_here),'-d: file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(fc_path),'<'
   write (logf,*) trim(l_here),'-d: last_char =',last_char
  end if

  if (f_name(1:1) == SLASH) then  
   stemp  = f_name
  else if (last_char == 0) then
   stemp  = f_name
  else
   if (f_path(last_char:last_char) == SLASH) then
    stemp = trim(fc_path) // f_name
   else 
    stemp = trim(fc_path)//SLASH//f_name
   end if
  end if

  result = stemp 

  if(debug) then
   write (logf,*) trim(l_here),': file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),': file-path >',trim(fc_path),'<'
   write (logf,*) trim(l_here),': full-name >',trim(stemp),'<'
  end if

  call exit (l_here,5)

  end function file_name_with_path

! ===============================================================================
  function file_inquire_open (f_name, f_path) result(result) 
! -------------------------------------------------------------------------------
!
! Name          : file_inquire_open
!
! Result        : unit 
!
! Argument (in) : f_name
! Argument (in) : f_path 
!
! Description   : inquires if file opened if so unit is returned else 0
! Description   : f_path is used only when it starts with a SLASH
!
! Authors:F.Colonna
! Date  :15 Jun 2001
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name
  character(len=*), intent(in), optional     :: f_path

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: l_file           
  character(len=max_string_len_file_name)    :: l_path     
  integer(dpi)                               :: l_unit
  logical(bool)                              :: l_opened

! begin:
  l_here ='file_inquire_open'
  call enter (l_here)

  if(.not.present(f_path)) then
   l_path = blanks ! intent(in)
  else
   l_path = f_path     ! intent(in)
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d: file-name >',trim(f_name),'<'
   write (logf,*)trim(routines_21),'-d: file-path >',trim(l_path),'<'
  end if

  l_file = file_name_with_path (f_name, l_path) 
  inquire (file = l_file, number = l_unit, opened = l_opened)
  if(l_opened) then
    result = l_unit 
    if(debug) write (logf,*)trim(routines_21),'-d: opened on unit ',l_unit        
  else
    result = 0
    if(debug) write (logf,*)trim(routines_21),'-d: not opened '
  end if

  if (l_opened .and. (print_level  >  5)) write (logf,'(1x,4a)') &
      trim(l_here),': file >',trim(l_file),'< opened'

  if (.not. l_opened .and. (warning_level  >  1)) write (logf,'(1x,4a)') & 
      trim(l_here),': file >',trim(l_file),'< not opened'

  call exit (l_here)

  end function file_inquire_open

! ===============================================================================
  function file_open_and_cat (f_name,     &
                              f_type,     &
                              f_action,   &
                              f_format,   &   ! optional
                              f_path,     &   ! optional
                              f_position, &   ! optional
                              f_program,  &   ! optional
                              f_title,    &   ! optional
                              f_status,   &   ! optional
                              f_is_open)  &   ! optional
                              result (result)
! -------------------------------------------------------------------------------
!
! Name                    : file_open_and_cat 
!
! Argument (in )          : f_name
! Argument (in )          : f_type
! Argument (in )          : f_action
! Argument (in, optional) : f_format
! Argument (in, optional) : f_path
! Argument (in, optional) : f_position
! Argument (in, optional) : f_program
! Argument (in, optional) : f_title
! Argument (in, optional) : f_status
! Argument (optional) : f_is_open

! Description    : to be opened a file must be catalogued first
! Description    : (defined by the file ... end  command)
! Description    : returns logical unit number
! Description    : fills files_open_is array with .true.
!
! Authors:F.Colonna
! Revision 25 Apr 2003: l_debug -> debug R. Assaraf
! Date  :21 Sep 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)            :: f_name
  character(len=*), intent(in)            :: f_type
  character(len=*), intent(in)            :: f_action

! result:
  integer(dpi)                            :: result

! i/o optional:
  logical(bool), optional                 :: f_is_open
  character(len=*), optional, intent(in)  :: f_format
  character(len=*), optional, intent(in)  :: f_path   
  character(len=*), optional, intent(in)  :: f_position
  character(len=*), optional, intent(in)  :: f_program
  character(len=*), optional, intent(in)  :: f_title
  character(len=*), optional, intent(in)  :: f_status

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit
  character(len=max_string_len_format)       :: fc_format   
  character(len=max_string_len_file_name)    :: fc_path     
  character(len=max_string_len_position)     :: fc_position
  character(len=max_string_len_program)      :: fc_program 
  character(len=max_string_len_title)        :: fc_title  
  character(len=max_string_len_status)       :: fc_status   
  logical(bool)                              :: fc_is_open 
!  logical(bool)                              :: l_debug

! begin:
  l_here ='file_open_and_cat'
  call enter (l_here)

! why ?
!  l_debug = debug 

  if(debug) then
   write (logf,*)trim(l_here),'-d: file-name >',trim(f_name),'< '
   write (logf,*)trim(l_here),'-d: file-type >',trim(f_type),'< '
   write (logf,*)trim(l_here),'-d: file-action >',trim(f_action),'< '
  end if

! check if catalogued 
  unit = unit_from_type (f_type) 

  if(debug) write (logf,*)trim(l_here),'-d: unit=',unit

  if (unit .eq. 0)  then ! catalogue the new file

   if(present(f_format))   then 
    fc_format   = f_format 
   else
    fc_format  = '?'
   end if
   if(present(f_path))     then 
    fc_position = f_position
   else
    fc_path    = blanks
   end if
   if(present(f_position)) then 
    fc_path     = f_path    
   else
    fc_position = 'asis'
   end if
   if(present(f_program))  then 
    fc_program  = f_program 
   else
    fc_program = '?'
   end if
   if(present(f_title))    then 
    fc_title    = f_title   
   else
    fc_title   = '?'
   end if
   if(present(f_status))   then 
    fc_status   = f_status 
   else
    fc_status  = '       '
   end if
   if(present(f_is_open))  then 
    fc_is_open  = f_is_open
   else
    fc_is_open = .false.  
   end if

   unit = file_catalogue (f_name,      &
                          f_type,      &
                          f_action,    &
                          fc_format,   &
                          fc_path,     &
                          fc_position, &
                          fc_program,  &
                          fc_title,    &
                          fc_status,   &
                          fc_is_open)

  else

   if(warning_level  >  5) write (logf,*)trim(l_here), &
   '-w: file-type >',trim(f_type),'< already catalogued. items over-written'

   if(present(f_format))   files_formats(unit)   =  f_format
   if(present(f_position)) files_positions(unit) =  f_position
   if(present(f_path))     files_paths(unit)     =  f_path
   if(present(f_program))  files_programs(unit)  =  f_program
   if(present(f_title))    files_titles(unit)    =  f_title
   if(present(f_status))   files_status(unit)    =  f_status
   if(present(f_is_open))  files_open_is(unit)   =  f_is_open

  end if

  if (debug) call catalogue_print_by_unit (unit)

  select case (trim(f_action))
   case ("append")
    call unit_open_append (unit)

   case ("delete")
    call unit_open (unit)

   case ("read")
    call unit_open_read (unit)

   case ("readwrite")
    call unit_open_readwrite (unit)

   case ("write")
    call unit_open_write (unit)

   case default
    call unit_open_readwrite (unit)

  end select 

  result = unit 
  files_open_is(unit) = .true.

  message ='file >'+f_name+'< type >'+f_type+'< has been opened on unit='+unit
  call message_out (l_here, message)

  call exit (l_here)

  end function file_open_and_cat

! ===============================================================================
  function file_open_and_cat_from_type (f_type) result(result) 
! -------------------------------------------------------------------------------
!
! Name          : file_open_and_cat_from_type
!
! Argument (in) : f_type
!
! Result        : catalogue number == logical fortran unit
!
! Description   : opens a catalogued file from its type 
! Description   : returns catalogue number == logical fortran unit
!
! Authors:F.Colonna
! Date   :13 Oct 2000
! Revision: 08 Oct 2002 F. Colonna
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_type

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: f_name   
  character(len=max_string_len_action)       :: f_action
  integer(dpi)                               :: unit

! begin:
  l_here ='file_open_and_cat_from_type'
  call enter (l_here)

  if (debug) &
  write (logf,*)trim(routines_21),'-d: file-type >',trim(f_type),'<'

  call check_association &
  ('files_types', files_types, 'files_init', files_init)

! get unit 
  unit = unit_from_type (f_type) 

  if(debug) then
   write (logf,*)trim(routines_21),'-d: at input unit=',unit
  end if

  if(unit .eq. 0) then 
   call files_show
   message = 'in routine'*l_here+' : ' &
            +CR+' no file with type >'+f_type+'< in catalogue' &
            +CR+' add this line in input file:' &
            +CR+' file name = ??? type '*f_type*' status=??? action=??? end_file'
   call require (message, unit /= 0)
  end if

  f_name = files_names(unit)
  f_action = files_actions(unit)

  if(debug) then
   write (logf,*)trim(routines_21),'-d: file-name >',trim(f_name),'<'
   write (logf,*)trim(routines_21),'-d: file-action >',trim(f_action),'<'
  end if

  unit = file_open_and_cat (f_name, f_type, f_action)

  if(debug) then
   write (logf,*)trim(routines_21),'-d: at exit unit=',unit 
  end if

  result = unit

  call exit (l_here)

  end function file_open_and_cat_from_type

! ===============================================================================
  subroutine file_uncatalogue (unit)
! -------------------------------------------------------------------------------
!
! Name          : file_uncatalogue
!
! Argument (in ): unit 
!
! Description   : all catalogue items are re-set      
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
! character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
! character(len=max_string_len_format)       :: f_format
! character(len=max_string_len_file_name)    :: f_path
! character(len=max_string_len_position)     :: f_position
! character(len=max_string_len_program)      :: f_program
! character(len=max_string_len_title)        :: f_title
! character(len=max_string_len_status)       :: f_status
! logical(bool)                              :: f_is_open

! begin:
  l_here ='file_uncatalogue'
  call enter (l_here)
  
  if(debug) then 
    write (logf,*)trim(l_here),'-d: file-unit =',unit
  end if

! unit not zero  
  message = l_here+'-e: file-unit = '+unit
  call require (message, unit /= 0)

! f_action = files_actions(unit)
! f_format = files_formats(unit)
  f_name = files_names(unit)
  f_type = files_types(unit)
! f_path = files_paths(unit)
! f_position = files_positions(unit)
! f_program = files_programs(unit)
! f_title = files_titles(unit)
! f_status = files_status(unit)
! f_is_open = files_open_is(unit)

  write (logf,*)trim(l_here),': file >',trim(f_name), &
         '< type >',trim(f_type), &
         '< unit ',unit,' has been uncatalogued'
   

  files_actions(unit)   =  blanks
  files_formats(unit)   =  blanks
  files_names(unit)     =  blanks
  files_types(unit)     =  blanks 
  files_paths(unit)     =  blanks
  files_positions(unit) =  blanks
  files_programs(unit)  =  blanks  
  files_titles(unit)    =  blanks 
  files_status(unit)    =  blanks 

  files_open_is(unit)   =  .false. 
  files_names(unit)(1:1) =  NOT_INIT_S
  files_types(unit)     =  'free'

  call exit (l_here)

  end subroutine file_uncatalogue

! ===============================================================================
  subroutine files_init
! -------------------------------------------------------------------------------
!
! Name        : files_init
!
! Description : initialize all file management arrays

! Description : two cases:
! Description : qmcmol.exe < t.inp                      
! Description :      in  file is stdin   files_names_in = NOT_INIT_S
! Description :      log file is stdout  (screen)
! Description :      out file is stdout  (screen)
! Description :      err file is stderr  in qmcmol.err   
! Description :      res file is results in qmcmol.res   

! Description : qmcmol.exe   t.inp                      
! Description :      in  file is stdin   files_names_in = t.inp      
! Description :      log file is stdout  in t.log 
! Description :      out file is stdout  in t.log 
! Description :      err file is stderr  in t.err   
! Description :      res file is results in t.res   
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! Revision: 24 Feb 2001 F.Colonna added res = 9
! Revision: 10 Apr 2002 F.Colonna added rtn = 7 dpn =8
! Revision: 16 DEc 2002 F.Colonna results with -r command
! -------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: file_i

! begin:
  l_here ='files_init'
  call enter (l_here)

  call allocation ('files_actions',   files_actions,   max_files)
  call allocation ('files_names',     files_names,     max_files)
  call allocation ('files_types',     files_types,     max_files)
  call allocation ('files_formats',   files_formats,   max_files)
  call allocation ('files_paths',     files_paths,     max_files)
  call allocation ('files_positions', files_positions, max_files)
  call allocation ('files_programs',  files_programs,  max_files)
  call allocation ('files_titles',    files_titles,    max_files)
  call allocation ('files_status',    files_status,    max_files)
  call allocation ('files_open_is',   files_open_is,   max_files)

  files_actions   = blanks
  files_names     = blanks
  files_types     = blanks
  files_formats   = blanks
  files_paths     = blanks
  files_positions = blanks
  files_programs  = blanks
  files_titles    = blanks
  files_status    = blanks

  files_types ='free'
  do file_i = 1,9
    files_types(file_i)     = 'neverused'
    files_programs(file_i)  = 'qmcmol'
  end do
  
! input.
  files_names(in )      = files_names_in 
  files_types(in )      = 'stdin   '
  if ( files_names_in  /= NOT_INIT_S) then
    files_paths(in)     = './      '
    files_status(in)    = 'old     '
    files_types(in)     = 'stdin   '         
    files_actions(in)   = 'read'
    files_positions(in) = 'asis    '

    call unit_open_read (in)
  end if

! output.
! if ( files_names_out == NOT_INIT_S) then
!   if (files_names_in /= NOT_INIT_S) then
!     files_names_out   = trim(file_name_basename (files_names_in))//".out"
!     files_names(out)  = files_names_out
!     call unit_open_write (out)
!   end if
! end if

! if(debug) then
!   write(*,*) trim(l_here),'-d: files_names_out >',trim(files_names_out),'<'
!   write(*,*) trim(l_here),'-d: files_names (',out,') = >',trim(files_names_out),'<'
! end if

! logf.
  files_actions(logf)   = 'write   '
  files_paths(logf)     = './      '
  files_positions(logf) = 'asis    '
  files_status(logf)    = 'new     '
  files_types(logf)     = 'stdout'         

  if ( files_names_logf == NOT_INIT_S) then
    if (files_names_in  /= NOT_INIT_S) then
      files_names_logf   = trim(file_name_basename (files_names_in))//".log"
      files_names(logf)  = files_names_logf

! untill here we are in stdout
! open log file
      call unit_open_write (logf)
    end if
  end if
  
  if(debug) then
    write(*,*) trim(l_here),'-d: files_names_logf >',trim(files_names_logf),'<'
    write(*,*) trim(l_here),'-d: files_names (',logf,') = >',trim(files_names_logf),'<'
  end if

! starts log file. -------------------------------------- log file 
!
! to disable somme printing outside the log file when the program starts
! print_level is set to -1 in control_mod 

  if (print_level == -1) print_level = 0

  call file_header_write (logf)

! errors file. unit=err
  files_actions(err)   = 'write   '
  files_paths(err)     = './      '
  files_positions(err) = 'asis    '
  files_status(err)    = 'unknown'
  files_types(err)     = 'stderr'

  if ( files_names_err == NOT_INIT_S) then
    if (files_names_in == NOT_INIT_S) then
      files_names_err   = trim(file_name_basename (exec_name))//".err"
    else
      files_names_err   = trim(file_name_basename (files_names_in))//".err"
    end if
  end if
  files_names(err)      = files_names_err

  if(debug) then
    write(*,*) trim(l_here),'-d: files_names_err >',trim(files_names_err),'<'
    write(*,*) trim(l_here),'-d: files_names (',err,') = >',trim(files_names_err),'<'
  end if

  call unit_open_write (err)
  call file_header_write (err)

! unit=rtn
  files_actions(rtn)   = 'read    '
  files_names(rtn)     = 'routines_names.inc'
  files_paths(rtn)     = './      '
  files_positions(rtn) = 'asis    '
  files_status(rtn)    = 'old '
  files_types(rtn)     = 'routines_names-in'

! unit=dpn
  files_actions(dpn)   = 'read    '
  files_names(dpn)     = 'routine-routine-depend.inc'
  files_paths(dpn)     = './      '
  files_positions(dpn) = 'asis    '
  files_status(dpn)    = 'old '
  files_types(dpn)     = 'dependences_list-in'

! results unit=res
  if ( files_names_res == NOT_INIT_S) then
    if (files_names_in == NOT_INIT_S) then
      files_names_res   = trim(file_name_basename (exec_name))//".res"
    else
      files_names_res   = trim(file_name_basename (files_names_in))//".res"
    end if
  end if
  files_names(res)      = files_names_res

  if(debug) then
    write(*,*) trim(l_here),'-d: files_names_res >',trim(files_names_res),'<'
    write(*,*) trim(l_here),'-d: files_names (',res,') = >',trim(files_names_res),'<'
  end if

  files_actions(res)   = 'write   '
  files_paths(res)     = './      '
  files_positions(res) = 'append  '
  files_types(res)     = 'results '
  files_status(res)    = 'unknown '

  call unit_open_append (res)
  call file_header_write (res)

! just for cosmetics. it is also true
  if(files_names_in == NOT_INIT_S) files_names_in = 'stdin'
  if(files_names_logf == NOT_INIT_S) files_names_logf = 'stdout'

  write (logf,*) trim(l_here),': files connections:'
  write (logf,*) trim(l_here), &
  ': log file     (unit ',logf,') is connected to >',trim(files_names_logf),'<'
  write (logf,*) trim(l_here), &
  ': input file   (unit ',in  ,') is connected to >',trim(files_names_in),'<'
  write (logf,*) trim(l_here), &
  ': errors file  (unit ',err ,') is connected to >',trim(files_names_err),'<'
  write (logf,*) trim(l_here), &
  ': results file (unit ',res ,') is appended  to >',trim(files_names_res),'<'
  write (logf,*) ' '

  call exit (l_here)

  end subroutine files_init

! ===============================================================================
  function file_sample (unit) result(result) 
! -------------------------------------------------------------------------------
!
! Name          : file_sample 
!
! Argument (in) : unit
!
! Description   : samples the number of lines in a file
! Description   : header must be read first if any
!
! Authors       : F. Colonna
! Date          : 21 Mar 2001
! -------------------------------------------------------------------------------
  implicit none
   
! i/o:
  integer(dpi)                               :: unit

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len= 1)                           :: record
  integer(dpi)                               :: l_count
  integer(dpi)                               :: iostat

! begin:
  l_here ='file_sample'
  call enter (l_here)

  if(debug) then
    write (logf,*)trim(l_here),'-d: sampling file >',trim(files_names(unit)), &
          '< on unit ',unit
  end if
  
  message ='in routine'*l_here*' unit'+unit+' must be in [1,99]'
  call require (message, unit>0.and.unit<100)
  
  message ='in routine'*l_here*' file '*files_names(unit)*' must be opened'
  call require (message, files_open_is(unit))

  l_count = 0
  iostat  = 0 
  do while (iostat== 0)
    read(unit,'(a)',iostat=iostat) record
    if (debug)write (logf,*)trim(l_here),'-d: record >',trim(record),'<'
    if (iostat /= 0) exit
    l_count = l_count+1
  enddo

  result = l_count

  rewind unit

  if(l_count == 0) then
  write (logf,*)trim(l_here),': file >',trim(files_names(unit)),'< is empty'
  end if

  write (logf,*)trim(l_here),': file >',trim(files_names(unit)), &
          '< rewinded on unit ',unit

  if(debug) write (logf,*)trim(l_here),'-d: nb of records = ',l_count 

  call exit (l_here)

  end function file_sample

! ===============================================================================
  function unit_from_name (f_name) result(result) 
! -------------------------------------------------------------------------------
!
! Name           : unit_from_name
!
! Argument (in ) : f_name files_types(res) = 'stdres  '
!
! Description    : returns (first found) catalogue number from file name or 0
! Remark         : this can be dangereous. better to use unit_from_type
!
! Authors:F.Colonna
! Date  :13 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_name

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit

! begin:
  l_here ='unit_from_name'
  call enter (l_here, 2)

  if (debug) &
  write (logf,'(4a)') trim(l_here),': name >',trim(f_name),'< '

  call check_association &
  ('files_names', files_names, 'files_init', files_init)

! get unit 
  unit = string_in_array_locate (f_name, files_names, 1, size(files_names))
  if (unit .eq. 0) then
   message = 'file name >'+f_name+'< not found in catalogue:'
   if(warning_level .ge. 1) call warning (l_here, message)
   if(print_level .ge. 5) call files_show
  end if

  result = unit

  if(print_level  >  0) then 
   write (logf,'(4a,i3)') routines_21, &
   ': file >',trim(files_names(unit)),'< catalogued # ',unit
   write (logf,'(4a)') routines_21,': with name >',f_name,'<'
   write (logf,'(4a)') routines_21,': with type >', &
   trim(files_types(unit)),'<'
  end if

  call exit (l_here, 2)

  end function unit_from_name

! ===============================================================================
  function unit_from_type (f_type) result(result) 
! -------------------------------------------------------------------------------
!
! Name           : unit_from_type
!
! Argument (in ) : f_type 
!
! Description    : returns catalogue number from file type or 0
! 
! Revision:F.Colonna 27 May 2001 and 30 Oct 2002 include case handled
! Authors:F.Colonna
! Date  :13 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)               :: f_type

! result:
  integer(dpi)                               :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: f_name, l_tail
  integer(dpi)                               :: unit
  logical(bool)                              :: is_extension_ok

! begin:
  l_here ='unit_from_type'
  call enter (l_here, 2)

! FCC 18 jan 2001  l_tail = '-'//string_tail (f_type, '-', back=.true.)
  l_tail = string_tail (f_type, '-', back = .true.)

  if (debug) then
  write (logf,*)trim(routines_21),': file-type >',trim(f_type),'<'
  write (logf,*)trim(l_here),': l_tail >',trim(l_tail),'<'
  end if

  is_extension_ok = trim(f_type) == "include" .or.        &
                    len_trim(l_tail) /= 0 .and.          &
                    verify (trim(l_tail), "-in-out-inout") .eq. 0

  if ( .not. is_extension_ok) then
    message ='in routine'*l_here*' file-type >'+f_type+'<' &
             +' has no -in -out -inout extension'
    call require (message, is_extension_ok)
  end if

  call check_association &
  ('files_types', files_types, 'files_init', files_init)

! get unit 
  unit = string_in_array_locate (f_type, files_types, 1, size(files_types))

  if(debug) then
   write (logf,*)trim(routines_21),'-d: unit=',unit
  end if

  if(unit .eq. 0) then
   message = 'file-type >'+f_type+'< not found in catalogue:'
   if(warning_level .ge. 10) call warning (l_here, message)
   if(print_level   .ge. 10) call files_show
  else
   f_name = files_names(unit)
   if(debug) then
    write (logf,*)routines_21,'-d: file-name >',trim(f_name),'<'
   end if
  end if

  result = unit

  if(debug) then 
   write (logf,*)trim(routines_21), &
   '-d: file-type >',trim(f_type),'< catalogued # ',unit
  end if

  call exit (l_here, 2)

  end function unit_from_type

! ===============================================================================
  function unit_inquire_open (unit) result(result) 
! -------------------------------------------------------------------------------
!
! Name          : unit_inquire_open
!
! Argument (in) : unit
!
! Result        : file name is opened. NOT_INIT_S if not
!
! Description   : inquires if unit opened returns file name or NOT_INIT_S
!
! Authors:F.Colonna
! Date  :21 Dec 2002
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! result:
  character(len=max_string_len_file_name)    :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: f_name 
  logical(bool)                              :: l_opened

! begin:
  l_here ='unit_inquire_open'
  call enter (l_here)

  inquire (unit = unit, name = f_name, opened = l_opened)

  if(l_opened .and. f_name /= "" ) then
    result = f_name  
  else
    result = NOT_INIT_S
  end if

  if(debug) then
   write (logf,*)trim(routines_21),'-d: unit = ',unit
   write (logf,*) trim(routines_21),'-d: opened ? =',l_opened
   write (logf,*) trim(routines_21),'-d: file name >',trim(result),'<'
  end if

  call exit (l_here)

  end function unit_inquire_open

! ===============================================================================
  subroutine files_print_res
! -------------------------------------------------------------------------------
!
! Name           : files_print_res
!
! Argument (in ) : text
!
! Description    : prints in result file all catalogue item for a all units
!
! Author      : F. Colonna
! Date        :27 Jan 2003
! -------------------------------------------------------------------------------
!
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string
  character(len=max_string_len)              :: text
  integer(dpi)                               :: fil_i, debut, fin
  integer(dpi)                               :: files_nb

! begin:
  l_here ='files_print_res'
  call enter (l_here)
  
  files_nb = size(files_types)

  text = "files catalogue"
  debut  = 40 - (len_trim(text)/2)
  fin    = 40 + (len_trim(text)/2)

  if(debug) then
    write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
    write (logf,*) trim(l_here),'-d: debut=', debut,' fin=',fin
    write (logf,*) trim(l_here),'-d: size(files_types)=',size(files_types)
  end if

  string                 = dashes
  string (debut-1:debut) = " "
  string (debut:)        = trim(text)//' '//dashes

  write (res,'(a)') ' '
  write (res,'(a80)') string
  write (res,'(a)') ' '

! message to log file
  write (logf,'(1x,2a,a80)') trim(l_here),': ',string

  write(res,'(a, t32,a)') 'file# type','name'

  do fil_i = 1, files_nb

   if (trim(files_types(fil_i)) /= "free") then

     write(res,'(i5,1x,a,1x,a60)') &
                 fil_i,files_types(fil_i)(1:24),  & 
                       files_names(fil_i)(1:60)
   end if
  end do

  call exit (l_here)

  end subroutine files_print_res

! ===============================================================================
  subroutine files_show
! -------------------------------------------------------------------------------
!
! Name        : files_show
!
! Description : prints all catalogue item for a all units
!
! Author      : F. Colonna
! Date        :08 Oct 2000
! -------------------------------------------------------------------------------
!
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: unit

! begin:
  l_here ='files_show'
  call enter (l_here)
  
  if(debug) then
   write (logf,*)trim(l_here),'-d: size(files_types)=',size(files_types)
  end if

  write (logf,*)trim(l_here),': catalogued files:'
  do unit = 1, size(files_types)
   if(trim(files_types(unit)) /= "free") then
    write (logf,'(1x,2a,i3,5a)') trim(l_here),': file unit #',unit,' type >',  &
    trim(files_types(unit)),'< name >',trim(files_names(unit)),'<'
   end if
  end do

  call exit (l_here)

  end subroutine files_show

! ===============================================================================
  subroutine unit_open (unit)
! -------------------------------------------------------------------------------
!
! Name          : unit_open
! Argument (in) : unit
!
! Description    : opens unit unit from catalogue information
!
! Authors: F.Colonna
! Date   : 21 Nov 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: file           

  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_format)       :: f_format
  character(len=max_string_len_file_name)    :: f_path
  character(len=max_string_len_position)     :: f_position
  character(len=max_string_len_program)      :: f_program
  character(len=max_string_len_title)        :: f_title
  character(len=max_string_len_status)       :: f_status
  logical(bool)                              :: f_is_open

! begin:
  l_here ='unit_open'
  call enter (l_here)

  f_action = files_actions(unit)
  f_format = files_formats(unit)
  f_name = files_names(unit)
  f_type = files_types(unit)
  f_path = files_paths(unit)
  f_position = files_positions(unit)
  f_program = files_programs(unit)
  f_title = files_titles(unit)
  f_status = files_status(unit)
  f_is_open = files_open_is(unit)

  if(debug) then
   write (logf,*) trim(l_here),'-d: file-path >',trim(f_path),'<'
   write (logf,*) trim(l_here),'-d: file-status >',trim(f_status),'<'
   write (logf,*) trim(l_here),'-d: file-action >',trim(f_action),'<'
   write (logf,*) trim(l_here),'-d: file-format >',trim(f_format),'<'
   write (logf,*) trim(l_here),'-d: file-position >',trim(f_position),'<'
   write (logf,*) trim(l_here),'-d: file-program >',trim(f_program),'<'
   write (logf,*) trim(l_here),'-d: file-title >',trim(f_title),'<'
   write (logf,*) trim(l_here),'-d: unit =',unit
   write (logf,*) trim(l_here),'-d: f_is_open =',f_is_open
  end if

  if(debug) then 
   write (logf,'(4a)')trim(l_here),': file >',trim(f_name),'< '
   write (logf,'(4a)')trim(l_here),': type >',trim(f_type),'< '
  end if

  file = file_name_with_path (f_name, f_path)

  open (file = file, unit = unit, iostat = iostat)

  if (iostat /= 0) then 
   write (logf,'(4a,i2,a,i2)') &
   trim(l_here),': open failed for file >',trim(file), &
   '< on unit ',unit,' iostat=',iostat
   message = 'open failed while trying to open file. iostat='+ &
   iostat
   call die (l_here, message)
  end if

  files_open_is(unit) = .true.

  if(debug .or. print_level  >  10) then 
   write (logf,'(4a,i2,3a)') trim(l_here),': file >', &
   trim(f_name), '< opened in write mode on unit ',unit, &
                  ' with type >',trim(f_type),'<' 
  end if

  call exit (l_here)

  end subroutine unit_open

! ===============================================================================
  subroutine unit_open_append (unit)
! -------------------------------------------------------------------------------
!
! Name           : unit_open_append
!
! Argument (in ) : unit
!
! Description    : opens unit unit from catalogue information 
! Description    : in append mode
!
! Authors:F.Colonna
! Date  :29 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: file           

  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_format)       :: f_format
  character(len=max_string_len_file_name)    :: f_path
  character(len=max_string_len_position)     :: f_position
  character(len=max_string_len_program)      :: f_program
  character(len=max_string_len_title)        :: f_title
  character(len=max_string_len_status)       :: f_status
  logical(bool)                              :: f_is_open

! begin:
  l_here ='unit_open_append'
  call enter (l_here)

  f_action = files_actions(unit)
  f_format = files_formats(unit)
  f_name = files_names(unit)
  f_type = files_types(unit)
  f_path = files_paths(unit)
  f_position = files_positions(unit)
  f_program = files_programs(unit)
  f_title = files_titles(unit)
  f_status = files_status(unit)
  f_is_open = files_open_is(unit)

  if(debug) then
   write (logf,*) trim(l_here),'-d: (in) unit =',unit
   write (logf,*) trim(l_here),'-d: file-action >',trim(f_action),'<'
   write (logf,*) trim(l_here),'-d: file-format >',trim(f_format),'<'
   write (logf,*) trim(l_here),'-d: file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(f_path),'<'
   write (logf,*) trim(l_here),'-d: file-position >',trim(f_position),'<'
   write (logf,*) trim(l_here),'-d: file-program >',trim(f_program),'<'
   write (logf,*) trim(l_here),'-d: file-status >',trim(f_status),'<'
   write (logf,*) trim(l_here),'-d: file-type >',trim(f_type),'<'
   write (logf,*) trim(l_here),'-d: file-title >',trim(f_title),'<'
   write (logf,*) trim(l_here),'-d: f_is_open =',f_is_open
  end if

  if(trim(f_type) /= "results") then
    write (logf,*)trim(l_here),': file >',trim(f_name),'< '
    write (logf,*)trim(l_here),': type >',trim(f_type),'< '
  end if

  if(f_status(1:1) == '?') f_status='unknown'
  if(f_action(1:1) == '?') f_action='write'
  if(f_position(1:1) == '?') f_position='append'
  if(trim(f_action) == 'append') f_action ='write'

  file = file_name_with_path (f_name, f_path)

  open (file = file, unit = unit, action = f_action, &
        position = f_position, status = f_status, iostat = iostat)

  files_open_is(unit) = .true.

  if (iostat /= 0) then 
   write (logf,'(4a,i2,a,i3)') &
   trim(l_here),': open failed for file >',trim(file), &
   '< on unit ',unit,' iostat=',iostat
   message = 'open failed while trying to open-append file. iostat='+iostat
   call die (l_here, message)
  end if

  if(print_level  >  0) then 
   write (logf,'(4a,i2,3a)') trim(l_here),': file >', &
   trim(f_name), '< opened in write mode on unit ',unit, &
                  ' with type >',trim(f_type),'<' 
  end if

  call exit (l_here)

  end subroutine unit_open_append

! ===============================================================================
  subroutine unit_open_read (unit)
! -------------------------------------------------------------------------------
!
! Name           : unit_open_read
!
! Argument (in ) : unit
!
! Description    : opens unit unit from catalogue information 
! Description    : in read mode
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: exist

  character(len=max_string_len_file_name)    :: file 

  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name, l_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_format)       :: f_format
  character(len=max_string_len_file_name)    :: f_path
  character(len=max_string_len_position)     :: f_position
  character(len=max_string_len_program)      :: f_program
  character(len=max_string_len_title)        :: f_title
  character(len=max_string_len_status)       :: f_status
  integer(dpi)                               :: l_unit
  logical(bool)                              :: f_is_open

! begin:
  l_here ='unit_open_read'
  call enter (l_here, 2)

  f_action = files_actions(unit)
  f_format = files_formats(unit)
  f_name = files_names(unit)
  f_type = files_types(unit)
  f_path = files_paths(unit)
  f_position = files_positions(unit)
  f_program = files_programs(unit)
  f_title = files_titles(unit)
  f_status = files_status(unit)
  f_is_open = files_open_is(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: file >',trim(f_name),'< '
   write (logf,*)trim(l_here),'-d: path >',trim(f_path),'< '
   write (logf,*)trim(l_here),'-d: type >',trim(f_type),'< '
   write (logf,*)trim(l_here),'-d: f_action >',trim(f_action),'< '
   write (logf,*)trim(l_here),'-d: f_format >',trim(f_format),'< '
   write (logf,*)trim(l_here),'-d: f_position >',trim(f_position),'< '
   write (logf,*)trim(l_here),'-d: f_program >',trim(f_program),'< '
   write (logf,*)trim(l_here),'-d: f_title >',trim(f_title),'< '
   write (logf,*)trim(l_here),'-d: f_status >',trim(f_status),'< '
   write (logf,*)trim(l_here),'-d: f_is_open =',f_is_open
  end if

! file must exist
  exist   = file_exists (f_name, f_path)

  if (.not. exist) then
    message = 'in routine'*l_here*CR &
              *'file >'+f_name+'< does not exist on disk'
    call require (message, exist)
  end if

! f_action(1:4) == "read"       
  if(f_action(1:4) /= "read") then
    message ='in routine'*l_here+'-error:'+CR+ &
            l_here+'-error: file >'+ &
            f_name+'< action is >'+f_action+&
            '< must be read or readwrite'
    call require (message, f_action(1:4) == "read")
  end if

! check if file already opened.
   l_unit = file_inquire_open (f_name, f_path)
   if( l_unit /= 0) then
    write (logf,*)trim(l_here),'-w: file >',trim(f_name),'< already open'
    write (logf,*)trim(l_here),'-w: with unit =',l_unit,' and type >', &
            trim(files_types(l_unit)),'<'
    write (logf,*)trim(l_here),'-w: while current unit=',unit,' has type >', &
            trim(files_types(unit)),'<'
    if(unit /= l_unit) then
      call die (l_here, 'file was already open with an other unit')
    end if
   end if

! check if status given        
  if (f_status(1:1) == ' ' .or. f_status == "") then
   f_status = 'old'
   files_status(unit) = f_status
   if (warning_level  >  1) write (logf,*) trim(l_here), &
     '-w: unknown file-status set to >',trim(f_status),'<'
  end if

! check if position given        
  if (f_position(1:1) == ' ' .or. f_position == "") then
   f_position = 'asis'
   files_positions(unit) = f_position
   if (warning_level  >  1) write (logf,*) trim(l_here), &
     '-w: unknown file-position set to >',trim(f_position),'<'
  end if

  file = file_name_with_path (f_name, f_path) 
  if(debug) then
   write (logf,*) trim(l_here),'-d: full-name >',trim(file),'<'
   write (logf,*) trim(l_here),'-d: f_action >',trim(f_action),'<'
   write (logf,*) trim(l_here),'-d: f_status >',trim(f_status),'<'
   write (logf,*) trim(l_here),'-d: f_position >',trim(f_position),'<'
   write (logf,*) trim(l_here),'-d: unit =',unit
  end if

! is unit already opened?
  l_name = unit_inquire_open (unit)
  if (l_name == NOT_INIT_S) then
    open (file = file, unit = unit, action = f_action, &
          position = f_position, status = f_status, iostat = iostat)

  else 
    write (logf,*) trim(l_here),': unit ',unit,' already opened on file >', &
                  trim(l_name),'<'
    if (l_name /= file) then
      message = 'in routine'*l_here*CR &
               *'file >'+file+'< not opened on unit '*unit
      call require (message, l_name == file)
    end  if
  end if

  files_open_is(unit) = .true.

  if(debug) then
   write (logf,*) trim(l_here),'-d: f_is_open =',f_is_open
   write (logf,*) trim(l_here),'-d: iostat =',iostat
  end if

  if (iostat /= 0) then 
   write (logf,*) trim(l_here),'-e: ', &
   trim(l_here),': open failed for file >',trim(file), &
   '< on unit ',unit,' iostat=',iostat
   message = 'open failed while trying to open file. iostat='+ &
   iostat
   call die (l_here, message)
  end if

  if(print_level >= 0) then 
   write (logf,*) trim(l_here),': file >',trim(f_name),'< opened on unit ',unit, &
                         ' in readonly mode '
   write (logf,*) trim(l_here),': with type >',trim(f_type),'<' 
  end if

  call exit (l_here, 2)

  end subroutine unit_open_read

! ===============================================================================
  subroutine unit_open_readwrite (unit)
! -------------------------------------------------------------------------------
!
! Name           : unit_open_readwrite
! Argument (in ) : unit
!
! Description    : opens unit unit from catalogue information 
! Description    : in readwrite mode
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: file           
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_format)       :: f_format
  character(len=max_string_len_file_name)    :: f_path
  character(len=max_string_len_position)     :: f_position
  character(len=max_string_len_program)      :: f_program
  character(len=max_string_len_title)        :: f_title
  character(len=max_string_len_status)       :: f_status
  logical(bool)                              :: f_is_open              

! begin:
  l_here ='unit_open_readwrite'
  call enter (l_here, 2)

  f_action = files_actions(unit)
  f_format = files_formats(unit)
  f_name = files_names(unit)
  f_type = files_types(unit)
  f_path = files_paths(unit)
  f_position = files_positions(unit)
  f_program = files_programs(unit)
  f_title = files_titles(unit)
  f_status = files_status(unit)
  f_is_open = files_open_is(unit)

  write (logf,'(4a)')trim(l_here),': file >',trim(f_name),'< '
  write (logf,'(4a)')trim(l_here),': type >',trim(f_type),'< '

  if(debug) then
   write (logf,'(4a)')trim(l_here),': file >',trim(f_name),'< '
   write (logf,'(4a)')trim(l_here),': path >',trim(f_path),'< '
   write (logf,'(4a)')trim(l_here),': type >',trim(f_type),'< '
   write (logf,'(4a)')trim(l_here),': f_action >',trim(f_action),'< '
   write (logf,'(4a)')trim(l_here),': f_format >',trim(f_format),'< '
   write (logf,'(4a)')trim(l_here),': f_position >',trim(f_position),'< '
   write (logf,'(4a)')trim(l_here),': f_program >',trim(f_program),'< '
   write (logf,'(4a)')trim(l_here),': f_title >',trim(f_title),'< '
   write (logf,'(4a)')trim(l_here),': f_status >',trim(f_status),'< '
   write (logf,'(4a)')trim(l_here),': f_is_open =',f_is_open
  end if

! check if file already open.
  if (f_is_open) then
   write (logf,'(4a)') &
   trim(l_here),': file >',trim(f_name),'< already opened'
  end if

  file = file_name_with_path (f_name, f_path)

  open (file = file, unit = unit, action = f_action, &
        position = f_position, status = f_status, iostat = iostat)

  files_open_is(unit) = .true.

  if (iostat /= 0) then 
   write (logf,'(4a,i2,a,i2)') &
   trim(l_here),': open failed for file >',trim(file), &
   '< on unit ',unit,' iostat=',iostat
   message = 'open failed while trying to open file. iostat='+ &
   iostat
   call die (l_here, message)
  end if

  if(print_level >= 0) then 
   write (logf,*) trim(l_here),': file >',trim(f_name),'< opened on unit ',unit, &
                         ' in readwrite mode '
   write (logf,*) trim(l_here),': with type >',trim(f_type),'<' 
  end if

  call exit (l_here, 2)

  end subroutine unit_open_readwrite

! ===============================================================================
  subroutine unit_open_write (unit)
! -------------------------------------------------------------------------------
!
! Name           : unit_open_write
! Argument (in ) : unit
!
! Description    : opens unit unit from catalogue information 
! Description    : in write mode
!
! Authors:F.Colonna
! Date  :14 Oct 2000
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_name)    :: file           

  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_format)       :: f_format
  character(len=max_string_len_file_name)    :: f_path
  character(len=max_string_len_position)     :: f_position
  character(len=max_string_len_program)      :: f_program
  character(len=max_string_len_title)        :: f_title
  character(len=max_string_len_status)       :: f_status
  logical(bool)                              :: f_is_open
  logical(bool)                              :: l_opened
  logical(bool)                              :: l_exists
  integer(dpi)                               :: l_unit

! begin:
  l_here ='unit_open_write'
  call enter (l_here, 2)

  f_action   = files_actions(unit)
  f_format   = files_formats(unit)
  f_name     = files_names(unit)
  f_type     = files_types(unit)
  f_path     = files_paths(unit)
  f_position = files_positions(unit)
  f_program  = files_programs(unit)
  f_title    = files_titles(unit)
  f_status   = files_status(unit)
  f_is_open  = files_open_is(unit)

  if(debug) then
   write (logf,*) trim(l_here),'-d: unit = ',unit
   write (logf,*) trim(l_here),'-d: file-path >',trim(f_path),'<'
   write (logf,*) trim(l_here),'-d: file-format >',trim(f_format),'<'
   write (logf,*) trim(l_here),'-d: file-program >',trim(f_program),'<'
   write (logf,*) trim(l_here),'-d: file-title >',trim(f_title),'<'
   write (logf,*) trim(l_here),'-d: f_is_open =',f_is_open
  end if

! require_unit:
  if(unit <= 0) then
    call die (l_here, ' unit ='+unit)
  end if

  if (f_name == NOT_INIT_S) then
    message = 'in routine'*l_here*CR &
             *' file-type >'+f_type+'< has undefined file name'
    call require (message, f_name /= NOT_INIT_S) 
  end if

  file     = file_name_with_path (f_name, f_path)

  l_exists = file_exists (f_name, f_path)

  if(debug) then
   write (logf,*) trim(l_here),'-d: file >',trim(file),'<'
   write (logf,*) trim(l_here),'-d: unit =',unit
   write (logf,*) trim(l_here),'-d: file-name >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: type >',trim(f_type),'< '
   write (logf,*) trim(l_here),'-d: file-action >',trim(f_action),'<'
   write (logf,*) trim(l_here),'-d: file-position >',trim(f_position),'<'
   write (logf,*) trim(l_here),'-d: file-status >',trim(f_status),'<'

   inquire (file = f_name, number = l_unit, opened = l_opened) 
   write (logf,*)trim(l_here),'-d: inquire => opened ?=',l_opened,' unit=',l_unit
   if(debug) write (logf,*)trim(l_here),'-d: file exist =',l_exists
  end if

! require_file:
  if(len_trim(file) <= 0) call die (l_here, 'file name empty')

! require_file:
  if(f_status == "new" .and. l_exists) then 
    message = 'in routine'*l_here &
              *CR*'status = new for existing file >'+f_name+'<' &
              *CR*'delete it by typing' &
              *CR*CR*'rm '*f_name*CR
    call require (message, .not. (f_status == "new" .and. l_exists) )
  end if

  open (file = file, unit = unit, action = f_action, &
        position = f_position, status = f_status, iostat = iostat)

  files_open_is(unit) = .true.

  if (iostat /= 0) then 
   write(err,*) trim(l_here),'-e: file >',trim(file),'<'
   write(err,*) trim(l_here),'-e: file-position >',trim(f_position),'<'
   write(err,*) trim(l_here),'-e: file-action >',trim(f_action),'<'
   write(err,*) trim(l_here),'-e: file-status >',trim(f_status),'<'
   write(err,*) trim(l_here),'-e: unit        =',unit
   write(err,*) trim(l_here),'-e: iostat      =',iostat

   message = 'in routine'*l_here*CR &
             *'open failed with iostat='*iostat &
             *'while trying to open file >'+file+'<'
   call require (message, iostat == 0)

  end if

  if(debug .or. print_level  >  10) then 
   write (logf,*) trim(l_here),': file >',trim(f_name),'< opened on unit ',unit, &
                         ' in write mode '
   write (logf,*) trim(l_here),': with type >',trim(f_type),'<' 
  end if

  call exit (l_here, 2)

  end subroutine unit_open_write

end module files_mod
