! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module force_bywlk_mod

  use basic_types_tools_mod
  use drift_bywlk_mod
  use e_loc_kin_bywlk_mod
  use e_loc_vne_bywlk_mod
  use e_loc_vee_bywlk_mod
  use e_loc_vnn_mod
  use ecp_bywlk_mod
  use e_loc_tot_bywlk_mod
!
  use parser_mod
!
  implicit none

  real(dp), pointer :: Q_grad_bywlk (:,:,:) => NULL ()
  real(dp), pointer :: atoms_coor_vector (:,:) => NULL ()
  real(dp), pointer :: delta_elec_coor_bywlk (:,:,:) => NULL ()
  real(dp), pointer :: delta_jacob_bywlk (:) => NULL ()
  real(dp), pointer :: one_ea_distance (:,:) => NULL ()
  real(dp), pointer :: epsilon_grid (:) => NULL ()
  real(dp), pointer :: delta_exp1 => NULL ()
  real(dp), pointer :: delta_exp2 => NULL ()  
  real(dp), pointer :: delta_exp3 => NULL ()
  real(dp), pointer :: delta_exp4 => NULL ()
  real(dp), pointer :: delta_exp5 => NULL ()  
  real(dp), pointer :: exp_factor => NULL ()
  real(dp)          :: grid_size, initial_point
  integer(dpi)      :: grid_points
  character(len=30) :: warp_coor_type
  real(dp)          :: geom_dist_2mos ! for moving the mos
  logical(bool)     :: mos_second_moved =.false.
  real(dp)          :: geom_finite_diff = 0.00001_dp

! simplifications of the local energy near the core for all electron calculations
  real(dp), pointer :: el_s_byat_bywlk (:,:) => NULL ()
  real(dp), pointer :: el_s_der_bycoor_byat_bywlk (:,:,:) => NULL ()  
  contains

! ========================================================================
  subroutine cutoff_menu
! ------------------------------------------------------------------------
!
! Name           : cutoff_menu
!
! Author         : Alexander Kollias
! Date           : 
! ------------------------------------------------------------------------
  implicit none

! local:
  logical(bool)                              :: done
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='cutoff_menu'
  call enter (l_here)

! initialize:
  done = .false.

  call next_command_get ('cutoff')
  do while (.not.done)
   call next_word_get('cutoff')
   if(trim(cur_word) .eq. 'help') then
    write (logf,*) 'cutoff-h: comments:'
    write (logf,*) 'cutoff-h: cutoff'
    write (logf,*) 'cutoff-h:   help'
    write (logf,*) 'cutoff-h:   end_cutoff'
    write (logf,*) 'cutoff: example'
    write (logf,*) 'cutoff-h:      end_cutoff'

   else if(token_is ('grid_points', grid_points))   then

   else if(token_is ('grid_size', grid_size)) then

   else if(token_is ('initial_point', initial_point)) then

   else
      call menu_exit (l_here, done)
   end if

  end do
 
  end subroutine cutoff_menu

! ========================================================================
  subroutine force_menu
! ------------------------------------------------------------------------
!
! Name           : force_menu
!
! Author         : Alexander Kollias
! Date           : 
! ------------------------------------------------------------------------
  implicit none

! local:
  logical(bool)                              :: done
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='force_menu'
  call enter (l_here)

! initialize:
  done = .false.
  
  call next_command_get ('forces')
  do while (.not.done)
   call next_word_get('forces')
   if(trim(cur_word) .eq. 'help') then
    write (logf,*) 'forces-h: comments:'
    write (logf,*) 'forces-h: forces'
    write (logf,*) 'forces-h:   help'
    write (logf,*) 'forces-h: warp_coor_type = [string]'&
    //':baricenter_pol_invert_r:baricenter_log:baricenter_r_power:rational:one_atom_warp_exp'
    write (logf,*) 'forces-h: geom_dist_mos = [real]'
    write (logf,*) 'forces-h:             distance first-secondary geometry when moving the mos'
    write (logf,*) 'forces-h: mos_second_moved = [boolean]'
    write (logf,*) 'forces-h:                    different mos for the 2nd geom'
    write (logf,*) 'forces-h:  geom_finite_diff = [real]'          
    write (logf,*) 'forces-h:                 finite difference parameter for 2nd geometry'
    write (logf,*) 'forces-h:  end_forces'
    write (logf,*) 'forces: example'
    write (logf,*) 'forces-h:      end_forces'

   else if (token_is ('warp_coor_type', warp_coor_type)) then

   else if (token_is ('mos_second_moved',mos_second_moved)) then 
! for moving the mos for the secondary geometry

  
   else if (token_is ('geom_dist_mos', geom_dist_2mos)) then 
! if the mos are moved, geometry parameter for the new mos

   else if (token_is ('geom_finite_diff', geom_finite_diff)) then

   else
      call menu_exit (l_here, done)
   end if

  end do
 
  end subroutine force_menu



! =========================================================================
  subroutine e_loc_tot2_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : e_loc_tot2_bywlk_bld
! 
! Creates        : e_loc_tot2_bywlk (walk)
!
! Description : provisoire
! Author: R. Assaraf
! Date  : 31 Aug 2002
! -------------------------------------------------------------------------
  use density_bywlk_mod
  implicit none
   
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
 
  integer(dpi)            :: walk_i
  integer(dpi)            :: walkers_nb, mos_nb, xpfs_c_nb
  integer(dpi)            :: atom_i, atoms_nb
  real(dp), pointer, save :: e_loc_tot2_bywlk (:) => NULL ()
  real(dp), pointer, save :: psi2_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_tot_w_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_tot_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_kin2_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_kin_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: potential_der_bywlk (:) => NULL ()

  real(dp), pointer, save :: w_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: weight2_bywlk (:) => NULL ()
  real(dp)                :: weight1,weight2
  real(dp)                :: eps
 
  integer(dpi), pointer   :: path_bynod (:) => NULL ()
  integer(dpi), pointer   :: roots (:) => NULL ()
  integer(dpi), pointer   :: targets (:) => NULL ()
  integer(dpi)            :: targets_nb = 0
  integer(dpi)            :: roots_nb   = 0
  real(dp), pointer       :: mos_to_xpfs_c_save (:,:) => NULL ()  ! for moving the mos
  integer(dpi)            :: electrons_nb ! for moving the mos  
! header:

  if (header_exe) then

    l_here = "e_loc_tot2_bywlk_bld"

    call object_create ('e_loc_tot2_bywlk')
    call object_create ('e_loc_tot_w_der_bywlk')
    call object_create ('e_loc_tot_der_bywlk')
    call object_create ('w_der_bywlk')
    call object_create ('psi2_bywlk')
    call object_create ('weight2_bywlk')
    call object_create ('e_loc_kin2_bywlk')
    call object_create ('e_loc_kin_der_bywlk')
    call object_create ('potential_der_bywlk')

    call object_needed ('atoms_coor_vector')
    call object_needed ('e_loc_tot_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('density_bywlk')
    call object_needed ('delta_elec_coor_bywlk')
    call object_needed ('delta_jacob_bywlk')
    call object_needed ('atoms_coordinates')
    return

  end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb = size(ea_distance_bywlk, dim = 3)
  mos_nb = size (mos_to_xpfs_c, dim = 1)
  xpfs_c_nb =  size (mos_to_xpfs_c, dim = 2)
  electrons_nb = size (ea_distance_bywlk, dim = 1) ! for moving the mos
! end dimensions

! allocations:


  call alloc ('e_loc_tot2_bywlk', e_loc_tot2_bywlk, walkers_nb)

  call alloc ('e_loc_tot_w_der_bywlk', e_loc_tot_w_der_bywlk, walkers_nb)
  
  call alloc ('e_loc_tot_der_bywlk',e_loc_tot_der_bywlk, walkers_nb)

  call alloc ('w_der_bywlk',w_der_bywlk, walkers_nb)

  call alloc ('psi2_bywlk',psi2_bywlk, walkers_nb)
  
  call alloc ('weight2_bywlk',weight2_bywlk, walkers_nb)
 
  call alloc ('e_loc_kin2_bywlk',e_loc_kin2_bywlk, walkers_nb)

  call alloc ('e_loc_kin_der_bywlk', e_loc_kin_der_bywlk, walkers_nb)

  call alloc ('potential_der_bywlk', potential_der_bywlk, walkers_nb)

  if (.not.associated (mos_to_xpfs_c_save)) allocate (mos_to_xpfs_c_save (mos_nb, xpfs_c_nb) ) ! moving the mos
! end allocations: 


 
 
  if (.not.associated (targets)) then
     call integer_in_list_add_once (object_index('e_loc_tot_bywlk'), targets, list_nb = roots_nb)
     call integer_in_list_add_once (object_index ('psi_bywlk'), targets, list_nb = targets_nb)
     call integer_in_list_add_once (object_index ('e_loc_kin_bywlk'), targets, list_nb = targets_nb)
  endif

  if (.not.associated (roots)) then
     call integer_in_list_add_once (object_index('atoms_coordinates'), roots, list_nb = roots_nb)
     call integer_in_list_add_once (object_index('elec_coor_bywlk'), roots, list_nb = roots_nb)

      
     if (ecp_method == "randomize") & 
          call integer_in_list_add_once (object_index('elec_ecp_random_rot_bywlk'), roots, list_nb = roots_nb)

     if (mos_second_moved) & 
          call integer_in_list_add_once (object_index('mos_to_xpfs_c'),  roots, list_nb = roots_nb) ! moving the mos

  endif

  if (.not.associated (path_bynod)) then
     call path_bynod_bld (path_bynod, roots (1:roots_nb), targets (1:targets_nb))
  endif

! finite difference parameter
  eps = geom_finite_diff * minval(ea_distance_bywlk)/100.

! moving the geometry
  atoms_nb = size(atoms)
  do atom_i = 1, atoms_nb
     atoms_coordinates(atom_i,:) = atoms_coordinates(atom_i,:) + eps*atoms_coor_vector(atom_i,:)
  enddo ! do atom_i

! moving the electrons
  elec_coor_bywlk = elec_coor_bywlk+eps*delta_elec_coor_bywlk

! moving the mos
  if (mos_second_moved) then
     mos_to_xpfs_c_save =  mos_to_xpfs_c  
     mos_to_xpfs_c (1:electrons_nb/2,:) =   mos_to_xpfs_c (1:electrons_nb/2,:) + & 
          (mos_to_xpfs_c (electrons_nb/2+1:2*electrons_nb, :) -  mos_to_xpfs_c (1:electrons_nb/2,:))/geom_dist_2mos*eps 
  endif

 
  call path_bynod_exe (path_bynod)
  
  e_loc_tot2_bywlk = e_loc_tot_bywlk
  psi2_bywlk = psi_bywlk
  e_loc_kin2_bywlk = e_loc_kin_bywlk

! back to the right roots
  do atom_i = 1, atoms_nb
     atoms_coordinates(atom_i,:) = atoms_coordinates(atom_i,:) - eps*atoms_coor_vector(atom_i,:)
  enddo ! do atom_i

  if (mos_second_moved)  mos_to_xpfs_c =   mos_to_xpfs_c_save 

  elec_coor_bywlk = elec_coor_bywlk-eps*delta_elec_coor_bywlk
  
! recomputing targets with the right roots
  call path_bynod_exe (path_bynod)
  
  do walk_i = 1,walkers_nb 
     weight2 = psi2_bywlk(walk_i)**2/psi_bywlk (walk_i)**2*exp(eps*delta_jacob_bywlk (walk_i)) 
     weight1 =  1_dp
     w_der_bywlk (walk_i) = (weight2 - weight1)/eps
     e_loc_tot_w_der_bywlk (walk_i) = (e_loc_tot2_bywlk (walk_i)*weight2-&
          e_loc_tot_bywlk (walk_i)*weight1)/eps
     
     e_loc_tot_der_bywlk (walk_i) = (e_loc_tot2_bywlk (walk_i)-&
          e_loc_tot_bywlk (walk_i))/eps
     
     e_loc_kin_der_bywlk (walk_i) = (e_loc_kin2_bywlk (walk_i)-&
          e_loc_kin_bywlk (walk_i))/eps
     
     potential_der_bywlk (walk_i) = e_loc_tot_der_bywlk (walk_i) - &
          e_loc_kin_der_bywlk (walk_i)                                     
      
  enddo

   call exit(l_here)
  
 end subroutine e_loc_tot2_bywlk_bld

! =========================================================================
  subroutine force_bare_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : force_bare_bywlk_bld
! 
! Creates        : force_bare_bywlk (walk)
!
! Description : force_bare_bywlk, force bare computed with finite differences
! Author: R. Assaraf
! Date  : 26 May 2005
! -------------------------------------------------------------------------
  use density_bywlk_mod
  implicit none
   
! i/o:
  real(dp), pointer, save :: force_bare_bywlk (:) => NULL ()
! local:
  character(len=max_string_len_routine_name),save :: l_here
 
  integer(dpi)            :: walk_i
  integer(dpi)            :: walkers_nb, mos_nb, xpfs_c_nb
  integer(dpi)            :: atom_i, atoms_nb
  real(dp), pointer, save :: e_loc_tot2_bywlk (:) => NULL ()
  real(dp), pointer, save :: psi2_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_tot_w_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_tot_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_kin2_bywlk (:) => NULL ()
  real(dp), pointer, save :: e_loc_kin_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: potential_der_bywlk (:) => NULL ()

  real(dp), pointer, save :: w_der_bywlk (:) => NULL ()
  real(dp), pointer, save :: weight2_bywlk (:) => NULL ()
  real(dp)                :: weight1,weight2
  real(dp)                :: eps
 
  integer(dpi), pointer   :: path_bynod (:) => NULL ()
  integer(dpi), pointer   :: roots (:) => NULL ()
  integer(dpi), pointer   :: targets (:) => NULL ()
  integer(dpi)            :: targets_nb = 0
  integer(dpi)            :: roots_nb   = 0
  real(dp), pointer       :: mos_to_xpfs_c_save (:,:) => NULL ()  ! for moving the mos
  integer(dpi)            :: electrons_nb ! for moving the mos  
! header:

  if (header_exe) then

    l_here = "force_bare_bywlk_bld"

    call object_create ('force_bare_bywlk')

    call object_needed ('atoms_coor_vector')
    call object_needed ('e_loc_tot_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('density_bywlk')
    call object_needed ('atoms_coordinates')
  
    return

  end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb = size(ea_distance_bywlk, dim = 3)
  mos_nb = size (mos_to_xpfs_c, dim = 1)
  xpfs_c_nb =  size (mos_to_xpfs_c, dim = 2)
  electrons_nb = size (ea_distance_bywlk, dim = 1) ! for moving the mos
! end dimensions

! allocations:


  call alloc ('force_bare_bywlk', force_bare_bywlk, walkers_nb)



! end allocations: 

 
   


! finite difference parameter
  eps = geom_finite_diff * minval(ea_distance_bywlk)/100.


! moving the centers of the wavefunction but not the nuclei


  call touch_bynam ("  ")

  call provide_bynam ("e_loc_tot_bywlk")

  



! moving the centers of the wavefunction
  atoms_nb = size(atoms)
  do atom_i = 1, atoms_nb
     atoms_coordinates(atom_i,:) = atoms_coordinates(atom_i,:) + eps*atoms_coor_vector(atom_i,:)
  enddo ! do atom_i

  call touch_bynam ("atoms_coordinates")
  
  call provide_bynam ("e_loc_tot_bywlk")

  e_loc_tot2_bywlk = e_loc_tot_bywlk
  psi2_bywlk = psi_bywlk
  e_loc_kin2_bywlk = e_loc_kin_bywlk

! back to the right roots
  do atom_i = 1, atoms_nb
     atoms_coordinates(atom_i,:) = atoms_coordinates(atom_i,:) - eps*atoms_coor_vector(atom_i,:)
  enddo ! do atom_i

  
! recomputing targets with the right roots
  call path_bynod_exe (path_bynod)
  
  do walk_i = 1,walkers_nb 
     e_loc_tot_der_bywlk (walk_i) = (e_loc_tot2_bywlk (walk_i)-&
          e_loc_tot_bywlk (walk_i))/eps
  enddo

   call exit(l_here)
  
 end subroutine force_bare_bywlk_bld



! =========================================================================
  subroutine Q_grad_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : Q_grad_bywlk_bld
! 
! Creates        : Q_grad_bywlk (elec, dim, walk)
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_grad_bywlk (elec, atom, dim, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
!
! Description : provisoire
! Author: R. Assaraf
! Date  : 31 Aug 2002
! -------------------------------------------------------------------------
  implicit none
   
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: walk_i, elec_i, atom_i, dim_i
  integer(dpi)      ::  dimensions_nb, walkers_nb, electrons_nb, atoms_nb
  real(dp)          :: r, xyz(3), ur (3)
  real(dp), pointer, save :: Q_lapl_bywlk (:)
  real(dp), pointer, save :: Q_bywlk (:)=> NULL ()
! header:

  if (header_exe) then

    l_here = "Q_grad_bywlk_bld"


    call object_create ('Q_bywlk')
    call object_create ('Q_grad_bywlk')
    call object_create ('Q_lapl_bywlk')

    call object_needed ('atoms_coor_vector')
    call object_needed ('elec_coor_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_grad_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
      
    return

  end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb = size(elec_coor_bywlk, dim = 3)
  electrons_nb = size (elec_coor_bywlk, dim = 1)
  dimensions_nb = size (elec_coor_bywlk, dim = 2)
  atoms_nb = size(ea_distance_bywlk, dim=2) 
! end dimensions

! allocations:

     call alloc ('Q_grad_bywlk',Q_grad_bywlk, electrons_nb, &
                 dimensions_nb, walkers_nb)

     call alloc ('Q_bywlk',Q_bywlk, walkers_nb)


      call alloc ('Q_lapl_bywlk', Q_lapl_bywlk, walkers_nb) 



! end allocations:

  Q_grad_bywlk = 0_dp
  Q_bywlk = 0_dp
  Q_lapl_bywlk = 0_dp
  do walk_i = 1, walkers_nb
     do atom_i=1, atoms_nb 
        do dim_i=1, dimensions_nb
           do elec_i=1, electrons_nb
              r=ea_distance_bywlk (elec_i,atom_i,walk_i)
              xyz (:) = ea_dist_xyz_bywlk (elec_i,atom_i,:,walk_i)
              ur (:) = ea_dist_grad_bywlk ( elec_i, atom_i, :, walk_i)

              Q_bywlk(walk_i) = Q_bywlk (walk_i)+ xyz (dim_i) / r &
                   * atoms(atom_i)%charge * atoms_coor_vector( atom_i, dim_i)
           
              Q_grad_bywlk(elec_i, :, walk_i)=Q_grad_bywlk(elec_i, :,walk_i) & 
                   - ur (:) / r**2 * xyz (dim_i)  &
                   * atoms(atom_i)%charge * atoms_coor_vector ( atom_i, dim_i) 

              Q_grad_bywlk (elec_i, dim_i, walk_i) = Q_grad_bywlk (elec_i, dim_i, walk_i) &
                   + 1_dp / r   &
                   * atoms(atom_i)%charge * atoms_coor_vector ( atom_i, dim_i) 

              Q_lapl_bywlk(walk_i) = Q_lapl_bywlk(walk_i)&
                   + 2_dp * xyz( dim_i)/r**3  & 
                   * atoms(atom_i)%charge * atoms_coor_vector (atom_i, dim_i) 

           enddo ! do elec_i
        enddo ! do dim_i
     enddo ! do atom_i
  enddo ! do walk_i

  call exit(l_here)
 
  end subroutine Q_grad_bywlk_bld

! =========================================================================
  subroutine force_zv_min_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name    : force_zv_min_bywlk_bld
!
! Creates : force_zv_min_bywlk (walk)
! Creates : force_bare_bywlk (walk)
! Needs   : Q_grad_bywlk (elec, dim, walk)
! Needs   : drift_bywlk (elec, dim, walk)
!
! Description : provisoire
! Author: R. Assaraf
! Date  : 31 Aug 2002
! -------------------------------------------------------------------------
  implicit none
   
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: walk_i, elec_i, dim_i, atom_i, atom_j
  integer(dpi)      ::  dimensions_nb, walkers_nb, electrons_nb, atoms_nb
  real(dp), pointer, save :: force_zv_min_bywlk (:) => NULL ()
  real(dp), pointer, save :: nuclei_force => NULL ()
  real(dp), pointer, save :: force_bare_bywlk (:) => NULL()
! header:


  if (header_exe) then

    l_here = "force_zv_min_bywlk_bld"

    call object_create ('force_zv_min_bywlk')
    call object_create ('nuclei_force')
    call object_create ('force_bare_bywlk')

    call object_needed ('Q_grad_bywlk')
    call object_needed ('drift_bywlk')   
    call object_needed ('atom_atom_distance')
    call object_needed ('ea_dist_xyz')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('atoms_coor_vector')
    return

 
 end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb = size(drift_bywlk, dim = 3)
  electrons_nb = size (drift_bywlk, dim = 1)
  dimensions_nb = size (drift_bywlk, dim = 2)
  atoms_nb  = size (atom_atom_distance, dim = 1)
! end dimensions

! allocations:
   

     call alloc ('force_zv_min_bywlk',force_zv_min_bywlk, walkers_nb)

     call alloc ('force_bare_bywlk',force_bare_bywlk, walkers_nb)
 

! end allocations:
  force_zv_min_bywlk = 0_dp

  do walk_i = 1, walkers_nb
     do dim_i = 1,dimensions_nb
        do elec_i = 1,electrons_nb
           force_zv_min_bywlk (walk_i) = force_zv_min_bywlk (walk_i)&
                -drift_bywlk (elec_i,dim_i,walk_i)*&
                Q_grad_bywlk (elec_i, dim_i, walk_i)
        enddo ! do elec_i
     enddo ! do dim_i
   
  enddo ! do walk_i

  if (.not.associated (nuclei_force)) then
     call alloc ('nuclei_force',nuclei_force)
     nuclei_force = 0_dp
     do atom_i = 1, atoms_nb
        do atom_j = 1, atoms_nb
           if (atom_i /= atom_j) then
              do dim_i = 1, dimensions_nb
                 nuclei_force = nuclei_force - atoms (atom_i)%charge*atoms (atom_j)%charge &
                      *( atoms (atom_i)%coordinates (dim_i)-atoms (atom_j)%coordinates (dim_i)) &
                      * atoms_coor_vector ( atom_i,dim_i) &
                      / atom_atom_distance (atom_i, atom_j)**3
              enddo ! do dim_i
           endif
        enddo ! do atom_j
     enddo ! do atom_i
  endif

  force_zv_min_bywlk (:) = force_zv_min_bywlk (:) + nuclei_force

  force_bare_bywlk = nuclei_force

  do walk_i = 1, walkers_nb
     do atom_i = 1, atoms_nb
        do dim_i = 1, dimensions_nb
           do elec_i = 1, electrons_nb
              force_bare_bywlk (walk_i) = force_bare_bywlk (walk_i)  &
                   + ea_dist_xyz_bywlk (elec_i,atom_i,dim_i, walk_i) &
                   / ea_distance_bywlk (elec_i,atom_i,walk_i)**3     &
                   * atoms (atom_i)%charge *atoms_coor_vector(atom_i,dim_i)

           enddo ! do elec_i
        enddo ! do dim_i
     enddo  ! do atom_i
  enddo   ! do walk_i

  call exit(l_here)
   
    end subroutine force_zv_min_bywlk_bld

! =========================================================================
    subroutine epsilon_grid_bld
! -------------------------------------------------------------------------
! 
! Name           : epsilon_grid_bld
! 
! Creates        : epsilon_grid (grid_points)
!
! Description : provisoire
! Author:  Alexander Kollias
! Date  : 
! -------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: epsilon_i,grid_nb
  real(dp)          :: initial

! header:
  if (header_exe) then

    l_here = "epsilon_grid_bld"

    call object_create ('epsilon_grid')
 
    return
    
  end if

! begin:
  call enter(l_here) 

! allocations:

  if(.not.associated(epsilon_grid)) then
     call alloc ('epsilon_grid', epsilon_grid, grid_points)
  else
     if(trace_depend) &
     write (logf,*)trim(l_here),': one_ea_distance left in memory'
  end if
 
! end allocations:

! dimensions
   grid_nb   = size(epsilon_grid, dim=1)
!end dimensions

  do epsilon_i = 1, grid_nb

     epsilon_grid(epsilon_i) = initial_point + epsilon_i*grid_size 

  enddo ! epsilon_i

end subroutine epsilon_grid_bld

! =========================================================================
  subroutine one_ea_distance_bld
! -------------------------------------------------------------------------
! 
! Name           : one_ea_distance_bld
! 
! Creates        : one_ea_distance (walk)
!
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : epsilon_grid (grid_size)
! Needs          : atoms_corr_vector
!
! Description : provisoire
! Author: 
! Date  : 
! -------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi) :: walk_i, elec_i, atom_i, epsilon_i, dim_i, elec_count
  integer(dpi) :: walkers_nb, electrons_nb, epsilon_nb, dimensions_nb

! header:
  if (header_exe) then

    l_here = "delta_elec_coor_bywlk_bld"

    call object_create ('one_ea_distance')
 
    call object_needed ('ea_distance_bywlk')
    call object_needed ('epsilon_grid')
    call object_needed ('atoms_coor_vector')
    return

  end if

! begin:
  call enter(l_here) 

! dimensions:

  electrons_nb  = size (elec_coor_bywlk, dim = 1)
  walkers_nb    = size (elec_coor_bywlk, dim = 3)
  dimensions_nb = size (elec_coor_bywlk, dim = 2)
  epsilon_nb    = size (epsilon_grid, dim = 1)

! end dimensions

! allocations:
   
  if(.not.associated(one_ea_distance)) then
     call alloc ('one_ea_distance',one_ea_distance,epsilon_nb,walkers_nb)
  else
     if(trace_depend) &
     write (logf,*)trim(l_here),': one_ea_distance left in memory'
  end if
 
! end allocations:

  one_ea_distance = 0_dp
  elec_count             = 0

  do epsilon_i = 1, epsilon_nb
     do atom_i = 1, size(atoms)
        do walk_i = 1,walkers_nb

           do elec_i = 1,electrons_nb
              do dim_i = 1,dimensions_nb

                 if(ea_distance_bywlk (elec_i, atom_i, walk_i) > epsilon_grid(epsilon_i)) then

                    elec_count = elec_count + 1*atoms_coor_vector(atom_i,dim_i)

                 endif

              end do ! dim_i
           enddo ! elec_i
        
           if(elec_count == electrons_nb) then
 
             one_ea_distance(epsilon_i,walk_i) = 1_dp

           endif
           
           elec_count = 0_dp
           
        enddo ! walk_i
     enddo ! atom_i
  enddo ! epsilon_i

end subroutine one_ea_distance_bld

! =========================================================================
  subroutine delta_elec_coor_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : delta_elec_coor_bywlk_bld
! 
! Creates        : delta_elec_coor_bywlk (elec, dim, walk)
! Creates        : delta_jacob_bywlk (walk)
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_grad_bywlk (elec, atom, dim, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
!
! Description : provisoire
! Author: R. Assaraf
! Date  : 31 Aug 2002
! -------------------------------------------------------------------------
  implicit none
   
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: walk_i, elec_i, dim_i, atom_i
  integer(dpi)      :: dimensions_nb, walkers_nb, electrons_nb
  real(dp)          :: funcr, funcr_deriv
  real(dp)          :: k1, k2, k3, k4, k5
  real(dp)          :: sum_ua_nabla_f, sum_a_f_ia, f_deriv, f_ia, r_ia, prod_scal
  real(dp)          :: delta_jacob_elec_i, denom 
! header:

  real(dp) :: sum_a_fu(3), nabla_f(3), r_ia_vec(3), sum_a_nabla_f(3)

  if (header_exe) then

    l_here = "delta_elec_coor_bywlk_bld"

    call object_create ('delta_elec_coor_bywlk')
    call object_create ('delta_jacob_bywlk')

    call object_needed ('elec_coor_bywlk')
    call object_needed ('atoms_coor_vector')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_grad_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('delta_exp1')
    call object_needed ('delta_exp2')
    call object_needed ('delta_exp3')
    call object_needed ('delta_exp4')
    call object_needed ('delta_exp5')
    call object_needed ('exp_factor')
          
    return

  end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb    = size(elec_coor_bywlk, dim = 3)
  electrons_nb  = size (elec_coor_bywlk, dim = 1)
  dimensions_nb = size (elec_coor_bywlk, dim = 2)
! end dimensions

! allocations:
   
  if(.not.associated(delta_elec_coor_bywlk)) then
     call alloc ('delta_elec_coor_bywlk',delta_elec_coor_bywlk, electrons_nb, &
                 dimensions_nb, walkers_nb)
  else
     if(trace_depend) &
     write (logf,*)trim(l_here),': delta_elec_coor_bywlk left in memory'
  end if
 
  if(.not.associated(delta_jacob_bywlk)) then
     call alloc ('delta_jacob_bywlk',delta_jacob_bywlk, walkers_nb)
  else
     if(trace_depend) &
     write (logf,*)trim(l_here),': delta_jacob_bywlk left in memory'
  end if

 ! allocate (sum_a_fu(dimensions_nb), nabla_f(dimensions_nb))
 ! allocate (r_ia_vec(dimensions_nb), sum_a_nabla_f(dimensions_nb))

! end allocations:

  delta_elec_coor_bywlk = 0_dp
  delta_jacob_bywlk     = 0_dp

  k1 = delta_exp1**2
  k2 = delta_exp2**2
  k3 = delta_exp3**2
  k4 = delta_exp4**2
  k5 = delta_exp5

  if( warp_coor_type(1:10) .eq. 'baricenter' ) then 

! computing delta_jacob_bywlk and delta_elec_coor_bywlk 

     do walk_i = 1,walkers_nb
        do elec_i = 1,electrons_nb

           sum_ua_nabla_f = 0.d0
           sum_a_f_ia     = 0.d0
           sum_a_fu (:)   = 0.d0
           sum_a_nabla_f  = 0.d0

           do atom_i = 1,size(atoms)

              r_ia  = ea_distance_bywlk (elec_i, atom_i, walk_i)

              r_ia_vec =  ea_dist_xyz_bywlk (elec_i,atom_i,:,walk_i)

              select case (warp_coor_type)
                 
              case ('baricenter_pol_invert_r') 

                 f_ia     =  k5 + k1 / r_ia + k2 /r_ia**2 + k3 / r_ia**3 + k4 / r_ia**4 
                 
                 f_deriv  =  -k1/r_ia**2 - 2_dp*k2/r_ia**3 - 3_dp * k3/r_ia**4 - 4_dp*k4/r_ia**5

              case ('baricenter_log')


                denom = k2+ k3 *  log (r_ia+1)

                f_ia = k1 / denom

                f_deriv = -k3*k1 /(r_ia +1) / denom**2


              case ('baricenter_r_power')
                 
                 f_ia     = r_ia**(-k1)

                 f_deriv  = -k1*r_ia**(-k1-1) 

              case default

                 call die(l_here,'option warp_coor_type >'+warp_coor_type+'< not valid')

              end select

              nabla_f (1) = f_deriv / r_ia *  r_ia_vec (1)
              nabla_f (2) = f_deriv / r_ia *  r_ia_vec (2)
              nabla_f (3) = f_deriv / r_ia *  r_ia_vec (3)

!  prod_scal

              prod_scal = 0.d0
              do dim_i = 1, dimensions_nb
                 prod_scal = prod_scal + atoms_coor_vector (atom_i,dim_i) * nabla_f (dim_i)
              enddo ! do coor

! ingredients of the divergence of the electron i 

              sum_ua_nabla_f = sum_ua_nabla_f + prod_scal
              
              sum_a_f_ia     = sum_a_f_ia + f_ia
              
              sum_a_nabla_f (:) =  sum_a_nabla_f (:) + nabla_f (:)
              
              sum_a_fu (:) = sum_a_fu (:) + f_ia * atoms_coor_vector (atom_i,:)
              
           enddo !  do atom_i
            
           delta_jacob_elec_i = sum_ua_nabla_f/sum_a_f_ia - dot_product(sum_a_nabla_f,sum_a_fu)/sum_a_f_ia**2

!     delta_jacob_bywlk the divergence of the warped is the sum of the divergence over the electrons
       
           delta_jacob_bywlk (walk_i) =   delta_jacob_bywlk (walk_i) + delta_jacob_elec_i

           delta_elec_coor_bywlk(elec_i,:,walk_i) = sum_a_fu(:)/sum_a_f_ia

        enddo ! do elec_i
     enddo ! do walk_i
     
  endif
    
  if(warp_coor_type .eq. 'rational') then

    do walk_i = 1, walkers_nb
       do atom_i =1 , size(atoms)
          do elec_i = 1, electrons_nb

             r_ia = ea_distance_bywlk (elec_i, atom_i, walk_i)
   
             do dim_i = 1, dimensions_nb
                funcr       =  k1/r_ia**k2
                funcr_deriv = -k1 * k2* ea_dist_xyz_bywlk (elec_i,atom_i,dim_i,walk_i)/r_ia**(k2+2_dp)
                
                delta_elec_coor_bywlk (elec_i, dim_i, walk_i) =      &
                     delta_elec_coor_bywlk (elec_i, dim_i, walk_i)   &
                     + funcr * atoms_coor_vector(atom_i,dim_i)
                
                delta_jacob_bywlk (walk_i) =     &            
                     delta_jacob_bywlk (walk_i)  &
                     + funcr_deriv * atoms_coor_vector(atom_i,dim_i)
                
             enddo  !do dim_i

          enddo  !do elec_i
       enddo  !do atom_i
    enddo ! do walk_i

 endif

 if(warp_coor_type .eq. 'one_atom_warp_exp') then

    do walk_i = 1, walkers_nb
       do atom_i =1 , size(atoms)
          do elec_i = 1, electrons_nb

             r_ia = ea_distance_bywlk (elec_i, atom_i, walk_i)
   
             do dim_i = 1, dimensions_nb
                funcr=exp(-delta_exp1*r_ia-delta_exp2**2*r_ia**2-delta_exp3**2*r_ia**3-delta_exp4**2*r_ia**4)
                funcr_deriv= (-2_dp*delta_exp2**2*r_ia-delta_exp1-3_dp*delta_exp3**2*r_ia**2-4_dp*delta_exp4**2*r_ia**3) * funcr
                
                delta_elec_coor_bywlk (elec_i, dim_i, walk_i) =      &
                     delta_elec_coor_bywlk (elec_i, dim_i, walk_i)   &
                     + funcr * atoms_coor_vector(atom_i,dim_i)
                
                delta_jacob_bywlk (walk_i) =     &            
                     delta_jacob_bywlk (walk_i)  &
                     + funcr_deriv               &
                     * ea_dist_xyz_bywlk (elec_i,atom_i,dim_i,walk_i)/r_ia &
                     * atoms_coor_vector(atom_i,dim_i)
                
             enddo  !do dim_i

          enddo  !do elec_i
       enddo  !do atom_i
    enddo ! do walk_i

 endif

  call exit(l_here)

 delta_jacob_bywlk = exp_factor * delta_jacob_bywlk
 delta_elec_coor_bywlk = exp_factor * delta_elec_coor_bywlk

!deallocate (sum_a_fu, nabla_f, r_ia_vec, sum_a_nabla_f) 

end subroutine delta_elec_coor_bywlk_bld

! =========================================================================
  subroutine delta_param_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name           : delta_param_bywlk_bld
! 
! Creates        : delta_param_bywlk

!
! Description : provisoire
! Author: R. Assaraf
! Date  : 31 Aug 2002
! -------------------------------------------------------------------------
 implicit none
   
! i/o:

! local:
 character(len=max_string_len_routine_name),save :: l_here

! header:

 if (header_exe) then

    l_here = "delta_param_bywlk_bld"
    call object_create ('delta_exp1')
    call object_create ('delta_exp2')
    call object_create ('delta_exp3')
    call object_create ('delta_exp4')
    call object_create ('delta_exp5')
    call object_create ('exp_factor')
    return

 end if

! begin:
  call enter(l_here) 

 if (.not.associated (delta_exp1)) call alloc ('delta_exp1',delta_exp1)
 if (.not.associated (delta_exp2)) call alloc ('delta_exp2',delta_exp2)
 if (.not.associated (delta_exp3)) call alloc ('delta_exp3',delta_exp3)
 if (.not.associated (delta_exp4)) call alloc ('delta_exp4',delta_exp4)
 if (.not.associated (delta_exp5)) call alloc ('delta_exp5',delta_exp5)
 if (.not.associated (exp_factor)) call alloc ('exp_factor',exp_factor)
 
 exp_factor = 1_dp
 delta_exp1 = 0_dp
 delta_exp2 = 0_dp
 delta_exp3 = 0_dp
 delta_exp4 = 0_dp
 delta_exp5 = 0_dp

 call exit(l_here)
 
end subroutine delta_param_bywlk_bld

! =========================================================================
  subroutine atoms_coor_vector_bld
! -------------------------------------------------------------------------
! 
! Name      : atoms_coor_vector_bld
! 
! Creates   : atoms_coor_vector (atom, dim)
!
! Description    : Builds a vector in the space of atoms coordinates
!
! Author: R. Assaraf
! Date  : 27 Mar 2003
! -------------------------------------------------------------------------
 implicit none
   
! i/o:

! local:
 character(len=max_string_len_routine_name),save :: l_here
 integer(dpi) :: atoms_nb, dimensions_nb
! header:

 if (header_exe) then

    l_here = "atoms_coor_vector_bld"
    call object_create ('atoms_coor_vector')

    call object_needed ('atoms_coordinates')
    return

 end if

! begin:
 call enter(l_here) 

! dimensions 
 atoms_nb = size (atoms_coordinates, dim = 1)
 dimensions_nb = size (atoms_coordinates, dim = 2)

 if (.not.associated (atoms_coor_vector)) then
    call alloc ('atoms_coor_vector', &
         atoms_coor_vector, atoms_nb, dimensions_nb)
 endif
 
 atoms_coor_vector = 0_dp
 atoms_coor_vector(1,3)=0.d0
 atoms_coor_vector(2,3)=1.d0

  call exit(l_here)
 
end subroutine atoms_coor_vector_bld





! =========================================================================
  subroutine  el_s_byat_bywlk_bld
! -------------------------------------------------------------------------
! 
! Name    : el_s_byat_bywlk_bld
!
! Creates : el_s_byat_bywlk (atom,walk)
! Creates : el_s_der_bycoor_byat_bywlk (coor, atom,walk)
! Description : 
! Author: R. Assaraf
! Date  : 31 Aug 2002
! -------------------------------------------------------------------------
  implicit none
   
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)      :: walk_i, elec_i, elec_j, dim_i, atom_i
  integer(dpi)      ::  dimensions_nb, walkers_nb, electrons_nb, atoms_nb
  

  real(dp), pointer :: u_ee (:,:,:)    => NULL ()
  real(dp), pointer :: u_ee_d1 (:,:,:) => NULL ()
  real(dp), pointer :: u_ee_d2 (:,:,:) => NULL ()
  
  real(dp), pointer :: u_ea (:,:,:)    => NULL ()
  real(dp), pointer :: u_ea_d1 (:,:,:) => NULL ()
  real(dp), pointer :: u_ea_d2 (:,:,:) => NULL ()
 
  real(dp), pointer :: delta_u (:,:)   => NULL ()
  real(dp), pointer :: nabla_u (:,:,:,:) => NULL ()
  real(dp) :: r_ij, r_ia
   
! header:


  if (header_exe) then

    l_here = "el_s_byat_bywlk_bld"

    call object_create ('el_s_byat_bywlk')
    call object_create ('el_s_der_bycoor_byat_bywlk')
 
    call object_needed ('ee_distance_bywlk')
    call object_needed ('ea_dist_xyz')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('atoms_coor_vector')
    return

 
 end if

! begin:
  call enter(l_here) 

! dimensions:
  walkers_nb = size(drift_bywlk, dim = 3)
  electrons_nb = size (drift_bywlk, dim = 1)
  dimensions_nb = size (drift_bywlk, dim = 2)
  atoms_nb  = size (atom_atom_distance, dim = 1)
! end dimensions

! allocations:
   

    call alloc ('el_s_byat_bywlk', el_s_byat_bywlk, atoms_nb, walkers_nb)


    if  (.not.associated (u_ee))    allocate (u_ee (electrons_nb,electrons_nb,walkers_nb))
    if  (.not.associated (u_ee_d1)) allocate (u_ee_d1 (electrons_nb, electrons_nb, walkers_nb))
    if  (.not.associated (u_ee_d2)) allocate (u_ee_d2 (electrons_nb, electrons_nb, walkers_nb))
    
    if  (.not.associated (u_ea))    allocate (u_ea (electrons_nb, atoms_nb, walkers_nb))
    if  (.not.associated (u_ea_d1)) allocate (u_ea_d1 (electrons_nb, atoms_nb, walkers_nb))
    if  (.not.associated (u_ea_d2)) allocate (u_ea_d2 (electrons_nb, atoms_nb, walkers_nb))

    if (.not.associated (delta_u)) allocate (delta_u (atoms_nb, walkers_nb))
  
    if (.not.associated (nabla_u)) allocate (nabla_u (atoms_nb, electrons_nb, 3, walkers_nb))
! end allocations:
! EL_s = local energy for a simplified trial function
! psi_s = e^U  where U = e^{\sum_{ia} u_ea (r_ia)+\sum_{i<j} u_ee (r_ij)}
    
! u_ee and derivatives
    do walk_i = 1, walkers_nb
       do elec_i = 1, electrons_nb
          do elec_j = 1, electrons_nb
             r_ij = ee_distance_bywlk (elec_i, elec_j, walk_i)
           
             u_ee (elec_i, elec_j, walk_i) = 0.5* r_ij
             u_ee_d1  (elec_i, elec_j, walk_i) = 0.5d0
             u_ee_d2  (elec_i, elec_j, walk_i) = 0.d0
          enddo ! do elec_j
       enddo ! do elec_i
    enddo ! do walk_i

! u_ea and derivatives

    do walk_i = 1, walkers_nb
       do elec_i = 1, electrons_nb
          do atom_i = 1, atoms_nb
             r_ia = ea_distance_bywlk (elec_i, atom_i, walk_i)
             u_ea (elec_i, atom_i, walk_i) = -2.d0 * r_ia
             u_ea_d1 (elec_i, atom_i, walk_i) = -2.d0
             u_ea_d2 (elec_i, atom_i, walk_i) = 0.d0
          enddo ! do atom_i
       enddo ! do elec_i
    enddo ! do walk_i

! laplacian of U
    delta_u   = 0.d0
    do walk_i =1, walkers_nb
       do atom_i =1, atoms_nb
          do elec_i =1, electrons_nb
             r_ia = ea_distance_bywlk (elec_i, atom_i, walk_i)
             delta_u (atom_i,walk_i) = delta_u (atom_i,walk_i) &
                  + u_ea_d2 (elec_i, atom_i, walk_i) +2_dp / r_ia * u_ea_d1 (elec_i, atom_i, walk_i)
          enddo ! do elec_i
       enddo ! do atom_i
    enddo ! do walk_i

    do walk_i = 1, walkers_nb
       do elec_i =1, electrons_nb
          do  elec_j =elec_i+1, electrons_nb
             r_ij = ee_distance_bywlk (elec_i, elec_j, walk_i) 
             delta_u (:,walk_i) = delta_u (:,walk_i) + 2.d0 * ( &
                  u_ee_d2 (elec_i, elec_j, walk_i) + &
                  2_dp / r_ij * u_ee_d1 (elec_i, elec_j, walk_i))
          enddo ! do elec_j
       enddo ! do elec_i
    enddo ! do walk_i

! gradient of U
    nabla_u = 0.d0
    do walk_i = 1, walkers_nb
       do elec_i =1, electrons_nb
          do  elec_j = 1, electrons_nb
             
             if (elec_i .ne. elec_j) then
                r_ij = ee_distance_bywlk (elec_i, elec_j, walk_i)
                do atom_i = 1, atoms_nb
                   nabla_u (atom_i,elec_i, :,walk_i) =  nabla_u (atom_i,elec_i, :,walk_i) &
                        + u_ee_d1 (elec_i, elec_j, walk_i) / r_ij * &
                        (elec_coor_bywlk (elec_i, :, walk_i) &
                        - elec_coor_bywlk (elec_j, :, walk_i))
                enddo ! do atom_i
             endif
             
          enddo ! do elec_j
       enddo ! do elec_i
    enddo ! do walk_i

    do walk_i = 1, walkers_nb
       do elec_i = 1, electrons_nb
          do atom_i = 1, atoms_nb
             r_ia = ea_distance_bywlk (elec_i, atom_i, walk_i)
             nabla_u (atom_i, elec_i, :,walk_i) =  nabla_u (atom_i, elec_i, :,walk_i) &
                  + u_ea_d1 (elec_i, atom_i, walk_i) / r_ia *         &
                  ea_dist_xyz_bywlk (elec_i, atom_i, : , walk_i)
          enddo ! do atom_i
       enddo ! do elec_i
    enddo ! do walk_i


  call exit(l_here)
   
  end subroutine el_s_byat_bywlk_bld

end module force_bywlk_mod


