! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


! ============================================================================
   function real_func (par, par_nb) result (result) 
! ----------------------------------------------------------------------------
!
! Name          : real_func
!
! Description   : computes a real from the parameters par 
!
! Authors: R. Assaraf
! Date   : 02 Aug 2002
! --------------------------------------------------------------------------------
  use objects_paths_mod
  use basic_types_tools_mod
  use func_mod

  implicit none
  
! i/o:
  real (dp)     :: result 
  integer(dpi)  :: par_nb, i
  real(dp)      :: par(par_nb)

! local:      
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi), pointer, save :: path_bynod (:) => NULL ()

! begin:
  l_here='real_func'
  
  call header_die
  
  call enter(l_here)
  if (par_nb  /= size (par)) call die (l_here, ' par_nb='+par_nb+ ' /= size (par)='+size (par))
!  call touch_byind (real_variables)

 if (.not.associated (path_bynod))  then
   call path_bynod_bld (path_bynod, real_variables, (/ (i, i= 1,objects_nb) /) )
  endif
  is_valid_bynod (path_bynod) = .false.


  call real_objects_scatter_bld (real_variables, par)
!  if (.not.associated (func_path)) call path_bynod_bld (func_path,func_roots_ind,(/func_target_ind/))
!  call path_bynod_exe (func_path)
   call provide_byind (func_target_ind)
 
  if (.not.associated (real_1_targets_aliases)) then
     allocate (real_1_targets_aliases)
     call real_1_from_obj_byindex_bld ( real_1_targets_aliases%p, func_target_ind)
  endif
  result = real_1_targets_aliases%p (1)
  call list_names_print
  call exit(l_here)

end function real_func
 
! ========================================================================
  subroutine func_menu
! ------------------------------------------------------------------------
!
! Name   : func_menu
!
! Description : menu to define the function real_func and or to apply methods 
! Description : of this function 
!
! Authors:F. Assaraf
! Date  : 05 June 2002
! ------------------------------------------------------------------------
  use basic_types_tools_mod
  use parser_mod
  use objects_paths_mod
  use func_mod
  use minimize_mod

  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                   :: done
  character(len=max_string_len), pointer :: roots_names     (:) => NULL()
  character(len=max_string_len), pointer :: real_variables_names (:) => NULL ()
  character(len=max_string_len)  :: result_name
  character(len=max_string_len)  :: action
  integer(dpi)                    ::  i
  real(dp), pointer :: par (:) => NULL ()
  
  external real_func
  real(dp) :: real_func

! begin:
  l_here ='func_menu'
  call enter (l_here)
 
  done = .false.

  call next_command_get ('func')
  do while (.not.done)
     call next_word_get('func')
     if(trim(cur_word) .eq. 'help') then
        write (logf,'(a)') 'func: func_menu'
        write (logf,'(a)') 'func: syntax'
        write (logf,'(a)') 'func: help'
        write (logf,'(a)') 'func: result  = [real scalar object]'
        write (logf,'(a)') 'func: roots   = [list of objects]'
        write (logf,'(a)') 'func: variables = [list of real objects] : tipically subset of roots'
        write (logf,'(a)') 'func: action    = [string] :{mini}mize: {val}_result_print'
        write (logf,'(a)') 'func: out = [list of strings]'
        write (logf,'(a)') 'func: example'

     else if (token_is('out',list_names)) then

     else if (token_is('result', result_name)) then


! define func%targets   
        func_target_ind = object_index (result_name)

     else if (token_is('roots',roots_names)) then
! improve : the roots should be inferred from the variables, not defined from the input 
          
        allocate (func_roots_ind (size(roots_names))) 
        do i = 1,size(roots_names)
           if (debug) write(logf,*) trim(l_here), '-d: root >',&
                trim(roots_names (i)),'<'
           func_roots_ind (i)=object_index (roots_names (i))
        enddo
 
        call provide_byind (func_target_ind)
        
 ! define real_variables 
     else if (token_is('variables',real_variables_names)) then
    
        if (associated (real_variables)) then 
           if (size (real_variables) /= size (real_variables_names)) then
              deallocate (real_variables)
              allocate (real_variables (size ( real_variables_names)))
            endif
         else
            allocate (real_variables (size ( real_variables_names)))
         endif
         
         do i = 1, size (real_variables_names)
 
            real_variables (i) = object_index ( real_variables_names (i))
            
         enddo
      
      else if (token_is ('action',action)) then 

         if (action (1:3) == 'val') then
! compute the values of the objects func%targets 
            call provide_byind (func_target_ind)

! and prints them  
          
         
            call object_value_byind_print (func_target_ind)
         
  
         else if (action=='minimize') then
        
            write (res,'(2a)') blanks(1:20),' '
            write (res,'(2a)') blanks(1:20),' '
            write (res,'(2a)') blanks(1:20),' ________________________ '
            write (res,'(2a)') blanks(1:20),'I                        I' 
            write (res,'(2a)') blanks(1:20),'I    Optimization        I'
            write (res,'(2a)') blanks(1:20),'I________________________I'
            write (res,'(2a)') blanks(1:20),'**************************'
        
            write (res,*) 
            write (res,*) 'The optimization method is >', trim (cmeth),'<'
            write (res,*) 'After each iteration of the optimization process, '
            write (res,*) 'the best parameters found are written in file of type >objects-out<'
 
             
            if (associated (par)) call die (l_here, 'par should not be associated')
 
            call real_objects_gather_bld (par, real_variables)
            write (logf,*) 'par=',par
            call minimize (par, real_func, real_variables_print)

            write (res,*) 'Minimum proposed :'
            call object_value_byind_write (func_target_ind, res)
            write (res,*)
            write (res,*) 'Optimal parameters found: '
 
            do i = 1, size (real_variables)
               
               call object_value_byind_write (real_variables (i), res)
               
            enddo

           
            deallocate (par)       
         else

            call die(l_here,'option '*action*' not valid')
            
         endif
        
      else
         if (associated (real_variables)) deallocate (real_variables)
         if (associated ( real_1_targets_aliases )) deallocate ( real_1_targets_aliases )
         call menu_exit (l_here, done)
     
      endif

   enddo

   call exit (l_here)

 end subroutine func_menu

