!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module grid_menu_mod

! ------------------------------------------------------------------------
! 
! Author: A. Scemama
! Date  : 25 Mar 2004
! Revision: 
! ------------------------------------------------------------------------
 
  use system_data_mod
  use parser_mod
  use expressions_mod
  use grid_coor_mod

  implicit none

 logical(bool)         :: grid_write
 logical(bool)         :: grid_read

  contains
 
! ========================================================================
  subroutine grid_menu
! ------------------------------------------------------------------------
! Name           : grid_menu
 
! Description    : input grid parameters
 
! Author: A. Scemama
! Date  : 25 Mar 2004
! ------------------------------------------------------------------------
  implicit none
    
! local:
  logical(bool)       :: done
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='grid_menu'
  call enter(l_here, 1)
  objects_desc ( object_index('grid_coor') ) = 'Coordinates of the grid points'
! initialize:
  done = .false.
  call next_command_get ('grid')
  do while (.not.done)
     call next_word_get ('grid')
     if(debug) write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
     if(trim(cur_word) .eq. 'help') then
      write (logf,*) 'grid-h: menu'
      write (logf,*) 'grid-h: syntax'
      write (logf,*) 'grid-h: gri{d}'
      write (logf,*) 'grid-h:   help'
      write (logf,*) 'grid-h:   typ{e}       = [string]'
      write (logf,*) 'grid-h:                  grid_xyz'
      write (logf,*) 'grid-h:                  grid_xy'
      write (logf,*) 'grid-h:                  grid_x'
      write (logf,*) 'grid-h:                  grid_r1r2'
      write (logf,*) 'grid-h:   dim{ensions} = [list of integers]'
      write (logf,*) 'grid-h:   ste{p}       = [list of doubles]'
      write (logf,*) 'grid-h:   ori{gin}     = [list of doubles]'
      write (logf,*) 'grid-h:   name         = [string]'
      write (logf,*) 'grid-h:   end_gri'
      write (logf,*) 'grid-h: example'
      write (logf,*) 'grid-h: grid'
      write (logf,*) 'grid-h:   type = grid_xyz'
      write (logf,*) 'grid-h:   dim  = (100,100,100)' 
      write (logf,*) 'grid-h:   ori  = (-5.,-5.,-5.)'
      write (logf,*) 'grid-h:   step = (.1,.1,.1)' 
      write (logf,*) 'grid-h:   end_grid'
      
  else if(token_is ('typ', grid_type)) then

  else if(token_is ('dim', grid_dimensions)) then
      
  else if(token_is ('ste', grid_step)) then
      
  else if(token_is ('ori', grid_origin)) then

  else
    call menu_exit (l_here, done)
  end if
     
  end do
    


  if(print_level .ge. 0 .or. debug) then
     
     write (logf,*) trim(l_here), &
          ': grid_type       =',grid_type
     
     write (logf,*) trim(l_here), &
          ': grid_dimensions =',grid_dimensions(:)
     
     write (logf,*) trim(l_here), &
          ': grid_origin     =',grid_origin(:)
     
     write (logf,*) trim(l_here), &
          ': grid_step       =',grid_step(:)
     
  end if
    

  call exit (l_here, 1)
    
  end subroutine grid_menu
 
end module grid_menu_mod

