! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module guide_menu_mod

  use configurations_guide_mod
! use jastrow_menu_mod
  use system_data_mod

! Menu for guiding wave function
 
  logical                                    :: use_guide = .false.
  real(dp)                                   :: power_rij     = 0.d0
  real(dp)                                   :: power_sum_rij = 0.d0
 
  contains
 
! ========================================================================
  subroutine guide_menu
! ------------------------------------------------------------------------
! Name           : guide_menu
 
! Description    : input guiding wavefunction information
 
! Created        : J. Toulouse, Sep 2005
! Revised        : J. Toulouse, 02 Oct 2005
! ------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len) :: what   
  integer(dpi)        :: atom_i
  integer(dpi)        :: n_deter,det_i ,n_alpha,n_beta
  integer(dpi)        :: conf_i, conf_nb
  integer(dpi)        :: electrons_nb
  integer(dpi)        :: obj_i
  logical(bool)       :: done
  logical(bool)       :: jastrow_guide_calc
  real(dp)            :: l_charge, n_charge
    
! begin:
  l_here ='guide_menu'
  call enter(l_here)

! initialize:
  jastrow_guide_calc = .false.
  done = .false.
  what = NOT_INIT_S
  
  call next_command_get ('guide')
  do while (.not.done)
     call next_word_get('guide')
     if(debug) write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
     if(trim(cur_word) .eq. 'help') then
      write (logf,*) ' guide-h: menu'
      write (logf,*) ' guide-h: comments:'
      write (logf,*) ' guide-h:  define guiding wave function'
      write (logf,*) ' guide-h: syntax:'
      write (logf,*) ' guide-h: guide'
      write (logf,*) ' guide-h:  help'
      write (logf,*) ' guide-h:  con{figuration} [command] (repeat)'
      write (logf,*) ' guide-h:  print = con{figurations}'
      write (logf,*) ' guide-h:  power_rij          = [real]'
      write (logf,*) ' guide-h:  power_sum_rij      = [real]'
      write (logf,*) ' guide-h:  end_guide'
      write (logf,*) ' guide-h: example'
      
  else if(cur_word(1:3) .eq. 'con') then
      call configuration_guide_menu  
      
  else if(token_is ('pri', what)) then
      
  else if(token_is ('power_rij', power_rij)) then

  else if(token_is ('power_sum_rij', power_sum_rij)) then

  else
      call menu_exit (l_here, done)
  end if
     
  end do
    
 use_guide = .true.
 write (logf,*) trim(l_here), ': use guiding wave function '
 write (logf,*) trim(l_here), ': WARNING: guiding wave function not fully checked!'
 write (logf,*) trim(l_here), ': are you sure of what you are doing?'

! factor [Sum_{i<j} rij^(power_rij)]^(power_sum_rij)
  if (power_sum_rij /= 0.d0) then
    write (logf,*) trim(l_here), ': guiding wave function function multiplied by [Sum_{i<j} rij^(power_rij)]^(power_sum_rij)'
    write (logf,*) trim(l_here), ': with power_rij = ',power_rij
    write (logf,*) trim(l_here), ': and power_sum_rij = ',power_sum_rij
  endif


! check the charge
  electrons_nb = system%electrons_nb
  message='in routine'*l_here*' system%electrons_nb='+system%electrons_nb
  call require (message, system%electrons_nb > 0)
  
  n_charge = zero
  do atom_i = 1,size(atoms)
      n_charge = n_charge + atoms(atom_i)%charge
  enddo

  l_charge = n_charge - float(electrons_nb)
  if(abs(l_charge - system%charge) > EPSILON(+1.0)) then
      message='system charge from input = '*system%charge &
              +CR*" "*l_here+'-die: sum of nuclei charges '+n_charge &
              +' - electrons charge '+electrons_nb+' = '*l_charge  &
              +CR*" "*l_here+'-die: please check charge in input'
      call warning (l_here, message)
  endif

! all configurations_guide fields are now known:
    
  call check_asso_conf_guide ('configurations_guide', configurations_guide, &
       'configurations_guide_init', configurations_guide_init)

  include 'chksiz_configurations_guide.inc'

  conf_nb = system%configurations_guide_nb
     
  do conf_i = 1, conf_nb

    if (configurations_guide(conf_i)%jastr_type      /= NOT_INIT_S  .and. &
        configurations_guide(conf_i)%jastr_type(1:3) /= "NON") jastrow_guide_calc = .true.

      if(print_level .ge. 1 .or. debug) then
        write (logf,*)trim(l_here),'-d: conf # ',conf_i, &
               ' named >',trim(configurations_guide(conf_i)%name),'<', &
               ' jastr_type >',trim(configurations_guide(conf_i)%jastr_type),'<'
    end if

  end do
     
! This to disable jastrow calculation if not used.
  if(jastrow_guide_calc) then
    write (logf,*)trim(l_here),': jastrow is USED in guiding function'
  else
    write (logf,*)trim(l_here),': NO jastrow used in guiding function'
    do obj_i = 1, objects_nb
      if (objects_names(obj_i)(1:len('jastrow_factor_guide_')) == 'jastrow_factor_guide_') then
          objects_used(obj_i) = .false.
          write (logf,*)trim(l_here),': object >',trim(objects_names(obj_i)),'< disabled'
      end if 
    end do
  end if

  if (debug) then
    do conf_i = 1,conf_nb
      n_deter = size(configurations_guide(conf_i)%conf_deter)
      write (logf,*)trim(l_here),' configuration ',conf_i,': n_deter = ',n_deter
      do det_i = 1,n_deter
       write (logf,*)trim(l_here),' determinant ',det_i 
       n_alpha = size(configurations_guide(conf_i)%conf_deter(det_i )%alfa_orbitals)
       n_beta = size(configurations_guide(conf_i)%conf_deter(det_i )%beta_orbitals)
       write (logf,*)trim(l_here),'-d: ',n_alpha, &
            ' alpha orbitals and ',system%alfa_electrons_nb,' alpha electrons'
       write (logf,*)trim(l_here),'-d: ',n_beta, &
            ' beta  orbitals and ',system%beta_electrons_nb,' beta  electrons'
      end do
    end do
  end if
    
  if(what(1:3) == "con") then
    message ='configurations of guiding wavefunction'
    call conf_jastr_guide_print_res (message)
  end if

  call exit(l_here)
    
  end subroutine guide_menu
 
end module guide_menu_mod
