! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module integration_mod

  use control_mod
  use strings_mod

  contains
! ========================================================================
  subroutine gtos_overlap (         &
                         pgcexpo,       &       ! input                  
                         pgcnorm,       &       ! input                  
                         pgccntx,       &       ! input                  
                         pgccnty,       &       ! input                  
                         pgccntz,       &       ! input                  
                         pgcpowx,       &       ! input                  
                         pgcpowy,       &       ! input                  
                         pgcpowz,       &       ! input                  
                         abscis,        &       ! input                  
                         weight,        &       ! input                  
                         ovlerpg,       &       ! output                 
                         nbpgc,         &       ! dimension              
                         maxordgh)              ! dimension              
! ------------------------------------------------------------------------
! Author : G. Jansen  
! ------------------------------------------------------------------------
  implicit none

! includes:

! input dimensions:
  integer(dpi)nbpgc                                            
  integer(dpi)maxordgh                                          

! input arrays:
      real(dp) pgcexpo(nbpgc)
      real(dp) pgcnorm(nbpgc)
      real(dp) pgccntx(nbpgc)
      real(dp) pgccnty(nbpgc)
      real(dp) pgccntz(nbpgc)
      real(dp) abscis(maxordgh,maxordgh)
      real(dp) weight(maxordgh,maxordgh)
  integer(dpi)pgcpowx(nbpgc)
  integer(dpi)pgcpowy(nbpgc)
  integer(dpi)pgcpowz(nbpgc)

! output arrays:
      real(dp) ovlerpg(nbpgc,nbpgc)

! local scalars:
  integer(dpi)gm, gn
  integer(dpi)lxQ, myQ, nzQ, Ordghx, Ordghy, Ordghz
  integer(dpi)lxN, myN, nzN
  integer(dpi)power, point
      real(dp) gamma, invgam, rtivgm
      real(dp) Q_fact, N_fact, d2_QtoN, prefac
      real(dp) x_QtoN, x_QtoP, x_NtoP
      real(dp) x_inte, x_ovlcnt, x_PQ, x_PN
      real(dp) y_QtoN, y_QtoP, y_NtoP
      real(dp) y_inte, y_ovlcnt, y_PQ, y_PN
      real(dp) z_QtoN, z_QtoP, z_NtoP
      real(dp) z_inte, z_ovlcnt, z_PQ, z_PN
  character(len=max_string_len_routine_name) :: l_here

! begin:
      l_here = 'gtos_overlap'
  call enter (l_here, 5)

      if(debug) then
         write (logf,'(a,i5)') &
      '   debug_gtos_overlap> # of primitives:', nbpgc
         write (logf,'(a,i5)') &
      '   debug_gtos_overlap>  maxordgh      :', maxordgh
      end if

! loop over first basis set <G_gm| :
      do gm = 1, nbpgc

! loop over second basis set |G_gn> :
        do gn = 1, nbpgc

! overlap exponent:
          gamma = pgcexpo(gm) + pgcexpo(gn)

      if(abs(gamma).lt. 1.d-14) then
      write (logf,'(a,g25.15)')' error_gtos_overlap> Gamma=',Gamma
      write (logf,'(a,i2,a,g25.15)')' > gm=',gm,' expo=',pgcexpo(gm)
      write (logf,'(a,i2,a,g25.15)')' > gn=',gn,' expo=',pgcexpo(gn)
      stop ' error_gtos_overlap> Gamma too small'
      end if

          invgam = one / gamma
          rtivgm = dsqrt(invgam)

! overlap center:
          x_ovlcnt = ( pgcexpo(gm) * pgccntX(gm) + &
                       pgcexpo(gn) * pgccntX(gn) ) * invgam
          y_ovlcnt = ( pgcexpo(gm) * pgccntY(gm) + &
                       pgcexpo(gn) * pgccntY(gn) ) * invgam
          z_ovlcnt = ( pgcexpo(gm) * pgccntZ(gm) + &
                       pgcexpo(gn) * pgccntZ(gn) ) * invgam

! prefactor:
          x_QtoN = pgccntX(gn) - pgccntX(gm)
          y_QtoN = pgccntY(gn) - pgccntY(gm)
          z_QtoN = pgccntZ(gn) - pgccntZ(gm)
          d2_QtoN = ( x_QtoN * x_QtoN +  y_QtoN * y_QtoN + z_QtoN * z_QtoN )

!         prefac = one / dexp(pgcexpo(gm) * pgcexpo(gn) * invgam * d2_QtoN)
          prefac = exp(- pgcexpo(gm) * pgcexpo(gn) * invgam * d2_QtoN)

      if(debug) then
       write (logf,'(a,i2)')' debug_gtos_overlap> gm=',gm
       write (logf,'(a,i2)')' debug_gtos_overlap> gn=',gn
       write (logf,'(a,f20.6)')' debug_gtos_overlap> xq=',pgccntX(gm)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> yq=',pgccnty(gm)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> zq=',pgccntz(gm)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> xn=',pgccntX(gn)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> yn=',pgccnty(gn)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> zn=',pgccntz(gn)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> nrm=',pgcnorm(gm)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> nrn=',pgcnorm(gn)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> x_QtoN=',x_QtoN
       write (logf,'(a,f20.6)')' debug_gtos_overlap> y_QtoN=',y_QtoN
       write (logf,'(a,f20.6)')' debug_gtos_overlap> z_QtoN=',z_QtoN
       write (logf,'(a,f20.6)')' debug_gtos_overlap> d2_QtoN=',d2_QtoN
       write (logf,'(a,f20.6)')' debug_gtos_overlap> invgam=',invgam
       write (logf,'(a,f20.6)')' debug_gtos_overlap> pgcexpo(gm)=',pgcexpo(gm)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> pgcexpo(gn)=',pgcexpo(gn)
       write (logf,'(a,f20.6)')' debug_gtos_overlap> prefac=',prefac
      end if

! vectors of centers to overlap center:
          x_QtoP = x_ovlcnt - pgccntX(gm)
          y_QtoP = y_ovlcnt - pgccntY(gm)
          z_QtoP = z_ovlcnt - pgccntZ(gm)

          x_NtoP = x_ovlcnt - pgccntX(gn)
          y_NtoP = y_ovlcnt - pgccntY(gn)
          z_NtoP = z_ovlcnt - pgccntZ(gn)

! azimuth exponents:
          lxQ = pgcpowX(gm)
          myQ = pgcpowY(gm)
          nzQ = pgcpowZ(gm)

          lxN = pgcpowX(gn)
          myN = pgcpowY(gn)
          nzN = pgcpowZ(gn)

! calculate x,y,z-components of integral:
          x_inte = zero
          y_inte = zero
          z_inte = zero

          OrdGHx =  max(2, (lxQ + lxN  + 2)/2)
          OrdGHy =  max(2, (myQ + myN  + 2)/2)
          OrdGHz =  max(2, (nzQ + nzN  + 2)/2)

          if(debug) then
           write (logf,'(a,i20)')' debug_gtos_overlap> lxQ=',lxQ
           write (logf,'(a,i20)')' debug_gtos_overlap> myQ=',myQ
           write (logf,'(a,i20)')' debug_gtos_overlap> nzQ=',nzQ
           write (logf,'(a,i20)')' debug_gtos_overlap> lxN=',lxN
           write (logf,'(a,i20)')' debug_gtos_overlap> myN=',myN
           write (logf,'(a,i20)')' debug_gtos_overlap> nzN=',nzN
           write (logf,'(a,i20)')' debug_gtos_overlap> OrdGHx=',OrdGHx
           write (logf,'(a,i20)')' debug_gtos_overlap> OrdGHy=',OrdGHy
           write (logf,'(a,i20)')' debug_gtos_overlap> OrdGHz=',OrdGHz
          end if

          if(OrdGHx.gt.MaxOrdGh .or.    &
             OrdGHy.gt.MaxOrdGh .or.    &
             OrdGHz.gt.MaxOrdGh) then
          write (logf,'(a,i20)')' error_gtos_overlap> OrdGHx=',OrdGHx
          write (logf,'(a,i20)')' error_gtos_overlap> OrdGHy=',OrdGHy
          write (logf,'(a,i20)')' error_gtos_overlap> OrdGHz=',OrdGHz
          write (logf,'(a,i2)')' error_gtos_overlap> MaxOrdGH=',MaxOrdGH
          stop ' error_gtos_overlap> MaxOrdGH exceeded'
          end if
          do point = 1, OrdGHx !. min. # of G.-Hermite int. points = 2

            x_PQ = abscis(point, OrdGHx) * rtivgm + x_QtoP
            Q_fact = one
            do power = 1, lxQ
              Q_fact = Q_fact * x_PQ
            end do

            x_PN = abscis(point, OrdGHx) * rtivgm + x_NtoP
            N_fact = one
            do power = 1, lxN
              N_fact = N_fact * x_PN
            end do

            x_inte = x_inte + weight(point, OrdGHx) * Q_fact * N_fact

          if(debug) then
           write (logf,'(a,f20.6)')   &
         ' debug_gtos_overlap> Absciss=',Abscis(point, OrdGHx)
           write (logf,'(a,f20.6)')   &
         ' debug_gtos_overlap> Weight=',weight(point, OrdGHx)
           write (logf,'(a,f20.6)')' debug_gtos_overlap> x_inte=',x_inte
          end if

          end do   ! point

          if(debug) then
           write (logf,'(a,f20.6)')' debug_gtos_overlap> Q_fact=',Q_fact
           write (logf,'(a,f20.6)')' debug_gtos_overlap> N_fact=',N_fact
          end if

          do point = 1, OrdGHy !. min. # of G.-Hermite int. points = 2

            y_PQ = abscis(point, OrdGHy) * rtivgm + y_QtoP
            Q_fact = one
            do power = 1, myQ
              Q_fact = Q_fact * y_PQ
            end do

            y_PN = abscis(point, OrdGHy) * rtivgm + y_NtoP
            N_fact = one
            do power = 1, myN
              N_fact = N_fact * y_PN
            end do

            y_inte = y_inte + weight(point, OrdGHy) * Q_fact * N_fact

          if(debug) then
           write (logf,'(a,f20.6)')' debug_gtos_overlap> y_inte=',y_inte
          end if

          end do ! point

          if(debug) then
           write (logf,'(a,f20.6)')' debug_gtos_overlap> Q_fact=',Q_fact
           write (logf,'(a,f20.6)')' debug_gtos_overlap> N_fact=',N_fact
          end if

          do point = 1, OrdGHz !. min. # of G.-Hermite int. points = 2

            z_PQ = abscis(point, OrdGHz) * rtivgm + z_QtoP
            Q_fact = one
            do power = 1, nzQ
              Q_fact = Q_fact * z_PQ
            end do

            z_PN = abscis(point, OrdGHz) * rtivgm + z_NtoP
            N_fact = one
            do power = 1, nzN
              N_fact = N_fact * z_PN
            end do

            z_inte = z_inte + weight(point, OrdGHz) * Q_fact * N_fact

          if(debug) then
           write (logf,'(a,f20.6)')' debug_gtos_overlap> z_inte=',z_inte
          end if

          end do !. G.-Hermite integration points

          if(debug) then
           write (logf,'(a,f20.6)')' debug_gtos_overlap> Q_fact=',Q_fact
           write (logf,'(a,f20.6)')' debug_gtos_overlap> N_fact=',N_fact
          end if

          x_inte = x_inte * rtivgm
          y_inte = y_inte * rtivgm
          z_inte = z_inte * rtivgm

          if(debug) then
           write (logf,'(a,f20.6)')' debug_gtos_overlap> x_inte=',x_inte
           write (logf,'(a,f20.6)')' debug_gtos_overlap> y_inte=',y_inte
           write (logf,'(a,f20.6)')' debug_gtos_overlap> z_inte=',z_inte
          end if

! total integral:
          ovlerpg(gm,gn) = pgcnorm(gm) * pgcnorm(gn) *  &
                           x_inte * y_inte * z_inte * prefac

        end do !. end loop over |G_gn>

      end do !. end loop over <G_gm|

      if(debug) then
        do gm = 1,nbpgc
        do gn = 1,nbpgc
        write (logf,'(2(a,i3),a,f20.6)') &
      ' debug_gtos_overlap> gm=',gm,' gn=',gn,' ovl=',ovlerpg(gm,gn)
        end do
        end do
      end if

  call exit (l_here, 5)

  end subroutine gtos_overlap 

! ========================================================================
  function gto_2_int (n, a) result (result)
! ------------------------------------------------------------------------
!
! Name           : gto_2_int
!
! Argument (in ) : n
! Argument (in ) : a
!
! Description    : evaluate the integral of (x^n*exp(-a*x^2))**2
!
! Authors:X. Krokidis
! Date  :23 Oct 2000
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent(in)    :: n
  real(dp), intent(in)        :: a

! result
  real(dp)                    :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)                    :: g
  real(dp)                    :: PI12

! begin:
  l_here ='gto_2_int'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(routines_21),'-d: exponent=:',a
   write (logf,*)trim(routines_21),'-d: x-power =:',n
  end if

    PI12 = SQRT(PI/TWO)

    g = dblefactorial(2*n -1)
    g = g / (two**(2*n) * a**n)
    g = g * PI12 / sqrt(a)

  result = g

  if(debug) write (logf,*)trim(routines_21),'-d: gto_2_int=',g

  call exit (l_here,5)

  end function gto_2_int

! ========================================================================
  function sto_xyzr_int (a, b, c, n, g) result(result)
! ------------------------------------------------------------------------
!
! Name           : sto_xyzr_int
!
! Argument (in ) : a
! Argument (in ) : b
! Argument (in ) : c
! Argument (in ) : n
! Argument (in ) : g
!
! Description    : evaluate the integral of x^a y^b z^c r^n exp(-g*r)
!  
! Authors:R. Assaraf
! Date  : 29 Nov 2001
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent(in) :: a, b, c,n
  real(dp), intent(in)     :: g

! result
  real(dp)                 :: result
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)                 :: s

! begin:
  l_here ='sto_xyzr_int'
  call enter (l_here,5)
 
  if(debug) then
   write (logf,*)trim(routines_21),'-d: x-power =:',a
   write (logf,*)trim(routines_21),'-d: y-power =:',b
   write (logf,*)trim(routines_21),'-d: z-power =:',c
   write (logf,*)trim(routines_21),'-d: r-power =:',n
   write (logf,*)trim(routines_21),'-d: exponent=:',g
  end if 

! improve: assertions on a,b,c,n,g

  if ( mod (a+1,2) == 0 .or.  mod (b+1,2) == 0  .or.  mod (c+1,2) == 0 ) then

   s = 0.d0

  else 

    s = FOURPI*factorial(a+b+c +2+n)/dblefactorial(a +b +c+1)
    s = s/(g**(a+b+c+n +3))
    s = s*dblefactorial(a -1)*dblefactorial(b -1)*dblefactorial(c -1)

  end if

  result  = s

  if(debug) write (logf,*)trim(routines_21),'-d: sto_xyzr_int=',s

  call exit (l_here,5)

  end function sto_xyzr_int

! ========================================================================
  function gauss_xyzr_int (a, b, c, n, g) result(result)
! ------------------------------------------------------------------------
!
! Name           : gauss_xyzr_int
!
! Argument (in ) : a
! Argument (in ) : b
! Argument (in ) : c
! Argument (in ) : n
! Argument (in ) : g
!
! Description    : evaluate the integral of x^a y^b z^c r^n exp(-g*r^2)
!  
! Authors: R. Assaraf
! Date  :29 Nov 2001
! 
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), intent(in) :: a, b, c,n
  real(dp), intent(in)     :: g
  integer(dpi)             :: p
! result 
  real(dp)                 :: result
  
! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)                 :: s

! begin:
  l_here ='gauss_xyzr_int'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(routines_21),'-d: x-power =:',a
   write (logf,*)trim(routines_21),'-d: y-power =:',b
   write (logf,*)trim(routines_21),'-d: z-power =:',c
   write (logf,*)trim(routines_21),'-d: r-power =:',n
   write (logf,*)trim(routines_21),'-d: exponent=:',g
  end if 
  
  if ( mod (a+1,2) == 0 .or.  mod (b+1,2) == 0  .or.  mod (c+1,2) == 0 ) then

   s = 0.d0

  else
    s = FOURPI *  dblefactorial(a -1) * dblefactorial(b -1) * dblefactorial(c -1) &
               / dblefactorial(a +b +c+1) 

    if (mod (n+1,2)+1 == 1 ) then  ! n odd
       s = s * (g**( -float (a+b+c+n+3) /2.d0))  &
           * factorial((a+b+c+n+1)/2)
    else
       p = (n+a+b+c+2)/2
       s = s * g**(-p-0.5) / 2**(p+1) & 
           * dblefactorial(2*p-1)*sqrt(PI)
    endif

  end if

  result = s
  
  if(debug) write (logf,*)trim(routines_21),'-d: gauss_xyzr_int=',s

  call exit (l_here,5)
  
  end function gauss_xyzr_int

! ========================================================================
  function slater_dot_gauss (a, b, n) result(result)
! ------------------------------------------------------------------------
!
! Name           : slater_dot_gauss
!
! Argument (in ) : a
! Argument (in ) : b
! Argument (in)  : a
!
! Description    : evaluate the scalar product of a gaussian 
! Description    : exp (-ar^2) and a slater r^n exp (-b*r)
! Description    : if modified modify also slater_dot_slater
!
! Authors: R. Assaraf
! Date  :20 Apr 2003
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), intent(in)     :: a, b
  integer(dpi), intent(in) :: n
! result 
  real(dp)                 :: result

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: p
  real(dp) :: dot_rpow(0:3), dot_r0_der, dot_r2_der
! begin:
  l_here ='slater_dot_gauss'
  call enter (l_here,5)

! dot_pow (n) = int r^p * r^n e^{-br} e{-ar^2}
  do p = 0, 3
     dot_rpow(p) = gauss_exp_rn_1d_int (a, b, n+p)  
  enddo ! do p

! dot_rp_der = int r^p * d/dx(r^n e^{-bx}) . d/dx (e^{-ax^2})
  
  dot_r0_der = 2*a*b*dot_rpow(1)-2*a*n*dot_rpow(0)
  dot_r2_der = 2*a*b*dot_rpow(3)-2*a*n*dot_rpow(2)

  result =   dot_rpow (0)

  call exit (l_here,5)
  
  end function slater_dot_gauss

! ========================================================================
  function slater_dot_slater (z1, z2, n1, n2) result(result)
! ------------------------------------------------------------------------
!
! Name           : slater_dot_gauss
!
! Argument (in ) : z1
! Argument (in ) : z2
! Argument (in)  : n1, n2
! Description    : evaluate the scalar product of two slaters
! Description    : r^n1 e^{- z1 r}, and r^n2 e^{-z2 r}
! Description    : if modified modify also slater_dot_gauss
!
! Authors: R. Assaraf
! Date  :23 Apr 2003
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), intent(in)     :: z1,z2
  integer(dpi), intent(in) :: n1,n2

! result 
  real(dp)                 :: result


! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp)     :: dot_rpow (-4:2)
  real(dp)     :: dot_r0_der, dot_r2_der, dot_r0_der2 
  integer(dpi) :: p
! begin:
  l_here ='slater_dot_gauss'
  call enter (l_here,5)

  dot_rpow = 0.d0

! Dot_pow (p) = int r^p * r^n1 r^n2 e^{-z1 r} e{-z2 r}
  do p = -4, 2
     if (n1+n2+p >= 0) &
          dot_rpow (p) = 1.d0 / (z1+z2)**(n1+n2+1+p)*factorial (n1+n2+p)
  enddo ! do n

! dot_rp_der = int r^p * d/dx(r^n1 e^{-z1 x}) . d/dx (r^n2 e^{-z2 x})

  dot_r0_der = n1*n2*dot_rpow(-2)+z1*z2*dot_rpow(0)-(z1*n2+z2*n1)*dot_rpow(-1)
  dot_r2_der = n1*n2*dot_rpow(0)+z1*z2*dot_rpow(2)-(z1*n2+z2*n1)*dot_rpow(1)

  dot_r0_der2 = n1*n2*(n1-1)*(n2-1)*dot_rpow(-4) &
               -2*n1*n2*(z1*(n2-1)+z2*(n1-1)) *dot_rpow (-3) &
                +z1*z2*(4*n1*n2+z1*n2*(n2-1)+z2*n1*(n1-1)) *dot_rpow(-2) &
                -2*(z1*n1*z2**2+z2*n2*z1**2)*dot_rpow(-1) &
                + z1**2*z2**2*dot_rpow (0)

  result =   dot_rpow(0)
  
  call exit (l_here,5)
  
  end function slater_dot_slater

! ========================================================================
  function gauss_exp_rn_1d_int (a, b, n) result(result)
! ------------------------------------------------------------------------
!
! Name           : gauss_exp_rn_1d_int
!
! Argument (in ) : a
! Argument (in ) : b
! Argument (in)  : n
! Description    : evaluate the integral of r**n e^{-a r^2-br} on [0,inf[
!  
! Authors: R. Assaraf
! Date   : 21 Apr 2003
! 
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), intent(in) :: a, b
  integer(dpi)         :: n
! result 
  real(dp)             :: result
 
! local:
  character(len=max_string_len_routine_name) :: l_here
  real(dp) :: x,y, gamma, erf_spline
  integer(dpi) :: i
  real(dp)     :: fac, gamma_der (1:5)
! begin:
  l_here ='gauss_exp_rn_1d_int'
  call enter (l_here,5)

! with the new variable 
 

 x = b/2.d0/sqrt(a)

! we use the equality 

! sqrt(a) int_[0,inf[  e^{-ar^2-br}dr = e^{x^2} int_[x,inf[ e^{-r^2} dr

!  introducing
!   gamma = int_[0,inf[ e^{-r^2-2rx} dr 
!         = exp (x**2) * (1.d0 - erf_spline (x)) * sqrt (pi)/2.d0 
! int_[0,inf[ e^{-ar^2-br} r^n dr = 1/sqrt(a) (-d/db)^n gamma
!                                 = 1/sqrt(a) (-1/2/sqrt(a))^n (d/dx)^n gamma


! we distinguish x small and x big to compute gamma
 
 if (x>5.d0) then 

! introducing
    y=-1.d0/(2.d0*x)**2
! gamma = 1/2x *(\sum_n (2n)!/ n! *  y^n)
    fac = 1.d0
    gamma = 1.d0
    do i = 1, 20
       fac = (2*i)*(2*i-1)*fac/float(i)*y 
       gamma = gamma + fac 
    enddo ! do i
   
    gamma = 1.d0 /(2.d0*x) * gamma
 
 else 
    gamma =  exp (x**2) * (1.d0 - erf_spline (x)) * sqrt (pi)/2.d0

 endif
 

! we suppose a = 1
! the prefactor will be added at the end
  
   gamma_der (1) =  2.d0 * x * gamma - 1.d0
   gamma_der (2) =  2.d0 * x * gamma_der (1) + 2.d0 * gamma
   gamma_der (3) =  2.d0 * x * gamma_der (2) + 4.d0 * gamma_der (1)
   gamma_der (4) =  2.d0 * x * gamma_der (3) + 6.d0 * gamma_der (2)
   gamma_der (5) =  2.d0 * x * gamma_der (4) + 8.d0 * gamma_der (3)

   select case (n)
  
   case (0) 
   
      result = gamma 

   case (1)
      
      result = gamma_der (1)
     
   case (2)
   
      result = gamma_der (2)

  case (3) 

      result = gamma_der (3)

  case (4)
      
      result = gamma_der (4)

  case (5) 
    
      result = gamma_der (5)

  case default
 
    call die (l_here, ' n='+n+' not implemented')

  end select

! prefactor 
  result = result / sqrt(a) 
  if (n>0) result = result *  (-0.5d0/sqrt(a))**n
  if (debug) write (logf,*) '-d: a,b,n result',a,b,n,result
  call exit (l_here,5)
  
end function gauss_exp_rn_1d_int


! ========================================================================
  function factorial (i) 
  implicit none

  integer(dpi) factorial
  integer(dpi) dum,i,j
  if (i < 0) stop 'factorial (i) with i < 0'
      dum = 1
      do j = 2,i
        dum = dum * j
      end do   
      factorial = dum
  end function factorial
! ========================================================================
  function dblefactorial (i) 
  implicit none
  integer(dpi) dblefactorial
  integer(dpi) dum,i,j
  if (i < -1) stop 'dblefactorial (i) with i < -1'

      dum = 1
      do j = i,2,-2
        dum = dum * j
      end do   
      dblefactorial = dum
  end function dblefactorial

end module integration_mod

