! -*- F90  -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------


 module intr_func_mod
 use basic_types_tools_mod
 use expressions_mod
 use integration_mod
 implicit none

contains


! ========================================================================
  subroutine sample_av_calc
! ------------------------------------------------------------------------
! Name :  sample_av_bld
!
! Description: actualize sample_av from real_1
! Description: ...
!
! Authors: R. Assaraf, Alexandre Kollias
! Date   : 30 May
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
   character(len=max_string_len_routine_name),save :: l_here
   real(dp), pointer     :: av_sample (:)
   real(dp), pointer     :: real_1 (:)
   integer(dpi), save    :: av_sample_size_max = 2000 
   integer(dpi)          :: av_i, i
! improve no fixed  upper bound

   l_here ='sample_av_calc'
   if (header_exe) then

    l_here = "sample_av_calc"

    return

  end if

! begin:
  call enter (l_here,1)

   call alias_byaddr_bld  (av_sample, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (real_1, objects_addr (cur_needed_ind (1)))
  
   if (.not.associated (av_sample)) then
      call alloc (objects_names (cur_create_ind (1)), av_sample, av_sample_size_max )
      av_sample  = NOT_INIT_DP
   endif

   write (logf,*) trim(l_here),'-d: av_sample_size_max',av_sample_size_max
   do av_i =1, av_sample_size_max
      if (debug) then
         write (logf,*) trim(l_here),'-d: av_i=',av_i
         write (logf,*) trim(l_here),'-d:  av_sample(av_i)=',av_sample(av_i)
         write (logf,*) trim(l_here),'-d:  av_sample (av_i)- NOT_INIT_DP=',av_sample (av_i)- NOT_INIT_DP
         write (logf,*) trim(l_here),'-d:   abs (spacing(NOT_INIT_DP))=', abs (spacing(NOT_INIT_DP))
      endif

      if ( abs ( av_sample (av_i)- NOT_INIT_DP) < abs (spacing(NOT_INIT_DP))) then
         if (debug) then
          
           write (logf,*) trim(l_here),'-d: condition av_sample = NOT_INIT_DP satisfied'
           write (logf,*) trim(l_here),'-d: modification of the component #',av_i
         endif

       
         av_sample (av_i) = 0.d0
         
         do i = 1, size (real_1)
            av_sample (av_i) = av_sample (av_i) + real_1 (i)
         enddo ! do i
         
         av_sample (av_i) =  av_sample (av_i) / size (real_1)
         call exit(l_here)
         return
      

      endif

   enddo ! do av_i

   call die (l_here, 'av_sample_size_max='+av_sample_size_max+'too small')

   call exit (l_here, 1)
  
 end subroutine sample_av_calc

! ========================================================================
  subroutine real_array_calc
! ------------------------------------------------------------------------
! Name :  real_array_calc
!
! Description: actualize real_array a real
! Description: Patch
!
! Authors: R. Assaraf and Alexandre Kollias
! Date   : 30 May 2005
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
   character(len=max_string_len_routine_name),save :: l_here
   real(dp), pointer     :: real_var
   real(dp), pointer     :: real_array (:)
   integer (dpi)         :: real_array_size_max = 2000
   integer(dpi)          :: i
   l_here ='real_array_calc'
   if (header_exe) then

    l_here = "real_array_calc"

    return

  end if

! begin:
  call enter (l_here,1)

   call alias_byaddr_bld  (real_array, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld  (real_var, objects_addr (cur_needed_ind (1)))
  
   if (.not.associated (real_array)) then
      call alloc (objects_names (cur_create_ind (1)), real_array, real_array_size_max )
      real_array = NOT_INIT_DP
   endif

   do i=1,  real_array_size_max

      if ( abs (real_array (i)- NOT_INIT_DP) < abs (spacing(NOT_INIT_DP))) then
         real_array (i) = real_var
         call exit (l_here,1)
         return
      endif
 
   enddo
   
   call die (l_here,"real_array_size_max="+real_array_size_max+"to small")
   
   call exit (l_here, 1)
  
 end subroutine real_array_calc



! ========================================================================
  subroutine average_calc
! ------------------------------------------------------------------------
!
! Name           : average_bld
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp), pointer     :: average
  real(dp), pointer     :: real_1_sample (:)
  integer(dpi) :: sample_nb
  integer(dpi) :: i
! header:
  if (header_exe) then
    l_here = "average_calc"
    return
  endif
! begin:
  call enter (l_here)
  
  call alias_byaddr_bld  (average, objects_addr (cur_create_ind (1)))
  call alias_byaddr_bld (real_1_sample, objects_addr (cur_needed_ind (1)))
  
  if (.not.associated (average)) then
     call alloc (objects_names (cur_create_ind (1)), average)
  end if

  if (.not.associated (real_1_sample)) call die (l_here, 'real_1_sample not associated')

  if (size (real_1_sample) <= 0) call die (l_here, 'size(real_1_sample)='&
       +size(real_1_sample))

  do i = 1,size (real_1_sample)

     if  ( abs (real_1_sample (i)-NOT_INIT_DP) < abs(spacing(NOT_INIT_DP))) then 
        sample_nb = i-1
        exit
     else if (i== size (real_1_sample)) then
        sample_nb = i
     endif

  enddo ! do i

  average = 0.d0
  
  do i = 1,sample_nb 
     
     average = average+real_1_sample (i)
   
  enddo
 

  if (sample_nb <= 0) then
     average  = NOT_INIT_DP
  else
     average = average / float (sample_nb)
  endif

  call exit (l_here)
  
end subroutine average_calc


! ========================================================================
  subroutine real_1_average_calc
! ------------------------------------------------------------------------
!
! Name           : real_1_average_bld
!
! Description    :  average
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp), pointer     :: averages  (:)
  real(dp), pointer     :: rand_vars (:)
  real(dp), pointer     :: variances (:)
  real(dp), pointer     :: errors    (:)
  character(len=max_string_len_object_name)   :: create (4)
  
  integer(dpi), pointer :: sample_nb
  integer(dpi) :: i
! header:
  if (header_exe) then
     l_here = "real_1_average_calc"
     if (create_index /= NOT_INIT_I) then
     
        create(1)='array_av{'+objects_names(needs(1))+'}'
        create(2)='array_err{'+objects_names(needs(1))+'}'
        create(3)='array_var{'+objects_names(needs(1))+'}'
        create(4)='nb_'+objects_names(needs(1))
        call node_create ('array_av', create, objects_names(needs(1:1)))
        create_index = object_index(create(1))
   
     endif
     return
  endif
! begin:
  call enter (l_here)
  
  call alias_byaddr_bld  (averages,  objects_addr (cur_create_ind (1)))
  call alias_byaddr_bld  (errors,    objects_addr (cur_create_ind (2)))
  call alias_byaddr_bld  (variances, objects_addr (cur_create_ind (3)))
  call alias_byaddr_bld  (sample_nb, objects_addr (cur_create_ind (4)))
  write (logf,*) trim (l_here),' address (sample_nb)', objects_addr (cur_create_ind (4))
  if (associated (sample_nb)) write (logf,*) trim (l_here), '  after address sample_nb=', sample_nb
  call alias_byaddr_bld (rand_vars, objects_addr (cur_needed_ind (1)))
    
  if (.not.associated (rand_vars)) call die (l_here, 'rand_vars not associated')
  if (.not.associated (averages)) then
     call alloc (objects_names (cur_create_ind (1)), averages,   size (rand_vars))
     averages = 0.d0
   endif
  
  call alloc (objects_names (cur_create_ind (2)), errors,     size (rand_vars))
  call alloc (objects_names (cur_create_ind (3)), variances,  size (rand_vars))
  if (.not.associated (sample_nb)) then
     call alloc (objects_names (cur_create_ind (4)), sample_nb)
     sample_nb = 0
   endif

  sample_nb = sample_nb + 1
  averages = (averages * (sample_nb-1) + rand_vars)/sample_nb

  variances = (variances * (sample_nb-1) + rand_vars * rand_vars)/sample_nb 

  

  if (sample_nb <= 1) then
     errors = NOT_INIT_DP
  else
     do i = 1, size (errors)
        errors (i) = sqrt (abs(variances (i) -averages(i)**2) ) &
        / float (sample_nb-1)
     enddo ! do i
  endif
 
  write (logf,*) trim (l_here), ' sample_nb = ', sample_nb, ' averages  =', averages
  write (logf,*) trim (l_here), ' sample_nb = ', sample_nb, ' rand_vars =', rand_vars
 write (logf,*) trim (l_here), ' sample_nb = ', sample_nb, ' errors  =', errors

  call exit (l_here)
  
end subroutine real_1_average_calc

! =======================================================================
     recursive  subroutine err_calc
! ------------------------------------------------------------------------
!
! Name           : err_calc
!
! Description    : operator error "err(x)" is the statistical error on x.
! 
! Improve: A mess 
!
! Authors: R. Assaraf
! Date  : 13 Fev 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name), save :: l_here
  character(len=max_string_len_object_name)        :: create (3)
  integer(dpi) :: nod_i, need_index
  integer(dpi) :: average_index
  real(dp), pointer  :: error, target
  real(dp)  :: eps
  integer(dpi), save :: av_nb_max = 20
  integer(dpi)       :: av_nb
  integer(dpi)       :: sample_nb
  integer(dpi)       :: i, j
  integer(dpi)       :: sample_index
  type (real_0_type), pointer, save :: average_aliases (:) => NULL ()
  type (real_1_type), pointer, save :: sample_aliases  (:) => NULL ()
  
  real(dp), pointer, save :: covariances (:,:) => NULL ()
  real(dp), pointer, save :: derivatives (:)   => NULL ()    
  real(dp) :: target_save 

  integer(dpi), pointer :: averages (:)
  integer(dpi), pointer :: path_loc (:)
! header

 
  if (header_exe) then
     if (create_index /= NOT_INIT_I) then

        l_here = 'err_calc'
        call enter (l_here) 
!  averages on which object need(1) depends on  put in samples 

        nullify (averages)
        nullify (path_loc)
   
        nod_i = node_from_operation ('err', objects_names(needs(1:1)))
   
        if (nod_i == 0)  then
! need_index is the object on which the statistical error has to be calculated
           need_index = needs(1)
 
! building the averages on which need_index depends 
           do nod_i = 1, nodes_nb 
              if (routines_name(nodes_rtn (nod_i)) == 'av' .and. &
                   nodes_create%first(nod_i) > 0 ) then
              
                 average_index = nodes_create%val(nodes_create%first(nod_i))
             
                 if (root2target_path_is (average_index, need_index)) then
                    
                    call integer_in_list_add (average_index, averages)
                 
                 endif
              endif
              
           enddo ! do nod_i

           call integer_array_reduce (averages)
        
           if (debug) then 
              write (logf,*) trim(l_here), '-d: object: ', trim(objects_names (need_index))
              write (logf,*) trim(l_here), '-d: depends on averages : ',&
                   (' >'//trim(objects_names (averages(average_index)))//'< ', average_index=1,size(averages))
           endif

! building path_loc, path from averages to need_index
        
           if (associated(averages)) then
              if (size(averages) > 0) then
                 call  path_bynod_bld (path_loc, averages, (/need_index/))
              endif
           endif

           debug_path_loc:  if (debug) then
              if (associated (path_loc)) then
                 write (logf,*) trim (l_here), '-d: path_loc:'
                 do nod_i = 1, size (path_loc)
                    call one_node_write (path_loc (nod_i), logf, l_here)
                 enddo ! do nod_i
              else
                 write (logf,*) trim (l_here), '-d: path_loc: not associated'
              endif
           endif debug_path_loc
! builds the creates of the current node 

           create(1)='err{'+objects_names(needs(1))+'}'
           create(2)='path{'+objects_names(needs(1))+'}'
           create(3)='av{'+objects_names(needs(1))+'}'
           call catalog (create(2),path_loc)
           call catalog (create(3),averages)
           call node_create ('err', create,objects_names(needs(1:1)))
           nod_i = node_from_operation ('err', objects_names(needs(1:1)))
     
           create_index = nodes_create%val(nodes_create%first(nod_i))
     
        endif ! nod_i
        call exit (l_here)
     endif
     return
  endif
   
  call enter (l_here)

  call alias_byaddr_bld (error, objects_addr (cur_create_ind (1)))
  call alias_byaddr_bld (path_loc, objects_addr (cur_create_ind (2)))
  call alias_byaddr_bld (averages, objects_addr (cur_create_ind (3)))

  call alias_byaddr_bld (target, objects_addr (cur_needed_ind (1)))
 
  if (.not.associated (target)) call die (l_here, 'target not associated')

  if (debug) write (logf,*) trim(l_here), '-d: target name >',&
       trim( objects_names (cur_needed_ind(1))),'<'  
  if (debug) write (logf,*) trim(l_here), '-d: target=',target

  if (.not.associated (error)) then
     call alloc (objects_names  (cur_create_ind(1)),error)
  endif

  if (.not.associated (average_aliases)) then
     allocate (average_aliases (av_nb_max))
     allocate (sample_aliases (av_nb_max))
     allocate (covariances (av_nb_max,av_nb_max))
     allocate (derivatives (av_nb_max))
  endif

  av_nb = size(averages)

  if (av_nb == 0) then
     call die (l_here, 'seems that '*objects_names (cur_create_ind(1))*&
          'does not depend on estimators, if the statistical '* &
          'error has a meaning it is 0.')
  endif

  if (av_nb > av_nb_max)  call die (l_here, 'av_nb='+av_nb+'>av_nb_max='+av_nb_max)
 
! average_aliases are  the averages
  do i = 1, av_nb
     call alias_byaddr_bld (average_aliases(i)%p, objects_addr (averages (i)))
     if (debug) write (logf,*) trim(l_here),'-d: average #',i,&
          '>',trim(objects_names (averages(i))),'< =',average_aliases(i)%p
  enddo

! sample_aliases are the samples
  do i = 1, av_nb
    nod_i = objects_node (averages  (i))
    sample_index = nodes_need%val(nodes_need%first(nod_i))
    call alias_byaddr_bld (sample_aliases(i)%p, objects_addr (sample_index))
  enddo ! do i

  if (debug) then
     do i=1, av_nb
      write (logf,*) trim(l_here), '-d: sample_aliases(',i,')', sample_aliases(i)%p
      call object_value_byind_write (sample_index,logf)
     enddo ! do i
  endif
  
! derivatives
  eps = 0.000001d0
  derivatives=0.d0

  target_save = target
  
! check target in correspondance with path_loc
  call  path_bynod_exe (path_loc)
  if (target /= target_save) &
       call die (l_here,'target='+target+'/= target_save='+target_save)

  do i = 1, av_nb
     if (debug) write (logf,*) trim(l_here),'-d: average_aliases (',i,')%p=',average_aliases (i)%p
    
     average_aliases (i)%p=average_aliases (i)%p+eps

     debug_path:   if (debug) then
        write (logf,*) trim(l_here), '-d: derivative of target / average #',i 
        write (logf,*) trim(l_here), '-d: path from averages to target:'

        if (associated (path_loc)) then
           call path_bynod_print (path_loc)
        else
           write (logf,*) trim(l_here), '-d: path_loc not associated'
        endif
        write (logf,*) trim(l_here), &
             '-d: average_aliases (',i,')%p=',average_aliases (i)%p
     endif debug_path
  
     call path_bynod_exe (path_loc)
    
     if (debug) write (logf,*) trim(l_here), '-d: new target=',target
     if (debug) write (logf,*) trim(l_here), '-d: old target=',target_save

     derivatives (i) = (target-target_save)/eps
     average_aliases (i)%p=average_aliases (i)%p-eps
  enddo ! do i
  if (debug) write(logf,*) trim(l_here), '-d: derivatives=',derivatives (1:av_nb) 

!  target = target_save
!  this is not sufficient, all the objects in the path have to bee refreshed
  call path_bynod_exe (path_loc)

! real size off the sample (without final NOT_INIT_DP)
  sample_nb = real_array_reduced_size (sample_aliases(1)%p)

  if (sample_nb<2) then
     error=NOT_INIT_DP
     call exit(l_here)
     return
  endif

! covariances
  do i=1, av_nb
     do j=1, av_nb
        covariances(i,j)=sum (sample_aliases(i)%p(1:sample_nb)*sample_aliases(j)%p(1:sample_nb))&
             -sum(sample_aliases(i)%p(1:sample_nb))*sum(sample_aliases(j)%p(1:sample_nb))/sample_nb
        covariances(i,j)=covariances(i,j)/sample_nb
     enddo ! do j
  enddo ! do i   

  if (debug) then
     do i=1, av_nb
        do j=1, av_nb
           write (logf,*) trim(l_here),'-d: covariances(',i,',',j,')=',covariances(i,j)
        enddo ! do j
     enddo ! do i
  endif

  error=0_dp
  do i=1, av_nb
     do j=1, av_nb
        error=error + covariances (i,j)*derivatives(i)*derivatives(j)
     enddo ! do j
  enddo  ! do i

  if (error<-100*eps) call die (l_here,'error='+error+'< should be >o')
  error=sqrt(abs(error)/(sample_nb-1))

  call exit (l_here)
end subroutine err_calc

! ========================================================================         
 recursive  subroutine cov_calc
! ------------------------------------------------------------------------
!
! Name           : cov_calc
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 13 Fev 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name), save :: l_here
      
! begin:
  if (header_exe) then
     l_here = 'cov_calc'

     if (create_index /= NOT_INIT_I) then
   
        create_index = object_from_expression ( &
             'av(('+objects_names(needs(1))+')*('+objects_names(needs(2))+'))'+ &
             '-av('+objects_names(needs(1))+')*av('+objects_names(needs(2))+')')

     endif
     return
  endif

  call die (l_here, 'this routine should not be called')

  end subroutine cov_calc

! ========================================================================         
  subroutine array_element_calc
! ------------------------------------------------------------------------
!
! Name           : array_element_calc
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 26 Fev 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  real(dp), pointer :: array(:)
  real(dp), pointer :: array_2 (:,:)  
  real(dp), pointer :: array_3 (:,:,:)
  real(dp), pointer :: array_4 (:,:,:,:) 
  integer(dpi), pointer :: i1,i2,i3, i4
  real(dp), pointer     :: elt
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: dim_nb
! begin:
  if (header_exe) then
  l_here = 'array_element_calc'  
  return
  endif
  call enter (l_here)

  dim_nb =  nodes_need%last(cur_nod)-nodes_need%first(cur_nod)
  if (dim_nb == 1) then
     call alias_byaddr_bld (elt, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (array, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (i1, objects_addr (cur_needed_ind (2)))
   
     if (.not.associated (i1)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(2)=',cur_needed_ind(2)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
          trim(objects_names(cur_needed_ind(2))),'< has an address=',address(cur_needed_ind(2))
        call die (l_here, 'i1 not associated')
     endif
                  

     if (.not.associated (elt)) then   
        allocate(real_0_type_alias)
        real_0_type_alias%p =>  array(i1)
        objects_addr (cur_create_ind (1)) = address (real_0_type_alias)
        objects_type (cur_create_ind (1)) = string_in_array_locate ('real_0',types_name)
      
     endif
  else if (dim_nb == 2) then
     call alias_byaddr_bld (elt, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (array_2, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (i1, objects_addr (cur_needed_ind (2)))
     call alias_byaddr_bld (i2, objects_addr (cur_needed_ind (3)))   
     if (.not.associated (i1)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(2)=',cur_needed_ind(2)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
             objects_names(cur_needed_ind(2)),'< has an address=',address(cur_needed_ind(2))
        call die (l_here, 'i1 not associated')
     endif
                  
     if (.not.associated (i2)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(3)=',cur_needed_ind(3)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
            trim(objects_names(cur_needed_ind(3))),'< has an address=',address(cur_needed_ind(3))
        call die (l_here, 'i2 not associated')
     endif


     if (.not.associated (elt)) then   
        allocate(real_0_type_alias)
        real_0_type_alias%p =>  array_2(i1,i2)
        objects_addr (cur_create_ind (1)) = address (real_0_type_alias)
        objects_type (cur_create_ind (1)) = string_in_array_locate ('real_0',types_name)
      
     endif
  else if (dim_nb == 3) then
     call alias_byaddr_bld (elt, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (array_3, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (i1, objects_addr (cur_needed_ind (2)))
     call alias_byaddr_bld (i2, objects_addr (cur_needed_ind (3)))   
     call alias_byaddr_bld (i3, objects_addr (cur_needed_ind (4)))

     if (.not.associated (i1)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(2)=',cur_needed_ind(2)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
             objects_names(cur_needed_ind(2)),'< has an address=',address(cur_needed_ind(2))
        call die (l_here, 'i1 not associated')
     endif
                  
     if (.not.associated (i2)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(3)=',cur_needed_ind(3)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
            trim(objects_names(cur_needed_ind(3))),'< has an address=',address(cur_needed_ind(3))
        call die (l_here, 'i2 not associated')
     endif

     if (.not.associated (i3)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(4)=',cur_needed_ind(4)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
             trim(objects_names(cur_needed_ind(4))),'< has an address=',address(cur_needed_ind(3))
        call die (l_here, 'i3 not associated')
     endif

     if (.not.associated (elt)) then   
        allocate(real_0_type_alias)
        real_0_type_alias%p =>  array_3(i1,i2,i3)
 
        objects_addr (cur_create_ind (1)) = address (real_0_type_alias)
        objects_type (cur_create_ind (1)) = string_in_array_locate ('real_0',types_name)
      
     endif

  else if (dim_nb == 4) then
     call alias_byaddr_bld (elt, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (array_4, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (i1, objects_addr (cur_needed_ind (2)))
     call alias_byaddr_bld (i2, objects_addr (cur_needed_ind (3)))   
     call alias_byaddr_bld (i3, objects_addr (cur_needed_ind (4)))
     call alias_byaddr_bld (i4, objects_addr (cur_needed_ind (5)))

     if (.not.associated (i1)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(2)=',cur_needed_ind(2)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
             objects_names(cur_needed_ind(2)),'< has an address=',address(cur_needed_ind(2))
        call die (l_here, 'i1 not associated')
     endif
                  
     if (.not.associated (i2)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(3)=',cur_needed_ind(3)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
            trim(objects_names(cur_needed_ind(3))),'< has an address=',address(cur_needed_ind(3))
        call die (l_here, 'i2 not associated')
     endif

     if (.not.associated (i3)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(4)=',cur_needed_ind(4)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
             trim(objects_names(cur_needed_ind(4))),'< has an address=',address(cur_needed_ind(4))
        call die (l_here, 'i3 not associated')
     endif

     if (.not.associated (i4)) then
        write (logf,*) trim(l_here),'-e: cur_needed_ind(5)=',cur_needed_ind(5)
        write (logf,*) trim(l_here) ,'-e: needed object >',&
             trim(objects_names(cur_needed_ind(5))),'< has an address=',address(cur_needed_ind(5))
        call die (l_here, 'i4 not associated')
     endif


     if (.not.associated (elt)) then   
        allocate(real_0_type_alias)
        real_0_type_alias%p =>  array_4(i1,i2,i3,i4)
 
        objects_addr (cur_create_ind (1)) = address (real_0_type_alias)
        objects_type (cur_create_ind (1)) = string_in_array_locate ('real_0',types_name)
      
     endif
  else

     call die (l_here,'number dimensions not valid')
  endif

   call  exit (l_here)
                  
  end subroutine array_element_calc

! ========================================================================
  subroutine real_1_from_real_2_bld
! ------------------------------------------------------------------------
!
! Name           : 
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
    real(dp), pointer     :: result_bywlk (:)
    real(dp) , pointer    :: real_1_bywlk (:,:)
  integer(dpi), pointer :: i
      
! begin:
  if (header_exe) then
      return
  endif
   call alias_byaddr_bld (result_bywlk, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (real_1_bywlk, objects_addr (cur_needed_ind (1)))
   call alias_byaddr_bld (i, objects_addr (cur_needed_ind (2)))

   if (.not.associated (result_bywlk)) call alloc (objects_names (cur_create_ind (1)),result_bywlk,size (real_1_bywlk,dim = 2))
 
    result_bywlk (:) =  real_1_bywlk(i,:)

  end subroutine real_1_from_real_2_bld

! ========================================================================
  subroutine real_1_from_real_3_bld
! ------------------------------------------------------------------------
!
! Name           : 
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
    real(dp), pointer     :: result_bywlk (:)
    real(dp) , pointer    :: real_2_bywlk (:,:,:)

    integer(dpi) :: size_1, addr, addr2
      
! begin:
  if (header_exe) then
      return
  endif

   call alias_byaddr_bld (result_bywlk, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (real_2_bywlk, objects_addr (cur_needed_ind (1)))
 
    size_1 = size (real_2_bywlk)
    addr =  address (real_2_bywlk(1,1,1))
    addr2 = address (size_1)

     call exe_by_address_2 (address (real_1_alias_bld), addr, addr2)
     call real_1_catalog (objects_names (cur_create_ind (1)), real_1_alias)

  end subroutine real_1_from_real_3_bld

! ========================================================================
  subroutine real_6_from_real_6_bld
! ------------------------------------------------------------------------
!
! Name           : 
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
    real(dp), pointer     :: result_bywlk (:,:,:,:,:,:)
    real(dp) , pointer    :: real_6_bywlk (:,:,:,:,:,:)
 
      
! begin:
  if (header_exe) then
      return
  endif
   call alias_byaddr_bld (result_bywlk, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (real_6_bywlk, objects_addr (cur_needed_ind (1)))
 
    result_bywlk =>  real_6_bywlk
    call real_6_catalog (objects_names (cur_create_ind (1)),result_bywlk)
  
  
  end subroutine real_6_from_real_6_bld

! ========================================================================
  subroutine real_7_from_real_7_bld
! ------------------------------------------------------------------------
!
! Name           : 
!
! Description    :  
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
    real(dp), pointer     :: result_bywlk (:,:,:,:,:,:,:)
    real(dp) , pointer    :: real_7_bywlk (:,:,:,:,:,:,:)
 
      
! begin:
  if (header_exe) then
      return
  endif
   call alias_byaddr_bld (result_bywlk, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (real_7_bywlk, objects_addr (cur_needed_ind (1)))
 
    result_bywlk =>  real_7_bywlk
    call real_7_catalog (objects_names (cur_create_ind (1)),result_bywlk)
  
  
  end subroutine real_7_from_real_7_bld

! ========================================================================
  subroutine graph_calc
! ------------------------------------------------------------------------
!
! Name           : graph_calc
!
! Description    : 
!
! Authors: R. Assaraf
! Date  : 04 May 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: graph_values (:) => NULL ()
  integer(dpi) :: target_object
  integer(dpi) :: root_object
  real(dp), pointer :: upper_bound
  real(dp), pointer :: lower_bound
  real(dp), pointer :: step
      
! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: needs_nb
  integer(dpi) :: values_nb 
  integer(dpi), pointer :: path_bynod_loc (:) => NULL ()
  integer(dpi) :: val_i
  integer(dpi) :: nod_i
  real(dp), pointer :: variable, func_value
! begin:
  if (header_exe) then

     l_here = 'graph_calc'

     return
  endif
  call enter (l_here)
! number of needed objects at current node cur_nod
  needs_nb =  nodes_need%last(cur_nod)-nodes_need%first(cur_nod)+1
  
  if (needs_nb /= 5) then
     call die (l_here, 'needs_nb='+needs_nb+' /= 5')
  endif

! puts objects at right addresses 
 
  call alias_byaddr_bld (graph_values, objects_addr (cur_create_ind (1)))

  target_object = cur_needed_ind (1)
  root_object   = cur_needed_ind (2)

  call alias_byaddr_bld (lower_bound, objects_addr (cur_needed_ind (3)))
  call alias_byaddr_bld (upper_bound, objects_addr (cur_needed_ind (4)))
  call alias_byaddr_bld (step, objects_addr (cur_needed_ind (5)))
    
  if (upper_bound <= lower_bound) &
       call die (l_here, 'upper_bound='+upper_bound+'should be > lower_bound='+lower_bound)
  if (step <= 0.d0) call die (l_here, 'step ='+step+' should be > 0')

! size of objects_names
  values_nb = (upper_bound-lower_bound)/step

  if (values_nb  <= 1 ) then
     call die (l_here, 'values_nb='+values_nb+'should be > 0')
  endif

  if (.not.associated (graph_values)) then 
     call alloc (objects_names (cur_create_ind (1)),graph_values, values_nb)
  else 
     if (size(graph_values) /= values_nb) then 
        call die (l_here, 'size(graph_values,2)='+size(graph_values,dim=1)+'/='+values_nb)
     endif
  endif

! initialization of the value of the target
  call provide_byind (target_object)
 
! nodes from root_object to target_object  
  call path_bynod_bld  (path_bynod_loc, (/root_object/), (/target_object/))

  if (debug) then
     write (logf,*) trim(l_here), '-d: path_bynod_loc: ', path_bynod_loc
  endif

! computation of target_object for the different values of the root
  call alias_byaddr_bld (func_value, objects_addr (cur_needed_ind (1)))

  if (.not.associated (func_value)) &
    call die (l_here, 'func_value not associated')

  call alias_byaddr_bld (variable, objects_addr (cur_needed_ind (2)))
   
  if (.not.associated (variable)) &
   call die (l_here, 'variable not associated')

  do val_i = 1, values_nb
     variable = lower_bound + step*val_i
     call path_bynod_exe (path_bynod_loc)
     graph_values (val_i) = func_value
     do nod_i = 1, size (path_bynod_loc)
        is_valid_bynod (path_bynod_loc (nod_i)) = .false.
     enddo
  enddo ! do val_i
 
  call exit (l_here)
end subroutine graph_calc

! ========================================================================
  subroutine histo_calc
! ------------------------------------------------------------------------
!
! Name           : histo_calc
!
! Description    : 
!
! Authors: R. Assaraf
! Date  : 12 Jul 2005
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: histo  (:) => NULL ()
  real(dp), pointer :: sample (:) => NULL ()
  real(dp), pointer :: step       => NULL ()
  real(dp), pointer :: lowerb     => NULL ()
  real(dp), pointer :: upperb     => NULL ()
      

! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: values_nb, i, histo_i

 ! begin:
  if (header_exe) then

     l_here = 'histo_calc'

     return
  endif
  call enter (l_here)

! puts objects at right addresses 
 
  call alias_byaddr_bld (histo,  objects_addr (cur_create_ind (1)))

  call alias_byaddr_bld (sample, objects_addr (cur_needed_ind (1)))
  call alias_byaddr_bld (lowerb, objects_addr (cur_needed_ind (2)))
  call alias_byaddr_bld (upperb, objects_addr (cur_needed_ind (3)))
  call alias_byaddr_bld (step  , objects_addr (cur_needed_ind (4)))
  
! size of objects_names
  values_nb = (upperb-lowerb)/step

  call alloc (objects_names (cur_create_ind (1)),histo, values_nb)

  if (values_nb  <= 1 ) then
     call die (l_here, 'values_nb='+values_nb+'should be > 0')
  endif

  histo = 0.d0
  do i = 1, size (sample)
     histo_i = int ( (sample (i)-lowerb)/step)
     if (histo_i .gt. 0 .and. histo_i .lt. values_nb) &
          histo(histo_i) = histo (histo_i) + 1.d0
  enddo ! do val_i
 
  call exit (l_here)
end subroutine histo_calc

! ========================================================================
  subroutine sum_calc
! ------------------------------------------------------------------------
!
! Name           : sum_calc
!
! Description    : result=x+y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result => NULL ()
  real(dp), pointer :: result1(:) => NULL()
  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: y      => NULL ()
  real(dp), pointer :: x1 (:)      => NULL ()
  real(dp), pointer :: y1 (:)     => NULL ()
      
! local:
  integer(dpi) :: type_need1, type_need2
  character(len=max_string_len_routine_name), save :: l_here
! begin:
  if (header_exe) then
     l_here = 'sum_calc'
     return
  endif

! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))

     if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = x+y
     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(x1))
    
        result1 = x1+y1
    else if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(y1))

      result1 = x+y1

     else 
        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',trim(types_name(type_need1)),'< and >',trim(types_name(type_need2)),'<'
        
        call die ('sum_calc','invalid types')

     endif

  end subroutine sum_calc


  subroutine heaviside_calc
! ------------------------------------------------------------------------
!
! Name           : heaviside_calc
!
! Description    : result=heaviside(x)
!
! Authors: R. Assaraf
! Date  : 11 Apr 2005
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), pointer :: result => NULL ()
  real(dp), pointer :: result1(:) => NULL()
  real(dp), pointer :: result2 (:,:) => NULL ()
  real(dp), pointer :: result3 (:,:,:) => NULL()

  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: x1 (:)      => NULL ()
  real(dp), pointer :: x2 (:,:)     => NULL ()
  real(dp), pointer :: x3 (:,:,:)   => NULL ()
! local:
  integer(dpi) :: type_need1
  integer(dpi) :: i1,i2,i3
  character(len=max_string_len_routine_name), save :: l_here
! begin:

  if (header_exe) then
     l_here = 'heaviside_calc'
     return
  endif

! puts objects at right addresses

  type_need1 = objects_type (cur_needed_ind(1))
    if (types_name (type_need1)=='real_0') then

        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)

        if (x < 0.d0) then
           result=0.d0
        else
           result=1.d0
        endif

     else if (types_name (type_need1)=='real_1') then

        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))

        if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)),result2,size(x2,1),size(x2,2))

        do i1 = 1, size(x1,1)
           do i2 = 1, size(x2,2)
              if (x2(i1,i2) < 0.d0) then
                 result2(i1,i2)=0.d0
              else
                 result2(i1,i2)=1.d0
              endif
           enddo ! do i2
       enddo ! do i1

   else if (types_name (type_need1)=='real_3') then

       call alias_byaddr_bld (result3, objects_addr (cur_create_ind (1)))
       call alias_byaddr_bld (x3, objects_addr (cur_needed_ind (1)))


       if (.not.associated (result3)) call alloc (objects_names (cur_create_ind (1)),result3,size(x3,1),size(x3,2),size(x3,3))

       do i1 = 1, size(x1,1)
          do i2 = 1, size(x2,2)
             do i3= 1, size(x3,3)
                if (x3(i1,i2,i3) < 0.d0) then
                   result3(i1,i2,i3)=0.d0
                else
                   result3(i1,i2,i3)=1.d0
                endif
             enddo ! do i3
          enddo ! do i2
       enddo ! do i1

     else
        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'

        call die ('heaviside_calc','invalid types')

     endif

  end subroutine heaviside_calc


! ========================================================================
  subroutine equal_set
! ------------------------------------------------------------------------
!
! Name           : equal_set
!
! Description    :
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp), pointer :: result  => NULL ()
  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: y      => NULL ()
  real(dp), pointer :: x1 (:) => NULL ()
  real(dp), pointer :: y1 (:) => NULL ()
  real(dp), pointer :: x2(:,:) => NULL ()
  real(dp), pointer :: x3(:,:,:) => NULL ()
  integer(dpi), pointer :: n1 (:) => NULL ()
! local:
  integer(dpi) :: type_need1, type_need2
  character(len=max_string_len_routine_name), save :: l_here
! begin:
  if (header_exe) then
     l_here = 'equal_set'
     return
  endif

! puts objects at right addresses

  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))

     if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_0') then

        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)

        x = y
        result = x
     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_1') then

         x1 = y1
         result = x1(1)
    else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_0') then

        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)

         x1 = y
         result = x1 (1)

     else if (types_name (type_need1)=='real_2'.and.types_name (type_need2)=='real_0') then

        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x2, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)


        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)

         x2 = y
         result = x2 (1,1)
      else if (types_name (type_need1)=='real_3'.and.types_name (type_need2)=='real_0') then

        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x3, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) &
             call alloc (objects_names (cur_create_ind (1)),result)

        x3 = y
        result = x3 (1,1,1)

     else  if (types_name (type_need1)=='integer_1'.and.types_name (type_need2)=='real_1') then
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (n1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (2)))

        call touch_byind (n1)
     else
        write (logf,*) trim(l_here),'-e: objects >',&
             trim(objects_names(cur_needed_ind(1))),'<, >',&
             trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',&
             trim(types_name(type_need1)),'< and >',&
             trim(types_name(type_need2)),'<'

        call die ('equal_set','invalid types')

     endif

     call touch_byind ((/cur_needed_ind(1)/))

   end subroutine equal_set





! ========================================================================
  subroutine list_calc
! ------------------------------------------------------------------------
!
! Name         : list_calc
!
! Description  : 
!
! Authors: R. Assaraf
! Date   : 27 Oct 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  integer(dpi), pointer :: list (:) => NULL ()

! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: needs_nb
! begin:
  if (header_exe) then
   
     l_here = 'list_calc'
     if (create_index /= NOT_INIT_I) then
        needs_nb=integer_in_asso_array_locate(NOT_INIT_I, needs)-1
  
        allocate(list(needs_nb))
        list = needs(1:needs_nb)

        call node_create ('@',(/'temporary_name'/) ,objects_names(list))
   
        call catalog('temporary_name', list)
 
        create_index=object_index('temporary_name')
        objects_names(create_index)=automatic_name(create_index, '@',&
        needs(1:needs_nb))
     endif
     
     return
  endif

!  call die (l_here, 'should not be called')    

end subroutine list_calc


! ========================================================================
  subroutine gather_calc
! ------------------------------------------------------------------------
!
! Name         : gather_calc
!
! Description  : gathers a list of arrays of indexes list_arrays_ind
! Description  : in a one dimensional array
!
! Authors: R. Assaraf
! Date   : 27 Oct 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer     :: result (:) => NULL ()
  integer(dpi), pointer :: list_arrays_ind (:) => NULL ()

! local:
  character(len=max_string_len_routine_name), save :: l_here

! begin:
 
  if (header_exe) then
      l_here = 'gather_calc'

     return
  endif
  
  call enter (l_here)

  call alias_byaddr_bld (list_arrays_ind, objects_addr (cur_needed_ind (1)))

  call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

  if (.not.associated (result)) then  
     call real_objects_gather_bld (result, list_arrays_ind)
    
     call catalog (objects_names (cur_create_ind (1)), result)
  else
   
     call real_objects_gather_bld (result, list_arrays_ind)
  endif

  call exit (l_here)

end subroutine gather_calc


! ========================================================================
  subroutine clone_calc
! ------------------------------------------------------------------------
!
! Name         : clone_calc
!
! Description  : 
!
! Authors: R. Assaraf
! Date   : 27 Oct 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 

! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi), pointer :: obj_list (:) => NULL()
! begin:
  
  if (header_exe) then
     if (create_index /= NOT_INIT_I) then
        
        l_here = 'clone_calc'

        call alias_byaddr_bld(obj_list, objects_addr(needs(2)))
        if (.not.associated(obj_list)) call die (l_here, 'obj_list not associated')
        create_index = clone_get (needs(1),&
             obj_list(1:size(obj_list)/2), &
             obj_list(size(obj_list)/2+1:size(obj_list)),&
             objects_names(needs(2))(1:3))
        
     endif
     return  
  endif

 call die (l_here, 'should not be called')
  
end subroutine clone_calc

! ========================================================================
  subroutine minus_calc
! ------------------------------------------------------------------------
!
! Name           : minus_calc
!
! Description    : result=-x
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result => NULL ()
  real(dp), pointer :: result1(:) => NULL()
  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: x1 (:)      => NULL ()
      
! local:
  integer(dpi) :: type_need1
! begin:
  if (header_exe) then

     return
  endif

! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))

     if (types_name (type_need1)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
  
        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = -x

     else if (types_name (type_need1)=='real_1') then 
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
  
        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(x1))
    
        result1 = -x1

     else

        call die ('minus_calc','invalid types')

     endif

  end subroutine minus_calc

! ========================================================================
  subroutine prod_calc
! ------------------------------------------------------------------------
!
! Name           : prod_calc
!
! Description    : result=x+y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result => NULL ()
  real(dp), pointer :: result1(:) => NULL()
  real(dp), pointer :: result2(:,:) => NULL()
  real(dp), pointer :: result3(:,:,:)
  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: y      => NULL ()
  real(dp), pointer :: x1 (:) => NULL ()
  real(dp), pointer :: x2 (:,:) => NULL ()
  real(dp), pointer :: y1 (:) => NULL ()
  real(dp), pointer :: y2 (:,:) => NULL ()
  real(dp), pointer :: y3 (:,:,:) => NULL ()
  integer(dpi), pointer :: x_int => NULL()
  integer(dpi), pointer :: x_int1 (:) => NULL()
  integer(dpi)  :: i,j
! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: type_need1, type_need2
  integer(dpi) :: reduced_size
! begin:  
if (header_exe) then
     l_here = 'prod_calc'
     return
  endif
 call enter (l_here)
! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))

     if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = x*y

     else  if (types_name (type_need1)=='integer_0'.and.types_name (type_need2)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x_int, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = x_int*y


     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(x1))
        result1= NOT_INIT_DP
        reduced_size = real_array_reduced_size (x1)
        result1(1:reduced_size) = x1(1:reduced_size)*y1(1:reduced_size)

     else if (types_name (type_need1)=='integer_1'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x_int1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(x1))
        result1(1:reduced_size) = x_int1*y1

     else if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(y1))
 
        result1 = x*y1

     else if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_2') then
        
        call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y2, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)),result2,size(y2,dim=1),size(y2,dim=2))
 
        result2 = x*y2
     else if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_3') then
        
        call alias_byaddr_bld (result3, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y3, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result3)) call alloc (objects_names (cur_create_ind (1)),result3,&
             size(y3,dim=1),size(y3,dim=2),size(y3,dim=3))
        result3 = x*y3
     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_2') then
      
        call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y2, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)),result2,&
             size(y2,dim=1),size(y2,dim=2))

        if (size (x1) /= size (y2, dim=2)) then 

           message = 'object >'+objects_names(cur_needed_ind(1))+&
                '< has dimension '*size (x1)
           message = message*' whereas object  >'+objects_names(cur_needed_ind(2))+&
                '< has first dimension '*size (y2, dim=1)
           message = message*' dimensions should be equal' 

           call die (l_here, message)   

        endif
        
        do i =1, size (result2, dim =2)

           do j =1, size (result2, dim=1)

              result2 (j,i) = x1 (i)  * y2 (j,i)

           enddo ! do j

        enddo ! do i 

     else if (types_name (type_need1)=='real_2'.and.types_name (type_need2)=='real_2') then
      
        call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x2, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y2, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)),result2,&
             size(y2,dim=1),size(y2,dim=2))
        
        if (size (x2,dim=1) /= size (y2, dim=1)) then 
           
           message = 'object >'+objects_names(cur_needed_ind(1))+&
                '< has 1st dimension '*size (x2, dim=1)
           message = message*' whereas object  >'+objects_names(cur_needed_ind(2))+&
                '< has 1st dimension '*size (y2, dim=1)
           message = message*' dimensions should be equal' 
           
           call die (l_here, message)   

        endif

        if (size (x2,dim=2) /= size (y2, dim=2)) then 
           
           message = 'object >'+objects_names(cur_needed_ind(1))+&
                '< has 2nd dimension '*size (x2, dim=2)
           message = message*' whereas object  >'+objects_names(cur_needed_ind(2))+&
                '< has 2nd dimension '*size (y2, dim=2)
           message = message*' dimensions should be equal' 
           
           call die (l_here, message)   

        endif

        

        do i =1, size (result2, dim =2)

           do j =1, size (result2, dim=1)

              result2 (j,i) = x2 (j,i)  * y2 (j,i)

           enddo ! do j

        enddo ! do i 

     else

        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',trim(types_name(type_need1)),'< and >',trim(types_name(type_need2)),'<'
        call die (l_here,'invalid types')

     endif
   call exit (l_here)
   end subroutine prod_calc


! ========================================================================
  subroutine matmult_calc
! ------------------------------------------------------------------------
!
! Name           : matmult_calc
!
! Description    : result= A * B
!
! Authors: R. Assaraf
! Date  : 16 Nov 2005
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result1(:)   => NULL ()
  real(dp), pointer :: result2(:,:) => NULL ()
  real(dp), pointer :: result       => NULL ()
  real(dp), pointer :: a (:,:) => NULL ()
  real(dp), pointer :: b (:,:) => NULL ()
  real(dp), pointer :: v (:)   => NULL ()
  real(dp), pointer :: w (:)   => NULL () 
  integer(dpi)      :: i1, i2, j
! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: type_need1, type_need2

! begin:  
  if (header_exe) then
     l_here = 'matmult_calc'
     return
  endif
  call enter (l_here)
! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))
  
  if (types_name (type_need1)=='real_2'.and.types_name (type_need2)=='real_2') then
 
     call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (a, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (b, objects_addr (cur_needed_ind (2)))

! check sizes
     if (size (a, dim =2) /= size (b, dim=1)) then
        message = "sizes of >"+objects_names(cur_needed_ind(1))+'< and >'  &
        +objects_names(cur_needed_ind(2))+"< not compatible"
        call die (l_here, message)
     endif

     if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)),result2, size(a,dim=1), size(b,dim=2))
        
     do i1 = 1, size (a, dim = 1)
        do i2= 1, size (b, dim =2)
           result2 (i1,i2) = 0.d0
           do j = 1, size (b, dim =1)
              result2 (i1, i2) = result2 (i1,i2)+a (i1,j) *b (j,i2)
           enddo ! do ji   
        enddo ! do i1
     enddo ! do i2
     
  else if (types_name (type_need1)=='real_2'.and.types_name (type_need2)=='real_1') then

     call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (a, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (v, objects_addr (cur_needed_ind (2)))

     if (size (a, dim =2) /= size (v, dim=1)) then
        message = "sizes of >"+objects_names(cur_needed_ind(1))+'< and  >'&
             +objects_names(cur_needed_ind(2))+"< not compatible"
        call die (l_here, message)
     endif

     if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1, size(a,dim=1))
     
     do i1 = 1, size (a, dim = 1)
        result1 (i1) = 0.d01
        do j = 1, size (v, dim =1)
           result1 (i1) = result1 (i1)+a (i1,j) *v (j)
        enddo ! do ji   
     enddo ! do i1
 

  else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_1') then
     call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (w, objects_addr (cur_needed_ind (1)))
     call alias_byaddr_bld (v, objects_addr (cur_needed_ind (2)))
     if (size (w, dim =1) /= size (v, dim=1)) then
        message = "sizes of >"+objects_names(cur_needed_ind(1))+'< and  >'&
             +objects_names(cur_needed_ind(2))+"< not compatible"
        call die (l_here, message)
     endif

     if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
     result = 0.d0
     do j =1, size (v)
        result = result + v(j)*w(j)
     enddo ! do j
     
  else

        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',trim(types_name(type_need1)),'< and >',trim(types_name(type_need2)),'<'
        call die (l_here,'invalid types')
        
     endif
     call exit (l_here)
   end subroutine matmult_calc

! ========================================================================
  subroutine div_calc
! ------------------------------------------------------------------------
!
! Name           : div_calc
!
! Description    : result=x+y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result => NULL ()
  real(dp), pointer :: result1(:) => NULL()
  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: y      => NULL ()
  real(dp), pointer :: x1 (:)      => NULL ()
  real(dp), pointer :: y1 (:)     => NULL ()
  integer(dpi)      :: i

! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi) :: type_need1, type_need2
! begin:
  if (header_exe) then
    l_here = 'div_calc'

     return
  endif

! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))

     if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = x/y
     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(x1))
    
        if (size (x1) /= size (y1)) call die (l_here, 'sizes not compatible')
        do i = 1, size (y1)
           result1 (i) = x1(i) / y1 (i)
        enddo
    else if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(y1))

      result1 = x/y1

     else
        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',trim(types_name(type_need1)),'< and >',trim(types_name(type_need2)),'<'

       call die ('div_calc','invalid types')

     endif

  end subroutine div_calc

! ========================================================================
  subroutine dif_calc
! ------------------------------------------------------------------------
!
! Name           : dif_calc
!
! Description    : result=x+y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result => NULL ()
  real(dp), pointer :: result1(:) => NULL()
  real(dp), pointer :: x      => NULL ()
  real(dp), pointer :: y      => NULL ()
  real(dp), pointer :: x1 (:)      => NULL ()
  real(dp), pointer :: y1 (:)     => NULL ()
  character(len=max_string_len_routine_name), save :: l_here            
! local:
  integer(dpi) :: type_need1, type_need2
! begin:
  if (header_exe) then
     l_here = 'dif_calc'
     return
  endif

! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))

     if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = x-y
     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(x1))
    
        result1 = x1-y1
    else if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_1') then
        
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y1, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1,size(y1))

      result1 = x-y1

     else
        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',trim(types_name(type_need1)),'< and >',trim(types_name(type_need2)),'<'
        

       call die ('dif_calc','invalid types')

     endif

  end subroutine dif_calc

! ========================================================================
  subroutine expo_calc 
! ------------------------------------------------------------------------
!
! Name           : expo_calc
!
! Description    : result=x**y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  real(dp), pointer :: result   => NULL ()
  real(dp), pointer :: result_1 (:) => NULL ()
  real(dp), pointer :: x        => NULL ()
  real(dp), pointer :: x_1 (:)  => NULL ()
  real(dp), pointer :: y        => NULL ()
  character(len=max_string_len_routine_name), save :: l_here           
  
! local:
  integer(dpi) :: type_need1, type_need2, i
! begin:
  if (header_exe) then
     l_here ='expo_calc'
     return
  endif

! puts objects at right addresses 
 
  type_need1 = objects_type (cur_needed_ind(1))
  type_need2 = objects_type (cur_needed_ind(2))

     if (types_name (type_need1)=='real_0'.and.types_name (type_need2)=='real_0') then
 
        call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

        if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
    
        result = x**y

     else if (types_name (type_need1)=='real_1'.and.types_name (type_need2)=='real_0') then
        call alias_byaddr_bld (result_1, objects_addr (cur_create_ind (1)))      
        call alias_byaddr_bld (x_1, objects_addr (cur_needed_ind (1)))
        call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))
        
        call alloc (objects_names (cur_create_ind (1)),result_1, size (x_1))
        
        do i = 1, size (x_1)
           result_1 (i) = x_1(i)**y
        enddo ! do i 

     else
        write (logf,*) trim(l_here),'-e: objects >',trim(objects_names(cur_needed_ind(1))),'<, >',trim(objects_names(cur_needed_ind(2))),'<'
        write(logf,*) trim(l_here),'-e: of types >',trim(types_name(type_need1)),'< and >',trim(types_name(type_need2)),'<'
        
       call die ('expo_calc','invalid types')
       
     endif

  end subroutine expo_calc

! ========================================================================
  subroutine alias_calc
! ------------------------------------------------------------------------
!
! Name           : alias_calc
!
! Description    :  builds alias of an object
!
! Authors: R. Assaraf
! Date  : 21 Dec 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 
 
! local:
   
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 

   objects_addr (cur_create_ind (1)) = objects_addr (cur_needed_ind (1))
   objects_type (cur_create_ind (1)) = objects_type (cur_needed_ind (1))
 
  end subroutine alias_calc

! ========================================================================
  subroutine slice_calc
! ------------------------------------------------------------------------
!
! Name           : slice_calc
!
! Description    :  slice of a 2-dimensional array
!
! Authors: R. Assaraf
! Date  : 21 Dec 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 
 
! local:
     real(dp), pointer :: slice (:)
     real(dp), pointer :: array (:,:)
     integer(dpi), pointer :: i
     integer(dpi) ::  dim1, dim2
! begin:
   if (header_exe) then

   return
   endif
!  l_here = 'slice_calc'
! puts objects at right addresses 
   call alias_byaddr_bld (slice, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (array, objects_addr (cur_needed_ind (1)))
   call alias_byaddr_bld (i, objects_addr (cur_needed_ind (2)))

   dim2 = size (array,2)
!   if (dim2 == 0) call die (l_here
   if (.not.associated (slice)) then 
      call alloc (objects_names (cur_create_ind(1)), slice, dim2) 
   endif

   slice = array (i,:)
 
  end subroutine slice_calc

! ========================================================================
  subroutine alias_obj_calc
! ------------------------------------------------------------------------
!
! Name           : alias_calc
!
! Description    :  builds alias of an object
!
! Authors: R. Assaraf
! Date  : 21 Dec 2002
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  integer(dpi), pointer :: obj_i
! local:
   
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 
   call alias_byaddr_bld (obj_i, objects_addr (cur_needed_ind (1)))
   objects_addr (cur_create_ind (1)) = objects_addr (obj_i)
   objects_type (cur_create_ind (1)) = objects_type (obj_i)
 
  end subroutine alias_obj_calc

! ========================================================================
  subroutine sin_calc
! ------------------------------------------------------------------------
!
! Name           : sin_calc
!
! Description    :  result= sin(x)
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result => NULL ()
    real(dp), pointer :: x      => NULL ()

! local:
  
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 

   call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

   call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
  
   if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
 
   result = sin(x)

  end subroutine sin_calc

! ========================================================================
  subroutine lower_calc
! ------------------------------------------------------------------------
!
! Name           : lower_calc
!
! Description    : lower =  x < y
!
! Authors: R. Assaraf
! Date  : 19 Jul 2005
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    integer(dpi), pointer :: result => NULL ()
    integer(dpi), pointer :: x      => NULL ()
    integer(dpi), pointer :: y      => NULL ()
! local:
  
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 

   call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

   call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
  
   call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))
  
   if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
 
   if (x < y) then 
       result = 1
   else
       result = 0
   endif

  end subroutine lower_calc

! ========================================================================
  subroutine abs_calc
! ------------------------------------------------------------------------
!
! Name           : abs_calc
!
! Description    :  result=abs(x)
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result     => NULL ()
    real(dp), pointer :: result1(:) => NULL ()
    real(dp), pointer :: x      => NULL ()
    real(dp), pointer :: x1(:)  => NULL ()
    real(dp), pointer :: x2 (:,:) => NULL ()
    real(dp), pointer :: result2 (:,:) => NULL()
! local:
    integer(dpi) :: type_need1
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
    integer(dpi) :: i 
! begin:
   if (header_exe) then
       l_here = 'float_calc'
       l_here_index = routine_cur_index ()
       
       
   return
   endif

   type_need1 = objects_type (cur_needed_ind(1))
     
      if (types_name(type_need1) == 'real_0') then 
         call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

         call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
  
         if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
 
         result = abs(x)
     else if (types_name(type_need1) == 'real_1') then
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1, size(x1))
    
        do i = 1, size(x1)        
           result1 (i)= abs (x1(i))
        enddo ! do i 
    
     else if (types_name(type_need1) == 'real_2') then
        call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x2, objects_addr (cur_needed_ind (1)))
        if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)), result2,&
             size(x2, dim=1), size(x2, dim=2))
         
        result2 = abs (x2)
 
     else
        call die (l_here, 'type >'+types_name(type_need1)+'< not implemented')
     endif
    
   end subroutine abs_calc


! ========================================================================
  subroutine float_calc
! ------------------------------------------------------------------------
!
! Name           : float_calc
!
! Description    :  
! Description    :  
!
! Authors: R. Assaraf
! Date  :24 Nov 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer  :: result => NULL()
    real(dp), pointer  :: result1 (:) => NULL ()
    integer(dpi), pointer :: arg => NULL()
    integer(dpi), pointer :: arg1 (:)  => NULL()
 
! local:
    integer(dpi) :: type_need1
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
! begin:
    if (header_exe) then
       l_here = 'float_calc'
       l_here_index = routine_cur_index ()

       return
    endif
    type_need1 = objects_type (cur_needed_ind(1))
    if (types_name(type_need1) == 'integer_0') then 

       call alias_byaddr_bld (arg, objects_addr(cur_needed_ind(1))) 
       call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
       if (.not.associated(result)) then
          call alloc(objects_names(cur_create_ind(1)), result)
       endif
       result = arg
        
    else if (types_name(type_need1) == 'integer_1') then 
       call alias_byaddr_bld (arg1, objects_addr(cur_needed_ind(1))) 
       call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
       
       if (.not.associated(result1)) then
          call alloc(objects_names(cur_create_ind(1)), result1,size(arg1))
       endif
       result1 = arg1
        
    else 
       write (logf,*) trim(l_here), '-e: cur_needed_ind(1)=',cur_needed_ind(1)
       write (logf,*) trim(l_here), '-e: name >'+objects_names(cur_needed_ind(1))+'<'
       write (logf,*) trim(l_here), '-e: size(objects_type)=',size(objects_type) 
       write (logf,*) trim(l_here), '-e: type >'+types_name(type_need1)+'<'
    
       call die (l_here,'invalid types')
    endif

  end subroutine float_calc

! ========================================================================
  subroutine exp_calc
! ------------------------------------------------------------------------
!
! Name           : exp_calc
!
! Description    :  result=exp(x)
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result => NULL ()
    real(dp), pointer :: x      => NULL ()
    real(dp), pointer :: result1 (:) => NULL ()
    real(dp), pointer :: x1 (:) => NULL ()
    real(dp), pointer :: result2 (:,:) => NULL ()
    real(dp), pointer :: x2 (:,:) => NULL ()
    integer(dpi)      :: type_need1
    integer(dpi)      :: i, j
    character(len=max_string_len_routine_name), save :: l_here
! local:
  
! begin:
   l_here = "exp_calc"
   if (header_exe) then
 
   return
   endif
   call enter(l_here)
   type_need1 = objects_type (cur_needed_ind(1))

   if (types_name (type_need1) == "real_0") then
      
      call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

      call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))

      if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
 
      result = exp(x)

   else if (types_name (type_need1) == "real_1") then

      call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))

      call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))

      if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result1, size(x1))

      do i =1, size (x1)
         
         result1(i) = exp(x1(i))
         
     enddo ! do i
 
  else if (types_name (type_need1) == "real_2") then
     
     call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))

     call alias_byaddr_bld (x2, objects_addr (cur_needed_ind (1)))

     if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)),result2, size (x2, dim = 1 ), size (x2, dim = 2 ))
     
     do i =1, size (x2, dim = 1 )
        do j =1, size (x2, dim = 2 )

           result2 (i,j) = exp(x2(i,j))
           
        enddo ! do j
     enddo ! do i

  else

     call die (l_here, 'invalid types')
    
  endif
  call exit (l_here)
end subroutine exp_calc

! ========================================================================
  subroutine sqrt_calc
! ------------------------------------------------------------------------
!
! Name           : sqrt_calc
!
! Description    :  result=sqrt(x)
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result => NULL ()
    real(dp), pointer :: x      => NULL ()

! local:
  
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 

   call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

   call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))

   if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
 
   result = sqrt(x)

  end subroutine sqrt_calc

! ========================================================================
  subroutine sum_calc_real_1
! ------------------------------------------------------------------------
!
! Name           : sum_calc_2
!
! Description    :  result=x+y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result (:) => NULL ()
    real(dp), pointer :: x  (:)    => NULL ()
    real(dp), pointer :: y  (:)    => NULL ()

! local:
  
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 

   call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))
   call alias_byaddr_bld (y, objects_addr (cur_needed_ind (2)))

   if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result, size (x))
 
   result = x+y

  end subroutine sum_calc_real_1

! ========================================================================
  subroutine scalar_times_array_bld
! ------------------------------------------------------------------------
!
! Name           : scalar_times_array_bld
!
! Description    :  result= scalar*array
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result(:) => NULL ()
    real(dp), pointer :: scalar    => NULL ()
    real(dp), pointer :: array (:)  => NULL ()
    character(len=max_string_len_routine_name), save :: l_here
! local:
  
! begin:
   if (header_exe) then

   return
   endif

! puts objects at right addresses 

   call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

   call alias_byaddr_bld (scalar, objects_addr (cur_needed_ind (1)))
   call alias_byaddr_bld (array, objects_addr (cur_needed_ind (2)))

   if (.not.associated (array)) call die (l_here, "array not associated")
   if (.not.associated(scalar)) call die (l_here, "scalar not assocaited")
   
   if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result, size(array))
 
   result = scalar*array

  end subroutine scalar_times_array_bld

! ========================================================================
  subroutine sum_calc_2 ( result , x , y )  
! ------------------------------------------------------------------------
!
! Name           : sum_calc_2
!
! Description    :  result=x+y, version equivalent to sum_calc_2 
! Description    :  but with arguments
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp)  :: result
    real(dp)  :: x
    real(dp)  :: y
 
! local:
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
! begin:
    if (header_exe) then
    l_here = "sum_calc_2"
       l_here_index = routine_cur_index ()
!       call catalog (l_here, l_here_index)
       return
    endif
    
    result = x+y

  end subroutine sum_calc_2

! ========================================================================
  subroutine prod_calc_2 ( result , x , y )  
! ------------------------------------------------------------------------
!
! Name           : prod_calc_2
!
! Description    :  result=x*y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
   real(dp) :: result
   real(dp)  :: x
    real(dp)  :: y
 
! local:
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
! begin:
    if (header_exe) then
    l_here = "prod_calc_2"
       l_here_index = routine_cur_index ()
!       call catalog (l_here, l_here_index)
       return
    endif
    
    result = x * y

  end subroutine prod_calc_2

! ========================================================================
  subroutine dif_calc_2 ( result , x , y )  
! ------------------------------------------------------------------------
!
! Name           : dif_calc_2
!
! Description    :  result=x*y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
   real(dp) :: result
   real(dp)  :: x
    real(dp)  :: y
 
! local:
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
! begin:
    if (header_exe) then
    l_here = "dif_calc_2"
       l_here_index = routine_cur_index ()
!       call catalog (l_here, l_here_index)
       return
    endif
    
    result = x - y

  end subroutine dif_calc_2

! ========================================================================
  subroutine div_calc_2 ( result , x , y )  
! ------------------------------------------------------------------------
!
! Name           : div_calc_2
!
! Description    :  result=x+y
!
! Authors: R. Assaraf
! Date  :26 Dec 2001
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp)  :: result
    real(dp)  :: x
    real(dp)  :: y
 
! local:
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
! begin:
    if (header_exe) then
    l_here = "div_calc_2"
       l_here_index = routine_cur_index ()
!       call catalog (l_here, l_here_index)
       return
    endif
    
    result = x/y

  end subroutine div_calc_2

! ========================================================================
  subroutine exesys_calc ( )
! ------------------------------------------------------------------------
!
! Name           : exesys_calc
!
! Description    : system command executes
!
! Authors: R. Assaraf
! Date  :25 Dec 2003
! ------------------------------------------------------------------------
  implicit none

! i/o: 
  integer(dpi), pointer :: result => NULL ()
  character(len=1), pointer :: system_command (:) => NULL()
! local:
  character(len=max_string_len_routine_name), save :: l_here
  integer(dpi), save :: l_here_index
  integer(dpi) :: type_need1, i

! header:
  if (header_exe) then
     l_here = "exesys_calc"
     l_here_index = routine_cur_index ()
    
     return
  endif

! begin:    

  type_need1 = objects_type (cur_needed_ind(1))
  
     if (types_name(type_need1) /= 'character_1') then
        call die (l_here, 'invalid types')
     endif

 
     call alias_byaddr_bld  (result, objects_addr (cur_create_ind (1)))
     call alias_byaddr_bld (system_command, objects_addr (cur_needed_ind (1)))

     if (.not.associated (system_command)) then
        call die (l_here,'system_command not associated')
     endif

     if (.not.associated (result)) then
        call alloc (objects_names (cur_create_ind (1)), result)
     end if
    
     message=blanks  
     do i=1, size(system_command)
        message=message(1:i)//system_command(i)
     enddo
     
     result = exesys(trim(message))
  
  end subroutine exesys_calc

! ========================================================================
  function real_array_reduced_size ( array ) result(result)  
! ------------------------------------------------------------------------
!
! Name           : real_1_reduced_size 
!
! Argument       : array
!
! Description    : size of array without final NOT_INIT_DP 
!
! Authors: R. Assaraf
! Date  :23 Fev 2001
! ------------------------------------------------------------------------
  implicit none

! result:
    real(dp)  :: result
 
! i/o
  real(dp) :: array (:)

! local:
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi) :: i
! begin:
    if (header_exe) then
      call header_die
    endif
    l_here = "real_array_reduced_size"

    result = 0_dp

    do i = 1,size (array)

       if  ( abs (array (i)-NOT_INIT_DP) < abs(spacing(NOT_INIT_DP))) then 
          result = i-1
          exit
       else if (i== size (array)) then
          result = i
       endif

    enddo ! do i

  end function real_array_reduced_size

! ========================================================================
  subroutine size_calc
! ------------------------------------------------------------------------
!
! Name           : size_calc
!
! Description    :  result= size(array_1) : give the size of a realArray of dimension one
!
! Authors: R. Assaraf and B. Braida
! Date  : 01 Aug 2005
! ------------------------------------------------------------------------
  implicit none
  
! i/o: 
    integer(dpi), pointer :: result      => NULL ()
    real(dp), pointer     :: array_1 (:)  => NULL () 

! local:
    integer(dpi)          :: type_need1 
    character(len=max_string_len_routine_name), save :: l_here

! begin:

   l_here = "size_calc"

   if (header_exe) then

   return
   endif

!Test on the dimension of array_1

  type_need1 = objects_type (cur_needed_ind(1))

     if (types_name (type_need1)/='real_1') then
        call die(l_here,"Type  >"+types_name (type_need1)+"< should be real_1")
     endif

! puts objects at right addresses 

   call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

   call alias_byaddr_bld (array_1, objects_addr (cur_needed_ind (1)))
  
   if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)
 
   result = size (array_1)

  end subroutine size_calc
    

! ========================================================================
  subroutine real_array_init
! ------------------------------------------------------------------------
! 
! Name           : real_array_init
!
! Description    :   Create a real Array of dimension one for the moment
!
! Authors: R. Assaraf and B. Braida
! Date  : 01 Aug 2005
! ------------------------------------------------------------------------
  implicit none
  
! i/o: 
    real(dp), pointer     :: real_array_1 (:)  => NULL ()
    integer(dpi), pointer :: size_array        => NULL ()

! local:
    integer(dpi)          :: type_need1 
    character(len=max_string_len_routine_name), save :: l_here

! begin:

   l_here = "real_array_init"

   if (header_exe) then

   return
   endif

!Test on the dimension of array_1

  type_need1 = objects_type (cur_needed_ind(1))

     if (types_name (type_need1)/='integer_0') then
        call die(l_here,"Type  >"+types_name (type_need1)+"< should be integer_0")
     endif

! puts objects at right addresses 

   call alias_byaddr_bld (real_array_1, objects_addr (cur_create_ind (1)))

   call alias_byaddr_bld (size_array, objects_addr (cur_needed_ind (1)))

   if (.not.associated (real_array_1)) call alloc (objects_names (cur_create_ind (1)),real_array_1,size_array)

! Initialize array to 0
   real_array_1 = 0.D0

  end subroutine real_array_init

!------------------------------------------------------------------
  subroutine log_calc
! -----------------------------------------------------------------
!
! Name           : log_calc
!
! Description    :  result=log(x)
!
! Authors: B. Braida 
! Date  : 04 Aug. 2005
! -----------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: result     => NULL ()
    real(dp), pointer :: result1(:) => NULL ()
    real(dp), pointer :: x      => NULL ()
    real(dp), pointer :: x1(:)  => NULL ()
    real(dp), pointer :: x2 (:,:) => NULL ()
    real(dp), pointer :: result2 (:,:) => NULL()
! local:
    integer(dpi) :: type_need1
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index
    integer(dpi) :: i    
! begin:
   if (header_exe) then
       l_here = 'float_calc'
       l_here_index = routine_cur_index ()


   return
   endif

   type_need1 = objects_type (cur_needed_ind(1))

      if (types_name(type_need1) == 'real_0') then
         call alias_byaddr_bld (result, objects_addr (cur_create_ind (1)))

         call alias_byaddr_bld (x, objects_addr (cur_needed_ind (1)))

         if (.not.associated (result)) call alloc (objects_names (cur_create_ind (1)),result)

         result = log(x)
     else if (types_name(type_need1) == 'real_1') then
        call alias_byaddr_bld (result1, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x1, objects_addr (cur_needed_ind (1)))

        if (.not.associated (result1)) call alloc (objects_names (cur_create_ind (1)),result1, size(x1))

        do i = 1, size(x1)
           result1 (i)= log (x1(i))
        enddo ! do i 

     else if (types_name(type_need1) == 'real_2') then
        call alias_byaddr_bld (result2, objects_addr (cur_create_ind (1)))
        call alias_byaddr_bld (x2, objects_addr (cur_needed_ind (1)))
        if (.not.associated (result2)) call alloc (objects_names (cur_create_ind (1)), result2,&
             size(x2, dim=1), size(x2, dim=2))

        result2 = log (x2)

     else
        call die (l_here, 'type >'+types_name(type_need1)+'< not implemented')
     endif

   end subroutine log_calc

!------------------------------------------------------------------------
  subroutine gtos_overlap_calc
! ------------------------------------------------------------------------
!
! Name           : gtos_overlap_calc
!
! Description    :  overlap 
!                :  computing the overlap matrices of gaussians with exponents 
!                :  gto_exp(1), gto_exp (2)
!                :  with the measure r^r_power
! Authors: S. Kollias 
! Date  : 14th Nov 2005
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: gto_exp (:)   => NULL ()
    real(dp), pointer :: overlap (:,:) => NULL ()
    integer(dpi), pointer :: r_power   => NULL ()
! local:
    integer(dpi) :: type_need1, type_need2
    integer(dpi) :: gto_exp_nb, gto_i, gto_j
    real(dp)     :: product_exp
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index

! begin:
   if (header_exe) then
       l_here = 'gtos_overlap_calc'
       l_here_index = routine_cur_index ()


   return
   endif

   call enter (l_here)
! the array of gto exponents  is needed

   type_need1 = objects_type (cur_needed_ind(1))
   type_need2 = objects_type (cur_needed_ind(2))

! checking types of the needs
   if (types_name (type_need1) /= "real_1")&
        call die (l_here, "object "+objects_names (cur_needed_ind (1))+" should be a one dimension array")

   if (types_name(type_need2) /= 'integer_0') &
        call die (l_here, "object "+objects_names (cur_needed_ind (2))+" should be an integer")

! referencing the needed and created objects   
   call alias_byaddr_bld (gto_exp, objects_addr (cur_needed_ind (1)))
   call alias_byaddr_bld (r_power, objects_addr (cur_needed_ind (2)))
   call alias_byaddr_bld (overlap, objects_addr (cur_create_ind (1)))

   gto_exp_nb = size (gto_exp)

   if (.not.associated (overlap)) call alloc (objects_names (cur_create_ind (1)),overlap, gto_exp_nb, gto_exp_nb)

   do gto_i = 1, gto_exp_nb
      do gto_j =1, gto_exp_nb
         
         product_exp = gto_exp(gto_i) + gto_exp (gto_j)
            
         overlap (gto_i, gto_j) = gauss_xyzr_int (0, 0, 0, r_power,  product_exp)
         
      enddo ! do gto_j
   enddo ! do gto_i
   
   call exit (l_here)
      
 end subroutine gtos_overlap_calc
  

!------------------------------------------------------------------------
  subroutine gto_sto_overlap_calc
! ------------------------------------------------------------------------
!
! Name           : gto_sto_overlap_calc
!
! Description    :  overlap 
!                :  computing the overlap matrices between gaussians 
!                :  with exponents 
!                :  gto_exp(1), gto_exp (2), with a slater r^sto_r_power exp (-r)
!                :  with the measure r^r_power
! Authors: S. Kollias 
! Date  : 14th Nov 2005
! ------------------------------------------------------------------------
  implicit none

! i/o: 
    real(dp), pointer :: gto_exp (:)   => NULL ()
    real(dp), pointer :: overlap (:)   => NULL ()
    integer(dpi), pointer :: sto_r_power => NULL ()
    integer(dpi), pointer :: r_power   => NULL ()
! local:
    integer(dpi) :: type_need1, type_need2, type_need3
    integer(dpi) :: gto_i,  gto_exp_nb
    real(dp)     :: product_exp, exp_sto
    character(len=max_string_len_routine_name), save :: l_here
    integer(dpi), save :: l_here_index

! begin:
   if (header_exe) then
       l_here = 'gto_overlap_calc'
       l_here_index = routine_cur_index ()


   return
   endif

   call enter (l_here)
! the array of gto exponents  is needed

   type_need1 = objects_type (cur_needed_ind(1))
   type_need2 = objects_type (cur_needed_ind(2))
   type_need3 = objects_type (cur_needed_ind(3))

! checking types of the needs
   if (types_name (type_need1) /= "real_1")&
        call die (l_here, "object "+objects_names (cur_needed_ind (1))+" should be a one dimension real array")

   if (types_name(type_need2) /= 'integer_0') &
        call die (l_here, "object "+objects_names (cur_needed_ind (2))+" should be an integer")

   if (types_name(type_need3) /= 'integer_0') &
        call die (l_here, "object "+objects_names (cur_needed_ind (3))+" should be an integer")

! referencing the needed and created objects   
   call alias_byaddr_bld (gto_exp, objects_addr (cur_needed_ind (1)))
   call alias_byaddr_bld (sto_r_power, objects_addr (cur_needed_ind (2)))
   call alias_byaddr_bld (r_power, objects_addr (cur_needed_ind (3)))
 
   call alias_byaddr_bld (overlap, objects_addr (cur_create_ind (1)))

   gto_exp_nb = size (gto_exp)

   if (.not.associated (overlap)) call alloc (objects_names (cur_create_ind (1)),overlap, gto_exp_nb)

   do gto_i = 1, gto_exp_nb

      exp_sto = 1.d0

      overlap (gto_i) = slater_dot_gauss (gto_exp (gto_i) , exp_sto, r_power) 
         
   enddo ! do gto_i
      
   call exit (l_here)
      
 end subroutine gto_sto_overlap_calc
  
end module intr_func_mod
