!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module intracule_mod

  use basic_types_tools_mod
  use tools_mod
  use ee_dist_bywlk_mod
  use psi_bywlk_mod
  use density_bywlk_mod
  use drift_bywlk_mod
  use guide_menu_mod

! Declaration of global variables and default values
  logical           :: do_intracule = .false.
  logical           :: read_intracule = .false.

  character(len=max_string_len) :: intracule_estimator = 'histogram'

  real(dp)          :: dist_step     = 0.d0
  real(dp)          :: dist_max      = 0.d0

  integer(dpi)      :: block_intracule_nb = 0

! For histogram estimator
  integer(dpi)      :: histo_nb      = 0
  integer(dp)       :: dist_nb       = 0
  real(dp), pointer :: guide_histo (:) => NULL()
  real(dp), pointer :: intracule_histo_num (:) => NULL()
  real(dp)          :: sum_intracule_weight
  real(dp), pointer :: guide_histo_tot (:) => NULL()
  real(dp), pointer :: guide_freq       (:) => NULL()
  real(dp), pointer :: r_histo      (:) => NULL()
  real(dp), pointer :: intracule_histo  (:) => NULL()
  real(dp), pointer :: intracule_histo_norm  (:) => NULL()
  real(dp), pointer :: sum_intracule_histo_norm   (:) => NULL()
  real(dp), pointer :: sum_intracule_histo_norm_square  (:) => NULL()
  real(dp), pointer :: intracule_histo_norm_av   (:) => NULL()
  real(dp), pointer :: intracule_histo_norm_var  (:) => NULL()
  real(dp), pointer :: intracule_histo_norm_av_err  (:) => NULL()
  real(dp), pointer :: intracule_f4pir2_av    (:) => NULL()
  real(dp), pointer :: intracule_f4pir2_av_err   (:) => NULL()
  real(dp), pointer :: intracule_f_av         (:) => NULL()
  real(dp), pointer :: intracule_f_av_err        (:) => NULL()
  logical           :: init_intracule_histo_step = .true.
  logical           :: init_intracule_histo_block = .true.

! For improved estimators
  logical               :: do_intracule_improved = .false.
  integer(dpi)          :: grid_over_rij_nb = 0
  real(dp), pointer     :: grid_over_rij      (:) => NULL()
  real(dp), pointer     :: intracule_improved_num (:) => NULL()
  real(dp), pointer     :: intracule_improved_norm (:) => NULL()
  real(dp), pointer     :: sum_intracule_improved_norm (:) => NULL()
  real(dp), pointer     :: sum_intracule_improved_norm_square (:) => NULL()
  real(dp), pointer     :: intracule_improved_f_av (:) => NULL()
  real(dp), pointer     :: intracule_improved_f_var (:) => NULL()
  real(dp), pointer     :: intracule_improved_f_av_err (:) => NULL()
  real(dp), pointer     :: intracule_improved_f4pir2_av (:) => NULL()
  real(dp), pointer     :: intracule_improved_f4pir2_av_err (:) => NULL()
  integer(dpi)          :: intracule_improved_nb
  logical               :: init_intracule_improved_step = .true.
  logical               :: init_intracule_improved_block = .true.

  contains

! ========================================================================
  subroutine  intracule_menu
! ------------------------------------------------------------------------
!
! Name           : intracule_menu
!
! Description    : menu for calculation of intracule
!
! Created        : J. Toulouse, 18 Jul 2005
! Revised        : J. Toulouse, 16 Sep 2005: add improved estimator
! Revised        : J. Toulouse, 02 Oct 2005: 2 improved estimators
! ------------------------------------------------------------------------
  implicit none

! local:
  logical(bool)                              :: done
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='intracule_menu'
  call enter (l_here)

! initialize:

  done = .false.
  call next_command_get ('intracule')
  do while (.not.done)
   call next_word_get('intracule')
   if(trim(cur_word) .eq. 'help') then
    write (logf,*) 'intracule-h: comments:'
    write (logf,*) 'intracule-h:  build intracule using histogram of e-e distances'
    write (logf,*) 'intracule-h: syntaxe:'
    write (logf,*) 'intracule-h: intracule'
    write (logf,*) 'intracule-h:   help'
    write (logf,*) 'intracule-h:   estimator = [string]  histogram:improved1:improved2'
    write (logf,*) 'intracule-h:   read      = [boolean] restart calculation by reading intracule file?'
    write (logf,*) 'intracule-h:   step      = [integer] step on e-e distance'
    write (logf,*) 'intracule-h:   distmax   = [real]    maximum e-e distance'
    write (logf,*) 'intracule-h: end_intracule'
    write (logf,*) 'intracule-h: example:'
    write (logf,*) 'intracule-h: intracule step = 0.05 distmax=10 end_intracule'

   else if(token_is ('estimator', intracule_estimator)) then

   else if(token_is ('read', read_intracule))   then

   else if(token_is ('step', dist_step)) then

   else if(token_is ('distmax', dist_max))   then

   else
      call menu_exit (l_here, done)
   end if

  end do

! For now, the calculation of intracule is controlled by the flag do_intracule
  write (logf,*) trim(l_here), ': activate calculation of intracule'
  do_intracule = .true.


  if(trim(intracule_estimator) == 'histogram') then

! Stantard estimator by histogram
   write (logf,*) trim(l_here), ': use standard estimator for calculation of intracule'

! Parameters for intracule
   if(read_intracule) then

    call intracule_histo_read

   else ! read_intracule

    histo_nb = int(dist_max/dist_step) + 1
    if(histo_nb <= 0) then
     call die(l_here,'histo_nb = '+histo_nb+' should be > 0')
    endif
  
    write (logf,*) trim(l_here), ': Parameters for intracule histogram:'
    write (logf,*) trim(l_here), ': step = ', dist_step
    write (logf,*) trim(l_here), ': dist_max = ', dist_max
    write (logf,*) trim(l_here), ': histo_nb =',histo_nb

   endif ! read_intracule


  elseif(trim(intracule_estimator) == 'improved1') then

! Improved estimator 1
   write (logf,*) trim(l_here), ': use first-order renormalized improved estimator for calculation of intracule'

   grid_over_rij_nb = int(dist_max/dist_step) + 1

   if(grid_over_rij_nb <= 0) then
    call die(l_here,'grid_over_rij_nb = '+grid_over_rij_nb+' should be > 0')
   endif
  
   write (logf,*) trim(l_here), ': Parameters for intracule grid:'
   write (logf,*) trim(l_here), ': step = ', dist_step
   write (logf,*) trim(l_here), ': dist_max = ', dist_max
   write (logf,*) trim(l_here), ': grid_over_rij_nb=',grid_over_rij_nb

  elseif(trim(intracule_estimator) == 'improved2') then

! Improved estimator 2
   write (logf,*) trim(l_here), ': use second-order renormalized improved estimator for calculation of intracule'

   grid_over_rij_nb = int(dist_max/dist_step) + 1

   if(grid_over_rij_nb <= 0) then
    call die(l_here,'grid_over_rij_nb = '+grid_over_rij_nb+' should be > 0')
   endif
  
   write (logf,*) trim(l_here), ': Parameters for intracule grid:'
   write (logf,*) trim(l_here), ': step = ', dist_step
   write (logf,*) trim(l_here), ': dist_max = ', dist_max
   write (logf,*) trim(l_here), ': grid_over_rij_nb=',grid_over_rij_nb

  else

   call die(l_here,'The specified estimator for intracule is unknown. Use one of the following: histogram, improved1, improved2')

  endif !intracule_estimator


  call exit (l_here)

  end subroutine intracule_menu

! ==============================================================================
  subroutine intracule_step
! ------------------------------------------------------------------------------
!
! Descriptions   : driver for the routines 'intracule_..._step_bld' called at each step
! Descriptions   : of the MC dynamics
!
! Created        : J. Toulouse, 02 Oct 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here

  l_here = "intracule_step_bld"

! begin:
  call enter (l_here)

   if(trim(intracule_estimator) == 'histogram') then
    call node_exe_byrtn_name ('intracule_histo_step_bld')
   
   elseif(trim(intracule_estimator) =='improved1') then
    call node_exe_byrtn_name ('intracule_improved1_step_bld')

   elseif(trim(intracule_estimator) =='improved2') then
    call node_exe_byrtn_name ('intracule_improved2_step_bld')

   else
    write(logf,*) trim(l_here),': intracule_estimator = ', trim(intracule_estimator),' is unknown'
    call die(l_here,'intracule_estimator unknown')
   endif

  end subroutine intracule_step

! ==============================================================================
  subroutine intracule_block
! ------------------------------------------------------------------------------
!
! Descriptions   : driver for the routines 'intracule_..._block_bld' called at each block 
! Descriptions   : of the MC dynamics
!
! Created        : J. Toulouse, 02 Oct 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here

  l_here = "intracule_block_bld"

! begin:
  call enter (l_here)

   if(trim(intracule_estimator) == 'histogram') then
    call node_exe_byrtn_name ('intracule_histo_block_bld')
   
   elseif(trim(intracule_estimator) =='improved1') then
    call node_exe_byrtn_name ('intracule_improved_block_bld')

   elseif(trim(intracule_estimator) =='improved2') then
    call node_exe_byrtn_name ('intracule_improved_block_bld')

   else
    write(logf,*) trim(l_here),': intracule_estimator = ', trim(intracule_estimator),' is unknown'
    call die(l_here,'intracule_estimator unknown')
   endif

  end subroutine intracule_block

! ==============================================================================
  subroutine intracule_histo_step_bld
! ------------------------------------------------------------------------------
!
! Name           : intracule_histo_step_bld
!
! Creates        : intracule_histo_num (histo_nb)
!
! Descriptions   : increment histogram of ee_distance_bywlk at each step of MC dynamics
!
! Needs          : ee_distance_bywlk (elec, elec, walk)
!
! Authors        : J. Toulouse and R. Assaraf
! Date           : 18 Jul 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: histo_i
  integer(dpi)                  :: elec_i, elec_j, walk_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: dij
  real(dp)                      :: intracule_weight

! header:

  if (header_exe) then

    l_here = "intracule_histo_step_bld"

    call object_create ('intracule_histo_num')
    call object_create ('guide_histo')

    call object_needed ('ee_distance_bywlk')
    call object_needed ('psi_bywlk')
    call object_needed ('density_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_distance_bywlk, 1)
  walkers_nb        = size (ee_distance_bywlk, 3)

  if(histo_nb <= 0) then
   call die (l_here,'histo_nb = '+histo_nb+' should be > 0')
  endif

  if(dist_step <= 0) then
   call die (l_here,'dist_step = '+dist_step+' should be > 0')
  endif
! end dimensions


! allocations
   call alloc ('intracule_histo_num', intracule_histo_num, histo_nb)
   call alloc ('guide_histo', guide_histo, histo_nb)
! end allocations


! Initializations
  if (init_intracule_histo_step) then
   intracule_histo_num = zero
   guide_histo = zero
   sum_intracule_weight = zero
   init_intracule_histo_step = .false.
  endif

! with guiding wave function
  do walk_i = 1, walkers_nb

      intracule_weight = psi_bywlk(walk_i)**2/density_bywlk(walk_i)

      do elec_i = 1, electrons_nb
        do elec_j = elec_i + 1, electrons_nb

          dij = ee_distance_bywlk (elec_i, elec_j, walk_i)
          histo_i = int(dij/dist_step) + 1

          if (histo_i <= histo_nb) then 
             dist_nb = dist_nb + 1
             guide_histo(histo_i) = guide_histo(histo_i) + 1
             intracule_histo_num(histo_i) = intracule_histo_num(histo_i) + intracule_weight
             sum_intracule_weight = sum_intracule_weight + intracule_weight
          endif

      end do
    end do

  end do

! without guiding wave function
!  do walk_i = 1, walkers_nb
!      do elec_i = 1, electrons_nb
!        do elec_j = elec_i + 1, electrons_nb
!
!          dij = ee_distance_bywlk (elec_i, elec_j, walk_i)
!          histo_i = int(dij/dist_step) + 1
!
!          if(histo_i <= histo_nb) then 
!             dist_nb = dist_nb + 1
!             intracule_histo_num(histo_i) = intracule_histo_num(histo_i) + 1
!          endif
!
!      end do
!    end do
!  end do
!
!endif

  call exit (l_here)

  end subroutine intracule_histo_step_bld

! ========================================================================
  subroutine intracule_histo_block_bld
! ------------------------------------------------------------------------
!
! Description    : compute intracule at each block of MC dynamics
! Description    : f(r) 4 pi r^2 : intracule with volume element
! Description    : f(r)          : intracule without volume element
!
! Authors: J. Toulouse
! Date   : 18 Jul 2005
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: histo_i
  real(dp)                                   :: intracule_f4pir2_var, norm, volume_elt
  real(dp)                                   :: sum_intracule_histo
  integer(dpi)                               :: electrons_nb, electron_pairs_nb

if (header_exe) then

    l_here = "intracule_histo_block_bld"

    call object_create ('r_histo')
    call object_create ('guide_histo_tot')
    call object_create ('guide_freq')
    call object_create ('intracule_histo')
    call object_create ('intracule_histo_norm')
    call object_create ('sum_intracule_histo_norm')
    call object_create ('sum_intracule_histo_norm_square')
    call object_create ('intracule_histo_norm_av')
    call object_create ('intracule_histo_norm_var')
    call object_create ('intracule_histo_norm_av_err')
    call object_create ('intracule_f4pir2_av')
    call object_create ('intracule_f4pir2_av_err')
    call object_create ('intracule_f_av')
    call object_create ('intracule_f_av_err')

    call object_needed ('intracule_histo_num')
    call object_needed ('guide_histo')

    return

  end if

! begin:
  call enter (l_here)

! allocations
   call alloc('r_histo', r_histo, histo_nb)
   call alloc('guide_histo_tot', guide_histo_tot,  histo_nb)
   call alloc('guide_freq', guide_freq, histo_nb)
   call alloc('intracule_histo', intracule_histo, histo_nb)

   call alloc('intracule_histo_norm', intracule_histo_norm, histo_nb)
   call alloc('sum_intracule_histo_norm', sum_intracule_histo_norm, histo_nb)
   call alloc('sum_intracule_histo_norm_square', sum_intracule_histo_norm_square, histo_nb)
   call alloc('intracule_histo_norm_av', intracule_histo_norm_av, histo_nb)
   call alloc('intracule_histo_norm_var', intracule_histo_norm_var, histo_nb)
   call alloc('intracule_histo_norm_av_err', intracule_histo_norm_av_err, histo_nb)

   call alloc('intracule_f4pir2_av', intracule_f4pir2_av, histo_nb)
   call alloc('intracule_f4pir2_av_err', intracule_f4pir2_av_err, histo_nb)

   call alloc('intracule_f_av', intracule_f_av, histo_nb)
   call alloc('intracule_f_av_err', intracule_f_av_err, histo_nb)
! end allocations

! Initializations
  if(init_intracule_histo_block) then
   block_intracule_nb = 0
   guide_histo_tot = zero
   sum_intracule_histo_norm = zero
   sum_intracule_histo_norm_square = zero
   init_intracule_histo_block = .false.
  endif

! block number
  block_intracule_nb = block_intracule_nb + 1

! Electron pairs
  electrons_nb      = system%electrons_nb
  electron_pairs_nb = electrons_nb*(electrons_nb-1)/2


! with guiding wave function:

! r values
  do histo_i = 1, histo_nb
   r_histo(histo_i) = dist_step/2.d0 + (histo_i-1)*dist_step
  enddo

! numbers of events from guiding function for each bin
   guide_histo_tot = guide_histo_tot + guide_histo

! frequence of events form guiding function for each bin
   guide_freq = guide_histo_tot/dist_nb

! unnormalized histogram of intracule
   intracule_histo = intracule_histo_num/sum_intracule_weight
   
! sum of histogram
  sum_intracule_histo = zero
  do histo_i = 1, histo_nb
   sum_intracule_histo = sum_intracule_histo + intracule_histo(histo_i)
  enddo

! histogram of intracule normalized to 1
   intracule_histo_norm = intracule_histo/(sum_intracule_histo*dist_step)

! sums over the blocks of intracule_histo_norm and square of intracule_histo_norm
   sum_intracule_histo_norm = sum_intracule_histo_norm + intracule_histo_norm
   sum_intracule_histo_norm_square = sum_intracule_histo_norm_square + intracule_histo_norm**2

! average of intracule_histo_norm
   intracule_histo_norm_av =  sum_intracule_histo_norm/block_intracule_nb


! variance of intracule_histo_norm
    intracule_histo_norm_var = sum_intracule_histo_norm_square/block_intracule_nb - intracule_histo_norm_av**2

! statistical error on intracule_histo_norm_av
   if (block_intracule_nb == 1 ) then
    intracule_histo_norm_av_err = zero
   else
    intracule_histo_norm_av_err = dsqrt(intracule_histo_norm_var/(block_intracule_nb-1))
   endif

! average of intracule f(r) 4 pi r^2 (normalized to electron pairs nummber)
   intracule_f4pir2_av =  intracule_histo_norm_av*electron_pairs_nb

! statistical error on the average of intracule f(r) 4 pi r^2
   intracule_f4pir2_av_err = intracule_histo_norm_av_err*electron_pairs_nb

! average and statistical error of intracule f(r)
  do histo_i = 1, histo_nb
   volume_elt = 4.d0*pi*r_histo(histo_i)*r_histo(histo_i)
   intracule_f_av(histo_i)   = intracule_f4pir2_av(histo_i)/volume_elt
   intracule_f_av_err(histo_i)  = intracule_f4pir2_av_err(histo_i)/volume_elt
  enddo


!
! without guiding wave function: compute statistical error assuming independence of each bin in this histogram
!  norm = electron_pairs_nb/dist_step
!  do histo_i = 1, histo_nb
!
!   r_histo(histo_i) = dist_step/2.d0 + (histo_i-1)*dist_step
!
!   intracule_histo(histo_i) = intracule_histo_num(histo_i)/dist_nb
!
!   norm = electron_pairs_nb/dist_step
!
!   intracule_f4pir2(histo_i) =  intracule_histo(histo_i)*norm
!   intracule_f4pir2_var = intracule_histo(histo_i)*(1.d0-intracule_histo(histo_i))*norm*norm/(dist_nb-1)
!   intracule_f4pir2_av_err(histo_i) = dsqrt(intracule_f4pir2_var)
!
!   volume_elt = 4.d0*pi*r_histo(histo_i)*r_histo(histo_i)
!   f(histo_i)      = intracule_f4pir2(histo_i)/volume_elt
!   intracule_f_av_err(histo_i)  = intracule_f4pir2_av_err(histo_i)/volume_elt
!
!  enddo

! Reinitialization of intracule_step
  init_intracule_histo_step = .true.

! Write intracule
  call intracule_histo_write

  call exit (l_here)

  end subroutine intracule_histo_block_bld

! ========================================================================
  subroutine intracule_histo_write
! ------------------------------------------------------------------------
!
! Description    : write intracule density on file
! Description    : f(r) 4 pi r^2 : intracule with volume element
! Description    : f(r)          : intracule without volume element
!
! Authors: J. Toulouse
! Date   : 18 Jul 2005
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit, histo_i


! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_title)        :: f_title

! begin:
  l_here ='intracule_histo_write'
  call enter (l_here)


! open
  f_type = 'intracule-out'//blanks
  unit   = unit_from_type (f_type)
  unit = file_open_and_cat_from_type (f_type)

  if(unit .eq. 0) then
    call files_show
    message = 'in routine'*l_here*' intracule type not found'
    call die (l_here, message)
  end if

  f_name   = files_names(unit)
  f_title  = files_titles(unit)
  f_action = files_actions(unit)
  
  call file_header_write (unit)
  write(unit,'(a)')         'end_of_header'
  write(unit,*) ''

  write(unit,'(a,i5)')      'number_of_electrons      =',system%electrons_nb
  write(unit,'(a,3e25.15)') 'dist_step                =',dist_step
  write(unit,'(a,3e25.15)') 'dist_max                 =',dist_max  
  write(unit,'(a,i25)')     'histo_nb                 =',histo_nb
  write(unit,'(a,i25)')     'dist_nb                  =',dist_nb
  write(unit,'(a,i25)')  'block_intracule_nb       =',block_intracule_nb

  write(unit,*) ''
  write(unit,*) '            r                intracule_f4pir2_av    intracule_f4pir2_av_err       intracule_f_av        intracule_f_av_err          guide_freq         sum_intracule_histo_norm  sum_intracule_histo_norm_square'
   
  do histo_i = 1, histo_nb

  write(unit,'(3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15)') r_histo(histo_i), &
        intracule_f4pir2_av(histo_i),intracule_f4pir2_av_err(histo_i),intracule_f_av(histo_i), &
        intracule_f_av_err(histo_i),guide_freq(histo_i),sum_intracule_histo_norm(histo_i),     &
        sum_intracule_histo_norm_square(histo_i)


  enddo


  call file_close_by_unit (unit) 

   write (logf,*)trim(l_here),': intracule has been written on file: >',trim(f_name),'<'

  call exit (l_here)

  end subroutine intracule_histo_write

! ========================================================================
  subroutine intracule_histo_read 
! ------------------------------------------------------------------------
!
! Description    : read  intracule information from file for restart
!
! Authors: J. Toulouse
! Date   : 11 Aug 2005
! ------------------------------------------------------------------------
  implicit none

! local:
  integer(dpi)                               :: unit, histo_i
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_title)        :: f_title

  integer(dpi)                               :: electrons_nb
  real(dp)                                   :: real_temp
  character(len=256)                         :: string_temp

! begin:
  l_here ='intracule_histo_read'
  call enter (l_here)



! open
  f_type = 'intracule-in'//blanks
  unit = unit_from_type (f_type)
  unit = file_open_and_cat_from_type (f_type)

  if(unit .eq. 0) then
    call files_show
    message = 'in routine'*l_here*' intracule type not found'
    call die (l_here, message)
  end if

  f_name   = files_names(unit)
  f_title  = files_titles(unit)
  f_action = files_actions(unit)
  
  write (logf,*) trim(l_here),': calculation of intracule will be restarted from file: >',trim(f_name),'<'

  call file_header_read (unit)
  read(unit,'(a)')  string_temp
  read(unit,*)      string_temp, string_temp, electrons_nb
  read(unit,*)      string_temp, string_temp, dist_step
  read(unit,*)      string_temp, string_temp, dist_max  
  read(unit,*)      string_temp, string_temp, histo_nb
  read(unit,*)      string_temp, string_temp, dist_nb
  read(unit,*)      string_temp, string_temp, block_intracule_nb

  write(logf,*) 'electrons_nb = ',electrons_nb
  write(logf,*) 'dist_step    = ',dist_step
  write(logf,*) 'dist_max     = ',dist_max
  write(logf,*) 'histo_nb     = ',histo_nb
  write(logf,*) 'dist_nb      = ',dist_nb
  write(logf,*) 'block_intracule_nb = ',block_intracule_nb

! allocations
   call alloc('guide_histo_tot',guide_histo_tot,histo_nb)
   call alloc('sum_intracule_histo_norm', sum_intracule_histo_norm, histo_nb)
   call alloc('sum_intracule_histo_norm_square', sum_intracule_histo_norm_square, histo_nb)
! end allocations

! Initialization
  guide_histo_tot = zero

  read(unit,'(a)') string_temp
  read(unit,'(a)') string_temp
   
  do histo_i = 1, histo_nb

  read(unit,'(3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15)') real_temp, &
        real_temp,real_temp,real_temp, &
        real_temp,real_temp,sum_intracule_histo_norm(histo_i),     &
        sum_intracule_histo_norm_square(histo_i)


  enddo

  call file_close_by_unit (unit) 

  write (logf,*) trim(l_here),': sum_intracule_histo_norm and sum_intracule_histo_norm_square have been read on file: >',trim(f_name),'<'

! Prevent initialization of intracule_histo_block_bld
  init_intracule_histo_block = .false.

  call exit (l_here)

  end subroutine intracule_histo_read

! ==============================================================================
  subroutine grid_over_rij_bld
! ------------------------------------------------------------------------------
!
! Name           : grid_over_rij_bld
!
! Creates        : grid_over_rij (grid_over_rij_nb)
!
! Descriptions   : Grid over rij for calculate of intracule with improved estimator
!
! Created        : J. Toulouse, 16 Sep 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: grid_i

  
! header:

  if (header_exe) then

    l_here = "grid_over_rij_bld"

    call object_create ('grid_over_rij')

    return

  end if

! begin:
  call enter (l_here)

! Requirements
  if(grid_over_rij_nb <= 0) then
   call die (l_here,'grid_over_rij_nb = '+grid_over_rij_nb+' should be > 0')
  endif

  if(dist_step <= 0) then
   call die (l_here,'dist_step = '+dist_step+' should be > 0')
  endif

! Allocation
  call alloc ('grid_over_rij', grid_over_rij, grid_over_rij_nb)

  do grid_i = 1, grid_over_rij_nb

   grid_over_rij(grid_i) = (grid_i - 1) * dist_step

  enddo


  call exit (l_here)

  end subroutine grid_over_rij_bld

! ==============================================================================
  subroutine intracule_improved_old_step_bld
! ------------------------------------------------------------------------------
!
! Creates        : intracule_improved_num (grid_over_rij_nb)
!
! Descriptions   : Calculate improved estimator of intracule at each step of the MC dynamics
!
! Needs          : ee_distance_bywlk
! Needs          : ee_dist_xyz_bywlk
! Needs          : drift_bywlk
!
! Created        : J. Toulouse, 16 Sep 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: grid_i
  integer(dpi)                  :: elec_i, elec_j, walk_i, dim_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dij, r 
  real(dp)                      :: dotproduct, intracule_improved
  real(dp)                      :: factor

  
! header:

  if (header_exe) then

    l_here = "intracule_improved_step_bld"

    call object_create ('intracule_improved_num')

    call object_needed ('grid_over_rij')
    call object_needed ('ee_dist_xyz_bywlk')
    call object_needed ('ee_distance_bywlk')
    call object_needed ('drift_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb  = size(ee_dist_xyz_bywlk,1)
  dimensions_nb = size(ee_dist_xyz_bywlk,3)
  walkers_nb    = size(ee_dist_xyz_bywlk,4)
! end dimensions


! allocations
   call alloc ('intracule_improved_num', intracule_improved_num, grid_over_rij_nb)
! end allocations


! Initializations
  if(init_intracule_improved_step) then
   intracule_improved_num = zero
   intracule_improved_nb = 0
   init_intracule_improved_step = .false.
  endif

! without guiding wave function

  do walk_i = 1, walkers_nb

     intracule_improved_nb = intracule_improved_nb + 1

      do elec_j = 1, electrons_nb
!        do elec_i = elec_j + 1, electrons_nb
        do elec_i = 1, electrons_nb

          if ( elec_i == elec_j ) cycle

          dij = ee_distance_bywlk (elec_i, elec_j, walk_i)

          dotproduct = zero
          do dim_i = 1, dimensions_nb
             dotproduct = dotproduct +  &
                ee_dist_xyz_bywlk(elec_j, elec_i, dim_i, walk_i) * drift_bywlk(elec_j, dim_i, walk_i) 
          end do

!          intracule_improved = 2.d0 * ( (dotproduct + 1.d0) / dij)
          intracule_improved = ( (dotproduct + 1.d0) / dij)
!          intracule_improved = ( (dotproduct + 0.d0) / dij)
!          intracule_improved = ( ( 1.d0) / dij)
!          intracule_improved = ( (dotproduct + 1.d0) )
!          write(logf,*) trim(l_here),': intracule_improved=',intracule_improved

          do grid_i = 1, grid_over_rij_nb

            r = grid_over_rij(grid_i)

            if ( dij < r ) then
             factor = 0.5d0
            else
             factor = -0.5d0
            endif
            
            intracule_improved_num(grid_i) = intracule_improved_num(grid_i) + factor*intracule_improved

        end do !grid_i

      end do !elec_j
    end do !elec_i
  end do !walk_i

  call exit (l_here)

  end subroutine intracule_improved_old_step_bld

! ==============================================================================
  subroutine intracule_improved1_step_bld
! ------------------------------------------------------------------------------
!
! Descriptions   : Calculate first-order renormalized improved estimator of intracule at each step of the MC dynamics
!
! Needs          : ee_distance_bywlk
! Needs          : ee_dist_xyz_bywlk
! Needs          : drift_bywlk
!
! Created        : J. Toulouse, 02 Oct 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: grid_i
  integer(dpi)                  :: elec_i, elec_j, walk_i, dim_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  real(dp)                      :: dij, r 
  real(dp)                      :: dotproduct, intracule_temp
  real(dp)                      :: oneover4pi

  
! header:

  if (header_exe) then

    l_here = "intracule_improved1_step_bld"

    call object_create ('intracule_improved_num')

    call object_needed ('grid_over_rij')
    call object_needed ('ee_dist_xyz_bywlk')
    call object_needed ('ee_distance_bywlk')
    call object_needed ('drift_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb  = size(ee_dist_xyz_bywlk,1)
  dimensions_nb = size(ee_dist_xyz_bywlk,3)
  walkers_nb    = size(ee_dist_xyz_bywlk,4)
! end dimensions


! allocations
   call alloc ('intracule_improved_num', intracule_improved_num, grid_over_rij_nb)
! end allocations


! Initializations
  if(init_intracule_improved_step) then
   intracule_improved_num = zero
   intracule_improved_nb = 0
   init_intracule_improved_step = .false.
  endif

! without guiding wave function

  oneover4pi = 1.d0/(4.d0*pi)

  do walk_i = 1, walkers_nb

     intracule_improved_nb = intracule_improved_nb + 1

      do elec_j = 1, electrons_nb
        do elec_i = 1, electrons_nb

          if ( elec_i == elec_j ) cycle

          dij = ee_distance_bywlk (elec_i, elec_j, walk_i)

          dotproduct = zero
          do dim_i = 1, dimensions_nb
             dotproduct = dotproduct +  &
                drift_bywlk(elec_j, dim_i, walk_i) * ee_dist_xyz_bywlk(elec_j, elec_i, dim_i, walk_i) 
          end do

          intracule_temp = oneover4pi * dotproduct / dij**3

          do grid_i = 1, grid_over_rij_nb

            r = grid_over_rij(grid_i)

            if ( dij >= r ) then

            intracule_improved_num(grid_i) = intracule_improved_num(grid_i) - intracule_temp

            endif
            

        end do !grid_i

      end do !elec_j
    end do !elec_i
  end do !walk_i

  call exit (l_here)

  end subroutine intracule_improved1_step_bld

! ==============================================================================
  subroutine intracule_improved2_step_bld
! ------------------------------------------------------------------------------
! Descriptions   : Calculate second-order renormalized improved estimator of intracule at each step of the MC dynamics
!
! Created        : J. Toulouse, 29 Sep 2005
! ------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: grid_i
  integer(dpi)                  :: elec_i, elec_j, walk_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: dij, r 
  real(dp)                      :: intracule_temp
  real(dp)                      :: oneover4pi

  
! header:

  if (header_exe) then

    l_here = "intracule_improved2_step_bld"

!    call object_create ('intracule_improved_num')

    call object_needed ('grid_over_rij')
    call object_needed ('ee_distance_bywlk')
    call object_needed ('drift_square_bywlk')
    call object_needed ('psi_lapl_over_psi_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb  = size(ee_distance_bywlk,1)
  walkers_nb    = size(ee_distance_bywlk,3)

! allocations
   call alloc ('intracule_improved_num', intracule_improved_num, grid_over_rij_nb)


! Initializations
  if(init_intracule_improved_step) then
   intracule_improved_num = zero
   intracule_improved_nb = 0
   init_intracule_improved_step = .false.
  endif

  oneover4pi = 1.d0/(4.d0*pi)

! without guiding wave function

  do walk_i = 1, walkers_nb

     intracule_improved_nb = intracule_improved_nb + 1

      do elec_j = 1, electrons_nb
        do elec_i = 1, electrons_nb

          if ( elec_i == elec_j ) cycle

          dij = ee_distance_bywlk (elec_i, elec_j, walk_i)

          intracule_temp  = oneover4pi*(drift_square_bywlk(elec_i, walk_i) + psi_lapl_over_psi_bywlk(elec_i, walk_i))

          do grid_i = 1, grid_over_rij_nb

            r = grid_over_rij(grid_i)

!            intracule_improved_num(grid_i) = intracule_improved_num(grid_i) &
!                    - (r**2) * drift_plus_lapl * ( dij + r - dabs(dij - r) ) / dij

            if ( dij < r ) then

             intracule_improved_num(grid_i) = intracule_improved_num(grid_i) - intracule_temp / r

            else

             intracule_improved_num(grid_i) = intracule_improved_num(grid_i) - intracule_temp / dij

            endif

          end do !grid_i

      end do !elec_j
    end do !elec_i
  end do !walk_i

  call exit (l_here)

  end subroutine intracule_improved2_step_bld

! ========================================================================
  subroutine intracule_improved_block_bld
! ------------------------------------------------------------------------
!
! Description    : compute intracule at each block of MC dynamics
! Description    : f(r) 4 pi r^2 : intracule with volume element
! Description    : f(r)          : intracule without volume element
! Description    : using improved estimators
!
! Created        : J. Toulouse, 16 Sep 2005
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: grid_i
  real(dp)                                   :: volume_elt

 if (header_exe) then

    l_here = "intracule_improved_block_bld"

    call object_create ('intracule_improved_norm')
    call object_create ('sum_intracule_improved_norm')
    call object_create ('sum_intracule_improved_norm_square')
    call object_create ('intracule_improved_f_av')
    call object_create ('intracule_improved_f_var')
    call object_create ('intracule_improved_f_av_err')
    call object_create ('intracule_improved_f4pir2_av')
    call object_create ('intracule_improved_f4pir2_av_err')

    call object_needed ('grid_over_rij')
    call object_needed ('intracule_improved_num')

    return

  end if

! begin:
  call enter (l_here)

! allocations
   call alloc('intracule_improved_norm', intracule_improved_norm, grid_over_rij_nb)
   call alloc('sum_intracule_improved_norm', sum_intracule_improved_norm, grid_over_rij_nb)
   call alloc('sum_intracule_improved_norm_square', sum_intracule_improved_norm_square, grid_over_rij_nb)
   call alloc('intracule_improved_f_av', intracule_improved_f_av, grid_over_rij_nb)
   call alloc('intracule_improved_f_var', intracule_improved_f_var, grid_over_rij_nb)
   call alloc('intracule_improved_f_av_err', intracule_improved_f_av_err, grid_over_rij_nb)
   call alloc('intracule_improved_f4pir2_av', intracule_improved_f4pir2_av, grid_over_rij_nb)
   call alloc('intracule_improved_f4pir2_av_err', intracule_improved_f4pir2_av_err, grid_over_rij_nb)
! end allocations

! Initializations
  if(init_intracule_improved_block) then
   block_intracule_nb = 0
   sum_intracule_improved_norm = zero
   sum_intracule_improved_norm_square = zero
   init_intracule_improved_block = .false.
  endif

! block number
  block_intracule_nb = block_intracule_nb + 1

! intracule normalized to N(N-1)/2
  intracule_improved_norm = intracule_improved_num/intracule_improved_nb
   
! sums over the blocks of intracule_improved_norm and square of intracule_improved_norm
  sum_intracule_improved_norm = sum_intracule_improved_norm + intracule_improved_norm
  sum_intracule_improved_norm_square = sum_intracule_improved_norm_square + intracule_improved_norm**2

! average of intracule_improved_norm = average of intracule f(r) 
  intracule_improved_f_av =  sum_intracule_improved_norm/block_intracule_nb

! variance of the average of intracule f(r) 
  intracule_improved_f_var = sum_intracule_improved_norm_square/block_intracule_nb - intracule_improved_f_av**2

! statistical error on the average of intracule f(r) 
   if (block_intracule_nb == 1 ) then
    intracule_improved_f_av_err = zero
   else
    intracule_improved_f_av_err = dsqrt(intracule_improved_f_var/(block_intracule_nb-1))
   endif

! average and statistical error of intracule f(r) 4 pi r^2
  do grid_i = 1, grid_over_rij_nb
   volume_elt = 4.d0*pi*grid_over_rij(grid_i)*grid_over_rij(grid_i)
   intracule_improved_f4pir2_av(grid_i)   = intracule_improved_f_av(grid_i)*volume_elt
   intracule_improved_f4pir2_av_err(grid_i)  = intracule_improved_f_av_err(grid_i)*volume_elt
  enddo


! Reinitialization of intracule_step
  init_intracule_improved_step = .true.

! Write intracule
  call intracule_improved_write

  call exit (l_here)

  end subroutine intracule_improved_block_bld


! ========================================================================
  subroutine intracule_improved_write
! ------------------------------------------------------------------------
!
! Description    : write intracule density on file
! Description    : f(r) 4 pi r^2 : intracule with volume element
! Description    : f(r)          : intracule without volume element
! Description    : using improved estimator
!
! Created        : J. Toulouse, 17 Sep 2005
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit, grid_i


! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_title)        :: f_title

! begin:
  l_here ='intracule_improved_write'
  call enter (l_here)


! open
  f_type = 'intracule-out'//blanks
  unit   = unit_from_type (f_type)
  unit = file_open_and_cat_from_type (f_type)

  if(unit .eq. 0) then
    call files_show
    message = 'in routine'*l_here*' intracule type not found'
    call die (l_here, message)
  end if

  f_name   = files_names(unit)
  f_title  = files_titles(unit)
  f_action = files_actions(unit)
  
  call file_header_write (unit)
  write(unit,'(a)')         'end_of_header'
  write(unit,*) ''

  write(unit,'(a,a)')      'intracule_estimator      =',trim(intracule_estimator)
  write(unit,'(a,i5)')      'number_of_electrons      =',system%electrons_nb
  write(unit,'(a,3e25.15)') 'dist_step                =',dist_step
  write(unit,'(a,3e25.15)') 'dist_max                 =',dist_max  
  write(unit,'(a,i25)')     'grid_over_rij_nb         =',grid_over_rij_nb
  write(unit,'(a,i25)')     'block_intracule_nb       =',block_intracule_nb

  write(unit,*) ''
  write(unit,*) '            r                intracule_f4pir2_av    intracule_f4pir2_av_err       intracule_f_av        intracule_f_av_err      sum_intracule_norm  sum_intracule_norm_square'
   
  do grid_i = 1, grid_over_rij_nb

  write(unit,'(3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15,3e25.15)') grid_over_rij(grid_i), &
        intracule_improved_f4pir2_av(grid_i),intracule_improved_f4pir2_av_err(grid_i),intracule_improved_f_av(grid_i), &
        intracule_improved_f_av_err(grid_i),sum_intracule_improved_norm(grid_i),     &
        sum_intracule_improved_norm_square(grid_i)


  enddo


  call file_close_by_unit (unit) 

   write (logf,*)trim(l_here),': intracule has been written on file: >',trim(f_name),'<'

  call exit (l_here)

  end subroutine intracule_improved_write

end module intracule_mod
