! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

! blds ortho_basis such that t ortho_basis dot_prod_mat orho_basis = Id
subroutine ortho_basis_bld (ortho_basis,dot_prod_mat,n)
  use control_mod
  implicit none
  integer(dpi) :: n
  real(dp)     :: dot_prod_mat (n,n)
  real(dp)     :: ortho_basis (n,n), ortho_t (n,n)
  integer(dpi) ::  nrot, i
  real(dp)     :: eigen_values (n)
  real(dp)     :: c(n,n), c2(n,n)
  character(len=max_string_len_routine_name) :: l_here
  l_here = 'ortho_basis_bld'
  call jacobi (dot_prod_mat, n, n, eigen_values, ortho_basis,nrot)
  print*, 'ortho_basis eigen_values',eigen_values 

  do i= 1, n
     if (eigen_values (i) < 0_dp) call die (l_here,'not positive definite')
     if (eigen_values (i) < 0.01d0) then
        ortho_basis (:,i) = 0.d0
     else
         ortho_basis (:,i) = ortho_basis (:,i)/ sqrt (eigen_values (i))
     endif
   enddo ! do i
 ! call  mat_mult (c,dot_prod_mat,ortho_basis,n,n,n)
  call mat_transpose (ortho_t,ortho_basis,n,n)
  call  mat_mult (c,dot_prod_mat,ortho_basis,n,n,n)
  call mat_mult (c,ortho_t,c,n,n,n)
  
end subroutine ortho_basis_bld

 subroutine mat_mult (c,a,b,m,n,p)
 use control_mod
 integer(dpi) :: m,n,p
 real(dp) :: c(m,p), work(m,p),a(m,n), b (n,p)
 integer(dpi) :: i,j
 do i= 1,m
  do j= 1,p
   work (i,j) = dot_product ( a (i,:), b (:,j) )
  enddo
  enddo
  c=work
  end subroutine mat_mult

 subroutine mat_transpose (ta,a,m,n)
 use control_mod
 integer(dpi) :: m,n
 real(dp) :: ta (n,m), work(n,m), a (n,m)
 integer(dpi) :: i,j
 do i= 1,n
  do j= 1,m
    work (i,j) = a (j,i)
  enddo
  enddo
  ta=work
 end subroutine mat_transpose

  subroutine invert_mat (a,n,det,work)
  use control_mod
  implicit none
  integer(dpi) :: n
  real(dp) :: a(n,n),det,work(n), b(n,1)
  call gauss_nopiv (a, n, det, work)
!    b = 0_dp
!   call gaussj(a,n,n,b,1,n,det)  
  end subroutine invert_mat

! from numerical recipes
  subroutine gauss_nopiv (a, n, det, work)
! invert matrix a in place-gaussian elimination with no pivoting
  use control_mod
  implicit none

  integer(dpi) :: n
  real(dp) :: a(n,n),det,work(n)
 
  integer(dpi) :: nm1,kp1,k,j,jj,i,kb,km1

! begin:
  nm1 = n - 1
  do k = 1, nm1
   kp1 = k + 1
 
!           compute multipliers
 
   work(1) = -1.d0/a(k,k)
   do j = kp1,n
    a(j,k) = work(1)*a(j,k)
   end do
 
!           row elimination with column indexing
 
   do j = kp1, n
    do jj = kp1,n
     a(jj,j) = a(jj,j)+a(k,j)*a(jj,k)
    end do
   end do
! close loop over k
  end do
!
  det = a(1,1)
  do i = 2,n
   det = det*a(i,i)
  end do
  do k = 1, n
   a(k,k) = 1.d0/a(k,k)
   km1 = k-1
   if (km1.ne.0) then
    do jj = 1,km1
     a(jj,k) = -a(k,k)*a(jj,k)
    end do
   end if
   kp1 = k + 1
   if (n .ge. kp1) then
    do j = kp1, n
     work(1) = a(k,j)
     a(k,j) = 0.
     do jj = 1,k
      a(jj,j) = a(jj,j)+work(1)*a(jj,k)
     end do
    end do
   end if
  end do
!
!        form inverse(u)*inverse(l)
!
  do kb = 1, nm1
   k = n - kb
   kp1 = k + 1
   do i = kp1, n
    work(i) = a(i,k)
    a(i,k) = 0.d0
   end do
   do j = kp1, n
    do jj = 1,n
     a(jj,k) = a(jj,k)+work(j)*a(jj,j)
    end do
   end do
  end do
  return
  end subroutine gauss_nopiv
