! -*-  f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_a_ee_1_4_mod

  use basic_types_tools_mod
  use configurations_mod
  use ee_dist_bywlk_mod
  use electrons_spins_mod
 
! Description : Calculated once:
! Creates     : jastrow_a_ee_1_4 (elec, elec, conf, deg)

  real(dp), pointer :: jastrow_a_ee_1_4 (:, :, :, :)  => NULL()

  contains

! ===========================================================================
  subroutine jastrow_a_ee_1_4_bld 
! ---------------------------------------------------------------------------
!
! Name           : jastrow_a_ee_1_4_bld
!
! Creates        : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Needs          : conf_jastr_a_ee (lik, deg)
!
! Descriptions   : the 4 coefficients of the polynomial in r_ee_bar 
! Descriptions   : independant of walkers
! Descriptions   : same lik configurations(conf_i)%conf_jastr%a_ee(1,:)
! Descriptions   : not-same lik configurations(conf_i)%conf_jastr%a_ee(2,:)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, conf_i, deg_i
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: max_degree_a_ee  

! header:

  if (header_exe) then

    l_here = "jastrow_a_ee_1_4_bld"

    call object_create ('jastrow_a_ee_1_4')

    call object_needed ('conf_jastr_a_ee')

    return

  end if

! begin:
  call enter (l_here)

  max_degree_a_ee = system%max_degree_a_ee

! requirements:
  call check_asso_conf ('configurations', configurations, &
       'configurations_init', configurations_init)

  call  check_size (l_here, 'system%configurations_nb',   &
        system%configurations_nb,'configurations', size (configurations, 1), 1)

  configurations_nb = system%configurations_nb

  do conf_i = 1, configurations_nb
!  call check_association ('configurations(conf_i)%conf_jastr%a_en',  &
!                           configurations(conf_i)%conf_jastr%a_en,    &
!                          'cur_conf_jastr_bld', cur_conf_jastr_bld, conf_i)
  end do
  
     if (.not.associated(electrons_spins)) then
    write (logf,*)trim(l_here),': electrons_spins not associated'
    write (logf,*)trim(l_here),': calling electrons_spins_bld'
    call electrons_spins_bld
  else
    if (trace_depend) write (logf,*)trim(l_here),': electrons_spins already associated'
  end if

! end requirements

! dimensions:
     configurations_nb = system%configurations_nb
     electrons_nb      = size (electrons_spins, 1)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
  end if 

  if(debug) then
   do conf_i = 1,configurations_nb
     write (logf,*)trim(l_here),'-d: conf_jastr%a_ee(1..2,:)'
     write (logf,*)trim(l_here),'-d: conf_jastr_a_ee(',conf_i,',1,:) =', &
          conf_jastr_a_ee(conf_i,1,:)
     write (logf,*)trim(l_here),'-d: conf_jastr(',conf_i,',2,:) =', &
          conf_jastr_a_ee(conf_i,2,:)
   end do  ! conf_i
  end if

! allocations
  if(.not.associated(jastrow_a_ee_1_4)) then
    write (logf,*)trim(l_here),': jastrow_a_ee_1_4 not associated'
    call allocation ('jastrow_a_ee_1_4', jastrow_a_ee_1_4,   &
                      electrons_nb, electrons_nb, configurations_nb, jastr_pow_max)
  else
    if (trace_depend) write (logf,*)trim(l_here),': jastrow_a_ee_1_4 left in memory'
  end if

! end allocations

  do conf_i = 1, configurations_nb
    do elec_j = 1, electrons_nb
       do elec_i = 1, electrons_nb

      if (electrons_spins(elec_i) == electrons_spins(elec_j)) then 

         jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, :) =  &
                           conf_jastr_a_ee(conf_i, 1,:)

      else

         jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, :) =  &
                            conf_jastr_a_ee(conf_i,2,:)
      endif

      end do
    end do
  end do

  if(debug) then
    do deg_i = 1, max_degree_a_ee
      do elec_j = 1,electrons_nb
         do elec_i = 1, electrons_nb
         do conf_i = 1, configurations_nb
            write (logf,*)trim(l_here),'-d: jastrow_a_ee_1_4 (elec, elec, conf, deg) '
            write (logf,*)trim(l_here),'-d: jastrow_a_ee_1_4 (',elec_i,',',elec_j,',',conf_i,',',deg_i,') =', &
                                      jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, deg_i)
         end do ! conf_i
       end do ! elec_i
      end do ! elec_j
    end do ! deg_i
  end if

  call exit (l_here)

  end subroutine jastrow_a_ee_1_4_bld

end module jastrow_a_ee_1_4_mod
