! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_a_en_1_4_mod

  use basic_types_tools_mod
  use configurations_mod

! Description : coefficient of r_bar^n of u_en (n= 1..4)

  real(dp), pointer :: jastrow_a_en_1_4 (:, :, :)     => NULL()

  contains

! ========================================================================
  subroutine jastrow_a_en_1_4_bld 
! ------------------------------------------------------------------------
!
! Name           : jastrow_a_en_1_4_bld
!
! Creates        : jastrow_a_en_1_4 (atom, conf, deg)
!
! Needs          : configurations(conf)%conf_jastr%a_en (deg, atom)
!
! Descriptions   : coefficient of r_bar^n of u_en (n= 1..4)
!
! Authors        : F. Colonna
! Date           : 14 Jan 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: conf_i, atom_i, deg_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb

! header:

  if (header_exe) then

    l_here = "jastrow_a_en_1_4_bld"

    call object_create ('jastrow_a_en_1_4')

    call object_needed ('conf_jastr_a_en')
    call object_needed ('configurations')
    call object_needed ('atoms_charges')
    return

  end if

! begin:
  call enter (l_here)

! requirements:	


    include 'chksiz_configurations.inc'

! dimensions
    configurations_nb = system%configurations_nb
    atoms_nb = size (atoms_charges)

    if(atoms_nb <= 0) call die (l_here, ' nb of atoms ='+atoms_nb)

! allocations:
  
    call alloc ('jastrow_a_en_1_4', jastrow_a_en_1_4,   &
         atoms_nb, configurations_nb, jastr_pow_max)
  
! end allocations

    do conf_i = 1, configurations_nb
       do atom_i = 1, atoms_nb
          jastrow_a_en_1_4 (atom_i, conf_i, :) =  &
               conf_jastr_a_en(conf_i,:, atom_i)
       end do
    end do
 
 if(debug) then
    do conf_i = 1,configurations_nb
       do deg_i = 1, jastr_pow_max
          do atom_i = 1, atoms_nb
             write (logf,*)trim(l_here),'-d: jastrow_a_en_1_4 (atom, conf, deg) '
             write (logf,*)trim(l_here),'-d: jastrow_a_en_1_4 (',atom_i,',',conf_i,',',deg_i,') =', &
                                      jastrow_a_en_1_4 (atom_i, conf_i, deg_i)
          end do ! atom_i
       end do ! deg_i
    end do ! walk_i
 end if

 call exit (l_here)

end subroutine jastrow_a_en_1_4_bld

end module jastrow_a_en_1_4_mod
