! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_exponent_bywlk_mod

! Improve: formulas not verified

  use basic_types_tools_mod
  use ee_r_bar_bywlk_mod
  use ea_r_bar_bywlk_mod
  use electrons_spins_mod
  use jastrow_u_en_bywlk_mod
  use jastrow_u_ee_bywlk_mod
  use jastrow_u_een_bywlk_mod

! Description : jastrow_exponent (c,w) = Sum i Sum j>i u_ee 
! Description :                        - Sum i Sum a   u_en 
! Description :                        + Sum i Sum j>i Sum a u_een

! Creates     : jastrow_exponent_bywlk (conf, walk)
! Creates     : jastrow_exponent_grad_bywlk (elec, conf, dim, walk)
! Creates     : jastrow_exponent_lapl_bywlk (elec, conf, walk)

  real(dp), pointer :: jastrow_exponent_bywlk (:,:)          => NULL()
  real(dp), pointer :: jastrow_exponent_grad_bywlk (:,:,:,:) => NULL()
  real(dp), pointer :: jastrow_exponent_lapl_bywlk (:,:,:)    => NULL()

  contains

! ===============================================================================
  subroutine jastrow_exponent_bywlk_bld 
! -------------------------------------------------------------------------------
!
! Name           : jastrow_exponent_bywlk_bld
!
! Creates        : jastrow_exponent_bywlk (conf, walk)
!
! Descriptions   : exponent= Sum_conf u_ee - Sum u_en + Sum u_een
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : u_en_bywlk (elec, atom, conf, walk)
! Needs          : u_ee_bywlk (elec, elec, conf, walk)
! Needs          : u_een_bywlk (elec, elec, atom, conf, walk)
!
! Improve : true that minus (-) u_en  in formula ?
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i, atom_i
  double precision              :: j_ee, j_en, j_een

! header:

  if (header_exe) then

    l_here = "jastrow_exponent_bywlk_bld"

    call object_create ('jastrow_exponent_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('u_en_bywlk')
    call object_needed ('u_ee_bywlk')
    call object_needed ('u_een_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  
  include 'chksiz_u_ee_bywlk.inc'
  include 'chksiz_u_en_bywlk.inc'
  include 'chksiz_u_een_bywlk.inc'

! end requirements

! dimensions:
     electrons_nb      = size (u_een_bywlk, 1) 
     atoms_nb          = size (u_een_bywlk, 3) 
     configurations_nb = size (u_een_bywlk, 4)    
     walkers_nb        = size (u_een_bywlk, 5)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if
  
! allocations

      call alloc ('jastrow_exponent_bywlk', &
                        jastrow_exponent_bywlk,  &
                        configurations_nb, &
                        walkers_nb)
! end allocations

! Improve : simplify formulas

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      j_ee = zero
      j_en = zero
      j_een = zero

! Jc_ee
      do elec_j = 1, electrons_nb
        do elec_i = elec_j + 1, electrons_nb
          j_ee = j_ee + u_ee_bywlk (elec_i, elec_j, conf_i, walk_i)
        end do
      end do

! Jc_en
      do atom_i = 1, atoms_nb
        do elec_i = 1, electrons_nb
          j_en = j_en - u_en_bywlk (elec_i, atom_i, conf_i, walk_i)
         end do
      end do

! Jc_een
      do atom_i = 1, atoms_nb
        do elec_j = 1, electrons_nb
          do elec_i = elec_j + 1, electrons_nb
            j_een = j_een + u_een_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i)
          end do
        end do 
      end do 

     if(debug) then
       write (logf,*)trim(l_here),'-d: '
       write (logf,*)trim(l_here),'-d: j_ (conf, walk)'
       write (logf,*)trim(l_here),'-d: j_ee (',conf_i,',', walk_i,') =', j_ee
       write (logf,*)trim(l_here),'-d: j_en (',conf_i,',', walk_i,') =', j_en
       write (logf,*)trim(l_here),'-d: j_een (',conf_i,',', walk_i,') =', j_een
     end if

     jastrow_exponent_bywlk (conf_i, walk_i) = j_ee + j_en + j_een

    end do 
  end do 

  include 'debug_jastrow_exponent_bywlk.inc'
!   include 'ensnb_jastrow_exponent_bywlk.inc'

  call exit (l_here)

  end subroutine jastrow_exponent_bywlk_bld

! ===============================================================================
  subroutine jastrow_exponent_grad_bywlk_bld 
! -------------------------------------------------------------------------------
!
! Name           : jastrow_exponent_grad_bywlk_bld
!
! Creates        : jastrow_exponent_grad_bywlk (elec, conf, dim, walk)
!
! Needs          : u_ee_grad_bywlk (elec, elec, dim, conf, walk)
! Needs          : u_en_grad_bywlk (elec, atom, dim, conf, walk)
! Needs          : u_een_grad_bywlk (elec, elec, atom, dim, conf, walk)
!
! Descriptions   : grad_i exp = Sum_c grad_i u_ee - grad_i _u_en + grad_i u_een
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)    :: elec_i, elec_j, walk_i, conf_i, atom_i, dim_i
  integer(dpi)    :: atoms_nb
  integer(dpi)    :: configurations_nb  
  integer(dpi)    :: dimensions_nb
  integer(dpi)    :: electrons_nb
  integer(dpi)    :: walkers_nb

! header:

  if (header_exe) then

    l_here = "jastrow_exponent_grad_bywlk_bld"

    call object_create ('jastrow_exponent_grad_bywlk')

    call object_needed ('u_ee_grad_bywlk')
    call object_needed ('u_en_grad_bywlk')
    call object_needed ('u_een_grad_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  
  include 'chksiz_u_ee_grad_bywlk.inc'
  include 'chksiz_u_en_grad_bywlk.inc'
  include 'chksiz_u_een_grad_bywlk.inc'

! end requirements

! dimensions:
  electrons_nb      = size (u_en_grad_bywlk, 1)
  atoms_nb          = size (u_en_grad_bywlk, 2)
  dimensions_nb     = size (u_en_grad_bywlk, 3)
  configurations_nb = size (u_en_grad_bywlk, 4)
  walkers_nb        = size (u_en_grad_bywlk, 5)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations:

  if(.not.associated(jastrow_exponent_grad_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': jastrow_exponent_grad_bywlk not yet associated'
      call alloc ('jastrow_exponent_grad_bywlk', &
                        jastrow_exponent_grad_bywlk,  &
                        electrons_nb, &
                        configurations_nb, &
                        dimensions_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': jastrow_exponent_grad_bywlk left in memory'
  end if

! end allocations:

! Improve :

  jastrow_exponent_grad_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_nb

     do elec_j = 1, electrons_nb
       do elec_i = 1, electrons_nb

! dJ_ee/di
! derivative on elec_j
! sum over elec_i
! + u_ee_grad_bywlk (elec_i, elec_j) = u_ee_grad_bywlk (elec_j, elec_i)

          jastrow_exponent_grad_bywlk (elec_j, conf_i, :, walk_i)       &
            = jastrow_exponent_grad_bywlk (elec_j, conf_i, :, walk_i)   &
            - u_ee_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i) 

        end do
      end do

      do atom_i = 1, atoms_nb
        do elec_i = 1, electrons_nb

! dJ_en/di
! derivative on elec_i
! sum over atom_i 

          jastrow_exponent_grad_bywlk (elec_i, conf_i, :, walk_i) &
            = jastrow_exponent_grad_bywlk (elec_i, conf_i, :, walk_i) &
            - u_en_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)

        end do
      end do

      do atom_i = 1, atoms_nb
        do elec_j = 1, electrons_nb
          do elec_i = 1, electrons_nb

            if(elec_i /= elec_j) then
! dJ_een/dj
! derivative on elec_i
! sum over elec_j 
! sum over atom_i 

           jastrow_exponent_grad_bywlk (elec_i, conf_i, :, walk_i)    &
             = jastrow_exponent_grad_bywlk (elec_i, conf_i, :, walk_i) &
						 + u_een_grad_bywlk (elec_i, elec_j, atom_i, :, conf_i, walk_i)

            end if

          end do  ! elec_i
        end do  ! elec_j
      end do ! atom_i

    end do ! conf     
  end do ! walk

  include 'debug_jastrow_exponent_grad_bywlk.inc'
!   include 'ensnb_jastrow_exponent_grad_bywlk.inc'

  call exit (l_here)

  end subroutine jastrow_exponent_grad_bywlk_bld

! ===============================================================================
  subroutine jastrow_exponent_lapl_bywlk_bld 
! -------------------------------------------------------------------------------
!
! Name           : jastrow_exponent_lapl_bywlk_bld
!
! Creates        : jastrow_exponent_lapl_bywlk (elec, conf, walk)
!
! Needs          : u_ee_lapl_bywlk (elec, elec,  conf, walk)
! Needs          : u_en_lapl_bywlk (elec, atom,  conf, walk)
! Needs          : u_een_lapl_bywlk (elec, elec, atom, conf, walk)
!
! Descriptions   : lapl_i exp = Sum_c lapl_i u_ee - lapl_i _u_en + lapl_i u_een
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)    :: elec_i, elec_j, walk_i, conf_i, atom_i
  integer(dpi)    :: atoms_nb
  integer(dpi)    :: configurations_nb  
  integer(dpi)    :: electrons_nb
  integer(dpi)    :: walkers_nb

! header:

  if (header_exe) then

    l_here = "jastrow_exponent_lapl_bywlk_bld"

    call object_create ('jastrow_exponent_lapl_bywlk')

    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('u_en_lapl_bywlk')
    call object_needed ('u_een_lapl_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_u_ee_lapl_bywlk.inc'
  include 'chksiz_u_en_lapl_bywlk.inc'
  include 'chksiz_u_een_lapl_bywlk.inc'

! end requirements

! dimensions:

  electrons_nb      = size (u_een_lapl_bywlk, 1)
  electrons_nb      = size (u_een_lapl_bywlk, 2)
  atoms_nb          = size (u_een_lapl_bywlk, 3)
  configurations_nb = size (u_een_lapl_bywlk, 4)
  walkers_nb        = size (u_een_lapl_bywlk, 5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
   write (logf,*)trim(l_here),'-d: atoms_nb =',atoms_nb
   write (logf,*)trim(l_here),'-d: configurations_nb =',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if 

! end dimensions
! allocations:

  if(.not.associated(jastrow_exponent_lapl_bywlk)) then
    if(trace_depend) &
      write (logf,*)trim(l_here),': jastrow_exponent_lapl_bywlk not yet associated'
      call alloc ('jastrow_exponent_lapl_bywlk', &
                        jastrow_exponent_lapl_bywlk,  &
                        electrons_nb, &
                        configurations_nb, &
                        walkers_nb)
  else 
    if(trace_depend) &
     write (logf,*)trim(l_here),': jastrow_exponent_lapl_bywlk left in memory'
  end if

! end allocations

! Improve : simplify AND VERIFY formulas

  jastrow_exponent_lapl_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_nb

     do elec_j = 1, electrons_nb
       do elec_i = 1, electrons_nb

! dJ_ee/di
! derivative on elec_j
! sum over elec_i
! + u_ee_lapl_bywlk (elec_i, elec_j) = u_ee_lapl_bywlk (elec_j, elec_i)

          jastrow_exponent_lapl_bywlk (elec_j, conf_i, walk_i)       &
            = jastrow_exponent_lapl_bywlk (elec_j, conf_i, walk_i)   &
            + u_ee_lapl_bywlk (elec_i, elec_j, conf_i, walk_i) 

        end do
      end do

      do atom_i = 1, atoms_nb
        do elec_i = 1, electrons_nb

! dJ_en/di
! derivative on elec_i
! sum over atom_i 

          jastrow_exponent_lapl_bywlk (elec_i, conf_i, walk_i) &
            = jastrow_exponent_lapl_bywlk (elec_i, conf_i, walk_i) &
            - u_en_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)

        end do
      end do

      do atom_i = 1, atoms_nb
        do elec_j = 1, electrons_nb
          do elec_i = 1, electrons_nb

            if(elec_i /= elec_j) then
! dJ_een/dj
! derivative on elec_i
! sum over elec_j 
! sum over atom_i 

           jastrow_exponent_lapl_bywlk (elec_i, conf_i, walk_i)    &
             = jastrow_exponent_lapl_bywlk (elec_i, conf_i, walk_i) &
						 + u_een_lapl_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i)

            end if

          end do  ! elec_i
        end do  ! elec_j
      end do ! atom_i

    end do ! conf     
  end do ! walk

  include 'debug_jastrow_exponent_lapl_bywlk.inc'
!   include 'ensnb_jastrow_exponent_lapl_bywlk.inc'
 
  call exit (l_here)

  end subroutine jastrow_exponent_lapl_bywlk_bld

end module jastrow_exponent_bywlk_mod
