! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_factor_guide_bywlk_mod

  use basic_types_tools_mod
  use jastrow_exponent_guide_bywlk_mod 

! Check
! Description : jastrow_factor_guide (c,w) = Exp (jastrow_exponent_guide(c,w))

! Creates     : jastrow_factor_guide_bywlk (conf, walk)
! Creates     : jastrow_factor_guide_grad_bywlk (elec, conf, dim, walk)
! Creates     : jastrow_factor_guide_lapl_bywlk (elec, conf, walk)
 
  real(dp), pointer :: jastrow_factor_guide_bywlk (:,:)           => NULL()
  real(dp), pointer :: jastrow_factor_guide_grad_bywlk (:,:,:,:)  => NULL()
  real(dp), pointer :: jastrow_factor_guide_lapl_bywlk (:,:,:)    => NULL()

! Needs       : jastrow_exponent_guide_bywlk (conf, walk) 
! Needs       : jastrow_exponent_guide_grad_bywlk (elec, conf, dim, walk) 
! Needs       : jastrow_exponent_guide_lapl_bywlk (elec, conf, walk) 

  contains

! ==============================================================================
  subroutine jastrow_factor_guide_bywlk_bld 
! ------------------------------------------------------------------------------
!
! Name        : jastrow_factor_guide_bywlk_bld
!
! Creates     : jastrow_factor_guide_bywlk (conf, walk)
!
! Needs       : jastrow_exponent_guide_bywlk (conf, walk) 
!
! Description : jastrow_factor_guide (c,w) = Exp (jastrow_exponent_guide(c,w))* Weight
!
! Authors     : F. Colonna
! Date        : 02 Feb 2002
! ------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: walk_i, conf_i

! header:

  if (header_exe) then

    l_here = "jastrow_factor_guide_bywlk_bld"

    call object_create ('jastrow_factor_guide_bywlk')

    call object_needed ('jastrow_exponent_guide_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  call check_asso_conf_guide ('configurations_guide', configurations_guide, &
       'configurations_guide_init', configurations_guide_init)

  include 'chksiz_configurations_guide.inc'
  include 'chksiz_jastrow_exponent_guide_bywlk.inc'

  configurations_guide_nb = size (jastrow_exponent_guide_bywlk, 1)
  walkers_nb        = size (jastrow_exponent_guide_bywlk, 2)

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if

! end requirements:

! allocations
     call alloc ('jastrow_factor_guide_bywlk', jastrow_factor_guide_bywlk, &
                      configurations_guide_nb, walkers_nb)
! end allocations

  jastrow_factor_guide_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb

      if(jastrow_exponent_guide_bywlk (conf_i, walk_i) > EXP_ARG_SUP ) then
        jastrow_factor_guide_bywlk (conf_i, walk_i) = HUGE(1.d0)
      else if(jastrow_exponent_guide_bywlk (conf_i, walk_i) < EXP_ARG_INF ) then
        jastrow_factor_guide_bywlk (conf_i, walk_i) = zero
      else
        jastrow_factor_guide_bywlk (conf_i, walk_i)     &
         = exp ( jastrow_exponent_guide_bywlk (conf_i, walk_i) )
      end if

    end do 
  end do 

  include 'debug_jastrow_factor_guide_bywlk.inc'    
!   include 'ensnb_jastrow_factor_guide_bywlk.inc'    
 
  call exit (l_here)

  end subroutine jastrow_factor_guide_bywlk_bld 

! ==============================================================================
  subroutine jastrow_factor_guide_grad_bywlk_bld 
! ------------------------------------------------------------------------------
!
! Name           : jastrow_factor_guide_grad_bywlk_bld
!
! Creates        : jastrow_factor_guide_grad_bywlk (elec, conf, dim, walk)
!
! Needs          : jastrow_factor_guide_bywlk (conf, walk) 
! Needs          : jastrow_exponent_guide_grad_bywlk (elec, conf, dim, walk)
!
! Description    : jastrow_factor_guide_grad (e, c,:,w) 
! Description    : = jastrow_factor_guide (c,w) * jastrow_exponent_guide_grad (e, c, :, w)
!
! Authors        : F. Colonna
! Date           : 02 Feb 2002
! ------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb 
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: elec_i, walk_i, conf_i, dim_i

! header:

  if (header_exe) then

    l_here = "jastrow_factor_guide_grad_bywlk_bld"

    call object_create ('jastrow_factor_guide_grad_bywlk')

    call object_needed ('jastrow_factor_guide_bywlk')
    call object_needed ('jastrow_exponent_guide_grad_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  call check_asso_conf_guide ('configurations_guide', configurations_guide, &
       'configurations_guide_init', configurations_guide_init)

  include 'chksiz_configurations_guide.inc'
  include 'chksiz_jastrow_factor_guide_bywlk.inc'
  include 'chksiz_jastrow_exponent_guide_grad_bywlk.inc'

! end requirements

! dimensions:

  configurations_guide_nb = size (jastrow_factor_guide_bywlk, 1)
  walkers_nb        = size (jastrow_factor_guide_bywlk, 2)
  electrons_nb      = size (jastrow_exponent_guide_grad_bywlk, 1)
  dimensions_nb     = size (jastrow_exponent_guide_grad_bywlk, 3)

! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb 
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

     call alloc ('jastrow_factor_guide_grad_bywlk', jastrow_factor_guide_grad_bywlk, &
                     electrons_nb, configurations_guide_nb, dimensions_nb, walkers_nb)

! end allocations

  jastrow_factor_guide_grad_bywlk = NOT_INIT_DP

  do walk_i = 1, walkers_nb
    do dim_i = 1, dimensions_nb
      do conf_i = 1, configurations_guide_nb
        do elec_i = 1, electrons_nb

          jastrow_factor_guide_grad_bywlk (elec_i, conf_i, dim_i, walk_i)     &
           = jastrow_factor_guide_bywlk (conf_i, walk_i)                      &
           * jastrow_exponent_guide_grad_bywlk (elec_i, conf_i, dim_i, walk_i) 

       	end do 
      end do 
    end do 
  end do 

  include 'debug_jastrow_factor_guide_grad_bywlk.inc'    
!   include 'ensnb_jastrow_factor_guide_grad_bywlk.inc'    
 
  call exit (l_here)

  end subroutine jastrow_factor_guide_grad_bywlk_bld 

! ==============================================================================
  subroutine jastrow_factor_guide_lapl_bywlk_bld 
! ------------------------------------------------------------------------------
!
! Name           : jastrow_factor_guide_lapl_bywlk_bld
!
! Creates        : jastrow_factor_guide_lapl_bywlk (elec, conf, walk)
!
! Needs          : jastrow_exponent_guide_bywlk (conf, walk) 
! Needs          : jastrow_exponent_guide_grad_bywlk (elec, conf, dim, walk) 
! Needs          : jastrow_exponent_guide_lapl_bywlk (elec, conf, walk)
!
! Description    : jastrow_factor_guide_lapl (e, c,w) = jastrow_factor_guide (c,w) 
! Description    : * ( jastrow_exponent_guide_lapl + jastrow_exponent_guide_grad^2)
!
! Authors        : F. Colonna
! Date           : 02 Feb 2002
! ------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb         
  integer(dpi)                  :: electrons_nb          
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: walk_i, conf_i, elec_i, dim_i
  real(dp)                      :: grad_2

! header:

  if (header_exe) then

    l_here = "jastrow_factor_guide_lapl_bywlk_bld"

    call object_create ('jastrow_factor_guide_lapl_bywlk')

    call object_needed ('jastrow_exponent_guide_bywlk')
    call object_needed ('jastrow_exponent_guide_grad_bywlk')
    call object_needed ('jastrow_exponent_guide_lapl_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_jastrow_exponent_guide_grad_bywlk.inc'
  include 'chksiz_jastrow_exponent_guide_lapl_bywlk.inc'
  include 'chksiz_jastrow_factor_guide_bywlk.inc'
! end requirements

! dimensions:

  configurations_guide_nb = size (jastrow_exponent_guide_bywlk, 1)
  walkers_nb        = size (jastrow_exponent_guide_bywlk, 2)
  electrons_nb      = size (jastrow_exponent_guide_grad_bywlk, 1)
  dimensions_nb     = size (jastrow_exponent_guide_grad_bywlk, 3)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if

! allocations:

       call alloc ('jastrow_factor_guide_lapl_bywlk', &
                        jastrow_factor_guide_lapl_bywlk,  &
                        electrons_nb, &
                        configurations_guide_nb, &
                        walkers_nb)
 ! end allocations

  jastrow_factor_guide_lapl_bywlk = NOT_INIT_DP

! Check
  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do elec_i = 1, electrons_nb

        grad_2 = zero
        do dim_i = 1, dimensions_nb
          grad_2 = grad_2 &
          + jastrow_exponent_guide_grad_bywlk (elec_i, conf_i, dim_i, walk_i)  &
          * jastrow_exponent_guide_grad_bywlk (elec_i, conf_i, dim_i, walk_i) 
       end do 

! jastrow_factor_guide_lapl (e, c, w) = jastrow_factor_guide (c,w) 
!                        * (jastrow_exponent_guide_lapl + jastrow_exponent_guide_grad^2)

       jastrow_factor_guide_lapl_bywlk (elec_i, conf_i, walk_i)                &
         = jastrow_factor_guide_bywlk (conf_i, walk_i)                         &
         * (grad_2 + jastrow_exponent_guide_lapl_bywlk (elec_i, conf_i, walk_i))

      end do 
    end do 
  end do 

  include 'debug_jastrow_factor_guide_lapl_bywlk.inc'    
!   include 'ensnb_jastrow_factor_guide_lapl_bywlk.inc'    
 
  call exit (l_here)

  end subroutine jastrow_factor_guide_lapl_bywlk_bld 

end module jastrow_factor_guide_bywlk_mod
