!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_menu_mod

  use jastrow_mod
  use jastrow_exponent_bywlk_mod

!                  jastrow_of_conf_c
!  sum_over_confs    |        sum_over_dets  dets_alpha_and_beta
!       |            |            |                  |
!       | weight_of_c|            | weight_of_d      |
!       |     |      |            |   |      .--------------.
!       v     v      v            v   v      v              v
! Psi = Sum_c W(c) * Jastrow(c) * S_d w(d) * det_alpha(d,c)*det_beta(d,c)
 
  contains
 
! ==================================================================================
  subroutine jastrow_menu
! ----------------------------------------------------------------------------------
!
! Name           : jastrow_menu
!
! Description    : input jastrow
! Description    : The correlation function is specified within each configuration
! Description    : by jastrow = none:default:read. If "read" is given, then the
! Description    : the suitable jastrow is read from file specified in the 
! Description    : jastrow .... end command of input file.
!
! Author:X. Krokidis, F. Colonna
! Date  :18 Oct 2000
! Revision: 20 Dec 2000 X. Krokidis (token "cor" is removed).
! Revision: 19 Feb 2002 by F. Colonna 
! ----------------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: done =.false.
  logical(bool)                              :: jastrow_read =.false.
    
! begin:
  l_here ='jastrow_menu'
  call enter(l_here)
    
! initialize:
    
    done = .false.
    call next_command_get ('jastrow')
    do while (.not.done)
     call next_word_get('jastrow')
     if (trim(cur_word) .eq. 'help') then 
      write (logf,*) 'jastrow: jastrow definitions in wavefunction menu'
      write (logf,*) 'jastrow: syntax:'
      write (logf,*) 'jastrow: jas{trow}'
      write (logf,*) 'jastrow:  help'
      write (logf,*) 'jastrow:  cor{relation} = [string] :read:none:default:'
      write (logf,*) 'jastrow:                  affected to all conf'
      write (logf,*) 'jastrow:  fil{e            [command]'
      write (logf,*) 'jastrow:  read          =  [boolean]'
      write (logf,*) 'jastrow: end'
      write (logf,*) 'jastrow: example:'
      write (logf,*) 'jastrow: jastrow'
      write (logf,*) 'jastrow:  file ... end'
      write (logf,*) 'jastrow: end_jas'
      
     else if(trim(cur_word) .eq. 'file') then
      call files_menu

     else if (token_is ('rea',jastrow_read)) then
      call conf_jastr_read
      
     else
      call menu_exit (l_here, done)
     end if
    end do
    
  call exit(l_here)
    
  end subroutine jastrow_menu

end module jastrow_menu_mod
