! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_u_ee_bywlk_mod

  use basic_types_tools_mod
  use ee_r_bar_bywlk_mod
  use jastrow_a_ee_1_4_mod

! Improve: formulas not verified

! Creates     : u_ee_bywlk                (elec, elec, conf, walk)
! Creates     : u_ee_grad_bywlk           (elec, elec, dim, conf, walk)
! Creates     : u_ee_lapl_bywlk           (elec, elec, conf, walk)
!
! Creates     : u_ee_a_der_bywlk        (elec, elec, jastr_pow_max, conf, walk)
! Creates     : u_ee_b_der_bywlk        (elec, elec, conf, walk)
! Creates     : u_ee_b_der2_bywlk       (elec, elec, conf, walk)
! Creates     : u_ee_ab_der2_bywlk      (elec, elec, jastr_pow_max, conf, walk)
!
! Creates     : u_ee_grad_a_der_bywlk   (elec, elec, dim, jastr_pow_max, conf, walk)
! Creates     : u_ee_grad_b_der_bywlk   (elec, elec, dim, conf, walk)
! Creates     : u_ee_grad_b_der2_bywlk  (elec, elec, dim, conf, walk)
! Creates     : u_ee_grad_ab_der2_bywlk (elec, elec, dim, jastr_pow_max, conf, walk)
!
! Creates     : u_ee_lapl_a_der_bywlk   (elec, elec, jastr_pow_max, conf, walk)
! Creates     : u_ee_lapl_b_der_bywlk   (elec, elec, conf, walk)
! Creates     : u_ee_lapl_b_der2_bywlk  (elec, elec, conf, walk)
! Creates     : u_ee_lapl_ab_der2_bywlk (elec, elec, jastr_pow_max, conf, walk)

  real(dp), pointer :: u_ee_bywlk                (:,:,:,:)       => NULL()

  real(dp), pointer :: u_ee_a_der_bywlk        (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_ee_b_der_bywlk        (:,:,:,:)       => NULL()
  real(dp), pointer :: u_ee_b_der2_bywlk       (:,:,:,:)       => NULL()
  real(dp), pointer :: u_ee_ab_der2_bywlk      (:,:,:,:,:)     => NULL()

  real(dp), pointer :: u_ee_grad_bywlk           (:,:,:,:,:)     => NULL()

  real(dp), pointer :: u_ee_grad_a_der_bywlk   (:,:,:,:,:,:)   => NULL()
  real(dp), pointer :: u_ee_grad_b_der_bywlk   (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_ee_grad_b_der2_bywlk  (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_ee_grad_ab_der2_bywlk (:,:,:,:,:,:)   => NULL()

  real(dp), pointer :: u_ee_lapl_bywlk           (:,:,:,:)       => NULL()

  real(dp), pointer :: u_ee_lapl_a_der_bywlk   (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_ee_lapl_b_der_bywlk   (:,:,:,:)       => NULL()
  real(dp), pointer :: u_ee_lapl_b_der2_bywlk  (:,:,:,:)       => NULL()
  real(dp), pointer :: u_ee_lapl_ab_der2_bywlk (:,:,:,:,:)     => NULL()

  contains

! ============================================================================
  subroutine u_ee_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_bywlk_bld
!
! Creates        : u_ee_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : u_ee = Sum_n a_ee (n) * r_bar^n  (deg= 1..4)
! Descriptions   : r_bar = ee_r_bar_bywlk (elec, atom, conf)
! Descriptions   : a_ee  = jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j 
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: i

  real(dp)                      :: r_ij_bar

! header:

  if (header_exe) then

    l_here = "u_ee_bywlk_bld"

    call object_create ('u_ee_bywlk')

    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('jastrow_a_ee_1_4')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_configurations.inc'
  include 'chksiz_ee_r_bar_bywlk.inc'
  include 'chksiz_jastrow_a_ee_1_4.inc'
! end requirements

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk, 1)
  configurations_nb = size (ee_r_bar_bywlk, 3)
  walkers_nb        = size (ee_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_bywlk', u_ee_bywlk,   &
                    electrons_nb, electrons_nb, configurations_nb, walkers_nb)

! end allocations

  u_ee_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              r_ij_bar = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i) 
              
              do i = 1, jastr_pow_max

                u_ee_bywlk (elec_i, elec_j, conf_i, walk_i)         &
                     =  u_ee_bywlk (elec_i, elec_j, conf_i, walk_i) & 
                     + r_ij_bar **i * jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, i)

             enddo ! do i 
   
          end do ! do elec_i

       end do ! do elec_j

    end do ! conf_i

 end do ! walk_i


  include 'debug_u_ee_bywlk.inc'
!   include 'ensnb_u_ee_bywlk.inc'

  call exit (l_here)

  end subroutine u_ee_bywlk_bld

! ============================================================================
  subroutine u_ee_a_der_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_a_der_bywlk_bld
!
! Creates        : u_ee_a_der_bywlk (elec, elec, jastr_pow_max, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : u_ee_a_deriv =  r_ij_bar^pow_i
! Descriptions   : r_ij_bar     =  ee_r_bar_bywlk (elec, elec, conf, walk)
! Descriptions   : pow_i        =  1, 2, 3,..,jastr_pow_max
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------

  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i 
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ij_bar

! header:

  if (header_exe) then

    l_here = "u_ee_a_der_bywlk_bld"

    call object_create ('u_ee_a_der_bywlk')

    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('jastrow_a_ee_1_4')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk, 1)
  configurations_nb = size (ee_r_bar_bywlk, 3)
  walkers_nb        = size (ee_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

  call alloc ('u_ee_a_der_bywlk', u_ee_a_der_bywlk, electrons_nb, electrons_nb, &
       jastr_pow_max, configurations_nb, walkers_nb)

! end allocations
  u_ee_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              r_ij_bar = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i) 
              
              do pow_i = 1, jastr_pow_max

                 u_ee_a_der_bywlk(elec_i, elec_j, pow_i, conf_i, walk_i) = &
                      r_ij_bar**pow_i

              enddo ! do pow_i 
              
           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_a_der_bywlk_bld

! ============================================================================
  subroutine u_ee_b_der_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_b_der_bywlk_bld
!
! Creates        : u_ee_b_der_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, pow)
!
! Descriptions   : u_ee_b_deriv        = Sum_{n=1} alpha * r_ij_bar_beta_deriv (n=1,2,jastr_pow_max)
! Descriptions   : alpha               = jastrow_a_ee_1_4 (elec, elec, conf, pow)
! Descriptions   : r_ij_bar_beta_deriv = ee_r_bar_beta_der_bywlk (elec, elec, pow, conf, walk)
! Descriptions   : pow                 = 1, 2, 3,...,jastr_pow_max 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  
  real(dp)                      :: alpha
  real(dp)                      :: r_ij_bar_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_ee_b_der_bywlk_bld"

    call object_create ('u_ee_b_der_bywlk')
    
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('jastrow_a_ee_1_4')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk,  1)
  configurations_nb = size (ee_r_bar_bywlk,  3)
  walkers_nb        = size (ee_r_bar_bywlk,  4)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_b_der_bywlk', u_ee_b_der_bywlk, electrons_nb, electrons_nb,  &
         configurations_nb, walkers_nb )

! end allocations

  u_ee_b_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max

                 alpha = jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, pow_i)

                 r_ij_bar_beta_deriv = ee_r_bar_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) 

                 u_ee_b_der_bywlk(elec_i, elec_j, conf_i, walk_i) =      &
                      u_ee_b_der_bywlk(elec_i, elec_j, conf_i, walk_i) + &
                      alpha * r_ij_bar_beta_deriv
                    
                 enddo ! do pow_i 
                 
           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_b_der_bywlk_bld

! ============================================================================
  subroutine u_ee_b_der2_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_b_der2_bywlk_bld
!
! Creates        : u_ee_b_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_beta_der2_bywlk (elec, elec, pow, conf, walk) 
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : u_ee_b_deriv2 = Sum_{n=1} alpha * r_ij_bar_beta_deriv2 (n=1,2,jastr_pow_max)
!
! Descriptions   : alpha                = jastrow_a_ee_1_4 (elec, elec, conf, pow)
! Descriptions   : r_ij_bar_beta_deriv2 = ee_r_bar_beta_der2_bywlk (elec, elec, pow, conf, walk) 
! Descriptions   : pow                  = 1, 2, 3,...,jastr_pow_max 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  
  real(dp)                      :: derivative
  real(dp)                      :: alpha
  real(dp)                      :: r_ij_bar_beta_deriv2

! header:

  if (header_exe) then

    l_here = "u_ee_b_der2_bywlk_bld"

    call object_create ('u_ee_b_der2_bywlk')
    
    call object_needed ('ee_r_bar_beta_der2_bywlk')
    call object_needed ('jastrow_a_ee_1_4')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk,  1)
  configurations_nb = size (ee_r_bar_bywlk,  3)
  walkers_nb        = size (ee_r_bar_bywlk,  4)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_b_der2_bywlk', u_ee_b_der2_bywlk, electrons_nb, electrons_nb,  &
         configurations_nb, walkers_nb )

! end allocations

  u_ee_b_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max

                 alpha = jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, pow_i)

                 r_ij_bar_beta_deriv2 = ee_r_bar_beta_der2_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) 

                 derivative = alpha * r_ij_bar_beta_deriv2

                 u_ee_b_der2_bywlk(elec_i, elec_j, conf_i, walk_i) =      &
                      u_ee_b_der2_bywlk(elec_i, elec_j, conf_i, walk_i) + &
                      derivative
                    
              end do ! do pow_i 
              
           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_b_der2_bywlk_bld

! ============================================================================
  subroutine u_ee_ab_der2_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_ab_der2_bywlk_bld
!
! Creates        : u_ee_ab_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : u_ee_ab_deriv2      = r_ij_bar_beta_deriv
! Descriptions   : r_ij_bar_beta_deriv = ee_r_bar_beta_der_bywlk (elec, elec, pow, conf, walk)
! Descriptions   : pow                 = 1, 2, 3,...,jastr_pow_max 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  
  real(dp)                      :: derivative
  real(dp)                      :: r_ij_bar_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_ee_ab_der2_bywlk_bld"

    call object_create ('u_ee_ab_der2_bywlk')
    
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('jastrow_a_ee_1_4')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk,  1)
  configurations_nb = size (ee_r_bar_bywlk,  3)
  walkers_nb        = size (ee_r_bar_bywlk,  4)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_ab_der2_bywlk', u_ee_ab_der2_bywlk, electrons_nb, electrons_nb,  &
         jastr_pow_max, configurations_nb, walkers_nb )

! end allocations

  u_ee_ab_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max

                 r_ij_bar_beta_deriv = ee_r_bar_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) 

                 u_ee_ab_der2_bywlk(elec_i, elec_j, pow_i, conf_i, walk_i) = r_ij_bar_beta_deriv
                    
                 enddo ! do pow_i 
                 
           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_ab_der2_bywlk_bld

! ============================================================================
  subroutine u_ee_grad_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_grad_bywlk_bld
!
! Creates        : u_ee_grad_bywlk (elec, elec, dim, conf, walk)
!
! Descriptions   : grad u_ee = Sum_n n * a_ee (n) * r_bar^(n-1) grad r_bar
! Descriptions   : (n= 1..4)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : ee_r_bar_grad_bywlk (elec, elec, dim, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: i

  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_grad(3)
  real(dp)                      :: term

! header:

  if (header_exe) then

    l_here = "u_ee_grad_bywlk_bld"

    call object_create ('u_ee_grad_bywlk')

    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('jastrow_a_ee_1_4')

    return

  end if

! begin:
  call enter (l_here)

! Improve check electrons_nb twice

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk,      1)
  dimensions_nb     = size (ee_r_bar_grad_bywlk, 3)
  configurations_nb = size (ee_r_bar_bywlk,      3)
  walkers_nb        = size (ee_r_bar_bywlk,      4)
! end dimensions

! allocation 
 
  call  alloc ('u_ee_grad_bywlk', u_ee_grad_bywlk, electrons_nb, electrons_nb, dimensions_nb,  configurations_nb, walkers_nb)
 
! end allocation 

  u_ee_grad_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              r_ij_bar      &
                   = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)

              r_ij_bar_grad &
                   = ee_r_bar_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

              term = 0.d0

              do i = 1, jastr_pow_max

                 term = term + &
                      r_ij_bar ** (i-1) * i * jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, i)

              enddo ! do i

              u_ee_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)     = term *  r_ij_bar_grad (:)

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_grad_bywlk_bld

! ============================================================================
  subroutine u_ee_grad_a_der_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_grad_a_der_bywlk_bld
!
! Creates        : u_ee_grad_a_der_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : ee_r_bar_grad_bywlk (elec, elec, dim, conf, walk)
!
! Descriptions   : u_ee_grad_a_deriv = r_ij_bar^(pow - 1) * pow * r_ij_bar_grad
! Descriptions   : r_ij_bar          = ee_r_bar_bywlk (elec, elec, conf, walk)
! Descriptions   : r_ij_bar_grad     = ee_r_bar_grad_bywlk (elec, elec, dim, conf, walk)
! Descriptions   : pow               = 1, 2, 3,...,jastr_pow_max 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  
  real(dp)                      :: gradient_derivative
  real(dp)                      :: r_ij_bar_grad
  real(dp)                      :: r_ij_bar

! header:

  if (header_exe) then

    l_here = "u_ee_grad_a_der_bywlk_bld"

    call object_create ('u_ee_grad_a_der_bywlk')
    
    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('ee_r_bar_bywlk')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk,      1)
  dimensions_nb     = size (ee_r_bar_grad_bywlk, 3)
  configurations_nb = size (ee_r_bar_bywlk,      3)
  walkers_nb        = size (ee_r_bar_bywlk,      4)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_grad_a_der_bywlk', u_ee_grad_a_der_bywlk, electrons_nb, electrons_nb,  &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb )

! end allocations

  u_ee_grad_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i =1, dimensions_nb
                 
                 r_ij_bar      = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)

                 r_ij_bar_grad = ee_r_bar_grad_bywlk (elec_i, elec_j, dim_i, conf_i, walk_i)

                 do pow_i = 1, jastr_pow_max

                    gradient_derivative = r_ij_bar ** (pow_i - 1) * pow_i * r_ij_bar_grad

                    u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,pow_i,conf_i,walk_i) = &
                         gradient_derivative
                       
                 enddo ! do pow_i

              enddo ! do dim_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_grad_a_der_bywlk_bld

! ============================================================================
  subroutine u_ee_grad_b_der_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_grad_b_der_bywlk_bld
!
! Creates        : u_ee_grad_b_der_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, pow)
!
! Descriptions   : u_ee_grad_b_deriv        = Sum_{n=1} alpha * r_ij_bar_grad_beta_deriv (n=1,2,3,...)
! Descriptions   : alpha                    = jastrow_a_ee_1_4 (elec, elec, conf, pow)
! Descriptions   : r_ij_bar_grad_beta_deriv = ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow, conf, walk) 
! Descriptions   : pow                      = 1, 2, 3,...,jastr_pow_max 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  
  real(dp)                      :: derivative
  real(dp)                      :: alpha
  real(dp)                      :: r_ij_bar_grad_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_ee_grad_b_der_bywlk_bld"

    call object_create ('u_ee_grad_b_der_bywlk')
    
    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('jastrow_a_ee_1_4')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_grad_b_der_bywlk', u_ee_grad_b_der_bywlk, electrons_nb, electrons_nb,  &
         dimensions_nb, configurations_nb, walkers_nb )

! end allocations

  u_ee_grad_b_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i =1, dimensions_nb

                 do pow_i = 1, jastr_pow_max

                    alpha = jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, pow_i)
                    
                    r_ij_bar_grad_beta_deriv = &
                         ee_r_bar_grad_beta_der_bywlk (elec_i, elec_j, dim_i, pow_i, conf_i, walk_i) 
                    
                    derivative = alpha * r_ij_bar_grad_beta_deriv
                    
                    u_ee_grad_b_der_bywlk(elec_i, elec_j, dim_i, conf_i, walk_i) =      &
                         u_ee_grad_b_der_bywlk(elec_i, elec_j, dim_i, conf_i, walk_i) + &
                         derivative
                    
                 enddo ! do pow_i 
                 
              enddo ! do dim_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_grad_b_der_bywlk_bld

! ============================================================================
  subroutine u_ee_grad_ab_der2_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_grad_ab_der2_bywlk_bld
!
! Creates        : u_ee_grad_ab_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow, conf, walk) 
!
! Descriptions   :  u_ee_grad_ab_der2_bywlk = r_ij_bar_grad_beta_deriv
! Descriptions   :  r_ij_bar_grad_beta_deriv  = ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow, conf, walk) 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  
  real(dp)                      :: r_ij_bar_grad_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_ee_grad_ab_der2_bywlk_bld"

    call object_create ('u_ee_grad_ab_der2_bywlk')
    
    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_grad_ab_der2_bywlk', u_ee_grad_ab_der2_bywlk, electrons_nb, electrons_nb,  &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb )

! end allocations

  u_ee_grad_ab_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i =1, dimensions_nb

                 do pow_i = 1, jastr_pow_max

                    r_ij_bar_grad_beta_deriv = &
                         ee_r_bar_grad_beta_der_bywlk (elec_i, elec_j, dim_i, pow_i, conf_i, walk_i) 
                    
                    u_ee_grad_ab_der2_bywlk(elec_i, elec_j, dim_i, pow_i, conf_i, walk_i) =      &
                         r_ij_bar_grad_beta_deriv
                    
                 enddo ! do pow_i 
                 
              enddo ! do dim_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_grad_ab_der2_bywlk_bld

! ============================================================================
  subroutine u_ee_grad_b_der2_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_ee_grad_b_der2_bywlk_bld
!
! Creates        : u_ee_grad_b_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : 
! Descriptions   : 
! Descriptions   : 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  
  real(dp)                      :: r_ij_bar_grad_beta_deriv2

! header:

  if (header_exe) then

    l_here = "u_ee_grad_b_der2_bywlk_bld"

    call object_create ('u_ee_grad_b_der2_bywlk')
    
    call object_needed ('ee_r_bar_grad_beta_der2_bywlk')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_grad_b_der2_bywlk', u_ee_grad_b_der2_bywlk, electrons_nb, electrons_nb,  &
         dimensions_nb, configurations_nb, walkers_nb )

! end allocations

  u_ee_grad_b_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i =1, dimensions_nb

                 do pow_i = 1, jastr_pow_max

                    r_ij_bar_grad_beta_deriv2 = &
                         ee_r_bar_grad_beta_der2_bywlk (elec_i, elec_j, dim_i, pow_i, conf_i, walk_i) 
                    
                    u_ee_grad_b_der2_bywlk(elec_i, elec_j, dim_i, conf_i, walk_i) =      &
                         u_ee_grad_b_der2_bywlk(elec_i, elec_j, dim_i, conf_i, walk_i) + &
                         r_ij_bar_grad_beta_deriv2
                    
                 enddo ! do pow_i 
                 
              enddo ! do dim_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_grad_b_der2_bywlk_bld

! ============================================================================
  subroutine u_ee_lapl_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_lapl_bywlk_bld
!
! Creates        : u_ee_lapl_bywlk (elec, elec, conf, walk)
!
! Descriptions   : lapl u_ee = Sum_n n * (n-1) *a_ee (n) * r_bar^(n-2)
! Descriptions   :                   ( grad r_bar . grad r_bar ) (n= 2..4)
! Descriptions   :  + Sum_n n * a_ee (n) * r_bar^(n-1) * lapl r_bar (n= 1..4)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : ee_r_bar_lapl_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_grad(3)
  real(dp)                      :: r_ij_bar_lapl
  real(dp)                      :: lapl
  real(dp)                      :: first_deriv
  real(dp)                      :: secon_deriv
  real(dp)                      :: dot_prod
  
! header:

  if (header_exe) then

    l_here = "u_ee_lapl_bywlk_bld"

    call object_create ('u_ee_lapl_bywlk')

    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('ee_r_bar_lapl_bywlk')
    call object_needed ('jastrow_a_ee_1_4')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_bywlk, 1)
  dimensions_nb     = size (ee_r_bar_grad_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_bywlk, 4)
  walkers_nb        = size (ee_r_bar_grad_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     =',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_lapl_bywlk', u_ee_lapl_bywlk, &
         electrons_nb, electrons_nb, configurations_nb, walkers_nb)

! end allocations

  u_ee_lapl_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

! Improve Symmetrize
           do elec_i = 1, electrons_nb

              r_ij_bar      &
                   = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)

              r_ij_bar_grad (:) &
                   = ee_r_bar_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

              r_ij_bar_lapl &
                   = ee_r_bar_lapl_bywlk (elec_i, elec_j, conf_i, walk_i)

                          first_deriv = 0.d0

                          do i = 1, jastr_pow_max
                             first_deriv = first_deriv + &
                                  r_ij_bar**(i-1)*i*jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, i) 
                          enddo ! do i


                          secon_deriv = 0.d0

                          do i = 2, jastr_pow_max
                             secon_deriv = secon_deriv + &
                                  r_ij_bar**(i-2)*(i-1)*i*jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, i) 
                          enddo ! do i

! Improve dot_prod
                          dot_prod = zero

                          do dim_i = 1, dimensions_nb
                             dot_prod = dot_prod + r_ij_bar_grad (dim_i) * r_ij_bar_grad (dim_i)
                          end do ! do dim_i

                          lapl = secon_deriv * dot_prod + first_deriv * r_ij_bar_lapl

                          u_ee_lapl_bywlk (elec_i, elec_j, conf_i, walk_i) = lapl
                          
                       end do ! do elec_i

                    end do ! do elec_j

                 end do ! do conf_i

              end do ! do walk_i

  call exit (l_here)

  end subroutine u_ee_lapl_bywlk_bld

! ============================================================================
  subroutine u_ee_lapl_a_der_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_lapl_a_der_bywlk_bld
!
! Creates        : u_ee_lapl_a_der_bywlk (elec, elec, conf, walk)
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ij_bar
  real(dp)                      :: inv_r
  real(dp)                      :: inv_r_4
  real(dp)                      :: laplacian_derivative

! header:

  if (header_exe) then

    l_here = "u_ee_lapl_a_der_bywlk_bld"

    call object_create ('u_ee_lapl_a_der_bywlk')

    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('ee_dist_inv_bywlk') 
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk, 1)
  configurations_nb = size (ee_r_bar_bywlk, 3)
  walkers_nb        = size (ee_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_ee_lapl_a_der_bywlk', u_ee_lapl_a_der_bywlk, &
         electrons_nb, electrons_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations

  u_ee_lapl_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              r_ij_bar = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)

              inv_r    = ee_dist_inv_bywlk(elec_i, elec_j, walk_i)

              inv_r_4  = inv_r**4
              
              do pow_i = 1, jastr_pow_max

                 laplacian_derivative = (pow_i + 1.d0) * pow_i * r_ij_bar**(pow_i + 2.d0) * inv_r_4

                 u_ee_lapl_a_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) = laplacian_derivative
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do elec_j

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

end subroutine u_ee_lapl_a_der_bywlk_bld

! ============================================================================
  subroutine u_ee_lapl_b_der_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_lapl_b_der_bywlk_bld
!
! Creates        : u_ee_lapl_b_der_bywlk (elec, elec, conf, walk)
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ij_bar_lapl_beta_deriv
  real(dp)                      :: alpha

! header:

  if (header_exe) then

    l_here = "u_ee_lapl_b_der_bywlk_bld"

    call object_create ('u_ee_lapl_b_der_bywlk')

    call object_needed ('jastrow_a_ee_1_4') 
    call object_needed ('ee_r_bar_lapl_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_lapl_beta_der_bywlk, 1)
  configurations_nb = size (ee_r_bar_lapl_beta_der_bywlk, 4)
  walkers_nb        = size (ee_r_bar_lapl_beta_der_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_ee_lapl_b_der_bywlk', u_ee_lapl_b_der_bywlk, &
         electrons_nb, electrons_nb, configurations_nb, walkers_nb)

! end allocations

  u_ee_lapl_b_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb
              
              do pow_i = 1, jastr_pow_max

                 alpha = jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, pow_i)

                 r_ij_bar_lapl_beta_deriv = & 
                      ee_r_bar_lapl_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i)

                 u_ee_lapl_b_der_bywlk (elec_i, elec_j, conf_i, walk_i) =      &
                      u_ee_lapl_b_der_bywlk (elec_i, elec_j, conf_i, walk_i) + &
                      alpha * r_ij_bar_lapl_beta_deriv
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do elec_j

     end do ! conf_i

  end do ! walk_i
  
  call exit (l_here)

end subroutine u_ee_lapl_b_der_bywlk_bld

! ============================================================================
  subroutine u_ee_lapl_ab_der2_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_lapl_ab_der2_bywlk_bld
!
! Creates        : u_ee_lapl_ab_der2_bywlk (elec, elec, conf, walk)
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ij_bar_lapl_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_ee_lapl_ab_der2_bywlk_bld"

    call object_create ('u_ee_lapl_ab_der2_bywlk')

    call object_needed ('jastrow_a_ee_1_4') 
    call object_needed ('ee_r_bar_lapl_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_lapl_beta_der_bywlk, 1)
  configurations_nb = size (ee_r_bar_lapl_beta_der_bywlk, 4)
  walkers_nb        = size (ee_r_bar_lapl_beta_der_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_ee_lapl_ab_der2_bywlk', u_ee_lapl_ab_der2_bywlk, &
         electrons_nb, electrons_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations

  u_ee_lapl_ab_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb
              
              do pow_i = 1, jastr_pow_max

                 r_ij_bar_lapl_beta_deriv = & 
                      ee_r_bar_lapl_beta_der_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i)

                 u_ee_lapl_ab_der2_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i) =      &
                      r_ij_bar_lapl_beta_deriv
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do elec_j

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

end subroutine u_ee_lapl_ab_der2_bywlk_bld

! ============================================================================
  subroutine u_ee_lapl_b_der2_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_lapl_b_der2_bywlk_bld
!
! Creates        : u_ee_lapl_b_der2_bywlk (elec, elec, conf, walk)
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ij_bar_lapl_beta_deriv2
  real(dp)                      :: alpha

! header:

  if (header_exe) then

    l_here = "u_ee_lapl_b_der2_bywlk_bld"

    call object_create ('u_ee_lapl_b_der2_bywlk')

    call object_needed ('jastrow_a_ee_1_4') 
    call object_needed ('ee_r_bar_lapl_beta_der2_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_lapl_beta_der2_bywlk, 1)
  configurations_nb = size (ee_r_bar_lapl_beta_der2_bywlk, 4)
  walkers_nb        = size (ee_r_bar_lapl_beta_der2_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_ee_lapl_b_der2_bywlk', u_ee_lapl_b_der2_bywlk, &
         electrons_nb, electrons_nb, configurations_nb, walkers_nb)

! end allocations

  u_ee_lapl_b_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb
              
              do pow_i = 1, jastr_pow_max

                 alpha = jastrow_a_ee_1_4 (elec_i, elec_j, conf_i, pow_i)

                 r_ij_bar_lapl_beta_deriv2 = & 
                      ee_r_bar_lapl_beta_der2_bywlk (elec_i, elec_j, pow_i, conf_i, walk_i)

                 u_ee_lapl_b_der2_bywlk (elec_i, elec_j, conf_i, walk_i) =      &
                      u_ee_lapl_b_der2_bywlk (elec_i, elec_j, conf_i, walk_i) + &
                      alpha * r_ij_bar_lapl_beta_deriv2
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do elec_j

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

end subroutine u_ee_lapl_b_der2_bywlk_bld

end module jastrow_u_ee_bywlk_mod
