! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_u_ee_guide_bywlk_mod

  use basic_types_tools_mod
  use ee_r_bar_guide_bywlk_mod
  use jastrow_a_ee_1_4_guide_mod

! Improve: formulas not verified

! Creates     : u_ee_guide_bywlk (elec, elec, conf, walk)
! Creates     : u_ee_guide_grad_bywlk (elec, elec, dim, conf, walk)
! Creates     : u_ee_guide_lapl_bywlk (elec, elec, conf, walk)

  real(dp), pointer :: u_ee_guide_bywlk (:,:,:,:)           => NULL()
  real(dp), pointer :: u_ee_guide_grad_bywlk (:,:,:,:,:)    => NULL()
  real(dp), pointer :: u_ee_guide_lapl_bywlk (:,:,:,:)      => NULL()

  contains

! ============================================================================
  subroutine u_ee_guide_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_guide_bywlk_bld
!
! Creates        : u_ee_guide_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4_guide (elec, elec, conf, deg)
!
! Descriptions   : u_ee = Sum_n a_ee (n) * r_bar^n  (deg= 1..4)
! Descriptions   : r_bar = ee_r_bar_guide_bywlk (elec, atom, conf)
! Descriptions   : a_ee  = jastrow_a_ee_1_4_guide (elec, elec, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ij_bar
  integer(dpi)                  :: i
! header:

  if (header_exe) then

    l_here = "u_ee_guide_bywlk_bld"

    call object_create ('u_ee_guide_bywlk')

    call object_needed ('ee_r_bar_guide_bywlk')
    call object_needed ('jastrow_a_ee_1_4_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_configurations_guide.inc'
  include 'chksiz_ee_r_bar_guide_bywlk.inc'
  include 'chksiz_jastrow_a_ee_1_4_guide.inc'
! end requirements

! dimensions:
  electrons_nb      = size (ee_r_bar_guide_bywlk, 1)
  configurations_guide_nb = size (ee_r_bar_guide_bywlk, 3)
  walkers_nb        = size (ee_r_bar_guide_bywlk, 4)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_ee_guide_bywlk', u_ee_guide_bywlk,   &
                    electrons_nb, electrons_nb, configurations_guide_nb, walkers_nb)

! end allocations
  u_ee_guide_bywlk = zero

  do walk_i = 1, walkers_nb
     do conf_i = 1, configurations_guide_nb
        do elec_j = 1, electrons_nb
           do elec_i = 1, electrons_nb

! r x (b1 + r * (b2 +  r x (b3 + r * b4 ) ) )

              r_ij_bar = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i) 
              
 !             u_ee_guide_bywlk (elec_i, elec_j, conf_i, walk_i)                       &
 !                  =    r_ij_bar*( jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 1)     &
 !                  + r_ij_bar*( jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 2)     &
 !                  + r_ij_bar*( jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 3)     &
 !                  + r_ij_bar*  jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 4) ) ) )

              do i = 1, jastr_pow_max_guide
                u_ee_guide_bywlk (elec_i, elec_j, conf_i, walk_i)         &
                     =  u_ee_guide_bywlk (elec_i, elec_j, conf_i, walk_i) & 
                     + r_ij_bar **i * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, i)
             enddo ! do i 
   
          end do
       end do
    end do
 end do


  include 'debug_u_ee_guide_bywlk.inc'
!   include 'ensnb_u_ee_guide_bywlk.inc'

  call exit (l_here)

  end subroutine u_ee_guide_bywlk_bld

! ============================================================================
  subroutine u_ee_guide_grad_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_guide_grad_bywlk_bld
!
! Creates        : u_ee_guide_grad_bywlk (elec, elec, dim, conf, walk)
!
! Descriptions   : grad u_ee = Sum_n n * a_ee (n) * r_bar^(n-1) grad r_bar
! Descriptions   : (n= 1..4)
!
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
! Needs          : ee_r_bar_guide_grad_bywlk (elec, elec, dim, conf, walk)
! Needs          : jastrow_a_ee_1_4_guide (elec, elec, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_grad(3)
  real(dp) :: term
  integer(dpi) :: i
! header:

  if (header_exe) then

    l_here = "u_ee_guide_grad_bywlk_bld"

    call object_create ('u_ee_guide_grad_bywlk')

    call object_needed ('ee_r_bar_guide_bywlk')
    call object_needed ('ee_r_bar_guide_grad_bywlk')
    call object_needed ('jastrow_a_ee_1_4_guide')

    return

  end if

! begin:
  call enter (l_here)

! Improve check electrons_nb twice

! requirements:
  include 'chksiz_ee_r_bar_guide_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_grad_bywlk.inc'
  include 'chksiz_jastrow_a_ee_1_4_guide.inc'
! end requirements

! dimensions:
  dimensions_nb = size (ee_r_bar_guide_grad_bywlk, 3)
  configurations_guide_nb = size (ee_r_bar_guide_bywlk, 3)
  electrons_nb = size (ee_r_bar_guide_bywlk, 1)
  walkers_nb = size (ee_r_bar_guide_bywlk, 4)
! end dimensions

! allocation 
 
  call  alloc ('u_ee_guide_grad_bywlk', u_ee_guide_grad_bywlk, electrons_nb, electrons_nb, dimensions_nb,  configurations_guide_nb, walkers_nb)
 
! end allocation 

  u_ee_guide_grad_bywlk = zero

  do walk_i = 1, walkers_nb
     do conf_i = 1, configurations_guide_nb
        do elec_j = 1, electrons_nb
           do elec_i = 1, electrons_nb

! r x (b1 + r * (b2 +  r x (b3 + r * b4 ) ) )

              r_ij_bar      &
                   = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i)

              r_ij_bar_grad &
                   = ee_r_bar_guide_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

!              u_ee_guide_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)                  &
!                   = (                jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 1) &
!                   + r_ij_bar*(2.d0 * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 2) &
!                   + r_ij_bar*(3.d0 * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 3) &
!                   + r_ij_bar* 4.d0 * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 4) &
!                   ) ) ) * r_ij_bar_grad

              term = 0.d0
              do i = 1, jastr_pow_max_guide
                 term = term + &
                      r_ij_bar ** (i-1) * i * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, i)
              enddo ! do i
              u_ee_guide_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)     = term *  r_ij_bar_grad (:)
           end do
        end do
     end do
  end do

  include 'debug_u_ee_guide_grad_bywlk.inc'
!   include 'ensnb_u_ee_guide_grad_bywlk.inc'

  call exit (l_here)

  end subroutine u_ee_guide_grad_bywlk_bld

! ============================================================================
  subroutine u_ee_guide_lapl_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_ee_guide_lapl_bywlk_bld
!
! Creates        : u_ee_guide_lapl_bywlk (elec, elec, conf, walk)
!
! Descriptions   : lapl u_ee = Sum_n n * (n-1) *a_ee (n) * r_bar^(n-2)
! Descriptions   :                   ( grad r_bar . grad r_bar ) (n= 2..4)
! Descriptions   :  + Sum_n n * a_ee (n) * r_bar^(n-1) * lapl r_bar (n= 1..4)
!
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
! Needs          : ee_r_bar_guide_lapl_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4_guide (elec, elec, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ij_bar, r_ij_bar_grad(3), r_ij_bar_lapl
  real(dp)                      :: lapl, first_deriv, secon_deriv, dot_prod
  integer(dpi) :: i
! header:

  if (header_exe) then

    l_here = "u_ee_guide_lapl_bywlk_bld"

    call object_create ('u_ee_guide_lapl_bywlk')

    call object_needed ('ee_r_bar_guide_bywlk')
    call object_needed ('ee_r_bar_guide_lapl_bywlk')
    call object_needed ('jastrow_a_ee_1_4_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	
  include 'chksiz_configurations_guide.inc'
  include 'chksiz_ee_r_bar_guide_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_grad_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_lapl_bywlk.inc'
! end requirements	

! dimensions:
  configurations_guide_nb = size (ee_r_bar_guide_grad_bywlk, 4)
  dimensions_nb     = size (ee_r_bar_guide_grad_bywlk, 3)
  electrons_nb      = size (ee_r_bar_guide_grad_bywlk, 1)
  walkers_nb        = size (ee_r_bar_guide_grad_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     =',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_ee_guide_lapl_bywlk', u_ee_guide_lapl_bywlk, &
         electrons_nb, electrons_nb, configurations_guide_nb, walkers_nb)

! end allocations

  u_ee_guide_lapl_bywlk = zero

  do walk_i = 1, walkers_nb
     do conf_i = 1, configurations_guide_nb
        do elec_j = 1, electrons_nb

! Improve Symmetrize
           do elec_i = 1, electrons_nb

              r_ij_bar      &
                   = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i)

              r_ij_bar_grad (:) &
                   = ee_r_bar_guide_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

              r_ij_bar_lapl &
                   = ee_r_bar_guide_lapl_bywlk (elec_i, elec_j, conf_i, walk_i)

!              first_deriv                                                          &
!                   =                  jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 1) &
!                   + r_ij_bar*(2.d0 * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 2) &
!                   + r_ij_bar*(3.d0 * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 3) &
!                   + r_ij_bar* 4.d0 * jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, 4) ) )

!              secon_deriv                                                         &
!                   =           2.d0 * jastrow_a_ee_1_4_guide(elec_i, elec_j, conf_i, 2) &
!                   + r_ij_bar*(6.d0 * jastrow_a_ee_1_4_guide(elec_i, elec_j, conf_i, 3) &
!                   + r_ij_bar*12.d0 * jastrow_a_ee_1_4_guide(elec_i, elec_j, conf_i, 4) ) 


                          first_deriv = 0.d0

                          do i = 1, jastr_pow_max_guide
                             first_deriv = first_deriv + &
                                  r_ij_bar**(i-1)*i*jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, i) 
                          enddo ! do i


                          secon_deriv = 0.d0
                          do i = 2, jastr_pow_max_guide
                             secon_deriv = secon_deriv + &
                                  r_ij_bar**(i-2)*(i-1)*i*jastrow_a_ee_1_4_guide (elec_i, elec_j, conf_i, i) 
                          enddo ! do i

! Improve dot_prod
                          dot_prod = zero
                          do dim_i = 1, dimensions_nb
                             dot_prod = dot_prod + r_ij_bar_grad (dim_i) * r_ij_bar_grad (dim_i)
                          end do

                          lapl = secon_deriv * dot_prod + first_deriv * r_ij_bar_lapl

                          u_ee_guide_lapl_bywlk (elec_i, elec_j, conf_i, walk_i) = lapl
                          
                       end do
                    end do
                 end do
              end do

  include 'debug_u_ee_guide_lapl_bywlk.inc'
!   include 'ensnb_u_ee_guide_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine u_ee_guide_lapl_bywlk_bld

end module jastrow_u_ee_guide_bywlk_mod
