! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_u_een_guide_bywlk_mod

  use basic_types_tools_mod
  use ee_r_bar_guide_bywlk_mod
  use ea_r_bar_guide_bywlk_mod

! Creates     : u_een_guide_bywlk (elec, elec, atom, conf, walk)
! Creates     : u_een_guide_grad_bywlk (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_guide_lapl_bywlk (elec, elec, atom, conf, walk)

  real(dp), pointer :: u_een_guide_bywlk (:,:,:,:,:)        => NULL()
  real(dp), pointer :: u_een_guide_grad_bywlk (:,:,:,:,:,:) => NULL()
  real(dp), pointer :: u_een_guide_lapl_bywlk (:,:,:,:,:)   => NULL()

  contains

! ===================================================================================
  subroutine u_een_guide_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_guide_bywlk_bld
!
! Creates        : u_een_guide_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
! Needs          : conf_jastr_a_een_guide (conf,lik, atom)    
!
! Descriptions   : u_een_guide_bywlk = a_een1*ria^2 rja^2 + a_een2*rij^2 (ria^2 + rja^2)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! Revision       : R. Assaraf 04 Apr 2002 conf_jastr_a_een_guide
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i, atom_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ia_bar, r_ia_bar_2
  real(dp)                      :: r_ja_bar, r_ja_bar_2
  real(dp)                      :: r_ij_bar, r_ij_bar_2
  real(dp)                      :: a_een_1   
  real(dp)                      :: a_een_2 
  real(dp)                      :: u_een

! header:

  if (header_exe) then

    l_here = "u_een_guide_bywlk_bld"

    call object_create ('u_een_guide_bywlk')

    call object_needed ('ea_r_bar_guide_bywlk')
    call object_needed ('ee_r_bar_guide_bywlk')
    call object_needed ('conf_jastr_a_een_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	
!  include 'chksiz_configurations%conf_jastr%a_een.inc'
  include 'chksiz_ea_r_bar_guide_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_bywlk.inc'
! end requirements	

! dimensions:
  electrons_nb      = size (ee_r_bar_guide_bywlk, 1)
  atoms_nb          = size (ea_r_bar_guide_bywlk, 2)
  configurations_guide_nb = size (ee_r_bar_guide_bywlk, 3)
  walkers_nb        = size (ee_r_bar_guide_bywlk, 4)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_guide_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een_guide (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een_guide(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een_guide (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een_guide (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een_guide(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een_guide (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_guide_bywlk', u_een_guide_bywlk, & 
         electrons_nb, electrons_nb, atoms_nb, configurations_guide_nb, walkers_nb)

! end allocations

  u_een_guide_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb

        a_een_1  = conf_jastr_a_een_guide (conf_i, 1, atom_i)
        a_een_2  = conf_jastr_a_een_guide (conf_i, 2, atom_i)

        do elec_j = 1, electrons_nb

          r_ja_bar = ea_r_bar_guide_bywlk (elec_j, atom_i, conf_i, walk_i) 
          r_ja_bar_2 = r_ja_bar * r_ja_bar

! Improve store squares in temporary arrays

          do elec_i = 1, electrons_nb

           r_ia_bar   = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)
           r_ia_bar_2 = r_ia_bar * r_ia_bar

           r_ij_bar   = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i)
           r_ij_bar_2 = r_ij_bar * r_ij_bar

! verify formula
           u_een      = a_een_1 * r_ia_bar_2 * r_ja_bar_2                 &
                      + a_een_2 * r_ij_bar_2 * ( r_ia_bar_2 + r_ja_bar_2)

           u_een_guide_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = u_een 

          end do
        end do
      end do
    end do
  end do

  include 'debug_u_een_guide_bywlk.inc'
!   include 'ensnb_u_een_guide_bywlk.inc'

  call exit (l_here)

  end subroutine u_een_guide_bywlk_bld

! ===================================================================================
  subroutine u_een_guide_grad_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_guide_grad_bywlk_bld
!
! Creates        : u_een_guide_grad_bywlk (elec, elec, atom, dim, conf, walk)
!
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
! Needs          : ea_r_bar_guide_grad_bywlk (elec, elec, dim, conf, walk)
! Needs          : ee_r_bar_guide_grad_bywlk (elec, elec, dim, conf, walk)
!
! Needs          : conf_jastr%a_een (conf,lik, atom)      
!
! Descriptions   : gardient over elec_i coordinates only
! Descriptions   : u_een_guide_grad_bywlk 
! Descriptions   :      = 2 * a_een1 *ria rja^2 * grad_i ria
! Descriptions   :      + 2 * a_een2 *rij (ria^2 + rja^2) * grad_i rij
! Descriptions   :      + 2 * a_een2 *rij^2 *ria * grad_i ria
! Descriptions   : u_een_guide_grad_bywlk 
! Descriptions   :      = 2 (( a_een1 * rja^2 + a_een2 * rij^2 ) * ria * grad_i ria
! Descriptions   :      + a_een2 * (ria^2 + rja^2) * rij * grad_i rij )
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i, atom_i, dim_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ia_bar, r_ia_bar_2
  real(dp)                      :: r_ja_bar, r_ja_bar_2
  real(dp)                      :: r_ij_bar, r_ij_bar_2
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp)                      :: r_ij_bar_grad(3)
  real(dp)                      :: a_een_1   
  real(dp)                      :: a_een_2 
  real(dp)                      :: d_u_een(3)

! header:

  if (header_exe) then

    l_here = "u_een_guide_grad_bywlk_bld"

    call object_create ('u_een_guide_grad_bywlk')

    call object_needed ('ea_r_bar_guide_bywlk')
    call object_needed ('ee_r_bar_guide_bywlk')
    call object_needed ('ea_r_bar_guide_grad_bywlk')
    call object_needed ('ee_r_bar_guide_grad_bywlk')
    call object_needed ('conf_jastr_a_een_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	
!  include 'chksiz_configurations%conf_jastr%a_een.inc'
  include 'chksiz_ea_r_bar_guide_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_grad_bywlk.inc'
  include 'chksiz_ea_r_bar_guide_grad_bywlk.inc'

! end requirements
! dimensions:
  configurations_guide_nb = system%configurations_guide_nb
  dimensions_nb     = system%dimensions_nb
  electrons_nb      = system%electrons_nb
  atoms_nb          = system%atoms_nb
  walkers_nb        = system%walkers_nb
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
    write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations
 
    call alloc ('u_een_guide_grad_bywlk', u_een_guide_grad_bywlk, & 
         electrons_nb, electrons_nb, atoms_nb,  &
         dimensions_nb, configurations_guide_nb, walkers_nb)

! end allocations

  u_een_guide_grad_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb

        a_een_1  = conf_jastr_a_een_guide (conf_i,1, atom_i)
        a_een_2  = conf_jastr_a_een_guide (conf_i,2, atom_i)

        do elec_j = 1, electrons_nb

          r_ja_bar = ea_r_bar_guide_bywlk (elec_j, atom_i, conf_i, walk_i) 
          r_ja_bar_2 = r_ja_bar * r_ja_bar

! Improve store squares in temporary arrays

          do elec_i = 1, electrons_nb

           r_ia_bar   = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)
           r_ia_bar_2 = r_ia_bar * r_ia_bar

           r_ia_bar_grad &
              = ea_r_bar_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)

           r_ij_bar   = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i)
           r_ij_bar_2 = r_ij_bar * r_ij_bar

           r_ij_bar_grad &
              = ee_r_bar_guide_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

! 2 (( a_een1 * rja^2 + a_een2 * rij^2 ) * ria * grad_i ria
!    + a_een2 * (ria^2 + rja^2) * rij * grad_i rij )

           d_u_een(:) = two * (                                  &
               (a_een_1 * r_ja_bar_2 + a_een_2 * r_ij_bar_2 )    &
                                 * r_ia_bar * r_ia_bar_grad (:)  & 
              + a_een_2 * (r_ia_bar_2 + r_ja_bar_2 )             &
                                 * r_ij_bar * r_ij_bar_grad (:)  )

           u_een_guide_grad_bywlk (elec_i, elec_j, atom_i, :, conf_i, walk_i) = d_u_een (:)

          end do
        end do
      end do
    end do
  end do

  include 'debug_u_een_guide_grad_bywlk.inc'
!   include 'ensnb_u_een_guide_grad_bywlk.inc'

  call exit (l_here)

  end subroutine u_een_guide_grad_bywlk_bld

! ===================================================================================
  subroutine u_een_guide_lapl_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_guide_lapl_bywlk_bld
!
! Creates        : u_een_guide_lapl_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_guide_bywlk (elec, elec, conf, walk)
! Needs          : conf_jastr_a_een_guide (conf, lik, atom)      
!
! Descriptions   : laplacian over elec_i coordinates
! Descriptions   : lapl u_een_guide_grad_bywlk 
! Descriptions   :  = 2 * (a_een1 * rja^2 + a_een2 *rij^2 ) ria * lapl_i ria
! Descriptions   :  + 2 * a_een2 * (ria^2 + rja^2) * rij * lapl_i rij
! Descriptions   :  + 2 * (a_een1 * rja^2 + a_een2 *rij^2 ) grad_i ria . grad_i ria
! Descriptions   :  + 2 * a_een2 * (ria^2 + rja^2) * grad_i rij . grad_i rij
! Descriptions   :  + 8 * a_een2 *rij *ria * grad_i ria . grad_i rij
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i, elec_j, walk_i, conf_i, atom_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ia_bar, r_ia_bar_2
  real(dp)                      :: r_ja_bar, r_ja_bar_2
  real(dp)                      :: r_ij_bar, r_ij_bar_2
  real(dp)                      :: r_ia_bar_lapl, r_ia_bar_grad(3)
  real(dp)                      :: r_ij_bar_lapl, r_ij_bar_grad(3)
  real(dp)                      :: a_een_1   
  real(dp)                      :: a_een_2 
  real(dp)                      :: dot_prod_ia
  real(dp)                      :: dot_prod_ij
  real(dp)                      :: dot_prod_ijia
  real(dp)                      :: lapl

! header:

  if (header_exe) then

    l_here = "u_een_guide_lapl_bywlk_bld"

    call object_create ('u_een_guide_lapl_bywlk')

    call object_needed ('ea_r_bar_guide_bywlk')
    call object_needed ('ee_r_bar_guide_bywlk')
    call object_needed ('conf_jastr_a_een_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

!  include 'chksiz_configurations%conf_jastr%a_een.inc'
  include 'chksiz_ea_r_bar_guide_bywlk.inc'
  include 'chksiz_ee_r_bar_guide_bywlk.inc'

! end requirements

! dimensions:

  configurations_guide_nb = system%configurations_guide_nb
  electrons_nb      = system%electrons_nb
  atoms_nb          = system%atoms_nb
  walkers_nb        = system%walkers_nb

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb 
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! end dimensions

! allocation 
 
    call alloc ('u_een_guide_lapl_bywlk', u_een_guide_lapl_bywlk, &
         electrons_nb, electrons_nb, atoms_nb, configurations_guide_nb, walkers_nb)
 
  u_een_guide_lapl_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb

        a_een_1  = conf_jastr_a_een_guide (conf_i,1, atom_i)
        a_een_2  = conf_jastr_a_een_guide (conf_i,2, atom_i)

        do elec_j = 1, electrons_nb

          r_ja_bar   = ea_r_bar_guide_bywlk (elec_j, atom_i, conf_i, walk_i) 

          r_ja_bar_2 = r_ja_bar * r_ja_bar

! Improve: verify formula
! Improve store squares in temporary arrays

          do elec_i = 1, electrons_nb

           r_ia_bar   = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)

           r_ia_bar_2 = r_ia_bar * r_ia_bar

           r_ia_bar_grad &
              = ea_r_bar_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)

           r_ia_bar_lapl &
              = ea_r_bar_guide_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)

           r_ij_bar   = ee_r_bar_guide_bywlk (elec_i, elec_j, conf_i, walk_i)

           r_ij_bar_2 = r_ij_bar * r_ij_bar

           r_ij_bar_grad &
              = ee_r_bar_guide_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

           r_ij_bar_lapl &
              = ee_r_bar_guide_lapl_bywlk (elec_i, elec_j, conf_i, walk_i)
   
           dot_prod_ia   = dot_product (r_ia_bar_grad, r_ia_bar_grad)
           dot_prod_ij   = dot_product (r_ij_bar_grad, r_ij_bar_grad)
           dot_prod_ijia = dot_product (r_ij_bar_grad, r_ia_bar_grad)

! = 2 * (a_een1 *  rja^2 + a_een2 * rij^2 ) * ria * lapl_i ria
! + 2 *  a_een2 * (ria^2 + rja^2) * rij * lapl_i rij
! + 2 * (a_een1 *  rja^2 + a_een2 * rij^2 ) * grad_i ria . grad_i ria
! + 2 *  a_een2 * (ria^2 + rja^2) * grad_i rij . grad_i rij
! + 8 *  a_een2 *  rij *ria * grad_i rij . grad_i ria
!
           lapl       = two * ( a_een_1 * r_ja_bar_2 + a_een_2 * r_ij_bar_2 ) &
                            *  r_ia_bar * r_ia_bar_lapl                       &

                      + two * a_een_2 * (r_ia_bar_2 + r_ja_bar_2)             &
                            *  r_ij_bar * r_ij_bar_lapl                       &

                      + two * ( a_een_1 * r_ja_bar_2 + a_een_2 * r_ij_bar_2 ) &
                            *  dot_prod_ia                                    &

                      + two * a_een_2 * (r_ia_bar_2 + r_ja_bar_2)             &
                            *  dot_prod_ij                                    &

                      + eight * a_een_2 * r_ij_bar * r_ia_bar * dot_prod_ijia 

           u_een_guide_lapl_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = lapl

          end do
        end do
      end do
    end do
  end do

  include 'debug_u_een_guide_lapl_bywlk.inc'
!   include 'ensnb_u_een_guide_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine u_een_guide_lapl_bywlk_bld

end module jastrow_u_een_guide_bywlk_mod
