! -*- f90 -*- 
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_u_en_bywlk_mod

  use basic_types_tools_mod
  use ea_r_bar_bywlk_mod
  use jastrow_a_en_1_4_mod

! Creates     : u_en_bywlk (elec, atom, conf, walk)
! Creates     : u_en_grad_bywlk (elec, atom, dim, conf, walk)
! Creates     : u_en_lapl_bywlk (elec, atom, conf, walk)

  real(dp), pointer :: u_en_bywlk                (:,:,:,:)       => NULL()

  real(dp), pointer :: u_en_a_der_bywlk        (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_en_b_der_bywlk        (:,:,:,:)       => NULL()
  real(dp), pointer :: u_en_ab_der2_bywlk      (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_en_b_der2_bywlk       (:,:,:,:)       => NULL()

  real(dp), pointer :: u_en_grad_bywlk           (:,:,:,:,:)     => NULL()

  real(dp), pointer :: u_en_grad_a_der_bywlk   (:,:,:,:,:,:)   => NULL()
  real(dp), pointer :: u_en_grad_b_der_bywlk   (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_en_grad_b_der2_bywlk  (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_en_grad_ab_der2_bywlk (:,:,:,:,:,:)   => NULL()

  real(dp), pointer :: u_en_lapl_bywlk           (:,:,:,:)       => NULL()

  real(dp), pointer :: u_en_lapl_a_der_bywlk   (:,:,:,:,:)     => NULL()
  real(dp), pointer :: u_en_lapl_b_der_bywlk   (:,:,:,:)       => NULL()
  real(dp), pointer :: u_en_lapl_b_der2_bywlk  (:,:,:,:)       => NULL()
  real(dp), pointer :: u_en_lapl_ab_der2_bywlk (:,:,:,:,:)     => NULL()

  contains

! ============================================================================
  subroutine u_en_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_bywlk_bld
!
! Creates        : u_en_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf)
! Needs          : jastrow_a_en_1_4 (atom, conf, deg)
!
! Descriptions   : u_en = Sum_n a_en (n) * r_bar^n  (n= 1..4)
! Descriptions   : r_bar = ea_r_bar_bywlk (elec, atom, conf)
! Descriptions   : a_en  = jastrow_a_en_1_4 (atom, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
! header:

  if (header_exe) then

    l_here = "u_en_bywlk_bld"

    call object_create ('u_en_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('jastrow_a_en_1_4')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	

  include 'chksiz_ea_r_bar_bywlk.inc'
  include 'chksiz_jastrow_a_en_1_4.inc'

! end requirements

! dimensions:
  electrons_nb      = size(ea_r_bar_bywlk, 1)
  atoms_nb          = size(ea_r_bar_bywlk, 2)
  configurations_nb = size(ea_r_bar_bywlk, 3)
  walkers_nb        = size(ea_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_en_bywlk', u_en_bywlk, electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_en_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              ! r x (b1 + r * (b2 +  r x (b3 + r * b4 ) ) )

              r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
              
              u_en_bywlk (elec_i, atom_i, conf_i, walk_i) = 0.d0

              do i = 1, jastr_pow_max

                 u_en_bywlk (elec_i, atom_i, conf_i, walk_i) =  u_en_bywlk (elec_i, atom_i, conf_i, walk_i) + &
                      r_ia_bar**i*( jastrow_a_en_1_4 (atom_i, conf_i, i))  

              end do ! do i         

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_u_en_bywlk.inc'
!   include 'ensnb_u_en_bywlk.inc'

  call exit (l_here)

  end subroutine u_en_bywlk_bld

! ============================================================================
  subroutine u_en_a_der_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_a_der_bywlk_bld
!
! Creates        : u_en_a_der_bywlk (elec, atom, pow, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf)
!
! Descriptions   : u_en_a_deriv =  r_ia_bar^pow
! Descriptions   : r_ia_bar     = ea_r_bar_bywlk (elec, atom, conf, walk)
! Descriptions   : pow          = 1, 2,...,jastr_pow_max
!
! Author         : A.C. Kollias
! Date           : 07/09/2005
!
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
! header:

  if (header_exe) then

    l_here = "u_en_a_der_bywlk_bld"

    call object_create ('u_en_a_der_bywlk')

    call object_needed ('ea_r_bar_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_bywlk, 1)
  atoms_nb          = size (ea_r_bar_bywlk, 2)
  configurations_nb = size (ea_r_bar_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ( 'u_en_a_der_bywlk', u_en_a_der_bywlk, electrons_nb, atoms_nb, jastr_pow_max, &
         configurations_nb, walkers_nb )

! end allocations

  u_en_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)

              do pow_i = 1, jastr_pow_max

                 u_en_a_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) =  r_ia_bar**pow_i
                 
              enddo ! do pow_i 
        
           end do ! elec_i

        end do ! atom_i

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

  end subroutine u_en_a_der_bywlk_bld

! ============================================================================
  subroutine u_en_b_der_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_b_der_bywlk_bld
!
! Creates        : u_en_b_der_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_r_bar_beta_der_bywlk(elec, atom, pow, conf, walk)
! Needs          : jastrow_a_en_1_4 (atom, conf, pow)
!
! Descriptions   : u_en_b_deriv = sum(n) alpha * r_ia_bar_beta_deriv
! Descriptions   : r_ia_bar     = ea_r_bar_bywlk (elec, atom, conf, walk)
! Descriptions   : alpha        = jastrow_a_en_1_4 (atom, conf, pow)
! Descriptions   : pow          = 1, 2,...,jastr_pow_max
!
! Author         : A.C. Kollias
! Date           : 07/09/2005
!
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: alpha
  real(dp)                      :: r_ia_bar_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_en_b_der_bywlk_bld"

    call object_create ('u_en_b_der_bywlk')

    call object_needed ('jastrow_a_en_1_4')
    call object_needed ('ea_r_bar_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_beta_der_bywlk, 1)
  atoms_nb          = size (ea_r_bar_beta_der_bywlk, 2)
  configurations_nb = size (ea_r_bar_beta_der_bywlk, 4)
  walkers_nb        = size (ea_r_bar_beta_der_bywlk, 5)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ( 'u_en_b_der_bywlk', u_en_b_der_bywlk, electrons_nb, atoms_nb, &
         configurations_nb, walkers_nb )

! end allocations

  u_en_b_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max
                 
                 r_ia_bar_beta_deriv = ea_r_bar_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)
                 
                 alpha =  jastrow_a_en_1_4 (atom_i, conf_i, pow_i)

                 u_en_b_der_bywlk (elec_i, atom_i, conf_i, walk_i) =  &
                      u_en_b_der_bywlk (elec_i, atom_i, conf_i, walk_i) +  &
                      alpha * r_ia_bar_beta_deriv
                                             
              enddo ! do pow_i 
        
           end do ! elec_i

        end do ! atom_i

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

  end subroutine u_en_b_der_bywlk_bld

! ============================================================================
  subroutine u_en_ab_der2_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_ab_der2_bywlk_bld
!
! Creates        : u_en_ab_der2_bywlk (elec, atom, pow, conf, walk)
!
! Needs          : ea_r_bar_beta_der_bywlk (elec, atom, pow, conf, walk)

! Descriptions   : u_en_b_deriv2        = r_ia_bar_beta_deriv
! Descriptions   : r_ia_bar_beta_deriv  = ea_r_bar_beta_der_bywlk (elec, atom, pow, conf, walk)
! Descriptions   : pow                  = 1, 2,...,jastr_pow_max
!
! Author         : A.C. Kollias
! Date           : 07/09/2005
!
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_en_ab_der2_bywlk_bld"

    call object_create ('u_en_ab_der2_bywlk')

    call object_needed ('ea_r_bar_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_beta_der_bywlk, 1)
  atoms_nb          = size (ea_r_bar_beta_der_bywlk, 2)
  configurations_nb = size (ea_r_bar_beta_der_bywlk, 4)
  walkers_nb        = size (ea_r_bar_beta_der_bywlk, 5)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ( 'u_en_ab_der2_bywlk', u_en_ab_der2_bywlk, electrons_nb, atoms_nb, &
         jastr_pow_max, configurations_nb, walkers_nb )

! end allocations

  u_en_ab_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max
                 
                 r_ia_bar_beta_deriv = ea_r_bar_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)

                 u_en_ab_der2_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) = r_ia_bar_beta_deriv
                                             
              enddo ! do pow_i 
        
           end do ! elec_i

        end do ! atom_i

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

  end subroutine u_en_ab_der2_bywlk_bld

! ============================================================================
  subroutine u_en_b_der2_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_b_der2_bywlk_bld
!
! Creates        : u_en_b_der2_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_r_bar_beta_der2_bywlk (elec, atom, pow, conf, walk)
! Needs          : jastrow_a_en_1_4 (atom, conf, pow)
!
! Descriptions   : u_en_b_deriv2        = sum(n) alpha * r_ia_bar_beta_deriv2
! Descriptions   : r_ia_bar_beta_deriv  = ea_r_bar_beta_der_bywlk (elec, atom, pow, conf, walk)
! Descriptions   : alpha                = jastrow_a_en_1_4 (atom, conf, pow)
! Descriptions   : pow                  = 1, 2,...,jastr_pow_max
!
! Author         : A.C. Kollias
! Date           : 07/09/2005
!
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: alpha
  real(dp)                      :: r_ia_bar_beta_deriv2

! header:

  if (header_exe) then

    l_here = "u_en_b_der2_bywlk_bld"

    call object_create ('u_en_b_der2_bywlk')

    call object_needed ('jastrow_a_en_1_4')
    call object_needed ('ea_r_bar_beta_der2_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_beta_der2_bywlk, 1)
  atoms_nb          = size (ea_r_bar_beta_der2_bywlk, 2)
  configurations_nb = size (ea_r_bar_beta_der2_bywlk, 4)
  walkers_nb        = size (ea_r_bar_beta_der2_bywlk, 5)
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ( 'u_en_b_der2_bywlk', u_en_b_der2_bywlk, electrons_nb, atoms_nb, &
         configurations_nb, walkers_nb )

! end allocations

  u_en_b_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              do pow_i = 1, jastr_pow_max
                 
                 r_ia_bar_beta_deriv2 = ea_r_bar_beta_der2_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)
                 
                 alpha =  jastrow_a_en_1_4 (atom_i, conf_i, pow_i)

                 u_en_b_der2_bywlk (elec_i, atom_i, conf_i, walk_i) =  &
                      u_en_b_der2_bywlk (elec_i, atom_i, conf_i, walk_i) +  &
                      alpha * r_ia_bar_beta_deriv2
                                             
              enddo ! do pow_i 
        
           end do ! elec_i

        end do ! atom_i

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

  end subroutine u_en_b_der2_bywlk_bld

! ============================================================================
  subroutine u_en_grad_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_grad_bywlk_bld
!
! Creates        : u_en_grad_bywlk (elec, atom, dim, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)
! Needs          : ea_r_bar_grad_bywlk (elec, atom, dim, conf, walk)
!
! Descriptions   : grad u_en = Sum_n n * a_en (n) * r_bar^(n-1) grad r_bar
! Descriptions   : (n= 1..4)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: i
 
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp)                      :: term

! header:

  if (header_exe) then

    l_here = "u_en_grad_bywlk_bld"

    call object_create ('u_en_grad_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('jastrow_a_en_1_4')
    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_ea_r_bar_bywlk.inc' 
  include 'chksiz_ea_r_bar_grad_bywlk.inc' 
  include 'chksiz_jastrow_a_en_1_4.inc'
! end requirements

! dimensions:
     electrons_nb      = size (ea_r_bar_grad_bywlk, 1) 
     atoms_nb          = size (ea_r_bar_grad_bywlk, 2)
     dimensions_nb     = size (ea_r_bar_grad_bywlk, 3)
     configurations_nb = size (ea_r_bar_grad_bywlk, 4)    
     walkers_nb        = size (ea_r_bar_grad_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations
    call alloc ('u_en_grad_bywlk', u_en_grad_bywlk, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_nb, walkers_nb)
! end allocation

  u_en_grad_bywlk = zero
  
  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)

              r_ia_bar_grad = ea_r_bar_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)

              term =0.d0

              do i= 1, jastr_pow_max

                 term = term+ i * r_ia_bar**(i-1)*jastrow_a_en_1_4 (atom_i, conf_i, i)

              end do ! do i 

              u_en_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i) = term * r_ia_bar_grad (:)
             
           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_en_grad_bywlk_bld

! ============================================================================
  subroutine u_en_grad_a_der_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_en_grad_a_der_bywlk_bld
!
! Creates        : u_en_grad_a_der_bywlk (elec, elec, pow, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)
! Needs          : ea_r_bar_grad_bywlk (elec, atom, dim, conf, walk)
! 
! Descriptions   : u_en_grad_a_deriv = r_ia_bar^(pow - 1) * pow * r_ia_bar_grad
! Descriptions   : r_ia_bar          = ea_r_bar_bywlk (elec, atom, conf, walk)
! Descriptions   : r_ia_bar_grad     = ea_r_bar_grad_bywlk (elec, atom, dim, conf, walk)
! Descriptions   : pow               = 1, 2,...,jastr_pow_max
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: dimensions_nb
  
  real(dp)                      :: gradient_derivative
  real(dp)                      :: r_ia_bar_grad
  real(dp)                      :: r_ia_bar

! header:

  if (header_exe) then

    l_here = "u_en_grad_a_der_bywlk_bld"

    call object_create ('u_en_grad_a_der_bywlk')
    
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('ea_r_bar_bywlk')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (ea_r_bar_bywlk,      1)
  atoms_nb          = size (ea_r_bar_bywlk,      2)
  configurations_nb = size (ea_r_bar_bywlk,      3)
  dimensions_nb     = size (ea_r_bar_grad_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk,      4)

 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_en_grad_a_der_bywlk', u_en_grad_a_der_bywlk, electrons_nb, atoms_nb,  &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb )

! end allocations

  u_en_grad_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb

              do dim_i =1, dimensions_nb
                 
                 r_ia_bar      = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)

                 r_ia_bar_grad = ea_r_bar_grad_bywlk (elec_i, atom_i, dim_i, conf_i, walk_i)

                 do pow_i = 1, jastr_pow_max

                    gradient_derivative = r_ia_bar ** (pow_i - 1.d0) * pow_i * r_ia_bar_grad

                    u_en_grad_a_der_bywlk(elec_i, atom_i, dim_i, pow_i, conf_i, walk_i) = &
                         gradient_derivative
                       
                 enddo ! do pow_i

              enddo ! do dim_i

           end do ! do atom_i

        end do ! do elec_i

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_en_grad_a_der_bywlk_bld

! ============================================================================
  subroutine u_en_grad_b_der_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_en_grad_b_der_bywlk_bld
!
! Creates        : u_en_grad_b_der_bywlk (elec, elec, pow, conf, walk)
!
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow, conf, walk) 
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, pow)
!
! Descriptions   : u_en_grad_b_deriv        = alpha * r_ia_bar_grad_beta_deriv
! Descriptions   : r_ia_bar_grad_beta_deriv = ea_r_bar_grad_beta_der_bywlk
! Descriptions   : alpha                    = jastrow_a_en_1_4
! Descriptions   : pow                      = 1, 2,...,jastr_pow_max
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: derivative
  real(dp)                      :: alpha
  real(dp)                      :: r_ia_bar_grad_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_en_grad_b_der_bywlk_bld"

    call object_create ('u_en_grad_b_der_bywlk')
    
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('jastrow_a_en_1_4')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (ea_r_bar_bywlk,                 1)
  atoms_nb          = size (ea_r_bar_bywlk,                 2)
  configurations_nb = size (ea_r_bar_bywlk,                 3)
  dimensions_nb     = size (ea_r_bar_grad_beta_der_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk,                 4)

 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

  call alloc ('u_en_grad_b_der_bywlk', u_en_grad_b_der_bywlk, electrons_nb, atoms_nb,  &
       dimensions_nb, configurations_nb, walkers_nb )

! end allocations

  u_en_grad_b_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb

              do dim_i =1, dimensions_nb

                 do pow_i = 1, jastr_pow_max

                    alpha = jastrow_a_en_1_4 (atom_i, conf_i, pow_i)
                    
                    r_ia_bar_grad_beta_deriv = &
                         ea_r_bar_grad_beta_der_bywlk (elec_i, atom_i, dim_i, pow_i, conf_i, walk_i) 
                    
                    derivative = alpha * r_ia_bar_grad_beta_deriv
                    
                    u_en_grad_b_der_bywlk(elec_i, atom_i, dim_i, conf_i, walk_i) =      &
                         u_en_grad_b_der_bywlk(elec_i, atom_i, dim_i, conf_i, walk_i) + &
                         derivative
                    
                 end do ! do pow_i 
                 
              enddo ! do dim_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_en_grad_b_der_bywlk_bld

! ============================================================================
  subroutine u_en_grad_ab_der2_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_en_grad_ab_der2_bywlk_bld
!
! Creates        : u_en_grad_ab_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : 
! Descriptions   : 
! Descriptions   : 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: r_ia_bar_grad_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_en_grad_ab_der2_bywlk_bld"

    call object_create ('u_en_grad_ab_der2_bywlk')
    
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_bywlk,                 1)
  atoms_nb          = size (ea_r_bar_bywlk,                 2)
  configurations_nb = size (ea_r_bar_bywlk,                 3)
  dimensions_nb     = size (ea_r_bar_grad_beta_der_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk,                 4)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_en_grad_ab_der2_bywlk', u_en_grad_ab_der2_bywlk, electrons_nb, atoms_nb,  &
         dimensions_nb, jastr_pow_max, configurations_nb, walkers_nb )

! end allocations

  u_en_grad_ab_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb

              do dim_i =1, dimensions_nb

                 do pow_i = 1, jastr_pow_max

                    r_ia_bar_grad_beta_deriv = &
                         ea_r_bar_grad_beta_der_bywlk (elec_i, atom_i, dim_i, pow_i, conf_i, walk_i) 
                    
                    u_en_grad_ab_der2_bywlk(elec_i, atom_i, dim_i, pow_i, conf_i, walk_i) =      &
                         r_ia_bar_grad_beta_deriv
                    
                 enddo ! do pow_i 
                 
              enddo ! do dim_i

           end do ! do elec_i

        end do ! do elec_j

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_en_grad_ab_der2_bywlk_bld

! ============================================================================
  subroutine u_en_grad_b_der2_bywlk_bld
! ----------------------------------------------------------------------------
!
! Name           : u_en_grad_b_der2_bywlk_bld
!
! Creates        : u_en_grad_b_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : jastrow_a_ee_1_4 (elec, elec, conf, deg)
!
! Descriptions   : 
! Descriptions   : 
! Descriptions   : 
!
! Author         : A. C. Kollias
! Date           : 30/08/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: pow_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: r_ia_bar_grad_beta_deriv2

! header:

  if (header_exe) then

    l_here = "u_en_grad_b_der2_bywlk_bld"

    call object_create ('u_en_grad_b_der2_bywlk')
    
    call object_needed ('ea_r_bar_grad_beta_der2_bywlk')
    
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_bywlk,                 1)
  atoms_nb          = size (ea_r_bar_bywlk,                 2)
  configurations_nb = size (ea_r_bar_bywlk,                 3)
  dimensions_nb     = size (ea_r_bar_grad_beta_der_bywlk, 3)
  walkers_nb        = size (ea_r_bar_bywlk,                 4)
 ! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_en_grad_b_der2_bywlk', u_en_grad_b_der2_bywlk, electrons_nb, atoms_nb,  &
         dimensions_nb, configurations_nb, walkers_nb )

! end allocations

  u_en_grad_b_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb

              do dim_i =1, dimensions_nb

                 do pow_i = 1, jastr_pow_max

                    r_ia_bar_grad_beta_deriv2 = &
                         ea_r_bar_grad_beta_der2_bywlk (elec_i, atom_i, dim_i, pow_i, conf_i, walk_i) 
                    
                    u_en_grad_b_der2_bywlk(elec_i, atom_i, dim_i, conf_i, walk_i) =      &
                         u_en_grad_b_der2_bywlk(elec_i, atom_i, dim_i, conf_i, walk_i) + &
                         r_ia_bar_grad_beta_deriv2
                    
                 enddo ! do pow_i 
                 
              enddo ! do dim_i

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_en_grad_b_der2_bywlk_bld

! ============================================================================
  subroutine u_en_lapl_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_lapl_bywlk_bld
!
! Creates        : u_en_lapl_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)
! Needs          : ea_r_bar_lapl_bywlk (elec, atom, conf, walk)
!
! Descriptions   : lapl u_en = Sum_n n * (n-1) *a_en (n) * r_bar^(n-2) 
! Descriptions   :                   ( grad r_bar . grad r_bar ) (n= 2..4)
! Descriptions   :  + Sum_n n * a_en (n) * r_bar^(n-1) * lapl r_bar (n= 1..4)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp)                      :: r_ia_bar_lapl
  real(dp)                      :: first_deriv
  real(dp)                      :: secon_deriv
  real(dp)                      :: dot_prod

! header:

  if (header_exe) then

    l_here = "u_en_lapl_bywlk_bld"

    call object_create ('u_en_lapl_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ea_r_bar_lapl_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('jastrow_a_en_1_4')
    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_ea_r_bar_bywlk.inc'
  include 'chksiz_ea_r_bar_lapl_bywlk.inc'
  include 'chksiz_jastrow_a_en_1_4.inc'
! end requirements

! dimensions:
     electrons_nb      = size (ea_r_bar_lapl_bywlk, 1)
     atoms_nb          = size (ea_r_bar_lapl_bywlk, 2)
     configurations_nb = size (ea_r_bar_lapl_bywlk, 3)
     walkers_nb        = size (ea_r_bar_lapl_bywlk, 4)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations
 
    call alloc ('u_en_lapl_bywlk', u_en_lapl_bywlk, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_en_lapl_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           do elec_i = 1, electrons_nb

              r_ia_bar      &
                   = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)

              r_ia_bar_grad &
                   = ea_r_bar_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)
              
              r_ia_bar_lapl &
                   = ea_r_bar_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)

              dot_prod = dot_product (r_ia_bar_grad, r_ia_bar_grad)

              first_deriv = 0.d0

              do i = 1, jastr_pow_max

                 first_deriv = first_deriv + r_ia_bar**(i-1)*i*jastrow_a_en_1_4 (atom_i, conf_i, i) 

              enddo ! do i

              secon_deriv = 0.d0

              do i = 2, jastr_pow_max

                 secon_deriv = secon_deriv + &
                      r_ia_bar**(i-2)*(i-1)*i*jastrow_a_en_1_4 (atom_i, conf_i, i) 

              enddo ! do i
              
              u_en_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)              &
                   = secon_deriv * dot_prod + first_deriv * r_ia_bar_lapl

           end do ! do elec_i

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_u_en_lapl_bywlk.inc'
!   include 'ensnb_u_en_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine u_en_lapl_bywlk_bld

! ============================================================================
  subroutine u_en_lapl_a_der_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_lapl_a_der_bywlk_bld
!
! Creates        : u_en_lapl_a_der_bywlk (elec, elec, conf, walk)
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: r_ia_bar
  real(dp)                      :: inv_r
  real(dp)                      :: inv_r_4
  real(dp)                      :: laplacian_derivative

! header:

  if (header_exe) then

    l_here = "u_en_lapl_a_der_bywlk_bld"

    call object_create ('u_en_lapl_a_der_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ea_dist_inv_bywlk') 
    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  configurations_nb = size (ea_r_bar_bywlk, 3)
  electrons_nb      = size (ea_r_bar_bywlk, 1)
  walkers_nb        = size (ea_r_bar_bywlk, 4)
  atoms_nb          = size (ea_r_bar_bywlk, 2)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_en_lapl_a_der_bywlk', u_en_lapl_a_der_bywlk, &
         electrons_nb, atoms_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations

  u_en_lapl_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb

              r_ia_bar = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)

              inv_r    = ea_dist_inv_bywlk(elec_i, atom_i, walk_i)

              inv_r_4  = inv_r**4
              
              do pow_i = 1, jastr_pow_max

                 laplacian_derivative = (pow_i + 1.d0) * pow_i * r_ia_bar**(pow_i + 2.d0) * inv_r_4

                 u_en_lapl_a_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) = laplacian_derivative
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do atom_i

     end do ! conf_i

  end do ! walk_i
  
  call exit (l_here)

end subroutine u_en_lapl_a_der_bywlk_bld

! ============================================================================
  subroutine u_en_lapl_b_der_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_lapl_b_der_bywlk_bld
!
! Creates        : u_en_lapl_b_der_bywlk (elec, elec, conf, walk)
!
! Needs          : ea_r_bar_lapl_beta_der_bywlk (elec, atom, pow, conf, walk)
! Needs          : jastrow_a_en_1_4 (atom, conf, pow)
!
! Description    : u_en_lapl_b_deriv        = sum (n) alpha * r_ia_bar_lapl_beta_deriv
! Description    : r_ia_bar_lapl_beta_deriv = ea_r_bar_lapl_beta_der_bywlk
! Description    : alpha                    = jastrow_a_en_1_4 (atom, conf, pow)
! Description    : pow                      = 1,2,...,jast_pow_max 
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: r_ia_bar_lapl_beta_deriv
  real(dp)                      :: alpha

! header:

  if (header_exe) then

    l_here = "u_en_lapl_b_der_bywlk_bld"

    call object_create ('u_en_lapl_b_der_bywlk')

    call object_needed ('jastrow_a_en_1_4') 
    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (ea_r_bar_lapl_beta_der_bywlk, 1)
  atoms_nb          = size (ea_r_bar_lapl_beta_der_bywlk, 2)
  configurations_nb = size (ea_r_bar_lapl_beta_der_bywlk, 4)
  walkers_nb        = size (ea_r_bar_lapl_beta_der_bywlk, 5)
 
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_en_lapl_b_der_bywlk', u_en_lapl_b_der_bywlk, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_en_lapl_b_der_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb
              
              do pow_i = 1, jastr_pow_max

                 alpha = jastrow_a_en_1_4 (atom_i, conf_i, pow_i)

                 r_ia_bar_lapl_beta_deriv = & 
                      ea_r_bar_lapl_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)

                 u_en_lapl_b_der_bywlk (elec_i, atom_i, conf_i, walk_i) =      &
                      u_en_lapl_b_der_bywlk (elec_i, atom_i, conf_i, walk_i) + &
                      alpha * r_ia_bar_lapl_beta_deriv
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do atom_i

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

end subroutine u_en_lapl_b_der_bywlk_bld


! ============================================================================
  subroutine u_en_lapl_ab_der2_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_lapl_ab_der2_bywlk_bld
!
! Creates        : u_en_lapl_ab_der2_bywlk (elec, elec, pow, conf, walk)
!
! Needs          : ea_r_bar_lapl_beta_der_bywlk (elec, atom, pow, conf, walk)
!  
! Description    : u_en_lapl_ab_deriv2      = r_ia_bar_lapl_beta_deriv
! Description    : r_ia_bar_lapl_beta_deriv = ea_r_bar_lapl_beta_der_bywlk
! Description    : pow                      = 1,2,3, jast_pow_max
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: r_ia_bar_lapl_beta_deriv

! header:

  if (header_exe) then

    l_here = "u_en_lapl_ab_der2_bywlk_bld"

    call object_create ('u_en_lapl_ab_der2_bywlk')

    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (ea_r_bar_lapl_beta_der_bywlk, 1)
  atoms_nb          = size (ea_r_bar_lapl_beta_der_bywlk, 2)
  configurations_nb = size (ea_r_bar_lapl_beta_der_bywlk, 4)
  walkers_nb        = size (ea_r_bar_lapl_beta_der_bywlk, 5)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_en_lapl_ab_der2_bywlk', u_en_lapl_ab_der2_bywlk, &
         electrons_nb, atoms_nb, jastr_pow_max, configurations_nb, walkers_nb)

! end allocations

  u_en_lapl_ab_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb
              
              do pow_i = 1, jastr_pow_max

                 r_ia_bar_lapl_beta_deriv = & 
                      ea_r_bar_lapl_beta_der_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)

                 u_en_lapl_ab_der2_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i) =      &
                      r_ia_bar_lapl_beta_deriv
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do atom_i

     end do ! conf_i

  end do ! walk_i

  call exit (l_here)

end subroutine u_en_lapl_ab_der2_bywlk_bld

! ============================================================================
  subroutine u_en_lapl_b_der2_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_lapl_b_der2_bywlk_bld
!
! Creates        : u_en_lapl_b_der2_bywlk (elec, elec, conf, walk)
!
! Needs          : ea_r_bar_lapl_beta_der2_bywlk (elec, atom, pow, conf, walk)
! Needs          : astrow_a_en_1_4 (atom, conf, pow)
!
! Description    : u_en_lapl_b_deriv2        = alpha * r_ia_bar_lapl_beta_deriv2
! Description    : r_ia_bar_lapl_beta_deriv2 = ea_r_bar_lapl_beta_der2_bywlk
! Description    : alpha                     = jastrow_a_en_1_4 (atom, conf, pow)
! Description    : pow                       = 1, 2,...,jastr_pow_max
!
! Author         : A.C. Kollias 
! Date           : 08/05/2005
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: pow_i

  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: atoms_nb

  real(dp)                      :: r_ia_bar_lapl_beta_deriv2
  real(dp)                      :: alpha

! header:

  if (header_exe) then

    l_here = "u_en_lapl_b_der2_bywlk_bld"

    call object_create ('u_en_lapl_b_der2_bywlk')

    call object_needed ('jastrow_a_en_1_4') 
    call object_needed ('ea_r_bar_lapl_beta_der2_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (ea_r_bar_lapl_beta_der2_bywlk, 1)
  atoms_nb          = size (ea_r_bar_lapl_beta_der2_bywlk, 2)
  configurations_nb = size (ea_r_bar_lapl_beta_der2_bywlk, 4)
  walkers_nb        = size (ea_r_bar_lapl_beta_der2_bywlk, 5)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations


    call alloc ('u_en_lapl_b_der2_bywlk', u_en_lapl_b_der2_bywlk, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_en_lapl_b_der2_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do elec_i = 1, electrons_nb

           do atom_i = 1, atoms_nb
              
              do pow_i = 1, jastr_pow_max

                 alpha = jastrow_a_en_1_4 (atom_i, conf_i, pow_i)

                 r_ia_bar_lapl_beta_deriv2 = & 
                      ea_r_bar_lapl_beta_der2_bywlk (elec_i, atom_i, pow_i, conf_i, walk_i)

                 u_en_lapl_b_der2_bywlk (elec_i, atom_i, conf_i, walk_i) =      &
                      u_en_lapl_b_der2_bywlk (elec_i, atom_i, conf_i, walk_i) + &
                      alpha * r_ia_bar_lapl_beta_deriv2
                 
              end do ! do pow_i
              
           end do ! do elec_i

        end do ! do atom_i

     end do ! conf_i

  end do ! walk_i
  
  call exit (l_here)

end subroutine u_en_lapl_b_der2_bywlk_bld

end module jastrow_u_en_bywlk_mod
