! -*- f90 -*- 
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_u_en_guide_bywlk_mod

  use basic_types_tools_mod
  use ea_r_bar_guide_bywlk_mod
  use jastrow_a_en_1_4_guide_mod

! Creates     : u_en_guide_bywlk (elec, atom, conf, walk)
! Creates     : u_en_guide_grad_bywlk (elec, atom, dim, conf, walk)
! Creates     : u_en_guide_lapl_bywlk (elec, atom, conf, walk)

  real(dp), pointer :: u_en_guide_bywlk (:,:,:,:)           => NULL()
  real(dp), pointer :: u_en_guide_grad_bywlk (:,:,:,:,:)    => NULL()
  real(dp), pointer :: u_en_guide_lapl_bywlk (:,:,:,:)      => NULL()

  contains

! ============================================================================
  subroutine u_en_guide_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_guide_bywlk_bld
!
! Creates        : u_en_guide_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf)
! Needs          : jastrow_a_en_1_4_guide (atom, conf, deg)
!
! Descriptions   : u_en = Sum_n a_en (n) * r_bar^n  (n= 1..4)
! Descriptions   : r_bar = ea_r_bar_guide_bywlk (elec, atom, conf)
! Descriptions   : a_en  = jastrow_a_en_1_4_guide (atom, conf, deg)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ia_bar
  integer(dpi)                  :: i
! header:

  if (header_exe) then

    l_here = "u_en_guide_bywlk_bld"

    call object_create ('u_en_guide_bywlk')

    call object_needed ('ea_r_bar_guide_bywlk')
    call object_needed ('jastrow_a_en_1_4_guide')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	

  include 'chksiz_ea_r_bar_guide_bywlk.inc'
  include 'chksiz_jastrow_a_en_1_4_guide.inc'

! end requirements

! dimensions:
  configurations_guide_nb = system%configurations_guide_nb
  electrons_nb      = system%electrons_nb
  atoms_nb          = system%atoms_nb
  walkers_nb        = system%walkers_nb
! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations

    call alloc ('u_en_guide_bywlk', u_en_guide_bywlk, &
         electrons_nb, atoms_nb, configurations_guide_nb, walkers_nb)

! end allocations

  u_en_guide_bywlk = NOT_INIT_DP

  do walk_i = 1, walkers_nb
     do conf_i = 1, configurations_guide_nb
        do atom_i = 1, atoms_nb
           do elec_i = 1, electrons_nb

! r x (b1 + r * (b2 +  r x (b3 + r * b4 ) ) )

              r_ia_bar = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)
 !             u_en_guide_bywlk (elec_i, atom_i, conf_i, walk_i) =                  &
 !                  r_ia_bar*( jastrow_a_en_1_4_guide (atom_i, conf_i, 1)     &
 !                  + r_ia_bar*( jastrow_a_en_1_4_guide (atom_i, conf_i, 2)     &
 !                  + r_ia_bar*( jastrow_a_en_1_4_guide (atom_i, conf_i, 3)     &
 !                  + r_ia_bar*  jastrow_a_en_1_4_guide (atom_i, conf_i, 4) ) ) )
              
              u_en_guide_bywlk (elec_i, atom_i, conf_i, walk_i) = 0.d0
              do i = 1, jastr_pow_max_guide
                 u_en_guide_bywlk (elec_i, atom_i, conf_i, walk_i) =  u_en_guide_bywlk (elec_i, atom_i, conf_i, walk_i) + &
                      r_ia_bar**i*( jastrow_a_en_1_4_guide (atom_i, conf_i, i))  
              enddo ! do i 
        
           end do
        end do
     end do
  end do

  include 'debug_u_en_guide_bywlk.inc'
!   include 'ensnb_u_en_guide_bywlk.inc'

  call exit (l_here)

  end subroutine u_en_guide_bywlk_bld

! ============================================================================
  subroutine u_en_guide_grad_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_guide_grad_bywlk_bld
!
! Creates        : u_en_guide_grad_bywlk (elec, atom, dim, conf, walk)
!
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf, walk)
! Needs          : ea_r_bar_guide_grad_bywlk (elec, atom, dim, conf, walk)
!
! Descriptions   : grad u_en = Sum_n n * a_en (n) * r_bar^(n-1) grad r_bar
! Descriptions   : (n= 1..4)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: dim_i
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb 
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp) :: term
  integer(dpi) :: i
! header:

  if (header_exe) then

    l_here = "u_en_guide_grad_bywlk_bld"

    call object_create ('u_en_guide_grad_bywlk')

    call object_needed ('ea_r_bar_guide_bywlk')
    call object_needed ('ea_r_bar_guide_grad_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_ea_r_bar_guide_bywlk.inc' 
  include 'chksiz_ea_r_bar_guide_grad_bywlk.inc' 
  include 'chksiz_jastrow_a_en_1_4_guide.inc'
! end requirements

! dimensions:
     electrons_nb      = size (ea_r_bar_guide_grad_bywlk, 1) 
     atoms_nb          = size (ea_r_bar_guide_grad_bywlk, 2)
     dimensions_nb     = size (ea_r_bar_guide_grad_bywlk, 3)
     configurations_guide_nb = size (ea_r_bar_guide_grad_bywlk, 4)    
     walkers_nb        = size (ea_r_bar_guide_grad_bywlk, 5)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations
    call alloc ('u_en_guide_grad_bywlk', u_en_guide_grad_bywlk, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_guide_nb, walkers_nb)
! end allocation

  u_en_guide_grad_bywlk = NOT_INIT_DP
  
  do walk_i = 1, walkers_nb
     do conf_i = 1, configurations_guide_nb
        do atom_i = 1, atoms_nb
           do elec_i = 1, electrons_nb

              r_ia_bar      &
                   = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)

              r_ia_bar_grad &
                   = ea_r_bar_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)

!              u_en_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)                &
!                   = (                jastrow_a_en_1_4_guide (atom_i, conf_i, 1)       &
!                   + r_ia_bar*(2.d0 * jastrow_a_en_1_4_guide (atom_i, conf_i, 2)       &
!                   + r_ia_bar*(3.d0 * jastrow_a_en_1_4_guide (atom_i, conf_i, 3)       &
!                   + r_ia_bar* 4.d0 * jastrow_a_en_1_4_guide (atom_i, conf_i, 4) ) ) ) &
!                   * r_ia_bar_grad (:)

              term =0.d0

              do i= 1, jastr_pow_max_guide
                 term= term+ i * r_ia_bar**(i-1)*jastrow_a_en_1_4_guide (atom_i, conf_i, i)
              enddo ! do i 
              u_en_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)           &
                   = term * r_ia_bar_grad (:)
             
           end do
        end do
     end do
  end do

  include 'debug_u_en_guide_grad_bywlk.inc'
!   include 'ensnb_u_en_guide_grad_bywlk.inc'

  call exit (l_here)

  end subroutine u_en_guide_grad_bywlk_bld

! ============================================================================
  subroutine u_en_guide_lapl_bywlk_bld 
! ----------------------------------------------------------------------------
!
! Name           : u_en_guide_lapl_bywlk_bld
!
! Creates        : u_en_guide_lapl_bywlk (elec, atom, conf, walk)
!
! Needs          : ea_r_bar_guide_bywlk (elec, atom, conf, walk)
! Needs          : ea_r_bar_guide_lapl_bywlk (elec, atom, conf, walk)
!
! Descriptions   : lapl u_en = Sum_n n * (n-1) *a_en (n) * r_bar^(n-2) 
! Descriptions   :                   ( grad r_bar . grad r_bar ) (n= 2..4)
! Descriptions   :  + Sum_n n * a_en (n) * r_bar^(n-1) * lapl r_bar (n= 1..4)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 Jun 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: elec_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp)                      :: r_ia_bar_lapl
  real(dp)                      :: first_deriv, secon_deriv, dot_prod
  integer(dpi)                  :: i
! header:

  if (header_exe) then

    l_here = "u_en_guide_lapl_bywlk_bld"

    call object_create ('u_en_guide_lapl_bywlk')

    call object_needed ('ea_r_bar_guide_bywlk')
    call object_needed ('ea_r_bar_guide_lapl_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_ea_r_bar_guide_bywlk.inc'
  include 'chksiz_ea_r_bar_guide_lapl_bywlk.inc'
  include 'chksiz_jastrow_a_en_1_4_guide.inc'
! end requirements

! dimensions:
     electrons_nb      = size (ea_r_bar_guide_lapl_bywlk, 1)
     atoms_nb          = size (ea_r_bar_guide_lapl_bywlk, 2)
     configurations_guide_nb = size (ea_r_bar_guide_lapl_bywlk, 3)
     walkers_nb        = size (ea_r_bar_guide_lapl_bywlk, 4)
! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations
 
    call alloc ('u_en_guide_lapl_bywlk', u_en_guide_lapl_bywlk, &
         electrons_nb, atoms_nb, configurations_guide_nb, walkers_nb)

! end allocations

  u_en_guide_lapl_bywlk = NOT_INIT_DP

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do atom_i = 1, atoms_nb
        do elec_i = 1, electrons_nb

          r_ia_bar      &
             = ea_r_bar_guide_bywlk (elec_i, atom_i, conf_i, walk_i)

          r_ia_bar_grad &
             = ea_r_bar_guide_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)

          r_ia_bar_lapl &
             = ea_r_bar_guide_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)

! Improve:
          dot_prod = dot_product (r_ia_bar_grad, r_ia_bar_grad)

!          first_deriv                                                     &
!              =                  jastrow_a_en_1_4_guide (atom_i, conf_i, 1)     &
!              + r_ia_bar*(2.d0 * jastrow_a_en_1_4_guide (atom_i, conf_i, 2)     &
!              + r_ia_bar*(3.d0 * jastrow_a_en_1_4_guide (atom_i, conf_i, 3)     &
!              + r_ia_bar* 4.d0 * jastrow_a_en_1_4_guide (atom_i, conf_i, 4) ) )  

!          secon_deriv                                                     &
!              = (         2.d0 * jastrow_a_en_1_4_guide(atom_i, conf_i, 2)      &
!              + r_ia_bar*(6.d0 * jastrow_a_en_1_4_guide(atom_i, conf_i, 3)      &
!              + r_ia_bar*12.d0 * jastrow_a_en_1_4_guide(atom_i, conf_i, 4) ) )   

           first_deriv = 0.d0
           do i = 1, jastr_pow_max_guide
              first_deriv = first_deriv + &
                   r_ia_bar**(i-1)*i*jastrow_a_en_1_4_guide (atom_i, conf_i, i) 
           enddo ! do i


           secon_deriv = 0.d0
           do i = 2, jastr_pow_max_guide
              secon_deriv = secon_deriv + &
                   r_ia_bar**(i-2)*(i-1)*i*jastrow_a_en_1_4_guide (atom_i, conf_i, i) 
           enddo ! do i

          u_en_guide_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)              &
              = secon_deriv * dot_prod + first_deriv * r_ia_bar_lapl

          end do
        end do
      end do
    end do

  include 'debug_u_en_guide_lapl_bywlk.inc'
!   include 'ensnb_u_en_guide_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine u_en_guide_lapl_bywlk_bld

end module jastrow_u_en_guide_bywlk_mod
