! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module kinds_parameters_mod

! Descriptions: defines constants

  integer, parameter :: big = selected_int_kind(11) ! -10**11 < n < 10**11
  integer, parameter :: i1b = selected_int_kind(2)  ! -10**2 < n < 10**2
  integer, parameter :: i2b = selected_int_kind(4)  ! -10**4 < n < 10**4
  integer, parameter :: i4b = selected_int_kind(9)  ! -10**9  < n < 10**9
  integer, parameter :: i8b = selected_int_kind(18) ! -10**18 < n < 10**18
  integer, parameter :: dpi = i4b
    
 
  integer, parameter :: sp  = kind(1.0)
  integer, parameter :: dp  = kind(1.0d0)
  integer, parameter :: spc = kind((1.0,1.0))
  integer, parameter :: dpc = kind((1.0d0,1.0d0))
  integer, parameter :: bool= kind(.true.)

  integer(dpi), parameter  :: zero_i = 0
  integer(dpi), parameter  :: one_i  = 1
  integer(dpi), parameter  :: two_i  = 2
  integer(dpi), parameter  :: huge_i = huge(1)

  real(sp), parameter :: zero_sp = 0.0_sp
  real(sp), parameter :: half_sp = 0.5_sp
  real(sp), parameter :: one_sp  = 1.0_sp
  real(sp), parameter :: two_sp  = 2.0_sp
  real(sp), parameter :: four_sp = 4.0_sp
  real(dp), parameter :: five_sp = 5.0_sp
  real(sp), parameter :: eight_sp = 8.0_sp
  real(sp), parameter :: pi_sp   = 3.141592653589793238462643383279502884197_sp
  real(sp), parameter :: pio2_sp = 1.57079632679489661923132169163975144209858_sp
  real(sp), parameter :: twopi_sp=6.283185307179586476925286766559005768394_sp
  real(sp), parameter :: sqrt2_sp= 1.41421356237309504880168872420969807856967_sp
  real(sp), parameter :: euler_sp=0.5772156649015328606065120900824024310422_sp

  real(dp), parameter :: zero    = 0.0_dp
  real(dp), parameter :: half    = 0.5_dp
  real(dp), parameter :: one     = 1.0_dp
  real(dp), parameter :: two     = 2.0_dp
  real(dp), parameter :: four    = 4.0_dp
  real(dp), parameter :: five    = 5.0_dp
  real(dp), parameter :: eight   = 8.0_dp
  real(dp), parameter :: pi      = 3.141592653589793238462643383279502884197_dp
  real(dp), parameter :: pio2    = 1.57079632679489661923132169163975144209858_dp
  real(dp), parameter :: sqrt2   = 1.41421356237309504880168872420969807856967_dp
  real(dp), parameter :: twopi   =6.283185307179586476925286766559005768394_dp
  real(dp), parameter :: fourpi  =four*pi
  real(dp), parameter :: euler   =0.5772156649015328606065120900824024310422_dp
  real(dp), parameter :: bigdp   = 1.d+99

! Description: CR carriage return
! Description: NL new-line same as CR
! Description: NULL_C null character
! Description: SLASH can be '\/'
! Description: TAB horizontal tabulation 
!
  character(len= 1)            :: CR = char(10)
  character(len= 1)            :: NL = char(10)
  character(len= 1)            :: NULL_C = char(0)
  character(len= 1)            :: SLASH = '/'      
  character(len= 1)            :: TAB= char(9)     !

! Description : NOT_INIT_B  = .false.
! Description : NOT_INIT_DP = -HUGE(1.d0)
! Description : NOT_INIT_I  = NOT_INIT_I  = -HUGE(1):
! Description : NOT_INIT_S  = '?'
! Description : NOT_INIT_SP = -HUGE(one_sp)

  logical(bool)               :: NOT_INIT_B  = .false.
  real(dp), parameter         :: NOT_INIT_DP = -HUGE(1.d0)
  real(sp), parameter         :: NOT_INIT_SP = -HUGE(one_sp)
  integer(dpi), parameter     :: NOT_INIT_I  = -HUGE(1)
  character(len= 1), parameter :: NOT_INIT_S = '?'         

! Description : exponentials arguments cut-offs:
! Description : exp(EXP_ARG_INF) ~= 1.d-16
! Description : exp(EXP_ARG_SUP) ~= huge(1.d0)

  real(dp), parameter         :: EXP_ARG_INF = -36.84d0 
  real(dp), parameter         :: EXP_ARG_SUP = 709.78d0 

end module kinds_parameters_mod

