/*
 * Le programme principal C
 * Il ne fait rien d'autre que
 * 1. de stocker argc et l'adresse d'argv dans une zone globale
 * 2. d'appeler le programme main Fortran
 *
 * Tout ce qui est non portable est defini par des macros (voir argc.h)
 *
 * Note : ce fichier contient egalement les fonctions getargc et getarg
 *
 Author: J.-M. Teuler 09 Oct 2001
*/

/* Pour strlen et strncpy */
#include <string.h>

/* Les definitions, a adapter selon la plateforme */

/* Le nom systeme du sous-programme principal Fortran */
#ifdef UNDERSCORE
#define MAIN_F main_f_
#else
#define MAIN_F main_f
#endif


#define main_c main   
/* Le nom systeme de la fonction C getargc */

#ifdef UNDERSCORE
#define GETARGC getargc_
#else
#define GETARGC getargc
#endif 

/* Le nom systeme de la fonction C getarg */

#ifdef UNDERSCORE
#define GETARG getarg_
#else
#define GETARG getarg
#endif

/* Le prototype de main */
long int MAIN_F();

/* Le type entier C equivalent a INTEGER Fortran */
#define  INT  int

/* Les variables globales, pointeurs vers les zones arguments */
int argc_g;
char **argv_g;

int main_c ( int argc, char *argv[] )
{ 
 
  /* On copie */
  argc_g = argc;
  argv_g = argv;

  /* On appelle le main Fortran (le nom est une macro) */
  MAIN_F();
}

/*
 * La fonction getargc (le nom est une macro)
 * Retourne le nombre d'arguments
 */

INT       /* Le nombre d'arguments */
GETARGC()
{
  return argc_g;
}

/*
 * La fonction getarg (le nom est une macro)
 * On met la chaine a la fin pour eviter les arguments parasites
 * transmis par Fortran avec les chaines
 */

/* Code de retour (voir ci-dessous)                  */
size_t  GETARG(
 long INT *n,      /* Le numero de l'argument (input)                   */
 long INT *lenMax, /* La longueur maximale de la chaine (input)         */
  char *chaine /* La chaine pour le stockage de l'argument (output) */
)
{
  /* n doit >= 0 et < argc_g */
  if ( *n < 0 || *n >= argc_g ) return -1;

  /* Il faut copier argv_g[n] dans chaine sans depasser lenMax */
  /* strncpy est fait pour ca */

  strncpy (chaine, argv_g[*n], *lenMax );

  /* On retourne la longueur de la chaine */
  return strlen(chaine);

}
