! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!          Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

! =================================================================================
  subroutine main_f ()
! ---------------------------------------------------------------------------------
!
! Name           : main
!
! Description    : QmcMol arguments
! Description    : some basic initialization
!
! Authors        : J.-M. Teuler and F. Colonna
! Date           : 09 Oct 2001
! Revision: 28 Oct 2002 F. Colonna
! ---------------------------------------------------------------------------------

  use parser_mod
  use basic_types_tools_mod 
  implicit none

! function:
  integer(i4b)                               :: getargc

! local:
  integer(dpi)                               :: nb_arg
  integer(dpi)                               :: i
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_argum
  character(len=max_string_len)              :: l_temp   
  character(len=max_string_len)              :: l_value  
  character(len=max_string_len)              :: get_arg_val
  integer(dpi)                               :: index_zero
  logical(bool)                              :: l_debug

! begin:
  l_here ='qmcmol'
  l_debug = .true.
  l_debug = .false.

! initialize constants.
  call init

! call argument list.
  nb_arg = getargc()

! get executable name.
  exec_name = get_arg_val (0)
  index_zero = index(exec_name, char(0))
  if(index_zero /= 0) then
    write (logf,*) trim(l_here),': executable name is >',trim(exec_name),'<'
    write (logf,*) trim(l_here), &
    ': there is a null character in  executable name at position ',index_zero
    write (logf,*) trim(l_here),': please check C routine get_arg_val'
    stop 'null character in executable name'
  end if

! initialization:
  files_names_in  = NOT_INIT_S
  files_names_out = NOT_INIT_S
  files_names_err = NOT_INIT_S
  files_names_res = NOT_INIT_S
  files_names_logf = NOT_INIT_S

  if(l_debug) then
    write (logf,*) 'main-d: nb_arg =',nb_arg
    write (logf,*) 'main-d: executable >',trim(exec_name),'<'
  end if

! loop over arguments.
  i = 1
  do while(i < nb_arg)
    l_argum = get_arg_val (i)
    if(l_debug) write (logf,*)trim(l_here),'-d: l_argum >',trim(l_argum),'<'

     select case (l_argum(1:2))    

     case ("-c")
       l_value = get_arg_val (i+1)
       l_temp = trim(cpu_routines_list)//','//blanks
       cpu_routines_list = trim(l_temp)//trim(l_value)//blanks
 
     case ("-d")
       l_value = get_arg_val (i+1)
       if(numeric_is(l_value)) then
         call decode(die_level, l_value)       
       else
         l_temp = trim(debug_routines_list)//','//blanks
         debug_routines_list = trim(l_temp)//trim(l_value)//blanks
       end if

     case ("-e")
       l_value = get_arg_val (i+1)
       call decode (echo, l_value)
 
     case ("-h")
       write (logf,*)'Usage ',trim(exec_name)
       write (logf,*)' -c routine-name -- activates cpu output'
       write (logf,*)' -d routine-name -- activates debug'
       write (logf,*)' -e t|f          -- activates echo'
       write (logf,*)' -i input-file   -- input file'
       write (logf,*)' -j job-name     -- "job name"'
       write (logf,*)' -l log-file     -- log file'
       write (logf,*)' -p integer      -- print_level'
       write (logf,*)' -r result-file  -- result file'
       write (logf,*)' -t integer      -- trace_level'
       write (logf,*)' -w integer      -- warning_level'
       write (logf,*)' -C              -- input commands'
       stop 'qmcmol ended'
 
     case ("-i")
       files_names_in = get_arg_val (i+1)
       write (logf,*)'main: input file >',trim(files_names_in),'<'

     case ("-j")
       job_name = get_arg_val (i+1)
 
     case ("-l")
       files_names_logf = get_arg_val (i+1)
       write (logf,*)'main: log file >',trim(files_names_logf),'<'

     case ("-o")
       files_names_out = get_arg_val (i+1)
       write (logf,*)'main: output file >',trim(files_names_out),'<'

     case ("-p")
       l_value = get_arg_val (i+1)
       call decode (print_level, l_value)       
 
     case ("-r")
       files_names_res = get_arg_val (i+1)
       write (logf,*)'main: results file >',trim(files_names_res),'<'
 
     case ("-t")
       l_value = get_arg_val (i+1)
       trace_level = string_to_integer (l_value)
 
     case ("-w")
       l_value = get_arg_val (i+1)
       call decode (warning_level, l_value)       
 
     case ("-C")
       write (logf,*)'qmcmol: qmcmol main commands:'
       write (logf,*)'qmcmol: bas{is}        [command]'
       write (logf,*)'qmcmol: ele{ctrons}    [command]'
       write (logf,*)'qmcmol: exi{t}         ends'
       write (logf,*)'qmcmol: fil{e}         [command]'
       write (logf,*)'qmcmol: fun{ction}     [command]'
       write (logf,*)'qmcmol: inc{lude}      [include file-name]'
       write (logf,*)'qmcmol: mos            [command]'
       write (logf,*)'qmcmol: min{imization} [command]'
       write (logf,*)'qmcmol: mon{tecarlo}   [command]'
       write (logf,*)'qmcmol: nod{es}        [command]'
       write (logf,*)'qmcmol: nuc{lei}       [command]'
       write (logf,*)'qmcmol: obj{ects}      [command]'
       write (logf,*)'qmcmol: pse{udo}       [command]'
       write (logf,*)'qmcmol: pri{nt}        [command]'
       write (logf,*)'qmcmol: set            [command]'
       write (logf,*)'qmcmol: sho{w}         [command]'
       write (logf,*)'qmcmol: sto{p}         ends'
       write (logf,*)'qmcmol: sym{bol}       [command]'
       write (logf,*)'qmcmol: wal{kers}      [command]'
       write (logf,*)'qmcmol: wav{efunction} [command]'
       write (logf,*)'qmcmol: wri{te}        [command]'
       stop 'qmcmol ended'

     case ("-")

       message=': argument # '+i+' >'+l_argum+  &
               '< of programm '*exec_name*' is illegal'
       write (logf,*)trim(l_here),message

       write (logf,*)' Usage ',trim(exec_name)
       write (logf,*)' -c{pu}          routine'
       write (logf,*)' -d{ebug}        routine'
       write (logf,*)' -D{ielevel}     number (not yet active)'
       write (logf,*)' -e{cho}         t{rue}|f{alse}        '
       write (logf,*)' -i{input-file}  "string" '
       write (logf,*)' -j{ob-name}     "string" '
       write (logf,*)' -o{output-file} "string" '
       write (logf,*)' -p{rintlevel}   number'
       write (logf,*)' -t{racelevel}   number'
       write (logf,*)' -w{arninglevel} number'
    
       stop 'qmcmol: illegal command argument '

     case default 

       files_names_in = get_arg_val (i)
       if(debug) then
         write (logf,*)'main: input file connected to >',trim(files_names_in),'<'
       end if

    end select

    i = i + 2

  end do

! further initializations
  call files_init
  call types_catalog_init
  call routines_catalog_init
  call objects_init
  
! montecarlo program
  call main_menu_mc 

  end subroutine main_f

! =================================================================================
  function get_arg_val (arg_index) result(result) 
! ---------------------------------------------------------------------------------
!
! Name          : get_arg_val 
!
! Argument (in) : arg_index 
!
! Description   : returns value of arg_index-th program argument
!
! Authors       : F. Colonna
! Date          : 10 Oct 2001
! ---------------------------------------------------------------------------------
  use parser_mod
  implicit none

! i/o:
  integer(dpi)                               :: arg_index

! result:
  character(len=max_string_len)              :: result   
  
! function:
  integer(dpi)                               :: getarg

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_argum
  integer(dpi)                               :: rc

! begin:
  l_here ='get_arg_val'
!  call enter (l_here)

! if(debug)write (logf,*)trim(l_here),'-d: '
  
  l_argum = blanks
  rc = getarg (arg_index, len(l_argum), l_argum)

  if ( rc == 0 ) then
    message = "Valeur invalide de arg_index : "+arg_index
    call die (l_here, message)
  end if

  result = l_argum(1:rc)//blanks

! if(debug) then
! write (logf,*)trim(l_here),'-d: arg_index =',arg_index
! write (logf,*)trim(l_here),'-d: rc =',rc
! write (logf,*)trim(l_here),'-d: result =',result
! end if

!  call exit (l_here)

  end function get_arg_val

! ======================================================================
  subroutine routines_catalog_init 
! ----------------------------------------------------------------------
!
! Name           : routines_catalog_init
!
! Description    : initializes routine address
! Description    : sees all modules where routines are 
! Description    : generated by "perl BuildRoutCatalFromCreatesAndNeeds.pl *.f90"
!
! -- Creates     : routines_address (rtn) 
!
! -- Needs       : routines_names (rtn) 
!
! Authors        : F. Colonna 
! Date           : 22 May 2002
! -------------------------------------------------------------------
!
! make all routines visible
    use setup_mod
    use basis_menu_mod
    use files_menu_mod
    use expressions_mod
    use mos_menu_mod
    use montecarlo_menu_mod
    use print_mod
    use walkers_menu_mod
    use wavefunction_menu_mod
    use write_mod
    use basic_types_tools_mod
    use intr_func_mod
    use e_reference_bywlk_mod
    use intracule_mod
    use force_bywlk_mod
    use mos_on_grid_mod
    use psi_guide_bywlk_mod
    use drift_guide_bywlk_mod
    use density_bywlk_mod
    use density_grid_mod
    use grid_coor_mod
    use density_guide_bywlk_mod
    use ci_mod
    use step_bywlk_secondary_mod
    implicit none

! external:
! include 'routines_external.inc'

! function:
! integer(dpi)                  :: address

! local:
    character(len=max_string_len_routine_name) :: l_here
! integer(dpi)                  :: routines_nb

! begin:
    l_here ='routines_catalog_init'
    call enter (l_here)

    call one_routine_catalog ('character_1_type_write', character_1_type_write)
    call one_routine_catalog ('character_2_type_write', character_2_type_write)
    call one_routine_catalog ('real_0_type_write', real_0_type_write)
    call one_routine_catalog ('real_1_type_write', real_1_type_write)
    call one_routine_catalog ('real_2_type_write', real_2_type_write)
    call one_routine_catalog ('real_3_type_write', real_3_type_write)
    call one_routine_catalog ('real_4_type_write', real_4_type_write)
    call one_routine_catalog ('real_5_type_write', real_5_type_write)
    call one_routine_catalog ('real_6_type_write', real_6_type_write)
    call one_routine_catalog ('real_7_type_write', real_7_type_write)
    call one_routine_catalog ('integer_0_type_write', integer_0_type_write)
    call one_routine_catalog ('integer_1_type_write', integer_1_type_write)
    call one_routine_catalog ('integer_2_type_write', integer_2_type_write)
    call one_routine_catalog ('integer_3_type_write', integer_3_type_write)
    call one_routine_catalog ('integer_4_type_write', integer_4_type_write)
    call one_routine_catalog ('integer_5_type_write', integer_5_type_write)
    call one_routine_catalog ('integer_6_type_write', integer_6_type_write)
    call one_routine_catalog ('real_1_byblk_bld',real_1_byblk_bld)
    call one_routine_catalog ('blk',real_1_byblk_bld)
    call one_routine_catalog ('real_1_from_real_3_bld',real_1_from_real_3_bld)
    call one_routine_catalog ('real_6_from_real_6_bld',real_6_from_real_6_bld)
    call one_routine_catalog ('real_7_from_real_7_bld',real_7_from_real_7_bld)
    call one_routine_catalog ('rand_2',real_1_from_real_2_bld)
    call one_routine_catalog ('rand_3',real_1_from_real_3_bld)

    call one_routine_catalog ('alfa_orbitals_init', alfa_orbitals_init)
!    call one_routine_catalog ('ao_first_in_xpfs_c_bld',ao_first_in_xpfs_c_bld)
    call one_routine_catalog ('aos_c_names_bld', aos_c_names_bld)
    call one_routine_catalog ('aos_c_norm_bld', aos_c_norm_bld)
    call one_routine_catalog ('aos_c_nrmd_to_xpfs_c_bld', aos_c_nrmd_to_xpfs_c_bld)
    call one_routine_catalog ('aos_c_to_xpfs_c_bld', aos_c_to_xpfs_c_bld)
!    call one_routine_catalog ('aos_radial_bywlk_bld',aos_radial_bywlk_bld)
    call one_routine_catalog ('aos_s_names_bld', aos_s_names_bld)
    call one_routine_catalog ('aos_s_norm_bld', aos_s_norm_bld)
    call one_routine_catalog ('aos_s_nrmd_to_xpfs_s_bld', aos_s_nrmd_to_xpfs_s_bld)
    call one_routine_catalog ('aos_s_to_xpfs_s_bld', aos_s_to_xpfs_s_bld)
    call one_routine_catalog ('atom_atom_dist_grad_bld', atom_atom_dist_grad_bld)
    call one_routine_catalog ('atom_atom_dist_xyz_bld', atom_atom_dist_xyz_bld)
    call one_routine_catalog ('atom_atom_distance_bld', atom_atom_distance_bld)
    call one_routine_catalog ('atoms_coor_vector_bld',atoms_coor_vector_bld)
    call one_routine_catalog ('atoms_init', atoms_init)
    call one_routine_catalog ('atoms_print', atoms_print)
    call one_routine_catalog ('atoms_print_res', atoms_print_res)
    call one_routine_catalog ('beta_orbitals_init', beta_orbitals_init)
    call one_routine_catalog ('center_xpfs_c_to_power_max_bld', center_xpfs_c_to_power_max_bld)
    call one_routine_catalog ('ci_eigenvectors_bld',ci_eigenvectors_bld)
    call one_routine_catalog ('ci_egenvalues_bld',ci_eigenvalues_bld)
    call one_routine_catalog ('ci_matrix_bld',ci_matrix_bld)
    call one_routine_catalog ('ci_matrix_var_bld',ci_matrix_var_bld)
    call one_routine_catalog ('ci_matrix_err_bld',ci_matrix_err_bld)
    call one_routine_catalog ('configurations_init', configurations_init)
    call one_routine_catalog ('conf_jastr_read', conf_jastr_read)
    call one_routine_catalog ('configuration_lapl_over_psi_bywlk_bld',configuration_lapl_over_psi_bywlk_bld)
    call one_routine_catalog ('configuration_over_psi_bywlk_bld',configuration_over_psi_bywlk_bld)
    call one_routine_catalog ('delta_elec_coor_bywlk_bld',delta_elec_coor_bywlk_bld)
    call one_routine_catalog ('delta_param_bywlk_bld',delta_param_bywlk_bld)
    call one_routine_catalog ('density_bywlk_bld', density_bywlk_bld)
    call one_routine_catalog ('density_grid_bld', density_grid_bld)
    call one_routine_catalog ('density_grid_var_bld', density_grid_var_bld)
    call one_routine_catalog ('density_grid_err_bld', density_grid_err_bld)
    call one_routine_catalog ('density_guide_bywlk_bld', density_guide_bywlk_bld)
    call one_routine_catalog ('det_alfa_bywlk_bld', det_alfa_bywlk_bld)
    call one_routine_catalog ('det_alfa_grad_bywlk_bld', det_alfa_grad_bywlk_bld)
    call one_routine_catalog ('det_alfa_lapl_bywlk_bld', det_alfa_lapl_bywlk_bld)
    call one_routine_catalog ('det_beta_bywlk_bld', det_beta_bywlk_bld)
    call one_routine_catalog ('det_beta_grad_bywlk_bld', det_beta_grad_bywlk_bld)
    call one_routine_catalog ('det_beta_lapl_bywlk_bld', det_beta_lapl_bywlk_bld)
    call one_routine_catalog ('det_sum_bywlk_bld', det_sum_bywlk_bld)
    call one_routine_catalog ('det_sum_grad_bywlk_bld', det_sum_grad_bywlk_bld)
    call one_routine_catalog ('det_sum_lapl_bywlk_bld', det_sum_lapl_bywlk_bld)
    call one_routine_catalog ('determinants_init', determinants_init)
    call one_routine_catalog ('drift_bywlk_bld', drift_bywlk_bld)
    call one_routine_catalog ('drift_square_bywlk_bld', drift_square_bywlk_bld)
    call one_routine_catalog ('drift_guide_bywlk_bld', drift_guide_bywlk_bld)
    call one_routine_catalog ('elec_ecp_random_rot_bywlk_bld', elec_ecp_random_rot_bywlk_bld)
    call one_routine_catalog ('e_loc_kin_bywlk_bld', e_loc_kin_bywlk_bld)
    call one_routine_catalog ('e_loc_hf_kin_bywlk_bld', e_loc_hf_kin_bywlk_bld)
    call one_routine_catalog ('e_loc_tot_bywlk_bld', e_loc_tot_bywlk_bld)
    call one_routine_catalog ('e_loc_hf_tot_bywlk_bld', e_loc_hf_tot_bywlk_bld)
    call one_routine_catalog ('e_loc_tot2_bywlk_bld', e_loc_tot2_bywlk_bld)
    call one_routine_catalog ('e_loc_vee_bywlk_bld', e_loc_vee_bywlk_bld)
    call one_routine_catalog ('e_loc_vne_bywlk_bld', e_loc_vne_bywlk_bld)
    call one_routine_catalog ('e_loc_vnn_bld', e_loc_vnn_bld)
    call one_routine_catalog ('ecp_bywlk_bld',ecp_bywlk_bld)
    call one_routine_catalog ('ecp_ea_bywlk_bld',ecp_ea_bywlk_bld)
    call one_routine_catalog ('e_reference_bywlk_bld', e_reference_bywlk_bld)
    call one_routine_catalog ('ea_dist_grad_bywlk_bld', ea_dist_grad_bywlk_bld)
    call one_routine_catalog ('ea_dist_xyz_bywlk_bld', ea_dist_xyz_bywlk_bld)
    call one_routine_catalog ('ea_distance_bywlk_bld', ea_distance_bywlk_bld)
    call one_routine_catalog ('ea_monom_coor_bywlk_bld', ea_monom_coor_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_bywlk_bld', ea_r_bar_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_power_bywlk_bld',ea_r_bar_power_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_beta_der_bywlk_bld', ea_r_bar_beta_der_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_beta_der2_bywlk_bld', ea_r_bar_beta_der2_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_grad_bywlk_bld', ea_r_bar_grad_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_grad_beta_der_bywlk_bld', ea_r_bar_grad_beta_der_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_grad_beta_der2_bywlk_bld', ea_r_bar_grad_beta_der2_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_lapl_bywlk_bld', ea_r_bar_lapl_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_lapl_beta_der_bywlk_bld', ea_r_bar_lapl_beta_der_bywlk_bld)
    call one_routine_catalog ('ea_r_bar_lapl_beta_der2_bywlk_bld', ea_r_bar_lapl_beta_der2_bywlk_bld)
    call one_routine_catalog ('ee_beta_jastrow_bld', ee_beta_jastrow_bld)
    call one_routine_catalog ('ee_dist_grd_bywlk_bld', ee_dist_grd_bywlk_bld)
!    call one_routine_catalog ('ee_dist_histo_bld', ee_dist_histo_bld)
!    call one_routine_catalog ('ee_dist_histo_norm_bld', ee_dist_histo_norm_bld)
    call one_routine_catalog ('ee_dist_xyz_bywlk_bld', ee_dist_xyz_bywlk_bld)
    call one_routine_catalog ('ee_distance_bywlk_bld', ee_distance_bywlk_bld)
    call one_routine_catalog ('ee_r_bar_bywlk_bld',             ee_r_bar_bywlk_bld                    )
    call one_routine_catalog ('ee_r_bar_power_bywlk_bld',       ee_r_bar_power_bywlk_bld              )
    call one_routine_catalog ('ee_r_bar_beta_der_bywlk_bld',  ee_r_bar_beta_der_bywlk_bld         )
    call one_routine_catalog ('ee_r_bar_beta_der2_bywlk_bld', ee_r_bar_beta_der2_bywlk_bld        )
    call one_routine_catalog ('ee_r_bar_grad_bywlk_bld',          ee_r_bar_grad_bywlk_bld             )
    call one_routine_catalog ('ee_r_bar_grad_beta_der_bywlk_bld',  ee_r_bar_grad_beta_der_bywlk_bld     )
    call one_routine_catalog ('ee_r_bar_grad_beta_der2_bywlk_bld', ee_r_bar_grad_beta_der2_bywlk_bld    )
    call one_routine_catalog ('ee_r_bar_lapl_bywlk_bld',            ee_r_bar_lapl_bywlk_bld           )
    call one_routine_catalog ('ee_r_bar_lapl_beta_der_bywlk_bld', ee_r_bar_lapl_beta_der_bywlk_bld)
    call one_routine_catalog ('ee_r_bar_lapl_beta_der2_bywlk_bld', ee_r_bar_lapl_beta_der2_bywlk_bld)
    call one_routine_catalog ('elec_xpfs_rexp_bywlk_bld',elec_xpfs_rexp_bywlk_bld)
    call one_routine_catalog ('elec_xpfs_rexp_grad_bywlk_bld',elec_xpfs_rexp_grad_bywlk_bld)
    call one_routine_catalog ('elec_xpfs_rexp_lapl_bywlk_bld',elec_xpfs_rexp_lapl_bywlk_bld)
!    call one_routine_catalog ('elec_xpfs_rexp_grla_bywlk_bld', elec_xpfs_rexp_grla_bywlk_bld)
    call one_routine_catalog ('electrons_spins_bld', electrons_spins_bld)
    call one_routine_catalog ('epfs_read', epfs_read)
    call one_routine_catalog ('epsilon_grid_bld', epsilon_grid_bld)
    call one_routine_catalog ('force_zv_min_bywlk_bld',force_zv_min_bywlk_bld)

    call one_routine_catalog ('intracule_histo_step_bld',intracule_histo_step_bld)
    call one_routine_catalog ('intracule_histo_block_bld',intracule_histo_block_bld)
    call one_routine_catalog ('intracule_improved1_step_bld',intracule_improved1_step_bld)
    call one_routine_catalog ('intracule_improved2_step_bld',intracule_improved2_step_bld)
    call one_routine_catalog ('intracule_improved_block_bld',intracule_improved_block_bld)
    call one_routine_catalog ('grid_over_rij_bld', grid_over_rij_bld)

    call one_routine_catalog ('grid_coor', grid_coor_bld)
    call one_routine_catalog ('grid_coor_atom_dist', grid_coor_atom_dist_bld)

    call one_routine_catalog ('jastrow_a_ee_1_4_bld', jastrow_a_ee_1_4_bld)
    call one_routine_catalog ('jastrow_a_en_1_4_bld', jastrow_a_en_1_4_bld)
    call one_routine_catalog ('jastrow_arrays_bld',jastrow_arrays_bld)
    call one_routine_catalog ('jastrow_atom_e_beta_bld', jastrow_atom_e_beta_bld)
    call one_routine_catalog ('jastrow_exponent_bywlk_bld', jastrow_exponent_bywlk_bld)
    call one_routine_catalog ('jastrow_exponent_grad_bywlk_bld', jastrow_exponent_grad_bywlk_bld)
    call one_routine_catalog ('jastrow_exponent_lapl_bywlk_bld', jastrow_exponent_lapl_bywlk_bld)
    call one_routine_catalog ('jastrow_factor_bywlk_bld', jastrow_factor_bywlk_bld)
    call one_routine_catalog ('jastrow_factor_grad_bywlk_bld', jastrow_factor_grad_bywlk_bld)
    call one_routine_catalog ('jastrow_factor_lapl_bywlk_bld', jastrow_factor_lapl_bywlk_bld)
! call one_routine_catalog ('jastrow_init', jastrow_init)
!   call one_routine_catalog ('langevin_classical_step_bywlk_bld',langevin_classical_step_bywlk_bld)
    call one_routine_catalog ('list_ecp_atoms_bld', list_ecp_atoms_bld)
    call one_routine_catalog ('mo_at_point_clc', mo_at_point_clc)
!    call one_routine_catalog ('mo_ao_s_coeffs_bld', mo_ao_s_coeffs_bld)
    call one_routine_catalog ('mos_bywlk_bld', mos_bywlk_bld)
    call one_routine_catalog ('mos_coeff_list_read', mos_coeff_list_read)
    call one_routine_catalog ('mos_grad_bywlk_bld', mos_grad_bywlk_bld)
    call one_routine_catalog ('mos_lapl_bywlk_bld', mos_lapl_bywlk_bld)
    call one_routine_catalog ('mos_names_bld', mos_names_bld)
    call one_routine_catalog ('mos_on_aos_c_to_xpfs_c_bld', mos_on_aos_c_to_xpfs_c_bld)
    call one_routine_catalog ('mos_on_aos_s_nrmd_to_xpfs_c_bld', mos_on_aos_s_nrmd_to_xpfs_c_bld)
    call one_routine_catalog ('mos_on_aos_c_nrmd_to_xpfs_c_bld', mos_on_aos_c_nrmd_to_xpfs_c_bld)
    call one_routine_catalog ('mos_overlap_bld', mos_overlap_bld)
    call one_routine_catalog ('mos_to_aos_c_bld', mos_to_aos_c_bld)
    call one_routine_catalog ('mos_to_aos_c_nrmd_bld', mos_to_aos_c_nrmd_bld)
    call one_routine_catalog ('mos_to_aos_s_nrmd_bld', mos_to_aos_s_nrmd_bld)
    call one_routine_catalog ('mos_to_xpfs_c_bld', mos_to_xpfs_c_bld)
    call one_routine_catalog ('mos_to_xpfs_s_bld', mos_to_xpfs_s_bld)
    call one_routine_catalog ('nuclei_init',nuclei_init)
    call one_routine_catalog ('one_ea_distance_bld',one_ea_distance_bld)
    call one_routine_catalog ('one_step_bywlk_bld', one_step_bywlk_bld)
    call one_routine_catalog ('one_walker_write', one_walker_write)
    call one_routine_catalog ('overlap_matrix_bld',overlap_matrix_bld)
    call one_routine_catalog ('overlap_matrix_var_bld',overlap_matrix_var_bld)
    call one_routine_catalog ('overlap_matrix_err_bld',overlap_matrix_err_bld)
    call one_routine_catalog ('pseudo_init', pseudo_init)
    call one_routine_catalog ('psi_bywlk_bld', psi_bywlk_bld)
    call one_routine_catalog ('psi_hf_bywlk_bld', psi_hf_bywlk_bld)
    call one_routine_catalog ('psi_grad_bywlk_bld', psi_grad_bywlk_bld)
    call one_routine_catalog ('psi_lapl_bywlk_bld', psi_lapl_bywlk_bld)
    call one_routine_catalog ('psi_hf_lapl_bywlk_bld', psi_hf_lapl_bywlk_bld)
    call one_routine_catalog ('psi_lapl_over_psi_bywlk_bld', psi_lapl_over_psi_bywlk_bld)
    call one_routine_catalog ('psi_guide_bywlk_bld', psi_guide_bywlk_bld)
    call one_routine_catalog ('psi_guide_grad_bywlk_bld', psi_guide_grad_bywlk_bld)
    call one_routine_catalog ('psi_guide_lapl_bywlk_bld', psi_guide_lapl_bywlk_bld)
    call one_routine_catalog ('psi_ylm_elec_ecp_bywlk_bld',psi_ylm_elec_ecp_bywlk_bld)
    call one_routine_catalog ('Q_grad_bywlk_bld',Q_grad_bywlk_bld)
    call one_routine_catalog ('random_seed_bld', random_seed_bld)
    call one_routine_catalog ('r_quad_atom_mos_grid_bld',r_quad_atom_mos_grid_bld)
    call one_routine_catalog ('sample_bld', sample_bld)    
!    call one_routine_catalog ('slater_basis_bld',slater_basis_bld)
!    call one_routine_catalog ('slater_ortho_basis_bld',slater_ortho_basis_bld)
    call one_routine_catalog (' sum_rij_power_bywlk_bld', sum_rij_power_bywlk_bld)
    call one_routine_catalog (' sum_rij_power_grad_bywlk_bld', sum_rij_power_grad_bywlk_bld)
    call one_routine_catalog (' sum_rij_power_power_bywlk_bld', sum_rij_power_power_bywlk_bld)
    call one_routine_catalog (' sum_rij_power_power_grad_bywlk_bld', sum_rij_power_power_grad_bywlk_bld)
    call one_routine_catalog ('test_eta_bld',test_eta_bld)
    call one_routine_catalog ('u_ee_bywlk_bld',                  u_ee_bywlk_bld                  )
    call one_routine_catalog ('u_ee_a_der_bywlk_bld',          u_ee_a_der_bywlk_bld          )
    call one_routine_catalog ('u_ee_b_der_bywlk_bld',          u_ee_b_der_bywlk_bld          )
    call one_routine_catalog ('u_ee_b_der2_bywlk_bld',         u_ee_b_der2_bywlk_bld         )
    call one_routine_catalog ('u_ee_ab_der2_bywlk_bld',        u_ee_ab_der2_bywlk_bld        )
    call one_routine_catalog ('u_ee_grad_bywlk_bld',             u_ee_grad_bywlk_bld             )
    call one_routine_catalog ('u_ee_grad_a_der_bywlk_bld',     u_ee_grad_a_der_bywlk_bld     )
    call one_routine_catalog ('u_ee_grad_b_der_bywlk_bld',     u_ee_grad_b_der_bywlk_bld     )
    call one_routine_catalog ('u_ee_grad_ab_der2_bywlk_bld',   u_ee_grad_ab_der2_bywlk_bld   )
    call one_routine_catalog ('u_ee_grad_b_der2_bywlk_bld',    u_ee_grad_b_der2_bywlk_bld    )
    call one_routine_catalog ('u_ee_lapl_bywlk_bld',             u_ee_lapl_bywlk_bld             )
    call one_routine_catalog ('u_ee_lapl_a_der_bywlk_bld',     u_ee_lapl_a_der_bywlk_bld     )
    call one_routine_catalog ('u_ee_lapl_b_der_bywlk_bld',     u_ee_lapl_b_der_bywlk_bld     )
    call one_routine_catalog ('u_ee_lapl_b_der2_bywlk_bld',    u_ee_lapl_b_der2_bywlk_bld    )
    call one_routine_catalog ('u_ee_lapl_ab_der2_bywlk_bld',   u_ee_lapl_ab_der2_bywlk_bld   )
    call one_routine_catalog ('u_een_bywlk_bld',                 u_een_bywlk_bld                 )
    call one_routine_catalog ('u_een_a_der_bywlk_bld',         u_een_a_der_bywlk_bld         ) 
    call one_routine_catalog ('u_een_bee_der_bywlk_bld',      u_een_bee_der_bywlk_bld      ) 
    call one_routine_catalog ('u_een_bee_der2_bywlk_bld',     u_een_bee_der2_bywlk_bld     ) 
    call one_routine_catalog ('u_een_ben_der_bywlk_bld',      u_een_ben_der_bywlk_bld      ) 
    call one_routine_catalog ('u_een_ben_der2_bywlk_bld',     u_een_ben_der2_bywlk_bld     ) 
    call one_routine_catalog ('u_een_bee_ben_der2_bywlk_bld',u_een_bee_ben_der2_bywlk_bld) 
    call one_routine_catalog ('u_een_a_ben_der2_bywlk_bld',   u_een_a_ben_der2_bywlk_bld   )   
    call one_routine_catalog ('u_een_a_bee_der2_bywlk_bld',   u_een_a_bee_der2_bywlk_bld   )
    call one_routine_catalog ('u_een_grad_bywlk_bld',                 u_een_grad_bywlk_bld                 )
    call one_routine_catalog ('u_een_grad_a_der_bywlk_bld',         u_een_grad_a_der_bywlk_bld         )
    call one_routine_catalog ('u_een_grad_ben_der_bywlk_bld',      u_een_grad_ben_der_bywlk_bld      )
    call one_routine_catalog ('u_een_grad_bee_der_bywlk_bld',      u_een_grad_bee_der_bywlk_bld      )
    call one_routine_catalog ('u_een_grad_ben_der2_bywlk_bld',     u_een_grad_ben_der2_bywlk_bld     )
    call one_routine_catalog ('u_een_grad_bee_der2_bywlk_bld',     u_een_grad_bee_der2_bywlk_bld     )
    call one_routine_catalog ('u_een_grad_bee_ben_der2_bywlk_bld',u_een_grad_bee_ben_der2_bywlk_bld) 
    call one_routine_catalog ('u_een_grad_a_ben_der2_bywlk_bld'   ,u_een_grad_a_ben_der2_bywlk_bld   )
    call one_routine_catalog ('u_een_grad_a_bee_der2_bywlk_bld'   ,u_een_grad_a_bee_der2_bywlk_bld   )
    call one_routine_catalog ('u_een_lapl_bywlk_bld',                 u_een_lapl_bywlk_bld                 )
    call one_routine_catalog ('u_een_lapl_a_der_bywlk_bld',         u_een_lapl_a_der_bywlk_bld         ) 
    call one_routine_catalog ('u_een_lapl_ben_der_bywlk_bld',      u_een_lapl_ben_der_bywlk_bld      )
    call one_routine_catalog ('u_een_lapl_ben_der2_bywlk_bld',     u_een_lapl_ben_der2_bywlk_bld     )
    call one_routine_catalog ('u_een_lapl_bee_der_bywlk_bld',      u_een_lapl_bee_der_bywlk_bld      )
    call one_routine_catalog ('u_een_lapl_bee_der2_bywlk_bld',     u_een_lapl_bee_der2_bywlk_bld     )
    call one_routine_catalog ('u_een_lapl_a_ben_der2_bywlk_bld'   ,u_een_lapl_a_ben_der2_bywlk_bld   )
    call one_routine_catalog ('u_een_lapl_a_bee_der2_bywlk_bld'   ,u_een_lapl_a_bee_der2_bywlk_bld   )
    call one_routine_catalog ('u_een_lapl_bee_ben_der2_bywlk_bld',u_een_lapl_bee_ben_der2_bywlk_bld) 
    call one_routine_catalog ('u_en_bywlk_bld',                  u_en_bywlk_bld                  )
    call one_routine_catalog ('u_en_a_der_bywlk_bld',          u_en_a_der_bywlk_bld          )
    call one_routine_catalog ('u_en_b_der_bywlk_bld',          u_en_b_der_bywlk_bld          )
    call one_routine_catalog ('u_en_b_der2_bywlk_bld',         u_en_b_der2_bywlk_bld         )
    call one_routine_catalog ('u_en_ab_der2_bywlk_bld',        u_en_ab_der2_bywlk_bld        )
    call one_routine_catalog ('u_en_grad_bywlk_bld',             u_en_grad_bywlk_bld             )
    call one_routine_catalog ('u_en_grad_a_der_bywlk_bld',     u_en_grad_a_der_bywlk_bld     )
    call one_routine_catalog ('u_en_grad_b_der_bywlk_bld',     u_en_grad_b_der_bywlk_bld     )
    call one_routine_catalog ('u_en_grad_b_der2_bywlk_bld',    u_en_grad_b_der2_bywlk_bld    )
    call one_routine_catalog ('u_en_grad_ab_der2_bywlk_bld',   u_en_grad_ab_der2_bywlk_bld   )
    call one_routine_catalog ('u_en_lapl_bywlk_bld',             u_en_lapl_bywlk_bld             )
    call one_routine_catalog ('u_en_lapl_a_der_bywlk_bld',     u_en_lapl_a_der_bywlk_bld     )
    call one_routine_catalog ('u_en_lapl_b_der_bywlk_bld',     u_en_lapl_b_der_bywlk_bld     )
    call one_routine_catalog ('u_en_lapl_b_der2_bywlk_bld',    u_en_lapl_b_der2_bywlk_bld    )
    call one_routine_catalog ('u_en_lapl_ab_der2_bywlk_bld',   u_en_lapl_ab_der2_bywlk_bld   )
    call one_routine_catalog ('walkers_bywlk_write', walkers_bywlk_write)
    call one_routine_catalog ('walkers_bywlk_init',walkers_bywlk_init)
    call one_routine_catalog ('xpfs_c_arrays_bld',xpfs_c_arrays_bld)
!    call one_routine_catalog ('xpfs_c_at_point_grad_lapl_clc', xpfs_c_at_point_grad_lapl_clc)
    call one_routine_catalog ('xpfs_c_bld', xpfs_c_bld)
    call one_routine_catalog ('xpfs_c_gaussian_read', xpfs_c_gaussian_read)
    call one_routine_catalog ('xpfs_c_bywlk_bld', xpfs_c_bywlk_bld)
    call one_routine_catalog ('xpfs_c_grad_bywlk_bld', xpfs_c_grad_bywlk_bld)
    call one_routine_catalog ('xpfs_c_lapl_bywlk_bld', xpfs_c_lapl_bywlk_bld)
!  call one_routine_catalog ('xpfs_c_grla_bywlk_bld', xpfs_c_grla_bywlk_bld)
!  call one_routine_catalog ('xpfs_c_gto_lapl_bywlk_bld', xpfs_c_gto_lapl_bywlk_bld)
    call one_routine_catalog ('xpfs_c_overlap_bld', xpfs_c_overlap_bld)
    call one_routine_catalog ('xpfs_s_bld', xpfs_s_bld)
    call one_routine_catalog ('xpfs_s_to_xpfs_c_bld', xpfs_s_to_xpfs_c_bld)
    call one_routine_catalog ('output_list_init', output_list_init)
    call one_routine_catalog ('targets_ind_bld', targets_ind_bld)
! call one_routine_catalog ('targets_name_bld', targets_name_bld)
! call one_routine_catalog ('targets_name_print', targets_name_print)
    call one_routine_catalog ('expressions_bld', expressions_bld)
    call one_routine_catalog ('expressions_print', expressions_print)
    call one_routine_catalog ('next_expression_get', next_expression_get)

! for guiding wave function JT
     call one_routine_catalog ('alfa_orbitals_guide_init', alfa_orbitals_guide_init)
     call one_routine_catalog ('beta_orbitals_guide_init', beta_orbitals_guide_init)
     call one_routine_catalog ('configurations_guide_init', configurations_guide_init)
     call one_routine_catalog ('conf_jastr_guide_read', conf_jastr_guide_read)
     call one_routine_catalog ('det_alfa_guide_bywlk_bld', det_alfa_guide_bywlk_bld)
     call one_routine_catalog ('det_alfa_guide_grad_bywlk_bld', det_alfa_guide_grad_bywlk_bld)
     call one_routine_catalog ('det_alfa_guide_lapl_bywlk_bld', det_alfa_guide_lapl_bywlk_bld)
     call one_routine_catalog ('det_beta_guide_bywlk_bld', det_beta_guide_bywlk_bld)
     call one_routine_catalog ('det_beta_guide_grad_bywlk_bld', det_beta_guide_grad_bywlk_bld)
     call one_routine_catalog ('det_beta_guide_lapl_bywlk_bld', det_beta_guide_lapl_bywlk_bld)
     call one_routine_catalog ('det_sum_guide_bywlk_bld', det_sum_guide_bywlk_bld)
     call one_routine_catalog ('det_sum_guide_grad_bywlk_bld', det_sum_guide_grad_bywlk_bld)
     call one_routine_catalog ('det_sum_guide_lapl_bywlk_bld', det_sum_guide_lapl_bywlk_bld)
     call one_routine_catalog ('determinants_guide_init', determinants_guide_init)
     call one_routine_catalog ('ea_r_bar_guide_bywlk_bld', ea_r_bar_guide_bywlk_bld)
     call one_routine_catalog ('ea_r_bar_guide_grad_bywlk_bld', ea_r_bar_guide_grad_bywlk_bld)
     call one_routine_catalog ('ea_r_bar_guide_lapl_bywlk_bld', ea_r_bar_guide_lapl_bywlk_bld)
     call one_routine_catalog ('ee_beta_jastrow_guide_bld', ee_beta_jastrow_guide_bld)
     call one_routine_catalog ('ee_r_bar_guide_bywlk_bld', ee_r_bar_guide_bywlk_bld)
     call one_routine_catalog ('ee_r_bar_guide_grad_bywlk_bld', ee_r_bar_guide_grad_bywlk_bld)
     call one_routine_catalog ('ee_r_bar_guide_lapl_bywlk_bld', ee_r_bar_guide_lapl_bywlk_bld)
     call one_routine_catalog ('jastrow_a_ee_1_4_guide_bld', jastrow_a_ee_1_4_guide_bld)
     call one_routine_catalog ('jastrow_a_en_1_4_guide_bld', jastrow_a_en_1_4_guide_bld)
     call one_routine_catalog ('jastrow_arrays_guide_bld',jastrow_arrays_guide_bld)
     call one_routine_catalog ('jastrow_atom_e_beta_guide_bld', jastrow_atom_e_beta_guide_bld)
     call one_routine_catalog ('jastrow_exponent_guide_bywlk_bld', jastrow_exponent_guide_bywlk_bld)
     call one_routine_catalog ('jastrow_exponent_guide_grad_bywlk_bld', jastrow_exponent_guide_grad_bywlk_bld)
     call one_routine_catalog ('jastrow_exponent_guide_lapl_bywlk_bld', jastrow_exponent_guide_lapl_bywlk_bld)
     call one_routine_catalog ('jastrow_factor_guide_bywlk_bld', jastrow_factor_guide_bywlk_bld)
     call one_routine_catalog ('jastrow_factor_guide_grad_bywlk_bld', jastrow_factor_guide_grad_bywlk_bld)
     call one_routine_catalog ('jastrow_factor_guide_lapl_bywlk_bld', jastrow_factor_guide_lapl_bywlk_bld)
     call one_routine_catalog ('time_step_bld', time_step_bld)
     call one_routine_catalog ('u_ee_guide_bywlk_bld', u_ee_guide_bywlk_bld)
     call one_routine_catalog ('u_ee_guide_grad_bywlk_bld', u_ee_guide_grad_bywlk_bld)
     call one_routine_catalog ('u_ee_guide_lapl_bywlk_bld', u_ee_guide_lapl_bywlk_bld)
     call one_routine_catalog ('u_een_guide_bywlk_bld', u_een_guide_bywlk_bld)
     call one_routine_catalog ('u_een_guide_grad_bywlk_bld', u_een_guide_grad_bywlk_bld)
     call one_routine_catalog ('u_een_guide_lapl_bywlk_bld', u_een_guide_lapl_bywlk_bld)
     call one_routine_catalog ('u_en_guide_bywlk_bld', u_en_guide_bywlk_bld)
     call one_routine_catalog ('u_en_guide_grad_bywlk_bld', u_en_guide_grad_bywlk_bld)
     call one_routine_catalog ('u_en_guide_lapl_bywlk_bld', u_en_guide_lapl_bywlk_bld)

! operators
    call one_routine_catalog ('alias',alias_calc)
    call one_routine_catalog ('abs', abs_calc)
    call one_routine_catalog ('array_av',real_1_average_calc)
    call one_routine_catalog ('array_element_calc',array_element_calc)
! call one_routine_catalog ('asin',asin_calc)   
! call one_routine_catalog ('atan',atan_calc)
! call one_routine_catalog ('cos',cos_calc)
    call one_routine_catalog ('cov',cov_calc)
    call one_routine_catalog ('clone', clone_calc)
    call one_routine_catalog ('dif',dif_calc)
    call one_routine_catalog ('div',div_calc)	
    call  one_routine_catalog ('err',err_calc)
    call one_routine_catalog ('equal',equal_set)
    call one_routine_catalog ('exesys',exesys_calc)
    call one_routine_catalog ('exp',exp_calc)	
    call one_routine_catalog ('expo',expo_calc)	
    call one_routine_catalog ('float',float_calc)
    call one_routine_catalog ('gather', gather_calc)
    call one_routine_catalog ('gtos_overlap', gtos_overlap_calc)
    call one_routine_catalog ('gto_sto_overlap', gto_sto_overlap_calc)
    call one_routine_catalog ('Heaviside', heaviside_calc)
    call one_routine_catalog ('history', history_bld)
! call one_routine_catalog ('log',log_calc)
    call one_routine_catalog ('matmult', matmult_calc)	
    call one_routine_catalog ('minus_calc',minus_calc)
    call one_routine_catalog ('real_array',real_array_calc)
    call one_routine_catalog ('real_array_init',real_array_init)
    call one_routine_catalog ('sample_av', sample_av_calc)
    call one_routine_catalog ('sin',sin_calc)
    call one_routine_catalog ('size',size_calc)
    call one_routine_catalog ('slice',slice_calc)
    call one_routine_catalog ('sqrt',sqrt_calc)
    call one_routine_catalog ('time_int', time_int_calc)
    call one_routine_catalog ('log',log_calc)
    call one_routine_catalog ('sum',sum_calc)
! call one_routine_catalog ('tan',tan_calc)

    call one_routine_catalog ('prod',prod_calc)
    call one_routine_catalog ('prod_calc',prod_calc)
    call one_routine_catalog ('prod_calc_2',prod_calc_2, args_nb=3)
    call one_routine_catalog ('dif_calc_2',dif_calc_2,args_nb=3)
    call one_routine_catalog ('div_calc_2',div_calc_2,args_nb=3)
    call one_routine_catalog ('av',average_calc)
    call one_routine_catalog ('graph_calc',graph_calc)
    call one_routine_catalog ('@', list_calc)

   call one_routine_catalog ('clones_roots_bld', clones_roots_bld)
   call one_routine_catalog ('drift_bywlk_2_bld', drift_bywlk_2_bld)
   call one_routine_catalog ('elec1_elec2_dist_bywlk_bld',elec1_elec2_dist_bywlk_bld)
   call one_routine_catalog ('one_step_bywlk_2_bld',one_step_bywlk_2_bld)

    call exit (l_here)

  end subroutine routines_catalog_init

! =================================================================================
  subroutine types_catalog_init 
! ---------------------------------------------------------------------------------
!
! Name           : types_catalog_init
!
! Description    : initializes types_names
! Description    : sees all modules where types are 
!
! -- Creates        : types_name (rtn) 
!
! -- Needs          : types_names (rtn) 
!
! Authors        : F. Colonna 
! Date           : 22 May 2002
! ---------------------------------------------------------------------------------
! include 'modules_uses.inc'
  use setup_mod
  use basic_types_tools_mod
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='types_catalog_init'
  call enter (l_here)

  call types_name_add_once ('real_0')
  call types_name_add_once ('real_1')
  call types_name_add_once ('real_2')
  call types_name_add_once ('real_3')
  call types_name_add_once ('real_4')
  call types_name_add_once ('real_5')
  call types_name_add_once ('real_6')
  call types_name_add_once ('real_7')
  call types_name_add_once ('integer_0')
  call types_name_add_once ('integer_1')
  call types_name_add_once ('integer_2')
  call types_name_add_once ('integer_3')
  call types_name_add_once ('integer_4')
  call types_name_add_once ('integer_5')
  call types_name_add_once ('integer_6')
  call types_name_add_once ('character_1')
  call types_name_add_once ('character_2') 
  call types_name_add_once ('character_3')
  call exit (l_here)

  end subroutine types_catalog_init
