#! /usr/bin/perl
#
# Usage: makemake {<program name> {<QMCMOL_F95 compiler or fc or f77 or cc or c>}}
#
# Generate a Makefile from the sources in the current directory.  The source
# files may be in either C, FORTRAN 77, Fortran 90 or some combination of
# these languages.  If the QMCMOL_F95 compiler specified is cray or parasoft, then
# the Makefile generated will conform to the conventions of these compilers.
# To run makemake, it will be necessary to modify the first line of this script
# to point to the actual location of Perl on your system.
#
# Written by Michael Wester <wester@math.unm.edu> February 16, 1995
# Cotopaxi (Consulting), Albuquerque, New Mexico
#
open(MAKEFILE, "> Makefile");
#
print MAKEFILE "PROG =\t$ARGV[0]\n\n";
#
# Source listing
#
print MAKEFILE "SRCS =\t";
@srcs = <*.f90 *.f95 *.f *.F *.c>;
&PrintWords(8, 0, @srcs);
print MAKEFILE "\n\n";
#
# Object listing
#
print MAKEFILE "OBJS =\t";
@objs = @srcs;
foreach (@objs) { s/\.[^.]+$/.o/ };
&PrintWords(8, 0, @objs);
print MAKEFILE "\n\n";
#
# Define common macros
#
# make
#
print MAKEFILE "all: \$(PROG)\n\n";
print MAKEFILE "\$(PROG): \$(OBJS)\n";
print MAKEFILE "\t\$(QMCMOL_LD) \$(QMCMOL_LDFLAGS) -o \$@ \$(OBJS) \$(QMCMOL_LIBS)\n\n";
#
# make clean
#
print MAKEFILE "clean:\n";
print MAKEFILE "\trm -f \$(PROG) \$(OBJS) *.dif *.lst *.sav *.mod *.F90\n\n";
#
# make depend
#
print MAKEFILE "depend:\n";
print MAKEFILE "\tperl makemake.pl $ARGV[0]\n";
print MAKEFILE "\t/usr/bin/ctags *.f *.f90 *.f95\n";
print MAKEFILE "\tetags -l fortran *.f *.f90 *.f95\n\n";
#
# Make .f95 a valid suffix
#
print MAKEFILE ".SUFFIXES: \n\n";
print MAKEFILE ".SUFFIXES: .f90 .f95 .o .f .c\n\n";
#
# .c -> .o
#
print MAKEFILE ".c.o:\n";
print MAKEFILE "\t\$(QMCMOL_CC) \$(QMCMOL_CFLAGS) -c \$<\n\n";
#
# .f95 -> .o
#
print MAKEFILE ".f95.o:\n";
print MAKEFILE "\t\$(QMCMOL_F95) \$(QMCMOL_F95FLAGS) -c \$<\n\n";
#
# .f90 -> .o
#
print MAKEFILE ".f90.o:\n";
# fpp preprocessor
#print MAKEFILE "\t\$(QMCMOL_PP) \$< > \$*.F90\n\n"; 
#print MAKEFILE "\t\$(QMCMOL_F95) \$(QMCMOL_F95FLAGS) -c \$*.F90\n\n";
# no preprocessor 
print MAKEFILE "\t\$(QMCMOL_F95) \$(QMCMOL_F95FLAGS) -c \$*.f90\n\n";
#
# .f   -> .o
#
print MAKEFILE ".f.o:\n";
print MAKEFILE "\t\$(QMCMOL_F77) \$(QMCMOL_F77FLAGS) -c \$<\n\n";
#
# defast:     
#
print MAKEFILE "defast:\n";
print MAKEFILE "\tperl defast.pl *.f90 *.f95\n";
print MAKEFILE "\trm -f *.o *.mod\n";
print MAKEFILE "\techo \"compiling with options: \$\{QMCMOL_F95FLAGS\}\"\n";
print MAKEFILE "\tmake \"QMCMOL_F95FLAGS=\$\{QMCMOL_F95FLAGS\}\"\n\n";
#
# fast:     
#
print MAKEFILE "fast:\n";
print MAKEFILE "\tperl fast.pl *.f90 *.f95\n";
print MAKEFILE "\trm -f *.o *.mod\n";
print MAKEFILE "\techo \"compiling with options: \$\{QMCMOL_F95FAST\}\"\n";
print MAKEFILE "\tmake \"QMCMOL_F95FLAGS=\$\{QMCMOL_F95FAST\}\"\n\n";
#
# opt:     
#
print MAKEFILE "opt:\n";
print MAKEFILE "\techo \"compiling with options: \$\{QMCMOL_F95FAST\}\"\n";
print MAKEFILE "\tmake \"QMCMOL_F95FLAGS=\$\{QMCMOL_F95FAST\}\"\n\n";
#
# Dependency listings
#
&MakeDependsf95($ARGV[1]);
&MakeDepends("*.f *.F", '^\s*include\s+["\']([^"\']+)["\']');
&MakeDepends("*.c",     '^\s*#\s*include\s+["\']([^"\']+)["\']');
#
# &PrintWords(current output column, extra tab?, word list); --- print words
#    nicely
#
sub PrintWords {
   local($columns) = 78 - shift(@_);
   local($extratab) = shift(@_);
   local($wordlength);
   #
   print MAKEFILE @_[0];
   $columns -= length(shift(@_));
   foreach $word (@_) {
      $wordlength = length($word);
      if ($wordlength + 1 < $columns) {
         print MAKEFILE " $word";
         $columns -= $wordlength + 1;
         }
      else {
         #
         # Continue onto a new line
         #
         if ($extratab) {
            print MAKEFILE " \\\n\t\t$word";
            $columns = 62 - $wordlength;
            }
         else {
            print MAKEFILE " \\\n\t$word";
            $columns = 70 - $wordlength;
            }
         }
      }
   }

#
# &LanguageCompiler(compiler, sources); --- determine the correct language
#    compiler
#
sub LanguageCompiler {
   local($compiler) = &toLower(shift(@_));
   local(@srcs) = @_;
   #
   if (length($compiler) > 0) {
      CASE: {
         grep(/^$compiler$/, ("fc", "f77")) &&
            do { $compiler = "F77"; last CASE; };
         grep(/^$compiler$/, ("cc", "c"))   &&
            do { $compiler = "QMCMOL_CC"; last CASE; };
         $compiler = "QMCMOL_F95";
         }
      }
   else {
      CASE: {
         grep(/\.(f90|f95)$/, @srcs)   && do { $compiler = "QMCMOL_F95"; last CASE; };
         grep(/\.(f|F)$/, @srcs) && do { $compiler = "F77";  last CASE; };
         grep(/\.c$/, @srcs)     && do { $compiler = "QMCMOL_CC";  last CASE; };
         $compiler = "???";
         }
      }
   $compiler;
   }

#
# &toLower(string); --- convert string into lower case
#
sub toLower {
   local($string) = @_[0];
   $string =~ tr/A-Z/a-z/;
   $string;
   }

#
# &uniq(sorted word list); --- remove adjacent duplicate words
#
sub uniq {
   local(@words);
   foreach $word (@_) {
      if ($word ne $words[$#words]) {
         push(@words, $word);
         }
      }
   @words;
   }

#
# &MakeDepends(language pattern, include file sed pattern); --- dependency
#    maker
#
sub MakeDepends {
   local(@incs);
   local($lang) = @_[0];
   local($pattern) = @_[1];
   #
   foreach $file (<${lang}>) {
      open(FILE, $file) || warn "Cannot open $file: $!\n";
      while (<FILE>) {
         /$pattern/i && push(@incs, $1);
         }
      if (defined @incs) {
         $file =~ s/\.[^.]+$/.o/;
         print MAKEFILE "$file: ";
         &PrintWords(length($file) + 2, 0, @incs);
         print MAKEFILE "\n";
         undef @incs;
         }
      }
   }

#
# &MakeDependsf95(f95 compiler); --- FORTRAN 90 dependency maker
#
sub MakeDependsf95 {
   local($compiler) = &toLower(@_[0]);
   local(@dependencies);
   local(%filename);
   local(@incs);
   local(@modules);
   local($objfile);
   #
   # Associate each module with the name of the file that contains it
   #
   foreach $file (<*.f90 *.f95>) {
      open(FILE, $file) || warn "Cannot open $file: $!\n";
      while (<FILE>) {
         /^\s*module\s+([^\s!]+)/i &&
            ($filename{&toLower($1)} = $file) =~ s/\.(f90|f95)$/.o/;
         }
      }
   #
   # Print the dependencies of each file that has one or more include's or
   # references one or more modules
   #
   foreach $file (<*.f90 *.f95>) {
      open(FILE, $file);
      while (<FILE>) {
         /^\s*include\s+["\']([^"\']+)["\']/i && push(@incs, $1);
         /^\s*use\s+([^\s,!]+)/i && push(@modules, &toLower($1));
         }
      if (defined @incs || defined @modules) {
         ($objfile = $file) =~ s/\.(f90|f95)$/.o/;
         print MAKEFILE "$objfile: ";
         undef @dependencies;
         foreach $module (@modules) {
            push(@dependencies, $filename{$module});
            }
         @dependencies = &uniq(sort(@dependencies));
         &PrintWords(length($objfile) + 2, 0,
                     @dependencies, &uniq(sort(@incs)));
         print MAKEFILE "\n";
         undef @incs;
         undef @modules;
         #
         # Cray QMCMOL_F95 compiler
         #
         if ($compiler eq "cray") {
            print MAKEFILE "\t\$(QMCMOL_F95) \$(QMCMOL_F95FLAGS) -c ";
            foreach $depend (@dependencies) {
               push(@modules, "-p", $depend);
               }
            push(@modules, $file);
            &PrintWords(30, 1, @modules);
            print MAKEFILE "\n";
            undef @modules;
            }
         #
         # ParaSoft QMCMOL_F95 compiler
         #
         if ($compiler eq "parasoft") {
            print MAKEFILE "\t\$(QMCMOL_F95) \$(QMCMOL_F95FLAGS) -c ";
            foreach $depend (@dependencies) {
               $depend =~ s/\.o$/.(f90|f95)/;
               push(@modules, "-module", $depend);
               }
            push(@modules, $file);
            &PrintWords(30, 1, @modules);
            print MAKEFILE "\n";
            undef @modules;
            }
         }
      }
   }
