! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module minimize_mod
  use parser_mod
  use strings_mod
  use control_mod

  real(dp) :: valmin
  character(len=max_string_len) :: cmeth = "va10a"
  integer(dpi) :: iter_max =  5000
  contains

! ========================================================================
  subroutine minimize_menu
! ------------------------------------------------------------------------
!
! Name   : minimize_menu
!
! Description : Parameters for the minimization method
!
! Authors:F. Assaraf
! Date  : 03 Mar 2003
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)              :: done

! begin:
  l_here ='minimize_menu'
  call enter (l_here)

  done = .false.
  call next_command_get ('minimize')
  do while (.not.done)
     call next_word_get ('minimize')
     if(trim(cur_word) .eq. 'help') then
        write (logf,*) 'minimize: minimize_menu'
        write (logf,*) 'minimize: syntax'
        write (logf,*) 'minimize: help'
        write (logf,*) 'minimize:   met{hod} = [string]'
        write (logf,*) 'minimize:   string : va10a'
        write (logf,*) 'minimize:   string : simplex'
        write (logf,*) 'minimize:   iter{ations_max} = [integer]'
        write (logf,*) 'minimize:   maximum number of iterations'
        write (logf,*) 'minimize:   end_{min}'
        write (logf,*) 'minimize: example'

     else if (token_is ('meth',cmeth)) then
 
     else if (token_is ('iter', iter_max))  then
 
     else
  
        call menu_exit (l_here, done)

     endif

  enddo
  
  call exit (l_here)

end subroutine minimize_menu

! ========================================================================
  subroutine minimize (par, func, param_opt_print_iter)
! ------------------------------------------------------------------------
!
! Name              : minimize
!
! Argument (in/out) : par
! Argument (in)     : func
! Argument (in)     : param_opt_print_iter
!
! Description       : minimize func 
! Description       : par are the initial parameters
! Description       : cmeth = simplex --> use of amoeba
! Description       : cmeth = default --> va10a
!
! author: R. Assaraf
! date  : 17 Jan 2000
! ------------------------------------------------------------------------
  implicit none

! i/o:
  real(dp) ::  par(:)

  external func
  real (dp ) :: func
  external param_opt_print_iter

! local:
  integer    :: nbpar
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: l_cmeth
  real(dp)   :: xprmt(size(par)),xtemp(size(par))
  integer    :: iprint, i, niter
  real(dp)   :: deps, f

! va10a
  real(dp), pointer :: hess(:),g(:),w(:) 
  real(dp)          :: hh,dfn
  integer           :: iexit,mode
     
! amoeba
  integer           :: arrmin ! function
  real(dp), pointer :: smpver(:,:),smpval(:) 
  integer           :: nbvert,iter,indmin

! begin:
  l_here ='minimize'
  call enter (l_here)

  nbpar = size(par)
 
      deps = .00000001d0

       do i = 1,nbpar
         xtemp(i) = par(i)
       enddo
   
       f = func(xtemp,nbpar)
   
       write (logf,*)trim (l_here),': func (xtemp)=',f
       write (logf,*)trim (l_here),': xtemp = ', (xtemp(i),i= 1,nbpar)
! parametre au debut de la minimisation  
      
      do i = 1,nbpar
      xprmt(i) = dabs(xtemp(i))/100.d0+0.001d0
      enddo

      niter = iter_max*nbpar

   l_cmeth = string_to_lower(cmeth)

   if (l_cmeth(1:7) .eq. 'simplex') then 

! Amoeba:
! nbpar= nb de variables
! nbvert= nb de vertex

   nbvert = nbpar+1
   allocate(smpver(nbvert,nbpar))
   allocate(smpval(nbvert))

   niter = 500*nbpar
       
! creation of the simplex
      do i = 1,nbpar
         xprmt(i) = dabs(xtemp(i))*0.1+0.01d0
      enddo
!     deps= 10.d0*deps
      call INISMPLX (         & 
                     FUNC,    & ! Function
                     smpver,  & ! Vertices matrix at initial Jumps
                     xprmt,   & ! initial jumps
                     smpval,  & ! nbpar+1 values at each vertic row
                     xtemp,   & ! nbpar+1 values at each vertic row
                     nbvert,  & 
                     nbpar)
      iprint = 5*nbpar*iprint
      call AMOEBA (smpver,    & ! input initial vertices (nbpar+1, nbpar)
                  smpval,     & ! nbpar+1 values at each vertic row
                  nbvert,     & 
                  nbpar,      & 
                  nbpar,      &
                  deps,       & ! Fractional tolerance function value
                  FUNC,       & ! Function
                  iter,       & ! Output number of Iterations
                  niter,      & ! Maximum number of iterations
                  iprint,     & ! Print Frequ_ency
                  param_opt_print_iter) ! subroutine writing optimized 
                                        ! parameters at each iteration 
                                        ! of amoeba 
                  
      indmin = arrmin (smpval, nbvert)

      call OUTSMPLX (indmin, & ! input index of minimum
                  smpver,    & ! Vertices matrix at initial Jumps
                  smpval,    & ! Vertices matrix at initial Jumps
                  xtemp,     & ! output
                  valmin,    & ! output
                  nbvert,    & 
                  nbpar)
      
     else  

      allocate(hess(nbpar*(nbpar+1)/2))
      allocate(g(nbpar))
      allocate(w(4*nbpar+1))

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! some data for va10a
      data iexit/0/
      data iprint/1/
      data hh/1.e-5/
      mode = 1
! use va10 for search
      dfn = -.5
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      call va10afunc (func, nbpar, xtemp, valmin, g, hess, w, & 
                      dfn, xprmt, hh, deps, mode, niter,      &
                      iprint, iexit,param_opt_print_iter)
      write (logf,*) trim (l_here), ' optimal parameters: ',(xtemp(i),i= 1,nbpar)
 
  select case(iexit)
     case(1)
        write (logf,*) trim (l_here),': normal exit: minimium found'
        write (logf,*)trim (l_here), ': minimum: ',valmin
     case(3)
        write (logf,*)trim (l_here), ': more iterations required'
     case  default
        write (logf,*)trim (l_here), ': WHAT ARE YOU DOING BUDDY ????  ...'
  end select
      
   endif

     do i = 1,nbpar
        par(i) = xtemp(i)
     enddo

  call exit (l_here)

  end subroutine minimize

end module minimize_mod      
