! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module mo_ao_s_coeffs_mod

  use basic_types_tools_mod
  use mos_mod
  use xpfs_mod

  integer(dpi), pointer :: slater_basis_dim (:)   => NULL()
  real(dp),     pointer :: slater_basis_exp (:,:) => NULL()
  integer(dpi), pointer :: slater_basis_rn  (:,:) => NULL()
  real(dp),     pointer :: z_ort_sla_transfo (:,:,:) => NULL()  
  integer(dpi), pointer :: ao_first_in_xpfs_c (:) => NULL()
 
  real (dp),    pointer :: mo_ao_s_coeffs (:,:,:) => NULL () 
  
  contains 



! =============================================================================
  subroutine mo_ao_s_coeffs_bld
! ----------------------------------------------------------------------------
!
! Name        : mo_ao_s_coeffs_bld
!
!
! Creates     : mo_ao_s_coeffs_bld (mo, ao, sto)
!
! Needs       : mos_to_xpfs_c (mo, xpf_c)
! Needs       : xpfs_c
!
! Author: R. Assaraf
! Date  : 23 Apr 2003 
! -------------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: ao_nb
  integer(dpi)                               :: slater_nb, g_nb, xpfs_c_nb
  integer(dpi)                               :: mo_i, ao_i, xpf_c_i, g_i, s
  integer(dpi) :: center, px, py, pz, z
  real(dp)     :: x, y
  real(dp)     :: expg(20), c(20)
  integer(dpi) :: k, s_1,p
! header:

  if (header_exe) then

    l_here = "mo_ao_s_coeffs_bld"

    call object_create ('mo_ao_s_coeffs')
  
    call object_needed ('ao_first_in_xpfs_c')
    call object_needed ('mos_to_xpfs_c')
    call object_needed ('xpfs_c')
    call object_needed ('slater_basis_exp')
    call object_needed ('z_ort_sla_transfo')
    return

  end if

! begin:
  call enter (l_here) 

! requirements:
   include 'chksiz_mos_to_xpfs_c.inc'

! end requirements

! dimensions:

   mos_nb        = size (mos_to_xpfs_c, dim = 1)
   ao_nb         = size (ao_first_in_xpfs_c)
   slater_nb     = size (slater_basis_exp, dim = 2)
   xpfs_c_nb     = size (xpfs_c)
! allocations
   if (.not.associated (mo_ao_s_coeffs)) then
      call alloc ('mo_ao_s_coeffs',mo_ao_s_coeffs,mos_nb, ao_nb, slater_nb)
   endif

   mo_ao_s_coeffs = 0_dp

  
   call slater_basis_bld

   do mo_i = 1, mos_nb
      do ao_i = 1, ao_nb

         xpf_c_i = ao_first_in_xpfs_c (ao_i)
         center = xpfs_c(xpf_c_i)%center
         px = xpfs_c(xpf_c_i)%x_power
         py = xpfs_c(xpf_c_i)%y_power
         pz = xpfs_c(xpf_c_i)%z_power

         g_i = 0
        do xpf_c_i = 1, xpfs_c_nb 

           if (xpfs_c (xpf_c_i)%center == center          &
                .and. xpfs_c (xpf_c_i)%x_power == px     &
                .and. xpfs_c (xpf_c_i)%y_power == py     &
                .and. xpfs_c (xpf_c_i)%z_power == pz ) then
      
              g_i = g_i +1
              expg (g_i) = xpfs_c(xpf_c_i)%exponents(1) 
              c(g_i)     = mos_to_xpfs_c (mo_i,xpf_c_i)
           endif
        enddo ! do xpf_c_i
        g_nb = g_i
     
        
        z = atoms(center)%charge
        do s = 1, slater_basis_dim (z)
        
           
           do g_i = 1, g_nb
              do s_1 = 1, slater_basis_dim(z)
                 do k= 1, slater_basis_dim (z)
                    x = expg(g_i)
                    y = slater_basis_exp (z,s_1)
                    p = slater_basis_rn (z,s_1)
                    mo_ao_s_coeffs (mo_i,ao_i,s) = mo_ao_s_coeffs (mo_i,ao_i,s) + &
                         z_ort_sla_transfo (z, s,k)*z_ort_sla_transfo (z,s_1,k) * &
                         slater_dot_gauss (x,y,p) * c(g_i)

                 enddo ! do k
              enddo ! do s_1
           enddo ! do g_i
        enddo ! do s
  
     enddo ! do ao_i
  enddo ! do mo_i 
 

  call exit (l_here)
  end subroutine mo_ao_s_coeffs_bld

! ==========================================================================
  subroutine ao_first_in_xpfs_c_bld
! --------------------------------------------------------------------------
!
! Name           : ao_first_in_xpfs_c_bld
!
! Creates        : ao_first_in_xpfs_c
!
! Needs          : xpfs_c (xpf_c)
! Needs          : atoms
! Description    :
! Description    :
!
! Author: R. Assaraf
! Date  : 23 Mar 2002
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)         :: xpfs_c_nb, aos_sto_nb, atoms_nb

  integer(dpi)         ::  xpf_c_i, atom_i, ao
  integer(dpi)  :: p_x,p_y,p_z
! header:

  if (header_exe) then

     l_here = "aos_sto_from_xpfs_c_bld"

     call object_create ( 'ao_first_in_xpfs_c')

     call object_needed ('xpfs_c')
     call object_needed ('atoms')
     return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_xpfs_c.inc'

! dimensions:

  atoms_nb = size(atoms)
  xpfs_c_nb     = size (xpfs_c)

! end dimensions
  if(debug) then
     write (logf,*)trim(l_here),'-d: xpfs_c_nb = ',xpfs_c_nb
  end if

! size of aos_sto_xpfs_c_firs

  aos_sto_nb=0
  do atom_i = 1, atoms_nb
     do p_x = 0,3
        do p_y = 0,3
           do p_z = 0,3
              do xpf_c_i = 1, xpfs_c_nb
                 if (xpfs_c (xpf_c_i)%center == atom_i       &
                      .and. xpfs_c (xpf_c_i)%x_power == p_x  &
                      .and. xpfs_c (xpf_c_i)%y_power == p_y  &
                      .and. xpfs_c (xpf_c_i)%z_power == p_z )  then
                    aos_sto_nb = aos_sto_nb + 1
                    exit
                 endif
              enddo ! do xpf_c_i
           enddo ! do p_z
        enddo ! do p_y
     enddo ! dp p_x
  enddo ! do atom_i

   if (debug) write (logf,*) trim (l_here), '-d: aos_sto_nb=',aos_sto_nb
! allocations:

  if (.not.associated(ao_first_in_xpfs_c)) then
    if(trace_depend) &
         write (logf,*)trim(l_here),': ao_first_in_xpfs_c not associated'
    call alloc ('ao_first_in_xpfs_c',ao_first_in_xpfs_c,aos_sto_nb)
 else
    if(trace_depend) &
         write (logf,*)trim(l_here),':ao_first_in_xpfs_c left in memory'
  end if

! end allocations

 ao = 0
  do atom_i = 1, atoms_nb
     do p_x = 0,3
        do p_y = 0,3
           do p_z = 0,3
              do xpf_c_i = 1, xpfs_c_nb
                 if (xpfs_c (xpf_c_i)%center == atom_i       &
                      .and. xpfs_c (xpf_c_i)%x_power == p_x  &
                      .and. xpfs_c (xpf_c_i)%y_power == p_y  &
                      .and. xpfs_c (xpf_c_i)%z_power == p_z )  then
                    ao = ao+1
                    ao_first_in_xpfs_c (ao) = xpf_c_i
                    exit
                 endif
              enddo ! do xpf_c_i
           enddo ! do p_z
        enddo ! do p_y
     enddo ! dp p_x
  enddo ! do atom_i

 call  exit (l_here)

end subroutine ao_first_in_xpfs_c_bld

! ==========================================================================
  subroutine slater_ortho_basis_bld
! --------------------------------------------------------------------------
!
! Name     :  slater_orho_basis_bld
!
! Creates  : z_ort_sla_transfo (z,ort,sla)
!
! Description :
!
! Author: R. Assaraf
! Date  : 20 Apr 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp), pointer :: slater_metric (:,:)
  integer(dpi) :: max_dim, max_z
  integer(dpi) :: i,j,z, cur_dim
  real(dp)     :: z1,z2
  integer(dpi) :: n1,n2
! header:

  if (header_exe) then

     l_here = "slater_ortho_basis_bld"
     call object_create ('z_ort_sla_transfo')

     call object_needed ('slater_basis_exp')
     call object_needed ('slater_basis_rn')
     call object_needed ('slater_basis_dim')

     return
  endif
  call enter (l_here)

  max_z = size (slater_basis_exp,dim = 1)
  max_dim = size (slater_basis_exp, dim = 2)

  if (.not.associated (z_ort_sla_transfo)) &
       call alloc ('z_ort_sla_transfo',z_ort_sla_transfo,max_z,max_dim,max_dim)

  if (.not.associated (slater_metric)) &
   allocate (slater_metric (max_dim,max_dim))

  do z = 1, max_z
     slater_metric = 0_dp
     cur_dim = slater_basis_dim (z)
     if (cur_dim > 0) then
        do i = 1, cur_dim
           do j= 1, cur_dim
              z1 =slater_basis_exp (z,i)
              z2 =slater_basis_exp (z,j)
              n1=slater_basis_rn (z,i)
              n2=slater_basis_rn (z,j)
          

              slater_metric (i,j) = slater_dot_slater (z1,z2,n1,n2)
! if normalization 
!                / sqrt( slater_dot_slater (a,a,n1,n1) &
!                      * slater_dot_slater (b,b,n2,n2)) 

              if (debug) write(logf,*) trim(l_here),'-d: z=',z, &
                   'slater_metric(',i,',',j,')=', slater_metric (i,j)
           enddo ! do j
        enddo  ! do i

        call ortho_basis_bld (z_ort_sla_transfo (z,1:cur_dim,1:cur_dim),slater_metric(1:cur_dim,1:cur_dim), cur_dim)
     endif
  enddo ! do z
  call exit (l_here)
end subroutine slater_ortho_basis_bld

! ==========================================================================
  subroutine slater_basis_bld
! --------------------------------------------------------------------------
!
! Name     :  slater_basis_bld
!
! Creates  : slater_basis_exp (z, sla)
! Creates  : slater_basis_rn  (z, sla)
!
! Description :
!
! Author: R. Assaraf
! Date  : 20 Apr 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: z
  integer(dpi), parameter :: max_z   = 20
  integer(dpi), parameter :: max_dim = 6
! header:

  if (header_exe) then

    l_here = "slater_basis_bld"

    call object_create ('slater_basis_exp')
    call object_create ('slater_basis_rn')
    call object_create ('slater_basis_dim')
    return

  end if

! begin:
   call enter (l_here)


   if (.not.associated (slater_basis_exp)) &
        call alloc ('slater_basis_exp', slater_basis_exp, max_z, max_dim)

   if (.not.associated (slater_basis_rn)) &
        call alloc ('slater_basis_rn', slater_basis_rn, max_z,max_dim)

   if (.not.associated (slater_basis_dim)) &
        call alloc ('slater_basis_dim',slater_basis_dim,max_z)

   slater_basis_exp = 0_dp
   slater_basis_rn = 0
   slater_basis_dim = 0
! H
   z = 1
   slater_basis_dim (z) = 3
   slater_basis_exp (z,1:3) = (/1.d0,0.7d0,0.4d0/)

! He
   z = 2
   slater_basis_dim (z) = 5
   slater_basis_exp (z,1:5) = (/1.41715d0,2.37682d0,4.39628d0,6.52699d0,7.94252d0/)
! Li
   z = 1
   slater_basis_dim (z) = 6
   slater_basis_exp (z,1:6) = (/2.47673d0,4.69873d0,0.38350d0,0.66055d0,1.07000d0,2.71830d0/)

! C
   z = 6
   slater_basis_dim (z) = 6
   slater_basis_exp (z,1:6) = (/5.43599d0,9.48256d0, 1.05749d0, 1.52427d0, 2.68435d0, 4.20096d0/)

 call exit (l_here)

end  subroutine slater_basis_bld

end module mo_ao_s_coeffs_mod
