! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module montecarlo_menu_mod

  use parser_mod
  use step_bywlk_mod
  use targets_mod   

  implicit none
 
  contains

! ========================================================================
  subroutine montecarlo_menu
! ------------------------------------------------------------------------
!
! Name           : montecarlo_menu
!
! Author         : X. Krokidis, F. Colonna
! Date           : 19 Oct 2000
! Revision: 12 Feb 2002 by F. Colonna 
! ------------------------------------------------------------------------
  implicit none

! local:
  logical(bool)                              :: done
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: outl_i 

! begin:
  l_here ='montecarlo_menu'
  call enter (l_here)

! initialize:
  outl_i = 0 ! counts output commands
  runtype = NOT_INIT_S
  blocks_nb = NOT_INIT_I  
  steps_per_block_nb = NOT_INIT_I
  call provide_bynam ("time_step")

  done = .false.
  call next_command_get ('montecarlo')
  do while (.not.done)
   call next_word_get('montecarlo')
   if(trim(cur_word) .eq. 'help') then
    write (logf,*) 'mon-h: comments:'
    write (logf,*) 'mon-h:  a calculation builds a sample of "block" blocks.'
    write (logf,*) 'mon-h:  block: consists in "step" steps.'
    write (logf,*) 'mon-h:  step:  produces objects defined in output command'
    write (logf,*) 'mon-h:  sample: averages of "step" steps on each block.'
    write (logf,*) 'mon-h: syntax:'
    write (logf,*) 'mon-h: montecarlo'
    write (logf,*) 'mon-h:   help'
    write (logf,*) 'mon-h:   blo{ck}    = [integer] number of blocks'
    write (logf,*) 'mon-h:   out{put}     [command] (repeat) expressions calculated'
    write (logf,*) 'mon-h:   run{type}  = [string] VMC:DMC:SRMC:G_OPT'
    write (logf,*) 'mon-h:   ste{p}     = [integer] number of steps per block'
    write (logf,*) 'mon-h:   tim{estep} = [real] '
    write (logf,*) 'mon-h:   walkers    = [integer]'
    write (logf,*) 'mon-h:   {rec}onf_last_block = [bool] reconfiguring walkers in the last block '
    write (logf,*) 'mon-h:   end_mon'
    write (logf,*) 'mon-h: example'
    write (logf,*) 'mon-h: mont bloc = 3 step = 2 run =vmc tim =0.001'
    write (logf,*) 'mon-h:      out  list=(energy, err(energy) ...) end_out'
    write (logf,*) 'mon-h:      end_mon'

   else if(cur_word(1:3) == 'out')      then
     outl_i = outl_i + 1
     call montecarlo_output_menu (outl_i)

   else if(token_is ('blo', blocks_nb)) then

   else if(token_is ('run', runtype))   then

   else if(token_is ('ste', steps_per_block_nb)) then

   else if(token_is ('tim', time_step)) then
  
   else if (token_is ('wal',system%walkers_nb)) then

   else if (token_is ('rec',reconf_last_block)) then
 
   else
      call menu_exit (l_here, done)
   end if

  end do
 
  if (runtype/=NOT_INIT_S) then  
  write (logf,*) trim(l_here), ': runtype          >',trim(runtype),'<'
  write (logf,*) trim(l_here), ': number of blocks= ', blocks_nb
  write (logf,*) trim(l_here), ': steps_per_block = ', steps_per_block_nb
  write (logf,*) trim(l_here), ': time_step       = ', time_step
  write (res,'(2a)') blanks(1:20),' ____________________________________'
  write (res,'(2a)') blanks(1:20),'I                                    I' 
  write (res,'(2a)') blanks(1:20),'I    Montecarlo calculation          I'
  write (res,'(2a)') blanks(1:20),'I____________________________________I'
  write (res,'(2a)') blanks(1:20),'**************************************'
  write (res,*) 
  write (res,'(3a)')   'Method                    >',trim(runtype),'<'
  write (res,'(a,i4)') 'Number of blocks           ', blocks_nb
  write (res,'(a,i4)') 'Number of steps per block  ', steps_per_block_nb
  write (res,'(a,i4,a,i4,a,i7)') 'Total number of steps      ', &
       blocks_nb,'*',steps_per_block_nb,'=',blocks_nb*steps_per_block_nb
  write (res,'(a,f13.6)')        'Time step                  ', time_step

! output targets. execute
  call node_exe_byrtn_name ("sample_bld")
  endif

  call exit (l_here)

  end subroutine montecarlo_menu

! ========================================================================
  subroutine montecarlo_output_menu (outl_i)
! ------------------------------------------------------------------------
!
! Name           : montecarlo_output_menu
!
! Author         : F. Colonna
! Date           : 29 Oct 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: outl_i
  
! local:
  logical(bool)                              :: done
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: wlk_i, expr_i
  integer(dpi)                               :: bloc_out_freq
  integer(dpi)                               :: step_out_freq
  logical(bool)                              :: qmc_print
  logical(bool)                              :: qmc_write
  character(max_string_len_input), pointer   :: list_of_string (:)
! begin:
  l_here ='montecarlo_output_menu'
  call enter (l_here)

! initialize:
  bloc_out_freq = 1
  step_out_freq = 1

  call provide_bynam ('output_list_expressions')
  call provide_bynam ('output_list_bloc_freq')
  call provide_bynam ('output_list_step_freq')
  call provide_bynam ('output_list_walkers')

  done = .false.
  call next_command_get ('output')
  do while (.not.done)
   call next_word_get('output')
   if(trim(cur_word) .eq. 'help') then
    write (logf,*) 'mon-out-h: comments:'
    write (logf,*) 'mon-out-h:  defines expressions to be output'
    write (logf,*) 'mon-out-h:  at some block and/or step frequency'
    write (logf,*) 'mon-out-h:  and for a selection of walkers'
    write (logf,*) 'mon-out-h: syntax:'
    write (logf,*) 'mon-out-h: out{put}'
    write (logf,*) 'mon-out-h:   help'
    write (logf,*) 'mon-out-h:   blo{ck}   = [integer] (blocksoutput frequency)'
    write (logf,*) 'mon-out-h:   lis{e}    = [list of expressions] (output objects)'
    write (logf,*) 'mon-out-h:   pri{nt}   = [boolean]'
    write (logf,*) 'mon-out-h:   ste{p}    = [integer] (steps output frequency)'
    write (logf,*) 'mon-out-h:   wri{te}   = [boolean]'
    write (logf,*) 'mon-out-h:   wal{kers} = [all|list of integers] '
    write (logf,*) 'mon-out-h:   end_out'
    write (logf,*) 'mon-out-h: example'
    write (logf,*) 'mon-out-h: output'
    write (logf,*) 'mon-out-h:  list = ( sum (e_loc_tot), avr (e_loc_tot_bywlk))'
    write (logf,*) 'mon-out-h:    print = true'
    write (logf,*) 'mon-out-h:    block = 5'
    write (logf,*) 'mon-out-h:    walk  = all'
    write (logf,*) 'mon-out-h:    end_out'
    write (logf,*) 'mon-out-h:  list = ( sum (e_loc_tot), cov (e_loc_tot_bywlk, weight_bywlk))'
    write (logf,*) 'mon-out-h:    write = true'
    write (logf,*) 'mon-out-h:    step  = 2'
    write (logf,*) 'mon-out-h:    walk  = (1, 3, 7)'
    write (logf,*) 'mon-out-h:    end_out'

   else if(token_is ('blo', bloc_out_freq))   then

     output_list_step_freq (outl_i) = bloc_out_freq

   else if(token_is ('lis', list_of_string)) then 
     
     if(debug) then
       write (logf,*) trim(l_here),'-d: outl_i=',outl_i
       write (logf,*) trim(l_here),'-d: size(list_of_string)=',size(list_of_string)
     end if

     if(size(list_of_string) <= size (output_list_expressions, 2)) then
       do expr_i = 1, size (list_of_string)
         output_list_expressions (outl_i, expr_i)  = list_of_string (expr_i)
         write (logf,*) trim(l_here),'-d: list_of_string (',expr_i,') = >', &
                  trim(list_of_string(expr_i)),'<'
       end do
     else
       message ='in routine'*l_here*' nb of expressions=' &
               * size(list_of_string) &
               *' exceeds array 2nd dimension=' &
               + size (output_list_expressions, 2)
       call require &
       (message, size(list_of_string) <= size (output_list_expressions, 2))
     end if

   else if(token_is ('pri', qmc_print))   then

   else if(token_is ('ste', step_out_freq)) then

     output_list_step_freq (outl_i) = step_out_freq

   else if(token_is ('wal', list_of_integer))   then

     if(size(list_of_integer) <= size (output_list_walkers, 2)) then
       do wlk_i = 1, size (list_of_integer)
         output_list_walkers (outl_i, wlk_i) = list_of_integer (wlk_i)
       end do
     else
       message ='in routine'*l_here*' nb of walkers=' &
               * size(list_of_integer) &
               *' exceeds array 2nd dimension=' &
               +size (output_list_walkers, 2)
       call require &
       (message, size(list_of_integer) <= size (output_list_walkers, 2))
     end if

   else if(token_is ('wri', qmc_write))   then

   else
      call menu_exit (l_here, done)
   end if

  end do
    
  message ='in routine'*l_here*' block output frequency is undefined'
  call ensure (message, bloc_out_freq /= NOT_INIT_I)

  message ='in routine'*l_here*' step output frequency is undefined'
  call ensure (message, step_out_freq /= NOT_INIT_I)

  write (logf,*) trim(l_here), ': output done every ',bloc_out_freq,' blocks'
  write (logf,*) trim(l_here), ': output done every ',step_out_freq,' steps'
  write (logf,*) trim(l_here), ': expression list # ',outl_i
  write (logf,*) trim(l_here), ': expression list first word # ',output_list_wordbeg(outl_i)
  write (logf,*) trim(l_here), ': expression list last  word # ',output_list_wordend(outl_i)

  do expr_i = 1, size(output_list_expressions, 2)
    if (output_list_expressions (outl_i, expr_i) /= NOT_INIT_S) then
      write (logf,*) trim(l_here), ': expression >',  &
               trim(output_list_expressions (outl_i, expr_i)),'<'
    end if
  end do
  
  if (associated(output_list_walkers)) then
     do wlk_i = 1, size(output_list_walkers, 2)
        write (logf,*) trim(l_here), ': for walker # ',output_list_walkers (outl_i, wlk_i)
     end do
  endif
  call exit (l_here)

  end subroutine montecarlo_output_menu

end module montecarlo_menu_mod
