! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module mos_mod

  use basic_types_tools_mod
  use aos_mod
  use epfs_mod
  use files_menu_mod
  use parser_mod
  use strings_mod
  use tools_mod
  use objects_paths_mod

! Revision: 13 Fev 2001: stemp, string => local
! Revision: 21 Sep 2001 by F.C.C n_mos => mos_nb
  implicit none

! Description : data
! Description : mos_on_what 'aos_c_nrmd' 'aos_s_nrmd' 'sos_nrmd'

  integer(dpi), parameter :: max_len_mos_basis_type = 15
  character(len=max_len_mos_basis_type) :: mos_on_what       = "?" !NOT_INIT_S
  character(len=max_len_mos_basis_type) :: mos_write_on_what = "?" !NOT_INIT_S

  integer(dpi), parameter :: max_len_mos_names = 14
  integer(dpi), parameter :: max_len_mos_spins = 8

! Description : mos_names(mo)
  character(len=max_len_mos_names), pointer :: mos_names(:)  => NULL()

! Description : mos_spin(mo)
  character(len=max_len_mos_spins), pointer :: mos_spins(:)  => NULL()

! Description : mos_occupations(mo)
  real(dp), pointer :: mos_energies(:)             => NULL()

! Description : mos_occupations(mo)
  real(dp), pointer :: mos_occupations(:)          => NULL()

! Description : mos_to_aos_c(mo, ao)
  real(dp), pointer :: mos_to_aos_c(:,:)        => NULL()

! Description : mos_overlap(mo, mo)
  real(dp), pointer :: mos_overlap(:,:)            => NULL()

! Description : mos_to_aos_c_nrmd(mo, ao)
  real(dp), pointer :: mos_to_aos_c_nrmd(:,:) => NULL()

! Description : mos_to_aos_s_nrmd(mo, ao)
  real(dp), pointer :: mos_to_aos_s_nrmd(:,:) => NULL()

! Description : mos_to_xpfs_c(mo,xp) 
  real(dp), pointer :: mos_to_xpfs_c(:,:)       => NULL()

! Description : mos_to_xpfs_s(mo,xp) 
  real(dp), pointer :: mos_to_xpfs_s(:,:)       => NULL()

! Description : mos_coeff_list(mos*aos) working array
  real(dp), pointer :: mos_coeff_list(:)           => NULL()

! local
  integer(dpi), pointer :: max_ao(:)               => NULL()

! Revision: 30 Nov 2001 by F. Colonna added mos_occupations
! Revision: 30 Nov 2001 by F. Colonna added mos_spins  
! Revision: 30 Nov 2001 by F. Colonna added mos_energies
! Revision: 15 Oct 2002 F. Colonna mos_write_on_what 

  contains

! ============================================================================
  subroutine mos_names_bld
! ----------------------------------------------------------------------------
!
! Name           : mos_names_bld
!
! Description    : builds Molecular Orbitals 
! Description    : dispatches building of Mos according to mos_on_what
!
! Creates        : mos_names (mo)
!
! Authors:F.Colonna
! Date   :26 Sep 2001
! Revision: 16 Nov 2001 by F. Colonna aos_c_nrmd  
! Revision: 30 Nov 2001 by F. Colonna molpro
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mo
  integer(dpi)                               :: mos_nb

! header:

  if (header_exe) then

    l_here = "mos_names_bld"
    description = 'MOs names array (string)'
    call object_create ('mos_names')

    return

  end if

! begin:
  call enter (l_here)

  if(interfaced_program(1:1) == '?') then
    message = 'in routine'*l_here*' interfaced_program is not defined' &
    +CR*'should be one of "GAMESS" "GAUSSIAN" "MOLPRO" "QMCMOL"'
    call require (message, interfaced_program(1:1) /= '?')
  end if

  if(mos_on_what(1:1) == '?') then
   call mos_on_what_read 
  end if

  if(debug) then
    write (logf,*)trim(l_here),'-d: interfaced_program >',trim(interfaced_program),'<'
    write (logf,*)trim(l_here),'-d: mos_on_what >',trim(mos_on_what),'<'
  end if

  select case (mos_on_what)

  case ("aos_c")

    call provide_bynam ('mos_to_aos_c')
    call provide_bynam ('aos_c_names')

    include 'chksiz_mos_to_aos_c.inc'
    include 'chksiz_aos_c_names.inc'

    mos_nb = size (mos_to_aos_c,1)

    call alloc ('mos_names', mos_names, mos_nb)

    write (logf,'(1x,2a,i3,a)') &
    trim(l_here),': ',mos_nb,' mos on unnormed cartesian aos'

! get approximate symmetry from max coeff on aos
    if(associated(max_ao)) then
     message ='object >max_ao< already associated. will be deallocated'
    else
     message ='object >max_ao< not associated. will be allocated'
    end if
    write (logf,*)trim(l_here),"-w: ",trim(message)

    call allocation ('max_ao', max_ao, mos_nb)

    max_ao = maxloc(abs(mos_to_aos_c),dim = 2)

    do mo = 1, mos_nb
     write(mos_names(mo),'(5a,i3.3)') &
    'MO_',aos_c_names(max_ao(mo))(symm_beg:symm_end),'_', &
          aos_c_names(max_ao(mo))(symcomp_beg:symcomp_end),'_',mo
    if(debug) write (logf,*)trim(l_here),'-d: mo # ',mo,' >',trim(mos_names(mo)),'<'
    end do

    deallocate(max_ao)

  case ("aos_c_nrmd")

    call provide_bynam ('mos_to_aos_c_nrmd')
    call provide_bynam ('aos_c_names')

    include 'chksiz_mos_to_aos_c_nrmd.inc'
    include 'chksiz_aos_c_names.inc'

    mos_nb = size (mos_to_aos_c_nrmd,1)

! require_mos_c: 
    if(mos_nb <= 0) call die (l_here, ' nb of mos ='+mos_nb)

    call alloc ('mos_names', mos_names, mos_nb)

    write (logf,'(1x,2a,i3,a)') &
    trim(l_here),': ',mos_nb,' mos on normed cartesian aos'

! get approximate symmetry from max coeff on aos
    if(associated(max_ao)) then
     message ='object >max_ao< already associated. will be deallocated'
    else
     message ='object >max_ao< not associated. will be allocated'
    end if
    write (logf,*)trim(l_here),"-w: ",trim(message)

    call allocation ('max_ao', max_ao, mos_nb)

    max_ao = maxloc(abs(mos_to_aos_c_nrmd),dim = 2)

    do mo = 1, mos_nb
     write(mos_names(mo),'(5a,i3.3)') &
    'MO_',aos_c_names(max_ao(mo))(symm_beg:symm_end),'_', &
          aos_c_names(max_ao(mo))(symcomp_beg:symcomp_end),'_',mo
    end do

    deallocate(max_ao)

  case ("aos_s_nrmd")

    call provide_bynam ('mos_to_aos_s_nrmd')
    include 'chksiz_mos_to_aos_s_nrmd.inc'

    mos_nb = size (mos_to_aos_s_nrmd,1)

    call alloc ('mos_names', mos_names, mos_nb)

    write (logf,'(1x,2a,i3,a)') &
    trim(l_here),': ',mos_nb,' mos on normed spherical aos'

! get approximate symmetry from max coeff on aos
    call allocation ('max_ao', max_ao, mos_nb)

    max_ao = maxloc(abs(mos_to_aos_s_nrmd),dim = 2)

    do mo = 1, mos_nb
     write(mos_names(mo),'(5a,i3.3)') &
    'MO_',aos_s_names(max_ao(mo))(symm_beg:symm_end),'_', &
          aos_s_names(max_ao(mo))(symcomp_beg:symcomp_end),'_',mo
    end do

    deallocate(max_ao)

  case ("xpfs_c")

    call provide_bynam ('mos_to_xpfs_c')
    include 'chksiz_mos_to_xpfs_c.inc'

    mos_nb = size (mos_to_xpfs_c,1)

    call alloc ('mos_names', mos_names, mos_nb)

    write (logf,'(1x,2a,i3,a)') &
    trim(l_here),': ',mos_nb,' mos on un-normed cartesian xpfs'

! get approximate symmetry from max coeff on aos
    call allocation ('max_ao', max_ao, mos_nb)

    max_ao = maxloc(abs(mos_to_xpfs_c),dim = 2)

    do mo = 1, mos_nb
     write(mos_names(mo),'(5a,i3.3)') &
    'MO_',xpfs_c(max_ao(mo))%name(symm_beg:symm_end),'_', &
          xpfs_c(max_ao(mo))%name(symcomp_beg:symcomp_end),'_',mo
    end do

    deallocate(max_ao)

  case default
    message ='unknown mos_on_what >'+mos_on_what+'<' &
    +CR+' should be one of "aos_s_nrmd" or "aos_c_nrmd"'
    call die (l_here, message)
  end select

  if(debug) then
    do mo = 1, mos_nb
     write (logf,*)trim(l_here),'-d: mo # ',mo,' >',mos_names(mo),'<'
    end do
  end if

  call exit (l_here)

  end subroutine mos_names_bld 

! ============================================================================
  subroutine mos_overlap_print
! ----------------------------------------------------------------------------
!
! Name           : mos_overlap_print
!
! Description    : prints out non-zero  Overlaps of Molecular Orbitals
!
! Authors:F.Colonna
! Date  :14 Dec 2001
! Revision: 07 Oct 2002 F. Colonna
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: e, f
  integer(dpi)                               :: mos_nb

! begin:
  l_here = "mos_overlap_print"
  call enter (l_here)

  call provide_bynam ('mos_overlap')

  mos_nb = size (mos_overlap, 1)

  if(debug) then
    write (logf,*)trim(l_here),'-d: nb of mos =',mos_nb
  end if

! require_mos: 
  if(mos_nb <= 0) call die (l_here, ' nb mos ='+mos_nb)

  write (logf,*)trim(l_here),': non-zero mos self-overlaps'
    
  do e = 1, mos_nb
     write (logf,'(1x,2a,i3,a,i3,a,d25.15)') &
     trim(l_here),': mo # ',e,' mo # ',e,' = ',mos_overlap (e,e)
  end do

  write (logf,'(1x,2a)')trim(l_here),': non-zero mos cross-overlaps'
  do e = 1, mos_nb
    do f = e+1, mos_nb
     if(abs(mos_overlap (e,f)) > 1.d-15) then
       write (logf,'(1x,2a,i3,a,i3,a,d25.15)') &
       trim(l_here),': mo # ',e,' mo # ',f,' = ',mos_overlap (e,f)
     end if
    end do
  end do

  call exit (l_here)

  end subroutine mos_overlap_print 

! ============================================================================
  subroutine mos_print
! ----------------------------------------------------------------------------
!
! Name           : mos_print
!
! Description    : prints Molecular Orbitals
!
! Authors:F.Colonna
! Date  :13 Oct 2000
! Revision: 25 Sep 2001 by F. Colonna
! Revision: 16 Nov 2001 by F. Colonna aos_c_nrmd  
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: prev_shell
  integer(dpi)                               :: mo, ao
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: xpf_i,  xpfs_c_nb

! begin:
  l_here = "mos_print"
  call enter (l_here)

  call provide_bynam ('mos_names')

  mos_nb = size (mos_names)

  if(debug) then
    write (logf,*)trim(l_here),'-d: interfaced_program >',trim(interfaced_program),'<'
    write (logf,*)trim(l_here),'-d: mos_on_what >',trim(mos_on_what),'<'
    write (logf,*)trim(l_here),'-d: nb of mos =',mos_nb
  end if

  include 'chksiz_mos_names.inc'

  prev_shell = NOT_INIT_S

  select case (mos_on_what)

  case ("aos_c")

    write (logf,*) trim(l_here),': ',mos_nb,' mos on un-normalized cartesian aos'
 
    call provide_bynam ('aos_c_names')
    call provide_bynam ('mos_to_aos_c')

! require_aos: 
    aos_nb = size (mos_to_aos_c, 2)
    if(aos_nb <= 0) call die (l_here, ' nb of cartesian aos ='+aos_nb)
    
! log file:
    do mo = 1, mos_nb
     write (logf,'(a)')' '
     do ao = 1, aos_nb
       write (logf,'(1x,2a,i3,a,i3,3a,d25.15)') trim(l_here),': mo # ',mo, &
       ' un-ao # ',ao,' >',aos_c_names(ao),'< coeff =',mos_to_aos_c(mo,ao)
     end do
    end do
     
! results file:
    message = 'molecular orbitals on cartesian un-normalized aos'
    call mos_print_res (message, mos_names, mos_to_aos_s_nrmd, aos_s_names)

  case ("aos_c_nrmd")

    write (logf,*)trim(l_here),': ',mos_nb,' mos on normalized cartesian aos'
 
		call provide_bynam ('aos_c_names')

!   require_aos: 
    aos_nb = size (mos_to_aos_c_nrmd, 2)
    if(aos_nb <= 0) call die (l_here, ' nb of cartesian aos ='+aos_nb)
    
    do mo = 1, mos_nb
     do ao = 1, aos_nb
       write (logf,'(1x,2a,i3,a,i3,3a,d25.15)')   &
       trim(l_here),': mo # ',mo,' n-ao # ',ao,' >',aos_c_names(ao), &
                    '< coeff =',mos_to_aos_c_nrmd(mo,ao)
     end do
    end do

! results file:
    message = 'molecular orbitals on cartesian normalized aos'
    call mos_print_res (message, mos_names, mos_to_aos_c_nrmd, aos_c_names)

  case ("aos_s_nrmd")

		call provide_bynam ('aos_s_names')

!   require_aos: 
    aos_nb = size (mos_to_aos_s_nrmd, 2)
    if(aos_nb <= 0) call die (l_here, ' nb of spherical aos ='+aos_nb)
    
    write (logf,*)trim(l_here),': ',mos_nb,' mos on normalized spherical aos'
  
    do mo = 1, mos_nb

     write (logf,'(a)')' '
     write (logf,'(1x,2a,i3,3a)') trim(l_here),': mo # ',mo,' >',mos_names(mo),'<'
                 
     do ao = 1, aos_nb

       if (abs(mos_to_aos_s_nrmd(mo,ao)) > 1.d-15) then
         if(aos_s_names(ao)(r_power_beg:symm_end) /= prev_shell) write (logf,'(a)')' '
         write (logf,'(1x,2a,i3,a,i3,3a,d25.15)')trim(l_here),': mo # ',mo, &
         ' ao # ',ao,' >',aos_s_names(ao),'< coeff =',mos_to_aos_s_nrmd(mo,ao)
         prev_shell = aos_s_names(ao)(r_power_beg:symm_end)

       end if

     end do
    end do

! results file:
    message = 'molecular orbitals on spherical normalized aos'
    call mos_print_res (message, mos_names, mos_to_aos_s_nrmd, aos_s_names)

  case ("xpfs_c")

		call provide_bynam ('xpfs_c')

!   require_xpfs: 
    xpfs_c_nb = size (mos_to_xpfs_c, 2)
    if(xpfs_c_nb <= 0) call die (l_here, ' nb of un-normed xpfs_c ='+xpfs_c_nb)
    
    write (logf,*)trim(l_here),': ',mos_nb,' mos on un-normalized cartesian xpfs'
  
    do mo = 1, mos_nb

     write (logf,'(a)')' '
     write (logf,'(1x,2a,i3,3a)') trim(l_here),': mo # ',mo,' >',mos_names(mo),'<'
                 
     do xpf_i = 1, xpfs_c_nb

       if (abs(mos_to_xpfs_c(mo, xpf_i)) > 1.d-15) then
         if(xpfs_c(xpf_i)%name(r_power_beg:symm_end) /= prev_shell) write (logf,'(a)')' '

         write (logf,'(1x,2a,i3,a,i3,3a,d25.15)')trim(l_here),': mo # ',mo, &
         ' xpf_c # ',xpf_i,' >',xpfs_c(xpf_i)%name,'< coeff =',mos_to_xpfs_c (mo, xpf_i)
         prev_shell = xpfs_c(xpf_i)%name(r_power_beg:symm_end)

       end if

     end do
    end do

! results file:
    message = 'molecular orbitals on cartesain un-normalized xpfs'
    call mos_print_res (message, mos_names, mos_to_xpfs_c, xpfs_c%name)

  case default

    message ='unknown mos_on_what >'+mos_on_what+'<'
    call die (l_here, message)

  end select

  call exit (l_here)

  end subroutine mos_print 

! ===============================================================================
  subroutine mos_print_res (text, mos_names, mos_coeff, aos_names)
! -------------------------------------------------------------------------------
!
! Name           : mos_print_res
!
! Argument (in ) : text
! Argument (in ) : mos_names (mo)
! Argument (in ) : mos_coeff (mo, ao)
! Argument (in ) : aos_names (ao)
!
! Description    : pretty mos printing
!
! Authors:F.Colonna
! Date  :23 Dec 2002
! -------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: text   
  character(len=*)                           :: mos_names (:)
  character(len=*)                           :: aos_names (:)
  real(dp)                                   :: mos_coeff (:,:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string  
  integer(dpi)                               :: mo, ao, debut, fin
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_nb

! begin:
  l_here ='mos_print_res'
  call enter (l_here)

  debut  = 40 - (len_trim(text)/2)
  fin    = 40 + (len_trim(text)/2)

  if(debug) then
    write (logf,*) trim(l_here),'-d: text >',trim(text),'<'
    write (logf,*) trim(l_here),'-d: debut=', debut,' fin=',fin
  end if

  mos_nb = size (mos_names)
  aos_nb = size (aos_names)

  string                 = dashes
  string (debut-1:debut) = " " 
  string (debut:)        = trim(text)//' '//dashes

! message to log file
  write (logf,'(1x,2a,a80)') trim(l_here),': ',string

  write (res,'(a)') ' ' 
  write (res,'(a80)') string 
  write (res,'(a)') ' ' 
  write (res,'(t1,a,       t21,a,      t64,a)') &
              'mo # name', 'ao # name','coefficient'
  write (res,'(a)') ' ' 

  do mo = 1, mos_nb
    write(res,'(i4,1x,a)') mo, mos_names (mo)
    do ao = 1, aos_nb
      if (abs(mos_coeff (mo,ao)) > 1.d-15) then
        write(res,'(t21,i4, t26,a, t50,f25.15)') &
                    ao, aos_names (ao), mos_coeff (mo,ao)
      end if
    end do
    write (res,'(a)') ' ' 
  end do

  call exit (l_here)

  end subroutine mos_print_res

! ============================================================================
  subroutine mos_to_aos_c_nrmd_bld 
! ----------------------------------------------------------------------------
!
! Name        : mos_to_aos_c_nrmd_bld
!
! Description : get cartesian normed mos from mos_coeff_list
!
! Creates     : mos_to_aos_c_nrmd (mo, ao_c)
!
! Needs       : aos_c_names (ao_c)
! Needs       : mos_coeff_list (mocl)
!
! Authors:F. Colonna
! Date  :24 Sep 2001
! Revision: 12 Dec 2001 by F. Colonna Gaussian interface
! Revision: 25 Sep 2002 by F. Colonna mos nb from QMCMOL
! Revision: 07 Oct 2002 F. Colonna
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb, mos_nb_inf
  integer(dpi)                               :: aos_c_nb
  integer(dpi)                               :: mos_coeff_list_nb
  integer(dpi)                               :: ao_c_i, mo_i

! header:

  if (header_exe) then

    l_here = "mos_to_aos_c_nrmd_bld"
    description = 'MOs to AOs cartesian transformation matrix'
    call object_create ('mos_to_aos_c_nrmd')

! Improve: the order is important. check this. read  mos_coeff_list first
    call object_needed ('mos_coeff_list')
    call object_needed ('aos_c_names')

    return

  end if

! begin:
  call enter (l_here)

  mos_coeff_list_nb = size (mos_coeff_list)

  require_mos_coeff_list: if(mos_coeff_list_nb == 0) then
    message = 'in routine'*l_here*' nb of mos_coeff_list .eq. 0'
    call require (message, mos_coeff_list_nb .gt. 0)
  end if require_mos_coeff_list

! in GAUSSIAN all information is taken from wfn file

  if (interfaced_program /= "GAUSSIAN") then

    call provide_bynam ('aos_c_names')

    aos_c_nb = size(aos_c_names)

    mos_nb = system%mos_nb

    if(aos_c_nb == 0) then
      message = 'in routine'*l_here*' nb of cartesian aos .eq. 0'
      call require (message, aos_c_nb .gt. 0)
    end if 

    if(system%mos_nb == 0) then
      message = 'in routine'*l_here*' system%mos_nb = 0'
      call require (message, system%mos_nb .gt. 0)
    end if

    mos_nb_inf = mos_coeff_list_nb/aos_c_nb

! Improve: in QMCMOL the nb of mos is unknown. only coefficients are known
    if(system%mos_nb == NOT_INIT_I) then
      if (trim(interfaced_program) == 'QMCMOL') then
        system%mos_nb = mos_nb_inf
        mos_nb        = system%mos_nb
			end if
    end if

    if(debug) then
      write (logf,*)trim(l_here),'-d: interfaced program >',trim(interfaced_program),'<'
      write (logf,*)trim(l_here),'-d: nb of cartesian aos normed  =',aos_c_nb
      write (logf,*)trim(l_here),'-d: nb of mos coefficients read =',mos_coeff_list_nb 
      write (logf,*)trim(l_here),'-d: nb of mos inferred          =',mos_nb_inf
      write (logf,*)trim(l_here),'-d: nb of mos read from file    =',mos_nb
    end if

    require_mos_correct: if(mos_coeff_list_nb /= mos_nb*aos_c_nb) then
      write (logf,*)trim(l_here),'-e: nb of mos read from file                 =',mos_nb   
      write (logf,*)trim(l_here),'-d: nb of mos inferred from aos              =',mos_nb_inf
      write (logf,*)trim(l_here),'-e: nb of aos inferred from basis set        =',aos_c_nb
      write (logf,*)trim(l_here),'-e: nb of aos calculated from mos_coeff_list =', &
                                mos_coeff_list_nb/mos_nb
      write (logf,*)trim(l_here),'-e: check ',trim(interfaced_program),' mos file'

      message = 'in routine'*l_here*CR*'nb of mos_coeff_list ='*mos_coeff_list_nb &
              *' /= aos_c_nb * mos_nb ='*aos_c_nb*' * '*mos_nb*' = '*(mos_nb*aos_c_nb)
      call require (message, mos_coeff_list_nb == mos_nb*aos_c_nb)
    end if require_mos_correct

! redistribute mos
    call alloc ('mos_to_aos_c_nrmd', mos_to_aos_c_nrmd, mos_nb, aos_c_nb)

    mos_to_aos_c_nrmd = &
        transpose (reshape (mos_coeff_list, (/aos_c_nb, mos_nb/)))
 
  end if ! interfaced program except GAUSSIAN

  require_mos_2: if(mos_nb == 0) then
    message = 'in routine'*l_here*' nb of mos .eq. 0'
    call require (message, mos_nb .gt. 0)
	end if require_mos_2

  include 'debug_mos_to_aos_c_nrmd.inc'
  call exit (l_here)

  end subroutine mos_to_aos_c_nrmd_bld

! ============================================================================
  function mos_coeff_list_gamess_count (unit) result(result) 
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_gamess_count
!
! Description    : returns number of mos in gamess's file
! Description    : rewinds unit at end
!
! Authors:X. Krokidis, F. Colonna
! Date  :26 Sep 2000
! Revision: 15 Oct 2001 by F. Colonna
! ----------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: string 
  logical(bool)                               :: found 
  integer(dpi)                               :: vec_pos   
  integer(dpi)                               :: n_ao, ao
  integer(dpi)                               :: coeffs_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: mos_nb

! begin:
  l_here ='mos_coeff_list_gamess_count'
  call enter (l_here)

  if(debug) write (logf,*)trim(l_here),'-d: unit =',unit

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! initialize
  coeffs_nb  = 0
  string = blanks

! go to word $VEC  
  vec_pos = file_fetch_word ("$VEC", 1, unit)

  message ='in routine'*l_here*'$VEC not found in file >'+files_names(unit)+'<'
  call require (message, vec_pos > 0)

! count until word $END         
  found = .false.
  do while (.not.found)
    read (unit,'(a)',iostat=iostat) string

    if(debug) then
      write (logf,*)trim(l_here),'-d: found=',found
      write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
      write (logf,*)trim(l_here),'-d: adjustl(string)=',adjustl(string)
    end if

    found = index (string, '$END') .ne. 0
    if (.not. found) then 
      ao        = string_to_integer (string(3:5))
      if ( ao == 1) aos_nb = 0  ! start counting aos

      n_ao      = (len_trim(string)-5)/15
      aos_nb    = aos_nb + n_ao
      coeffs_nb = coeffs_nb + n_ao
      mos_nb    = string_to_integer (string(1:2))

    end if  ! found

    if(debug) then
      write (logf,*)trim(l_here),'-d: number of n_ao   =',n_ao    
      write (logf,*)trim(l_here),'-d: number of aos_nb =',aos_nb
      write (logf,*)trim(l_here),'-d: number of mos_nb =',mos_nb    
    end if

  end do ! while until word $END

  system%mos_nb  = mos_nb
  system%aos_nb  = aos_nb
  system%mocl_nb = coeffs_nb

  write (logf,*) trim(l_here),': number of aos   counted ',aos_nb
  write (logf,*) trim(l_here),': number of mos   counted ',mos_nb
  write (logf,*) trim(l_here),': number of coeff counted ',coeffs_nb

  if(system%aos_nb == 0) then
    message = 'in routine'*l_here*' system%aos_nb = 0'
    call require (message, system%aos_nb .gt. 0)
  end if 

  if(system%mos_nb == 0) then
    message = 'in routine'*l_here*' system%mos_nb = 0'
    call require (message, system%mos_nb .gt. 0)
  end if 

  message ='in routine'*l_here*' $END not found in file >'*files_names(unit)*'<'
  call require (message, iostat == 0)

  message ='in routine'*l_here*' number of coeff ='* & 
            coeffs_nb*' in file >'*files_names(unit)*'<'
  call ensure (message, coeffs_nb >= 0)

  message = 'in routine'*l_here*' nb of coeffs_nb /= mos_nb*aos_nb'
  call ensure (message, coeffs_nb == mos_nb*aos_nb)

  result = coeffs_nb

  rewind (unit)

  call exit (l_here)

  end function mos_coeff_list_gamess_count

! ============================================================================
  subroutine mos_coeff_list_gamess_read (unit)
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_gamess_read
!
! Description    : mos coefficients "mos_coeff_list" on aos_c_nrmd 
! Description    : from GAMESS's file
!
! Fonctionality  : Utility
!
! -- Creates     : mos_to_aos_c_nrmd (mo, ao_c)
!
! Authors:X. Krokidis, F. Colonna
! Date  :13 Oct 2000
! Revision: 04 Oct 2001 by F. Colonna
! Revision: 23 Jul 2002 by F. Colonna utility routine out of header
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len)              :: string 
  logical(bool)                              :: found 
  integer(dpi)                               :: n_mo, n_lin, last_mo
  integer(dpi)                               :: coeffs_nb, coeff_cur
  integer(dpi)                               :: ao, f_ao, l_ao, n_ao
  integer(dpi)                               :: vec_pos
  integer(dpi)                               :: count_100_mos
  real(dp), dimension(5)                     :: cur_list        

! header:
  call header_die

! begin:
  l_here = 'mos_coeff_list_gamess_read'
  call enter (l_here)
  
  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! count
  coeffs_nb = mos_coeff_list_gamess_count (unit) 
  call alloc ('mos_coeff_list', mos_coeff_list, coeffs_nb)
  if(debug) write (logf,*)trim(l_here),'-d: coeffs_nb=',coeffs_nb      

! initialize
  string = blanks

! go to word $VEC  
  vec_pos = file_fetch_word ("$VEC", 1, unit)

  message ='in routine'*l_here*'$VEC not found in file >'+files_names(unit)+'<'
  call require (message, vec_pos > 0)

! read mos until word $END         

! $VEC
! 1  1-1.00679622E+00
!5.21551791E-17-1.11805393E-16-4.12359476E-18-5.09290295E-03
! 1  2 4.72532390E-19 2.16712576E-16 6.78147569E-18 9.37638917E-03
!2.88957726E-19
! 1  3-1.17213653E-16-9.46302539E-18 4.15565305E-03
!4.15565859E-03-3.56058605E-03
! 1  4 2.01723410E-23-1.07517773E-13-2.81455793E-18
! 2  1 6.15150729E-19-2.08379041E-11
!1.01556128E+00-2.43962964E-16-5.63878036E-19
!
!17  3 3.38607720E-16 1.36790302E+00 6.18270436E-14 6.18763319E-14
!6.34829685E-14
!17  4 6.70164868E-17 2.40685548E-15-2.97642995E-16
! $END

  count_100_mos = 0
  found = .false.
  last_mo = -1
  n_mo = 1
  coeff_cur = 0

  do while (.not.found)

   cur_list = 0.d0

! f_ao first ao in the current line 
! l_ao last  ao in the current line 
! n_ao nb of ao in the current line (max= 5)
   
   read (unit,'(a)',iostat=iostat) string 
   if(debug) write (logf,*)trim(l_here),'-d: string=',trim(string)  
   if(debug) write (logf,*)trim(l_here),'-d: len=',len_trim(string)  

   found = index (string, '$END') .ne. 0
   if(debug) write (logf,*)trim(l_here),'-d: found=',found

   if (found) exit

! get current mo:
   n_mo = string_to_integer (string(1:2))
   
! these 2 lines take into account stupid conventions 
! of GAMESS of numbering mos ...1,2,3..99, 00, 1,2,3 ...
   if (n_mo == 0) count_100_mos = count_100_mos + 1
    n_mo = n_mo + count_100_mos*100

! if new mo
   if(n_mo  /= last_mo) then
     f_ao    = 1
     last_mo = n_mo
   else
     f_ao    = l_ao + 1
   end if

   n_ao = (len_trim(string)-4)/15
   l_ao = f_ao + n_ao -1

   if(debug) then
     write (logf,*)trim(l_here),'-d: f_ao=',f_ao, ' l_ao=',l_ao, ' n_ao=',n_ao,' n_mo=',n_mo
   end if

   read (string(1:len_trim(string)),'(i2,i3,5d15.8)') &
        n_mo, n_lin, cur_list(1:n_ao)

   if(debug) then
     write (logf,*)trim(l_here),'-d: n_mo=',n_mo,' n_lin=',n_lin,&
        ' n_ao=',n_ao,' cur_list ',cur_list(1:n_ao)
   end if

   do ao = 1, n_ao
     coeff_cur = coeff_cur +1
     mos_coeff_list (coeff_cur) = cur_list (ao)
     if(debug) write (logf,*)trim(l_here),'-d: coeff_cur =',coeff_cur, &
                       ' mos_coeff_list=',mos_coeff_list(coeff_cur)
     if(coeff_cur > coeffs_nb) then 
       message = 'coeff_cur ='+coeff_cur+' > coeffs_nb='+coeffs_nb
       call die (l_here, message)
     end if
  end do

  end do ! while

  message =' in routine'*l_here*' $END not found in file >' &
           +files_names(unit)+'<'
  call require (message, iostat == 0)

  call exit (l_here)

  end subroutine mos_coeff_list_gamess_read

! ============================================================================
  function mos_qmcmol_count (unit) result (result) 
! ----------------------------------------------------------------------------
!
! Name           : mos_qmcmol_count
!
! Argument (in ) : unit  
!
! Result         : nb of mos coefficients from QMCMOL file
!
! Description    : returns nb of mos coefficients from QMCMOL file 
! Description    : and ckeck things
!
! Authors:R. Assaraf, F. Colonna
! Date  :20 Sep 2001
! ----------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                                 :: result

! i/o:
  integer(dpi)                                 :: unit

! local:
  character(len=max_string_len_routine_name)   :: l_here
  logical(bool)                                :: done, exist, opened
  integer(dpi)                                 :: mos_nb

! string_to_words
  integer(dpi)                                 :: number_of_words 
  integer(dpi)                                 :: word_max_len

  character(len=max_string_len_file_name)      :: f_name
  character(len=max_string_len)                :: l_string , string

! begin:
  l_here ='mos_qmcmol_count'
  call enter (l_here)

  if(debug) then
   write (logf,*) trim(l_here),'-d: file-name >',trim(files_names(unit)),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(files_paths(unit)),'<'
   inquire (unit = unit, name = f_name, opened = opened, exist = exist)
   write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
   write (logf,*) trim(l_here),'-d: f_name    >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: exist =',exist,' opened=',opened
  end if

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! read header
  call file_header_read (unit)

! initialize
  number_of_words = 0
  mos_nb = 0

  done = .false.
  l_while: do while (.not.done)

   l_string = blanks
   read (unit,'(a)',iostat=iostat) l_string 

   if(debug) then
     write (logf,*)trim(l_here),'-d:' 
     write (logf,*)trim(l_here),'-d: l_string >',trim(l_string),'<'
     write (logf,*)trim(l_here),'-d:'
   end if

   if(iostat == -1) then
    message = 'end of file detected when reading file >'+trim(files_names(unit))+'<' &
   +CR+' "end_of_mos" is probably missing at the end of file'+CR+' last line read >'+l_string+'<'
    call die (l_here, message)
   else if(iostat /= 0) then
    message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
    call die (l_here, message)
   end if

   string = adjustl(string_to_lower (l_string))
   
   if(debug) write (logf,*)trim(l_here),'-d: string=',trim(string)  

   number_of_words = string_to_words_count (string,' ', word_max_len)

   if(debug) then
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
   end if

   if(number_of_words /= 0 .and. string(1:2) == "mo") then
     mos_nb = mos_nb + 1
   end if 

   if(debug) then
       write (logf,*)trim(l_here),'-d: mos_nb=',mos_nb
   end if

  end do l_while

  require_mos_nb: if(mos_nb == 0) then
    message = 'in routine'*l_here*' nb of mos_nb .eq. 0'
    call require (message, mos_nb .gt. 0)
  end if require_mos_nb

  result = mos_nb

  write (logf,'(2a,i3)')trim(l_here),': nb of mos in file=',mos_nb

  call exit (l_here)

  end function mos_qmcmol_count

! ============================================================================
  function mos_coeff_qmcmol_count (unit) result (result) 
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_qmcmol_count
!
! Argument (in ) : unit  
!
! Result         : nb of mos coefficients from QMCMOL file
!
! Description    : returns nb of mos coefficients from QMCMOL file 
! Description    : and ckeck things
!
! Authors:R. Assaraf, F. Colonna
! Date  :20 Sep 2001
! Revision: 14 Nov 2002 F. Colonna
! ----------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: done, exist, opened
  integer(dpi)                               :: coeffs_nb

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi),                        pointer :: lenwrd(:) => NULL()
  integer(dpi)                               :: number_of_words = 0
  integer(dpi)                               :: word_max_len
  integer(dpi)                               :: word_i, line_i

  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len)              :: l_string , string

! begin:
  l_here ='mos_coeff_qmcmol_count'
  call enter (l_here)

  if(debug) then
   write (logf,*) trim(l_here),'-d: file-name >',trim(files_names(unit)),'<'
   write (logf,*) trim(l_here),'-d: file-path >',trim(files_paths(unit)),'<'
   inquire (unit = unit, name = f_name, opened = opened, exist = exist)
   write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
   write (logf,*) trim(l_here),'-d: f_name    >',trim(f_name),'<'
   write (logf,*) trim(l_here),'-d: exist =',exist,' opened=',opened
  end if

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! read header
  call file_header_read (unit)

! initialize
  line_i    = 0
  coeffs_nb = 0

  done = .false.
  l_while: do while (.not.done)

   l_string = blanks
   read (unit,'(a)',iostat=iostat) l_string 
   line_i = line_i + 1

   if(iostat == -1) then
    message = 'end of file detected when reading file >'+trim(files_names(unit))+'<' &
   +CR+' "end_of_mos" probably missing at end of file'+CR+' last line read >'+l_string+'<'
    call die (l_here, message)
   else if(iostat /= 0) then
    message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
    call die (l_here, message)
   end if

   string = string_to_lower (l_string)
   
   if(debug) write (logf,*)trim(l_here),'-d: line # ',line_i,' >',trim(string),'<'

   number_of_words = string_to_words_count (string,' ', word_max_len)

   if(debug) then
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
   end if

! empty line: end of basis, if something has been read !
   if(number_of_words == 0) then
      if(debug)write (logf,*)trim(l_here),'-d: empty line skipped' 
      cycle 
   else 

     call split_string (string, ' ', words, lenwrd, number_of_words, len(words))

     if(debug) then
      do word_i = 1, number_of_words
       write (logf,*)trim(l_here),'-d: word # ',word_i,' >',trim(words(word_i)),'<'
      end do
     end if

   end if! if(number_of_words .eq. 0)

! count until end_of_mos is reached
   if (string_to_lower(words(1)) ==  'end_of_mos') then
       exit
    else
       do word_i= 1, number_of_words
          if (numeric_is (words (word_i))) coeffs_nb = coeffs_nb + 1
       enddo ! do word_i
! coeffs_nb = coeffs_nb + number_of_words
   end if 

   if(debug) then
       write (logf,*)trim(l_here),'-d: coeffs_nb=',coeffs_nb
   end if

  end do l_while

  require_coeffs_nb: if(coeffs_nb == 0) then
    message = 'in routine'*l_here*' nb of coeffs_nb .eq. 0'
    call require (message, coeffs_nb .gt. 0)
  end if require_coeffs_nb

  result = coeffs_nb

  write (logf,*)trim(l_here),': nb of coeff in file=',coeffs_nb

  call exit (l_here)

  end function mos_coeff_qmcmol_count

! ============================================================================
  subroutine mos_coeff_list_read 
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_read
!
! Description    : dispatches "mos_coeff_list" reading
! Description    : according to interfaced_program
!
! Creates        : mos_coeff_list (mocl)    
!
! Authors:R. Assaraf, F. Colonna
! Date  :20 Sep 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mocl_i
  integer(dpi)                               :: mos_coeff_list_nb

! header:

  if (header_exe) then

    l_here = "mos_coeff_list_read"
    description = 'MOs coefficients on AOs as a linear array'
    call object_create ('mos_coeff_list')

    return

  end if

! begin:
  call enter (l_here)

  unit = unit_from_type ('mos-in')

  if(debug) then
   write (logf,*)trim(l_here),'-d: unit =',unit
  end if

  write (logf,*)trim(l_here),': interfaced_program >',trim(interfaced_program),'<'

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit &
    +CR*' probably file name missing in input'
    call require (message, unit .gt. 0)
  end if require_unit

  call unit_open_read (unit)

  select case (interfaced_program)
  case ("GAMESS")
    call mos_coeff_list_gamess_read (unit)
  case ("GAUSSIAN")
    call mos_coeff_list_gaussian_read (unit)
  case ("MOLPRO")
    call mos_coeff_list_molpro_read (unit)
  case ("QMCMOL")
    call mos_coeff_list_qmcmol_read (unit)
  case default
    message = 'in routine'*l_here*' interfaced_program is not defined' &
    +CR*'should be one of "GAMESS" "GAUSSIAN" "MOLPRO" "QMCMOL"'
    call die (l_here, message)
  end select

! keep catalogued
  call file_close_by_unit (unit)

  mos_coeff_list_nb = size(mos_coeff_list)

  include 'debug_mos_coeff_list.inc'

  call exit (l_here)

  end subroutine mos_coeff_list_read

! ============================================================================
  subroutine mos_coeff_list_gaussian_read (unit)
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_gaussian_read
!
! Argument (in ) : unit
!
! Description    : read mos list of coefficients from gaussian (wfn) file
! Description    : do not care of the content (on aos_c, on aos_s, on sos_c)
!
! Fonctionality  : Utility
!
! -- Creates     : mos_coeff_list (mocl)
! -- Creates     : mos_occupations (mo)
! -- Creates     : mos_energies (mo)   
! -- Creates     : mos_spins (mo)
! -- Creates     : mos_to_aos_c (mo, ao)
!
! Authors:F. Colonna
! Date  :30 Nov 2001
! Revision: 22 Jul 2002 by F. Colonna utility routine out of header
! Revision: 07 Nov 2002 by F. Colonna header
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here = NOT_INIT_S
  logical(bool)                              :: eof
  character(len=max_string_len)              :: record
  integer(dpi)                               :: mos_coeff_list_nb

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i
  integer(dpi)                                 :: mos_nb, aos_nb
  integer(dpi)                                 :: ao
  integer(dpi)                                 :: mo, coef, tab(4)

! header:
  call header_die

! begin:
  l_here = 'mos_coeff_list_gaussian_read'
  call enter (l_here)

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! count:
  tab = mos_aos_gaussian_count (unit)
  mos_nb = tab(1)
  system%mos_nb = mos_nb

  aos_nb = tab(2)
  system%aos_nb = aos_nb
  mos_coeff_list_nb = mos_nb*aos_nb

  if(system%aos_nb == 0) then
    message = 'in routine'*l_here*' system%aos_nb = 0'
    call require (message, system%aos_nb .gt. 0)
  end if 

  if(system%mos_nb == 0) then
    message = 'in routine'*l_here*' system%mos_nb = 0'
    call require (message, system%mos_nb .gt. 0)
  end if 

  rewind (unit)

  call alloc ('mos_coeff_list', mos_coeff_list, mos_coeff_list_nb)

! Improve this is not used
  call allocation ('mos_energies', mos_energies, mos_nb)
  call allocation ('mos_spins', mos_spins, mos_nb)
  call allocation ('mos_occupations', mos_occupations, mos_nb)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: mos_nb=',mos_nb,' aos_nb=',aos_nb, &
                         ' mos_coeff_list_nb=',mos_coeff_list_nb
  end if

!  call alloc ('mos_to_aos_c', mos_to_aos_c, mos_nb, aos_nb)

! initialize
  ao = 0
  mo = 0
  coef = 0

  eof = .false.
  while_loop: do while (.not.eof)

   read (unit,'(a)',iostat=iostat) record 
   eof = (record(1:8) == "END DATA")

! exit at eof
   if(iostat /= 0 .or. eof) then
    if(debug .or. print_level > 0 ) then
    write (logf,*)trim(l_here),': eof reached for file >',trim(files_names(unit)),'<'
    end if
    exit
   end if
   
   if(debug) then 
    write (logf,*)trim(l_here),'-d: '
    write (logf,*)trim(l_here),'-d: current string >',trim(record),'<'
    write (logf,*)trim(l_here),'-d: '
   end if

   number_of_words = string_to_words_count (record,' ', word_max_len)

   if(debug) then 
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
   end if

! parse:
   call split_string (record, ' ', words, lenwrd, number_of_words, len(words))

   if(debug) then
    do i = 1, number_of_words
     write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
    end do
   end if

! parse MO    4     MO 0.0        OCC NO =    2.0000000  ORB. ENERGY = -.5661499
    parse_loop: if(number_of_words == 12    &   
               .and. (words(1) == "MO" )    &
               .and. (words(5) == "OCC")    &
               .and. integer_is(words(2))   &
               .and. numeric_is(words(8)) ) then
       
      mo = string_to_integer(words(2))
      ao = 0

      if(mo > mos_nb .or. mo < 1) then
       write (logf,*)trim(l_here),'-e: mo # ',mo,' mos_nb= ',mos_nb
       call die (l_here, 'mo > mos_nb .or. mo < 1')
      end if

      mos_occupations (mo) = string_to_double (words(8))
      mos_energies (mo)    = string_to_double (words(12))

      if(debug) then
        write (logf,*)trim(l_here),'-d: mo # ',mo,' energy= ',mos_energies (mo)
        write (logf,*)trim(l_here),'-e: mo # ',mo,' occupation= ',mos_occupations (mo)
        write (logf,*)trim(l_here),'-e: ao # ',ao
      end if

! parse coefficients   -.60157855D-01  -.11072257D+00  -.17917001D+00 -.23552400D+00  -.20200756D+00

   else if (number_of_words <= 5  &
      .and. numeric_is(words(1)) ) then
       
     do i = 1, number_of_words
       coef = coef + 1
       ao = ao + 1
       mos_coeff_list (coef) = string_to_double (words(i))
!       mos_to_aos_c (mo, ao) = string_to_double (words(i))
       
      if(debug) then
        write (logf,*)trim(l_here),'-d: mo # ',mo,' ao # ',ao,' coef # ',coef, &
                             ' = ',mos_coeff_list (coef)
      end if

     end do

   else
   end if parse_loop

  end do while_loop

  write (logf,*)trim(l_here),': nb of mos read =',mo,' nb of mos counted =',mos_nb

  if(mos_coeff_list_nb /= coef) then

   write (logf,*)trim(l_here),'-e: nb of coeffs read =',coef, &
                        ' nb of coeffs counted =',mos_coeff_list_nb
   write (logf,*)trim(l_here),'-e: list of mos read:'
   do i = 1, mo
     write (logf,*)trim(l_here),'-e: '
     write (logf,*)trim(l_here),'-e: mo # ',i,' occupation= ',mos_occupations (i)
     write (logf,*)trim(l_here),'-e: mo # ',i,' energy= ',mos_energies (i)
   end do
   call die (l_here,  'mos_coeff_list_nb /= coef ')

  end if

  if(debug) then
   do i = 1, mos_coeff_list_nb
     write (logf,*)trim(l_here),'-d: coeff # ',i,' = ',mos_coeff_list(i)
   end do
  endif

  write (logf,*)trim(l_here),': ',mos_coeff_list_nb,' mos coeff list read'

  call exit (l_here)

  end subroutine mos_coeff_list_gaussian_read

! ============================================================================
  subroutine mos_coeff_list_qmcmol_read (unit)
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_qmcmol_read
!
! Argument (in ) : unit
!
! Description    : read mos list of coefficients from QMCMOL file
! Description    : do not care of the content (on aos_c, on aos_s, on sos_c)
!
! Fonctionality  : Utility
!
! -- Creates     : mos_coeff_list (mocl)
!
! Authors:R. Assaraf, F. Colonna
! Date  :20 Sep 2001
! Revision: 22 Jul 2002 by F. Colonna utility routine out of header
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool)                              :: done
  character(len=max_string_len)              :: c_string
  integer(dpi)                               :: mos_coeff_list_nb

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i, mocl_i

! header:
  call header_die

! begin:
  l_here = 'mos_coeff_list_qmcmol_read'
  call enter (l_here)

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! count:
  mos_coeff_list_nb = mos_coeff_qmcmol_count (unit)
  rewind (unit)

  call alloc ('mos_coeff_list', mos_coeff_list, mos_coeff_list_nb)

! read header
  call file_header_read (unit)

! initialize
  mos_coeff_list_nb = 0
  done = .false.

  l_while: do while (.not.done)

   read (unit,'(a)',iostat=iostat) c_string 
   if(iostat /= 0) then
    message ='error in reading file >'+trim(files_names(unit))+'<'+ &
    ' iostat='+iostat
    call files_show
    call die (l_here, message)
   end if
   
   if(debug) then 
    write (logf,*)trim(l_here),'-d: '
    write (logf,*)trim(l_here),'-d: current string >',trim(c_string),'<'
   end if

   number_of_words = string_to_words_count (c_string,' ', word_max_len)

   if(debug) then 
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
   end if

! empty line: end of symbol basis

   if(number_of_words .eq. 0) cycle 

   call split_string (c_string, ' ', words, lenwrd, number_of_words, len(words))

   if(debug) then
    do i = 1, number_of_words
     write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
    end do
   end if

! end of mos file
   if (number_of_words .ge. 1 .and.  &
       string_to_lower(words(1)) == 'end_of_mos') exit
       
    do i = 1, number_of_words
    if (numeric_is (words(i))) then
      mos_coeff_list_nb = mos_coeff_list_nb + 1
      call decode (mos_coeff_list(mos_coeff_list_nb), words(i))
     endif
      if(debug) then
        write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)), &  
       ' mos_coeff_list_nb # ',mos_coeff_list_nb,  &
       ' coeff =',mos_coeff_list(mos_coeff_list_nb)    
      end if
    end do

  end do l_while

  message ='found '*mos_coeff_list_nb*'mos coeffs in QMCMOL basis file >'+files_names(unit)+'<'
  call message_out (l_here, message)

  if(mos_coeff_list_nb == 0) then
    message ='in routine'*l_here*' mos_coeff_list_nb='+mos_coeff_list_nb
    call require (message,  mos_coeff_list_nb /= 0)
  end if

  include 'debug_mos_coeff_list.inc'
  include 'ensnb_mos_coeff_list.inc'


  call exit (l_here)

  end subroutine mos_coeff_list_qmcmol_read

! ============================================================================
  function mos_coeff_list_molpro_count (unit) result(result) 
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_molpro_count
!
! Description    : returns array of number of mos, of aos 
! Description    : in molpro's (MOLDEN) file
! Description    : rewinds unit at end
!
! Authors:F. Colonna
! Date  :29 Nov 2001
! ----------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result (2)

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: f_name   
  logical(bool)                              :: eof 
  logical(bool)                              :: opened, exist
  integer(dpi)                               :: coeffs_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: ene_pos
  integer(dpi)                               :: mo_pos
  integer(dpi)                               :: occ_pos

! begin:
  l_here ='mos_coeff_list_molpro_count'
  call enter (l_here)

  if(debug) write (logf,*)trim(l_here),'-d: unit =',unit

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

  inquire (unit = unit, name = f_name, opened = opened, exist = exist)
  if(debug) then
    write (logf,*) trim(l_here),'-d: checking unit=',unit,' with inquire'  
    write (logf,*) trim(l_here),'-d: file name >',trim(f_name),'<'
    write (logf,*) trim(l_here),'-d: opened =',opened
    write (logf,*) trim(l_here),'-d: exist  =',exist
  end if

  rewind (unit)

  mos_nb = 0
  aos_nb = 0
  coeffs_nb = 0

! go to word [MO]  
  mo_pos = file_fetch_word ("[MO]", 1, unit)
  if(mo_pos < 0 ) then
    write (logf,*)trim(l_here),'-e: [MO] not found at line # ',-mo_pos
    write (logf,*)trim(l_here),'-e: file >',trim(f_name),'<'
    write (logf,*)trim(l_here),'-e: is probably not a MOLPRO-molden file'
    call die (l_here, 'file is not a MOLPRO-molden file')
  end if

  if(debug) then 
    write (logf,*)trim(l_here),'-d: [MO] found at line # ',mo_pos
  end if

  message =' in routine'+l_here+' no [MO] item in file >'+ &
  trim(files_names(unit))+'<'
  call require (message, mo_pos /= 0)

  eof = .false.

  loop_while: do while (.not.eof)

! search for "Occup=" to set up cursor at the top of coefficients
  occ_pos = file_fetch_word ("Occup = ", 1, unit) 
  if(debug) write (logf,*)trim(l_here),'-d: occ_pos =',occ_pos
  
! search for "Ene=". to count coefficients plus one line. except at the end
    ene_pos = file_fetch_word ("Ene = ", 1, unit)
    eof = ene_pos <= 0

     if(debug) then
       write (logf,*)trim(l_here),'-d: ene_pos         =',ene_pos   
     end if

! eof reached
    if (eof) then 
     ene_pos = - ene_pos + 1
    end if ! eof

    mos_nb    = mos_nb + 1
    aos_nb    = ene_pos - 1 
    coeffs_nb = coeffs_nb + aos_nb

     if(debug) then
       write (logf,*)trim(l_here),'-d: ene_pos         =',ene_pos   
       write (logf,*)trim(l_here),'-d: number of aos   =',aos_nb    
       write (logf,*)trim(l_here),'-d: number of mos   =',mos_nb
       write (logf,*)trim(l_here),'-d: number of coeff =',coeffs_nb
     end if

  end do loop_while

  system%mos_nb = mos_nb
  system%aos_nb = aos_nb
  system%mocl_nb = coeffs_nb

  if(system%aos_nb == 0) then
    message = 'in routine'*l_here*' system%aos_nb = 0'
    call require (message, system%aos_nb .gt. 0)
  end if 

  if(system%mos_nb == 0) then
    message = 'in routine'*l_here*' system%mos_nb = 0'
    call require (message, system%mos_nb .gt. 0)
  end if 

  write (logf,'(2a,i5)') trim(l_here),': number of coeff counted ',coeffs_nb
  write (logf,'(2a,i5)') trim(l_here),': number of mos   counted ',mos_nb
  write (logf,'(2a,i5)') trim(l_here),': number of aos   counted ',aos_nb

  message =' in routine'*l_here*' no mos found in file >'+ &
  trim(files_names(unit))+'<'
  call require (message, mos_nb /= 0)

  message =' in routine'*l_here*' no aos  found in file >'+ &
  trim(files_names(unit))+'<'
  call ensure (message, aos_nb /= 0)

  message =' in routine'*l_here*' nb aos='+aos_nb+' nb mos='+mos_nb &
           +' while nb of coeffs ='+coeffs_nb
  call ensure (message, aos_nb*mos_nb == coeffs_nb)

  result = (/mos_nb, aos_nb /)

  rewind (unit)

  call exit (l_here)

  end function mos_coeff_list_molpro_count

! ============================================================================
  subroutine mos_coeff_list_molpro_read (unit)
! ----------------------------------------------------------------------------
!
! Name           : mos_coeff_list_molpro_read
!
! Argument (in ) : unit
!
! Description    : read mos list of coefficients from molpro (MOLDEN) file
! Description    : do not care of the content (on aos_c, on aos_s, on sos_c)
!
! Creates        : mos_coeff_list (mocl)    
! Creates        : mos_occupations (mo)
! Creates        : mos_energies (mo)    
! Creates        : mos_spins (mo)    
!
! Authors:F. Colonna
! Date  :30 Nov 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name),save :: l_here
  logical(bool)                              :: eof
  character(len=max_string_len)              :: record
  integer(dpi)                               :: mos_coeff_list_nb

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi)                       , pointer :: lenwrd(:) => NULL()
  integer(dpi)                                 :: number_of_words = 0
  integer(dpi)                                 :: word_max_len, i
  integer(dpi)                                 :: mos_nb, aos_nb
  integer(dpi)                                 :: ao
  integer(dpi)                                 :: mo, coef, tab(2)
  integer(dpi)                                 :: mo_pos

! header:

  if (header_exe) then

    l_here = "mos_coeff_list_molpro_read"

    description = 'MOs coefficients on AOs as a linear array'
    call object_create ('mos_coeff_list')
    description = 'MOs occupations array'
    call object_create ('mos_occupations')
    description = 'MOs energies array'
    call object_create ('mos_energies')
    description = 'MOs spins array'
    call object_create ('mos_spins')

    return

  end if

! begin:
  call enter (l_here)

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

! count:
  tab = mos_coeff_list_molpro_count (unit)
  mos_nb = tab(1)
  system%mos_nb = mos_nb

  aos_nb = tab(2)
  system%aos_nb = aos_nb

  mos_coeff_list_nb = mos_nb*aos_nb
  system%mocl_nb = mos_coeff_list_nb

  if(system%aos_nb == 0) then
    message = 'in routine'*l_here*' system%aos_nb = 0'
    call require (message, system%aos_nb .gt. 0)
  end if 

  if(system%mos_nb == 0) then
    message = 'in routine'*l_here*' system%mos_nb = 0'
    call require (message, system%mos_nb .gt. 0)
  end if 

  rewind (unit)

  call alloc ('mos_coeff_list', mos_coeff_list, mos_coeff_list_nb)
  call alloc ('mos_occupations', mos_occupations, mos_nb)
  call alloc ('mos_energies', mos_energies, mos_nb)
  call allocation ('mos_spins', mos_spins, mos_nb)

  if(debug) then 
    write (logf,*)trim(l_here),'-d: mos_nb=',mos_nb,' aos_nb=',aos_nb, &
                         ' mos_coeff_list_nb=',mos_coeff_list_nb
  end if

! initialize
  mo_pos = 0
  mo = 0
  coef = 0

! go to word [MO]
  mo_pos = file_fetch_word ("[MO]", 1, unit)

  if(debug) then
    write (logf,*)trim(l_here),'-d: [MO] found at line # ',mo_pos
  end if

  eof = .false.
  while_loop: do while (.not.eof)

   read (unit,'(a)',iostat=iostat) record 

! exit at eof
   if(iostat /= 0 ) then
    if(debug .or. print_level > 0 ) then
    write (logf,*)trim(l_here),': eof reached for file >',trim(files_names(unit)),'<'
    end if
    exit
   end if
   
   if(debug) then 
    write (logf,*)trim(l_here),'-d: '
    write (logf,*)trim(l_here),'-d: current string >',trim(record),'<'
    write (logf,*)trim(l_here),'-d: '
   end if

   number_of_words = string_to_words_count (record,' ', word_max_len)

   if(debug) then 
    write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words  
    write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len  
   end if

! parse:
   call split_string (record, ' ', words, lenwrd, number_of_words, len(words))

   if(debug) then
    do i = 1, number_of_words
     write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
    end do
   end if

! parse Ene=       0.5194
   parse_loop: if (number_of_words == 2      &
               .and. words(1)(1:4) == "Ene = " &
               .and. (numeric_is(words(2)) .or. stars_is(words(2)) ) ) then
       
      mo = mo +1
      ao = 0

      if(stars_is(words(2)) ) then
        mos_energies (mo) = 99999.9999d0
      else
        mos_energies (mo) = string_to_double (words(2))
      end if

      if(debug) then
        write (logf,*)trim(l_here),'-d: parsing energies '
        write (logf,*)trim(l_here),'-d: mo # ',mo,' energy= ',mos_energies (mo)
      end if

! parse Spin= Alpha
   else if (number_of_words == 2       &
       .and. words(2)(1:5)  == "Spin = " &
       .and. alphabetic_is(words(2))  ) then
       
      mos_spins (mo) = words(2)
      if(debug) then
        write (logf,*)trim(l_here),'-d: parsing spins '
        write (logf,*)trim(l_here),'-d: mo # ',mo,' spin= ',mos_spins (mo)
      end if

! parse Occup=   0.000000
   else if (number_of_words == 2        &
       .and. words(2)(1:6)  == "Occup = " &
       .and. numeric_is(words(2))  ) then
       
      mos_occupations (mo) =  string_to_double (words(2))
      if(debug) then
        write (logf,*)trim(l_here),'-d: parsing occupations '
        write (logf,*)trim(l_here),'-d: mo # ',mo,' spin= ',mos_occupations (mo)
      end if

! parse ao-number coefficients      
   else if (number_of_words == 2  &
       .and. integer_is(words(1)) &
       .and. stars_is(words(2)) ) then
       
        ao = ao + 1
        write (logf,*)trim(l_here),'-e: invalid ao coeff '
        write (logf,*)trim(l_here),'-e: mo # ',mo,' spin= ',mos_spins (mo)
        write (logf,*)trim(l_here),'-e: mo # ',mo,' occupation= ',mos_occupations (mo)
        write (logf,*)trim(l_here),'-e: mo # ',mo,' energy= ',mos_energies (mo)
        write (logf,*)trim(l_here),'-e: ao # ',ao
        call die (l_here, ' current ao coeff is *******')

! parse ao-number coefficients      
   else if (number_of_words == 2  &
       .and. integer_is(words(1)) &
       .and. numeric_is(words(2)) ) then
       
      ao = ao + 1
      if(ao /= string_to_integer(words(1))) then
        write (logf,*)trim(l_here),'-e: invalid ao number '
        write (logf,*)trim(l_here),'-e: mo # ',mo,' spin= ',mos_spins (mo)
        write (logf,*)trim(l_here),'-e: mo # ',mo,' occupation= ',mos_occupations (mo)
        write (logf,*)trim(l_here),'-e: mo # ',mo,' energy= ',mos_energies (mo)
        write (logf,*)trim(l_here),'-e: ao # ',ao,' while in file ao # ', words(1)
        call die (l_here, ' current ao number /= ao number in file')
      end if

      coef = coef + 1
      mos_coeff_list (coef) = string_to_double (words(2))
      if(debug) then
        write (logf,*)trim(l_here),'-d: parsing coefficients '
        write (logf,*)trim(l_here),'-d: mo # ',mo,' ao # ',ao,' coef # ',coef, &
                             ' = ',mos_coeff_list (coef)
      end if

   else
   end if parse_loop

  end do while_loop

  write (logf,*)trim(l_here),': nb of mos read =',mo,' nb of mos counted =',mos_nb

  if(mos_coeff_list_nb /= coef) then

   write (logf,*)trim(l_here),'-e: nb of coeffs read =',coef, &
                        ' nb of coeffs counted =',mos_coeff_list_nb
   write (logf,*)trim(l_here),'-e: list of mos read:'
   do i = 1, mo
     write (logf,*)trim(l_here),'-e: '
     write (logf,*)trim(l_here),'-E: mo # ',i,' spin= ',mos_spins (i)
     write (logf,*)trim(l_here),'-e: mo # ',i,' occupation= ',mos_occupations (i)
     write (logf,*)trim(l_here),'-e: mo # ',i,' energy= ',mos_energies (i)
   end do
   call die (message,  'mos_coeff_list_nb /= coef ')

  end if

  if(debug) then
   do i = 1, mos_coeff_list_nb
     write (logf,*)trim(l_here),'-d: coeff # ',i,' = ',mos_coeff_list(i)
   end do
  endif

  call exit (l_here)

  end subroutine mos_coeff_list_molpro_read

! ============================================================================
  subroutine mos_to_aos_s_nrmd_bld 
! ----------------------------------------------------------------------------
!
! Name           : mos_to_aos_s_nrmd_bld
!
! Description    : get spherical normed mos from mos_coeff_list
!
! Needs          : aos_s_names (ao_s)          
! Needs          : mos_coeff_list (mocl) 
!
! Creates        : mos_to_aos_s_nrmd (mo, ao_s)
!
! Authors:F. Colonna
! Date  :24 Sep 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_s_nb
  integer(dpi)                               :: mos_coeff_list_nb
  integer(dpi)                               :: ao, mo

! header:

  if (header_exe) then

    l_here = "mos_to_aos_s_nrmd_bld"
    description = 'MOs to AOs spherical normalized transformation matrix'
    call object_create ('mos_to_aos_s_nrmd')

! Improve: the order is important. check this. read  mos_coeff_list first
    call object_needed ('mos_coeff_list')
    call object_needed ('aos_s_names')

    return

  end if

! begin:
  call enter (l_here)

  mos_coeff_list_nb = size (mos_coeff_list)
  if(mos_coeff_list_nb == 0) then
    message = 'in routine'*l_here*' nb of mos_coeff_list .eq. 0'
    call require (message, mos_coeff_list_nb .gt. 0)
  end if 

  aos_s_nb =  size(aos_s_names)
  if(aos_s_nb == 0) then
    message = 'in routine'*l_here*' nb of spherical aos .eq. 0'
    call require (message, aos_s_nb .gt. 0)
  end if 

  mos_nb = mos_coeff_list_nb/aos_s_nb

  system%aos_nb = aos_s_nb
  system%mos_nb = mos_nb
  system%mocl_nb = mos_coeff_list_nb

  if(aos_s_nb == 0) then
    message = 'in routine'*l_here*' nb of spherical aos .eq. 0'
    call require (message, aos_s_nb .gt. 0)
  end if 

  if(system%mos_nb == 0) then
    message = 'in routine'*l_here*' system%mos_nb = 0'
    call require (message, system%mos_nb .gt. 0)
  end if

  if(debug) then
    write (logf,*)trim(l_here),'-d: nb of spherical aos normed  =',aos_s_nb
    write (logf,*)trim(l_here),'-d: nb of mos coefficients read =',mos_coeff_list_nb 
    write (logf,*)trim(l_here),'-d: nb of mos inferred          =',mos_nb 
  end if

  require_mos_correct: if(mos_coeff_list_nb /= mos_nb*aos_s_nb) then
    message = 'mos_nb='+mos_nb+' aos_s_nb='+aos_s_nb+' mos_nb*aos_s_nb=' &
              +mos_nb*aos_s_nb+' /= nb of mos_coeff_list ='+mos_coeff_list_nb
    call die (l_here, message)
  end if require_mos_correct

! redistribute mos
  if (associated(mos_to_aos_s_nrmd)) then
    deallocate (mos_to_aos_s_nrmd)
    message = 'object >mos_to_aos_s_nrmd< already allocated. deallocated'
    call warning (l_here, message)
  end if
  allocate (mos_to_aos_s_nrmd(mos_nb, aos_s_nb), stat = all_err)
  if (all_err /= 0) &
  call die (l_here, 'allocate (mos_to_aos_s_nrmd(mos_nb, aos_s_nb))')

  mos_to_aos_s_nrmd = &
        transpose (reshape (mos_coeff_list, (/aos_s_nb, mos_nb/)))

! deallocate (mos_coeff_list)

  if(debug) then
   do mo = 1, mos_nb
    do ao = 1, aos_s_nb
       write (logf,*)trim(l_here),'-d: mo # ',mo,' ao spherical normed # ',ao, &
       ' coeff = ',mos_to_aos_s_nrmd(mo,ao)     
    end do
   end do
  endif

  require_mos: if(mos_nb == 0) then
  message = 'in routine'*l_here*' nb of mos .eq. 0'
  call require (message, mos_nb .gt. 0)
  end if require_mos

  call exit (l_here)

  end subroutine mos_to_aos_s_nrmd_bld

! ============================================================================
  subroutine mos_file_header_read (unit)
! ----------------------------------------------------------------------------
!
! Name           : mos_file_header_read 
! 
! Argument (in ) : unit
!
! Description    : read file headers
!
! Authors: X. Krokidis
! Date   : 16 Jan 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_input)        :: line
  integer(dpi)                               :: n_rec
  logical(bool)                              :: eof

! string_to_words
  character(len=max_string_len_input), pointer :: words(:) => NULL()
  integer(dpi),                        pointer :: lenwrd(:) => NULL()
  integer(dpi)                               :: number_of_words = 0
  integer(dpi)                               :: word_max_len

! begin:
  l_here ='mos_file_header_read'
  call enter (l_here)

  require_unit: if(unit <= 0) then
    message = 'in routine'*l_here*' unit = '*unit
    call require (message, unit .gt. 0)
  end if require_unit

  rewind(unit)

  if(debug)write (logf,*)trim(l_here),'-d: unit=',unit

  write (logf,*)trim(l_here),': file >',trim(files_names(unit)),'<'

  line = ' '
  n_rec = 0
  eof = .false.
  do while (.not.eof) 
    n_rec = n_rec + 1
    read(unit,'(a)') line
    
    number_of_words = string_to_words_count (line,' ', word_max_len)
    call split_string (line, ' ', words, lenwrd, number_of_words, len(words))
    
    select case(words(1))
    case("mos_on_what")
      mos_on_what = words(2)
      if(words(2) == "=") mos_on_what = words(3)
   end select

   write (logf,*) trim(l_here),': header: ',trim(line)

    eof = trim(line) == 'end_of_header'
  enddo

  write (logf,*)trim(l_here),': mos coefficients on ',trim(mos_on_what)

  if(.not.eof) then
    rewind(unit)
    if(warning_level > -1) then
      write (logf,*)trim(l_here),'-w: no header lines file >', &
             trim(files_names(unit)),'< rewind '
    end if
  end if 

  select case (mos_on_what)
  case ("aos_c")
  case ("aos_cart")
   mos_on_what = "aos_c"
  case ("aos_sphe_normed")
   mos_on_what = "aos_s_nrmd"
  case ("aos_cart_normed")
   mos_on_what = "aos_c_nrmd"
  case ("aos_c_nrmd")
  case ("aos_s_nrmd")
  case ("xpfs_c")
  case ("xpfs_cart")
   mos_on_what = "xpfs_c"
  case ("xpfs_c_nrmd")
  case ("xpfs_cart_normed")
   mos_on_what = "xpfs_c_nrmd"
  case default
    message ='unknown mos_on_what >'+mos_on_what+'<' &
    +CR+' check file header. missing line "mos_on_what = aos_s_nrmd"' &
    +CR+' or missing line "mos_on_what = aos_c_nrmd"' &
    +CR+' check file header >'+trim(files_names(unit))+'<'
    call die (l_here, message)
  end select
  call file_close_by_unit (unit)
  call exit (l_here)

  end subroutine mos_file_header_read

! ============================================================================
  subroutine mos_on_what_read 
! ----------------------------------------------------------------------------
!
! Name           : mos_on_what_read
!
! Description    : get mos_on_what token from mos-in file
! Description    : according to interfaced_program
!
! Fills          : mos_on_what  
!
! Authors:R. Assaraf, F. Colonna
! Date  :20 Sep 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='mos_on_what_read'
  call enter (l_here)

  select case (interfaced_program)
  case ("GAMESS")
    mos_on_what = "aos_c_nrmd"
! Improve should be in:
! call mos_on_what_gamess_read (unit)
  case ("GAUSSIAN")
    mos_on_what = "aos_c"
    mos_on_what = "xpfs_c"
    write (logf,*)trim(l_here),'-w: For GAUSSIAN mos_on_what is set to >', &
                  trim(mos_on_what),'<'
  case ("MOLPRO")
    mos_on_what = "aos_c_nrmd"
  case ("QMCMOL")

  unit = unit_from_type ('mos-in')

  if(debug) write (logf,*)trim(l_here),'-d: unit =',unit
    if(unit <= 0) then
      message = 'in routine'*l_here*'unit ='*unit &
      +CR*'probably mos-file not defined in input'
      call require (message, unit .gt. 0)
  end if 

  call unit_open_read (unit)

  call mos_file_header_read (unit)

  case default
    message = 'in routine'*l_here*' interfaced_program is not defined' &
    +CR*'should be one of "GAMESS" "GAUSSIAN" "MOLPRO" "QMCMOL"'
    call die (l_here, message)
  end select

  write (logf,*)trim(l_here),': mos_on_what >',trim(mos_on_what),'<'

  call exit (l_here)

  end subroutine mos_on_what_read

! ============================================================================
  subroutine mos_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_to_xpfs_c_bld
!
! Description : mos_to_xpfs_c is the ULTIMATE object actually used
! Description : to compute the MOs
!
! Description : the routine dispatches building of mos_to_xpfs_c
! Description : if mos_on_what == aos_c_nrmd
! Description : call mos_on_aos_c_nrmd_to_xpfs_c_bld
! 
! Description : if mos_on_what == aos_s_nrmd
! Description : call mos_on_aos_s_nrmd_to_xpfs_c_bld
! Remark      : Creates and Needs nrmd_to_xpfs_c_bld
!
! Creates     : mos_to_xpfs_c (mo, xpf_c)
!
! Authors : F. Colonna
! Date    : 27 Sep 2001
! Revision: 16 Nov 2001 by F. Colonna aos_c_nrmd  
! Revision: 09 Oct 2002 F. Colonna read from QMCMOL file
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb, aos_c_nb, xpfs_s_nb
  integer(dpi)                               :: xpfs_c_nb, xpf_c_i, mo_i, aos_nb
  integer(dpi)                               :: mocl_nb

! header:

  if (header_exe) then

    l_here = "mos_to_xpfs_c_bld"
    description = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_to_xpfs_c')

!   call object_needed ('xpfs_c')
   
    return

  end if

! begin:
  call enter (l_here)

! read mos_on_what
  if(trim(mos_on_what) == NOT_INIT_S) then
     write (logf,*)trim(l_here),'-w: mos_on_what empty calling mos_on_what_read'
     call mos_on_what_read 
  end if

  write (logf,*)trim(l_here),': mos_on_what "',trim(mos_on_what),'"' 

  select case (mos_on_what)

  case ("aos_c")

     call provide_bynam("mos_to_aos_c")
     call provide_bynam("aos_c_to_xpfs_c")

     mos_nb    = size (mos_to_aos_c, 1)
     aos_c_nb  = size (mos_to_aos_c, 2)
     xpfs_c_nb = size (aos_c_to_xpfs_c, 2)

     if(debug) then
        write (logf,*)trim(l_here),': mos_nb    =',mos_nb
        write (logf,*)trim(l_here),': aos_c_nb  =',aos_c_nb
        write (logf,*)trim(l_here),': xpfs_c_nb =',xpfs_c_nb
     end if

     call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

     mos_to_xpfs_c = &
          matmul (mos_to_aos_c, aos_c_to_xpfs_c)

  case ("aos_c_nrmd")

     call provide_bynam("mos_to_aos_c_nrmd")
     call provide_bynam("aos_c_nrmd_to_xpfs_c")

     mos_nb    = size (mos_to_aos_c_nrmd,1)
     aos_nb    = size (mos_to_aos_c_nrmd,2)
     xpfs_c_nb = size (aos_c_nrmd_to_xpfs_c, 2)
     
     write (logf,*)trim(l_here),': nb of mos =',mos_nb
     write (logf,*)trim(l_here),': nb of cartesian aos =',aos_nb
     write (logf,*)trim(l_here),': nb of cartesian xpfs =',xpfs_c_nb
     
     include 'chksiz_mos_to_aos_c_nrmd.inc'
     include 'chksiz_aos_c_nrmd_to_xpfs_c.inc'

! allocation
     if (.not.associated (mos_to_xpfs_c)) &
          call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

     mos_to_xpfs_c = &
          matmul (mos_to_aos_c_nrmd, aos_c_nrmd_to_xpfs_c)

  case ("aos_s_nrmd")
     
     call provide_bynam("mos_to_xpfs_s")
     call provide_bynam("xpfs_s_to_xpfs_c")
   
     mos_nb = size (mos_to_xpfs_s,1)
     require_mos_s: if(mos_nb == 0) then
        message = 'in routine'*l_here*' nb of mos .eq. 0'
        call require (message, mos_nb .gt. 0)
     end if require_mos_s

     xpfs_s_nb = size (xpfs_s_to_xpfs_c, 1)
     require_xpfs_s: if(xpfs_s_nb == 0) then
        message = 'in routine'*l_here*' nb of xpfs_s .eq. 0'
        call require (message, xpfs_s_nb .gt. 0)
     end if require_xpfs_s

     xpfs_c_nb = size (xpfs_s_to_xpfs_c, 2)
     require_xpfs_c: if(xpfs_c_nb == 0) then
        message = 'in routine'*l_here*' nb of xpfs_c .eq. 0'
        call require (message, xpfs_c_nb .gt. 0)
     end if require_xpfs_c

! allocation
     call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

     mos_to_xpfs_c = matmul (mos_to_xpfs_s, xpfs_s_to_xpfs_c )
  
  case ("xpfs_c")

! mos_coeff_list directly read from QMCMOL file

     call provide_bynam ('xpfs_c')
     call provide_bynam ('mos_coeff_list')

     xpfs_c_nb = size(xpfs_c, 1)
     mocl_nb   = size(mos_coeff_list, 1)
     mos_nb    = mocl_nb/xpfs_c_nb

     if(system%mos_nb > 0) then
        message = 'in routine'*l_here*' system%mos_nb=' &
             +system%mos_nb+' while mos_nb='+mos_nb
        call require (message, system%mos_nb == mos_nb)
     end if
     system%mos_nb = mos_nb

     if(system%mos_nb == 0) then
        message = 'in routine'*l_here*' system%mos_nb = 0'
        call require (message, system%mos_nb .gt. 0)
     end if

     call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

     mos_to_xpfs_c = transpose (reshape (mos_coeff_list, (/xpfs_c_nb, mos_nb/)))

  case default
     message ='unknown mos_on_what >'+mos_on_what+'<' &
          +CR+' should be one of "aos_c", "aos_s_nrmd", "aos_c_nrmd", "xpfs_c"'
     call die (l_here, message)
  end select
  
  mos_nb  = size(mos_to_xpfs_c, 1)
  xpfs_c_nb = size(mos_to_xpfs_c, 2)

  include 'chksiz_mos_to_xpfs_c.inc'

  require_empty: if(count(abs(mos_to_xpfs_c) > 1.d-15 ) == 0) then
     message = 'in routine'*l_here*' mos_to_xpfs_c is empty'
     call die (l_here, message)
  end if require_empty
  
  include 'debug_mos_to_xpfs_c.inc'
  include 'ensnb_mos_to_xpfs_c.inc'
  
  call exit (l_here)

end subroutine mos_to_xpfs_c_bld

! ============================================================================
  subroutine mos_to_aos_c_bld 
! ----------------------------------------------------------------------------
!
! Name           : mos_to_aos_c_bld
!
! Description    : get mos from mos_coeff_list
!
! Needs          : aos_c_names (ao_s)          
! Needs          : mos_coeff_list (mocl)  
!
! Creates        : mos_to_aos_c (mo, ao)
!
! Authors:F. Colonna
! Date  :16 Nov 2001
! Revision: 13 Dec 2001 by F. Colonna Gaussian interface
! Revision: 13 Aug 2002 by F. Colonna Clean up for QMCMOL aos_c
! Revision: 13 Aug 2002 by F. Colonna added header
! Revision: 07 Oct 2002 F. Colonna
! Revision: 07 Nov 2002 F. Colonna GAUSSIAN no stop
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_c_nb
  integer(dpi)                               :: mos_coeff_list_nb
  integer(dpi)                               :: ao, mo
! logical(bool)                              :: l_stop

! header:

  if (header_exe) then

    l_here = "mos_to_aos_c_bld"

    description = 'MOs to AOs cartesian un-normalized transformation matrix'
    call object_create ('mos_to_aos_c')

! Improve: the order is important. check this. read  mos_coeff_list first
    call object_needed ('mos_coeff_list')
    call object_needed ('aos_c_names')

    return

  end if

! begin:
! l_here ='mos_to_aos_c_bld'
  call enter (l_here)

  if(debug) then
    write (logf,*)trim(l_here),': interfaced_program >',trim(interfaced_program),'<'
  end if

  if(interfaced_program == "GAUSSIAN") then

    ! everything done in mos_coeff_list_read
    ! call provide_bynam ('mos_coeff_list')
    mos_nb = system%mos_nb

! l_stop=.true.
  write (logf,*)trim(l_here),': interfaced_program "',trim(interfaced_program),&
                       '" routine should be skipped'
! if(l_stop) then
! write (logf,*)trim(l_here),': ----------- ambiguous mos_coeff_list aos_c normed or not ?' 
! stop ' ----------- ambiguous mos_coeff_list aos_c normed or not ?' 
! end if

  else

    mos_coeff_list_nb = size (mos_coeff_list)
    if(mos_coeff_list_nb == 0) then
      message = 'in routine'*l_here*' nb of mos_coeff_list .eq. 0'
      call require (message, mos_coeff_list_nb .gt. 0)
    end if 

! Improve: should not be called, but it is NOT
!   call check_association ('aos_c_names', aos_c_names, &
!                           'aos_c_names_bld', aos_c_names_bld)

    aos_c_nb =  size(aos_c_names)
    if(aos_c_nb == 0) then
      message = 'in routine'*l_here*' nb of cartesian aos .eq. 0'
      call require (message, aos_c_nb .gt. 0)
    end if 

    mos_nb = mos_coeff_list_nb/aos_c_nb

    if(debug) then
      write (logf,*)trim(l_here),'-d: nb of cartesian aos normed  =',aos_c_nb
      write (logf,*)trim(l_here),'-d: nb of mos coefficients read =',mos_coeff_list_nb 
      write (logf,*)trim(l_here),'-d: nb of mos inferred          =',mos_nb 
    end if

    require_mos_correct: if(mos_coeff_list_nb /= mos_nb*aos_c_nb) then
      message = 'in routine'*l_here*' nb of mos_coeff_list /= mos_nb*aos_c_nb'
      call require (message, mos_coeff_list_nb == mos_nb*aos_c_nb)
    end if require_mos_correct

! redistribute mos
    call alloc ('mos_to_aos_c', mos_to_aos_c, mos_nb, aos_c_nb)

    mos_to_aos_c = &
        transpose (reshape (mos_coeff_list, (/aos_c_nb, mos_nb/)))

!   deallocate (mos_coeff_list)

  end if ! Gaussian

    l_debug: if(debug) then
     write (logf,*)trim(l_here),'-d: mos_to_aos_c :'
     mos_nb = size(mos_to_aos_c, 1)
     aos_c_nb = size (mos_to_aos_c, 2)

     message = 'in routine'*l_here*' nb of mos = 0'
     call require (message, mos_nb /= 0)

     message = 'in routine'*l_here*' nb of aos_c = 0'
     call require (message, aos_c_nb /= 0)

     do mo = 1, mos_nb
      do ao = 1, aos_c_nb
         write (logf,*)trim(l_here),'-d: mos_to_aos_c (',mo,',',ao,') = ', & 
                                   mos_to_aos_c(mo,ao)     
      end do
     end do

    require_mos: if(mos_nb == 0) then
      message = 'in routine'*l_here*' nb of mos .eq. 0'
      call require (message, mos_nb .gt. 0)
    end if require_mos

   end if l_debug

  system%mos_nb = mos_nb
  
  if(system%mos_nb == 0) then
    message = 'in routine'*l_here*' system%mos_nb = 0'
    call require (message, system%mos_nb .gt. 0)
  end if

  call exit (l_here)

  end subroutine mos_to_aos_c_bld

! ============================================================================
  subroutine mos_to_xpfs_s_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_to_xpfs_s_bld
!
! Description : builds matrix mos_to_xpfs_s expressing mos in spherical xpfs
! Description : mos_to_xpfs_s(mo,xp) = mos_to_aos_s_nrmd(mo, ao)
! Description :                         * aos_s_nrmd_to_xpfs_s(ao, xp)
!
! Creates     : mos_to_xpfs_s (mo, xpf_s)
!
! Needs       : mos_to_aos_s_nrmd (mo, ao_s)
! Needs       : aos_s_nrmd_to_xpfs_s (ao_s, xpf_s)
!
! Authors : F. Colonna
! Date    : 27 Sep 2001
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: mo, xp 

! header:

  if (header_exe) then

    l_here = "mos_to_xpfs_s_bld"

<<<<<<< mos_mod.f90
    message = 'MOs to XPFs spherical normalized transformation matrix'
    call object_create ('mos_to_xpfs_s', message)
=======
    description = 'MOs to XPFs spherical normalized transformation matrix'
    call object_create ('mos_to_xpfs_s')
>>>>>>> 1.7

    call object_needed ('mos_to_aos_s_nrmd')
    call object_needed ('aos_s_nrmd_to_xpfs_s')

    return

  end if

! begin:
  call enter (l_here)

! check association
  call check_association  &
        ('mos_to_aos_s_nrmd', mos_to_aos_s_nrmd, &
        'mos_to_aos_s_nrmd_bld', mos_to_aos_s_nrmd_bld)

  call check_association  &
        ('aos_s_nrmd_to_xpfs_s', aos_s_nrmd_to_xpfs_s, &
        'aos_s_nrmd_to_xpfs_s_bld', aos_s_nrmd_to_xpfs_s_bld)

  mos_nb  = size (mos_to_aos_s_nrmd,1)
  aos_nb  = size (mos_to_aos_s_nrmd,2)
  xpfs_nb = size (aos_s_nrmd_to_xpfs_s, 2)

  if(debug) then
    write (logf,*)trim(l_here),': nb of mos =',mos_nb
    write (logf,*)trim(l_here),': nb of spherical xpfs =',xpfs_nb
    write (logf,*)trim(l_here),': nb of spherical aos  =',aos_nb
  end if

! requirements       
  require_mos: if(mos_nb == 0) then
      message = 'in routine'*l_here*' nb of mos .eq. 0'
      call require (message, mos_nb .gt. 0)
  end if require_mos

  require_aos: if(aos_nb == 0) then
      message = 'in routine'*l_here*' nb of aos .eq. 0'
      call require (message, aos_nb .gt. 0)
  end if require_aos

  call check_same_size (l_here, &
    'aos_s_nrmd_to_xpfs_s', size(aos_s_nrmd_to_xpfs_s, 1), 1, &
    'mos_to_aos_s_nrmd',       size(mos_to_aos_s_nrmd, 2), 2)

! allocation
  call alloc ('mos_to_xpfs_s', mos_to_xpfs_s, mos_nb,  xpfs_nb)

   mos_to_xpfs_s =  &
               matmul (mos_to_aos_s_nrmd, aos_s_nrmd_to_xpfs_s)
 
  if(debug) then
   write (logf,*)trim(l_here),'-d: mos_nb=',size(mos_to_xpfs_s,1)
   write (logf,*)trim(l_here),'-d: xpfs_nb=',size(mos_to_xpfs_s,2)
   do mo = 1, size(mos_to_xpfs_s,1)
    do xp = 1, size(mos_to_xpfs_s,2)
     write (logf,*)trim(l_here),'-d: mo # ',mo,' xpfs_s # ',xp, &
           ' mos_to_xpfs_s=',mos_to_xpfs_s(mo,xp)
    end do
   end do
  end  if

  call exit (l_here)

  end subroutine mos_to_xpfs_s_bld

! ============================================================================
  subroutine mos_overlap_bld 
! ----------------------------------------------------------------------------
!
! Name         : mos_overlap_bld
!
! Description  : get cartesian normed mos from mos_coeff_list
!
! Needs        : mos_to_xpfs_c (mo, xpf_c)   
! Needs        : xpfs_c_overlap (xpf_c, xpf_c)
!
! Creates      : mos_overlap (mo, mo)
!
! Authors:F. Colonna
! Date  :16 Nov 2001
! Revision: 13 Dec 2001 by F. Colonna Gaussian interface
! Revision: 07 Oct 2002 F. Colonna
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: mos_nb, mo_i, mo_j
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: mi, mj, xi, xj

! header:

  if (header_exe) then

<<<<<<< mos_mod.f90
    l_here = "mos_overlap_bld"

    message = 'MOs overlap'
    call object_create ('mos_overlap', message)

    call object_needed ('mos_to_xpfs_c')
    call object_needed ('xpfs_c_overlap')

    return

  end if

! begin:
  call enter (l_here)
=======
    l_here = "mos_overlap_bld"

    description = 'MOs overlap'
    call object_create ('mos_overlap')

    call object_needed ('mos_to_xpfs_c')
    call object_needed ('xpfs_c_overlap')

    return

  end if

! begin:
  call enter (l_here)
>>>>>>> 1.7

  mos_nb = size (mos_to_xpfs_c, 1)
  xpfs_nb = size (mos_to_xpfs_c,2)

  if(debug) then
    write (logf,*)trim(l_here),'-d: nb of mos =',mos_nb 
    write (logf,*)trim(l_here),'-d: nb of xpfs=',xpfs_nb 
  end if

  include 'chksiz_mos_to_xpfs_c.inc'

! redistribute mos
  call alloc ('mos_overlap', mos_overlap, mos_nb, mos_nb)

  mos_overlap = 0.d0

  do mi = 1, mos_nb
    do mj = 1, mos_nb
      do xi = 1, xpfs_nb
        if(abs(mos_to_xpfs_c (mi, xi)) <= 1.d-15) cycle
        do xj = 1, xpfs_nb
          if(abs(mos_to_xpfs_c (mj, xj)) <= 1.d-15) cycle
          if(abs(xpfs_c_overlap (xi, xj)) <= 1.d-15) cycle

          mos_overlap (mi, mj) = mos_overlap (mi, mj)       &
                               + mos_to_xpfs_c (mi, xi)  &
                               * mos_to_xpfs_c (mj, xj)  &
                               * xpfs_c_overlap (xi, xj) 
        end do
      end do
    end do
  end do

  include 'debug_mos_overlap.inc'
  include 'ensnb_mos_overlap.inc'

  call exit (l_here)

  end subroutine mos_overlap_bld

! ========================================================================
  subroutine mos_write_menu
! ------------------------------------------------------------------------
!
! Name           : mos_write_menu
!
! Description    : input for mos_write
!
! Author: F. Colonna
! Date  : 16 Nov 2000
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: done

! begin:
  l_here ='mos_write_menu'
  call enter (l_here) 

! initialize:

  done = .false.
  call next_command_get ('mos_write')
  do while (.not.done)
   call next_word_get('mos_write')
   if(trim(cur_word) .eq. 'help') then

    write (logf,'(a)') 'mos_write-h: write mos'
    write (logf,'(a)') 'mos_write-h: syntax:'
    write (logf,'(a)') 'mos_write-h: mos'
    write (logf,'(a)') 'mos_write-h:  help'
    write (logf,'(a)') 'mos_write-h:  mos_write_on{_what} = [string]'
    write (logf,'(a)') 'mos_write-h:    [string] :aos_c:aos_c_normed:'
    write (logf,'(a)') 'mos_write-h:    [string] :aos_s:aos_s_normed:'
    write (logf,'(a)') 'mos_write-h:    [string] :xpfs_c:xpfs_c_normed:'
    write (logf,'(a)') 'mos_write-h:    [string] :xpfs_s:xpfs_s_normed:'
    write (logf,'(a)') 'mos_write-h: end'
    write (logf,'(a)') 'mos_write-h: example'
    write (logf,'(a)') 'mos_write-h: mos mos_write_on_what = aos_c end'

   else if(token_is ('mos_write_on', mos_write_on_what)) then

   else
    call menu_exit (l_here, done)
   end if

  end do

  write (logf,*)trim(l_here),': mos_write_on_what >',trim(mos_write_on_what),'<'

  call mos_write

  call exit (l_here)

  end subroutine mos_write_menu

! ============================================================================
  subroutine mos_write
! ----------------------------------------------------------------------------
!
! Name           : mos_write
!
! Description    : writes Molecular Orbitals
!
! Authors:F.Colonna
! Date  :07 Oct 2002
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  character(len=max_string_len_file_type)    :: f_type 
  integer(dpi)                               :: mo, ao, xp
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: xpfs_nb
  integer(dpi)                               :: unit  

! begin:
  l_here = "mos_write"
  call enter (l_here)

  call provide_bynam ('mos_names')

  mos_nb = size (mos_names)

  if(debug) then
    write (logf,*)trim(l_here),'-d: mos_write_on_what >',trim(mos_write_on_what),'<'
    write (logf,*)trim(l_here),'-d: nb of mos =',mos_nb
  end if

  include 'chksiz_mos_names.inc'

  f_type   = 'mos-out'//blanks
  unit     = file_open_and_cat_from_type (f_type)

  message  = 'in routine'*l_here*' mos-out-file unit not found'
  call require (message, unit /= 0)

  f_name   = files_names(unit)
  f_action = files_actions(unit)  

  if(debug) then
   write (logf,*)trim(l_here),'-d: mos file unit   =',unit
   write (logf,*)trim(l_here),'-d: mos file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: mos file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: mos file action >',trim(f_action),'<'
  end if

! header
  call file_header_write (unit)
  write(unit,'(a,i5)') 'number mos  = ',mos_nb
  write(unit,'(2a)')   'mos_on_what = ',trim(mos_write_on_what)
  write(unit,'(a)')    'end_of_header'

  select case (mos_write_on_what)

  case ("aos_c")

    write (logf,'(1x,2a,i3,a)')  &
    trim(l_here),': ',mos_nb,' mos on un-normalized cartesian aos'
 
		call provide_bynam ('aos_c_names')
    call provide_bynam ('mos_to_aos_c')

!   require_aos: 
    aos_nb = size (mos_to_aos_c, 2)
    if(aos_nb <= 0) call die (l_here, ' nb of cartesian aos ='+aos_nb)
    
    do mo = 1, mos_nb
     write(unit,'(5d25.15)') (mos_to_aos_c (mo,ao), ao= 1, aos_nb)
    end do

  case ("aos_c_nrmd")

    write (logf,*)trim(l_here),': ',mos_nb,' mos on normalized cartesian aos'
 
		call provide_bynam ('aos_c_names')

    aos_nb = size (mos_to_aos_c_nrmd, 2)
!   require_aos: 
    if(aos_nb <= 0) call die (l_here, ' nb of cartesian aos ='+aos_nb)
    
    do mo = 1, mos_nb
     write(unit,'(5d25.15)') (mos_to_aos_c_nrmd (mo,ao), ao= 1, aos_nb)
    end do

  case ("aos_s_nrmd")

		call provide_bynam ('aos_s_names')

!   require_aos: 
    aos_nb = size (mos_to_aos_s_nrmd, 2)
    if(aos_nb <= 0) call die (l_here, ' nb of spherical aos ='+aos_nb)
    
    write (logf,*)trim(l_here),': ',mos_nb,' mos on normalized spherical aos'
  
    do mo = 1, mos_nb
     write(unit,'(5d25.15)') (mos_to_aos_s_nrmd (mo,ao), ao= 1, aos_nb)
    end do

  case ("xpfs_c")

    write (logf,'(1x,2a,i3,a)')  &
    trim(l_here),': ',mos_nb,' mos on un-normalized cartesian xpfs'
 
    call provide_bynam ('mos_to_xpfs_c')

    mos_nb = size (mos_to_xpfs_c, 1)
    xpfs_nb = size (mos_to_xpfs_c, 2)
    if(mos_nb <= 0) call die (l_here, ' nb of cartesian mos ='+mos_nb)
    if(xpfs_nb <= 0) call die (l_here, ' nb of cartesian xpf ='+xpfs_nb)
    
    do mo = 1, mos_nb
      write(unit,'(5d25.15)') (mos_to_xpfs_c (mo, xp),  xp= 1, xpfs_nb)
    end do

  case ("xpfs_c_nrmd")

    write (logf,'(1x,2a,i3,a)')  &
    trim(l_here),': ',mos_nb,' mos on normalized cartesian xpfs'
 
    write (logf,*)trim(l_here),'-e: mos on normalized cartesian xpfs not yet implemented'
    stop ' mos on normalized cartesian xpfs not yet implemented'

!   call provide_bynam ('mos_to_xpfs_c_nrmd')

!   mos_nb = size (mos_to_xpfs_c_nrmd, 1)
!   xpfs_nb = size (mos_to_xpfs_c_nrmd, 2)
!   if(mos_nb <= 0) call die (l_here, ' nb of cartesian mos ='+mos_nb)
!   if(xpfs_nb <= 0) call die (l_here, ' nb of cartesian xpf ='+xpfs_nb)
    
!   do mo = 1, mos_nb
!     write(unit,'(5d25.15)') (mos_to_xpfs_c_nrmd (mo, xp), xp= 1, xpfs_nb)
!   end do

  case default
    message ='unknown mos_on_what '*mos_on_what
    call die (l_here, message)
  end select

  write(unit,'(a)') 'end_of_mos'

  write (logf,*)trim(l_here),': written on file >',trim(f_name),'<'

  call exit (l_here)

 end subroutine mos_write 

<<<<<<< mos_mod.f90

! these routines are no more used have been explicitly included in mos_to_xpfs_c_bld

! ============================================================================
  subroutine mos_on_aos_c_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_on_aos_c_to_xpfs_c_bld
!
! Description : called when mos_on_what == aos_c
! Description : builds matrix mos_to_xpfs_c which express mos in xpfs_c
! Description : through aos_c (unnormalized)
! Description : mos_to_xpfs_c(mo,xp) = mos_to_aos_c(mo, ao)
! Description :                       * aos_c_to_xpfs_c(ao, xp)
!
! -- Creates     : mos_to_xpfs_c (mo, xpf_c)
!
! -- Needs       : mos_to_aos_c (mo, ao_c)
! -- Needs       : aos_c_to_xpfs_c (ao_c, xpf_c)
!
! Authors : F. Colonna
! Date    : 16 Nov 2001 
! Improve: ao => ao_i xp=> xp_i etc...
! Revision: 13 Aug 2002 by F. Colonna 
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)  :: mos_nb, mo_i
  integer(dpi)  :: aos_c_nb
  integer(dpi)  :: xpfs_c_nb, xpf_c_i

! header:

  if (header_exe) then

    l_here = "mos_on_aos_c_to_xpfs_c_bld"

    message = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_on_aos_c_to_xpfs_c', message)

    call object_needed ('mos_to_aos_c')
    call object_needed ('aos_c_to_xpfs_c')

    return

  end if

! begin:
  l_here ='mos_on_aos_c_to_xpfs_c_bld'
  call enter (l_here)

  include 'chksiz_mos_to_aos_c.inc'
  include 'chksiz_aos_c_to_xpfs_c.inc'

  mos_nb    = size (mos_to_aos_c, 1)
  aos_c_nb  = size (mos_to_aos_c, 2)
  xpfs_c_nb = size (aos_c_to_xpfs_c, 2)

  if(debug) then
    write (logf,*)trim(l_here),': mos_nb    =',mos_nb
    write (logf,*)trim(l_here),': aos_c_nb  =',aos_c_nb
    write (logf,*)trim(l_here),': xpfs_c_nb =',xpfs_c_nb
  end if

! requirements

! allocation
  if (.not.associated (mos_to_xpfs_c)) then 
    write (logf,*)trim(l_here),': mos_to_xpfs_c not yet associated'
    call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)
  else
    write (logf,*)trim(l_here),': mos_to_xpfs_c left in memory'
  end if

  mos_to_xpfs_c = &
     matmul (mos_to_aos_c, aos_c_to_xpfs_c)
 
  include 'debug_mos_to_xpfs_c.inc' 
  include 'ensnb_mos_to_xpfs_c.inc'

  call exit (l_here)

  end subroutine mos_on_aos_c_to_xpfs_c_bld


! ============================================================================
  subroutine mos_on_aos_c_nrmd_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_on_aos_c_nrmd_to_xpfs_c_bld
!
! Description : used whem mos_on_what == aos_c_nrmd
! Description : builds matrix mos_to_xpfs_c which express mos in xpfs_c
! Description : mos_to_xpfs_c(mo,xp) = mos_to_aos_c_nrmd (mo, ao)
! Description :                      * aos_c_nrmd_to_xpfs_c (ao, xp)
!
! Improve     ; mos_on_aos_c_nrmd_to_xpfs_c does not EXIST
! Improve     ; it is in fact mos_to_xpfs_c 
!
! -- Creates     : mos_to_xpfs_c (mo, xpf_c)
!
! -- Needs       : mos_to_aos_c_nrmd (mo, ao_c)
! -- Needs       : aos_c_nrmd_to_xpfs_c (ao_c, xpf_c)
!
! Authors : F. Colonna
! Date    : 27 Sep 2001 
! Revision: 16 Nov 2001 by F. Colonna aos_c_nrmd  
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: mo, xp

! header:

  if (header_exe) then

    l_here = "mos_on_aos_c_nrmd_to_xpfs_c_bld"

    message = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_on_aos_c_nrmd_to_xpfs_c', message)

    call object_needed ('mos_to_aos_c_nrmd')
    call object_needed ('aos_c_nrmd_to_xpfs_c')

    return

  end if


! begin:
  l_here ='mos_on_aos_c_nrmd_to_xpfs_c_bld'
  call enter (l_here)

  mos_nb    = size (mos_to_aos_c_nrmd,1)
  aos_nb    = size (mos_to_aos_c_nrmd,2)
  xpfs_c_nb = size (aos_c_nrmd_to_xpfs_c, 2)

  write (logf,*)trim(l_here),': nb of mos =',mos_nb
  write (logf,*)trim(l_here),': nb of cartesian aos =',aos_nb
  write (logf,*)trim(l_here),': nb of cartesian xpfs =',xpfs_c_nb

  include 'chksiz_mos_to_aos_c_nrmd.inc'
  include 'chksiz_aos_c_nrmd_to_xpfs_c.inc'

! allocation
  if (.not.associated (mos_to_xpfs_c)) &
  call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

  mos_to_xpfs_c = &
     matmul (mos_to_aos_c_nrmd, aos_c_nrmd_to_xpfs_c)
   if(debug) then
    do mo = 1, mos_nb
     write (logf,'(a)')' '
     do xp = 1, xpfs_c_nb
       write (logf,'(2a,i3,a,i3,a,d25.15)')   &
       trim(l_here),': mo # ',mo,' xpf # ',xp,' mos_to_xpfs_c =', &
                       mos_to_xpfs_c(mo,xp)
     end do
    end do

  end if

  call exit (l_here)

  end subroutine mos_on_aos_c_nrmd_to_xpfs_c_bld

! ============================================================================
  subroutine mos_on_aos_s_nrmd_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_on_aos_s_nrmd_to_xpfs_c_bld
!
! Description : if mos_on_what == aos_s_nrmd
! Description : builds matrix mos_to_xpfs_c which express mos in xpfs
! Description : mos_to_xpfs_c(mo,cp) = mos_to_xpfs_s(mo, sp)
! Description :                      * xpfs_s_to_xpfs_c(sp, cp)
!
! -- Creates     : mos_to_xpfs_c (mo,xpf) from mos_on_aos_s_nrmd
!
! -- Needs       : mos_to_xpfs_s (mo, xpf_s) 
! -- Needs       : xpfs_s_to_xpfs_c (xpf_s, xpf_c)
!
! Authors : F. Colonna
! Date    : 27 Sep 2001 
! Revision: 16 Nov 2001 by F. Colonna aos_s_nrmd  
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: xpfs_s_nb

! header:

  if (header_exe) then

    l_here = "mos_on_aos_s_nrmd_to_xpfs_c_bld"

    message = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_on_aos_s_nrmd_to_xpfs_c', message)

    call object_needed ('mos_to_xpfs_s')
    call object_needed ('xpfs_s_to_xpfs_c')

    return

  end if

! begin:
	
  l_here ='mos_on_aos_s_nrmd_to_xpfs_c_bld'
  call enter (l_here)
 
! check association
! call check_association  &
!       ('mos_to_xpfs_s', mos_to_xpfs_s, &
!       'mos_to_xpfs_s_bld', mos_to_xpfs_s_bld)

! call check_association  &
!       ('xpfs_s_to_xpfs_c', xpfs_s_to_xpfs_c, &
!       'xpfs_s_to_xpfs_c_bld', xpfs_s_to_xpfs_c_bld)
!  
! requirements
  mos_nb = size (mos_to_xpfs_s,1)
  require_mos_s: if(mos_nb == 0) then
      message = 'in routine'*l_here*' nb of mos .eq. 0'
      call require (message, mos_nb .gt. 0)
  end if require_mos_s

  xpfs_s_nb = size (xpfs_s_to_xpfs_c, 1)
    require_xpfs_s: if(xpfs_s_nb == 0) then
      message = 'in routine'*l_here*' nb of xpfs_s .eq. 0'
      call require (message, xpfs_s_nb .gt. 0)
  end if require_xpfs_s

  xpfs_c_nb = size (xpfs_s_to_xpfs_c, 2)
  require_xpfs_c: if(xpfs_c_nb == 0) then
      message = 'in routine'*l_here*' nb of xpfs_c .eq. 0'
      call require (message, xpfs_c_nb .gt. 0)
  end if require_xpfs_c

! allocation
  if (.not.associated (mos_to_xpfs_c)) &
  call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

  mos_to_xpfs_c = matmul (mos_to_xpfs_s, xpfs_s_to_xpfs_c )
 
  call exit (l_here)

  end subroutine mos_on_aos_s_nrmd_to_xpfs_c_bld

=======

! these routines are no more used have been explicitly included in mos_to_xpfs_c_bld

! ============================================================================
  subroutine mos_on_aos_c_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_on_aos_c_to_xpfs_c_bld
!
! Description : called when mos_on_what == aos_c
! Description : builds matrix mos_to_xpfs_c which express mos in xpfs_c
! Description : through aos_c (unnormalized)
! Description : mos_to_xpfs_c(mo,xp) = mos_to_aos_c(mo, ao)
! Description :                       * aos_c_to_xpfs_c(ao, xp)
!
! -- Creates     : mos_to_xpfs_c (mo, xpf_c)
!
! -- Needs       : mos_to_aos_c (mo, ao_c)
! -- Needs       : aos_c_to_xpfs_c (ao_c, xpf_c)
!
! Authors : F. Colonna
! Date    : 16 Nov 2001 
! Improve: ao => ao_i xp=> xp_i etc...
! Revision: 13 Aug 2002 by F. Colonna 
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)  :: mos_nb, mo_i
  integer(dpi)  :: aos_c_nb
  integer(dpi)  :: xpfs_c_nb, xpf_c_i

! header:

  if (header_exe) then

    l_here = "mos_on_aos_c_to_xpfs_c_bld"

    description = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_on_aos_c_to_xpfs_c')

    call object_needed ('mos_to_aos_c')
    call object_needed ('aos_c_to_xpfs_c')

    return

  end if

! begin:
  l_here ='mos_on_aos_c_to_xpfs_c_bld'
  call enter (l_here)

  include 'chksiz_mos_to_aos_c.inc'
  include 'chksiz_aos_c_to_xpfs_c.inc'

  mos_nb    = size (mos_to_aos_c, 1)
  aos_c_nb  = size (mos_to_aos_c, 2)
  xpfs_c_nb = size (aos_c_to_xpfs_c, 2)

  if(debug) then
    write (logf,*)trim(l_here),': mos_nb    =',mos_nb
    write (logf,*)trim(l_here),': aos_c_nb  =',aos_c_nb
    write (logf,*)trim(l_here),': xpfs_c_nb =',xpfs_c_nb
  end if

! requirements

! allocation
  if (.not.associated (mos_to_xpfs_c)) then 
    write (logf,*)trim(l_here),': mos_to_xpfs_c not yet associated'
    call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)
  else
    write (logf,*)trim(l_here),': mos_to_xpfs_c left in memory'
  end if

  mos_to_xpfs_c = &
     matmul (mos_to_aos_c, aos_c_to_xpfs_c)
 
  include 'debug_mos_to_xpfs_c.inc' 
  include 'ensnb_mos_to_xpfs_c.inc'

  call exit (l_here)

  end subroutine mos_on_aos_c_to_xpfs_c_bld


! ============================================================================
  subroutine mos_on_aos_c_nrmd_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_on_aos_c_nrmd_to_xpfs_c_bld
!
! Description : used whem mos_on_what == aos_c_nrmd
! Description : builds matrix mos_to_xpfs_c which express mos in xpfs_c
! Description : mos_to_xpfs_c(mo,xp) = mos_to_aos_c_nrmd (mo, ao)
! Description :                      * aos_c_nrmd_to_xpfs_c (ao, xp)
!
! Improve     ; mos_on_aos_c_nrmd_to_xpfs_c does not EXIST
! Improve     ; it is in fact mos_to_xpfs_c 
!
! -- Creates     : mos_to_xpfs_c (mo, xpf_c)
!
! -- Needs       : mos_to_aos_c_nrmd (mo, ao_c)
! -- Needs       : aos_c_nrmd_to_xpfs_c (ao_c, xpf_c)
!
! Authors : F. Colonna
! Date    : 27 Sep 2001 
! Revision: 16 Nov 2001 by F. Colonna aos_c_nrmd  
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: aos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: mo, xp

! header:

  if (header_exe) then

    l_here = "mos_on_aos_c_nrmd_to_xpfs_c_bld"

    description = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_on_aos_c_nrmd_to_xpfs_c')

    call object_needed ('mos_to_aos_c_nrmd')
    call object_needed ('aos_c_nrmd_to_xpfs_c')

    return

  end if


! begin:
  l_here ='mos_on_aos_c_nrmd_to_xpfs_c_bld'
  call enter (l_here)

  mos_nb    = size (mos_to_aos_c_nrmd,1)
  aos_nb    = size (mos_to_aos_c_nrmd,2)
  xpfs_c_nb = size (aos_c_nrmd_to_xpfs_c, 2)

  write (logf,*)trim(l_here),': nb of mos =',mos_nb
  write (logf,*)trim(l_here),': nb of cartesian aos =',aos_nb
  write (logf,*)trim(l_here),': nb of cartesian xpfs =',xpfs_c_nb

  include 'chksiz_mos_to_aos_c_nrmd.inc'
  include 'chksiz_aos_c_nrmd_to_xpfs_c.inc'

! allocation
  if (.not.associated (mos_to_xpfs_c)) &
  call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

  mos_to_xpfs_c = &
     matmul (mos_to_aos_c_nrmd, aos_c_nrmd_to_xpfs_c)
   if(debug) then
    do mo = 1, mos_nb
     write (logf,'(a)')' '
     do xp = 1, xpfs_c_nb
       write (logf,'(2a,i3,a,i3,a,d25.15)')   &
       trim(l_here),': mo # ',mo,' xpf # ',xp,' mos_to_xpfs_c =', &
                       mos_to_xpfs_c(mo,xp)
     end do
    end do

  end if

  call exit (l_here)

  end subroutine mos_on_aos_c_nrmd_to_xpfs_c_bld

! ============================================================================
  subroutine mos_on_aos_s_nrmd_to_xpfs_c_bld
! ----------------------------------------------------------------------------
!
! Name        : mos_on_aos_s_nrmd_to_xpfs_c_bld
!
! Description : if mos_on_what == aos_s_nrmd
! Description : builds matrix mos_to_xpfs_c which express mos in xpfs
! Description : mos_to_xpfs_c(mo,cp) = mos_to_xpfs_s(mo, sp)
! Description :                      * xpfs_s_to_xpfs_c(sp, cp)
!
! -- Creates     : mos_to_xpfs_c (mo,xpf) from mos_on_aos_s_nrmd
!
! -- Needs       : mos_to_xpfs_s (mo, xpf_s) 
! -- Needs       : xpfs_s_to_xpfs_c (xpf_s, xpf_c)
!
! Authors : F. Colonna
! Date    : 27 Sep 2001 
! Revision: 16 Nov 2001 by F. Colonna aos_s_nrmd  
! ----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: mos_nb
  integer(dpi)                               :: xpfs_c_nb
  integer(dpi)                               :: xpfs_s_nb

! header:

  if (header_exe) then

    l_here = "mos_on_aos_s_nrmd_to_xpfs_c_bld"

    description = 'MOs to XPFs cartesian un-normalized transformation matrix'
    call object_create ('mos_on_aos_s_nrmd_to_xpfs_c')

    call object_needed ('mos_to_xpfs_s')
    call object_needed ('xpfs_s_to_xpfs_c')

    return

  end if

! begin:
	
  l_here ='mos_on_aos_s_nrmd_to_xpfs_c_bld'
  call enter (l_here)
 
! check association
! call check_association  &
!       ('mos_to_xpfs_s', mos_to_xpfs_s, &
!       'mos_to_xpfs_s_bld', mos_to_xpfs_s_bld)

! call check_association  &
!       ('xpfs_s_to_xpfs_c', xpfs_s_to_xpfs_c, &
!       'xpfs_s_to_xpfs_c_bld', xpfs_s_to_xpfs_c_bld)
!  
! requirements
  mos_nb = size (mos_to_xpfs_s,1)
  require_mos_s: if(mos_nb == 0) then
      message = 'in routine'*l_here*' nb of mos .eq. 0'
      call require (message, mos_nb .gt. 0)
  end if require_mos_s

  xpfs_s_nb = size (xpfs_s_to_xpfs_c, 1)
    require_xpfs_s: if(xpfs_s_nb == 0) then
      message = 'in routine'*l_here*' nb of xpfs_s .eq. 0'
      call require (message, xpfs_s_nb .gt. 0)
  end if require_xpfs_s

  xpfs_c_nb = size (xpfs_s_to_xpfs_c, 2)
  require_xpfs_c: if(xpfs_c_nb == 0) then
      message = 'in routine'*l_here*' nb of xpfs_c .eq. 0'
      call require (message, xpfs_c_nb .gt. 0)
  end if require_xpfs_c

! allocation
  if (.not.associated (mos_to_xpfs_c)) &
  call alloc ('mos_to_xpfs_c', mos_to_xpfs_c, mos_nb, xpfs_c_nb)

  mos_to_xpfs_c = matmul (mos_to_xpfs_s, xpfs_s_to_xpfs_c )
 
  call exit (l_here)

  end subroutine mos_on_aos_s_nrmd_to_xpfs_c_bld

>>>>>>> 1.7
end module mos_mod
