! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
!n for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module mos_on_grid_mod

  use basic_types_tools_mod
  use mos_bywlk_mod
  use ecp_mod
  use walkers_bywlk_mod
  implicit none
  real(dp), pointer :: r_quad_atom_mos_grid (:,:,:,:) => NULL()
  

  real(dp), allocatable  :: coef_ecp_l(:,:)
  integer(dpi)           :: radial_mesh_nb = 4000   
contains


! =======================================================  
  subroutine r_quad_atom_mos_grid_bld 
! -------------------------------------------------------  
! Name :  r_quad_atom_mos_grid_bld
!
! Create :  r_quad_atom_mos_grid (radial_mesh, quad, atom, mos)
!
!
! Needs : list_ecp_atoms (ecp_i)
!
! Description:  mos values on spherical grids centered on atoms 
! 
! Authors    : R. Assaraf
! Date       : 18 Apr 2005
! ------------------------------------------------------- 
  implicit none
    
! i/o:

! local:
  integer(dpi) :: ecp_atoms_nb, walkers_nb, dimensions_nb, electrons_nb
  integer(dpi) :: mos_nb
  integer(dpi) :: atom_i, elec_i, ecp_i, walk_i, i_r
  integer(dpi) :: iquad
  real(dp)     :: radial_distance
  real(dp), pointer      :: coor_save(:,:,:)     => NULL ()
  character(max_string_len) :: l_here

! header:

  if (header_exe) then

    l_here = "r_quad_atom_mos_grid_bld"

    description = "mos_values on spherical grids centered on atoms"
    call object_create ('r_quad_atom_mos_grid')

    return

  end if

! begin:
  l_here ='r_quad_atom_mos_grid_bld'
  call enter (l_here)
  if (ecp_method == 'grid') then
! dimensions:
  if (.not.associated (elec_coor_bywlk))  call provide_bynam('elec_coor_bywlk')
  if (.not.associated (atoms_coordinates)) call provide_bynam ('atoms_coordinates')
  if (.not.associated (mos_bywlk)) call provide_bynam ('mos_bywlk')
  if (.not.associated (list_ecp_atoms)) call provide_bynam ('list_ecp_atoms')

  electrons_nb = size (elec_coor_bywlk, dim=1)
  ecp_atoms_nb = size (list_ecp_atoms)
  dimensions_nb= size (elec_coor_bywlk, dim=2)
  walkers_nb   = size (elec_coor_bywlk, dim=3)
  mos_nb       = size (mos_bywlk, dim=2)
  ecp_atoms_nb = size (list_ecp_atoms)


! allocations: 
 
 if (.not.associated(coor_save)) then
     allocate (coor_save (electrons_nb, dimensions_nb, walkers_nb))
     call pseudo_init
  endif
  
  coor_save = elec_coor_bywlk

  
  if (.not.associated(r_quad_atom_mos_grid)) then
       call alloc ('r_quad_atom_mos_grid',r_quad_atom_mos_grid, &
                    radial_mesh_nb, n_quad, ecp_atoms_nb, mos_nb ) 
  endif
! end allocations 
  
  elec_i=1
  walk_i=1
  
do ecp_i=1, size (list_ecp_atoms)
     atom_i=list_ecp_atoms (ecp_i)
     
     do  i_r = 1, radial_mesh_nb 
        radial_distance = i_r * range (atom_i) / radial_mesh_nb
        QUAD_LOOP: do iquad = 1, n_quad
       
! absolute coordinates of the quadrature points
 
           elec_coor_bywlk(elec_i,1,walk_i) =&
                radial_distance * quad(iquad,1) &
                + atoms_coordinates(atom_i, 1)
   
           elec_coor_bywlk(elec_i,2,walk_i) =&
                radial_distance * quad(iquad,2) &
                + atoms_coordinates(atom_i, 2)

           elec_coor_bywlk(elec_i,3,walk_i) =&
                radial_distance * quad(iquad,3) &
                + atoms_coordinates (atom_i, 3)
           
           elec_valid_bywlk = .true.
           elec_valid_bywlk (elec_i,walk_i) = .false.
                  
           only_one_electron = elec_i
   
           call mos_from_coor_bywlk_calc
           r_quad_atom_mos_grid (i_r, iquad, ecp_i, :) = mos_bywlk (elec_i, :, walk_i)     
        enddo QUAD_LOOP
      enddo ! do i_r
  
   enddo ! do ecp_i

   elec_coor_bywlk = coor_save
   call mos_from_coor_bywlk_calc 
   elec_valid_bywlk = .true.
   only_one_electron = 0

  endif

  call exit (l_here)

end subroutine r_quad_atom_mos_grid_bld 

! ============================================================================
 subroutine mos_from_coor_bywlk_calc 
! ----------------------------------------------------------------------------
!
! Name           : 
!
! Argument (in ) : 
!
! Description    : 
!
! Authors:
! ----------------------------------------------------------------------------
  implicit none

! i/o:
  
! result:

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi), pointer :: path_bynod (:) => NULL ()
! begin:
  l_here ='mos_from_coor_bywlk_calc'
  call enter (l_here)
  

  if (.not.associated (path_bynod)) then
     
     call path_bynod_bld (path_bynod, (/object_index('elec_coor_bywlk')/),&
          (/object_index('mos_bywlk')/))
  endif
 
  call path_bynod_exe (path_bynod)
 
  
  call exit (l_here)
  
end subroutine mos_from_coor_bywlk_calc
  

end module mos_on_grid_mod
