! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module mos_sto_bywlk_mod

  use basic_types_tools_mod
  use mos_mod
  use mo_ao_s_coeffs_mod
  use ea_bywlk_mod
  use mos_bywlk_mod
  
 real(dp), pointer :: rad_ea_bywlk (:,:,:,:)        => NULL()
 real(dp), pointer :: rad_ea_grad_bywlk (:,:,:,:,:) => NULL()
 real(dp), pointer :: rad_ea_lapl_bywlk (:,:,:,:)   => NULL()
  
 real(dp), pointer :: mos_sto_bywlk (:,:,:) => NULL()
 real(dp), pointer :: mos_sto_grad_bywlk (:,:,:,:) => NULL()
 real(dp), pointer :: mos_sto_lapl_bywlk (:,:,:)    => NULL()


 real(dp), pointer :: mos_xpf_xpf_dot (:,:,:) => NULL()

 real(dp), pointer   :: xpfs_c2_bywlk (:,:,:) => NULL()
 real(dp), pointer   :: xpfs_c2_lapl_bywlk (:,:,:)   => NULL()
 real(dp), pointer   :: xpfs_c2_grad_bywlk (:,:,:,:) => NULL()
 real(dp), pointer   :: mos_on_xpf_new (:,:)  => NULL ()

  integer(dpi), pointer  :: xpfs_c2_nb => NULL  ()
  character(len=max_len_epf_type), pointer :: xpfs_c2_type  (:) => NULL ()
  real (dp), pointer    :: xpfs_c2_exponents (:) => NULL ()
  integer(dpi), pointer :: xpfs_c2_center    (:) => NULL ()
  integer(dpi), pointer :: xpfs_c2_r_power   (:) => NULL ()
  integer(dpi), pointer :: xpfs_c2_x_power   (:) => NULL ()
  integer(dpi), pointer :: xpfs_c2_y_power   (:) => NULL ()
  integer(dpi), pointer :: xpfs_c2_z_power   (:) => NULL ()


contains

! ==========================================================================
  subroutine mos_sto_grla_bywlk_bld
! --------------------------------------------------------------------------
!
! Name     : mos_sto_grla_bywlk_bld
!
! Creates :  mos_sto_bywlk (elec, mo, walk)
! Creates :  mos_sto_grad_bywlk (elec, mo, dim, walk)
! Creates :  mos_sto_lapl_bywlk (elec, mo, walk)
!
! Author: R. Assaraf
! Date  : 26 Apr 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) ::  electrons_nb,  walkers_nb, dimensions_nb, aos_nb, mos_nb
  integer(dpi) :: walk_i, elec_i, ao_i, xpf_c_i, s_i, mo_i, i 
  integer(dpi) :: atom_i, px, py, pz,  z
  real(dp)     :: monomx,monomy,monomz,polyn
  real(dp)     :: epf_loc, epf_grad_loc(3), epf_lapl_loc
  real(dp)     :: plapl,d2xmonomx,d2ymonomy,d2zmonomz,dxmonomx,dymonomy,dzmonomz
  real(dp)     :: pgrad (3)
! header:

  if (header_exe) then

     l_here = "mos_sto_grla_bywlk_bld"
     call object_create ('mos_sto_bywlk')
     call object_create ('mos_sto_grad_bywlk')
     call object_create ('mos_sto_lapl_bywlk')
 
     call object_needed ('rad_ea_bywlk')
     call object_needed ('rad_ea_grad_bywlk')
     call object_needed ('rad_ea_lapl_bywlk')
     call object_needed ('ao_first_in_xpfs_c')
     call object_needed ('mo_ao_s_coeffs')
     call object_needed ('atoms')
     call object_needed ('slater_basis_dim')
     call object_needed ('ea_monom_coor_bywlk')
     return
  endif

  call enter (l_here)

! dimensions

  walkers_nb = size (rad_ea_bywlk, dim = 4)

  electrons_nb = size (rad_ea_bywlk, dim = 2)
  aos_nb   = size (ao_first_in_xpfs_c)
  dimensions_nb = size (rad_ea_grad_bywlk, dim = 4)

  mos_nb = size (mo_ao_s_coeffs, dim = 1)
! allocations
  if (.not.associated (mos_sto_bywlk)) then
     call alloc ('mos_sto_bywlk',mos_sto_bywlk, &
          electrons_nb, mos_nb, walkers_nb)
 endif

 if (.not.associated (mos_sto_grad_bywlk)) then
    call alloc ('mos_sto_grad_bywlk',mos_sto_grad_bywlk, &
          electrons_nb, mos_nb, dimensions_nb, walkers_nb)
 endif

 if (.not.associated (mos_sto_lapl_bywlk)) then
    call alloc ('mos_sto_lapl_bywlk',mos_sto_lapl_bywlk, &
          electrons_nb, mos_nb, walkers_nb)
 endif

!$OMP PARALLEL DEFAULT(PRIVATE) SHARED(mos_sto_bywlk,mos_sto_grad_bywlk,mos_sto_lapl_bywlk,rad_ea_bywlk,rad_ea_grad_bywlk,rad_ea_lapl_bywlk,ao_first_in_xpfs_c,mo_ao_s_coeffs,atoms,slater_basis_dim,ea_monom_coor_bywlk,xpfs_c)
 mos_sto_bywlk = 0.d0
 mos_sto_grad_bywlk = 0.d0
 mos_sto_lapl_bywlk = 0.d0
!$OMP DO
 do walk_i = 1, walkers_nb
    do mo_i = 1, mos_nb
       do elec_i = 1, electrons_nb
          do ao_i = 1, aos_nb

          
             xpf_c_i = ao_first_in_xpfs_c (ao_i)
             px = xpfs_c (xpf_c_i)%x_power
             py = xpfs_c (xpf_c_i)%y_power
             pz = xpfs_c (xpf_c_i)%z_power
             atom_i = xpfs_c (xpf_c_i)%center
             z = atoms (atom_i)%charge

             epf_loc  = 0.d0
             epf_grad_loc = 0.d0
             epf_lapl_loc = 0.d0
   
             do s_i = 1, slater_basis_dim (z)
                epf_loc = epf_loc + mo_ao_s_coeffs (mo_i,ao_i,s_i) * &
                     rad_ea_bywlk (s_i, elec_i, atom_i, walk_i)
                epf_grad_loc = epf_grad_loc + mo_ao_s_coeffs (mo_i,ao_i,s_i) * &
                     rad_ea_grad_bywlk (s_i, elec_i, atom_i, :, walk_i)
                epf_lapl_loc = epf_lapl_loc + mo_ao_s_coeffs (mo_i,ao_i,s_i) * &
                     rad_ea_lapl_bywlk (s_i, elec_i, atom_i, walk_i)
                
             enddo ! do s_i

             if (debug) then
                write (logf,*) trim(l_here), '-d: ao_i,elec_i,mo_i, walk_i, atom_i,  z =',&
                     ao_i,elec_i,mo_i, walk_i, atom_i, z 
               
                write (logf,*) trim(l_here), '-d: mo_ao_s_coeffs (mo_i,ao_i,:)',&
                     mo_ao_s_coeffs (mo_i,ao_i,:)
                write (logf,*) trim(l_here), '-d: rad_ea_bywlk (:,elec_i, atom_i, walk_i)' &
                     ,rad_ea_bywlk (:,elec_i, atom_i, walk_i)
                write (logf,*) trim(l_here), '-d:  product: epf_loc ', epf_loc
                write (logf,*) trim(l_here), '-d: epf_grad_loc:', epf_grad_loc
                write (logf,*) trim(l_here), '-d: epf_lapl_loc:', epf_lapl_loc
             endif
             

             monomx = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
             monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
             monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)
             polyn = monomx*monomy*monomz

             mos_sto_bywlk (elec_i,mo_i,walk_i) = mos_sto_bywlk (elec_i,mo_i,walk_i)+ epf_loc * polyn
 
! gradient
             pgrad = zero
         
             if (px >= 1) then
                dxmonomx  = ea_monom_coor_bywlk (elec_i, atom_i, px-1, 1, walk_i)
                pgrad (1) = px * dxmonomx * monomy * monomz
             end if

             if (py >= 1) then
                dymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-1, 2, walk_i)
                pgrad (2) = monomx * py * dymonomy * monomz
             end if

             if (pz >= 1) then
                dzmonomz  = ea_monom_coor_bywlk (elec_i, atom_i, pz-1, 3, walk_i)
                pgrad (3) = monomx * monomy * pz * dzmonomz
             end if
   
             if (debug) then
               write (logf,*) trim(l_here), '-d:  pgrad=',pgrad
             endif
             do  i = 1,3
                mos_sto_grad_bywlk (elec_i,mo_i,i,walk_i) = mos_sto_grad_bywlk (elec_i,mo_i,i,walk_i) &
                     + epf_grad_loc(i) *polyn + epf_loc*pgrad(i)
             enddo ! do i 
! laplacian
            
             d2xmonomx = ea_monom_coor_bywlk (elec_i, atom_i, px-2, 1, walk_i)
             d2ymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-2, 2, walk_i)
             d2zmonomz = ea_monom_coor_bywlk (elec_i, atom_i, pz-2, 3, walk_i)
             plapl     = zero

             if (px >= 2 ) plapl = plapl + px * (px-1) * d2xmonomx * monomy * monomz
             if (py >= 2 ) plapl = plapl + monomx * py * (py-1) * d2ymonomy * monomz
             if (pz >= 2 ) plapl = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz

             mos_sto_lapl_bywlk (elec_i,mo_i,walk_i)=mos_sto_lapl_bywlk (elec_i,mo_i,walk_i) &
                  + plapl*epf_loc + polyn*epf_lapl_loc + two*(pgrad (1)*epf_grad_loc(1) &
                  + pgrad (2) *epf_grad_loc(2)+pgrad (3) *epf_grad_loc(3))
          
          enddo ! do ao_i
       enddo ! do elec_i
    enddo ! do mo_i
 enddo ! do walk_i
!$OMP END DO
!$OMP END PARALLEL 

 if (debug) write (logf,*) trim(l_here), '-d: mos_sto_bywlk:', mos_sto_bywlk 
 if (debug) write (logf,*) trim(l_here), '-d : mos_sto_grad_bywlk:',mos_sto_grad_bywlk
 if (debug) write (logf,*) trim(l_here), '-d : mos_sto_lapl_bywlk:',mos_sto_lapl_bywlk
call exit(l_here)
end subroutine mos_sto_grla_bywlk_bld

! ==========================================================================
  subroutine rad_ea_grla_bywlk_bld
! --------------------------------------------------------------------------
!
! Name     : rad_ea_grla_bywlk_bld
!
! Creates :  rad_ea_bywlk (slat,elec,atom,wlk)
! Creates :  rad_ea_grad_bywlk (slat,elec,atom,dim,wlk)
! Creates :  rad_ea_lapl_bywlk (slat,elec,atom,wlk)
!
! Author: R. Assaraf
! Date  : 26 Apr 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: atoms_nb, electrons_nb, slaters_nb, walkers_nb, dimensions_nb
  integer(dpi) :: walk_i, atom_i, elec_i, s_i
  real(dp)  :: xoverr,yoverr,zoverr,r_pr, epf_loc, epf_grad_loc (3), epf_lapl_loc, gamma, rinv, r
  integer(dpi) :: pr, z
! header:

  if (header_exe) then

     l_here = "rad_ea_grla_bywlk_bld"
     call object_create ('rad_ea_bywlk')
     call object_create ('rad_ea_grad_bywlk')
     call object_create ('rad_ea_lapl_bywlk')

     call object_needed ('slater_basis_exp')
     call object_needed ('slater_basis_rn')
     call object_needed ('slater_basis_dim')
     call object_needed ('ea_distance_bywlk')
     call object_needed ('ea_dist_inv_bywlk')
     call object_needed ('ea_dist_grad_bywlk')
     call object_needed ('ea_slater_expo_bywlk')
     call object_needed ('atoms')
     return
  endif
  call enter (l_here)

  atoms_nb = size (ea_distance_bywlk, dim = 2)
  electrons_nb = size (ea_distance_bywlk, dim = 1)
  slaters_nb   = size (slater_basis_exp, dim = 2)
  walkers_nb = size (ea_distance_bywlk, dim = 3)
! improve 
  dimensions_nb = 3

  if (.not.associated (rad_ea_bywlk)) then
     call alloc ('rad_ea_bywlk',rad_ea_bywlk, &
          slaters_nb, electrons_nb,  atoms_nb, walkers_nb)
     rad_ea_bywlk  = 0.d0
  endif
  
  if (.not.associated (rad_ea_grad_bywlk)) then
     call alloc ('rad_ea_grad_bywlk',rad_ea_grad_bywlk, &
          slaters_nb, electrons_nb,  atoms_nb, dimensions_nb,walkers_nb)
     rad_ea_grad_bywlk  = 0.d0
  endif

  if (.not.associated (rad_ea_lapl_bywlk)) then
     call alloc ('rad_ea_lapl_bywlk',rad_ea_lapl_bywlk, &
      slaters_nb, electrons_nb,  atoms_nb, walkers_nb)
      rad_ea_lapl_bywlk  = 0.d0
  endif


  
!$OMP PARALLEL DO PRIVATE (walk_i,atom_i,elec_i,z,s_i,r,rinv,xoverr,yoverr,zoverr,gamma,pr,r_pr,epf_loc,epf_grad_loc,epf_lapl_loc)
  do walk_i = 1, walkers_nb
     do atom_i= 1, atoms_nb
        do elec_i = 1, electrons_nb
           z = atoms(atom_i)%charge
           do s_i = 1, slater_basis_dim (z)
              
              r = ea_distance_bywlk (elec_i,atom_i,walk_i)
              rinv   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
              xoverr = ea_dist_grad_bywlk (elec_i, atom_i, 1, walk_i)
              yoverr = ea_dist_grad_bywlk (elec_i, atom_i, 2, walk_i)
              zoverr = ea_dist_grad_bywlk (elec_i, atom_i, 3, walk_i)

              gamma = slater_basis_exp (z,s_i)
              pr =  slater_basis_rn (z,s_i)

              if (pr>0) r_pr = r**pr
             
              epf_loc      = r_pr * exp(-gamma * r)
  
              epf_grad_loc = epf_loc *(-gamma + pr*rinv) * (/xoverr, yoverr, zoverr/)
              
              epf_lapl_loc = epf_loc  &
                   * ( pr*(pr+1)*rinv*rinv -2_dp*gamma*(pr+1)*rinv+gamma**2)


              rad_ea_bywlk (s_i, elec_i,atom_i,walk_i) = epf_loc
              rad_ea_grad_bywlk (s_i, elec_i, atom_i, :, walk_i) = epf_grad_loc
              rad_ea_lapl_bywlk (s_i, elec_i,atom_i,walk_i) = epf_lapl_loc

           enddo ! do s_i
        enddo ! do elec_i
     enddo ! do atom_i
  enddo  ! do walk_i
!$OMP END PARALLEL DO
 
  call exit (l_here)
end subroutine rad_ea_grla_bywlk_bld

! ==========================================================================
  subroutine mos_xpf_xpf_dot_bld
! --------------------------------------------------------------------------
!
! Name     : mos_xpf_xpf_dot_bld
!
! Creates :  mos_xpf_xpf (mo, xpf, xpf)
! 
! Needs : mos_bywlk
! Needs : mos_lapl_bywlk
!
! Author: R. Assaraf
! Date  : 02 May 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: elec_i, walk_i, mo_i, xp1, xp2
  integer(dpi) :: mos_nb, xpf_slater_nb, electrons_nb, walkers_nb
  integer(dpi), save :: iter = 0
  real(dp), pointer :: mos_xpf_xpf_bywlk (:,:,:,:) => NULL ()
  real(dp) :: coeff_xp1, coeff_xp2
! header:

  if (header_exe) then

     l_here = "mos_xpf_xpf_dot_bld"

     call object_create ('mos_xpf_xpf_dot')
     call object_create ('mos_xpf_xpf_bywlk')

     call object_needed ('mos_bywlk')
     call object_needed ('mos_lapl_bywlk')
 
     call object_needed ('xpfs_c2_bywlk')
     call object_needed ('xpfs_c2_lapl_bywlk') 

     return
  endif

  call enter (l_here)
  
  xpf_slater_nb = size (xpfs_c2_bywlk, dim = 2)
  walkers_nb = size (xpfs_c2_bywlk, dim = 3)
  electrons_nb  = size (xpfs_c2_bywlk, dim = 1)
  mos_nb = size (mos_bywlk, dim = 2)
  
! improve 
  
  if (.not.associated (mos_xpf_xpf_dot)) then 
     call alloc ('mos_xpf_xpf_dot',mos_xpf_xpf_dot,  &
          mos_nb, xpf_slater_nb, xpf_slater_nb)
     mos_xpf_xpf_dot = 0.d0
  endif

  if (.not.associated (mos_xpf_xpf_bywlk)) then 
     call alloc ('mos_xpf_xpf_bywlk',mos_xpf_xpf_bywlk,  &
          mos_nb, xpf_slater_nb, xpf_slater_nb, walkers_nb)
     mos_xpf_xpf_bywlk = 0.d0
  endif

  
  mos_xpf_xpf_dot = mos_xpf_xpf_dot * iter 

!$OMP PARALLEL DO DEFAULT(PRIVATE) SHARED(mos_lapl_bywlk,xpfs_c2_bywlk,mos_bywlk,xpfs_c2_lapl_bywlk,mos_xpf_xpf_bywlk,mos_xpf_xpf_dot)
  do walk_i = 1, walkers_nb
     do elec_i = 1, electrons_nb
        do mo_i = 1, mos_nb
           do xp1 = 1 , xpf_slater_nb
              do xp2= 1, xpf_slater_nb
                 coeff_xp1 =  mos_lapl_bywlk (elec_i, mo_i,  walk_i)  &
                      * xpfs_c2_bywlk (elec_i, xp1, walk_i)           &
                      - mos_bywlk (elec_i, mo_i, walk_i)              & 
                     * xpfs_c2_lapl_bywlk (elec_i, xp1, walk_i)     
                 coeff_xp2 =  mos_lapl_bywlk (elec_i, mo_i, walk_i)   &
                      * xpfs_c2_bywlk (elec_i, xp2, walk_i)           &
                      - mos_bywlk (elec_i, mo_i, walk_i)              & 
                      * xpfs_c2_lapl_bywlk (elec_i, xp2, walk_i)

                  mos_xpf_xpf_bywlk (mo_i,xp1,xp2,walk_i) =  coeff_xp1*coeff_xp2
              enddo ! do xp2
           enddo ! do xp1
           if (maxval(mos_xpf_xpf_bywlk (mo_i,:,:,walk_i)) < 14.d0) &
              mos_xpf_xpf_dot (mo_i,:,:)=  mos_xpf_xpf_dot (mo_i,:,:) &
                     +  mos_xpf_xpf_bywlk (mo_i,:,:,walk_i)
        enddo ! do mo_i 
     enddo ! do elec_i
  enddo ! do walk_i
!$OMP END PARALLEL DO

  iter = iter +1
  mos_xpf_xpf_dot = mos_xpf_xpf_dot / float (iter)
  call exit (l_here)
end subroutine mos_xpf_xpf_dot_bld

! ==========================================================================
  subroutine mos_on_xpf_new_bld
! --------------------------------------------------------------------------
!
! Name     : mos_on_xpf_new_bld
!
! Creates :  mos_on_xpf_new
! 
! Needs : mos_xpf_xpf_dot
! Needs :
!
! Author: R. Assaraf
! Date  : 02 May 2003
! --------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: mo_i, xpf_i
  integer(dpi) :: mos_nb, xpf_slater_nb, nrot
  real(dp), pointer, save  :: ortho_basis (:,:) => NULL()
  real(dp), pointer, save  :: eigen_values (:) => NULL ()
  real(dp), pointer, save  :: lower_eigen_value (:) => NULL()
  real(dp)  :: min_eigen
! header:

  if (header_exe) then

     l_here = 'mos_on_xpf_new'

     call object_create ('lower_eigen_value')
     call object_create ('mos_on_xpf_new')
     call object_needed ('mos_xpf_xpf_dot')

     return
  endif
  call enter (l_here)

  mos_nb = size (mos_xpf_xpf_dot, dim = 1)
  xpf_slater_nb = size (mos_xpf_xpf_dot, dim = 2)

  if (.not.associated (mos_on_xpf_new)) &
       call alloc ('mos_on_xpf_new',mos_on_xpf_new,mos_nb,xpf_slater_nb)

  if (.not.associated (lower_eigen_value)) call alloc ('lower_eigen_value', &
       lower_eigen_value, mos_nb)

  if (.not.associated(ortho_basis))  &
       allocate (ortho_basis (xpf_slater_nb, xpf_slater_nb))
  
  if (.not.associated(eigen_values)) & 
       allocate (eigen_values (xpf_slater_nb))

  do mo_i = 1, mos_nb

     call jacobi (mos_xpf_xpf_dot (mo_i,:,:), xpf_slater_nb, &
                                        xpf_slater_nb, &
                                        eigen_values,  &
                                        ortho_basis,   &
                                        nrot)

      min_eigen=minval(eigen_values)
      lower_eigen_value ( mo_i) = min_eigen
      do xpf_i= 1, xpf_slater_nb
         if (eigen_values (xpf_i) <min_eigen+0.000001d0) &
        mos_on_xpf_new (mo_i,:) = ortho_basis (:,xpf_i)
      enddo ! do xpf_i

      if (debug) write (logf,*) trim(l_here), '-d: eigen_values=',&
           eigen_values
  enddo ! do mo_i

 
  call exit (l_here)
end subroutine mos_on_xpf_new_bld

! ==========================================================================
  subroutine xpfs_c2_grla_bywlk_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c2_grla_bywlk_bld
!
! Creates        : xpfs_c2_bywlk (elec, xpf_c, walk)
! Creates        : xpfs_c2_grad_bywlk (elec, xpf_c, dim, walk)
! Creates        : xpfs_c2_lapl_bywlk (elec, xpf_c, walk)
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : ea_monom_coor_bywlk (elec, atom, pow, dim, walk)
! Needs          : ea_distanc2_bywlk (elec, atom, walk)
! Needs          : ea_distance_bywlk (elec, atom, walk)
! Needs          : ea_dist_inv_bywlk (elec, atom, walk)
! Needs          : ea_dist_xyz_bywlk (elec, atom, dim, walk)
! Needs          : elec_xpfs_rexp_bywlk (elec, xpf,  walk)
! Needs          : xpfs_c2 (xpf_c)
!
! Description    : calculates the values, gradient and laplacian 
! Description    : of all xpfs (gaussians) at elec_i x,y,z for all walkers 
!
! Author: F.Colonna
! Date  : 29 Mar 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp)             :: monomx, monomy, monomz
  real(dp)             :: dxmonomx, dymonomy, dzmonomz
  real(dp)             :: d2xmonomx, d2ymonomy, d2zmonomz
  real(dp)             :: r2, r, rinv, gamma
  real(dp)             :: xx,yy,zz
  real(dp)             :: epf_loc
  real(dp)             :: epf_loc_grad (3)
  real(dp)             :: epf_loc_lapl  
  real(dp)             :: ggrad_x, ggrad_y, ggrad_z
  real(dp)             :: pgrad_x, pgrad_y, pgrad_z
  real(dp)             :: plapl, glapl, polyn

  integer(dpi)         :: walkers_nb
  integer(dpi)         :: electrons_nb
  integer(dpi)         :: dimensions_nb
  integer(dpi)         :: atoms_nb
  integer(dpi)         :: xpfs_c2_nb

  integer(dpi)         :: px, py, pz, pr
  integer(dpi)         :: atom_i, walk_i, elec_i, xpf_c_i
    
! header:

  if (header_exe) then

    l_here = "xpfs_c2_grla_bywlk_bld"

    call object_create ('xpfs_c2_bywlk')
    call object_create ('xpfs_c2_grad_bywlk')
    call object_create ('xpfs_c2_lapl_bywlk')

    call object_needed ('elec_coor_bywlk')
    call object_needed ('ea_monom_coor_bywlk')
    call object_needed ('ea_distanc2_bywlk')
    call object_needed ('ea_distance_bywlk')
    call object_needed ('ea_dist_inv_bywlk')
    call object_needed ('ea_dist_xyz_bywlk')
    call object_needed ('xpfs_c2_exponents')   
    call object_needed ('xpfs_c2_center')
    call object_needed ('xpfs_c2_type')
    call object_needed ('xpfs_c2_r_power')
    call object_needed ('xpfs_c2_x_power')
    call object_needed ('xpfs_c2_y_power')
    call object_needed ('xpfs_c2_z_power')
   
    return

  end if

! begin:
  call enter (l_here)

! requirements:
  include 'chksiz_elec_coor_bywlk.inc'
  include 'chksiz_ea_distance_bywlk.inc'
  include 'chksiz_ea_distanc2_bywlk.inc'
  include 'chksiz_ea_dist_inv_bywlk.inc'
  include 'chksiz_ea_dist_xyz_bywlk.inc'
  include 'chksiz_ea_monom_coor_bywlk.inc'

! end requirements

! dimensions:
  atoms_nb      = size (ea_distance_bywlk, 2)
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)
  xpfs_c2_nb     = size (xpfs_c2_exponents)

! end dimensions
  if(debug) then 
   write (logf,*)trim(l_here),'-d: atoms_nb=',atoms_nb 
   write (logf,*)trim(l_here),'-d: electrons_nb  = ',electrons_nb
   write (logf,*)trim(l_here),'-d: xpfs_c2_nb = ',xpfs_c2_nb
   write (logf,*)trim(l_here),'-d: xpfs_c2_type >',xpfs_c2_type (1),'<'
   write (logf,*)trim(l_here),'-d: dimensions_nb = ',dimensions_nb
  end if 
      
! allocations:

  if(.not.associated(xpfs_c2_bywlk)) then
     if(trace_depend) &
          write (logf,*)trim(l_here),': xpfs_c2_bywlk not yet associated'
     call alloc ('xpfs_c2_bywlk', &
          xpfs_c2_bywlk,  &
          electrons_nb, &
          xpfs_c2_nb, &
          walkers_nb)
  else
     if(trace_depend) &
          write (logf,*)trim(l_here),': xpfs_c2_bywlk left in memory'
  end if

  if(.not.associated(xpfs_c2_grad_bywlk)) then
     if(trace_depend) &
          write (logf,*)trim(l_here),': xpfs_c2_grad_bywlk not yet associated'
     call alloc ('xpfs_c2_grad_bywlk', &
          xpfs_c2_grad_bywlk,  &
          electrons_nb, &
          xpfs_c2_nb, &
          dimensions_nb, &
          walkers_nb)
  else 
     if(trace_depend) &
          write (logf,*)trim(l_here),': xpfs_c2_grad_bywlk left in memory'
  end if

  if(.not.associated(xpfs_c2_lapl_bywlk)) then
     if(trace_depend) &
          write (logf,*)trim(l_here),': xpfs_c2_lapl_bywlk not associated'
     call alloc ('xpfs_c2_lapl_bywlk', xpfs_c2_lapl_bywlk, &
          electrons_nb, xpfs_c2_nb, walkers_nb)
  else
     if(trace_depend) &
          write (logf,*)trim(l_here),': xpfs_c2_lapl_bywlk left in memory'
  end if

! end allocations

!$OMP PARALLEL DEFAULT(PRIVATE) SHARED (elec_coor_bywlk,ea_monom_coor_bywlk,ea_distanc2_bywlk,ea_distance_bywlk,ea_dist_inv_bywlk,ea_dist_xyz_bywlk,xpfs_c2_exponents,xpfs_c2_center,xpfs_c2_type,xpfs_c2_r_power,xpfs_c2_x_power,xpfs_c2_y_power,xpfs_c2_z_power)
! initialize
  xpfs_c2_bywlk      = zero
  xpfs_c2_grad_bywlk = zero
  xpfs_c2_lapl_bywlk = zero

! calculation of the xpfs   

!$OMP DO
  l_walk:  do walk_i = 1 , walkers_nb
     l_xpfs:  do xpf_c_i = 1, xpfs_c2_nb
        l_elec:  do elec_i = 1 , electrons_nb

           atom_i = xpfs_c2_center (xpf_c_i)
           r      = ea_distance_bywlk (elec_i, atom_i, walk_i)
           r2     = ea_distanc2_bywlk (elec_i, atom_i, walk_i)
           rinv   = ea_dist_inv_bywlk (elec_i, atom_i, walk_i)
           gamma  = xpfs_c2_exponents (xpf_c_i)
  
           pr = xpfs_c2_r_power (xpf_c_i) -1
 
           xx = ea_dist_xyz_bywlk (elec_i, atom_i, 1, walk_i)
           yy = ea_dist_xyz_bywlk (elec_i, atom_i, 2, walk_i)
           zz = ea_dist_xyz_bywlk (elec_i, atom_i, 3, walk_i)

           px = xpfs_c2_x_power (xpf_c_i)
           py = xpfs_c2_y_power (xpf_c_i)
           pz = xpfs_c2_z_power (xpf_c_i)

! pr means r^(r_power-1) in calculations (Slater's convention)
             
           monomx = ea_monom_coor_bywlk (elec_i, atom_i, px, 1, walk_i)
           monomy = ea_monom_coor_bywlk (elec_i, atom_i, py, 2, walk_i)
           monomz = ea_monom_coor_bywlk (elec_i, atom_i, pz, 3, walk_i)

           select case (xpfs_c2_type(xpf_c_i))
              
           case ("gto")
              epf_loc = exp(-gamma * r2 ) 
              if (pr > 0) epf_loc = (r**pr) * epf_loc
              epf_loc_grad = (-2*gamma*r + pr*rinv) * epf_loc * rinv * (/xx,yy,zz/)
              epf_loc_lapl = ( pr*(pr+1)/r2 -2*gamma*(pr+3) + 4*(gamma**2)*r2) * epf_loc 
           case ("sto")
            
              epf_loc =  exp(-gamma * r ) 
              if (pr > 0) epf_loc = (r**pr) * epf_loc
              epf_loc_grad = (-gamma + pr*rinv) * epf_loc * rinv * (/xx,yy,zz/)
              epf_loc_lapl = ( pr*(pr+1)/r2-2*gamma*(pr+1)*rinv+gamma**2) * epf_loc
           case default

              call die (l_here, "type = >"+xpfs_c2_type(xpf_c_i)+"<")
              
           end select

!       epf_loc      = elec_xpfs_rexp_bywlk (elec_i, xpf_c_i, walk_i)
!       epf_loc_grad = elec_xpfs_rexp_grad_bywlk (elec_i, xpf_c_i, :, walk_i)
!       epf_loc_lapl = elec_xpfs_rexp_lapl_bywlk (elec_i, xpf_c_i, walk_i)

       polyn = monomx * monomy * monomz

! values:
       xpfs_c2_bywlk (elec_i, xpf_c_i, walk_i) = epf_loc * polyn
 
       ggrad_x  = epf_loc_grad (1)
       ggrad_y  = epf_loc_grad (2)
       ggrad_z  = epf_loc_grad (3)
       
       pgrad_x = zero
       pgrad_y = zero
       pgrad_z = zero

       if (px >= 1) then
          dxmonomx  = ea_monom_coor_bywlk (elec_i, atom_i, px-1, 1, walk_i)
          pgrad_x = px * dxmonomx * monomy * monomz
       end if

       if (py >= 1) then
          dymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-1, 2, walk_i)
          pgrad_y = monomx * py * dymonomy * monomz
       end if

       if (pz >= 1) then
          dzmonomz  = ea_monom_coor_bywlk (elec_i, atom_i, pz-1, 3, walk_i)
          pgrad_z = monomx * monomy * pz * dzmonomz
       end if

! gradient:
       xpfs_c2_grad_bywlk (elec_i, xpf_c_i, 1 , walk_i) =  &
            pgrad_x*epf_loc + ggrad_x*polyn

       xpfs_c2_grad_bywlk (elec_i, xpf_c_i, 2 , walk_i) =  &
            pgrad_y*epf_loc + ggrad_y*polyn

       xpfs_c2_grad_bywlk (elec_i, xpf_c_i, 3 , walk_i) =  &
               pgrad_z*epf_loc + ggrad_z*polyn

       glapl = epf_loc_lapl 
 
       d2xmonomx = ea_monom_coor_bywlk (elec_i, atom_i, px-2, 1, walk_i)
       d2ymonomy = ea_monom_coor_bywlk (elec_i, atom_i, py-2, 2, walk_i)
       d2zmonomz = ea_monom_coor_bywlk (elec_i, atom_i, pz-2, 3, walk_i)
       plapl     = zero

       if (px >= 2 ) plapl = plapl + px * (px-1) * d2xmonomx * monomy * monomz
       if (py >= 2 ) plapl = plapl + monomx * py * (py-1) * d2ymonomy * monomz
       if (pz >= 2 ) plapl = plapl + monomx * monomy * pz * (pz-1) * d2zmonomz

! laplacian:
       xpfs_c2_lapl_bywlk (elec_i, xpf_c_i, walk_i) &
            = plapl*epf_loc + polyn*glapl          &
            + two*( pgrad_x * ggrad_x + pgrad_y * ggrad_y + pgrad_z * ggrad_z )

    end do l_elec
 end do l_xpfs
end do l_walk
!$OMP END DO
!$OMP END PARALLEL

call exit (l_here)
       
end subroutine xpfs_c2_grla_bywlk_bld


! ==========================================================================
  subroutine xpfs_c2_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c2_bld
!
! Creates        : xpfs_c2
!

! Author: F.Colonna
! Date  : 29 Mar 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
 
! header:

  if (header_exe) then

    l_here = "xpfs_c2_bld"

    call object_create ('xpfs_c2')
    call object_create ('xpfs_c2_center')
    call object_create ('xpfs_c2_exponents')
    call object_create ('xpfs_c2_type')
    call object_create ('xpfs_c2_r_power')
    call object_create ('xpfs_c2_x_power')
    call object_create ('xpfs_c2_y_power')
    call object_create ('xpfs_c2_z_power')
    call object_needed ('xpfs_c2_nb')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

 if (.not.associated (xpfs_c2_center)) call alloc ( &
  'xpfs_c2_center',xpfs_c2_center,xpfs_c2_nb)

 if (.not.associated (xpfs_c2_exponents)) call alloc ( &
  'xpfs_c2_exponents',xpfs_c2_exponents,xpfs_c2_nb)

 if (.not.associated (xpfs_c2_type)) call alloc ( &
  'xpfs_c2_type',xpfs_c2_type,xpfs_c2_nb)

 if (.not.associated (xpfs_c2_r_power)) call alloc ( &
  'xpfs_c2_r_power',xpfs_c2_r_power,xpfs_c2_nb)

 if (.not.associated (xpfs_c2_x_power)) call alloc ( &
  'xpfs_c2_x_power',xpfs_c2_x_power,xpfs_c2_nb)

 if (.not.associated (xpfs_c2_y_power)) call alloc ( &
  'xpfs_c2_y_power',xpfs_c2_y_power,xpfs_c2_nb)
 
if (.not.associated (xpfs_c2_z_power)) call alloc ( &
  'xpfs_c2_y_power',xpfs_c2_z_power,xpfs_c2_nb)

 xpfs_c2_type = 'sto' 
 xpfs_c2_center = 1
 xpfs_c2_exponents(1) = 2.69063d0
 xpfs_c2_exponents(2) = 0.63961d0
 xpfs_c2_x_power = 0
 xpfs_c2_y_power = 0
 xpfs_c2_z_power = 0
 xpfs_c2_r_power (1) = 1
 xpfs_c2_r_power (2) = 2
! dimensions:

! end dimensions
    
! allocations:

  call exit (l_here)
       
  end subroutine xpfs_c2_bld

! ==========================================================================
  subroutine xpfs_c2_nb_bld 
! --------------------------------------------------------------------------
!
! Name           : xpfs_c2_bld
!
! Creates        : xpfs_c2_nb
!

! Author: F.Colonna
! Date  : 29 Mar 2002
! --------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name),save :: l_here
 
! header:

  if (header_exe) then

    l_here = "xpfs_c2_nb_bld"

    call object_create ('xpfs_c2_nb')

    return

  end if

! begin:
  call enter (l_here)

! requirements:
 if (.not.associated (xpfs_c2_nb)) call alloc ('xpfs_c2_nb',xpfs_c2_nb)

 xpfs_c2_nb = 3

! end dimensions
    
! allocations:

  call exit (l_here)
       
  end subroutine xpfs_c2_nb_bld

end module mos_sto_bywlk_mod
