! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

! =============================================================================
  subroutine objects_write (objects_byind, obj_nb)
! -----------------------------------------------------------------------------
!
! Name         : objects_write
!
! Argument     : objects_byind
!
! Description  : write objects in file 
!
! Authors: R. Assaraf
! Date  : 11 Aug  2002
! ------------------------------------------------------------------------------ 

  use objects_mod 
  use files_mod 
  implicit none

! i/o:
   integer(dpi) :: obj_nb
   integer(dpi) :: objects_byind (obj_nb)
 
 
! local:
  integer(dpi)  :: unit
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  integer(dpi) :: i

! begin:
  l_here ='objects_write'
  call enter (l_here)

  f_type = 'objects-out'//blanks
  unit = file_open_and_cat_from_type (f_type)

  message = 'in '*l_here*' configuration unit not found'
  call require (message, unit /= 0)

  f_name = files_names(unit)
  f_action = files_actions(unit)  

  if(debug) then
   write (logf,*)trim(l_here),'-d: parameters file unit   =',unit
   write (logf,*)trim(l_here),'-d: parameters file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: parameters file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: parameters file action >',trim(f_action),'<'
  end if

  call file_header_write (unit)

  write(unit,'(a)')'end_of_header'
  
  if (debug) then
     write (logf,*) trim (l_here), '-d: size (objects_byind)=',size (objects_byind) 
     do i = 1, size (objects_byind)
        write (logf,*) trim (l_here), '-d: objects_byind(',i,')='!,objects_byind (i)
     enddo
  endif


  do i = 1,size (objects_byind)
     call object_value_byind_write (objects_byind (i), unit)
  enddo

  call file_close_by_unit (unit)

   call exit (l_here)

  end subroutine objects_write


! ==========================================================================
  subroutine objects_read
! --------------------------------------------------------------------------
!
! Name           : objects_read
!
! Argument       : objects
!
! Description:
!
! Authors: R. Assaraf
! Date  :21 Mars 2001
! --------------------------------------------------------------------------
  use files_mod
  use basic_types_tools_mod
  use objects_paths_mod
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  integer(dpi)                               :: unit
  integer(dpi)                               :: n_lines,cur_line
  character(max_string_len)                  :: equality
  integer(dpi)                               :: n_sub_fields
  character(max_string_len), pointer         :: sub_fields(:) => NULL()
  integer(dpi)                     :: i
  integer(dpi)                     :: i_inf (maxdim) 
  integer(dpi)                     :: i_sup (maxdim)
  integer(dpi)                     :: step  (maxdim)

  integer(dpi) :: dim_nb
  character(max_string_len_object_name) :: array_name 
  character(max_string_len_object_name), pointer, save :: array_names (:) => NULL ()
  integer(dpi), pointer                               :: array_index (:) => NULL ()  
  integer(dpi)                                        :: array_names_nb 
  integer(dpi)                                        :: obj_i, array_i, nod_i
  integer(dpi), pointer :: path_bynod (:) => NULL()
! begin:
  l_here ='objects_read'
  call enter (l_here)

  f_type = 'objects-in'//blanks
  unit = file_open_and_cat_from_type (f_type)

  message = 'in '*l_here*' configuration unit not found'
  call require (message, unit /= 0)

  f_name = files_names(unit)
  f_action = files_actions(unit)

  if(debug) then
   write (logf,*)trim(l_here),'-d: param file unit   =',unit
   write (logf,*)trim(l_here),'-d: param file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: param file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: param file action >',trim(f_action),'<'
  end if

  rewind (unit)
  call file_header_read (unit)

  n_lines = file_sample(unit)

  if(debug) then
    write (logf,*)trim(l_here),'-d: n_lines=',n_lines
   end if

! the file is a succession "token = values", example : A(i,j) = 3.33
! here we read all the array names, in  this example: A
  
 if (associated (array_names)) array_names = NOT_INIT_S

 array_names_nb = 0

 rewind(unit)
 call file_header_read (unit)
 
 do cur_line = 1, n_lines 

    read (unit, '(a)') equality 

    if(debug) write (logf,*)trim(l_here),'-d: equality= >',&
         trim(equality),'<'
  
    n_sub_fields=string_split(equality,'=',sub_fields, len(sub_fields))

    message='in subroutine'*l_here*'n_sub_fields='*n_sub_fields&
         *'should be equal to 2'
    
    call require(message,n_sub_fields== 2)

    if (debug) write (logf,*)trim(l_here),'-d: sub_fields(1)=',trim(sub_fields(1))
    
    if (debug) write (logf,*)trim(l_here),'-d: sub_fields(2)=',trim(sub_fields(2))

    array_name = array_name_get (sub_fields (1))

    call string_in_list_add_once (array_name, array_names, list_nb = array_names_nb)
    
 enddo ! do cur_line

  if (associated ( array_index)) deallocate (array_index)
 
  allocate (array_index (array_names_nb))

  do array_i = 1, array_names_nb

    array_index (array_i) = object_index (array_names (array_i))

  enddo
 

  if (debug) then 
     write(logf,*) trim(l_here), '-d: arrays read in  files:', &
          (trim('>'+array_names(array_i)+'< '),array_i= 1, array_names_nb)
  endif

  do array_i = 1, array_names_nb 

     call provide_byind (array_index (array_i))

  enddo ! do array_i

! fills the fields of the arrays
  rewind(unit)
  
  call file_header_read (unit)

  do cur_line = 1, n_lines
  
   read(unit,'(a)') equality
  
   n_sub_fields=string_split(equality,'=',sub_fields, len(sub_fields))

! read the value 

   sub_fields(2) = adjustl(sub_fields(2))

   array_name = array_name_get (sub_fields (1))
 
   array_i = string_in_array_locate_or_die (array_name, array_names)
   
   obj_i = array_index (array_i)

   dim_nb = object_dimension (obj_i)

   if (types_name (objects_type (obj_i)) (1:4) == 'char') dim_nb = dim_nb-1

   call  array_regular_section_decode (sub_fields (1), array_name, i_inf (1:dim_nb), &
                                                             i_sup (1:dim_nb), &
                                                             step  (1:dim_nb))

  call array_component_value_set (obj_i, i_inf (1:dim_nb), sub_fields (2)) 

  enddo ! do cur_line
 
! all the objects which  depend of array_names are invalidated 
 
  if (associated (path_bynod)) deallocate (path_bynod)
  
  call path_bynod_bld (path_bynod, array_index, (/ (i, i= 1,objects_nb) /))

  if (associated (path_bynod)) is_valid_bynod (path_bynod) = .false.

  call file_close_by_unit (unit)

  call exit (l_here)

  end subroutine objects_read
