! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

! ========================================================================
  subroutine objects_menu
! ------------------------------------------------------------------------
!
! Name   : objects_menu
!
! Authors:F. Assaraf
! Date  : 17 May 2002
! ------------------------------------------------------------------------
  use basic_types_tools_mod
  use parser_mod
  use objects_paths_mod
  use expressions_mod
 
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)              :: done
  integer(dpi), pointer :: path_bynod (:) => NULL ()
  character(max_string_len_object_name), pointer :: targets_names (:) => NULL ()
  character(max_string_len_object_name), pointer :: roots_names   (:) => NULL ()
  integer(dpi), pointer                          :: targets_ind (:)   => NULL ()
  integer(dpi), pointer                          :: roots_ind (:)     => NULL ()
  
  character(max_string_len_object_name), pointer :: objects_trace_names (:) => NULL()
  character(len=max_string_len)  :: what
  integer(dpi)                   :: i



! begin:
  l_here ='objects_menu'
  call enter (l_here)
  
  done = .false.
  call next_command_get ('objects')
  do while (.not.done)
     call next_word_get ('objects')
     if(trim(cur_word) .eq. 'help') then
        write (logf,*) 'objects: objects_menu'
        write (logf,*) 'objects: syntax'
        write (logf,*) 'objects: help'
        write (logf,*) 'objects:  [string1] = real ([string2])'
        write (logf,*) 'objects:              string1 : any new variable name'
        write (logf,*) 'objects:              string2 : any mathematical expression'
        write (logf,*) 'objects:   list      = [list of  strings]'
        write (logf,*) 'objects:   roo{ts}   = [list of  strings]'
        write (logf,*) 'objects:   trace = [list of strings]'
        write (logf,*) 'objects:   act{ion} = [string] '
        write (logf,*) 'objects:               string : inf{os}  print infos'
        write (logf,*) 'objects:               string : val{ues} print values '
        write (logf,*) 'objects:               string : pat{hes} print pathes'
        write (logf,*) 'objects:               string : wri{te}:rea{d}'
        write (logf,*) 'objects:   end_{obj}'
        write (logf,*) 'objects: example'

     else if (input_words_semans(cur_word_pos) == "expression" ) then
        
        call node_exe_byrtn_name ("next_expression_get")
        
     else if (token_is('lis', targets_names)) then
        
        if(debug) then
         do i = 1,size(targets_names)
           write (logf,*)trim(l_here),'-d: targets_names (',i,') >',trim(targets_names(i)),'<'
         end do
      end if

! define targets_ind   
      if (associated (targets_ind)) deallocate (targets_ind)
      allocate (targets_ind (size(targets_names)))
    
      do i= 1, size(targets_names)
         targets_ind (i) = object_index (targets_names (i))
      enddo
      
   else if (token_is('roo',roots_names)) then
 
      if (associated (roots_ind)) deallocate (roots_ind)
      if (size (roots_names) >= 1) allocate (roots_ind (size (roots_names)))
     
      do i= 1, size(roots_names)
         roots_ind (i) = object_index (roots_names (i))
      enddo

     else if (token_is('tra', objects_trace_names)) then
 
        if (associated(objects_trace)) deallocate (objects_trace)
        
        allocate (objects_trace (size(objects_trace_names)))

        do i = 1, size (objects_trace)
           objects_trace (i) = object_index (objects_trace_names (i))
        enddo ! do i



   else if(token_is('act',what)) then
      
      if (what (1:3) == 'pat') then

         call path_bynod_bld (path_bynod, roots_ind, targets_ind)
           
         call path_bynod_print (path_bynod)
           
      else  if (what (1:3) == 'all') then
         call objects_info_print

      else if (what (1:3) == 'inf') then 
         if (.not.associated(targets_names)) &
              call die(l_here,' list should have beed defined')
         write (res,'(2a)') blanks(1:20),' '
         write (res,'(2a)') blanks(1:20),' '
         write (res,'(2a)') blanks(1:20),' ________________________ '
         write (res,'(2a)') blanks(1:20),'I                        I' 
         write (res,'(2a)') blanks(1:20),'I    Objects Info        I'
         write (res,'(2a)') blanks(1:20),'I________________________I'
         write (res,'(2a)') blanks(1:20),'**************************'
         
         do i = 1,size(targets_names)
            
            call one_object_info_write (targets_ind(i),logf, l_here)
            write (res,*) ' '
            write(res,'(3a)') '--------------------  >',&
                 trim(targets_names (i)),               &
                 '<  -------------------------'

            call one_object_info_write (targets_ind(i),res,"")
         enddo ! do i
                
        else if (what (1:3) == 'val') then

           do i = 1,size (targets_names)
              call provide_bynam (targets_names (i))
           enddo ! do i

           write (res,'(2a)') blanks(1:20),' '
           write (res,'(2a)') blanks(1:20),'___________________________ '
           write (res,'(2a)') blanks(1:20),'I                           I' 
           write (res,'(2a)') blanks(1:20),'I    Object Values          I'
           write (res,'(2a)') blanks(1:20),'I___________________________I'
           write (res,'(2a)') blanks(1:20),'*****************************'
           
           do i = 1,size (targets_names)
              call object_value_bynam_print (targets_names (i))
              write (res,*) ' '
       
              write(res,'(3a)') '--------------------  >',&
                   trim(targets_names (i)),               &
                   '<  -------------------------'
    
              write(res,*) trim(objects_desc (targets_ind(i)))
              write(res,*) 
              call object_value_bynam_write (targets_names (i), res)
           enddo ! do i

        else if (what (1:3) == 'cur') then

           write (res,'(2a)') blanks(1:20),' '
           write (res,'(2a)') blanks(1:20),'___________________________ '
           write (res,'(2a)') blanks(1:20),'I                           I' 
           write (res,'(2a)') blanks(1:20),'I    Object Values          I'
           write (res,'(2a)') blanks(1:20),'I___________________________I'
           write (res,'(2a)') blanks(1:20),'*****************************'
           
           do i = 1,size (targets_names)
              call object_value_bynam_print (targets_names (i))
              write (res,*) ' '
       
              write(res,'(3a)') '--------------------  >',&
                   trim(targets_names (i)),               &
                   '<  -------------------------'
              write(res,*) 
              call object_value_bynam_write (targets_names (i), res)
           enddo ! do i

       else if (what (1:3) == 'wri') then

          do i= 1, size (targets_ind)
             call provide_byind (targets_ind (i))
          enddo ! do i
        
          call  objects_write (targets_ind, size(targets_ind))
       
       else if (what(1:3) == 'rea') then
     
          call  objects_read
       
       else
          call die(l_here,'option '*what*' not valid')
          
       endif
         
    else
       call menu_exit (l_here, done)
       
    endif

 enddo

 call exit (l_here)

  end subroutine objects_menu

! ========================================================================
  subroutine routines_menu
! ------------------------------------------------------------------------
!
! Name   : objects_menu
!
! Authors:F. Assaraf
! Date  : 17 May 2002
! ------------------------------------------------------------------------
  use basic_types_tools_mod
  use routines_mod
  use parser_mod

  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: done

  character(len=max_string_len)              :: action

  character(max_string_len_routine_name), pointer :: rtn_names(:) => NULL()
  integer(dpi) :: rtn_i,rtn_index

! begin:
  l_here ='routines_menu'
  call enter (l_here)

  done = .false.
  call next_command_get ('routines')
  do while (.not.done)
     call next_word_get('routines')
     if(trim(cur_word) .eq. 'help') then
        write (logf,*) 'routines: routines_menu'
        write (logf,*) 'routines: syntax'
        write (logf,*) 'routines: help'
        write (logf,*) 'routines: list = [list of  strings] '
        write (logf,*) 'routines: action = [string] : list{_print} :'// &
             ' all{_print}: cpu{_print}: exe_{list}' 
        write (logf,*) 'routines: example'

     else if (token_is('list',rtn_names)) then

     else if (token_is ('action',action)) then

        if (trim(action) == 'all') then 
 
           call routines_print
                  
        else if (trim(action) == 'list') then 
                  
           if (.not.associated(rtn_names)) &
                call die(l_here, 'routines_list should have been defined')
 
           do rtn_i = 1 , size(rtn_names)
              rtn_index = string_in_array_locate_or_die(rtn_names(rtn_i),routines_name)
              call one_routine_print(rtn_index)
           enddo
        else if (trim(action) == 'cpu') then
           write (res,'(2a)') blanks(1:20),' '
           write (res,'(2a)') blanks(1:20),'___________________________ '
           write (res,'(2a)') blanks(1:20),'I                           I' 
           write (res,'(2a)') blanks(1:20),'I    Total cpu by routine   I'
           write (res,'(2a)') blanks(1:20),'I___________________________I'
           write (res,'(2a)') blanks(1:20),'*****************************'
 
           if (.not.associated(routines_cpu))  &
                call die (l_here,'routines_cpu not associated')
           write (logf, *) trim(l_here), 'size(rouutines_cpu)=',size (routines_cpu)
           if (.not.associated(routines_name))  &
                call die (l_here,'routines_name not associated')
           write (logf, *) trim(l_here), 'size(routines_name)=',size (routines_name)
           
           write (res,'(a,t15,a,t50,a)') 'Routine #', 'Routine name', 'Cumulative cpu'
           write (res,*)
           do rtn_i = 1, size (routines_cpu)
              write (res,'(i3,t15,a,t50,f13.6)')  rtn_i, trim(routines_name (rtn_i)), routines_cpu (rtn_i)
           enddo ! do rtn_i
           
           write (logf,'(a,a, t20,a,t35,a,t70,a)') trim(l_here),':', 'Routine #', 'Routine name', 'Cumulative cpu'
           write (logf,*) trim(l_here), ':'
           do rtn_i = 1, size (routines_cpu)
              write (logf,'(a,a,t20, i3,t35,a,t70,f13.6)') &
                   trim(l_here),':', rtn_i, &
                   trim(routines_name (rtn_i)), routines_cpu (rtn_i)
           enddo ! do rtn_i
 
        else if (trim(action) == 'exe') then
           if (.not.associated(rtn_names)) &
                call die(l_here, 'routines_list should have been defined')
 
           do rtn_i = 1 , size(rtn_names)
              rtn_index = string_in_array_locate_or_die(rtn_names(rtn_i),routines_name)
              call routine_byind_exe(rtn_index)
           enddo
  
        else 
           call die(l_here,'print '*action*'not permitted ')
             
       endif

     else

        call menu_exit (l_here, done)
        
     endif

  enddo
  
  call exit (l_here)
     
  end subroutine routines_menu


