! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module objects_mod

  use tools_mod
  use list_mod    
  use address_tools_mod 
  use routines_mod

  implicit none

! Description : objects 

  character(len=max_string_len_object_name), pointer :: objects_names (:) => NULL()
  integer(dpi)                   :: objects_nb        = NOT_INIT_I
  integer(dpi) , pointer         :: objects_addr (:)  => NULL() 
  integer(dpi) , pointer         :: objects_type (:)  => NULL()
  integer(dpi) , pointer         :: objects_node (:)  => NULL() 
  logical(bool), pointer         :: objects_used (:)  => NULL()
  character(len=max_string_len) , pointer :: objects_desc (:)  => NULL()
! Description : objects_types (obj_i) = index in array types_name (typ_i)  
! Description : objects_node (obj_i)  = index of a node, this index tells us how
! Description : to compute the object obj_i 

! Description : nodes :

  integer(dpi), pointer     :: nodes_rtn(:) => NULL()
  type(int_array_list), save :: nodes_need
  type(int_array_list), save :: nodes_create
  integer(dpi)              :: nodes_nb = 0  
  logical(bool), pointer    :: is_valid_bynod (:) => NULL ()
  integer(dpi)              :: cur_nod = NOT_INIT_I 

  integer(dpi)              :: cur_create = NOT_INIT_I
  integer(dpi)              :: cur_needed = NOT_INIT_I
! Description : a node "node_i" has only one creation-routine 
! Description : but may create several objects.
! Description : "nodes_create%val (nodes%first(node_i) : nodes%last (node_i))"
! Description : store indices of these objects sequentially in nodes_create%val

! Description : the creation-routine at node_i is "nodes_rtn (node_i)" 
! Description : "nodes_need%val (nodes%first(node_i) : nodes%last (node_i))"
! Description : store indices of neede-objects sequentially in nodes_need%val

! Description : is_valid_bynod (nod_i) = .false. if the node has been never executed
! Description : in other words the objects created at nod_i  are not associated
! Description : is_valid_bynod (nod_i) must be put to 1 if objects at nod_i
! Description : are built 

! Description : cur_nod is the current node when
! Description : the current routine is executed 
! -----------------------------------------------------------------------------
 
  integer(dpi), parameter        :: max_string_len_fort_type = 16

! Description : types_name = list of object types "integer", "real", "real_1" ....
! Description : example types( objects_type ( obj_i) ) = "integer" 
! Description : types_write is the list of print routines of each type

  character(max_string_len_fort_type), pointer :: types_name  (:) => NULL()
  integer(dpi),                        pointer :: types_write (:) => NULL()
  integer(dpi)                                 :: types_nb        =  NOT_INIT_I

! Improve HORREUR !!!!!!
  character(len=max_string_len)                :: text_print = "?" ! NOT_INIT_S

!  list of objects to be traced
  integer(dpi), pointer :: objects_trace (:) => NULL ()  
  contains

! ==================================================================
  function automatic_name (create_i, rtn_name, needs) result(result)
! ------------------------------------------------------------------
!
! Name           : automatic_create_names_bld 
!
! Argument (in/out ) : create_i
! Argument (in ) :  rtn_i
! Argument (in ) :  needs
!
! Description    : set the name from the ith create of the 
! Description    : node defined by rtn_i and the needs
! 
! Authors: R. Assaraf
! Date   : 03 Nov 2003
! --------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len_object_name) :: result

! i/o:
  integer(dpi) :: create_i
  character(len=*) :: rtn_name
  integer(dpi) :: needs (:)

! local:
  character(len=max_string_len_routine_name)  :: l_here 
  integer(dpi) :: i
         
! begin:
  l_here='automatic_name'
  call enter (l_here)

  result='C-'+create_i+'-'+rtn_name+'('

  do i = 1, size (needs)-1
  result = result+objects_names(needs(i))+','
  enddo ! do i
  result = result+objects_names(size(needs))+')'
  
  call exit (l_here)

  end function automatic_name

! ==============================================================
  subroutine node_create (rtn_name, creates_names, needs_names)
! --------------------------------------------------------------
!
! Name           : node_create
!
! Argument (in ) : rtn_name
! Argument (in ) : creates_names
! Argument (in ) : needs_names
!
! Description    : define a new node 
! 
! Authors: R. Assaraf
! Date   : 04 June 2002
! --------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: rtn_name 
  character(len=*) :: creates_names (:)
  character(len=*) :: needs_names (:)

! local:
  character(len=max_string_len_routine_name)  :: l_here 
  integer(dpi)  :: index, rtn_i, i, obj_i
! begin:
  l_here='node_create'
  call enter (l_here)

  if(debug) then
    write (logf,*)trim(l_here),'-d: routine >',trim(rtn_name),'<'
    do obj_i = 1, size(creates_names)
      write (logf,*)trim(l_here),'-d: creates_names (',obj_i,') >',trim(creates_names(obj_i)),'<'
    end do
    do obj_i = 1, size(needs_names)
      write (logf,*)trim(l_here),'-d: needs_names (',obj_i,') >',trim(needs_names(obj_i)),'<'
    end do
  end if

! checks
  if (size (creates_names) == 0) then
    call die (l_here, 'a node should have created objects') 
  end if

! check creates are not built by another node

  do i = 1, size (creates_names)

     index = string_in_array_locate (creates_names (i), objects_names)
  
     if (index /= 0) then
        if (objects_node (index)/= NOT_INIT_I) then
           call one_node_write (objects_node (index),logf,l_here)
           call die (l_here, ' object already created at one node')
        endif

     endif
  enddo

! get routine index
  rtn_i = string_in_array_locate (rtn_name, routines_name)

  if (rtn_i == 0) then
    message = 'in routine'*l_here*CR*' routine >'+rtn_name &
   +'< does not exits. Add this line to file routines_catalog.inc:' &
    *CR*'   call one_routine_catalog ("'+rtn_name+'", '+rtn_name+')'
    call require (message, rtn_i /= 0)
  endif


! add routine index (trn_i) to nodes_rtn array
  call integer_in_list_add (rtn_i, nodes_rtn, list_nb = nodes_nb)

! add 0 to is_valid_bynod
  
  call boolean_in_list_add (.false., is_valid_bynod)

! creates 
  do i = 1, size (creates_names)
   if(debug) write (logf,*)trim(l_here),'-d: object_create (',i,') = >',&
                     trim(creates_names (i)),'<'
   call object_create (creates_names (i))
  enddo 
 
! needs
  do i = 1, size (needs_names)

    if(debug) then
      write (logf,*)trim(l_here),'-d: needs_names (',i,') = >',trim(needs_names(i)),'<'
    end if

    if (string_in_array_locate (needs_names (i), objects_names) == 0) then
       call die (l_here, 'needed object >'+trim(needs_names (i)) &
            +'< does not exits, check the syntax'&
            *'or check file routines_catalog.inc')
    endif
  
    call object_needed (needs_names (i))
 
 enddo
   
  call nodes_check_size_or_die 
 
  call exit (l_here)

end subroutine node_create

! =============================================================================
  subroutine nodes_check_size_or_die
! -----------------------------------------------------------------------------
!
! Name           :  nodes_check_size_or_die
!
! Description    : checks consitencies of nodes
! 

! Authors: R. Assaraf
! Date   : 10 June 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name)  :: l_here 
          
! begin:
  l_here='nodes_check_size_or_die'
  call enter (l_here)
  
! checks consistency of dimensions 
  if (nodes_nb == 0) then 
        
    if (associated (nodes_rtn)) &
      call die (l_here, 'nodes_rtn should not be associated')

    if (associated (nodes_need%first)) &
      call die (l_here, 'nodes_need%first should not be associated' )
        
    if (associated (nodes_need%last)) &
      call die (l_here, 'nodes_need%last should not be associated' )
   
    if (associated (nodes_create%first)) &
      call die (l_here, 'nodes_create%first should not be associated' )

    if (associated (nodes_create%last)) &
    call die (l_here, 'nodes_create%last should not be associated' )

    if (associated (is_valid_bynod)) &
    call die (l_here, 'is_valid_bynod should not be associated' )

  else

    if (nodes_need%nb /= nodes_nb) &
     call die (l_here, 'nodes_need%nb ='+nodes_need%nb+' /= nodes_nb ='+nodes_nb) 

    if (nodes_create%nb /= nodes_nb) &
     call die (l_here, 'nodes_create%nb ='+nodes_create%nb+' /= nodes_nb ='+nodes_nb) 

!   if (.not. pos_in_boolean_list_last_is (nodes_nb, is_valid_bynod)) &
!      call die (l_here, '  nodes_nb ='+nodes_nb+' not the last element in '+ &
!                        ' is_valid_bynod') 

  endif

  call exit (l_here)

  end subroutine nodes_check_size_or_die

! =============================================================================
  subroutine nodes_init 
! -----------------------------------------------------------------------------
!
! Name           :  nodes_init
!
! Description    : initialize nodes
! 
! Authors: R. Assaraf
! Date   : 04 June 2002
! Revision: 29 Oct 2002 F. Colonna print_level
! -----------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name)  :: l_here 
  integer(dpi)                                :: rtn_i 
  integer(dpi)                                :: all_err
          
! begin:
  l_here='nodes_init'
  call enter (l_here)
  
! initialize:

  if (.not.associated(routines_name)) then
    call die(l_here,'routines_name is not associated')
  endif

  if (routines_nb <= 0 ) then
    call die(l_here,'nb of routines ='*routines_nb)
  endif

! include 'debug_routines_name.inc'

  if (nodes_nb /= 0) call die &
     (l_here, ' nodes_nb ='+nodes_nb+l_here+' should be called once')

  call int_array_list_init (nodes_need)
  call int_array_list_init (nodes_create)

  call nodes_check_size_or_die 
  
  if(print_level > 0) then
    write (logf,*)trim(l_here),': ',routines_nb,' headers to be processed' 
  end if
  if (trace_depend) then
    write (logf,*)trim(l_here),': allocate (nodes_rtn (',routines_nb,')'
  end if

  allocate (nodes_rtn (routines_nb), stat = all_err)
  if (all_err /= 0) then
    write (logf,'(4a)') trim(l_here), &
   '-e: allocation failed for object >nodes_rtn ('+routines_nb+')<'
    call die (l_here, 'allocation failed for object nodes_rtn')
  end if

  nodes_rtn = NOT_INIT_I 

! gather information from routines headers
  do rtn_i = 1, routines_nb
    nodes_nb = rtn_i
    call one_routine_header_exe (rtn_i)
    nodes_rtn (nodes_nb) = rtn_i   ! Improve qu'est-ce que c'est ?
  enddo
  
  call boolean_list_expand (is_valid_bynod, nodes_nb)
  is_valid_bynod = .false.

  call int_array_list_expand (nodes_need, nodes_nb)
  call int_array_list_expand (nodes_create, nodes_nb)
 
! Improve quelle taille 
  call nodes_check_size_or_die 

  call exit (l_here)

  end subroutine nodes_init

! =============================================================================
  subroutine object_needed (needed_name)
! -----------------------------------------------------------------------------
!
! Name           : object_needed
!
! Argument (in ) : needed_name
!
! Improve obscur
! Description    : routine_cur needs object 'needed_name'
! Description    : add this information in 'objects' and in  'nodes'
!                  
! Authors: R. Assaraf
! Date   : 9 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: needed_name

! local:                                                                      
! character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)                                :: needed_index

! begin:
! l_here='object_needs'
!  call enter (l_here)

  call objects_name_add_once (needed_name)

  needed_index = string_in_array_locate_or_die (needed_name, objects_names)
    
  call int_array_list_elem_add (needed_index, nodes_nb ,nodes_need)

!  call exit (l_here)

  end subroutine object_needed

! =============================================================================
  function cur_needed_ind (need_i)  result (result)
! -----------------------------------------------------------------------------
!
! Name           : cur_needed_ind
!
! Description    : cycle on the needed objects at node cur_nod
!                  
! Authors: R. Assaraf
! Date   : 14 Jul 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)  :: result 
  integer (dpi) :: need_i

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here 

! begin:
  l_here='cur_needed_ind'
  call enter (l_here, 3)
  if (debug) write(logf,*) trim(l_here), '-d: need',nodes_need%first (cur_nod)
  result = nodes_need%val(nodes_need%first (cur_nod)+need_i-1)

  call exit (l_here, 3)

  end function cur_needed_ind

! =============================================================================
  function cur_create_ind (create_i)  result (result)
! -----------------------------------------------------------------------------
!
! Name           : cur_create_ind
!
! Description    : cycle on the needed objects at node cur_nod
!                  
! Authors: R. Assaraf
! Date   : 14 Jul 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: result 
  integer(dpi) :: create_i

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here 

! begin:
  l_here='cur_create_ind'
  call enter (l_here, 3)

  result = nodes_create%val(nodes_create%first (cur_nod)+create_i-1)

  call exit (l_here, 3)

  end function cur_create_ind

! =============================================================================
  subroutine object_create (build_name)
! -----------------------------------------------------------------------------
!
! Name           :  object_create
!
! Argument (in ) :  build_name
!
! Description    : object build_name depends on routine 
! Description    : add this information in 'objects' and in  'nodes'
!
! Authors: R. Assaraf
! Date   : 9 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: build_name
! local:                                                                      
  character(len=max_string_len_routine_name)          :: l_here 
  integer(dpi)                                         :: built_index 

! begin:
  l_here='object_create'
  call enter (l_here, 2)

  if (debug) write (logf,*) trim(l_here), '-d: build_name  >',trim(build_name),'<' 

  call objects_name_add_once (build_name)

  built_index = string_in_array_locate (build_name, objects_names)

  if(built_index == 0) then
    message = 'in routine'*l_here*' object name >'+build_name+'< not catalogued'
    call require (message, built_index /= 0)
  end if

  call int_array_list_elem_add (built_index, nodes_nb, nodes_create)
 
  objects_node (built_index) = nodes_nb
  
  objects_used (built_index) = .true.
 
  objects_desc (built_index) = trim(description)

  description = NOT_INIT_S 

  call exit (l_here, 2)

  end subroutine object_create

! =============================================================================
  subroutine one_node_write (node_i, unit, prefix)
! -----------------------------------------------------------------------------
!
! Name           :  one_node_write 
!
! Argument (in ) : node_i
! Argument (in)  : unit
! Argument (in)  : prefix 
!
! Description    : writes the creation node node_i
!                  
! Authors: R. Assaraf
! Date   : 4 June 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: node_i
  integer(dpi) :: unit
  character(len = *) :: prefix

! local:      
  character(len=max_string_len) :: l_prefix
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi) :: first, last, val_i

! begin:
  l_here='one_node_write'
  call enter(l_here, 3)

  l_prefix = prefix 

  if (unit==logf) then 
     if (prefix == blanks) call die (l_here, 'prefix should be a routine name')
     l_prefix=l_prefix+'-'+l_here+': '
  endif
  
  if (nodes_nb == 0) call die (l_here, 'nodes_nb='+nodes_nb+' should be > 0')

  if (node_i <= 0 ) call die (l_here, 'node_i='*node_i*' should be > 0')
  if (node_i > nodes_nb) call die (l_here, 'node_i='*node_i*' should be <='*nodes_nb)

  write (unit,'(2a)')trim (l_prefix),' '
  write (unit,'(2a,i5)')trim (l_prefix),' node # ',node_i
  write (unit,'(4a)')trim (l_prefix),' routine >',trim(routines_name (nodes_rtn (node_i))),'<'

  first = nodes_need%first (node_i)
  last  = nodes_need%last  (node_i)

  if (first <= 0 ) then
    write (unit, '(2a)')trim (l_prefix),' needs: none '
  else
     do val_i = first, last
        write (unit,'(3a)') trim (l_prefix),' needs: ',&
             trim(objects_names(nodes_need%val (val_i)))
     enddo

  endif
   
  first = nodes_create%first (node_i)
  last  = nodes_create%last  (node_i)

  if (first <= 0 ) then
     write (unit,'(3a)') trim (l_prefix),' creates: none '
  else
     do val_i = first, last
        write (unit,'(3a)') trim (l_prefix),' creates: ', &
             trim(objects_names(nodes_create%val (val_i)))
     enddo
  endif
  write (unit,*)trim (l_prefix),' '

  call exit(l_here, 3)

  end subroutine one_node_write 

! =============================================================================
  subroutine nodes_print 
! -----------------------------------------------------------------------------
!
! Name           :  nodes_print
!
! Description    : prints the creation node node_i
!                  
! Authors: R. Assaraf
! Date   : 4 June 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:

! local:      
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi) :: node_i

  l_here='nodes_print'
  call enter(l_here)
  
  if (nodes_nb == 0) call die (l_here, 'nodes_nb='+nodes_nb+' should be > 0')

  do node_i= 1,nodes_nb
    call one_node_write (node_i,logf,l_here)
  enddo 

  call exit(l_here)

  end subroutine nodes_print

! =============================================================================
  subroutine types_name_add_once (name)
! -----------------------------------------------------------------------------
!
! Name           : types_name_add_once
!
! Argument (in ) : name
!
! Description    : put name in objects_names if not already present  
!                  
! Authors: R. Assaraf
! Date   : 26 May 2002
! Revision: 29 Oct 2002 F. Colonna trace_depend
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: name

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)                                :: size_init= 20
  integer(dpi)                                :: index 

! begin:
  l_here='types_name_add_once'
  call enter (l_here, 5)

  if (.not.associated (types_name)) then
    if(trace_depend) then
      write (logf,*)trim(l_here),': types_names not associated'
      write (logf,*)trim(l_here),': calling types_resize (',size_init,')'
    end if

    call types_resize (size_init)

    if (types_nb /= NOT_INIT_I) then
      call die (l_here,'types_nb='*types_nb*' should be NOT_INIT_I')
    end if
 
    types_nb = 0
    index = 0

  else

! Improve
!    write (logf,*)trim(l_here),': types_name already associated'
  end if

  index = string_in_array_locate (name, types_name)
  if (index == 0) then 
    call  string_in_list_add  (name, types_name)
    types_nb = types_nb + 1
  endif

  call types_resize (size (types_name))
 
  call exit (l_here, 5)

  end subroutine types_name_add_once

! =============================================================================
  subroutine types_resize (new_size)
! -----------------------------------------------------------------------------
!
! Name           :  types_resize
!
! Argument       :  new_size
!
! Description    :  the sizes of the fields of types are reset 
! Description    :  to new_size
!                  
! Authors: R. Assaraf
! Date   : 9 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                  :: new_size          

! local:                                                                      
  integer(dpi)          :: size_fields 
  character(len=max_string_len_routine_name)           :: l_here 

! begin:
  l_here='types_resize'
  call enter (l_here, 5)

! check sizes

  if (associated(types_name)) then
     size_fields = size (types_name)
  else
     size_fields = 0
  endif

  if  (size_fields>new_size) then
    message='size_fields='+size_fields+' should be <= new_size='+new_size
    call die (l_here,message)
  endif

! field name
  call string_list_expand (types_name,new_size)
  
! field print routine 
  call integer_list_expand (types_write, new_size)  
 
  call exit (l_here, 5)

  end subroutine types_resize

! ===========================================================================
  subroutine object_value_bynam_print (obj_name)
! ---------------------------------------------------------------------------
!
! Name           :  object_value_bynam_print
!
! Argument (in ) : obj_name
!
! Description    : prints object named obj_name  
!
! Authors: R. Assaraf
! Date   : 14 June 2002
! ---------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: obj_name

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)  :: unit 
 
! begin
  l_here='object_value_bynam_print'
  call enter (l_here)

  unit = out
  call object_value_bynam_write (obj_name, unit)
 
  call exit (l_here)
  
  end subroutine object_value_bynam_print

! =============================================================================
  subroutine object_value_bynam_write (obj_name, unit)
! -----------------------------------------------------------------------------
!
! Name           :  object_value_bynam_write
!
! Argument (in ) : obj_name
!
! Description    : prints object named obj_name  
!
! Authors: R. Assaraf
! Date   : 14 June 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: obj_name
  integer(dpi)      :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)  :: obj_i
 
! begin
  l_here='object_value_bynam_write'
  call enter (l_here)
 
  obj_i = string_in_array_locate_or_die (obj_name,objects_names)

  call object_value_byind_write (obj_i, unit)
 
  call exit (l_here)
  
  end subroutine object_value_bynam_write


! =============================================================================
  subroutine object_value_byind_print (index)
! -----------------------------------------------------------------------------
!
! Name           :  object_value_byind_print
!
! Argument (in ) : index
!
! Description    : prints object  
!
! Authors: R. Assaraf
! Date   : 17 May 2001
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: index
 
! local:
  character(len=max_string_len_routine_name) :: l_here 
 
! begin:
  l_here='object_value_byind_print'
  call enter(l_here)

  call object_value_byind_write (index, out)

  call exit(l_here)

  end subroutine object_value_byind_print

! =============================================================================
  subroutine object_value_byind_write (index, unit)
! -----------------------------------------------------------------------------
!
! Name           :  object_value_byind_write
!
! Argument (in ) : index
! Argument (in ) : unit 
!
! Description    : writes object at index "index" on unit "unit"
! Improve quelle est la routine qui ecrit? il doit etre facile de le retrouver
!
! Authors: R. Assaraf
! Date   : 17 May 2001
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: index
  integer(dpi) :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi) :: cur_type
  integer(dpi) :: cur_write ,cur_write_addr
  integer(dpi) :: cur_obj_addr
  integer(dpi) :: unit_addr

! begin:
  l_here='object_value_byind_write'
  call enter(l_here)

! object characteristics 
 
  if (objects_nb <= 0) call die (l_here,' objects_nb='+objects_nb)
  if (index <= 0 ) call die (l_here, 'index ='+index+' should be > 0')
  if (index > objects_nb) call die (l_here, 'index ='+index+' should be < '+objects_nb)

  cur_type = objects_type (index)

  cur_obj_addr = objects_addr (index) 

! Improve: one condition at a time
  if (cur_type <= 0 .or. cur_obj_addr <= 0 ) then
    call one_object_info_write (index,logf,l_here)
    write (logf,*)trim(l_here),'-e: object at index =',index,' has not been allocated'
    call exit(l_here)
    return
  endif

  if (cur_type > types_nb) &
    call die(l_here,'type '*cur_type*'should be < types_nb='*types_nb)

! Improve obscur
! cur_write is the index of the wrting routine for the current type 
  cur_write = types_write ( cur_type )

  if(cur_write <=0 ) call die(l_here,'cur_write='*cur_write)

! get print routine address.
  cur_write_addr = routines_addr (cur_write)
  
  if (cur_write_addr < 0 ) &
     call die(l_here,'cur_write_addr='*cur_write_addr)

! define text_print : print message 

  if (unit == logf)  then 
     text_print = ' '//trim(l_here)//': '//trim(objects_names ( index))
  else
      text_print = trim(objects_names ( index))
  endif

! call cur_write (cur_obj, unit) from the addresses 
  unit_addr = address (unit)  

  call exe_by_address_2 (cur_write_addr,(/ cur_obj_addr, unit_addr/))

  call exit(l_here)

  end subroutine object_value_byind_write

! =============================================================================
  subroutine objects_init 
! -----------------------------------------------------------------------------
!
! Name           :  routines_init
!
! Description    : initialize all objects arrays  
! Improve which ?
! 
! Authors: R. Assaraf
! Date   : 11 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
! external :: routines_cat

! local:
  character(len=max_string_len_routine_name)  :: l_here 
  integer(dpi)                                 ::  node_i,obj_i
  integer(dpi) :: first, last

! begin:
  l_here='objects_init'
  call enter (l_here)
 
! initialize:
  
  if (associated (objects_names)) call die (l_here, 'must be called once')
 
  if (nodes_nb == 0) call nodes_init
     
! check consistency between objects_node and nodes%create 
  do node_i = 1,nodes_create%nb
    first = nodes_create%first (node_i)
    last  = nodes_create%last  (node_i)
  
    if (first /= NOT_INIT_I) then
      do obj_i = first, last 

! Improve obscur
        if (objects_node (nodes_create%val (obj_i)) /= node_i) then
             
          write (logf,*) trim (l_here),'-w: object ',                      &
                 '>',trim(objects_names (nodes_create%val (obj_i))), &
                 '< created by more than one node' 

          call one_node_write (node_i, logf, l_here)

          write (logf,*) trim (l_here),'-w: creation node assumed: ' 

          call one_node_write (objects_node (nodes_create%val (obj_i)) &
               ,logf, l_here)
                 
          print * 

        endif
      enddo
    endif
  enddo ! node_i 

  call exit (l_here)

  end subroutine objects_init

! =============================================================================
  subroutine object_bynam_in_list_add (obj_name,obj_byind_list, obj_list_nb)
! -----------------------------------------------------------------------------
!
! Name           :  object_bynam_in_list_add
!
! Argument (in ) : obj_name
! Argument       : obj_byind_list
! Argument       : obj_list_nb
!
! Authors: R. Assaraf
! Date   : 14 June 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: obj_name
  integer(dpi), pointer :: obj_byind_list (:)
  integer(dpi)          :: obj_list_nb

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)  :: obj_i

! begin:
  l_here='object_bynam_in_list_add'
  call enter(l_here)

! object characteristics 
   if (objects_nb <= 0) &
     call die (l_here,' objects_nb='+objects_nb+' has a not allowed value')
 
   obj_i = string_in_array_locate_or_die (obj_name,objects_names)
   if (obj_i < 0 .or. obj_i > objects_nb) &
     call die (l_here, 'obj_i ='+obj_i+' should be in [0,obejcts_nb]')

  call integer_in_list_add_once (obj_i, obj_byind_list, obj_list_nb)
 
  call exit(l_here)

  end subroutine object_bynam_in_list_add

! =============================================================================
  subroutine objects_name_add_once (name)
! -----------------------------------------------------------------------------
!
! Name           :  objects_complete
!
! Argument (in ) :  name
!
! Description    : put object in objects_names if not already present  
!                  
! Authors: R. Assaraf
! Date   : 9 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: name

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi) :: size_init= 100 ! Improve NON trop cache mem si pas dangereux
  integer(dpi) :: index 

! begin:
  l_here='objects_name_add_once'
  call enter (l_here, 2)

  if  (.not.associated (objects_names)) then
    if (trace_depend) then
      write (logf,*)trim(l_here),': objects_names not associated'
      write (logf,*)trim(l_here),': calling objects_resize (',size_init,')'
    end if
    call objects_resize (size_init)
    index = 0
    if (objects_nb/= NOT_INIT_I) &
      call die(l_here,'objects_nb='*objects_nb*' should be NOT_INIT_I')
      objects_nb = 0
  else
    if (trace_depend) then
      write (logf,*)trim(l_here),': >objects_names< already associated'
    end if
  end if

  index = string_in_array_locate (name, objects_names)
  if (index == 0) then 
    call  string_in_list_add  (name, objects_names)
    objects_nb = objects_nb + 1
  endif

  call objects_resize (size (objects_names))
 
  call exit (l_here, 2)

  end subroutine objects_name_add_once

! =============================================================================
  subroutine objects_resize (new_size)
! -----------------------------------------------------------------------------
!
! Name           :  objects_resize
!
! Argument (in ) :  new_size
!
! Description    :  the sizes of the fields of objects are reset 
! Description    :  to new_size
!                  
! Authors: R. Assaraf
! Date   : 9 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: new_size          

! local:                                                                      
  character(len=max_string_len_routine_name)           :: l_here 
  integer(dpi) :: size_fields, add 

! begin:
  l_here='objects_resize'
  call enter (l_here, 2)

! check sizes
  if (associated(objects_names)) then 
     size_fields = size (objects_names)
  else
     size_fields = 0
  endif

  if  (size_fields>new_size) then
        message='size_fields='+size_fields+' should be <= new_size='+new_size
        call die (l_here,message)
  endif

! field name
  call string_list_expand (objects_names,new_size)

! field addr
  call integer_list_expand (objects_addr, new_size)  
  do add = 1,size (objects_addr)
   if (objects_addr(add) < 0)  objects_addr(add) = 0
  enddo 

! field type 
  call integer_list_expand (objects_type, new_size) 

! field node 
  call integer_list_expand (objects_node, new_size) 

! field used

  call boolean_list_expand (objects_used, new_size)

! field desc
  call string_list_expand (objects_desc, new_size)
  call exit (l_here, 2)

  end subroutine objects_resize

! =============================================================================
  subroutine objects_info_print
! -----------------------------------------------------------------------------
!
! Name           :  objects_info_print
!
! Description    :  print objects 
!
! Authors: R. Assaraf
! Date  :  11 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
   
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: obj

! begin:
  l_here='objects_info_print'
  call enter (l_here)
  
  write (logf,*)trim (l_here),': number of objects: ',objects_nb

!  call nodes_print
  do obj = 1,objects_nb
      write (logf,*)trim (l_here),': object # ',obj
      call one_object_info_write (obj, logf, trim(l_here)//":") 
      call one_object_info_write (obj,res,'')
      
  enddo ! do obj
   
  call exit (l_here)

  end subroutine objects_info_print

! =============================================================================
  subroutine one_object_info_write (obj, unit, prefix)
! -----------------------------------------------------------------------------
!
! Name           : one_object_info_write
!
! Argument       : obj
! Argument       : unit
! Argument       : prefix
! Description    : prints objects (obj)
!
! Authors: R. Assaraf
! Date  :   11 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: obj 
  integer(dpi) :: unit
  character(len = *) :: prefix
 
! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len= max_string_len)  :: l_prefix

! begin:
  l_here='one_object_info_write'
  call enter (l_here)

  l_prefix = prefix

  if (unit==logf) then 
     if (prefix == blanks) call die (l_here, 'prefix should be a routine name')
     l_prefix=prefix+'-'+l_here+':'
  endif

  write (unit,*) trim(l_prefix)
  write (unit,'(4a)') trim(l_prefix),   ' Object name:   >',trim (objects_names (obj)),'<'
  
  if (objects_desc (obj) == NOT_INIT_S) then 
     write (unit,'(3a)') trim(l_prefix),' Description:   ','not set'
  else
     write (unit,'(3a)') trim(l_prefix),' Description:   ',trim(objects_desc (obj))
  endif

 
  if (objects_type (obj) == NOT_INIT_I) then
    write (unit,'(3a)') trim(l_prefix), ' Type:          ',' not set'
  else
    write (unit,'(3a)') trim(l_prefix), ' Type:          ',trim (types_name (objects_type (obj)))
  endif

  if (objects_addr (obj) == NOT_INIT_I) then
    write (unit,'(3a)') trim(l_prefix),' Address:           ',' not set'
  else
    write (unit,'(2a,i10)') trim(l_prefix),' Address:        ',objects_addr (obj)
  endif

  if (objects_node (obj) == NOT_INIT_I ) then
     write (unit,'(3a)') trim(l_prefix), ' Creation node: ',' not set'
  else
     write (unit,'(2a,i5)') trim(l_prefix), ' Creation node: ', objects_node (obj)
     if (unit == logf) l_prefix=prefix+'-'+l_here+blanks
     call one_node_write (objects_node (obj),unit, l_prefix)
  endif

  call exit (l_here)

  end subroutine one_object_info_write

! =============================================================================
  function object_index (obj_name) result (result)
! -----------------------------------------------------------------------------
!
! Name           : object_index
!
! Argument       : obj_name
!
!
! Authors: R. Assaraf
! Date  :   15 May 2002
! -----------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: obj_name
  integer(dpi)     :: result
! local:
  character(len=max_string_len_routine_name) :: l_here
 
! begin:
  l_here='object_index'
  call enter (l_here, 3)

  result =  string_in_array_locate (obj_name,objects_names)

  if (result == 0) then
    call die (l_here, 'object >'+obj_name+'< does not exist in' &
     +' objects_names, please check spelling.' &
     +CR+' Or add creation routine in file routine routines_catalog_init')
  end if
                                   
  call exit (l_here, 3)

  end function object_index

! =============================================================================
  function node_from_operation (operator_name, operands_names) result (result)
! -----------------------------------------------------------------------------
!
! Name           : object_index_from_operation
!
! Argument(in)   : operator
! Argument(in)   : needs_names (need)
!
! Description : Give the node at which the creation operator is "operator_name"
! Description : and the needed objects are "operands_names
! Description : if node not found result = 0
!
! Authors: R. Assaraf
! Date  :   14 Fev 2002
! -----------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi) :: result

! i/o:
  character(len=*) :: operator_name
  character(len=*) :: operands_names (:)
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)  :: node_i
  integer(dpi)  :: need_i
  integer(dpi)  :: first, last
  logical(bool) :: is_identical
! begin:
  l_here='node_from_operation'
  call enter (l_here, 3)

  result = 0

  do node_i = 1, nodes_nb
     if (routines_name (nodes_rtn (node_i)) == operator_name) then

        first = nodes_need%first (node_i)
        last  = nodes_need%last  (node_i)
     
        is_identical = .false.
      
        if (first /= NOT_INIT_I) then
           if (last-first+1 == size(operands_names)) then

              is_identical = .true.

              do need_i = first, last
              
                 if (objects_names(nodes_need%val(need_i)) /= operands_names (need_i-first+1)) &
                      is_identical=.false.

              enddo ! do need_i
           endif
        endif

        if (is_identical) then
           result = node_i
           call exit (l_here)
           return
        endif

     endif
     
  enddo ! do node_i

  call exit (l_here, 3)

 end function node_from_operation


end module objects_mod
