!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

  subroutine poly4 (r, p, par)
! ------------------------------------------------------------------------
! Description    : calculates (((par(4)*r+par(3))*r+par(2))*r+par(1))*r
!
! author: P.Reinhardt
! date  : 24 Oct 2000
! ------------------------------------------------------------------------
!  
  use control_mod
  implicit none

  real(dp)     :: r,p
  real(dp)     :: par(4)
  integer(dpi) :: i

  p=par(4)
  do i= 3,1,-1
   p=r*p+par(i)
  end do
  p=r*p
  return
end subroutine poly4

! ========================================================================
  subroutine poly4_deriv (r, poly_deriv, par)
! ------------------------------------------------------------------------
! Description    : calculates the derivate of the polynome above
!
!
! Authors : X. Krokidis and R. assaraf
! Date    : 27 June 2001
! ------------------------------------------------------------------------
!
 use control_mod
  implicit none

!i/o
  real(dp) :: r, poly_deriv, par(4)

  poly_deriv= par(1)+r*(2.D0*par(2)+r*(3.D0*par(3)+r*4.D0*par(4)))
 
end subroutine poly4_deriv

! ========================================================================
  subroutine poly4_second_deriv (r, poly_second_deriv, par)
! ------------------------------------------------------------------------
! Description    :  calculates the second derivative
!
! author: P.Reinhardt
! date  : 30 Nov 2000
! ------------------------------------------------------------------------
!  
  use control_mod
  implicit none

  real(dp) :: r, par(4), poly_second_deriv

  poly_second_deriv= 2.D0*par(2)+r*(6.D0*par(3)+r*12.D0*par(4))

end subroutine poly4_second_deriv

! ========================================================================
  subroutine poly4d (r, p, p1, p2, par)
! ------------------------------------------------------------------------
! Description    : calculates (((par(4)*r+par(3))*r+par(2))*r+par(1))*r
! Description    : and derivatives
!
! author: P.Reinhardt
! date  : 30 Nov 2000
! ------------------------------------------------------------------------
  use control_mod
  implicit none

  real(dp) :: r,p,p1,p2
  real(dp) :: par(4)

  p = r*(par(1)+r*(par(2)+r*(par(3)+r*par(4))))
  p1= par(1)+r*(2.D0*par(2)+r*(3.D0*par(3)+r*4.D0*par(4)))
  p2= 2.D0*(par(2)+r*(3.D0*par(3)+r*6.D0*par(4)))
  return
end subroutine poly4d

