! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module psi_guide_bywlk_mod

  use basic_types_tools_mod
  use jastrow_factor_guide_bywlk_mod
  use det_sum_guide_bywlk_mod
  use psi_bywlk_mod
  use guide_menu_mod

  implicit none

  real(dp), pointer :: psi_guide_bywlk (:)          => NULL ()
  real(dp), pointer :: psi_guide_grad_bywlk (:,:,:) => NULL ()
  real(dp), pointer :: psi_guide_lapl_bywlk (:,:)   => NULL ()

  real(dp), pointer :: sum_rij_power_bywlk (:)   => NULL ()
  real(dp), pointer :: sum_rij_power_grad_bywlk (:,:,:)   => NULL ()
  real(dp), pointer :: sum_rij_power_power_bywlk (:)   => NULL ()
  real(dp), pointer :: sum_rij_power_power_grad_bywlk (:,:,:)   => NULL ()

  contains 

! ===================================================================================
  subroutine psi_guide_bywlk_bld
! -----------------------------------------------------------------------------------
 
! Description    : guiding wave function
!
! Created        : J. Toulouse, Aug 2005
! Revised        : J. Toulouse, 02 Oct 2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_len_conf_type_guide) :: l_jastr_type
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, walk_i
  logical(bool)                 :: no_jastrow

! header:

  if (header_exe) then

    l_here = "psi_guide_bywlk_bld"

    call object_create ('psi_guide_bywlk')

    call object_needed ('psi_bywlk')
    call object_needed ('jastrow_factor_guide_bywlk')
    call object_needed ('det_sum_guide_bywlk')
    call object_needed ('configurations_guide_weight')
    call object_needed ('sum_rij_power_power_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! requirements:
  include 'chksiz_configurations_guide.inc'
  include 'chksiz_det_sum_guide_bywlk.inc'

  l_jastr_type = configurations_guide(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_guide_bywlk.inc'
  end if

! end requirements

! dimensions:
  configurations_guide_nb = size (det_sum_guide_bywlk, 1)
  walkers_nb        = size (det_sum_guide_bywlk, 2)
! end dimensions:

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
     write (logf,*)trim(l_here),'-d: no_jastrow        =',no_jastrow         
     write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'
  end if        

! allocations:
      call alloc ('psi_guide_bywlk', &
                        psi_guide_bywlk,  &
                        walkers_nb)
! end allocations

  psi_guide_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb

! Psi(w)      = Sum_c W(c) * Jastrow(c,w) * S_det (c,w)
! S_det (c,w) = w(d) * det_alpha(d,c,w) * det_beta(d,c,w)

    if( .not. no_jastrow) then
      psi_guide_bywlk (walk_i) = psi_guide_bywlk (walk_i)                    &
                        + configurations_guide_weight (conf_i)         &
                        * jastrow_factor_guide_bywlk (conf_i, walk_i)  &
                        * det_sum_guide_bywlk (conf_i, walk_i)
    else 
      psi_guide_bywlk (walk_i) = psi_guide_bywlk (walk_i)                     &
                        + configurations_guide_weight (conf_i)          &
                        * det_sum_guide_bywlk (conf_i, walk_i)                                           
    end if

    end do  ! conf_i 

  end do    ! walk_i 

! Improve: check this!!!!!
  if (power_sum_rij /= 0.d0) then
    psi_guide_bywlk = psi_guide_bywlk * sum_rij_power_power_bywlk
  endif


  include 'debug_psi_guide_bywlk.inc'
!   include 'ensnb_psi_guide_bywlk.inc'

  call exit(l_here)

  end subroutine psi_guide_bywlk_bld 

! ===================================================================================
  subroutine psi_guide_grad_bywlk_bld
! -----------------------------------------------------------------------------------
! 
! Description    : gradient of guiding wave function
!
! Created        : J. Toulouse, Aug 2005
! Revised        : J. Toulouse, 02 Oct 2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_len_conf_type_guide) :: l_jastr_type
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb 
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, walk_i, dim_i, elec_i
  logical(bool)                 :: no_jastrow

! header:

  if (header_exe) then

    l_here = "psi_guide_grad_bywlk_bld"

    call object_create ('psi_guide_grad_bywlk')

    call object_needed ('psi_grad_bywlk')
    call object_needed ('jastrow_factor_guide_bywlk')
    call object_needed ('jastrow_factor_guide_grad_bywlk')
    call object_needed ('det_sum_guide_bywlk')
    call object_needed ('det_sum_guide_grad_bywlk')
    call object_needed ('configurations_guide_weight')
    call object_needed ('sum_rij_power_power_bywlk')
    call object_needed ('sum_rij_power_power_grad_bywlk')
    return

  end if

! begin:
  call enter(l_here)

! requirements:

  include 'chksiz_det_sum_guide_bywlk.inc'
  include 'chksiz_det_sum_guide_grad_bywlk.inc'

  l_jastr_type = configurations_guide(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_guide_bywlk.inc'
    include 'chksiz_jastrow_factor_guide_grad_bywlk.inc'
  end if

! end requirements

! dimensions:

  electrons_nb      = size (det_sum_guide_grad_bywlk, 1)
  configurations_guide_nb = size (det_sum_guide_grad_bywlk, 2)
  dimensions_nb     = size (det_sum_guide_grad_bywlk, 3)
  walkers_nb        = size (det_sum_guide_grad_bywlk, 4)

! end dimensions

  debug_a: if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
     write (logf,*)trim(l_here),'-d: no_jastrow        =',no_jastrow         
     write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'

     write (logf,*)trim(l_here),'-d: det_sum_guide_bywlk (conf,walk)'
     do conf_i = 1, configurations_guide_nb
        write (logf,*)trim(l_here),'-d: configurations_guide_weight (conf) =', &
             configurations_guide_weight (conf_i)
     end do

     if( .not. no_jastrow) then
        write (logf,*)trim(l_here),'-d: jastrow_factor_guide_bywlk (conf, walk)'
        do walk_i = 1, walkers_nb
           do conf_i = 1, configurations_guide_nb
              write (logf,*)trim(l_here), &
                   '-d: jastrow_factor_guide_bywlk (',conf_i,',',walk_i,') =', &
                   jastrow_factor_guide_bywlk (conf_i, walk_i)
           end do
        end do
     end if
     
  end if debug_a

! allocations


    call alloc ('psi_guide_grad_bywlk', psi_guide_grad_bywlk, &
                      electrons_nb, dimensions_nb, walkers_nb)

! end allocations

  psi_guide_grad_bywlk = zero

  do walk_i = 1, walkers_nb
    do conf_i = 1, configurations_guide_nb
      do elec_i = 1, electrons_nb

! Psi(e,w)       = Sum_c W(c,w) * Jastrow(c,w) * Sum_det (c,w)
! Grad Psi(e, w) = Sum_c W(c,w) * ( Grad Jastrow(e, c,w) * Sum_det (c,w)
!                            +   Jastrow(c,w) * Grad Sum_det (e, c,w) )

    if( .not. no_jastrow) then

      psi_guide_grad_bywlk (elec_i, :, walk_i) = psi_guide_grad_bywlk (elec_i, :, walk_i) & 
               + configurations_guide_weight (conf_i)                              &
               * ( jastrow_factor_guide_grad_bywlk (elec_i, conf_i, :, walk_i)     &
               *   det_sum_guide_bywlk (conf_i, walk_i)                            & 
               +   jastrow_factor_guide_bywlk (conf_i, walk_i)                     &
               *   det_sum_guide_grad_bywlk (elec_i, conf_i, :, walk_i) )

    else 

      psi_guide_grad_bywlk (elec_i, :, walk_i) = psi_guide_grad_bywlk (elec_i, :, walk_i) & 
               + configurations_guide_weight (conf_i)                              &
               * det_sum_guide_grad_bywlk (elec_i, conf_i, :, walk_i)                                           
    end if

      end do ! elec_i 
    end do ! conf_i 
  end do ! walk_i 

! Improve: check this!!!!!
  if (power_sum_rij /= 0.d0) then

    do walk_i = 1, walkers_nb
      do dim_i = 1, dimensions_nb
        do elec_i = 1, electrons_nb
    psi_guide_grad_bywlk(elec_i, dim_i, walk_i) =  &
    psi_guide_grad_bywlk(elec_i, dim_i, walk_i) * sum_rij_power_power_bywlk(walk_i) &
    + (psi_guide_bywlk(walk_i)/sum_rij_power_power_bywlk(walk_i))  &
      * sum_rij_power_power_grad_bywlk(elec_i, dim_i, walk_i)
        enddo
      enddo
    enddo

  endif ! power_sum_rij


  include 'debug_psi_guide_grad_bywlk.inc'
!   include 'ensnb_psi_guide_grad_bywlk.inc'

  call exit(l_here)

  end subroutine psi_guide_grad_bywlk_bld 

! ===================================================================================
  subroutine psi_guide_lapl_bywlk_bld
! -----------------------------------------------------------------------------------
 
! Description    : Laplacian of guiding wave function
!
! Created        : J. Toulouse, Aug 2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_len_conf_type_guide) :: l_jastr_type
  integer(dpi)                  :: configurations_guide_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: conf_i, walk_i, dim_i, elec_i
  logical(bool)                 :: no_jastrow
  real(dp)                      :: dot_prod

! header:

  if (header_exe) then

    l_here = "psi_guide_lapl_bywlk_bld"

    call object_create ('psi_guide_lapl_bywlk')

    call object_needed ('jastrow_factor_guide_bywlk')
    call object_needed ('jastrow_factor_guide_grad_bywlk')
    call object_needed ('jastrow_factor_guide_lapl_bywlk')
    call object_needed ('det_sum_guide_bywlk')
    call object_needed ('det_sum_guide_grad_bywlk')
    call object_needed ('det_sum_guide_lapl_bywlk')
    call object_needed ('configurations_guide_weight')
    return

  end if

! begin:
  call enter(l_here)

! requirements:

  include 'chksiz_det_sum_guide_bywlk.inc'
  include 'chksiz_det_sum_guide_grad_bywlk.inc'
  include 'chksiz_det_sum_guide_lapl_bywlk.inc'

  l_jastr_type = configurations_guide(1)%jastr_type
  no_jastrow = (l_jastr_type == 'NONE' .or. l_jastr_type == NOT_INIT_S)

  if( .not. no_jastrow) then
    include 'chksiz_jastrow_factor_guide_bywlk.inc'
    include 'chksiz_jastrow_factor_guide_grad_bywlk.inc'
    include 'chksiz_jastrow_factor_guide_lapl_bywlk.inc'
  end if

! end requirements

! dimensions:

  electrons_nb = size (det_sum_guide_grad_bywlk, 1)
  configurations_guide_nb = size (det_sum_guide_grad_bywlk, 2)
  dimensions_nb = size (det_sum_guide_grad_bywlk, 3)
  walkers_nb = size (det_sum_guide_grad_bywlk, 4)

! end dimensions

  if(debug) then
    write (logf,*)trim(l_here),'-d: configurations_guide_nb = ',configurations_guide_nb
    do conf_i = 1, configurations_guide_nb
     write (logf,*)trim(l_here),'-d: configurations_guide_weight (conf_i) = ',configurations_guide_weight (conf_i)
    end do
    write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
    write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb 
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
    write (logf,*)trim(l_here),'-d: no_jastrow        = ',no_jastrow         
    write (logf,*)trim(l_here),'-d: jastrow type      "',trim(l_jastr_type),'"'
  end if 

! allocations


      call alloc ('psi_guide_lapl_bywlk', &
                        psi_guide_lapl_bywlk,  &
                        electrons_nb, &
                        walkers_nb)

! end allocations

  include 'debug_jastrow_factor_guide_bywlk.inc'
  include 'debug_jastrow_factor_guide_lapl_bywlk.inc'
  include 'debug_jastrow_factor_guide_grad_bywlk.inc'
  include 'debug_det_sum_guide_bywlk.inc'
  include 'debug_det_sum_guide_grad_bywlk.inc'
  include 'debug_det_sum_guide_lapl_bywlk.inc'

  psi_guide_lapl_bywlk = zero

! Psi(w) = Sum_c W(c,w) * Jastrow(c,w) * S_det (c,w)
!
! Lapl Psi(e, w) = Sum_c W(c,w) 
!                * ( Lapl Jastrow(e, c,w) * Sum_det (c,w)
!                +   Jastrow(c,w) * Lapl Sum_det (e, c,w) 
!                + 2 Grad Jastrow(e, c,w) . Grad Sum_det (e, c,w) 

! Improve: dot product

  if( .not. no_jastrow) then
    do walk_i = 1, walkers_nb
      do conf_i = 1, configurations_guide_nb
        do elec_i = 1, electrons_nb      

      dot_prod = zero
		  do dim_i = 1, dimensions_nb
        dot_prod = dot_prod &
           +  jastrow_factor_guide_grad_bywlk (elec_i, conf_i, dim_i, walk_i) &
           *  det_sum_guide_grad_bywlk (elec_i, conf_i, dim_i, walk_i) 
      end do  ! dim_i 

           psi_guide_lapl_bywlk (elec_i, walk_i)                          &
             = psi_guide_lapl_bywlk (elec_i, walk_i)                      &
             + configurations_guide_weight (conf_i)                       &
             * ( jastrow_factor_guide_lapl_bywlk (elec_i, conf_i, walk_i) &
                 * det_sum_guide_bywlk (conf_i, walk_i)                   & 

             +     jastrow_factor_guide_bywlk (conf_i, walk_i)             &
                  * det_sum_guide_lapl_bywlk (elec_i, conf_i, walk_i)      &

             +     two * dot_prod )

          end do  ! elec_i 
        end do  ! conf_i 
    end do  ! walk_i 

  else 

    do walk_i = 1, walkers_nb
		  do conf_i = 1, configurations_guide_nb
        do elec_i = 1, electrons_nb      

          psi_guide_lapl_bywlk (elec_i, walk_i)                 &
            = psi_guide_lapl_bywlk (elec_i, walk_i)             &
            + configurations_guide_weight (conf_i)              &
            * det_sum_guide_lapl_bywlk (elec_i, conf_i, walk_i)                                           
      end do  ! elec_i 
    end do  ! conf_i 
  end do    ! walk_i 

  end if

  include 'debug_psi_guide_lapl_bywlk.inc'
!   include 'ensnb_psi_guide_lapl_bywlk.inc'

  call exit(l_here)

  end subroutine psi_guide_lapl_bywlk_bld 

! ===================================================================================
  subroutine sum_rij_power_bywlk_bld
! -----------------------------------------------------------------------------------

! Description    : factor Sum_{i<j} rij^(power_rij)] for guiding wave function
!
! Created :  J. Toulouse, 01 Aug 2005
! Revised :  J. Toulouse, 02 Oct 2005
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: elec_i, elec_j, walk_i

! header:

  if (header_exe) then

    l_here = "sum_rij_power_power_bywlk_bld"

    call object_create ('sum_rij_power_bywlk')

    call object_needed ('ee_distance_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! dimensions:
  electrons_nb      = size (ee_distance_bywlk, 1)
  walkers_nb        = size (ee_distance_bywlk, 3)

! allocations:
   call alloc ('sum_rij_power_bywlk', sum_rij_power_bywlk, walkers_nb)

   if (power_sum_rij == 0.d0) then
    sum_rij_power_bywlk  = 1.d0

   else

     sum_rij_power_bywlk = zero

   do walk_i = 1, walkers_nb

        do elec_i = 1, electrons_nb
         do elec_j = elec_i + 1, electrons_nb
    sum_rij_power_bywlk(walk_i) = sum_rij_power_bywlk(walk_i) + ee_distance_bywlk(elec_i, elec_j, walk_i)**power_rij

         enddo
        enddo

     enddo ! walk_i

   endif

  call exit(l_here)

  end subroutine sum_rij_power_bywlk_bld

! ===================================================================================
  subroutine sum_rij_power_power_bywlk_bld
! -----------------------------------------------------------------------------------

! Description    : factor [Sum_{i<j} rij^(power_rij)]^(power_sum_rij) for guiding wave function
!
! Created :  J. Toulouse, 03 Oct 2005
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: walkers_nb

! header:

  if (header_exe) then

    l_here = "sum_rij_power_power_bywlk_bld"

    call object_create ('sum_rij_power_power_bywlk')

    call object_needed ('sum_rij_power_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! dimensions:
  walkers_nb        = size (sum_rij_power_bywlk)

! allocations:
   call alloc ('sum_rij_power_power_bywlk', sum_rij_power_power_bywlk, walkers_nb)

   if (power_sum_rij == 0.d0) then
    sum_rij_power_power_bywlk  = 1.d0

   else

    sum_rij_power_power_bywlk = sum_rij_power_bywlk**power_sum_rij

   endif

  call exit(l_here)

  end subroutine sum_rij_power_power_bywlk_bld

! ===================================================================================
  subroutine sum_rij_power_grad_bywlk_bld
! -----------------------------------------------------------------------------------
 
! Description    : gradient of sum_rij_power

! Created: J. Toulouse, 02 Aug 2005
! Revised: J. Toulouse, 02 Oct 2005
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: elec_i, elec_j, dim_i, walk_i

! header:

  if (header_exe) then

    l_here = "sum_rij_power_grad_bywlk_bld"

    call object_create ('sum_rij_power_grad_bywlk')

    call object_needed ('sum_rij_power_bywlk')
    call object_needed ('ee_distance_bywlk')
    call object_needed ('ee_dist_grd_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! dimensions:
  electrons_nb      = size (ee_dist_grd_bywlk, 1)
  dimensions_nb     = size (ee_dist_grd_bywlk, 3)
  walkers_nb        = size (ee_dist_grd_bywlk, 4)
! end dimensions:

! allocations:
   call alloc ('sum_rij_power_grad_bywlk', sum_rij_power_grad_bywlk, electrons_nb, dimensions_nb, walkers_nb)
! end allocations

 sum_rij_power_grad_bywlk = zero

 if (power_sum_rij /= 0.d0) then

  do walk_i = 1, walkers_nb
    do dim_i = 1, dimensions_nb
     do elec_j = 1, electrons_nb

        do elec_i = elec_j + 1, electrons_nb

         sum_rij_power_grad_bywlk (elec_j, dim_i, walk_i) = sum_rij_power_grad_bywlk (elec_j, dim_i, walk_i) &
             + power_rij * ee_distance_bywlk (elec_j, elec_i, walk_i)**(power_rij -1.d0) &
                 * ee_dist_grd_bywlk (elec_j, elec_i, dim_i, walk_i)

        end do !elec_i

      end do  ! elec_j
    end do !dim_i
  end do !walk_i

 endif

  call exit(l_here)

  end subroutine sum_rij_power_grad_bywlk_bld

! ===================================================================================
  subroutine sum_rij_power_power_grad_bywlk_bld
! -----------------------------------------------------------------------------------
 
! Description    : gradient of sum_rij_power_power
!
! Created: J. Toulouse, 03 Oct 2005
! -----------------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: elec_i, elec_j, dim_i, walk_i

! header:

  if (header_exe) then

    l_here = "sum_rij_power_power_grad_bywlk_bld"

    call object_create ('sum_rij_power_power_grad_bywlk')

    call object_needed ('sum_rij_power_bywlk')
    call object_needed ('sum_rij_power_grad_bywlk')

    return

  end if

! begin:
  call enter(l_here)

! dimensions:
  electrons_nb      = size (sum_rij_power_grad_bywlk, 1)
  dimensions_nb     = size (sum_rij_power_grad_bywlk, 2)
  walkers_nb        = size (sum_rij_power_grad_bywlk, 3)

! allocations:
  call alloc ('sum_rij_power_power_grad_bywlk', sum_rij_power_power_grad_bywlk, electrons_nb, dimensions_nb, walkers_nb)


  sum_rij_power_power_grad_bywlk = zero


 if (power_sum_rij /= 0.d0) then

  do walk_i = 1, walkers_nb
    do dim_i = 1, dimensions_nb
     do elec_j = 1, electrons_nb

     sum_rij_power_power_grad_bywlk (elec_j, dim_i, walk_i) =    &
       power_sum_rij * sum_rij_power_bywlk(walk_i)**(power_sum_rij - 1.d0) * sum_rij_power_grad_bywlk (elec_j, dim_i, walk_i)


      end do  ! elec_j
    end do !dim_i
  end do !walk_i

 endif

  call exit(l_here)

  end subroutine sum_rij_power_power_grad_bywlk_bld

end module psi_guide_bywlk_mod
