! -*- f90 -*- 
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module routines_mod

  use tools_mod
  use list_mod    
  use address_tools_mod 

  implicit none

  character(max_string_len_routine_name), pointer :: routines_name (:) => NULL()
  integer(dpi), pointer                           :: routines_addr (:) => NULL()
  integer(dpi), pointer                           :: routines_args (:) => NULL()
  integer(dpi)                                    :: routines_nb = NOT_INIT_I

  integer(dpi), private :: routine_cur
  
  logical(bool)         :: header_exe = .false. ! flag to execute routines 
                                                ! headers 
  real(dp), pointer :: routines_cpu (:) => NULL ()

  contains

! ========================================================================
  function routine_cur_index ()  result (result)
! ------------------------------------------------------------------------
!
! Name           :  routine_index
! Argument       : 
!
! Description    : returns routine_cur which is private
!             
! Authors: R. Assaraf
! Date   : 17 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: result
     
! local: 
     
! begin:
    result = routine_cur

  end function  routine_cur_index


 
! ========================================================================
  function routine_cur_name ()  result (result)
! ------------------------------------------------------------------------
!
! Name           :  routine_index
! Argument       : 
!
! Description    : returns the name for the routine routine_cur
!             
! Authors: R. Assaraf
! Date   : 17 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=max_string_len_routine_name) :: result
     
! local: 
     
! begin:

  result = routines_name (routine_cur)

  end function routine_cur_name
  
! ========================================================================
  subroutine one_routine_header_exe (rtn_i)
! ------------------------------------------------------------------------
!
! Name           : one_routine_header_exe
!
! Argument (in ) : rtn_i
!
! Description    : executes the header of routine number rtn_i
! Description    : sets routine_cur 
!  
! Authors: R. Assaraf
! Date   : 9 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: rtn_i

! local:
  character(len=max_string_len_routine_name) :: l_here 

! begin:
  l_here='one_routine_header_exe'
  call enter (l_here, 2)
  
  if (debug) then
!  if (debug .or. trace_level >= 2) then
     write (logf,*) trim (l_here),'-d: rtn_i=',rtn_i,' >',&
              trim(routines_name(rtn_i)),'< address=',routines_addr(rtn_i) 
  end if

! routine header execution. 
  header_exe  = .true.
  routine_cur = rtn_i

  call exe (routines_addr (rtn_i))

  header_exe  = .false.

  call exit (l_here, 2)
  
  end subroutine one_routine_header_exe

! ========================================================================
  subroutine routine_byind_exe (rtn_i)
! ------------------------------------------------------------------------
!
! Name           : routine_byind_exe
!
! Argument (in ) : rtn_i
!
! Description    : executes the routine # rtn_i
! Description    : sets routine_cur 
!  
! Authors: R. Assaraf
! Date   : 26 Nov 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: rtn_i

! local:
  character(len=max_string_len_routine_name) :: l_here 
  real(dp) :: cpu1, cpu2
! begin:
  l_here='routine_byind_exe'
  call enter (l_here, 2)
  
  debug_rout: if (debug .or. trace_level >= 2) then
     write (logf,*) trim (l_here),'-d: rtn_i=',rtn_i,' >',&
              trim(routines_name(rtn_i)),'< address=',routines_addr(rtn_i) 
  end if debug_rout
  

! routine  execution. 

  if (.not.associated (routines_cpu)) then
     allocate (routines_cpu (routines_nb))
     routines_cpu = 0.d0
  endif

  routine_cur = rtn_i

  call cpu_time(cpu1)
  call exe (routines_addr (rtn_i))
  call cpu_time(cpu2)

  routines_cpu (rtn_i) = routines_cpu (rtn_i) + cpu2-cpu1

  call exit (l_here, 2)
  
  end subroutine routine_byind_exe

! ========================================================================
  subroutine routine_bynam_exe (rtn_name)
! ------------------------------------------------------------------------
!
! Name           : routine_bynam_exe
!
! Argument (in ) : rtn_name
!
! Description    : executes the routine of name rtn_name
!  
! Authors: R. Assaraf
! Date   : 26 Nov 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: rtn_name

! local:
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi) :: ind

! begin:
  l_here='routine_bynam_exe'
  call enter (l_here, 2)

! routine header execution. 
  ind = string_in_array_locate (rtn_name, routines_name)
  if ( ind == 0 ) call die (l_here, 'routine >'+rtn_name+'<does not exist'&
                            +' is not catalogued in array rtn_name '+CR+ &
                             ' please check its name or add it in '//   &
                             '  routines_catalog.inc')

  call routine_byind_exe (ind)

  call exit (l_here, 2)
  
  end subroutine routine_bynam_exe

! ========================================================================
  subroutine header_die
! ------------------------------------------------------------------------
!
! Name           : header_die
!
! Description    : to put in routines with no header
!  
! Authors: R. Assaraf
! Date   : 30 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:

! local:
   character(len=max_string_len_routine_name) :: l_here 

! begin:
   l_here = 'header_die'
   call enter(l_here,3)

   if (header_exe) then
    l_here = "header_die"
    call die (l_here,'delete this routine from file routines_catalog.inc')
    return
  end if

  call exit(l_here,3)

  end subroutine header_die

! ========================================================================
  subroutine one_routine_catalog (rtn_name, routine, args_nb)
! ------------------------------------------------------------------------
!
! Name           : one_routine_catalog
!
! Argument (in ) : rtn_name
! Argument (in ) : routine
! Argument (in ) : args_nb
!                  
! Improve Description
!
! Authors: R. Assaraf
! Date   : 9 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: rtn_name
  external  routine
  integer(dpi), optional :: args_nb 

! local:                                                                      
  integer(dpi)  :: index 
  character(len=max_string_len_routine_name)           :: l_here 

! begin:
  l_here='one_routine_catalog'
  call enter (l_here, 2)
    
  call routines_name_add_once (rtn_name)

  index = string_in_array_locate_or_die (rtn_name, routines_name)
    
  routines_addr (index) = address (routine)

  routines_args (index) = 0

  if (present(args_nb)) then 
    if (args_nb < 0) call die (l_here, 'args_nb = '*args_nb)
    routines_args (index) = args_nb
  endif

  call exit (l_here, 2)

  end subroutine one_routine_catalog

! ========================================================================
  subroutine routines_name_add_once (name)
! ------------------------------------------------------------------------
!
! Name           : routines_name_add_once
!
! Argument (in ) : name
!
! Description    : put name in routines_name if not already present  
! Description    : and manage all routines arrays sizes
! 
! Authors: R. Assaraf
! Date   : 9 May 2002
! Revision: 29 Oct 2002 F. Colonna trace_depend
! ------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: name

! local:                                                                      
  character(len=max_string_len_routine_name) :: l_here 
  integer(dpi)                                :: size_init= 100
  integer(dpi)                                :: index 

! begin:
  l_here='routines_name_add_once'
  call enter (l_here, 2)

! initialization of routines arrays and current routine number routines_nb 
  if (.not.associated (routines_name)) then
    if (trace_depend) then
      write (logf,*)trim(l_here),': routines_name not associated'
      write (logf,*)trim(l_here),': calling routines_arrays_size_update (',size_init,')'
    end if
    call routines_arrays_size_update (size_init)
    routines_nb = 0

  else
!    write (logf,*)trim(l_here),': routines_names already associated'
  end if

  index = string_in_array_locate (name, routines_name) !, 1, size(routines_name))
  if (index == 0) then 
    call  string_in_list_add  (name, routines_name)
    routines_nb = routines_nb + 1
  endif

  call routines_arrays_size_update (size (routines_name))
 
  call exit (l_here, 2)

  end subroutine routines_name_add_once

! ========================================================================
  subroutine routines_arrays_size_update (new_size)
! ------------------------------------------------------------------------
!
! Name           :  routines_arrays_size_update
!
! Argument       :  new_size
!
! Description    :  the sizes of the fields of routines are reset 
! Description    :  to new_size
!                  
! Authors: R. Assaraf
! Date   : 9 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                                :: new_size          

! local:                                                                      
  integer(dpi)                                :: size_fields 
  character(len=max_string_len_routine_name) :: l_here 

! begin:
  l_here='routines_arrays_size_update'
  call enter (l_here, 2)

! check sizes
  if (associated (routines_name)) then
     size_fields = size (routines_name)
  else
    size_fields = 0
  endif

  if  (size_fields > new_size) then
    message='size_fields='+size_fields+' should be <= new_size='+new_size
    call die (l_here,message)
  endif

! field name
  call string_list_expand (routines_name,new_size)
  
! field addr
  call integer_list_expand (routines_addr, new_size)  

! field args
  call integer_list_expand (routines_args, new_size) 
  
  call exit (l_here, 2)

  end subroutine routines_arrays_size_update

! ========================================================================
  subroutine routines_print
! ------------------------------------------------------------------------
!
! Name           :  routines_print
!
! Description    :  print routines 
!
! Authors: R. Assaraf
! Date  :  11 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
   
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: rtn_i

! begin:
    l_here='routines_print'
  call enter (l_here)
  
    write (logf,*)trim (l_here),' number of routines: ',routines_nb

    do rtn_i = 1,routines_nb
      write (logf,*)trim (l_here),' routine # ',rtn_i
      call one_routine_print (rtn_i)
      print * 
    enddo 
   
  call exit (l_here)

  end subroutine routines_print

! ========================================================================
  subroutine one_routine_print (rtn_i)
! ------------------------------------------------------------------------
!
! Name           : one_routine_print
!
! Argument       : rtn_i
!
! Description    : prints routine rtn_i
!
! Authors: R. Assaraf
! Date  :   11 May 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi) :: rtn_i

! local:
  character(len=max_string_len_routine_name) :: l_here
 
! begin:
  l_here='one_routine_print'
  call enter (l_here)

  if ( rtn_i < 0 .or. rtn_i > routines_nb) &
  call die (l_here,' rtn_i = '+rtn_i+' has not an allowed value')

       write (logf,*)trim (l_here),' routine : ', trim (routines_name (rtn_i))

       if (routines_addr (rtn_i)== NOT_INIT_I) then

         call die(l_here,' address : not set')

       else
          write (logf,*)trim (l_here),' address : ',routines_addr (rtn_i)

       endif
    
       write (logf,*)trim(l_here), ' args    : ',routines_args (rtn_i)
 
  call exit (l_here)

  end subroutine one_routine_print

end module routines_mod
