! -*- f90 -*-
module secondary_mod
  use objects_mod
  use objects_paths_mod
  use drift_bywlk_mod
  use tools_mc_mod
  use walkers_bywlk_mod
  use targets_mod
  use walkers_bywlk_mod
  implicit none

  real(dp), pointer  :: atoms_coordinates_2 (:,:) => NULL ()
  real(dp), pointer  :: elec_coor_bywlk_2 (:,:,:) => NULL () 


  real(dp), pointer  :: drift_bywlk_2 (:,:,:) => NULL () 
  
  real(dp), pointer  :: e_loc_tot_bywlk_2 (:) => NULL () 
  
  integer(i4b), pointer :: drift_2_i => NULL ()
  integer(i4b), pointer :: e_loc_2_i => NULL ()


  contains

! ========================================================================
  subroutine drift_bywlk_2_bld 
! ------------------------------------------------------------------------
! Name : gap_bld
!
! 
!                  
! Authors :  R. Assaraf
! Date    : 18  June 2006
! ------------------------------------------------------------------------
  implicit none
  
! i/o:
  integer(i4b), pointer :: c2 (:) => NULL ()

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)       :: dimensions_nb
  integer(dpi)       :: electrons_nb
  integer(dpi)       :: walkers_nb
  integer(dpi)       :: dim_i, elec_i, walk_i

 
  integer(i4b) :: cloned(2) = 0
  integer(i4b) :: clones(2) = 0
  
! header:
  
  if (header_exe) then

     l_here = "drift_2_bywlk_bld"
     call object_create ('drift_2_i')
     call object_create ('e_loc_2_i')
     call object_create ('c2')
     return

  end if

! begin
  l_here = 'drift_bywlk_2_bld'
  call enter (l_here)
 
  call alloc ('drift_2_i',drift_2_i)
  call alloc ('e_loc_2_i', e_loc_2_i)
  call alloc ('c2',c2,4)

     cloned = (/ &
          object_index ("atoms_coordinates"), &
          object_index ("elec_coor_bywlk")    &
          /)
 
     clones = (/ &
          object_index ("atoms_coordinates_2"), &
          object_index ("elec_coor_bywlk_2")    &
             /)

     c2 = (/clones,cloned/)

     drift_2_i =  clone_get (object_index ("drift_bywlk"), &
          clones, cloned, "c2")

     e_loc_2_i = clone_get (object_index("e_loc_tot_bywlk"), &
          clones, cloned, "c2")
  
  call exit (l_here)
end subroutine drift_bywlk_2_bld

! ========================================================================
  subroutine elec1_elec2_dist_bywlk_bld
! ------------------------------------------------------------------------
! Name : elec1_elec2_dist_bywlk_bld
!
! 
!                  
! Authors :  R. Assaraf
! Date    :  3 Oct  2006
! ------------------------------------------------------------------------
  implicit none
  
! i/o:
  real(dp), pointer :: elec1_elec2_dist_bywlk (:,:,:) => NULL ()
  real(dp), pointer :: d_1_2_bywlk (:) => NULL ()
! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)       :: dimensions_nb
  integer(dpi)       :: electrons_nb
  integer(dpi)       :: walkers_nb
  integer(dpi)       :: dim_i, elec_i, elec_j, walk_i

  
! header:
  
  if (header_exe) then
     call object_create ('elec1_elec2_dist_bywlk')
     call object_create ('d_1_2_bywlk')

     call object_needed ('elec_coor_bywlk')
     call object_needed ('elec_coor_bywlk_2')

     return

  end if

! begin
  l_here = 'elec1_elec2_dist_bywlk_bld'
  call enter (l_here)
  call is_needed(elec_coor_bywlk)
  call is_needed(elec_coor_bywlk_2)
 
  electrons_nb = size(elec_coor_bywlk,1)
  dimensions_nb = size(elec_coor_bywlk,2)
  walkers_nb    = size(elec_coor_bywlk,3)

  call alloc ('elec1_elec2_dist_bywlk',elec1_elec2_dist_bywlk, electrons_nb, electrons_nb, walkers_nb)
 call alloc ('d_1_2_bywlk', d_1_2_bywlk, walkers_nb)
  
  elec1_elec2_dist_bywlk = 0.d0


  do elec_i = 1, electrons_nb
     do elec_j = 1, electrons_nb
        do walk_i = 1, walkers_nb
           do dim_i = 1, dimensions_nb

              elec1_elec2_dist_bywlk (elec_i, elec_j, walk_i) = &
                   elec1_elec2_dist_bywlk (elec_i, elec_j, walk_i) + &
                   (elec_coor_bywlk (elec_i, dim_i, walk_i) - elec_coor_bywlk_2 (elec_j, dim_i, walk_i))**2

           enddo ! do dim_i
        enddo ! do walk_i
     enddo ! do elec_j
  enddo ! do elec_i

  d_1_2_bywlk = 0.d0
  
  do elec_i = 1, electrons_nb
     do walk_i = 1, walkers_nb
        d_1_2_bywlk (walk_i) = d_1_2_bywlk(walk_i) + elec1_elec2_dist_bywlk (elec_i, elec_i, walk_i)
     enddo ! do walk_i
  enddo ! do elec_i
  
  call exit (l_here)
end subroutine elec1_elec2_dist_bywlk_bld


! ========================================================================
  subroutine clones_roots_bld 
! ------------------------------------------------------------------------
! Name : gap_bld
!
! 
!                  
! Authors :  R. Assaraf
! Date    : 18  June 2006
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)       :: dimensions_nb
  integer(dpi)       :: electrons_nb
  integer(dpi)       :: walkers_nb
  integer(dpi)       :: atoms_nb

! header:
  
  if (header_exe) then

     l_here = "clones_roots_bld"
 
    
     call object_create ("atoms_coordinates_2")
     call object_create ("elec_coor_bywlk_2")  
 
     return

  end if

! begin
  l_here = 'clones_roots_bld'
  call enter (l_here)
  
  call provide_bynam ("atoms_coordinates")
  call provide_bynam ("elec_coor_bywlk")
 
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)
  atoms_nb      = size (atoms_coordinates,1)

  call alloc ("atoms_coordinates_2", atoms_coordinates_2, &
       atoms_nb, dimensions_nb)

  call alloc("elec_coor_bywlk_2",elec_coor_bywlk_2, &
       electrons_nb, dimensions_nb, walkers_nb)
  
  atoms_coordinates_2 = atoms_coordinates
  atoms_coordinates_2(2,3) = atoms_coordinates_2(2,3)+0.01d0
  elec_coor_bywlk_2   = elec_coor_bywlk
    
  
  call exit (l_here)
end subroutine clones_roots_bld

end module secondary_mod
