! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!              Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module setup_mod

 use parser_mod
 use objects_mod

  implicit none

  contains

! ========================================================================
  subroutine setup_menu
! ------------------------------------------------------------------------
!
! Description    : set command
!
! Authors :X. Krokidis, F. Colonna
! Date    :10 Oct 2000
! Revision: 05 Mar 2001 F. Colonna token_is two steps
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                               :: l_inte
  logical(bool)                              :: done
  logical(bool)                              :: syntax
  character(len=max_string_len_debug_list)   :: l_string 
  character(len=max_string_len)              :: routine_name 

! begin:
  l_here ='setup_menu'
  call enter (l_here)

! initialize:
  syntax = .false.
  l_string = ' '
  routine_name = ' '

  done = .false.
  call next_command_get ('set')
  do while (.not.done)
   call next_word_get('setup')
   if     (trim(cur_word) .eq. 'help') then
  
    write (logf,*) 'setup-h: setup control parameters'
    write (logf,*) 'setup-h: syntax:'           
    write (logf,*) 'setup-h: set{up}'           
    write (logf,*) 'setup-h:   help'       
    write (logf,*) 'setup-h:   cpu             = [routine-name]'       
    write (logf,*) 'setup-h:   die{_level}     = [integer]'
    write (logf,*) 'setup-h:   ech{o}          = [boolean]'       
    write (logf,*) 'setup-h:   debug_level     = [integer] not implemented'
    write (logf,*) 'setup-h:   deb{ug}         = [all|routine-name]'       
    write (logf,*) 'setup-h:   fas{t}          = [boolean]'           
    write (logf,*) 'setup-h:   job{_name}      = [string]'
    write (logf,*) 'setup-h:   pri{nt_level}   = [integer]'
    write (logf,*) 'setup-h:   rou{tine_stack_level} = [integer]'
    write (logf,*) 'setup-h:   sho{w}          = [boolean]'
    write (logf,*) 'setup-h:   syn{tax}        = [boolean]'
    write (logf,*) 'setup-h:   tit{le}         = [string_with_blanks]'
    write (logf,*) 'setup-h:   trace_level     = [integer] '
    write (logf,*) 'setup-h:   trace_depend    = [boolean] trace depend infos'
    write (logf,*) 'setup-h:   war{ning_level} = [integer]'
    write (logf,*) 'setup-h:   end_set'
    write (logf,*) 'setup-h: example'
    write (logf,*) 'setup-h: set debug = input_words_bld end '

  else if(token_is ('cpu', routine_name)) then
    l_string = trim(cpu_routines_list)//','//blanks
    cpu_routines_list = trim(l_string)//trim(routine_name)//blanks

! Improve: the order is important
  else if(token_is ('debug_level', l_inte)) then
    debug_level = l_inte

  else if(token_is ('deb', routine_name)) then

    l_string = trim(debug_routines_list)//','//blanks
    debug_routines_list = trim(l_string)//trim(routine_name)//blanks
      write (logf,*)'debug!!!!!!!!!!!!!!!!!!!!!!!!!'

    if(len_trim(debug_routines_list) == len(debug_routines_list)) then
      write (logf,*)'setup-e: max length ',len(debug_routines_list), &
              ' reached for debug_routines_list :'
      write (logf,*)'setup-e: >',debug_routines_list,'<'
      call die ('setup',  &
     'debug list exceeds max. see max_string_len_debug_list in control_mod.f90')
    end if
! end Improve

  else if(token_is ('die', l_inte)) then
   die_level = l_inte

  else if(token_is ('ech', echo)) then

  else if(token_is ('fas', fast)) then

  else if(token_is ('job', job_name)) then

  else if(token_is ('pri', print_level)) then

  else if(token_is ('rou', routine_stack_level)) then

  else if(token_is ('sho', show)) then

    if(show) then
     write (logf,'(1x,a,l1)') 'setup: echo             =',echo
     write (logf,'(1x,a,i5)') 'setup: debug_level (not implemented) =',debug_level
     write (logf,'(1x,a,i5)') 'setup: die_level        =',die_level  
     write (logf,'(1x,a,i5)') 'setup: print_level      =',print_level
     write (logf,'(1x,a,i5)') 'setup: trace_level      =',trace_level
     write (logf,'(1x,a,i5)') 'setup: warning_level    =',warning_level
     write (logf,'(1x,3a)')   'setup: job_name            >',trim(job_name),'<'
     write (logf,'(1x,3a)')   'setup: title               >',trim(title),'<'
     write (logf,'(1x,3a)')   'setup: debug_routines_list >',trim(debug_routines_list),'<'
    end if

  else if(token_is ('syn', syntax)) then

  else if(token_is ('tit', l_string)) then
		if (len_trim(title) > 0) then
		  message = title+CR
		  title = message + l_string
    else
		  title = l_string
    end if

  else if(token_is ('trace_depend', trace_depend)) then

  else if(token_is ('trace_level', trace_level)) then

  else if(token_is ('war', warning_level)) then

  else
    call menu_exit (l_here, done)
  end if

  if(debug)write (logf,*)trim(l_here),'-d: syntax=',syntax

! if(syntax) call input_word_semans_bld
  if(syntax) call command_array_bld

  end do

  call exit (l_here)

  end subroutine setup_menu

! ========================================================================
  subroutine show_menu
! ------------------------------------------------------------------------
!
! Name        : show_menu
!
! Description : show command menu    
! 
! Author     : X. Krokidis Xenophon.KROKIDIS@ifp.fr
! Author     : F. Colonna  colonna@lct.jussieu.fr
! Date       : 01 Nov 2000
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)              :: what
  logical(bool)                              :: done
  real(dp)                                   :: r_value                                                  
! begin:
  l_here ='show_menu'
  call enter (l_here)

  done = .false.
  call next_command_get ('show')
  do while (.not.done)
   call next_word_get('show')
   if     (trim(cur_word) .eq. 'help') then
  
    write (logf,*) 'show-h: show things'
    write (logf,*) 'show-h: syntax:'           
    write (logf,*) 'show-h: show'           
    write (logf,*) 'show-h:   help'       
    write (logf,*) 'show-h:   arc{hitecture-name} = true ' 
    write (logf,*) 'show-h:   dir{_cur-name}      = true '       
    write (logf,*) 'show-h:   fil{es}             = true '       
    write (logf,*) 'show-h:   job{-name}          = true '       
    write (logf,*) 'show-h:   mac{hine-name}      = true '       
    write (logf,*) 'show-h:   obj{ect}            = [string|all]'       
    write (logf,*) 'show-h:   par{ameters}        = true '       
    write (logf,*) 'show-h:   tit{le}             = true '       
    write (logf,*) 'show-h:   use{r-name}         = true '       
    write (logf,*) 'show-h:   sem{antics}         = true '       
    write (logf,*) 'show-h:   sym{bol}            = [symbol-name|all] '       
    write (logf,*) 'show-h:   end_show'
    write (logf,*) 'show-h: example'
    write (logf,*) 'show-h: show file = true end_sho '

   else if(token_is('arc', what)) then
    arch_name = architecture_get ()
    write (logf,*)'show: arch name >',trim(arch_name),'<'

   else if(token_is('dir', what)) then
    dir_cur_name = directory_get ()
    write (logf,*) 'show: current directory >',trim(dir_cur_name),'<'

   else if(token_is('fil', what)) then
    call files_show 

   else if(token_is('job', what)) then
    write (logf,*) 'show: job name >',trim(job_name),'<'

   else if(token_is('mac', what)) then
    mach_name = machine_name_get ()
    write (logf,*) 'show: machine name >',trim(mach_name),'<'

   else if(token_is('obj', what)) then
    call objects_show (what)

   else if(token_is('par', what)) then

    write (logf,*) trim(l_here),': program parameters setup:'
    write (logf,*) trim(l_here),': echo  =',echo
    write (logf,*) trim(l_here),': job_name >',trim(job_name),'<'
    write (logf,*) trim(l_here),': title >',trim(title),'<'
    write (logf,*) trim(l_here),': debug_level=',debug_level
    write (logf,*) trim(l_here),': print_level=',print_level
    write (logf,*) trim(l_here),': trace_level=',trace_level
    write (logf,*) trim(l_here),': warning_level=',warning_level
    write (logf,*) trim(l_here),': debug_routines_list >',trim(debug_routines_list),'<'

   else if(token_is('tit', what)) then
    write (logf,*) 'show: title:'
    write (logf,'(a)') '>',trim(title),'<'

   else if(token_is('use', what)) then
    user_name = username_get ()
    write (logf,*) 'show: user name >',trim(user_name),'<'

   else if(token_is('sem', what)) then
    call input_words_semans_print

   else if(token_is('sym', what)) then
    call symbol_table_print (what)

   else if(token_is ('num', r_value)) then
    write (logf,*)trim(l_here),': number=',r_value

   else
    call menu_exit (l_here, done)
   end if

  end do

  call exit (l_here)

  end subroutine show_menu

! ======================================================================================
  subroutine objects_show (what) 
! --------------------------------------------------------------------------------------
!
! Name           : objects_show
!
! Argument (in ) : what  (name or all)
!
! Description    : shows object information 
!
! Authors:F. Colonna
! Date  :15 Dec 2002
! --------------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*) :: what   

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi) :: obj_i, obj_nb 

! begin:
  l_here ='objects_show'
  call enter (l_here)

  obj_nb = objects_nb
  if(obj_nb == 0) then
   message = 'in routine'*l_here*' no objects catalogued'
   call require (message, obj_nb /= 0)
  end if

  select case(what)

  case ("all")

   do obj_i = 1, obj_nb
     call one_object_info_write (obj_i,logf, l_here)
   end do

  case default

   obj_i = string_in_array_locate (what, objects_names)

   if(obj_i == 0) then
     message = 'in routine'*l_here*' object name >'+what+'< not catalogued'
     call require (message, obj_i /= 0)
   end if

   call one_object_info_write (obj_i, logf, l_here)

  end select

  call exit (l_here)

  end subroutine objects_show

end module setup_mod
