! -*- f90 -*-
module step_bywlk_mod

  use objects_mod  
  use objects_paths_mod
  use drift_bywlk_mod
  use tools_mc_mod
  use walkers_bywlk_mod
  use targets_mod
  use walkers_bywlk_mod
  use e_loc_tot_bywlk_mod
  use intracule_mod
  use density_bywlk_mod

  implicit none

! Description        : performs one step of stochastic dynamic 
! Description        : for all walkers
!
!
! Creates            : elec_coor_old_bywlk (elec, dim, walk)
! Creates            : drift_old_bywlk (elec, dim, walk)
! Creates            : psi_old_bywlk (walk)
 
  character(len=max_string_len_run_type) :: runtype   = "?" !NOT_INIT_S    


  real(dp), pointer :: elec_coor_old_bywlk (:,:,:) => NULL ()
  real(dp), pointer :: drift_old_bywlk (:,:,:)     => NULL ()
  real(dp), pointer :: psi_old_bywlk (:)           => NULL ()
  real(dp), pointer :: e_loc_tot_old_bywlk (:)      => NULL ()
 
  
! Creates            : prob_old_to_cur_bywlk (walk)
! Creates            : prob_cur_to_old_bywlk (walk)
! Creates            : prob_accept_bywlk (walk)
! Creates            : weights_bywlk (walk)

  real(dp), pointer :: weights_bywlk (:)                => NULL ()
  real(dp), pointer :: prob_old_to_cur_bywlk (:)        => NULL ()
  real(dp), pointer :: prob_cur_to_old_bywlk (:)        => NULL ()
  real(dp), pointer :: prob_accept_bywlk (:)            => NULL ()

! Creates            : elec_coor_random_bywlk (elec, dim, walk)

  real(dp), pointer :: elec_coor_temp_bywlk (:,:,:) => NULL ()
  real(dp), pointer :: drift_temp_bywlk (:,:,:)     => NULL ()
  real(dp), pointer :: psi_temp_bywlk (:)           => NULL ()

  real(dp), pointer :: elec_coor_random_bywlk (:,:,:) => NULL ()

! Needs              : elec_coor_bywlk (elec, dim, walk)  
! Needs              : drift_bywlk (elec, dim, walk)  
! Needs              : psi_bywlk (walk)  
 
  logical(bool)     :: constant  ! no move all steps identical


  integer(dpi) :: blocks_nb
  integer(dpi) :: steps_per_block_nb
  real(dp), pointer   :: time_step => NULL ()
  integer(dpi) :: block_cur, step_cur
  integer(dpi), pointer :: obj_print_per_block (:) => NULL ()
  integer(dpi), pointer, save  :: walk_reconf_ind (:) => NULL ()
  integer(dpi) :: acc_phase 
  integer(dpi), parameter :: acceptation_count_limit = 100
  logical(bool) :: reconf_last_block = .false.
  contains

! ========================================================================
  subroutine sample_bld  
! ------------------------------------------------------------------------
!
! Name : sample_bld
!
! Needed : targets_ind
!
! Description  : QMC iterations 
! Descripttion : each step produces objects defined by targets_ind
!
! Authors : R. Assaraf
! Date    : 12 Feb 2002
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) ::  i, step_i

  if (header_exe) then

     l_here = "sample_bld"

     call object_needed ('targets_ind')
     
     return

  end if

! begin:
  call enter (l_here)
  
  if (time_step <=0) call die (l_here, 'time_step='*time_step*&
       'must be defined in the menu montecarlo')

  message ='in routine'*l_here*' number of steps per blocks is undefined'
  call ensure (message, steps_per_block_nb /= NOT_INIT_I)

  message ='in routine'*l_here*' number of blocks is undefined'
  call ensure (message, blocks_nb /= NOT_INIT_I)

  runtype = string_to_upper (runtype)

! Improve: implement guiding wave function with only one version of step_bywlk_mod.f90
   if(use_guide) then
   call die(l_here,'If guiding wave function used, this is not the correct version of step_bywlk_mod.f90')
  endif


!  loop over iterations
  step_cur = 0
  do block_cur = 1, blocks_nb
     
     do step_i = 1, steps_per_block_nb
        step_cur = step_cur + 1
! at each iteration the random numbers of the ecp should be different
! improve: put that in the routine step_bywlk_bld
        if (ecp_method == "randomize") call elec_ecp_random_rot_bywlk_bld


            call node_exe_byrtn_name ('one_step_bywlk_bld')
! targets_ind: objects defined in the menu out 
        do i=1,size(targets_ind)
           call provide_byind (targets_ind(i))
        enddo
 
! JT: Calculation of intracule
! Improve: do better than this do_intracule flag
        if(do_intracule) then
           call intracule_step
        endif

     enddo ! do step_cur
 
! print values  after one block 
     if (block_cur == 1) then
        write (res,*)
        write (res,*) '--------------- Values after each block  ---------'
        write (res,*) 
        write (res,*) 'After each block of QMC iterations, values are printed'
        write (res,*) 'and walkers are copied in  the file walkers-out'
        write (res,*)
     endif
     write (logf,*)trim (l_here),' block # ',block_cur     
     write (res,*) 
   
     if (block_cur == blocks_nb) then
        write (res,'(2a)') blanks(1:5), '------------------------------'
        write (res,'(2a)') blanks(1:5), '|  Final results: last block |'
        write (res,'(2a)') blanks(1:5), '------------------------------' 
     endif
     
     write (res,'(a,i5,a)') 'block # ',block_cur,':'
     write (res,'(a)')    '----------------'    
  
     do i = 1,size (targets_ind)
        call object_value_byind_write (targets_ind (i), res)
        call object_value_byind_write (targets_ind (i), logf)
     enddo ! do i
! improve
     call walkers_bywlk_write
 
! JT: Calculation of intracule
! Improve: do better than this do_intracule flag
      if(do_intracule) then
        call intracule_block
      endif

! flush buffers
      call flush(logf)
      call flush(res)

  enddo ! do block_cur 

  call exit (l_here)
  
end subroutine sample_bld


! ========================================================================
  subroutine one_step_bywlk_bld 
! ------------------------------------------------------------------------
! Name : one_step_bywlk_bld
!
! 

!                  
! Authors :  R. Assaraf
! Date    : 18  June 2006
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)       :: dimensions_nb
  integer(dpi)       :: electrons_nb
  integer(dpi)       :: walkers_nb
  integer(dpi)       :: dim_i, elec_i, i
  integer(dpi)       :: walk_i
  real(dp)           :: dot_prod
  integer(dpi), pointer, save :: path_bynod (:) => NULL ()


  logical(bool)               :: already_reconf_var = .false.
  integer(dpi), pointer, save :: acceptation_count_bywlk (:) => NULL ()
  real(dp), pointer, save     :: acceptation_bywlk (:)       => NULL ()

 
  real(dp), pointer  :: global_weight => NULL ()
  real(dp) :: eta
 
  real(dp), pointer, save :: e_loc_tot_temp_bywlk (:) => NULL ()
  real(dp), pointer, save :: eta_bywlk (:) => NULL ()  

! header:
  
  if (header_exe) then

  l_here = "one_step_bywlk_bld"

 
  call object_create ('elec_coor_random_bywlk') 

  call object_create ('elec_coor_old_bywlk')
  call object_create ('drift_old_bywlk')
  call object_create ('psi_old_bywlk')
  call object_create ('e_loc_tot_old_bywlk')


  call object_create ('prob_old_to_cur_bywlk')
  call object_create ('prob_cur_to_old_bywlk')
  call object_create ('prob_accept_bywlk')

  call object_create ('weights_bywlk')
  call object_create ('walk_reconf_ind')
  call object_create ('global_weight')
  call object_create ('eta_bywlk') 
  call object_create ('acceptation_count_bywlk')
  
  call object_needed ('elec_coor_bywlk')
  call object_needed ('drift_bywlk')
  call object_needed ('psi_bywlk')
  call object_needed ('e_loc_tot_bywlk')
 return

  end if

! begin
  l_here = 'one_step_bywlk_bld'
  call enter (l_here)
 
  electrons_nb  = size (drift_bywlk, 1)
  dimensions_nb = size (drift_bywlk, 2)
  walkers_nb    = size (drift_bywlk, 3)

! dimensions
 call alloc ('elec_coor_random_bywlk', elec_coor_random_bywlk, electrons_nb, dimensions_nb, walkers_nb)

 call alloc ('elec_coor_old_bywlk', elec_coor_old_bywlk, electrons_nb, dimensions_nb, walkers_nb)

 call alloc ('drift_old_bywlk', drift_old_bywlk, electrons_nb, dimensions_nb, walkers_nb)

 call alloc ('psi_old_bywlk', psi_old_bywlk, walkers_nb)

 call alloc ('e_loc_tot_old_bywlk', e_loc_tot_old_bywlk, walkers_nb)

 elec_coor_old_bywlk = elec_coor_bywlk

 drift_old_bywlk     = drift_bywlk
 
 psi_old_bywlk       = psi_bywlk

 e_loc_tot_old_bywlk = e_loc_tot_bywlk
 
! build new coordinates set, Langevin step 

 do walk_i = 1, walkers_nb

    do dim_i= 1,dimensions_nb
       
       do elec_i = 1,electrons_nb
          
          elec_coor_random_bywlk (elec_i, dim_i, walk_i)  = gauss ()
          
       enddo ! do 
       
    enddo ! do 
    
    elec_coor_bywlk (:,:,walk_i) = elec_coor_old_bywlk (:,:, walk_i) &
         + drift_old_bywlk (:,:, walk_i)*time_step                   &
         + sqrt(time_step)*elec_coor_random_bywlk (:,:,walk_i)
    
    
 enddo ! do walk_i

 if (.not.associated (path_bynod))  then
    call path_bynod_bld (path_bynod, (/ object_index('elec_coor_bywlk') /), (/ (i, i=1, objects_nb) /))
 endif
 is_valid_bynod (path_bynod) = .false.

 call provide_bynam("drift_bywlk")
 call provide_bynam("psi_bywlk")
 call provide_bynam("e_loc_tot_bywlk")

!!!!!!!!!!!!!!!!!!!!!!!!!! 
! prob_accept_bywlk  
!!!!!!!!!!!!!!!!!!!!!!!!1
 call alloc ('prob_old_to_cur_bywlk', prob_old_to_cur_bywlk, walkers_nb)
  
 call alloc ('prob_cur_to_old_bywlk', prob_cur_to_old_bywlk, walkers_nb)
  
 call alloc ('prob_accept_bywlk',prob_accept_bywlk, walkers_nb)
   
  if(.not.associated(elec_coor_temp_bywlk)) then
     write (logf,*)trim(l_here),': elec_coor_temp_bywlk not yet associated'
     call allocation ('elec_coor_temp_bywlk', elec_coor_temp_bywlk,  electrons_nb, dimensions_nb, walkers_nb)
  else
     if (debug) write (logf,*)trim(l_here),&
          '-d: elec_coor_temp_bywlk left in memory'
  end if
  


! prob_old_to_cur
  elec_coor_temp_bywlk = elec_coor_bywlk - elec_coor_old_bywlk &
       - drift_old_bywlk*time_step

  include 'debug_elec_coor_temp_bywlk.inc'

  do walk_i = 1, walkers_nb

     dot_prod = zero

     do dim_i = 1, dimensions_nb
        do elec_i = 1, electrons_nb
           
           dot_prod = dot_prod                                 &
                + elec_coor_temp_bywlk (elec_i, dim_i, walk_i) &
                * elec_coor_temp_bywlk (elec_i, dim_i, walk_i)
  
        end do  ! elec_i
     end do  ! dim_i

     prob_old_to_cur_bywlk (walk_i) = exp (-dot_prod /(Two*time_step) )
     
  end do  ! walk_i

  
! prob_cur_to_old

  elec_coor_temp_bywlk = elec_coor_old_bywlk - elec_coor_bywlk &
       - drift_bywlk * time_step

  do walk_i = 1, walkers_nb

     dot_prod = zero
     
     do dim_i = 1, dimensions_nb
        do elec_i = 1, electrons_nb
           
           dot_prod = dot_prod                                 &
                + elec_coor_temp_bywlk (elec_i, dim_i, walk_i) &
                * elec_coor_temp_bywlk (elec_i, dim_i, walk_i)
  
        end do  ! elec_i
     end do  ! dim_i
 
     prob_cur_to_old_bywlk (walk_i) = exp (-dot_prod /(Two*time_step) ) 
  
  end do  ! walk_i
 

  do walk_i = 1, walkers_nb
     
     prob_accept_bywlk (walk_i) = min ( 1.d0,                          &
          (psi_bywlk (walk_i)* prob_cur_to_old_bywlk (walk_i) )        &
          / (psi_old_bywlk (walk_i) * prob_old_to_cur_bywlk (walk_i) ) &
          *psi_bywlk (walk_i)/psi_old_bywlk (walk_i))
     
  end do ! walk_i
 

! VMC or DMC step  

  if (.not.associated (acceptation_bywlk))  allocate (acceptation_bywlk (walkers_nb))

  call alloc ('weights_bywlk',weights_bywlk,walkers_nb)

  call alloc ('walk_reconf_ind',walk_reconf_ind,walkers_nb)

  call alloc ('global_weight',global_weight)

  call alloc ('eta_bywlk',eta_bywlk, walkers_nb)

  if (.not.associated (acceptation_count_bywlk)) then 
     call alloc ('acceptation_count_bywlk',acceptation_count_bywlk,walkers_nb)
     acceptation_count_bywlk = 0_dp
  endif
  
  do walk_i = 1, walkers_nb
     eta_bywlk (walk_i) = ranfmc ()
  enddo ! do walk_i

  weights_bywlk = 1_dp
   
  do walk_i = 1, walkers_nb

     if (runtype == "DMC") weights_bywlk (walk_i) =  exp (-time_step * e_loc_tot_old_bywlk (walk_i))
     
! old is not equal to new, new is accepted or rejected 
     eta =  eta_bywlk (walk_i)

     if ( eta >=  prob_accept_bywlk (walk_i) ) then
! rejection  cur = old
        acceptation_bywlk (walk_i) = 0 
        elec_coor_bywlk (:, :, walk_i) = elec_coor_old_bywlk (:, :, walk_i)
        psi_bywlk (walk_i)             = psi_old_bywlk (walk_i) 
        drift_bywlk (:, :, walk_i)     = drift_old_bywlk (:, :, walk_i)
        e_loc_tot_bywlk (walk_i)       = e_loc_tot_old_bywlk (walk_i)
           
     else
! acceptation 
        acceptation_bywlk (walk_i) = 1
        acceptation_count_bywlk (walk_i)=acceptation_count_bywlk (walk_i)+1
     endif
  enddo ! do walk_i
     
  if (runtype == "DMC") eta = ranfmc ()
  if (runtype == "VMC") eta = 0.5_dp


  call  reconfgen(weights_bywlk,weights_bywlk,eta,walk_reconf_ind,global_weight)

  if (.not.associated(elec_coor_temp_bywlk)) & 
       allocate(elec_coor_temp_bywlk( electrons_nb, dimensions_nb, walkers_nb))

  if (.not.associated(drift_temp_bywlk)) & 
       allocate(drift_temp_bywlk( electrons_nb, dimensions_nb, walkers_nb))
  
  if (.not.associated(psi_temp_bywlk)) &
       allocate (psi_temp_bywlk(walkers_nb))

  if (.not.associated(e_loc_tot_temp_bywlk)) &
       allocate (e_loc_tot_temp_bywlk (walkers_nb))

  do walk_i= 1,walkers_nb
     elec_coor_temp_bywlk (:,:,walk_i)=elec_coor_bywlk (:,:,walk_reconf_ind(walk_i))
     psi_temp_bywlk (walk_i) = psi_bywlk (walk_reconf_ind (walk_i))
     
     drift_temp_bywlk (:,:,walk_i) = drift_bywlk (:,:,walk_reconf_ind (walk_i))

     e_loc_tot_temp_bywlk (walk_i) = e_loc_tot_bywlk (walk_reconf_ind(walk_i))
  enddo ! do walk_i
  
  elec_coor_bywlk  = elec_coor_temp_bywlk 
  psi_bywlk        = psi_temp_bywlk 
  drift_bywlk      = drift_temp_bywlk 
  e_loc_tot_bywlk  = e_loc_tot_temp_bywlk


  if (.not.associated (density_bywlk)) &
       call alloc ('density_bywlk',density_bywlk,walkers_nb)
  
  do walk_i = 1, walkers_nb
     density_bywlk (walk_i) = psi_bywlk (walk_i)*psi_bywlk(walk_i)
  enddo ! do walk_i
  
  if (step_cur == steps_per_block_nb * blocks_nb.and.reconf_last_block) then
     if (.not.already_reconf_var) & 
          call reconf_var (acceptation_count_bywlk)
     already_reconf_var = .true.
  endif
   
  is_valid_bynod (path_bynod) = .false.

  is_valid_bynod (objects_node(object_index('psi_bywlk'))) = .true.
  is_valid_bynod (objects_node(object_index('drift_bywlk'))) = .true.
  is_valid_bynod (objects_node(object_index('e_loc_tot_bywlk')))= .true.

  call exit (l_here)
  
end subroutine one_step_bywlk_bld

! ========================================================================
  subroutine test_eta_bld
! ------------------------------------------------------------------------
!
! Name           : eta_bld
!
! Description    : 
!
! Authors: R. Assaraf
! Date  : 15 Jul 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp), pointer, save :: test_eta => NULL ()
  real(dp) :: test_eta_num, test_eta_denom
  integer(dpi) :: walk_i
! header:
  l_here= 'test_eta_bld' 
  if (header_exe) then
     call object_create ('test_eta')
     call object_needed('density_bywlk')
     call object_needed('psi_bywlk')
     return     
  endif

!begin:
  call enter (l_here)
  call alloc('test_eta',test_eta)

  test_eta_num = 0_dp
  test_eta_denom = 0_dp

  do walk_i = 1, size (density_bywlk)

   test_eta_num = test_eta_num+psi_bywlk (walk_i)**2/density_bywlk (walk_i)

   test_eta_denom = test_eta_denom+ (psi_bywlk (walk_i)**2/density_bywlk (walk_i))**2

  enddo
  test_eta = (test_eta_num)**2/test_eta_denom/size(density_bywlk)

  call exit (l_here)

  end subroutine test_eta_bld


! =====================================================================
  subroutine reconf_var (acceptation_count_bywlk)
! ---------------------------------------------------------------------
!
! Name           : reconf_var
! Description    : remove walkers for which acceptation rate is too low
! Description    : and replace them 
! 
! Author: R. Assaraf, X. Krokidis
! Date  : 10 Mar 2001
! ---------------------------------------------------------------------
  implicit none

! i/o :
  integer (dpi) :: acceptation_count_bywlk (:)

! local:
  integer(dpi)   ::  walk_i,walkers_nb,walk_ranf
  logical(bool)  :: ok
  integer (dpi)  :: cur_acc,max_acc
  real(dp), save :: accept_ratio_max = 0.7_dp 
  character(len=max_string_len_routine_name) :: l_here

! begin:

  call header_die

  l_here = 'reconf_var'
  call enter (l_here)
 
  walkers_nb = size (acceptation_count_bywlk)
  max_acc = maxval(acceptation_count_bywlk)

  do walk_i = 1,walkers_nb 
     cur_acc = acceptation_count_bywlk (walk_i)
     if (cur_acc < accept_ratio_max*max_acc) then
        ok = .false.
         do while(.not.ok)
           walk_ranf = int(ranfmc()*walkers_nb+1)
           cur_acc = acceptation_count_bywlk (walk_ranf)
           if (cur_acc >= accept_ratio_max*max_acc) then
             ok = .true.
             elec_coor_bywlk (:,:,walk_i) = elec_coor_bywlk (:,:,walk_ranf)
             elec_coor_old_bywlk (:,:,walk_i) = elec_coor_old_bywlk (:,:,walk_ranf)
             drift_bywlk (:,:,walk_i) = drift_bywlk (:,:,walk_ranf)
             drift_old_bywlk (:,:,walk_i) = drift_old_bywlk (:,:,walk_ranf)

             write (logf,'(2a,i5,a,i5)') &
             trim(l_here),': walker # ',walk_ranf,&
                          ' has been copied on walker # ',walk_i
           endif
         enddo
    endif
  enddo ! do walk_i

  call exit (l_here)
  end subroutine reconf_var

! ========================================================================
  subroutine real_1_byblk_bld
! ------------------------------------------------------------------------
! Name :  real_1_byblk_bld
!
! Description: actualize rand_var_byblk from rand_var_bywlk and the current block
! Description: the current realization of rand_var_bywlk and the current block block_cur
!
! Authors: R. Assaraf
! Date   : 01 Aug 2002
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
   character(len=max_string_len_routine_name),save :: l_here
   real(dp), pointer     :: rand_var_byblk (:)
   real(dp), pointer     :: rand_var_bywlk (:)
   integer(dpi)  :: walkers_nb 
   integer(dpi)  :: walk_i   
   l_here ='real_1_byblk_bld'
   if (header_exe) then

    l_here = "real_1_byblk_bld"

    return

  end if

! begin:
  call enter (l_here,1)

   call alias_byaddr_bld  (rand_var_byblk, objects_addr (cur_create_ind (1)))
   call alias_byaddr_bld (rand_var_bywlk, objects_addr (cur_needed_ind (1)))
  
   call alloc (objects_names (cur_create_ind (1)), rand_var_byblk, blocks_nb )
   
   if (block_cur <= 1.and. step_cur <= 1)  rand_var_byblk = NOT_INIT_DP
   
   if (block_cur>0) then

      walkers_nb = size (rand_var_bywlk)  

      if ( abs (rand_var_byblk (block_cur)- NOT_INIT_DP) < abs (spacing(NOT_INIT_DP))) rand_var_byblk (block_cur) = 0_dp
      
      do walk_i = 1, walkers_nb
         
         rand_var_byblk (block_cur) = rand_var_byblk (block_cur) + &
                              rand_var_bywlk (walk_i)/(walkers_nb*steps_per_block_nb)
             
      enddo
     endif

  call exit (l_here, 1)
  
  end subroutine real_1_byblk_bld

! ========================================================================
  subroutine history_bld
! ------------------------------------------------------------------------
! Name :  history_bld
!
! Description: 
! Description: history
!
! Authors: R. Assaraf
! Date   : 25 Nov 2005
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp), pointer     :: rand_var_bywlk (:)
  real(dp), pointer     :: history_bywlk  (:,:)
 
  integer(dpi), pointer :: history_nb => NULL ()
  integer(dpi)  :: walkers_nb 
  integer(dpi)  :: step_cur_old = 0
  integer(dpi)  :: walk_i
  integer(dpi)  :: index, walk_ind
  l_here ='history_bld'
  if (header_exe) then

     l_here = "history_bld"

     return

  end if

! begin:
  call enter (l_here,1)
  
  call alias_byaddr_bld (rand_var_bywlk, objects_addr (cur_needed_ind (1)))
  call alias_byaddr_bld (history_nb, objects_addr (cur_needed_ind (2)))
  call alias_byaddr_bld (history_bywlk,  objects_addr (cur_create_ind (1)))
  
  walkers_nb = size (rand_var_bywlk)

  if (.not.associated (history_bywlk)) then
     call alloc (objects_names (cur_create_ind (1)), history_bywlk, history_nb, walkers_nb )
     history_bywlk = 0.d0
  endif
   
  if (step_cur > step_cur_old+1) call die (l_here, 'step_cur > step_cur_old') 
   
  do walk_i = 1, walkers_nb
     walk_ind = walk_reconf_ind (walk_i)
     if (walk_ind /= walk_i) then
        if (walk_ind > walkers_nb.or.walk_ind < 1) &
             call die (l_here,"walk_ind="+walk_ind+"out of range")
        history_bywlk (:,walk_i) = history_bywlk (:, walk_ind)
     endif
  enddo ! do walk_i

  index = mod (step_cur-1, history_nb)+1
  
  history_bywlk (index, : ) = rand_var_bywlk   
   
  step_cur_old = step_cur
  call exit (l_here, 1)
  
end subroutine history_bld

! ========================================================================
  subroutine time_int_calc
! ------------------------------------------------------------------------
! Name :  time_int_calc
!
! Description: 
! Description: 
!
! Authors: R. Assaraf
! Date   : 25 Nov 2005
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  real(dp), pointer     :: history_bywlk  (:,:)
  real(dp), pointer     :: time_int (:,:)
  integer(dpi)          :: step_cur_old = 0 
  integer(dpi)  :: walkers_nb 
  integer(dpi)  :: index_cur, index_last, i, history_nb
  
  l_here ='time_int_calc'
  if (header_exe) then

     l_here = "time_int_calc"

     return

  end if

! begin:
  call enter (l_here,1)


  call alias_byaddr_bld (history_bywlk,  objects_addr (cur_needed_ind (1)))
  call alias_byaddr_bld (time_int, objects_addr (cur_create_ind (1)))  

  walkers_nb = size (history_bywlk, dim = 2)
  history_nb = size (history_bywlk, dim = 1)
 
   
  if (.not.associated (time_int)) then
     call alloc (objects_names (cur_create_ind (1)), time_int, history_nb, walkers_nb )
     time_int = 0.d0
  endif

  if (step_cur > step_cur_old+1) call die (l_here, 'step_cur > step_cur_old')    
  index_cur  = mod (step_cur-1, history_nb)+1

  time_int (1, :) = history_bywlk (index_cur,:)

  do i = 2, history_nb
 
     index_last = mod (step_cur-i+history_nb, history_nb)+1

     time_int (i, :) = time_int (i-1, :) + history_bywlk (index_last,:)

  enddo ! do i

  step_cur_old = step_cur
  call exit (l_here, 1)
  
end subroutine time_int_calc


! ========================================================================
  subroutine time_step_bld
! ------------------------------------------------------------------------
! Name :  time_step_calc
!
! Description: 
! Description: 
!
! Authors: R. Assaraf
! Date   : 25 Nov 2005
! ------------------------------------------------------------------------
  implicit none
  
! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here
  
  l_here ='time_step_bld'
  if (header_exe) then

     l_here = "time_step_bld"
     call object_create ('time_step')
     return

  end if

! begin:
  call enter (l_here,1)

  call alloc ("time_step", time_step)
  time_step  = 0.d0
  call exit (l_here, 1)
  
end subroutine time_step_bld


end module step_bywlk_mod
