! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module strings_mod

 use control_mod

  implicit none

! Revision: 11 Feb 2001 F. Colonna (+) and (*) operators
! Revision: 13 Fev 2001: stemp, string => local
!
! string_to_words
  character(len=max_string_len), pointer :: string_words(:) => NULL()
  integer(dpi), pointer                  :: string_words_length (:) => NULL ()
  integer(dpi)                           :: n_string_words = 0

  interface operator(*)
    module procedure  &
      string_blank_boolean_cat, &
      string_blank_double_cat, &
      string_blank_integer_cat, &
      string_blank_real_cat, &
      string_blank_string_cat
  end interface

  interface operator(+)
    module procedure                          &
      string_integer_cat, integer_string_cat, &
      string_boolean_cat, bool_str_cat,       &
      string_double_cat, double_string_cat,   &
      string_real_cat, real_string_cat,       &
      string_string_cat
  end interface

  interface decode
   module procedure          &
     string_to_boolean_tine, &
     string_to_double_tine,  &
     string_to_integer_tine, &
     string_to_real_tine     
   end interface

  interface encode
   module procedure    &
    boolean_to_string, &
    double_to_string,  &
    integer_to_string, &
    real_to_string       
  end interface 

  contains

! ==================================================================================
  function string_blank_string_cat (f_string, l_string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_blank_string_cat
!
! Argument (in ) : f_string
! Argument (in ) : l_string
!
! Description    : concatenates (trim(string) + ' ' + string)
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: f_string
  character(len=*), intent(in)  :: l_string

! result:
  character(len=max_string_len) :: result

! local:
  character(len=max_string_len) :: l_str
! character(len=max_string_len) :: l_here

! begin:
! l_here = 'string_blank_string_cat'
! call enter (l_here, 3)

  l_str = blanks
  l_str = trim(f_string)//' '//l_string
  result = l_str

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

!  call exit (l_here, 3)

  end function string_blank_string_cat

! ==================================================================================
  function string_blank_boolean_cat (f_string, bol) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_blank_boolean_cat
!
! Argument (in ) : f_string
! Argument (in ) : bol
!
! Description    : concatenates (trim(string)) + ' ' + encode(bol))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  logical(bool), intent(in)     :: bol

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_blank_boolean_cat'
!  call enter (l_here, 3)

  l_str = blanks
  l_string = boolean_to_string (bol)
  l_str = trim(f_string)//' '//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_blank_boolean_cat

! ==================================================================================
  function string_blank_real_cat (f_string, real) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_blank_real_cat
!
! Argument (in ) : f_string
! Argument (in ) : real
!
! Description    : concatenates (trim(string)) + ' ' + encode(real))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  real(sp), intent(in)          :: real

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_blank_real_cat'
!  call enter (l_here, 3)

  l_str    = blanks
  l_string = real_to_string (real)
  l_str = trim(f_string)//' '//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_blank_real_cat

! ==================================================================================
  function string_blank_double_cat (f_string, dble) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_blank_double_cat
!
! Argument (in ) : f_string
! Argument (in ) : dble
!
! Description    : concatenates (trim(string)) + ' ' + encode(dble))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  real(dp), intent(in)          :: dble

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_blank_double_cat'
!  call enter (l_here, 3)

  l_str    = blanks
  l_string = double_to_string (dble)
  l_str    = trim(f_string)//' '//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_blank_double_cat

! ==================================================================================
  function string_blank_integer_cat (f_string, inte) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_blank_integer_cat
!
! Argument (in ) : f_string
! Argument (in ) : inte
!
! Description    : concatenates (trim(string)) + ' ' + encode(integer))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  integer(dpi), intent(in)      :: inte

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_blank_integer_cat'
!  call enter (l_here, 3)

  l_str = blanks
  l_string = integer_to_string (inte)
  l_str = trim(f_string)//' '//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_blank_integer_cat

! ==================================================================================
  function bool_str_cat (bol, l_string)    result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : bool_str_cat
!
! Argument (in ) : bol
! Argument (in ) : l_string
!
! Description    : concatenates (encode(bol))  + trim(string))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: l_string
  logical(bool), intent(in)     :: bol

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: f_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'bool_str_cat'
!  call enter (l_here, 3)

  l_str = blanks
  f_string = boolean_to_string (bol)
  l_str = trim(adjustl(f_string))//l_string
! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function bool_str_cat

! ==================================================================================
  function string_boolean_cat (f_string, bol) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_boolean_cat
!
! Argument (in ) : f_string
! Argument (in ) : bol
!
! Description    : concatenates (trim(string)) + encode(bol))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  logical(bool), intent(in)     :: bol

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_boolean_cat'
!  call enter (l_here, 3)

  l_str = blanks
  l_string = boolean_to_string (bol)
  l_str = trim(f_string)//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_boolean_cat

! ==================================================================================
  function real_string_cat (real, l_string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : real_string_cat
!
! Argument (in ) : real
! Argument (in ) : l_string
!
! Description    : concatenates (encode(real))  + trim(string))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: l_string
  real(sp), intent(in)          :: real

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: f_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'real_string_cat'
!  call enter (l_here, 3)

  l_str = blanks
  f_string = real_to_string (real)
  l_str = trim(adjustl(f_string))//l_string
! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function real_string_cat

! ==================================================================================
  function string_real_cat (f_string, real) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_real_cat
!
! Argument (in ) : f_string
! Argument (in ) : real
!
! Description    : concatenates (trim(string)) + encode(real))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  real(sp), intent(in)          :: real

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_real_cat'
!  call enter (l_here, 3)

  l_str = blanks
  l_string = real_to_string (real)
  l_str = trim(f_string)//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_real_cat

! ==================================================================================
  function double_string_cat (dble, l_string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : double_string_cat
!
! Argument (in ) : dble
! Argument (in ) : l_string
!
! Description    : concatenates (encode(dble)) + trim(string))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: l_string
  real(dp), intent(in)          :: dble

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: f_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'double_string_cat'
!  call enter (l_here, 3)

  l_str = blanks
  f_string = double_to_string (dble)
  l_str = trim(adjustl(f_string))//l_string

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function double_string_cat

! ==================================================================================
  function string_double_cat (f_string, dble) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_double_cat
!
! Argument (in ) : f_string
! Argument (in ) : dble
!
! Description    : concatenates (trim(string)) + encode(dble))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  real(dp), intent(in)          :: dble

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'string_double_cat'
!  call enter (l_here, 3)

  l_str = blanks
  l_string = double_to_string (dble)
  l_str = trim(f_string)//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

!  call exit (l_here, 3)

  end function string_double_cat

! ==================================================================================
  function string_string_cat (f_string, l_string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_string_cat
!
! Argument (in ) : f_string
! Argument (in ) : l_string
!
! Description    : concatenates (trim(string) + string)
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: f_string
  character(len=*), intent(in)  :: l_string

! result:
  character(len=max_string_len) :: result

! local:
! character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len) :: l_str 

! begin:
! l_here = 'string_string_cat'
!  call enter (l_here, 3)

  l_str = blanks
  l_str = trim(f_string)//l_string
  result = l_str

! write (logf,*)trim(l_here),'-d: f_string >',trim(f_string),'<'
! write (logf,*)trim(l_here),'-d: l_string >',trim(l_string),'<'
! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

!  call exit (l_here, 3)

  end function string_string_cat

! ==================================================================================
  function integer_string_cat (inte, l_string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : integer_string_cat
!
! Argument (in ) : inte
! Argument (in ) : l_string
!
! Description    : concatenates (encode(inte)) + trim(string))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: l_string
  integer(dpi), intent(in)      :: inte

! local:
! character(len=max_string_len) :: l_here
  character(len=max_string_len) :: f_string
  character(len=max_string_len) :: l_str

! begin:
! l_here = 'integer_string_cat'
!  call enter (l_here, 3)

  l_str = blanks
  f_string = integer_to_string (inte)
  l_str = trim(adjustl(f_string))//l_string
  result = l_str
! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

!  call exit (l_here, 3)

  end function integer_string_cat

! ==================================================================================
  function string_integer_cat (f_string, inte) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_integer_cat
!
! Argument (in ) : f_string
! Argument (in ) : inte
!
! Description    : concatenates (trim(string)) + encode(integer))
!
! Authors: R. Assaraf
! Date  : 5 Fev 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len) :: result

! i/o:
  character(len=*), intent(in)  :: f_string
  integer(dpi), intent(in)      :: inte

! local:
  character(len=max_string_len) :: l_here
  character(len=max_string_len) :: l_string
  character(len=max_string_len) :: l_str

! begin:
  l_here = 'string_integer_cat'
  call enter (l_here, 3)

  l_str = blanks
  l_string = integer_to_string (inte)
  l_str = trim(f_string)//adjustl(l_string)

! write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  result = l_str

  call exit (l_here, 3)

  end function string_integer_cat

! ==================================================================================
  function string_null_char_at (string) result (result) 
! ----------------------------------------------------------------------------------
!
! Name       : function string_null_char_at (string)
!
! Arguments  : string
!
! Description    : returns the position of the first null character in string
! Description    : returns zero if no null
! Result     : result position of first null character in string
!
! Authors:F.Colonna
! Date  :21 Nov 2000
! Revision: 14 Feb 2001 F. Colonna pb with  call enter do not move comments
! ----------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                 :: result

! i/o:
  character(len=*), intent(in) :: string

! local:
! character(len=max_string_len_routine_name) :: l_here
  character(len=len(string))   :: l_str
  integer(dpi)                 :: position

! begin:
! l_here = "string_null_char_at"
! call enter (l_here, 6)

  if(len (string) .eq. 0) then
   position = 0
  else
   position = index (string, NULL_C)
  end if

  if (position .gt. 0 .and.  (warning_level > 0 .or. debug)) then
   write (logf,*)trim(routines_21),'-d: string >',string,'<'
   l_str = repeat(' ',position-1)
   l_str(position:position)='|'
   write (logf,*)trim(routines_21),'-d: null    ',trim(l_str)
   write (logf,*)trim(routines_21),'-d: position =',position
  end if

  result = position

! call exit (l_here, 6)

  end function string_null_char_at

! ==================================================================================
  function alphabetic_is (string) 
! ----------------------------------------------------------------------------------
!
! Name           : alphabetic_is
!
! Argument (in ) : string
!
! Description    : returns .true. if string contains only 
! Description    :   "abcdefghijklmnopqrstuvwxyz"
! Description    : (upper or lower case)
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                 :: alphabetic_is
  
! i/o
  character(len=*), intent (in) :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=len(string))    :: lowstr
  character(len=26)             :: set ="abcdefghijklmnopqrstuvwxyz"

! begin:
  l_here ='alphabetic_is'
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  lowstr = string_to_lower (trim(string))
  alphabetic_is = verify (trim(lowstr), set) .eq. 0

  if(debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(lowstr),'<'
   write (logf,*)trim(l_here),'-d: =',alphabetic_is        
  end if

  call exit (l_here,5)

  end function alphabetic_is 

! ==================================================================================
  function alphabetic_extended_is (string) 
! ----------------------------------------------------------------------------------
!
! Name           : alphabetic_extended_is
!
! Argument (in ) : string
!
! Description    : returns .true. if string contains only 
! Description    :    "_abcdefghijklmnopqrstuvwxyz"
! Description    : extended alphabetic (upper or lower case)
!
! Authors:F.Colonna
! Date  :08 Oct 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                  :: alphabetic_extended_is
  
! i/o
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here

  character(len=len(string))     :: lowstr
  character(len=27)              :: set ="_abcdefghijklmnopqrstuvwxyz"

! begin:
  l_here ='alphabetic_extended_is'
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  lowstr = string_to_lower (trim(string))
  alphabetic_extended_is = verify (trim(lowstr), set) .eq. 0

  if (debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(lowstr),'<'
   write (logf,*)trim(l_here),'-d: =',alphabetic_extended_is        
  end if

  call exit (l_here,5)

  end function alphabetic_extended_is 

! ==================================================================================
  function alphanumeric_is (string) 
! ----------------------------------------------------------------------------------
!
! Name           : alphanumeric_is
!
! Argument (in ) : string
!
! Description    : returns .true. if string alphanumeric 
! Description    : "0123456789_abcdefghijklmnopqrstuvwxyz"  no "-"
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                  :: alphanumeric_is
  
! i/o
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here

  character(len=len(string))     :: lowstr
  character(len=37)              :: set ="0123456789_abcdefghijklmnopqrstuvwxyz"

! begin:
  l_here ='alphanumeric_is'
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  lowstr = string_to_lower (string)
  alphanumeric_is = verify (trim(lowstr), set) .eq. 0

  if (debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(lowstr),'<'
   write (logf,*)trim(l_here),'-d: =',alphanumeric_is          
  end if

  call exit (l_here,5)

  end function alphanumeric_is 

! ==================================================================================
  function boolean_is (string) 
! ----------------------------------------------------------------------------------
!
! Name           : boolean_is
!
! Argument (in ) : string
!
! Description    : returns .true. if string 'true' or 'false' 
!
! Authors:F.Colonna
! Date  :08 Oct 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)                  :: boolean_is
  
! i/o
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=len(string))     :: lowstr

! begin:
  l_here ='boolean_is'
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  lowstr = string_to_lower (string)
  boolean_is = trim(lowstr) .eq. 'true' .or.  trim(lowstr) .eq. 'false'

  if (debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(lowstr),'<'
   write (logf,*)trim(l_here),'-d: =',boolean_is          
  end if

  call exit (l_here,5)

  end function boolean_is 

! ==================================================================================
  function integer_is (string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : integer_is
!
! Argument (in ) : string
!
! Description    : returns true or false 
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)     :: result

! i/o:
  character(len=*), intent(in) :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=11)             :: set ="0123456789-"
  integer(dpi)                  :: c

! begin:
  l_here = "integer_is"
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  result = verify (trim(string), set) .eq. 0

  if (debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   do c = 1, len_trim(string)
   write (logf,*)trim(l_here),'-d: c #',c,' >',string(c:c),'< ichar =',ichar(string(c:c))
   end do
   write (logf,*)trim(l_here),'-d: >',set,'<'
   write (logf,*)trim(l_here),'-d: =',result
  end if

  call exit (l_here,5)

  end function integer_is

! ==================================================================================
  function integer_positive_is (string) 
! ----------------------------------------------------------------------------------
!
! Name           : integer_positive_is
!
! Argument (in ) : string
!
! Description    : returns true or false 
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)     :: integer_positive_is

! i/o:
  character(len=*), intent(in) :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=10) :: set ="0123456789"

! begin:
  l_here = "integer_positive_is"
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  integer_positive_is = verify (trim(string), set) .eq. 0

  if (debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*)trim(l_here),'-d: =',integer_positive_is
  end if

  call exit (l_here,5)

  end function integer_positive_is

! ==================================================================================
  function name_is (string) 
! ----------------------------------------------------------------------------------
!
! Name           : name_is
!
! Argument (in ) : string
!
! Description    : is alphanumeric with first character alphabetic
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  logical(bool)     :: name_is

! i/o:
  character(len=*), intent(in) :: string

! local:
  character(len=max_string_len) :: l_here

! begin:
  l_here = "name_is"
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  if (debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

  name_is = alphanumeric_is (trim(string))    &
            .and. alphabetic_is (string(1:1)) 
            
  if (debug) write (logf,*)trim(l_here),'-d: =',name_is

  call exit (l_here,5)

  end function name_is

! ==================================================================================
  function number_is (string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : number_is
!
! Argument (in ) : string
!
! Description    : returns true or false if in set 0123456789.
!
! Authors:F.Colonna
! Date  :04 Dec 2002
! ----------------------------------------------------------------------------------
  implicit none
! function:
  logical(bool)     :: result

! i/o:
  character(len=*), intent(in) :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=11)             :: set ="0123456789."
  character(len=max_string_len) :: l_str
  integer(dpi)                  :: verif

! begin:
  l_here = "number_is"
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  l_str  = string_to_lower (adjustl(trim(string)))
  verif  = verify (trim(l_str), set)
  result = verif == 0

  if (debug .and. trace_level > 4) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*)trim(l_here),'-d: result =',result 

   if(verif /= 0) then
   write (logf,*)trim(l_here),'-d: verify =',verif,' character >',l_str(verif:verif),'<'
   end if
  end if

  call exit (l_here,5)

  end function number_is

! ==================================================================================
  function numeric_is (string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : numeric_is
!
! Argument (in ) : string
!
! Description    : returns true or false
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! Revision: FCC 15 Dec 2000 l_str = string_to_lower (string)
! Revision: FCC 01 Apr 2002 blanks are removed from check   
! ----------------------------------------------------------------------------------
  implicit none
! function:
  logical(bool)     :: result

! i/o:
  character(len=*), intent(in) :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=15)             :: set ="0123456789.ed-+"
  character(len=max_string_len) :: l_str
  integer(dpi)                  :: verif

! begin:
  l_here = "numeric_is"
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  l_str = string_to_lower (adjustl(trim(string)))
  verif  = verify (trim(l_str), set)
  result = (verif == 0 .and. .not. alphabetic_is (l_str(1:1)) )

  if (debug .and. trace_level > 4) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*)trim(l_here),'-d: result =',result 
   if(verif /= 0) then
   write (logf,*)trim(l_here),'-d: verify =',verif,' character >',l_str(verif:verif),'<'
   end if
  end if

  call exit (l_here,5)

  end function numeric_is

! ==================================================================================
  function stars_is (string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : stars_is
!
! Argument (in ) : string
!
! Description    : returns true or false if string is ******** 
!
! Authors:F.Colonna
! Date  :04 Dec 2001
! ----------------------------------------------------------------------------------
  implicit none
! function:
  logical(bool)     :: result

! i/o:
  character(len=*), intent(in) :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=1)              :: set ="*"

! begin:
  l_here = "stars_is"
  call enter (l_here,5)

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  result = verify (trim(string), set) .eq. 0

  if (debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*)trim(l_here),'-d: =',result 
  end if

  call exit (l_here,5)

  end function stars_is

! ==================================================================================
  function boolean_to_string (value) 
! ----------------------------------------------------------------------------------
!
! Name           : boolean_to_string
!
! Argument (in ) : value
!
! Description    : converts a Boolean value into a string 
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

  character(len=max_string_len) :: boolean_to_string 
  logical(bool), intent (in)    :: value

  boolean_to_string ='FALSE'
  if(value) boolean_to_string ='TRUE'

  end function boolean_to_string

! ==================================================================================
  function double_to_string (value) result (result)
! ----------------------------------------------------------------------------------
!
! Name           : double_to_string
!
! Argument (in ) : value
!
! Description    : Converts a double precision word into a string
! Description    : left justifies the string
! Description    : if the value has an integer value conversion 
! Description    : to integer is done
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
!
  implicit none

! result:
  character(len=max_string_len)      :: result

! i/o:
  real(dp), intent (in)              :: value

! local:
  character(len=max_string_len) :: l_here
  character(len=30)                          :: zeroes, l_str, string
  character(len=250)                         :: temp
  integer(dpi)                               :: lenstr, ivalue
  integer(dpi)                               :: decimal, nbzero
  logical(bool)                              :: Moins
  real(dp)                                   :: tempval
 
! begin:
  l_here ='double_to_string'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: value,',value 

! special case NOT_INIT_DP
  if(abs(value - NOT_INIT_DP) < EPSILON(+1.d0)) then
   result = "NOT_INIT_DP"

  call exit (l_here,5)
   return

  end if

  Moins = value .lt. 0.d0
  tempval = value
  if(Moins) tempval = -value

  zeroes='0000000000000000000000000000000'
  string = repeat (' ',len(string))

! convert value into integer ivalue , get number of decimals:
  call count_decimals  (tempval, ivalue, decimal)

  if     (decimal.eq.0) then

    write(l_str,'(I30)') ivalue
    string = adjustl(l_str)

  else if(decimal.gt.0) then

    write(l_str,'(I30)') ivalue
    string = adjustl(l_str)
    lenstr = len_trim(string)
    nbzero = decimal-lenstr

    if(nbzero.lt.0) then
      temp = string(1:lenstr-decimal)//'.'//string(lenstr-decimal+1:)
    else
      temp = zeroes(1:nbzero+1)//string(1:lenstr)
      string = temp(1:1)//'.'//temp(2:)
      temp = string
    end if

    string = temp

  else if(decimal.lt.0) then

    write(l_str,'(D30.15)') tempval
    string = adjustl(l_str)

  end if
 
  if(Moins) then
    temp = string
    string='-'//temp
  end if

  result = string

  call exit (l_here,5)

  end function double_to_string

! ==================================================================================
  function real_to_string (value) 
! ----------------------------------------------------------------------------------
!
! Name           : real_to_string
!
! Argument (in ) : value
!
! Description    : Converts a real(sp) word into a string
! Description    : left justifies the string
! Description    : if the value has an integer value conversion 
! Description    : to integer is done
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
!
  implicit none

! function:
  character(len=max_string_len)      :: real_to_string

! i/o:
  real(sp), intent (in)              :: value

! local:
  character(len=max_string_len) :: l_here
  character(len=30)                  :: zeroes, l_str, string
  character(len=250)                 :: temp
  integer(dpi)                       :: lenstr, ivalue
  integer(dpi)                       :: decimal, nbzero
  logical(bool)                      :: Moins
  real(dp)                           :: tempval
 
! begin:
  l_here ='real_to_string'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: value=',value  

  Moins = value .lt. 0.0
  tempval = value
  if(Moins) tempval = -value

  zeroes='0000000000000000000000000000000'
  string = repeat(' ',len(string))

! convert value into integer ivalue , get number of decimals:
  call count_decimals  (tempval, ivalue, decimal)

  if     (decimal.eq.0) then

    write(l_str,'(I30)') ivalue
    string = adjustl(l_str)

  else if(decimal.gt.0) then

    write(l_str,'(I30)') ivalue
    string = adjustl(l_str)
    lenstr = len_trim(string)
    nbzero = decimal-lenstr

    if(nbzero.lt.0) then
      temp = string(1:lenstr-decimal)//'.'//string(lenstr-decimal+1:)
    else
      temp = zeroes(1:nbzero+1)//string(1:lenstr)
      string = temp(1:1)//'.'//temp(2:)
      temp = string
    end if

    string = temp

  else if(decimal.lt.0) then

    write (logf,*)' tempval=',tempval
    write(l_str,'(e30.15)') tempval
    write (logf,*)'  l_str >',trim(l_str),'<'

    string = adjustl(l_str)
    write (logf,*)'  string >',trim(string),'<'

  end if
 
  if(Moins) then
    temp = string
    string='-'//temp
  end if

  real_to_string = string

  call exit (l_here,5)

  end function real_to_string

! ==================================================================================
  function integer_to_string (value) result (result)
! ----------------------------------------------------------------------------------
!
! Name           : integer_to_string
!
! Argument (in ) : value
!
! Description    : converts an integer value into a string
! Description    : left justifies the string
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
!
  implicit none

! result:
  character(len=max_string_len)  :: result

! i/o :
  integer(dpi),     intent (in)  :: value

! local:
  character(len=max_string_len) :: l_here
  character(len=30)              :: l_str

! begin:
  l_here ='integer_to_string'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: value =',value

  if(value == NOT_INIT_I) then

! special case NOT_INIT_I
    result = "NOT_INIT_I"

  else

    l_str = repeat(' ', 30)
    write(l_str,'(I30)') value
    result = adjustl(l_str)

  end if

  if(debug) write (logf,*) trim(l_here),'-d: result =',trim(result)

  call exit (l_here,5)

  end function integer_to_string

! ==================================================================================
  function char_to_upper (charac) 
! ----------------------------------------------------------------------------------
!
! Name           : char_to_upper
!
! Argument (in ) : charac
!
! Description    : Converts a character from any case to Upper-case
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=1), intent (in)  :: charac   

! function:
  character(len=1)               :: char_to_upper 

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=1),parameter :: lita='a',litz='z',biga='A',space=' '
  integer(dpi)                               :: ichr

! begin:
  l_here ='char_to_upper'
  call enter (l_here, 10)

  if(debug) write (logf,*) trim(l_here),'-d: charac >',charac,'<'

  ichr = ichar(charac)
  if(ichr.ge.ichar(lita).and.ichr.le.ichar(litz)) then
    char_to_upper = char(ichr+(ichar(biga)-ichar(lita)))
  else
    char_to_upper = charac
  end if
  if(ichr.lt.ichar(space)) char_to_upper = space

  call exit (l_here, 10)

  end function char_to_upper

! ==================================================================================
  function char_to_lower (charac) 
! ----------------------------------------------------------------------------------
!
! Name           : char_to_lower
!
! Argument (in ) : charac
!
! Description    : Converts a character from any case to lower-case
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=1)  :: char_to_lower

! function:
  character(len=1), intent (in)  :: charac

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=1),parameter                :: lita='a',biga='A'
  character(len=1),parameter                :: bigz='Z',space=' '
  integer(dpi)                               :: ichr

! begin:
  l_here ='char_to_lower'
  call enter (l_here, 10)

! if(debug) write (logf,*)trim(l_here),'-d: charac >',trim(charac),'<'

  ichr = ichar(charac)
  if(ichr.ge.ichar(biga).and.ichr.le.ichar(bigz)) then
    char_to_lower = char(ichr-(ichar(biga)-ichar(lita)))
  else
    char_to_lower = charac
  end if
  if(ichr.lt.ichar(space)) char_to_lower = space

  call exit (l_here, 10)

  end function char_to_lower

! ==================================================================================
  function string_to_upper (string) 
! ----------------------------------------------------------------------------------
!
! Name           : string_to_upper
!
! Argument (in ) : string
!
! Description    : Converts a string from any case to Upper-case
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent (in)  :: string
  character(len=len(string))     :: string_to_upper 

! local:
  character(len=max_string_len) :: l_here

  character(len=len(string))                 :: l_str    
  integer(dpi)                               :: char

! begin:
  l_here ='string_to_upper'
  call enter (l_here,5)

  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  l_str = string
  if (len_trim(string) .ne. 0) then
   do char = 1, len_trim(string)
    l_str(char:char) = char_to_upper (string(char:char))
   end do
  end if

  string_to_upper = l_str

  call exit (l_here,5)

  end function string_to_upper

! ==================================================================================
  function string_to_lower (string) result (result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_to_lower
!
! Argument (in ) : string
!
! Description    : Converts a string from any case to lower-case
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent (in) :: string

! result:
  character(len=len(string))    :: result 

! local:
  character(len=max_string_len) :: l_here

  character(len=len(string))                 :: l_str    
  integer(dpi)                               :: char

! begin:
  l_here ='string_to_lower'
  call enter (l_here,5)

  if(debug .and. trace_level > 4) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
 
  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  l_str = string
  if (len_trim(string) .ne. 0) then
   do char = 1, len_trim(string)
    l_str(char:char) = char_to_lower (string(char:char))
   end do
  end if

  result = l_str

  call exit (l_here,5)

  end function string_to_lower

! ==================================================================================
  function string_to_boolean (string) 
! ----------------------------------------------------------------------------------
!
! Name           : string_to_boolean
!
! Argument (in ) : string
!
! Description    : Converts a String into a Boolean value
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  logical(bool)                  :: string_to_boolean
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here

  logical(bool)                              :: local_bool               
  character(len=1)                           :: charac, upperc

! begin:
  l_here ='string_to_boolean'
  call enter (l_here,5)

  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

  local_bool = .false.
  if(len(string).eq.0) then

   if(warning_level .gt. 2) write (logf,'(a)') &
   'string_to_boolean_warning: zero length string. converted to .false.'

    local_bool = .false.

  else

    charac = adjustl(string)
    upperc = char_to_upper(charac)
    if(debug) write (logf,'(5a)') &
    'string_to_boolean: charac >',charac,'< upperc >',upperc,'<'
    
    if(upperc .eq. 'T') then
      local_bool = .true.
    else if(upperc .eq. 'F') then
      local_bool = .false.
    else
      write (logf,'(3a)') &
     'error_string_to_boolean> not a  Boolean value "',trim(string),'"'
      call die ('string_to_boolean','not a Boolean value')
    end if

  end if

  string_to_boolean = local_bool

  if(debug) write (logf,*)' string_to_boolean=',string_to_boolean

  call exit (l_here,5)

  end function string_to_boolean

! ==================================================================================
  subroutine string_to_boolean_tine (boolean_result, string)
! ----------------------------------------------------------------------------------
!
! Name           : boolean_result_tine
!
! Argument (in ) : boolean_result
! Argument (in ) : string
!
! Description    : Converts a String into a Boolean value
!
! Authors:R. Assaraf
! Date  :27 Jan 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  logical(bool)                  :: boolean_result
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here
  logical(bool)                              :: local_bool               
  character(len=1)                           :: charac, upperc

! begin:
  l_here ='string_to_boolean_tine'
  call enter (l_here,5)

  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

  local_bool = .false.
  if(len(string).eq.0) then

   if(warning_level .gt. 2) write (logf,'(a)') &
   'boolean_result_warning: zero length string. converted to .false.'

    local_bool = .false.

  else

    charac = adjustl(string)
    upperc = char_to_upper(charac)
    if(debug) write (logf,'(5a)') &
    'boolean_result: charac >',charac,'< upperc >',upperc,'<'
    
    if(upperc .eq. 'T') then
      local_bool = .true.
    else if(upperc .eq. 'F') then
      local_bool = .false.
    else
      write (logf,'(3a)') &
     'error_boolean_result> not a  Boolean value "',trim(string),'"'
      call die ('boolean_result','not a Boolean value')
    end if

  end if

  boolean_result = local_bool

  if(debug) write (logf,*)' boolean_result=',boolean_result

  call exit (l_here,5)

  end subroutine string_to_boolean_tine

! ==================================================================================
  function string_to_double (string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_to_double
!
! Argument (in ) : string
!
! Description    : Converts a String into a double  value
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  real(dp)                       :: result
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here

  character(len=30)                          :: text

! begin:
  l_here ='string_to_double'
  call enter (l_here,5)
  
  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

  if(len(string).eq.0) then

   if(warning_level .gt. 2) write (logf,'(a)') &
   'warning_string_to_double> zero length string. converted to 0'
   result = 0

  else

! problem with portland
!   text = adjustr(trim(string)) 
    text = string
    read(text,'(e30.0)') result
    if (debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'< -> result=',result
  end if

  call exit (l_here,5)

  return

 999 continue
     write (logf,*) trim(l_here),'-e: error in converting string >',trim(string),'< to double'
     call die (routines_21,'error in converting to double')

  end function string_to_double

! ==================================================================================
  subroutine string_to_double_tine (double_result, string)
! ----------------------------------------------------------------------------------
!
! Name           : string_to_double
!
! Argument (in ) : double_result
! Argument (in ) : string
!
! Description    : Converts a String into a double  value
!
! Authors:R. Assaraf
! Date  :27 Jan 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  real(dp)                       :: double_result
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=30)             :: text

! begin:
  l_here ='string_to_double_tine'
  call enter (l_here,5)
  
  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'

  if(len(string).eq.0) then

   if(warning_level .gt. 2) write (logf,'(a)') &
   'warning_double_result> zero length string. converted to 0'
   double_result = 0

  else
! problem with portland
! text = adjustr(trim(string))  
    text = string
    read(text,'(d30.0)',err=999) double_result

  end if

  call exit (l_here,5)

  return

 999 continue

     write (logf,*) trim(l_here),'-e: error in converting string >',trim(String),'< to double'
     call die (routines_21,'error in converting to double')

  end subroutine string_to_double_tine

! ==================================================================================
  function string_to_integer (string) 
! ----------------------------------------------------------------------------------
!
! Name           : string_to_integer
!
! Argument (in ) : string
!
! Description    : Converts a string into a integer value
! Description    : blanks are ignored
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  integer(dpi)                   :: string_to_integer
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here

  character(len=30)  :: text

! begin:
  l_here = 'string_to_integer'
  call enter (l_here,5)

  if(len(string).eq.0) then

    if(warning_level .gt. 0) write (logf,'(a)') &
   'string_to_integer-w: zero length string. converted to 0'

    string_to_integer = 0

  else if(.not.integer_is(trim(adjustl(string)))) then

    write (logf,*) trim(l_here),'-e: string >',trim(string),'< is not an integer'
    call die (routines_21,'string to be converted not an integer')

  else

    text = trim(string)
   
    read(text,'(i30)') string_to_integer

  end if

  call exit (l_here,5)

  end function string_to_integer

! ==================================================================================
  subroutine string_to_integer_tine (integer_result, string)
! ----------------------------------------------------------------------------------
!
! Description    :  subroutine string_to_integer_tine (integer_result,string)
! Description    : Converts a string into a integer value
! Description    : blanks are ignored
!
! Authors:R. Assaraf
! Date  :27  Jan 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  integer(dpi)                   :: integer_result
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here

  character(len=30)  :: text

! begin:
  l_here = 'string_to_integer_tine'
  call enter (l_here,5)

  if(len(string).eq.0) then

    if(warning_level .gt. 0) write (logf,'(a)') &
   'integer_result_warning: zero length string. converted to 0'

  integer_result = 0

  else if(.not.integer_is(trim(adjustl(string)))) then

    write (logf,*) trim(l_here),'-e: string >',trim(string),'< is not an integer'
    call die (routines_21,'string to be converted not an integer')

  else
! problem with portland
!   text = adjustr(trim(string))
    text = string 
    read(text,'(i30)') integer_result

  end if

  call exit (l_here,5)

  end subroutine string_to_integer_tine

! ==================================================================================
  function string_to_real (string) 
! ----------------------------------------------------------------------------------
!
! Name           : string_to_real
!
! Argument (in ) : string
!
! Description    : Converts a String into a double  value
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  real(sp)                       :: string_to_real
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here
  character(len=30)              :: text

! begin:
  l_here ='string_to_real'
  call enter (l_here,5)

  if(len(string).eq.0) then

   if(warning_level .gt. 2) write (logf,'(a)') &
   ': zero length string. converted to 0'
   string_to_real = 0

  else
! problem with portland
!   text = adjustr(trim(string)) 
    text = string
    read(text,'(e30.0)',err=999) string_to_real

  end if

  call exit (l_here,5)

  return

 999 continue

    write (logf,*) trim(l_here),'-e: error in converting string >',trim(String),'< to real'
    call die (routines_21,'error in converting to real')

  end function string_to_real

! ==================================================================================
  subroutine string_to_real_tine (real_result, string)
! ----------------------------------------------------------------------------------
!
! Description    :  subroutine string_to_real_tine (real_result,string)
! Description    : Converts a String into a double  value
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Modification: copy from the function  string_to_real_tine
! Date: 9 Jan 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  real(sp)                       :: real_result
  character(len=*), intent (in)  :: string

! local:
  character(len=max_string_len) :: l_here
 
  character(len=30)                          :: text

! begin:
  l_here ='string_to_real_tine'
  call enter (l_here,5)

  if(len(string).eq.0) then

   if(warning_level .gt. 2) write (logf,'(a)') &
   ': zero length string. converted to 0'
  real_result = 0

  else
! problem with portland
!   text = adjustr(trim(string)) 
    text = string
    read(text,'(e30.0)',err=999) real_result

  end if

  call exit (l_here,5)

  return

 999 continue

    write (logf,*) trim(l_here),'-e: error in converting string >',trim(String),'< to real'
    call die (routines_21,'error in converting to real')

  end subroutine string_to_real_tine

! ==================================================================================
  function char_substitute (fchar, tchar, this_str) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : char_substitute
!
! Argument (in ) : fchar
! Argument (in ) : tchar
! Argument (in ) : this_str
!
! Description    : Substitutes all Characters "fchar" by "tchar" in this_str
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=1), intent (in) :: fchar, tchar 
  character(len=*), intent (in) :: this_str

! result:
  character(len=len(this_str))    :: result

! local:
  character(len=max_string_len) :: l_here
  character(len=len(this_str))    :: l_str
  integer(dpi)                  :: i

! begin:
  l_here ='char_substitute'
  call enter (l_here,10)

! write (logf,*)trim(l_here),'-d: this_str >',trim(this_str),'<'
! write (logf,*)trim(l_here),'-d: fchar  >',fchar,'< code =',ichar(fchar)
! write (logf,*)trim(l_here),'-d: tchar  >',tchar,'< code =',ichar(tchar)

  if(len(this_str).gt.max_string_len) then

    write (logf,'(4a)') trim(l_here), &
   ': this_str "',this_str,'" too long'
    write (logf,'(2a,i10)') trim(l_here), &
   ': increase max_string_len=',max_string_len

  else if(len(trim(adjustl(this_str))).eq.0) then

    if(warning_level .gt. 0) &
    write (logf,'(4a)') trim(l_here),'-w: zero length string'
    result = this_str

  else

    l_str = this_str

    do i = 1,len(this_str)
     if(l_str(i:i) == fchar) then
        l_str(i:i)  = tchar 
     end if
    end do

   result = l_str

!  write (logf,*)trim(l_here),'-d: result >',trim(l_str),'<'

  end if

  call exit (l_here,10)

  end function char_substitute

! ==================================================================================
  function string_head_get (string, substring, back) 
! ----------------------------------------------------------------------------------
!
! Name           : string_head_get
!
! Argument (in ) : string
! Argument (in ) : substring
! Argument (in ) : back  (optional)
!
! Description    : returns string head until "substring" (excluded)
! Description    : string = head[substring]tail 
! Description    : if "substring" not in "string" return blank string
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent (in) :: string, substring
  logical(bool), optional       :: back

! function:
  character(len=len(string))    :: string_head_get

! local:
  character(len=max_string_len) :: l_here

  integer(dpi)                               :: ind

! begin:
  l_here ='string_head_get'
  call enter (l_here,5)

  if(present(back)) then
   ind = index(string, substring, back = back)
   if(debug)write (logf,*) trim(l_here),'-d: back=',back
  else
   ind = index(string, substring, back = .false.)
  end if

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*) trim(l_here),'-d: substring >',trim(substring),'<'
   write (logf,*) trim(l_here),'-d: ind =',ind
  end if

  if(ind.eq.0) then
    string_head_get = " "

  else

   string_head_get = string(:ind-1)

  end if

  call exit (l_here,5)

  end function string_head_get

! ==================================================================================
  function string_tail (string, substring, back) 
! ----------------------------------------------------------------------------------
!
! Name           : string_tail
!
! Argument (in ) : string
! Argument (in ) : substring
! Argument (in ) : back
!
! Description    : returns string tail from "substring" (excluded) to end
! Description    : string = head[substring]tail 
! Description    : if "substring" not in "string" returns blanks 
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 22 Jan 2002 by F. Colonna len(substring)
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent (in)  :: string, substring
  logical(bool), optional        :: back

! function:
  character(len=len(string))     :: string_tail

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: ind

! begin:
  l_here ='string_tail'
  call enter (l_here,5)

  if(present(back)) then
   ind = index(string, substring, back = back)
   if(debug)write (logf,*) trim(l_here),'-d: back=',back
  else
   ind = index(string, substring, back = .false.)
  end if

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*) trim(l_here),'-d: substring >',trim(substring),'<'
   write (logf,*) trim(l_here),'-d: ind =',ind
  end if

  if(ind.eq.0) then

    string_tail = blanks

  else

   string_tail = adjustl(string(ind+len(substring):))

  end if

  call exit (l_here,5)

  end function string_tail

! ==================================================================================
  function string_next_double_get (string, substring)  result(result)
! ----------------------------------------------------------------------------------
!
! Name           : string_next_double_get
!
! Argument (in ) : string
! Argument (in ) : substring
!
! Description    : returns integer following "substring" in 'string"
! Description    : if "substring" not in "string" wero   is returned
!
! Authors:F.Colonna
! Date  :22 Jan 2002
! ----------------------------------------------------------------------------------
  implicit none

! result:
  real(dp)                       :: result

! i/o
  character(len=*), intent (in)  :: string, substring

! local:
  character(len=max_string_len) :: l_here
  character(len=len(string))    :: stemp

! begin:
  l_here ='string_next_double_get'
  call enter (l_here,5)

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*) trim(l_here),'-d: substring >',trim(substring),'<'
  end if

  stemp = string_tail (string, substring, .true.)     

! skip "=" sign
  if(stemp(1:1) == "=") then
    stemp = string_tail (string, "=", .true.)
  end if

! skip "#" sign
  if(stemp(1:1) == "#") then
    stemp = string_tail (string, "#", .true.)
  end if

  if(len_trim(stemp) > 0) then
    result  = string_to_double (stemp)
  else 
    result = zero
  end if

  if(debug) then
   write (logf,*) trim(l_here),'-d: result =',result
  end if

  call exit (l_here,5)

  end function string_next_double_get

! ==================================================================================
  function string_next_integer_get (string, substring)  result(result)
! ----------------------------------------------------------------------------------
!
! Name           : string_next_integer_get
!
! Argument (in ) : string
! Argument (in ) : substring
!
! Description    : returns integer following "substring" in "string"
! Description    : if "substring" not in "string" wero   is returned
!
! Authors:F.Colonna
! Date  :22 Jan 2002
! ----------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                   :: result

! i/o
  character(len=*), intent (in)  :: string, substring

! local:
  character(len=max_string_len) :: l_here
  character(len=len(string))    :: stemp

! begin:
  l_here ='string_next_integer_get'
  call enter (l_here,5)

  if(debug) then
   write (logf,*) trim(l_here),'-d: searching for substring >',trim(substring),'<'
   write (logf,*) trim(l_here),'-d: in string >',trim(string),'<'
  end if

  stemp = string_tail (string, substring, .true.)     

! skip "=" sign
  if(stemp(1:1) == "=") then
    stemp = string_tail (string, "=", .true.)
  end if

! skip "#" sign
  if(stemp(1:1) == "#") then
    stemp = string_tail (string, "#", .true.)
  end if

  if(len_trim(stemp) > 0) then
    result  = string_to_integer (stemp)
  else 
    result = zero
   !call die (l_here, 'missing "'+substring+'" in string "'+string+'"')
  end if

  if(debug) then
   write (logf,*) trim(l_here),'-d: result =',result
  end if

  call exit (l_here,5)

  end function string_next_integer_get

! ==================================================================================
  function string_piece_get (string, fstring, lstring) result (result) 
! ----------------------------------------------------------------------------------
!
! Name          : string_piece_get
!
! Argument (in) : string
! Argument (in) : fstring
! Argument (in) : lstring
! 
! Description    : returns string piece from "fstring" to "lstring" (excluded)
! Description    : if "fstring" not in "string" returns the whole string
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent (in) :: string, fstring, lstring

! result 
  character(len=len(string))    :: result 

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: indf, indl
  integer(dpi)                  :: lenf! lenl

! begin:
  l_here ='string_piece_get'
  call enter (l_here,5)

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*) trim(l_here),'-d: fstring >',trim(fstring),'<'
   write (logf,*) trim(l_here),'-d: lstring >',trim(lstring),'<'
  end if

  indf = index(string, trim(fstring))
  indl = index(string, trim(lstring))

  lenf = len_trim(fstring)
! lenl = len_trim(lstring)

  if(indf.eq.0 .and. indl.eq.0) then
    result = string 
  else if(indf.eq.0 .and. indl.ne.0) then
    result = string(:indl-1)
  else if(indf.ne.0 .and. indl.eq.0) then
    result = string(indf+lenf:)
  else
    result = string(indf+lenf:indl-1)
  end if

  if(debug) then
   write (logf,*) trim(l_here),'-d: result >',trim(result),'<'
  end if

  call exit (l_here,5)

  end function string_piece_get

! ==================================================================================
  function string_substitute (string, target, new_string) result (result) 
! ----------------------------------------------------------------------------------
!
! Name          : string_substitute
!
! Argument (in) : string
! Argument (in) : target
! Argument (in) : new_string
! 
! Description    : substitute "target" with by "new_string" in string  
!
! Authors:F.Colonna
! Date  :21 Jun 2001
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent (in) :: string, target, new_string

! result 
  character(len=len(string))    :: result 

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: indf, indl

! begin:
  l_here ='string_substitute'
  call enter (l_here,5)

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*) trim(l_here),'-d: target >',trim(target),'<'
   write (logf,*) trim(l_here),'-d: substituted by >',trim(new_string),'<'
  end if

  indf = index(string, trim(target))

  if(indf == 0) then
    message ='in routine'*l_here*' string to be substituted >' &
           +target+'< is not found in string >'+string+'<'
    call require (message, indf /= 0)
  end if

  if(len_trim(target) == 0) then
    message ='in routine'*l_here*' string to be substituted >'+target+'< is empty'
    call require (message, len_trim(target) /= 0)
  end if

  if(len_trim(new_string) == 0) then
    message ='in routine'*l_here*' string to substitute with >'+new_string+'< is empty'
    call require (message, len_trim(new_string) /= 0)
  end if

  indl = indf+len_trim(target)-1

  result = string(1:indf-1)//trim(new_string)//string(indl+1:)

  if(debug) then
   write (logf,*) trim(l_here),'-d: result >',trim(result),'<'
  end if

  call exit (l_here,5)

  end function string_substitute

! ==================================================================================
  function string_remove (string, substring) result(result)
! ----------------------------------------------------------------------------------
!
! Name           : string_remove
!
! Argument (in ) : string
! Argument (in ) : substring
!
! Description    : returns string without "substring"
! Description    : if "substring" not in "string" return the whole string
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o
  character(len=*), intent (in)             :: string, substring

! result:
  character(len=len(string)-len(substring)) :: result

! local:
  character(len=max_string_len) :: l_here

  integer(dpi)                              :: ind, lens

! begin:
  l_here ='string_remove'
  call enter (l_here,5)

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*) trim(l_here),'-d: substring >',trim(substring),'<'
  end if

  ind = index(string, substring)
  lens = len(substring)
  if(ind.eq.0) then
    result = string 
  else
    result = string(:ind-1)//string(ind+lens:)
  end if

  call exit (l_here,5)

  end function string_remove

! ==================================================================================
  function char_in_string_count (charac, string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name       : char_in_string_count 
!
! Argument (in) : charac
! Argument (in) : string
!
! Description    : returns the number of times charac was found in string
!
! Authors:F.Colonna
! Date  :21 Oct 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  character(len=1), intent(in)               :: charac

! function:
  character(len=*), intent(in)               :: string

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                               :: count, i

! begin:
  l_here = "char_in_string_count"
  call enter (l_here,10)

  if(debug) write (logf,*) trim(l_here),'-d: charac >',charac,'<'
  if(debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'<'

  count = 0
  do i = 1, len(string)
   if (string(i:i) == charac) count = count+1
   if(debug) write (logf,*)trim(l_here), &
                   '-d: i=',i,' string(i:i) >',string(i:i),'<'
  end do
  if(debug) write (logf,*)trim(l_here),'-d: count =',count

  result = count

  call exit (l_here,10)

  end function char_in_string_count

! ==================================================================================
  function char_null_to_blank_all (string) result (result) 
! ----------------------------------------------------------------------------------
!
! Name           : char_null_to_blank_all 
!
! Argument (in ) : string
!
! Description    : change null into blank
!
! Authors:F.Colonna
! Date   :21 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: string

! result:
  character(len=len(string))    :: result

! local:
  character(len=max_string_len) :: l_here
  character(len=len(string))    :: l_string, t_string

! begin:
  l_here ='char_null_to_blank_all'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'<'

  l_string = string
  do while (string_null_char_at (l_string) .ne. 0)
   write (logf,*)trim(routines_21),'-w: null_char at =',string_null_char_at (l_string)
   t_string = char_substitute (null_c, ' ', l_string)
   l_string = t_string
  end do

  result = l_string
  if(debug) write (logf,*) trim(l_here),'-d: result >',trim(l_string),'<'

  call exit (l_here,5)

  end function char_null_to_blank_all

! ==================================================================================
  function string_in_array_count (string, array_string) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_in_array_count 
!
! Argument (in)  : string
! Argument (in)  : string_array
!
! Description    : returns the number of times string was found in array_string
! Description    : this equality is defined up to next blank in array_string(i)
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! Revision:24 Dec 2000 cosmetic
! ----------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                               :: result

! i/o:
  character(len=*), intent(in)               :: array_string(:)
  character(len=*), intent(in)               :: string

! function:

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                               :: i, l_str

! begin:
  l_here = "string_in_array_count"
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'<'

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  result = 0
  l_str = len_trim(string)
  do i = 1, size(array_string)
   if(array_string(i)(l_str+1:l_str+1) == ' ' .and. &
      array_string(i)(1:l_str) .eq. string(1:l_str)) result = result +1
    if(debug) then
     write (logf,*)trim(l_here),'-d: l_str=',l_str
     write (logf,*)trim(l_here),'-d: i=',i,' string >',trim(string),'<'
     write (logf,*)trim(l_here),'-d: array_string(i) >',array_string(i)(1:l_str),'<'
     write (logf,*)trim(l_here),'-d: result  =',result 
    end if
  end do
  if(debug) write (logf,*)trim(l_here),'-d: whole result  =',result 

  call exit (l_here,5)

  end function string_in_array_count

! ==================================================================================
  function string_head_in_array_count (string, array_string, first, last) &
           result(result)
! ----------------------------------------------------------------------------------
!
! Name           : string_head_in_array_count 
!
! Argument (in)  : string
! Argument (in)  : string_array
! Argument (opt) : first
! Argument (opt) : last
!
! Description    : returns the number of times an element of array_string 
! Description    : begins by string                            
!
! Authors:F.Colonna
! Date  :22 Sep 2000
! Revision:24 Dec 2000 cosmetic
! Revision: 19 Nov 2002 F. Colonna first, last
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in) :: string
  character(len=*), intent(in) :: array_string(:)
  integer(dpi), optional       :: first
  integer(dpi), optional       :: last

! result:
  integer(dpi)                  :: result

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: count, i, strlen
  integer(dpi)                  :: l_first, l_last  
  character(len=max_string_len) :: l_str

! begin:
  l_here = "string_head_in_array_count"
  debug = .false.
  call enter (l_here,1)

  if (present(first)) then
   l_first = max (1, first)
  else
   l_first = 1
  end if
  if (present(last))  then
    l_last  = min (last, size (array_string))
  else
    l_last  = size (array_string)
  end if

  if(debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'<'
  if(debug) write (logf,*) trim(l_here),'-d: l_first =',l_first,' l_last =',l_last

  if(l_first > l_last) then
    message = 'in routine'*l_here+' first ='*l_first*' last='*l_last
    call require (message, l_first <= l_last)
  end if

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  strlen = len_trim (string)
  l_str = blanks

  count = 0
  do i = l_first, l_last 
   l_str = trim(array_string(i))
   if (l_str(1:strlen) == string) count = count+1
   if(debug) then
    write (logf,*)trim(l_here),'-d: i=',i,' string >',string,'<'
    write (logf,*)trim(l_here),'-d: array_string(i) >',trim(l_str),'<'
    write (logf,*)trim(l_here),'-d: count=',count 
   end if
  end do

  if(debug) write (logf,*)trim(l_here),'-d: count =',count

  result = count

  call exit (l_here,1)

  end function string_head_in_array_count

! ==================================================================================
  subroutine count_decimals (value, ivalue, decimal)
! ----------------------------------------------------------------------------------
!
! Name           : count_decimals
!
! Argument (in ) : value
! Argument (in ) : ivalue
! Argument (in ) : decimal
!
! Description    : counts decimals in floating point number "value"
! Description    : decimal   : number of decimals
! Description    : ivalue    : integer value returned if decimal inf to  7
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! function:
  integer(dpi), intent(out) :: ivalue, decimal

! i/o:
  real(dp),intent(in)       :: value

! local:
  character(len=max_string_len) :: l_here
  real(dp)      :: mult, dif
  integer(dpi)  :: dec

! begin:
  l_here ='count_decimals'
  call enter (l_here,10)

  if(debug) write (logf,*) trim(l_here),'-d: value=',value

  if(abs(value) > bigdp) then
   write (logf,*)trim(l_here),'-e: value=',value
   call die (l_here,'real value too big to count number of decimals')
  end if

  decimal = -1
  do dec = 0, 6
    mult   = 10.**dec
    ivalue = nint(mult*value)
    dif =  abs(float(ivalue) - mult*value)/mult
    if(debug) write (logf,*) trim(l_here),'-d: dec=',dec,' mult=',mult,' ivalue=',ivalue,' dif=',dif
    if(abs(dif).lt.1.d-12)then
      decimal = dec
      exit
    end if
  end do

  call exit (l_here,10)

  end subroutine count_decimals 

! ==================================================================================
  function string_after_char_insert_blank (string, charac, beg_esc, end_esc) &
                                    result(result)
! ----------------------------------------------------------------------------------
!
! Name           : string_after_char_insert_blank
!
! Argument (in ) : string
! Argument (in ) : charac
! Argument (in ) : beg_esc
! Argument (in ) : end_esc
!
! Description    : inserts a blank after "charac" in "string"
! Description    : all substring in between beg_esc and end_esc is kept as it is
! Description    : returns string with inserted blanks
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 11 Sep 2002 string truncation
! Revision: 08 Nov 2002 insertion after first character
! Revision: 04 Dec 2002 DA + -
! Revision: 09 Dec 2002 /*  */ 
! Revision: 10 Dec 2002 , -2.3 or ( -17 ...
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len)  :: result

! i/o
  character(len=*), intent (in)  :: string
  character(len=1), intent (in)  :: charac
  character(len=1), intent (in)  :: beg_esc, end_esc

! local:
  character(len=max_string_len) :: l_here
  character(len=max_string_len)              :: sout, l_str
  integer(dpi)                               :: cur, ind
  integer(dpi)                               :: len_s    
  logical(bool)                              :: done, escape, skip
  character(len=1)                           :: cur_char

! begin:
  l_here ='string_after_char_insert_blank'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*)trim(l_here),'-d: charac >',charac,'<'
  end if

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  done  = .false.
  skip  = .false.
  escape = .false.
  l_str = blanks 
  l_str = string
  len_s = len_trim(string)
  sout  = blanks 

  cur   = 0
  sout  = string

  do while(.not.done)

   cur = cur + 1
   ind = index (l_str(cur:), charac)

   if(ind == 0) exit

   cur_char = l_str(cur:cur)

   if(cur_char == beg_esc ) then 
     escape = .true.
   else if(cur_char == end_esc ) then
     escape = .false.
   end if

   if(l_str(cur:cur+1) == "/*" ) then 
     skip = .true.
   else if(l_str(cur:cur+1) == "*/" ) then 
     skip = .true.
   else if(cur > 1 .and. l_str(cur-1:cur) == "/*" ) then 
     skip = .true.
   else if(cur > 1 .and. l_str(cur-1:cur) == "*/" ) then 
     skip = .true.
!  else if( (cur > 1 .and. (charac == "+" .or. charac == "-") )  &  
! skip these: ,-1.d-5 or (-.1d-05
!      .and. (     l_str(cur-1:cur-1) == ","   &
!             .or. l_str(cur-1:cur-1) == "("   &
!             .or. l_str(cur-1:cur-1) == " " ) &
!      .and. (verify (l_str(cur+1:cur+1), '.abcdefghijklmnopqrstuvwxyz0123456789') == 0 )) then
!  
!    skip =.true.
   else if( (cur > 2 .and. (charac == "+" .or. charac == "-") )  &  
! skip these: 1.d-5  1.d-05 1.23e+4 .e-3
       .and. (l_str(cur-1:cur-1) == "e" .or. l_str(cur-1:cur-1) == "d") &
       .and. (verify (l_str(cur-2:cur-2), ' 0123456789') == 0 ) &
       .and. (verify (l_str(cur+1:cur+1), '0123456789') == 0 )) then
     skip  = .true.
   else
     skip = .false.
   end if

   if(debug) then 
    write (logf,*)trim(l_here),'-d: l_str >',trim(l_str),'<'
    write (logf,*)trim(l_here),'-d: ind ',ind
    write (logf,*)trim(l_here),'-d: cur # ',cur,' cur_char >',cur_char,'<'
    write (logf,*)trim(l_here),'-d: escape ',escape
    write (logf,*)trim(l_here),'-d: skip   ',skip  
   end if

! insert after charac
   if(cur_char == charac) then 
    if(.not.escape .and. l_str(cur+1:cur+1).ne.' ') then

     if(skip) then
       if (debug) write (logf,*)trim(l_here),'-d: skipped '
     else

       sout  = l_str(1:cur)//' '//l_str(cur+1:)
       len_s = len_s + 1

       if(debug) then 
         write (logf,*)trim(l_here),'-d: len_s=',len_s,' len(string)=',len(string)
       end if

       if( len_s .ge. len(string) ) then
         write (logf,*)trim(l_here),'-e: expanded string length=',len_s, &
                             ' string length=',len(string)
         message = 'in routines '*routines_321+' truncated string >'+sout+'<'
         call require (message, len_s < len(string))
       end if

       l_str = sout
       cur   = cur + 1
      end if ! + -
    end if
   end if

  end do

  if(debug) then 
   write (logf,*)trim(l_here),'-d: sout >',trim(sout),'<'
  end if

  result = sout

  call exit (l_here,5)

  end function string_after_char_insert_blank

! ==================================================================================
  function string_before_char_insert_blank (string, charac, beg_esc, end_esc) &
                                     result(result)
! ----------------------------------------------------------------------------------
!
! Name           : string_before_char_insert_blank
!
! Argument (in ) : string
! Argument (in ) : charac
! Argument (in ) : beg_esc
! Argument (in ) : end_esc
!
! Description    : inserts a blank before "charac" in "string"
! Description    : all substring in between beg_esc and end_esc is kept as it is
! Description    : returns string with inserted blanks
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 11 Sep 2002 string truncation
! Revision: 04 Dec 2002 + and -                
! Revision: 09 Dec 2002 /* and */              
! ----------------------------------------------------------------------------------
  implicit none

! result:
  character(len=max_string_len)  :: result

! i/o
  character(len=*), intent (in)  :: string
  character(len=1), intent (in)  :: charac
  character(len=1), intent (in)  :: beg_esc, end_esc

! local:
  character(len=max_string_len) :: l_here
  character(len=max_string_len) :: sout
  character(len=max_string_len) :: l_str
  integer(dpi)                  :: cur, ind, len_s
  logical(bool)                 :: done, escape, skip
  character(len=1)              :: cur_char

! begin:
  l_here ='string_before_char_insert_blank'
  call enter (l_here,5)

  if(debug) then
   write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
   write (logf,*)trim(l_here),'-d: charac >',charac,'<'
  end if

  if(len (string) == 0) then
    message = 'in routine'*l_here+' zero length string'
    call require (message, len (string) .gt. 0)
  end if

  done = .false.
  skip = .false.
  escape = .false.

  l_str = blanks 
  l_str = string
  sout  = blanks 
  len_s = len_trim(string)

  cur   = 0
  sout  = string

  do while(.not.done)

   cur = cur + 1
   ind = index (l_str(cur:), charac)

   if(ind == 0) exit !<------------------ exit

   cur_char = l_str(cur:cur)

   if(cur_char == beg_esc ) then 
     escape = .true.
   else if(cur_char == end_esc ) then
     escape = .false.
   end if

   if(l_str(cur:cur+1) == "/*" ) then 
     skip = .true.
   else if(l_str(cur:cur+1) == "*/" ) then 
     skip = .true.
   else if(cur > 1 .and. l_str(cur-1:cur) == "/*" ) then 
     skip = .true.
   else if(cur > 1 .and. l_str(cur-1:cur) == "*/" ) then 
     skip = .true.
   else if( (cur > 2 .and. (charac == "+" .or. charac == "-") )  &  
! skip these: 1.d-5  1.d-05 1.23e+4 .e-3
       .and. (l_str(cur-1:cur-1) == "e" .or. l_str(cur-1:cur-1) == "d") &
       .and. (verify (l_str(cur-2:cur-2), ' 0123456789') == 0 ) &
       .and. (verify (l_str(cur+1:cur+1), '0123456789') == 0 )) then
     skip = .true.
   else
     skip = .false.
   end if

   if(debug) then 
    write (logf,*)trim(l_here),'-d: l_str >',trim(l_str),'<'
    write (logf,*)trim(l_here),'-d: ind ',ind
    write (logf,*)trim(l_here),'-d: cur # ',cur,' cur_char >',cur_char,'<'
    write (logf,*)trim(l_here),'-d: escape ',escape
    write (logf,*)trim(l_here),'-d: skip   ',skip  
   end if

! insert before charac
   if(cur >  1 .and. cur_char == charac) then 
    if(.not.escape .and. l_str(cur-1:cur-1) .ne. ' ') then

     if(skip) then 
       if (debug) write (logf,*)trim(l_here),'-d: skipped '
     else

     sout = l_str(1:cur-1)//' '//l_str(cur:)
     len_s = len_s + 1
     if(debug) then 
       write (logf,*)trim(l_here),'-d: len_s=',len_s,' len(string)=',len(string)
     end if

     if( len_s .ge. len(string) ) then
       write (logf,*)trim(l_here),'-e: expanded string length=',len_s, &
                           ' string length=',len(string)
       message = 'in routines '*routines_321+' truncated string >'+sout+'<'
       call require (message, len_s < len(string))
     end if

     cur  = cur + 1

     end if ! + -
    end if
   end if

   l_str = sout

  end do

  if(debug) then 
    write (logf,*)trim(l_here),'-d: sout >',trim(sout),'<'
    write (logf,*)trim(l_here),'-d: '
  end if

  result = sout

  call exit (l_here,5)

  end function string_before_char_insert_blank

! ==================================================================================
  subroutine string_array_char_isolate (sarray, charac)
! ----------------------------------------------------------------------------------
!
! Name             : string_array_char_isolate 
!
! Argument (inout) : sarray (:)
! Argument (in)    : charac
!
! Description      : surround char with blanks
! Description      : changes "xchary" into "x char y" for all elements of
! Description      : the array-string "sarray"
!
! Authors:F.Colonna
! Date   :29 Oct 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)              :: sarray(:)      
  character(len=1)              :: charac         

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: l, sizarr
  character(len=len(sarray))    :: l_string
  character(len=len(sarray))    :: string 

! begin:
  l_here ='string_array_char_isolate'
  call enter (l_here,5)

  sizarr = size(sarray)
  if(debug) then
    write (logf,*)trim(l_here),'-d: charac >',charac,'<'   
    write (logf,*)trim(l_here),'-d: array size=',sizarr   
  end if

  if(sizarr .gt. 0) then
   do l = 1, sizarr   
    l_string   = trim(sarray(l))//blanks

    if(index(l_string, charac) /= 0)  then 
! beging escape character ` and end escape character '
     string    = string_before_char_insert_blank (l_string, charac,"`", "'")
     l_string  = trim(string)//blanks
     string    = string_after_char_insert_blank (l_string, charac,"`", "'")
     sarray(l) = string   

     if(debug) then
       write (logf,*)trim(routines_21),'-d: null_char at =', &
               string_null_char_at (string)
		 end if
    end if

   end do
  end if

  call exit (l_here,5)

  end subroutine string_array_char_isolate

! ==================================================================================
  function string_split (string, separator, string_words_arg, len_string_words) result (result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_split
! 
! Argument (in)  : string               input string to be splitted
! Argument (in)  : separator            character used to split the string
! Argument (in)  : len_string_words     length of string_words
! Argument (out) : string_words_arg     allocated in function
! Argument (out) : string_words_arg     declared in strings_mod
!
! Result         : number_of_words 
!
! Description    : returns the number of words obtained by splitting "string"
! Description    : with character "separator"
! Description    : returns the array string_words (declared in module strins_mod) 
! Description    : obtained by splitting "string"
! Description    : returns "string" if number of words is zero or one
!
! Authors:F.Colonna
! Date  :04 Oct 2000
! Revision: R. Assaraf
! Revision: 26 Apr 2006  argument len_string_words to overcome portland bug
! ----------------------------------------------------------------------------------
  implicit none

! i/o :
  integer(dpi),     intent(in)        :: len_string_words
  character(len=*), intent(in)        :: string
  character(len=1), intent(in)        :: separator
  character(len=len_string_words), pointer           :: string_words_arg(:)

! result:
  integer(dpi)                        :: result

! local:
  character(len=max_string_len)       :: l_here
  integer(dpi)                        :: word_max_len
  integer(dpi)                        :: number_of_words
  integer(dpi)                        :: i

! begin:
  l_here ='string_split'
  call enter (l_here,3)
    if (len(string)==0) call die(l_here, "len(string)=0")
    if (debug) write (logf,*) trim (l_here),"-d: string=", trim(string)
   
! empty line:
  number_of_words = string_to_words_count (string ,separator, word_max_len)

   if(number_of_words .eq. 0 .or. number_of_words .eq. 1) then
    if(warning_level .gt. 5) then
       message = 'number_of_words ='+number_of_words
       call warning (l_here, message)
    end if

   if(associated(string_words_arg)) then
    deallocate (string_words_arg)
    if(print_level.gt.3) write (logf,*)l_here,': array "string_words_arg" deallocated'
   end if

   allocate (string_words_arg(1), stat = all_err)
   if(all_err /= 0) &
   call die (l_here, 'allocate(string_words_arg(1))')
   string_words_arg(1) = string

  else

   if(debug) write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words

   if(associated(string_words_arg)) then
    deallocate (string_words_arg)
    if(print_level.gt.5)write (logf,*)trim(l_here),': array "string_words_arg" deallocated'
   end if

   allocate (string_words_arg(number_of_words), stat = all_err)
   if(all_err /= 0) &
   call die (l_here, 'allocate(string_words_arg(number_of_words))')

   do i = 1, number_of_words
     string_words_arg(i) = repeat (' ',len(string_words_arg))
   end do
   
   if (debug) then
      write(logf,*) trim(l_here),"-d: size(string_words_arg)=",size(string_words_arg)
      write(logf,*) trim(l_here),"-d: number_of_words=",number_of_words
      write(logf,*) trim(l_here),"-d: len(string_words_arg)=",len(string_words_arg)
   endif

   call string_get_words (string, separator, string_words_arg, len_string_words)
 
   if (number_of_words /= size(string_words_arg)) call die (l_here,"size of string_words_arg="+size(string_words_arg))

   if(debug) then 
     do i = 1, number_of_words
      write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(string_words_arg(i)),'<'
     end do
   end if

  end if

  result = number_of_words

  call exit (l_here,3)

  end function string_split

! ==================================================================================
  subroutine string_get_words (string, separator, words, len_words)
! ----------------------------------------------------------------------------------
!
! Name           : string_get_words  
!
! Description    : returns the number of words obtained by splitting "string"
! Description    : with character "separator"
! 
! Argument (in)  : string          input string to be split
! Argument (in)  : separator       character used to split string
! Argument (inout) : words         stores each word 
! Argument (in)  : len_words       necessary 
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! Revision: 26 Apr 2006
! Revision: adding argument len_words to overcome portland bug
! ----------------------------------------------------------------------------------
  implicit none

! i/o :
  integer(dpi), intent(in)     :: len_words
  character(len=*), intent(in) :: string
  character(len=1), intent(in) :: separator

  character(len=len_words), intent(inout) :: words(:)

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)   :: number_of_words

  integer(dpi) :: lenstr, indf, indl, ind2
  integer(dpi) :: NF
  character(len=max_string_len) :: temp, copy
  character(len=1)  :: pad
  character(len=2)  :: char2
  logical(bool) :: found, maxl, maxw

! begin:
  l_here = 'string_get_words'
  call enter (l_here,3)
  
  number_of_words = size (words)

  if(number_of_words == 0) then
    message ='in routines '+routines_21+' number_of_words ='+ &
    trim(integer_to_string(number_of_words))
    call require (message, number_of_words .gt. 0)
  end if

  lenstr = len_trim(string)     
  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
  if(debug) write (logf,*)trim(l_here),'-d: separator >',separator,'<'
  if(debug) write (logf,*)trim(l_here),'-d: number_of_words=',number_of_words
  if(debug) write (logf,*)trim(l_here),'-d: lenstr=',lenstr
  if(debug) write (logf,*)trim(l_here),'-d: len(words)=',len(words)
  pad = ' '
  if(separator.eq.' ') pad ='.'

  if(lenstr.eq.0) then
!  write(6,'(a)')' warning_cut_string_in_words> string zero length '
   NF = 0
   if(number_of_words .ne. 0) then
    words(1) = string
  call exit (l_here,3)
    return
   else
    call die (l_here, 'number_of_words .eq. 0')
   end if
  end if

  if(index(string, separator).eq.0) then
   words(1) = string
   NF = 1
   if(number_of_words .lt. 1) then
    write (logf,'(2a,i5,a,i5,3a)') trim(l_here), &
    ': number of words found =', &
    NF,' instead of ',number_of_words,' with delimiter "',separator,'"'
    call die (l_here, 'number_of_words .ne. 1')
   end if
  call exit (l_here,3)
   return
  end if

! delete double occurences of separator:
  char2 = separator//separator
  copy = string
  ind2 = index(copy(1:lenstr),char2)
  found = ind2.ne.0
  do while(found)

   temp = copy (1:ind2)//copy(ind2+2:)
   copy = temp
   lenstr = lenstr -1

   ind2 = index(copy(1:lenstr),separator//separator)
   found = ind2.ne.0
  end do
  
! delete occurence in first position:
  if(copy(1:1).eq.separator) then
   temp = copy(2:lenstr)
   copy = temp(1:lenstr-1)
   lenstr = lenstr-1
  end if
  if (debug) write (logf,*) "-d: string without double occurences of separator copy >", trim(copy),"<"
  NF    = 0
  indf  = 0
  found = .true.
  maxw  = .false.
  maxl  = .false.
  temp  = copy
  do while (found.and..not.maxw.and..not.maxl)
   NF   = NF +1
   indl = index(temp(1:lenstr),separator)
   if (debug) write (logf,*) trim(l_here),"-d: NF=",NF, "indl=",indl,"indf=",indf
   if(indf+1.le.indl-1) then
  
    words(NF) = copy(indf+1:indl-1)
   if (debug) write (logf,*) trim(l_here), "-d: copy=>",trim(copy(indf+1:indl-1)),"<"
   if (debug) write (logf,*) trim(l_here), "-d: words(NF)=>",trim(words(NF)),"<"
    temp (indl:indl) = pad
    indf       = indl
   else  !. end of string
    words(NF) = copy(indf+1:)
   end if
   found = indl .gt.0
   maxl  = indl .eq. lenstr
   maxw   = NF .gt. number_of_words
  end do
   if (debug) write (logf,*) trim(l_here),"-d: words(1) >",words(1),"<"
  if(maxw) then
   write (logf,'(2a,i5,a,i5,3a)') trim(l_here), &
   ': number of words found =',NF,' instead of ',number_of_words, &
   ' with delimiter "',separator,'"'
    call die (l_here, ' NF > number_of_words ')
  end if

  call exit (l_here,3)

  end subroutine string_get_words

! ==================================================================================
  function string_to_words_count (string, separator, word_max_len) &
                          result (number_of_words)
! ----------------------------------------------------------------------------------
!
! Name           : function string_to_words_count 
! Result         : number_of_words
!
! Description    : returns the number of words obtained by splitting "string"
! Description    : with character "separator" 
! Description    : if zero length string returns 0
! Description    : if separator not in string returns 1 
!
! Argument (in)  : string 
! Argument (in)  : separator   
! Argument (out) : word_max_len maximum length of all words           
!
! Author : F.Colonna
! Date   : 21 Sep 2000, 19 Nov 2000
! ----------------------------------------------------------------------------------
  implicit none

! result:
  integer(dpi)                 :: number_of_words

! i/o :
  character(len=*), intent(in) :: string
  character(len=1), intent(in) :: separator
  integer(dpi),    intent(out) :: word_max_len

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: lenstr, indf, indl, ind2, NF
  character(len=max_string_len) :: temp, copy
  character(len=1)              :: pad
  character(len=2)              :: char2
  logical(bool)                 :: found, maxl

! begin:
  l_here ='string_to_words_count'
  call enter (l_here,5)

  if(debug) then
   write (logf,*) trim(l_here),'-d: string >',trim(string),'<'   
   write (logf,*) trim(l_here),'-d: separator >',trim(separator),'<'   
  end if

  word_max_len = 0
  lenstr = len_trim(string)     

  pad = ' '
  if(separator.eq.' ') pad ='.'

  if(lenstr.eq.0) then
   if(warning_level .gt. 0) &
   write (logf,'(2a)')trim(l_here),'-warning: string zero length '
   number_of_words = 0
  call exit (l_here,5)
   return
  end if

  if(index(string, separator).eq.0) then
   number_of_words = 1
  call exit (l_here,5)
   return
  end if

! delete double occurences of separator:
  char2 = separator//separator
  copy = string
  ind2 = index(copy(1:lenstr),char2)
  found = ind2.ne.0
  do while(found)

   temp = copy (1:ind2)//copy(ind2+2:)
   copy = temp
   lenstr = lenstr -1

   ind2 = index(copy(1:lenstr),separator//separator)
   found = ind2.ne.0
  end do

! delete occurence in first position:
  if(copy(1:1).eq.separator) then
   temp = copy(2:lenstr)
   copy = temp(1:lenstr-1)
   lenstr = lenstr-1
  end if

  NF    = 0
  indf  = 0
  found = .true.
  maxl  = .false.
  temp  = copy

  do while (found.and..not.maxl)
   NF   = NF +1
   indl = index(temp(1:lenstr),separator)

   if(indf+1.le.indl-1) then
    word_max_len = max(indl - indf - 1,word_max_len)
    temp (indl:indl) = pad
    indf       = indl
    else  !. end of string
     word_max_len = max(lenstr - indf,word_max_len)
    end if

    found = indl .gt.0
    maxl  = indl .eq. lenstr

  end do

  if(nf <= 0 ) then
    message ='in routines '+routines_21+' NF ='+NF
    call ensure (message, NF .gt. 0)
  end if

  number_of_words = NF

  if(debug) then
   write (logf,*)trim(l_here),'-d: NF ',NF
   write (logf,*)trim(l_here),'-d: word_max_len=',word_max_len
  end if

  call exit (l_here,5)

  end function string_to_words_count

! ==================================================================================
  subroutine string_to_words (string, separator, words, words_lengths, &
                                  number_of_words, len_string, len_words)
! ----------------------------------------------------------------------------------
!
! Name           : string_to_words 
!
! Argument (in)  : string          input string to be split
! Argument (in)  : separator       character used to split string
! Argument (out) : words           stores each word (already allocated)
! Argument (out) : words_lengths   stores length of each word
! Argument (in)  : number_of_words number of words produced
!
! Authors:F.Colonna
! Date  :21 Sep 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o :
  integer(dpi)                 :: len_string, len_words
  character(len=len_string), intent(in) :: string
  character(len=1), intent(in) :: separator

  character(len=len_words), intent(out) :: words(:)
  integer(dpi),     intent(out) :: words_lengths(:)
  integer(dpi),     intent(in)  :: number_of_words

! local:
  character(len=max_string_len) :: l_here
  integer(dpi) :: lenstr, indf, indl, ind2, i
  integer(dpi) :: NF
  character(len=max_string_len) :: temp, copy
  character(len=1)  :: pad
  character(len=2)  :: char2
  logical(bool) :: found, maxl, maxw

! begin:
  l_here = 'string_to_words'
  call enter (l_here,3)
  if (len(words)==0) call die(l_here, "len(words)=0")
  if(debug) write (logf,*)trim(l_here),'-d: string >',trim(string),'<'
  lenstr = len_trim(string)     

  pad = ' '
  if(separator.eq.' ') pad ='.'

  if(lenstr.eq.0) then
!  write(6,'(a)')' warning_cut_string_in_words> string zero length '
   NF = 0
   if(number_of_words .ne. 0) then
    write (logf,'(2a,i5,a,i5,2a)') trim(l_here), &
    ': number of words found =', &
    NF,' instead of ',number_of_words,' with delimiter "',separator,'"'
    call die (l_here, 'number_of_words .ne. 0')
    end if
  call exit (l_here,3)
    return
  end if

  if(index(string, separator).eq.0) then
   words(1) = string
   NF = 1
   if(number_of_words .ne. 1) then
    write (logf,'(2a,i5,a,i5,2a)') trim(l_here), &
    ': number of words found =', &
    NF,' instead of ',number_of_words,' with delimiter "',separator,'"'
    call die (l_here, 'number_of_words .ne. 1')
   end if
  call exit (l_here,3)
   return
  end if

! delete double occurences of separator:
  char2 = separator//separator
  copy = string
  ind2 = index(copy(1:lenstr),char2)
  found = ind2.ne.0
  do while(found)

   temp = copy (1:ind2)//copy(ind2+2:)
   copy = temp
   lenstr = lenstr -1

   ind2 = index(copy(1:lenstr),separator//separator)
   found = ind2.ne.0
  end do

! delete occurence in first position:
  if(copy(1:1).eq.separator) then
   temp = copy(2:lenstr)
   copy = temp(1:lenstr-1)
   lenstr = lenstr-1
  end if

  NF    = 0
  indf  = 0
  found = .true.
  maxw  = .false.
  maxl  = .false.
  temp  = copy
  do while (found.and..not.maxw.and..not.maxl)
   NF   = NF +1
   indl = index(temp(1:lenstr),separator)

   if(indf+1.le.indl-1) then
    words(NF) = copy(indf+1:indl-1)
    words_lengths(NF) = indl - indf - 1
    temp (indl:indl) = pad
    indf       = indl
   else  !. end of string
    words(NF) = copy(indf+1:)
    words_lengths(NF) = lenstr - indf
   end if
   found = indl .gt.0
   maxl  = indl .eq. lenstr
   maxw   = NF .eq. number_of_words
  end do

  if(.not.maxw) then
   write (logf,'(2a,i5,a,i5,2a)') trim(l_here), &
   ': number of words found =',NF,' instead of ',number_of_words, &
   ' with delimiter "',separator,'"'
  end if
  if (debug) write(logf,*) trim(l_here), "-d: len(words)=",len(words)
  if (debug) write(logf,*) trim(l_here), "-d: words", (">"//trim(words(i))//"< ",i=1, size(words))
  call exit (l_here,3)

  end subroutine string_to_words

! ==================================================================================
  subroutine split_string  &
               (string, separator, words, words_lengths, number_of_words,len_words)
! ----------------------------------------------------------------------------------
!
! Name           : split_string 
!
! Argument (in ) : string
! Argument (in ) : separator
! Argument (in ) : words
! Argument (in ) : words_lengths
! Argument (in ) : number_of_words
!
! Description    : string          input string to be split
! Description    : separator          character used to split string
! Description    : words           stores each word 
! Description    : words_lengths   stores length of each word
! Description    : number_of_words number of words produced
!
! Description    : encapsulates string_to_words
! Description    : allocate (words(number_of_words))
! Description    : allocate (words_lengths(number_of_words))
! 
! Authors:F.Colonna
! Date  :04 Oct 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o :
  integer(dpi)                 :: len_words
  character(len=*), intent(in) :: string
  character(len=1), intent(in) :: separator
  character(len=len_words), pointer    :: words(:) 
  integer(dpi), pointer        :: words_lengths(:) 
  integer(dpi), intent(in)     :: number_of_words

! local:
  character(len=max_string_len) :: l_here

  integer(dpi)                               :: i

! begin:
  l_here ='split_string'
  call enter (l_here,3)

! empty line:
  if (len(string)==0) call die (l_here, "len(string)=0")
  if(number_of_words .eq. 0) then
   message = 'number_of_words == 0. give a max for allocation'
   call warning (l_here, message)

  else
    if(debug) then 
     write (logf,*)trim(l_here), &
           ': number_of_words=',number_of_words
     write (logf,*)trim(l_here), &
           ': len(words)     =',len(words)      

    end if
    if(associated(words)) deallocate (words)
    allocate (words(number_of_words), stat = all_err)
    if(all_err /= 0) &
    call die (l_here, 'allocate(words(number_of_words))')

    do i = 1, number_of_words
     words(i) = repeat (' ',len(words))
    end do

    if(associated(words_lengths)) deallocate (words_lengths)
    allocate (words_lengths(number_of_words), stat = all_err)
    if(all_err /= 0) &
    call die (l_here, 'allocate (words_lengths(number_of_words))')
    if (debug) write(logf,*) trim(l_here),'len(string)=',len(string)
    call string_to_words &
         (string, separator, words, words_lengths, number_of_words, len(string)+0, len(words)+0)

    if(debug) then 
     do i = 1, number_of_words
      write (logf,*)trim(l_here),'-d: word # ',i,' >',trim(words(i)),'<'
     end do
    end if

  end if

  call exit (l_here, 3)

  end subroutine split_string

! ==================================================================================
  function string_append (this_str, addition) result(result) 
! ----------------------------------------------------------------------------------
!
! Name           : string_append
!
! Argument (in ) : this_str
! Argument (in ) : addition
!
! Description    : append string addition to this_str
!
! Authors:F.Colonna
! Date  :29 Oct 2000
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)              :: this_str, addition 

! result:
  character(len=max_string_len) :: result

! local:
  character(len=max_string_len) :: l_here
  character(len=max_string_len) :: string 

! begin:
  l_here ='string_append'
  call enter (l_here,3)

  if(debug)write (logf,*)trim(routines_21),'-d: input string >',trim(this_str),'<'
  string = trim(this_str)//addition
  result = string
  
  if(debug)write (logf,*)trim(routines_21),'-d: added  string >',trim(addition),'<'
  if(debug)write (logf,*)trim(routines_21),'-d: output string >',trim(string),'<'

  if(len_trim(string) > max_string_len) then
    message = 'in string_append. resulting string :'+CR+' >' &
    +string+'<'+CR+' exceeds max_string_len='+ &
    trim(integer_to_string(max_string_len))
    call ensure (message, len_trim(string) <= max_string_len)
  end if
     
  call exit (l_here,3)

  end function string_append

! ==========================================================================
  subroutine array_regular_section_decode (string, name, lb, up, step)
! --------------------------------------------------------------------------
!
! Name           : string_to_unknown_sub_array
!
! Argument (in)  : string
! Argument (out) : name (array_name)
! Argument (out) : lb (:)  (lbound, lower bounds) 
! Argument (out) : up (:)  (ubound, upper bounds)
! Argument (out) : step (:)
!
! Description    : decodes  an array regular section
! Description    : example:
! Description    : string='A(1:5:2 , 3:10 , *:* , 4)' 
! Description    : gives: 
! Description    : name='A'.
! Description    : lb=(1,2,-HUGE,4) (HUGE is  convention for *).
! Description    : up=(5,3,+HUGE,4)
! Description    : step=(2,1,1,1)
! Description    : if string reduces to 'A' and size(step)=size(lb)=size(up)=3
! Description    : lb=(-HUGE,-HUGE,-HUGE)
! Description    : up=(HUGE,HUGE,HUGE)
! Description    : step=(1,1,1)
!
! Authors: R. Assaraf
! Date   : 9 Jan 2000 
! --------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)    :: string
  character(len=*), intent(inout) :: name
  integer(dpi), intent(out)       :: lb(:)
  integer(dpi), intent(out)       :: up(:)
  integer(dpi), intent(out)       :: step(:)

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)          :: tail
  integer(dpi)                           :: dim_i,dimensions_nb,n_tens
  character(len=max_string_len), pointer :: string_dimensions(:) 
  integer(dpi)                           :: ub_field,n_sub_fields
  character(len=max_string_len), pointer :: sub_field(:)

! begin:
  l_here ='array_regular_section_decode'
  call enter (l_here, 5)

  nullify (string_dimensions)
  nullify (sub_field)

  name = array_name_get (string)
  tail = array_regular_section_get (string)

  n_tens = size(lb)
  
  message='in subroutine '*l_here*'size(up)=' *&
          encode(size(up))*'must be equal to '*n_tens
  message='in subroutine '*l_here*'size(step)=' *&
          encode(size(step))*'must be equal to '*n_tens

  dimensions_nb = string_split(tail,',',string_dimensions, len (string_dimensions))

 IF  (dimensions_nb== 0) then
       lb = -HUGE(1)
       up = HUGE(1)
       step = 1

 ELSE

   if(debug) then
    do dim_i = 1,dimensions_nb
      write (logf,*)trim(l_here)//': string_dimensions(',dim_i,')',&
            trim(string_dimensions(dim_i))
     enddo
   endif

   if(dimensions_nb /= n_tens) then
    message='in subroutine '*l_here*' dimensions_nb='&
           *dimensions_nb*' n_tens='*n_tens&
           *' should be equal'*' syntax error in '*string
    call require(message,dimensions_nb== n_tens)
   end if
 
    lb = -HUGE(1)
    up = HUGE(1)
    step = 1

    do dim_i = 1,dimensions_nb
      n_sub_fields = string_split(string_dimensions(dim_i),':',sub_field, len(sub_field))
     
      select case(n_sub_fields)
         case(1)
           if (integer_is(sub_field(1))) call decode(lb(dim_i),sub_field(1))
           up(dim_i) = lb(dim_i)
           step(dim_i) = 1 
         case(2)		
           if (integer_is(sub_field(1))) call decode(lb(dim_i),sub_field(1))
           if (integer_is(sub_field(2))) call decode(up(dim_i),sub_field(2))
           step(dim_i) = 1
         case(3)
           if (integer_is(sub_field(1))) call decode(lb(dim_i),sub_field(1))
           if (integer_is(sub_field(2))) call decode(up(dim_i),sub_field(2))
           call decode(step(dim_i),sub_field(3))
       end select
      
     if(debug) then
       write (logf,*)'in '//trim(l_here)//' -d: '
       write (logf,*)'idim=',dim_i,'n_sub_fields=',n_sub_fields
       do ub_field = 1,n_sub_fields
          write (logf,*)'sub_field(',ub_field,')=',trim(sub_field(ub_field))
       enddo ! do ub_field 
       write (logf,*)'step(',dim_i,')',step(dim_i)
       write (logf,*)'lb=',lb,'up=',up,'step=',step
     endif 
 
    enddo ! do dim_i 
 
 ENDIF

 call exit (l_here, 5)

  end subroutine array_regular_section_decode 

! ==========================================================================
  function number_of_diff_strings (list_of_strings) 
! --------------------------------------------------------------------------
!
! Name           : number_of_diff_strings
!
! Argument (in)  : list_of_strings
! Argument (out) : number_of_diff_strings
!
! Description    : computes the number of different strings in 
! Description    : the array list_of_strings
!
! Authors: R. Assaraf
! Date   : 28 Jan 2000 
! --------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)     :: list_of_strings(:)
  integer(dpi)         :: number_of_diff_strings

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)  :: n_strings,i_pot_new,i_old,i
  logical(bool) :: new_test

! begin:
  l_here ='number_of_diff_strings'
  call enter (l_here)

  n_strings = size(list_of_strings)
 
  if(n_strings == 0) then
    message='in function '*l_here*'size(list_of_strings)='&
            *n_strings*'must be > 0'
    call require(message,n_strings>0)
  end if
  
  if (debug) then
    write (logf,*)'in ',trim(l_here),' -d:'
    write (logf,*)'n_strings in list_of_strings=',n_strings

    do i = 1,n_strings
      write (logf,*)'list_of_strings(i)=',trim(list_of_strings(i))
    enddo ! do i
  endif

 number_of_diff_strings = 1 ! list_of_strings(1) is a new string

 if (n_strings>1) then
  do i_pot_new = 2,n_strings 
    new_test = .true.      
    do i_old = 1,i_pot_new-1
     if (list_of_strings(i_old)== list_of_strings(i_pot_new))  new_test = .false.
    enddo ! do i_old
    if (new_test) number_of_diff_strings = number_of_diff_strings+1
  enddo ! do i_pot_new
 endif
 
  call exit (l_here)

 end function number_of_diff_strings

! ==========================================================================
  function array_name_get (string)  result(result) 
! --------------------------------------------------------------------------
!
! Name           : array_name_get
!
! Argument (in)  : string
! Argument (out) : result
!
! Description: If string ends with tail=`(...)', array_name_get
! Description: is  string without  tail.
!
! Authors: R. Assaraf
! Date   : 27 Jan 2001
! --------------------------------------------------------------------------
  implicit none

! i/o: 
  character(len=*)               :: string

! result:
  character(len=len(string))     :: result 

! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                  :: pos_last_parenthesis_left  
  integer(dpi)                  :: length

! begin:
  l_here ='array_name_get'
  call enter (l_here,10)

  length = len_trim(string)
  if (string(length:length)==')') then

     pos_last_parenthesis_left= index(string,'(',back = .true.)
 
     if(pos_last_parenthesis_left == 0) then
       message='in function '*l_here*'a left parenthesis should'&
       *'correspond to the right parenthesis of '*string
       call require (message, pos_last_parenthesis_left /= 0)
     end if

     result = string(1:pos_last_parenthesis_left-1)

  else
     result = string
  endif

  result = adjustl(result)

  if (debug) write (logf,*)trim(l_here),' -d: ','result(',trim(string),')=',&
            trim(result)

  call exit (l_here,10)

 end function array_name_get

! ==========================================================================
  function array_regular_section_get (string) result(result) 
! --------------------------------------------------------------------------
!
! Name           : array_regular_section_get
!
! Argument (in)  : string
!
! Description    : If string ends with `(...)', array_regular_section_get is
! Description    : the substring between the parenthesis.
! Description    : If not, array_regular_section_get=`  '
!
! Authors: R. Assaraf
! Date   : 27 Jan
! --------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)               :: string

! result:
  character(len=len(string))     :: result
 
! local:
  character(len=max_string_len_routine_name) :: l_here
  integer(dpi)                  :: pos_last_parenthesis_left  
  integer(dpi)                  :: length

! begin:
  l_here ='array_regular_section_get'
  call enter (l_here,10)

  length = len_trim(string)
  if (string(length:length)==')') then

     pos_last_parenthesis_left= index(string,'(',back = .true.)
 
     if(pos_last_parenthesis_left == 0) then
       message='in function '*l_here*'a left parenthesis should'&
       *'correspond to the right parenthesis of '*string
       call require (message, pos_last_parenthesis_left /= 0)
     end if

     result = string(pos_last_parenthesis_left+1:length-1)

  else
     result = blanks
  endif
 
     result = trim(result)
 
 if (debug) write (logf,*)trim(l_here),' -d: ','result(',trim(string),')=',&
            trim(result)

  call exit (l_here,10)

 end function array_regular_section_get

! ==================================================================================
  subroutine enter (r_name, trace_lev)
! ----------------------------------------------------------------------------------
!
! Name           : subroutine enter
!
! Argument (in ) : r_name    -- current routine name (l_here)
! Argument (in ) : trace_lev -- trace level for tracing message emission
!
! Description    : gets current routine name stored in "l_here" string 
! Description    : current routine name i.e. l_here is stacked in:
! Description    : routine_stack_names (routine_stack_level) 
! Description    : manage routines stack  
! Description    : displays CPU, allows debugging, tracing, displaying 
! Description    : currently called routines
!
! Authors:F. Colonna
! Date  :24 Oct 2000
! Revision : 04 Dec 2000. FCC: take out 'encode' because 
! Revision : 04 Dec 2000. FCC: of false recursivity
! Revision : 13 Aug 2002. FCC: r_name == NOT_INIT_S .or. ichar(r_name(1:1)) == 0
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)       :: r_name
  integer(dpi), optional :: trace_lev

! local:
  character(len=max_string_len_routine_name) :: l_here
  logical(bool)                              :: l_debug
  character(len=max_string_len)              :: l_message
  character(len=max_string_len)              :: l_str
  character(len=max_string_len)              :: string 
  integer(dpi)                               :: l_trace
  integer(dpi)                               :: rtn_i
 
! begin:
  l_here='enter'
  l_debug = .true.
  l_debug = .false.
  l_trace = 0
  if(present(trace_lev)) l_trace = trace_lev

  if(l_debug) then
    write (logf,*)trim(l_here),'-d: '
    write (logf,*)trim(l_here),'-d: entering enter'
    write (logf,*)trim(l_here),'-d: routine >',trim(r_name),'<'
    write (logf,*)trim(l_here),'-d: routine_stack_level=',routine_stack_level
    write (logf,*)trim(l_here),'-d: l_trace=',l_trace,' trace_level=',trace_level
  end if

  routine_stack_level = routine_stack_level+1

  if(routine_stack_level >= routine_max_number) then

! leave this. do NOT use encode l_here >= infinite loop
    do rtn_i = 1, routine_max_number
      write (logf,*)'enter-e: routine # ',rtn_i,' >',trim(routine_stack_names(rtn_i)),'<'
    end do

    write(message,'(3a,i3,a,i3,2a)') &
    'in routine',trim(routines_21),' routine_stack_level =',       &
    routine_stack_level,' exceeds max number =',routine_max_number, &
    ' probably "call exit" is missing somewhere ', &
    ' run "perl check_exit.pl *.f90" on all fortran files'

    write (logf,*)' >>',trim(message),'<<'
    stop ' routine_stack_level .gt. routine_max_number'

  end if

  routine_stack_names(routine_stack_level) = trim(r_name)

  routines_1 = r_name
  routines_2 = blanks 
  routines_3 = blanks 
  if(routine_stack_level .gt. 1) &
  routines_2 = routine_stack_names(routine_stack_level-1)
  if(routine_stack_level .gt. 2) &
  routines_3 = routine_stack_names(routine_stack_level-2)
  routines_21 = trim(routines_2)//'-'//routines_1
  routines_32 = trim(routines_3)//'-'//routines_2
  routines_321 = trim(routines_3)//'-'//routines_21

! check that current routine has got a name. 
! i.e. is defined in file routines_catalog.inc

  if (r_name == NOT_INIT_S .or. ichar(r_name(1:1)) == 0) then
    write (logf,*)trim(l_here),'-e: current routine has no name '
    write (logf,*)trim(l_here),'-e: was called by >',trim(routines_2), &
   '< is not set in file routines_catalog.inc . Or l_here not set'
    call die (l_here,'current routine not set in routine routines_catalog_init')
  end if

 if(l_debug) then
  write (logf,*)trim(l_here),'-d: routine_stack_level=',routine_stack_level
  write (logf,*)trim(l_here),'-d: current-routine >',trim(r_name),'<'
  write (logf,*)trim(l_here),'-d: debug_routines_list >',trim(debug_routines_list),'<'
  write (logf,*)trim(l_here),'-d: r_name >',trim(r_name),'<'
  write (logf,*)trim(l_here),'-d: routines_1      >',trim(routines_1),'<'
  write (logf,*)trim(l_here),'-d: routines_2      >',trim(routines_2),'<'
  write (logf,*)trim(l_here),'-d: routines_3      >',trim(routines_3),'<'
  write (logf,*)trim(l_here),'-d: routines_21     >',trim(routines_21),'<'
  write (logf,*)trim(l_here),'-d: routines_32     >',trim(routines_32),'<'
  write (logf,*)trim(l_here),'-d: routines_321    >',trim(routines_321),'<'
 end if

  cpu   = (index (cpu_routines_list, 'all') .ne. 0 .or. &
          index (cpu_routines_list, trim(r_name)) .ne. 0) 
          
  debug = (index (debug_routines_list, 'all') .ne. 0 .or.  &
          index (debug_routines_list, trim(r_name)) .ne. 0)

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: cpu=',cpu 
   write (logf,*)trim(l_here),'-d: debug=',debug
  end if

  l_message = points(1:routine_stack_level)//' entering '//trim(r_name)

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: l_message >',trim(l_message),'<'
  end if

  if(cpu) then 
   call cpu_now 
   cpu_start_routines(routine_stack_level) = actual_cpu
   string = cpu_to_string (actual_cpu)
   l_str = trim(l_message)//' cpu '//trim(string)
   l_message = l_str
  end if

! if(cpu .or. routine_stack_level <= trace_level) write (logf,'(1x,3a)') trim (l_message)
  if(cpu .or. l_trace < trace_level) write (logf,'(1x,3a)') trim (l_message)

!if(l_debug) then
! write (logf,*)trim(l_here),'-d: routine_stack_level=',routine_stack_level
! write (logf,*)trim(l_here),'-d: entering routine >',trim(r_name),'<'
!end if

  end subroutine enter

! ==================================================================================
  subroutine exit (r_name, trace_lev)
! ----------------------------------------------------------------------------------
!
! Name           : exit
!
! Argument (in ) : trace_lev -- trace level for tracing message emission
!
! Description    : re-set current routine name stored in "r_name" string 
! Description    : un-stacks: r_name = routine_stack_names(routine_stack_level) 
! Description    : re-set CPU, debug
! Description    : emits message at routine end
!
! Authors:F. Colonna
! Date  :11 Oct 2000
! Revision :02 Avr 2001 c_routine, p_routine
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*)                           :: r_name
  integer(dpi), optional                     :: trace_lev

! local:
  real(sp)                                   :: top_cpu, end_cpu, dif_cpu
  integer(dpi)                               :: r
  integer(dpi)                               :: l_trace
  character(len=max_string_len_routine_name) :: l_here, c_routine, p_routine
  character(len=max_string_len)              :: l_message
  character(len=max_string_len)              :: l_str
  character(len=max_string_len)              :: string 
  logical(bool)                              :: l_debug

! begin:
  l_here ='exit'
  l_debug = .true.
  l_debug = .false.

  c_routine = r_name
  l_trace = 0
  if(present(trace_lev)) l_trace = trace_lev

  cpu = (index (cpu_routines_list, 'all') .ne. 0 .or. &
        index (cpu_routines_list, c_routine) .ne. 0) 

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: '
   if(present(trace_lev)) write (logf,*)trim(l_here),'-d: trace_lev=',trace_lev
   write (logf,*)trim(l_here),'-d: entering exit'
   write (logf,*)trim(l_here),'-d: routine >',trim(c_routine),'<'
   write (logf,*)trim(l_here),'-d: routine_stack_level=',routine_stack_level
  end if

! rare but may occur:
  if(routine_stack_level == 0) then
   write (logf,*)trim(l_here),':error routine_stack_level= 0'
   write (logf,*)trim(l_here),  &
   ': check that no "call enter (l_here)" is missing'
   stop 'exit: routine_stack_level = 0'
  end if

! check routine name
  if(trim(c_routine) /= trim(routine_stack_names(routine_stack_level))) then
   write (logf,*)trim(l_here),'-e: current routine name >',trim(c_routine),'< '
   write (logf,*)trim(l_here),'-e: differs from that indexed by routine_stack_level >', &
           trim(routine_stack_names(routine_stack_level)),'<'
   write (logf,*)trim(l_here),'-e: check that no "call enter" is missing in routine>', &
           trim(c_routine),'<'
   write (logf,*)trim(l_here),'-e: or that no "return" miss a "call enter" in routine>', &
           trim(routine_stack_names(routine_stack_level)),'<'
   write (logf,*)trim(l_here),'-e: or that (using fast) and extra  call exit is in skipping.inc'
   write (logf,*)trim(l_here),'-e: routines stack:'
   do r = 1, routine_stack_level
    write (logf,'(3a)') ' >',trim(routine_stack_names(r)),'<'
   end do
   write (logf,'(a)')' '

   stop 'exit-e: routine /= routine_stack_names(routine_stack_level)'
  end if

! erase current routine name from routine_stack_names stack
  routine_stack_names(routine_stack_level) = blanks
  l_message = points(1:routine_stack_level)//' exiting  '//trim(c_routine)

  if(cpu) then 
   call cpu_now  ! gives actual_cpu
   end_cpu = actual_cpu
   top_cpu = cpu_start_routines (routine_stack_level) 
   dif_cpu = end_cpu - top_cpu
   string  = cpu_to_string (dif_cpu)
   l_str   = trim(l_message)//' dcpu '//trim(string)
   l_message = l_str
  end if

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: l_message >',trim(l_message),'<'
   write (logf,*)trim(l_here),'-d: cpu =',cpu 
   write (logf,*)trim(l_here),'-d: l_trace =',l_trace
   write (logf,*)trim(l_here),'-d: trace_level =',trace_level
  end if

  if(cpu .or. l_trace < trace_level) write (logf,'(1x,3a)') trim (l_message)
! if(cpu .or. routine_stack_level <= trace_level) write (logf,'(1x,3a)') trim (l_message)

! un-stack routine names.
! set current routine to previous routine:

  routine_stack_level = routine_stack_level-1

  routines_1 = blanks
  routines_2 = blanks
  routines_3 = blanks
  if(routine_stack_level .ge. 1) &
   routines_1 = routine_stack_names(routine_stack_level)
   p_routine = routines_1
  if(routine_stack_level .ge. 2) &
   routines_2 = routine_stack_names(routine_stack_level-1)
  if(routine_stack_level .ge. 3) &
   routines_3 = routine_stack_names(routine_stack_level-2)

  routines_21  = routines_2+'-'+routines_1
  routines_32  = routines_3+'-'+routines_2
  routines_321 = routines_3+'-'+routines_21

! re-actualizes debug and cpu
  cpu   = (index (cpu_routines_list, 'all') .ne. 0 .or.  &
          index (cpu_routines_list,   trim(p_routine)) .ne. 0)
          
  debug = (index (debug_routines_list, 'all') .ne. 0 .or.  &
          index (debug_routines_list, trim(p_routine)) .ne. 0) 

  if(l_debug) then
   write (logf,*)trim(l_here),'-d: previous routine_stack_level=', &
                            routine_stack_level
   write (logf,*)trim(l_here),'-d: previous routine >',trim(p_routine),'<'
   write (logf,*)trim(l_here),'-d: debug_routines_list >',trim(debug_routines_list),'<'
   write (logf,*)trim(l_here),'-d: previous debug value =',debug  
  end if

  end subroutine exit

!========================================================================
  subroutine char_1_to_string (string, char_1_obj)
!------------------------------------------------------------------------
!
! Name           : char_1_to_string
!
! Argument (in ) : string
! Argument (in ) : char_1_obj
!
! Description    : converts char_1_obj to string 
!
! Authors: R. Assaraf
! Date  : 15 Aug 2002
! -------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=1)  :: char_1_obj (:)
  character(len=*)  :: string 

! local:
  character(len=max_string_len) :: l_here
  integer(dpi) :: i

! begin:
  l_here ='char_1_to_string'
  call enter (l_here,3)
  
  string = BLANKS
 
  do i = 1, min (size (char_1_obj), len (string))

    string (i:i) = char_1_obj (i)

  enddo ! do i
     
  call exit (l_here,3)

  end subroutine char_1_to_string

! ==================================================================================
  subroutine string_to_char_1 (char_1_obj, string)
! ----------------------------------------------------------------------------------
!
! Name           : char_1_to_string
!
! Argument (in ) : string
! Argument (in ) : char_1_obj
!
! Description    : converts string to char_1
!
! Authors: R. Assaraf
! Date  : 15 Aug 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=1)  :: char_1_obj (:)
  character(len=*)  :: string 

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                 :: i

! begin:
  l_here ='string_to_char_1'
  call enter (l_here,3)

  char_1_obj = BLANKS

  do i = 1, min (size (char_1_obj), len (string))

    char_1_obj (i) = string (i:i)

  enddo ! do i
     
  call exit (l_here,3)

  end subroutine string_to_char_1

! ==================================================================================
  subroutine string_character_code_print (string, string_name)
! ----------------------------------------------------------------------------------
!
! Name           : string_character_code_print 
!
! Argument (in ) : string
! Argument (in ) : string_name
!
! Description    : prints each character code of a string
!
! Authors:F.Colonna
! Date   :21 Dec 2002
! ----------------------------------------------------------------------------------
  implicit none

! i/o:
  character(len=*), intent(in)  :: string
  character(len=*), intent(in)  :: string_name

! local:
  character(len=max_string_len) :: l_here
  integer(dpi)                  :: chr_i

! begin:
  l_here ='string_character_code_print'
  call enter (l_here,5)

  if(debug) write (logf,*) trim(l_here),'-d: string >',trim(string),'<'

   do chr_i = 1, len(string)
     write (logf,*) trim(l_here),': ',trim(string_name),'(',chr_i,':',chr_i, &
                  ') >',string(chr_i:chr_i),' ascii code =',ichar(string(chr_i:chr_i))
   end do

  call exit (l_here,5)

  end subroutine string_character_code_print

end module strings_mod

