/**********************************************************************/
/* This routine gets strings containing system relevant information,  */
/* i.e. USER, HOME, PATH, SHELL, TERM, PWD ( Current Working Direc-   */
/* tory ), DATE.                                                      */
/*                                                                    */
/* author: E. Mauger                                                  */
/* date: Wed Nov 15 16:25:40 'FRA Hiver 1995                          */
/**********************************************************************/

#include <sys/param.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

enum eVarVal { MACHINE=1, UNAME_a, LOGNAME ,HOME, PATH, SHELL, TERM, PWD, DATE, USER };

struct sVarList
{	char *name;
	long int val;
};

/*== List of seuname_aable variable ==*/
struct sVarList var_list[] =
{	{ "MACHINE", MACHINE },
	{ "machine", MACHINE },
	{ "UNAME_a", UNAME_a },
	{ "uname_a", UNAME_a },
	{ "LOGNAME", LOGNAME },
	{ "logname", LOGNAME },
	{ "HOME", HOME },
	{ "home", HOME },
	{ "PATH", PATH },
	{ "path", PATH },
	{ "SHELL", SHELL },
	{ "shell", SHELL },
	{ "TERM", TERM },
	{ "term", TERM },
	{ "PWD", PWD },
	{ "pwd", PWD },
	{ "DATE", DATE },
	{ "date", DATE },
	{ "USER", USER },
	{ "user", USER },
	{ NULL, 0 }
};

/*== Returns the index of 'var_name' in 'var_list' or -1 if no match
==============================================================================*/
static long int _WhichVar(char* var_name)
/* char *var_name:		name of variable to look for */
{
	long int i;
  size_t len;
	
	if(var_name == NULL)		return -1;
	for(i=0; var_list[i].name; i++)
	{	len = strlen(var_list[i].name);
		if(strncmp(var_list[i].name, var_name, len) == 0)		return var_list[i].val;
	}

	return -1;
}

/*============================================================================*/
#ifdef UNDERSCORE
void sysinf_(char env_var_value[], char env_var_name[], size_t *env_var_length , size_t *ret_code)
#else
void sysinf (char env_var_value[], char env_var_name[], size_t *env_var_length , size_t *ret_code)
#endif

/*sysinf (env_var_value, env_var_name, env_var_length , ret_code)*/
/* char env_var_value[];		value of environment variable (output) */
/* char env_var_name[];	  name of environment variable (input) */
/* long int *env_var_length;		length of env. variable (output) */
/* long int *ret_code;		return code 
                                 = -1 : env_var_name empty
                                 =  0 : successful execution
                                 =  1 : env_var_name not found
                              */

{
  char *varpnt;   /* pointer to environment variable */
	long int varidx;     /* variable index in var_list */

	*env_var_length = 0;

	/* Empty env_var_name: */
	if(env_var_name == NULL || *env_var_name == ' ')
	{	strcpy(env_var_value, " ");
    *ret_code = -1;
	}

	varidx = _WhichVar(env_var_name);

	switch(varidx)
	{	case MACHINE :
		case UNAME_a :
		case USER :
		case HOME :
		case PATH :
		case SHELL :
		case TERM :
		{	if((varpnt = getenv(env_var_name)) == NULL)
			{	strcpy(env_var_value, "UNSET");
				*env_var_length = strlen(varpnt);
        *ret_code = 0;
			}
			else
			{ strcpy(env_var_value, varpnt);
				*env_var_length = strlen(varpnt);
				*ret_code = 0;
			}
		}break;

		case PWD :
			getcwd(env_var_value, *env_var_length);
			*env_var_length = strlen(env_var_value);
			*ret_code = 0;
			break;

		case DATE :
		{ time_t now;
			now=time(NULL);
			strcpy(env_var_value, ctime(&now));
			*env_var_length = strlen(env_var_value);
			*ret_code = 0;
		}break;

		default :
			strcpy(env_var_value, " ");
			*env_var_length = 0;
			*ret_code = 1;
  }
}

#ifdef TEST

main()
{
	char varval[512];
	long int len,retcode;

	sysinf (varval,"PATH",&len,&retcode);
	printf("PATH=%s\n", varval);
	sysinf (varval,"USER",&len,&retcode);
	printf("USER=%s\n", varval);
	sysinf (varval,"SHELL",&len,&retcode);
	printf("SHELL=%s\n", varval);
	sysinf (varval,"TZ",&len,&retcode);
	printf("TZ=%s\n", varval);
	sysinf (varval,"TOTO",&len,&retcode);
	printf("TOTO=%s\n", varval);
}

#endif

