!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module system_data_mod

  use kinds_parameters_mod
  implicit none

! Description : stores data read from input 

  type system_data 
    integer(dpi)  :: alfa_electrons_nb ! NOT_INIT_I
    integer(dpi)  :: aos_nb            ! NOT_INIT_I
    integer(dpi)  :: aos_c_nb          ! NOT_INIT_I
    integer(dpi)  :: aos_s_nb          ! NOT_INIT_I
    integer(dpi)  :: atoms_nb          ! NOT_INIT_I
    integer(dpi)  :: beta_electrons_nb ! NOT_INIT_I
    integer(dpi)  :: configurations_nb ! NOT_INIT_I
    integer(dpi)  :: configurations_guide_nb ! NOT_INIT_I
    integer(dpi)  :: dimensions_nb     ! NOT_INIT_I
    integer(dpi)  :: dependences_nb    ! NOT_INIT_I
    integer(dpi)  :: electrons_nb      ! NOT_INIT_I
    integer(dpi)  :: epfs_nb           ! NOT_INIT_I 
    integer(dpi)  :: expressions_nb    ! NOT_INIT_I
    integer(dpi)  :: input_words_nb    ! NOT_INIT_I
    integer(dpi)  :: lik_nb              ! = 2  (lik/unlike electrons)
    integer(dpi)  :: max_determinants_nb ! NOT_INIT_I
    integer(dpi)  :: mos_nb              ! NOT_INIT_I
    integer(dpi)  :: mocl_nb             ! NOT_INIT_I ! mos_nb*aos_nb coeff list
    integer(dpi)  :: power_max           ! NOT_INIT_I ! monomial max power
    integer(dpi)  :: max_degree_a_ee     ! NOT_INIT_I ! = 4   
    integer(dpi)  :: objects_nb          ! NOT_INIT_I
    integer(dpi)  :: routines_nb         ! NOT_INIT_I
    integer(dpi)  :: s_multiplicity      ! NOT_INIT_I
    integer(dpi)  :: walkers_nb          ! NOT_INIT_I
    integer(dpi)  :: xpfs_nb             ! NOT_INIT_I
    integer(dpi)  :: xpfs_c_nb           ! NOT_INIT_I
    integer(dpi)  :: xpfs_s_nb           ! NOT_INIT_I
    real(dp)      :: charge              ! -HUGE(1_dp)
  end type system_data

  type(system_data), save :: system

  contains

! ========================================================================
  subroutine system_data_init
! ------------------------------------------------------------------------
!
! Name           : system_data_init 
!
! Description    : initialize system data
!
! Authors        : F. Colonna
! Date           : 19 Feb 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:

! begin:
  system%alfa_electrons_nb   = NOT_INIT_I
  system%atoms_nb            = NOT_INIT_I
  system%aos_nb              = NOT_INIT_I
  system%beta_electrons_nb   = NOT_INIT_I
  system%charge              = NOT_INIT_DP
  system%configurations_nb   = NOT_INIT_I
  system%configurations_guide_nb   = NOT_INIT_I
  system%dimensions_nb       = NOT_INIT_I
  system%electrons_nb        = NOT_INIT_I
  system%epfs_nb             = NOT_INIT_I
  system%expressions_nb      = NOT_INIT_I
  system%input_words_nb      = NOT_INIT_I
  system%max_determinants_nb = NOT_INIT_I
  system%mos_nb              = NOT_INIT_I
  system%power_max           = NOT_INIT_I
  system%s_multiplicity      = NOT_INIT_I
  system%walkers_nb          = NOT_INIT_I
  system%xpfs_nb             = NOT_INIT_I
  system%xpfs_c_nb           = NOT_INIT_I
  system%xpfs_s_nb           = NOT_INIT_I

  end subroutine system_data_init

end module system_data_mod
