! -*- F90 -*-
module targets_mod

  use routines_mod
  use parser_mod
  use objects_mod
  use objects_paths_mod
  use tools_mc_mod
  use basic_types_tools_mod
  use expressions_mod
! Descriptions : output_list_expressions (output_list_nb, list_elem_max)
! Descriptions : output_list_bloc_freq   (output_list_nb)
! Descriptions : output_list_step_freq   (output_list_nb)
! Descriptions : output_list_walkers     (output_list_nb, list_walk_max)
! Descriptions : output_list_wordbeg     (output_list_nb) ! list begins at word
! Descriptions : output_list_wordend     (output_list_nb) ! list ends at word

  character(len=max_string_len), pointer :: output_list_expressions (:, :) => NULL ()
  integer(dpi), pointer                  :: output_list_walkers     (:, :) => NULL ()
  integer(dpi), pointer                  :: output_list_bloc_freq   (:) => NULL ()
  integer(dpi), pointer                  :: output_list_step_freq   (:) => NULL ()
  integer(dpi), pointer                  :: output_list_wordbeg     (:) => NULL ()
  integer(dpi), pointer                  :: output_list_wordend     (:) => NULL ()

! Description    : targets are defined form lists of expressions or objects

  character(len=max_string_len_object_name), pointer :: targets_name (:) => NULL ()
  integer(dpi), pointer :: targets_ind (:) => NULL()
  contains


! ==========================================================================
  subroutine targets_name_bld
! --------------------------------------------------------------------------
!
! Name           : targets_name_bld
!
! Description    : builds targets defined by lists of expressions 
! Description    : expression = object
! Description    : expression = operator (obj_1, obj_2, ...)  
! Description    : operator = avr sum cov prod div sqr
! Description    : object name created is operator_00n_00m (n obj_1 #, m obj_2 #, etc...)
!
! Needs          : output_list_expressions (outl, expr)
! Needs          : output_list_bloc_freq (outl)
! Needs          : output_list_step_freq (outl)
! Needs          : output_list_walkers (outl, wlk)
! Needs          : output_list_wordbeg (outl)
! Needs          : output_list_wordend (outl)
!
! Creates        : targets (tar)
!
! Authors:F. Colonna
! Date  :20 Nov 2002
! Revision :11 Dec 2002 one more case of operator
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_routine_name),save :: rtn_name 
  character(len=max_string_len_object_name)       :: operator 
  character(len=3) :: snumber  
  integer(dpi) :: cursor
  integer(dpi) :: lis_f, lis_l
  integer(dpi) :: targets_nb
  integer(dpi) :: lis_nb
  integer(dpi) :: expr_nb 
  integer(dpi) :: tar_i, tar_nb 
  integer(dpi) :: lis_i, expr_i
  integer(dpi) :: obj_i, obj_ind, obj_nb

! local arrays:
  character(len=max_string_len_object_name), pointer :: objects_need_temp (:)
  character(len=max_string_len_object_name), pointer :: objects_need (:)
  character(len=max_string_len_object_name), pointer :: objects_new (:)

! header:

  if (header_exe) then

    l_here = "targets_name_bld"

    description = 'targets name are defined by operator_nnn_mmm' &
           *CR*' nnn and mmm are needed objects indices'
    call object_create ('targets_name')

    call object_needed ('output_list_expressions')
    call object_needed ('output_list_bloc_freq')
    call object_needed ('output_list_step_freq')
    call object_needed ('output_list_walkers')
    call object_needed ('output_list_wordbeg')
    call object_needed ('output_list_wordend')

    return

  end if

! begin:
  call enter (l_here)

  lis_nb = size (output_list_expressions, 1)
  expr_nb = size (output_list_expressions, 2)

  if(debug) then
    write (logf,*)trim(l_here), '-d: lis_nb =',lis_nb
    write (logf,*)trim(l_here), '-d: expr_nb =',expr_nb
  end if

! targets_nb = count of non Void expressions. 
  targets_nb = 0
  do lis_i = 1, lis_nb
   do expr_i = 1, expr_nb
     if(debug) then
       write (logf,*)trim(l_here), '-d: output_list_expressions (', &
               lis_i,',', expr_i,') => ', &
               trim(output_list_expressions(lis_i, expr_i)),'<'
     end if
     if(output_list_expressions(lis_i, expr_i) (1:1) /= NOT_INIT_S) then
       targets_nb = targets_nb + 1
     end if
   end do
  end do

  if(debug) then
    write (logf,*)trim(l_here), '-d: targets_nb =',targets_nb
  end if

  call alloc ('targets_name', targets_name, targets_nb)
  targets_name = NOT_INIT_S

  tar_i  = 0
  expr_i = 0

  allocate (objects_need_temp (20), stat = all_err)
  if ( all_err /= 0) call die (l_here,'allocate (objects_need_temp (20))')

  allocate (objects_new (1), stat = all_err)
  if ( all_err /= 0) call die (l_here,'allocate (objects_new (1))')

! loop over all lists of expressions in montecarlo output command
  do lis_i = 1, lis_nb  

! list begins at lis_f word, ends at lis_l word
		lis_f = output_list_wordbeg (lis_i)
		lis_l = output_list_wordend (lis_i) 

    cursor = lis_f + 1

    do cursor = lis_f + 1 , lis_l

      if(debug) then
        write (logf,*)trim(l_here), '-d: lis_i =',lis_i,' lis_f=',lis_f,' lis_l=',lis_l
        write (logf,*)trim(l_here), '-d: input_words (',cursor,') = >', &
                trim(input_words (cursor)),'<'
        write (logf,*)trim(l_here), '-d: input_words_semans (',cursor,') = >', &
                trim(input_words_semans (cursor)),'<'
      end if

! new expression
      if ( input_words_semans (cursor) == 'list 1 alphabetic'  &   
      .or. input_words_semans (cursor) == 'list 1 string'      &
      .or. input_words_semans (cursor) == 'list 1 alphanumeric') then

        operator = input_words (cursor)
        objects_new(1) = trim (operator)

      if(debug) then
        write (logf,*)trim(l_here), '-d: cursor =',cursor,' operator >',trim(operator),'<'
      end if

        obj_i = 0

      else if (input_words_semans (cursor) == 'list 2 comma') then
!        between two objects
      else if (input_words_semans (cursor) == 'list 2 l_parenthesis') then
!        new expression

        expr_i = expr_i + 1

      else if (input_words_semans (cursor) == 'list 2 alphanumeric' ) then
!        new needed object in expression

         obj_i      = obj_i + 1
         objects_need_temp (obj_i) = input_words (cursor)

      else if (input_words_semans (cursor) == 'list 2 r_parenthesis') then
!       end of expression

      else if (input_words_semans (cursor) == 'list 1 comma' &
         .or.  input_words_semans (cursor) == 'list 1 r_parenthesis') then

        obj_nb = obj_i

        do obj_i = 1, obj_nb

          if(debug) then
            write (logf,*)trim(l_here), '-d: objects_need_temp (',obj_i,') >', &
                    trim(objects_need_temp (obj_i)),'<'
          end if

!         build objects_new name as operator_002_005
          obj_ind = string_in_array_locate (objects_need_temp (obj_i), objects_names)
          write(snumber,'(i3.3)') obj_ind
          strjunk = trim(objects_new(1))//"_"//snumber

          objects_new(1) = strjunk

        end do

! store new object as a target 
        tar_i = tar_i + 1

        if (tar_i > targets_nb) then
          message = 'in routine'*l_here*' nb of targets processed='*tar_i &
          *' exceeds total found='*targets_nb
          call ensure (message, tar_i < targets_nb)
        end if
     
        targets_name (tar_i) = objects_new(1)

        if(operator(1:3) == "sum") then
          rtn_name = "sum_calc"
        else if(operator(1:3) == "pro") then
          rtn_name = "prod_calc"
        else
        end if

! resize objects_need.
        if (obj_nb > 0) then
          allocate (objects_need (obj_nb), stat = all_err)
          if ( all_err /= 0) call die (l_here,'allocate (objects_need (20))')

          objects_need (1:obj_nb) = objects_need_temp (1:obj_nb)
          call node_create (rtn_name, objects_new, objects_need)
          deallocate (objects_need)

        else
! no needed object. operator is pure object. check name.
          obj_ind = string_in_array_locate (operator, objects_names)          
          if(obj_ind == 0) then
            message = 'in routine'*l_here*' object (operator) name >' &
                    +operator+'< not catalogued. check name in input'
            call require (message, obj_ind /= 0)
          end if
        end if

      else
        write (logf,*)trim(l_here), '-e: input_words (',cursor,') = >', &
                trim(input_words (cursor)),'<'
        write (logf,*)trim(l_here), '-e: input_words_semans (',cursor,') = >', &
                trim(input_words_semans (cursor)),'<'
         message = 'in routine'*l_here*' invalid word semantic ' 
         call die (l_here, message)
      end if ! menu

     ! cursor = cursor + 1

    end do ! while on cursor

  end do ! lis_i      

  if(debug) then
    tar_nb = tar_i
    do tar_i =  1, tar_nb
      write (logf,*)trim(l_here),'-d: targets_name (',tar_i,') >', &
              trim(targets_name (tar_i)),'<'
    end do
  end if

  deallocate (objects_new)
  deallocate (objects_need_temp)

  call exit (l_here)

  end subroutine targets_name_bld

! ==========================================================================
  subroutine targets_ind_bld
! --------------------------------------------------------------------------
!
! Name           : targets_ind_bld
!
! Description    : builds targets_ind defined by lists of expressions 
!
! Needs          : output_list_expressions (outl, expr)
! Needs          : output_list_bloc_freq (outl)
! Needs          : output_list_step_freq (outl)
! Needs          : output_list_walkers (outl, wlk)
! Needs          : output_list_wordbeg (outl)
! Needs          : output_list_wordend (outl)
!
!
! Authors:R. Assaraf
! Date  : 27 Dec 2002
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: lis_nb
  integer(dpi) :: expr_nb
  integer(dpi) :: expr_i, lis_i
  integer(dpi) :: targets_nb
  integer(dpi) :: cursor
  integer(dpi) :: lis_f,lis_l
  integer(dpi) :: expr_f, expr_l
  character(len=max_string_len) :: expr_nam
  character(len=max_string_len) :: expression_string
  integer(dpi)                  :: wrd_i
! header:

  if (header_exe) then

    l_here = "targets_ind_bld"

    description = 'targets_ind are the indexes of targets objects, defined by an expression' 
    call object_create ('targets_ind')

    call object_needed ('output_list_expressions')
    call object_needed ('output_list_bloc_freq')
    call object_needed ('output_list_step_freq')
    call object_needed ('output_list_walkers')
    call object_needed ('output_list_wordbeg')
    call object_needed ('output_list_wordend')

    return

  end if

! begin:
  call enter (l_here)
  
  lis_nb = size (output_list_expressions, 1)
  expr_nb = size (output_list_expressions, 2)

  if(debug) then
    write (logf,*)trim(l_here), '-d: lis_nb =',lis_nb
    write (logf,*)trim(l_here), '-d: expr_nb =',expr_nb
  end if

! targets_nb = count of non Void expressions. 
  targets_nb = 0
  do lis_i = 1, lis_nb
   do expr_i = 1, expr_nb
     if(debug) then
       write (logf,*)trim(l_here), '-d: output_list_expressions (', &
               lis_i,',', expr_i,') => ', &
               trim(output_list_expressions(lis_i, expr_i)),'<'
     end if
     if(output_list_expressions(lis_i, expr_i) (1:1) /= NOT_INIT_S) then
       targets_nb = targets_nb + 1
     end if
   end do
  end do

  if(debug) then
    write (logf,*)trim(l_here), '-d: targets_nb =',targets_nb
  end if

  call alloc ('targets_ind', targets_ind, targets_nb)
  targets_ind = NOT_INIT_I

!  tar_i  = 0
  expr_i = 0


! loop over all lists of expressions in montecarlo output command
  do lis_i = 1, lis_nb  

! list begins at lis_f word, ends at lis_l word
     lis_f = output_list_wordbeg (lis_i)
     lis_l = output_list_wordend (lis_i) 

     expr_f = lis_f+1
     do cursor = lis_f + 1 , lis_l

        if(debug) then
           write (logf,*)trim(l_here), '-d: lis_i =',lis_i,' lis_f=',lis_f,' lis_l=',lis_l
           write (logf,*)trim(l_here), '-d: input_words (',cursor,') = >', &
                trim(input_words (cursor)),'<'
           write (logf,*)trim(l_here), '-d: input_words_semans (',cursor,') = >', &
                trim(input_words_semans (cursor)),'<'
        end if

        if (input_words_semans (cursor) == 'list 1 comma'.or.cursor==lis_l) then
!        between two objects
           expr_l=cursor-1
           if (debug) then
              write (logf,*)trim(l_here),'-d: input_words(expr_f=',expr_f,')=',trim(input_words (expr_f))
              write (logf,*)trim(l_here),'-d: input_words(expr_l=',expr_l,')=',trim(input_words (expr_l))
           endif
           call expr_nam_bld (input_words, expr_nam, expr_f,expr_l)
           
           if (debug) then 
              write (logf,*) trim(l_here), '-d: expr_nam >', trim (expr_nam),'<'
           endif
           expr_i=expr_i+1
           expression_string=BLANKS

           do wrd_i=expr_f, expr_l
              expression_string = expression_string*' '*input_words(wrd_i)
           enddo !

           call expression_decode (trim(expr_nam), expression_string)

           targets_ind (expr_i) = object_index (trim(expr_nam))
           expr_f=cursor+1
        endif
     enddo ! do cursor
  enddo ! do lis_i

  call exit (l_here)

  end subroutine targets_ind_bld

! ==========================================================================
  subroutine targets_name_print
! --------------------------------------------------------------------------
!
! Name           : targets_name_print
!
! Description    : builds targets defined by lists of expressions 
!
! Needs          : targets (targets_nb)
!
! Authors:F. Colonna
! Date  :20 Nov 2002
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: tar_nb
  integer(dpi) :: tar_i

! header:

  if (header_exe) then
    l_here = "targets_name_print"
    call object_needed ('targets_name')
    return
  end if

! begin:
  call enter (l_here)

  tar_nb = size (targets_name, 1)
  if(tar_nb <= 0) then
    message = 'in routine'*l_here*' nb of targets = '*tar_nb 
    call ensure (message, tar_nb > 0)
  end if

  write (logf,*)trim(l_here),': targets to be computed by qmc'
  do tar_i = 1, tar_nb  
    write (logf,*)trim(l_here),': targets_name (',tar_i,') = >', &
            trim(targets_name(tar_i)),'<'
  end do

  call exit (l_here)

  end subroutine targets_name_print

! ==========================================================================
  subroutine output_list_init
! --------------------------------------------------------------------------
!
! Name           : output_list_init
!
! Description    : initialize array output_list_expressions (expressions to produce)
! Description    :            array output_list_bloc_freq  block frequency
! Description    :            array output_list_step_freq  step  frequency
! Description    :            array output_list_wordbeg    first word for an output_list
! Description    :            array output_list_wordend    last  word for an output_list
! Description    :            array output_list_walkers    list of walkers for one output
!
! Creates        : output_list_wordbeg (outl)
! Creates        : output_list_wordend (outl)
! Creates        : output_list_expressions (outl, expr)
! Creates        : output_list_bloc_freq (outl)
! Creates        : output_list_step_freq (outl)
! Creates        : output_list_walkers (outl, wlk)
!
! Authors:F. Colonna
! Date  :19 Nov 2002
! --------------------------------------------------------------------------
  implicit none 

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi) :: output_list_nb
  integer(dpi) :: list_expr_max 
  integer(dpi) :: list_walk_max 
  integer(dpi) :: expr_nb, walk_nb, out_com, first, last , outl
  integer(dpi) :: f_out, l_out, f_lis, l_lis
  integer(dpi) :: w_lis, w_wlk, f_wlk, l_wlk

! header:

  if (header_exe) then

    l_here = "output_list_init"
 
    description = 'word in input_words where output expressions list begins'
    call object_create ('output_list_wordbeg')


    description = 'word in input_words where output expressions list ends'
    call object_create ('output_list_wordend')
 
    description = 'list of expressions to be produced by montecarlo'
    call object_create ('output_list_expressions')

    description = 'block frequency to output expressions'
    call object_create ('output_list_bloc_freq')

    description = 'step  frequency to output expressions'
    call object_create ('output_list_step_freq')

    description = 'list of walkers to output'
    call object_create ('output_list_walkers')

 
    return

  end if

! begin:
  call enter (l_here)

! get how many walkers from walkers command menu:
!   wal_com = string_head_locate ('wal', command_array%name)

! output command begins at f_wal word, ends at l_wal word
!   f_wlk   = command_array(wal_com)%begin  
!   l_wlk   = command_array(wal_com)%end   
!   num_tok = string_head_locate ('num', input_words, f_wlk, l_wlk)
!    wlk_nb  = system%walkers_nb
!    write (logf,*)trim(l_here),'-d: wlk_nb =',wlk_nb

! count how many "out" commands in montecarlo command menu:
  output_list_nb = string_head_inside_cmd_count ('out','mon','command')

  if(debug) then
    write (logf,*)trim(l_here),'-d: output_list_nb =',output_list_nb
  end if

! output_list_wordbeg: word in input_words where list begins
! output_list_wordend: word in input_words where list ends

  call alloc ('output_list_wordbeg', output_list_wordbeg, output_list_nb)
  call alloc ('output_list_wordend', output_list_wordend, output_list_nb)

  first = 1
  last = size(command_array)
  list_expr_max = 0
  list_walk_max = 0

  do outl = 1, output_list_nb

! current output command location in command_array
    out_com = string_head_locate ('out', command_array%name, first, last)

! output command begins at f_out word, ends at l_out word
    f_out   = command_array(out_com)%begin  
    l_out   = command_array(out_com)%end   

! list begins at f_lis word, ends at l_lis word
    w_lis = string_head_locate ('lis', input_words, f_out, l_out)

    if(w_lis == 0) then
			message = 'in routine'*l_here*' no expressions defined' &
			        *CR*' add list = (expressions list) to output command'
      call require (message, w_lis /= 0)
    end if
                                  
    f_lis = string_head_locate ('list 1 l_parenthesis',  &
                                 input_words_semans, w_lis, l_out)
    l_lis = string_head_locate ('list 1 r_parenthesis',  &
                                 input_words_semans, f_lis, l_out)
		
		output_list_wordbeg (outl) = f_lis
		output_list_wordend (outl) = l_lis
    
! walkers begins at f_wlk word, ends at l_wlk word
    w_wlk = string_head_locate ('wal', input_words, f_out, l_out)

    if(w_wlk /= 0) then
                                  
      f_wlk = string_head_locate ('list 1 l_parenthesis',  &
                                   input_words_semans, w_wlk, l_out)
      l_wlk = string_head_locate ('list 1 r_parenthesis',  &
                                   input_words_semans, f_wlk, l_out)
      walk_nb = string_head_in_array_count &
                ('list 1 comma', input_words_semans, f_wlk, l_wlk) + 1
      list_walk_max = max (walk_nb, list_walk_max)

    end if ! walkers list defined

    if(debug) then
      write (logf,*)trim(l_here), '-d: f_out =',f_out,' l_out =',l_out 
      write (logf,*)trim(l_here), '-d: f_lis =',f_lis,' l_lis =',l_lis 
      write (logf,*)trim(l_here), '-d: f_wlk =',f_wlk,' l_wlk =',l_wlk 
    end if

! nb of expression = number of , plus one.
    expr_nb = string_head_in_array_count &
              ('list 1 comma', input_words_semans, f_lis, l_lis) + 1

    list_expr_max = max (expr_nb, list_expr_max)

		first = out_com  + 1

  end do

  if(debug) then
    write (logf,*)trim(l_here),'-d: output_list_nb =',output_list_nb
    write (logf,*)trim(l_here),'-d: list_expr_max  =',list_expr_max  
    write (logf,*)trim(l_here),'-d: list_walk_max  =',list_walk_max  
  end if

  call require ('in routine'//trim(l_here)//' list_expr_max = 0', &
                list_expr_max /= 0)

  call require ('in routine'//trim(l_here)//' output_list_nb = 0', &
                output_list_nb /= 0)

  call alloc ('output_list_expressions', output_list_expressions, &
               output_list_nb, list_expr_max)
  call alloc ('output_list_bloc_freq', output_list_bloc_freq, output_list_nb)
  call alloc ('output_list_step_freq', output_list_step_freq, output_list_nb)

  if(list_walk_max == 0) then

!    if(wlk_nb == 0) then
!			message = 'in routine'*l_here*' no walkers defined' &
!			        *CR*' add walkers command'
!      call require (message, wlk_nb /= 0)
!    end if
                                  
 !   call alloc ('output_list_walkers',   output_list_walkers, &
 !               output_list_nb, wlk_nb)
 !   do outl_i = 1, output_list_nb
!	    output_list_walkers(outl_i, :)  = (/(wlk_i ,wlk_i = 1, wlk_nb)/)
!    end do

  else
    call alloc ('output_list_walkers',   output_list_walkers, &
                output_list_nb, list_walk_max)
	  output_list_walkers     = NOT_INIT_I
  end if
 
        output_list_expressions = NOT_INIT_S
  if (associated(output_list_bloc_freq)) output_list_bloc_freq   = NOT_INIT_I
  if (associated(output_list_step_freq)) output_list_step_freq   = NOT_INIT_I

  call exit (l_here)

  end subroutine output_list_init

end module targets_mod
