
! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module walkers_bywlk_mod

  use basic_types_tools_mod
  use system_data_mod
  use files_mod
  use tools_mc_mod
!  use psi_bywlk_mod
  use density_bywlk_mod
!  use drift_bywlk_mod
!  use e_reference_bywlk_mod
!  use e_loc_tot_bywlk_mod

! Description    : iteration                 current iteration 

! Author: R. Assaraf
! Date : 10 Aug 2004

  contains

!=============================================================================
 subroutine walkers_bywlk_init 
!-----------------------------------------------------------------------------
!
! Name           : walkers_bywlk_init
!
! Creates        : elec_coor_bywlk (elec, dim, walk)
! Creates        : density_bywlk (walk)
!
! Description    : initializes walkers: i.e electron coordinates and densities
! Description    : dispatches creation towards elec_coor_bywlk_random_init
! Description    : or walkers_bywlk_read
!
! Author: F. Colonna 
! Date  : 23 Jan 2002
! Revision: 26 Jan 2003 R. Assaraf
! -----------------------------------------------------------------------------
  implicit none
! i/o;
  integer(dpi) :: walkers_nb_read

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: walkers_nb 
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: dimensions_nb
  integer(dpi)                               :: unit
  integer(dpi)                               :: walk_i, elec_i, dim_i
  integer(dpi)                               :: copied
! header:

  if (header_exe) then

    l_here = routine_cur_name ()


    call object_create ('elec_coor_bywlk')
    call object_create ('density_bywlk')
    call object_create ('elec_valid_bywlk')

  
    return

  end if

! begin:

  l_here='walkers_bywlk_init'
  call enter (l_here)


! requirements:
  electrons_nb = system%electrons_nb
  require_elec: if(electrons_nb <= 0) then
    call die (l_here, ' nb of electrons ='+electrons_nb)
  end if require_elec

  dimensions_nb = system%dimensions_nb
  require_dim: if(dimensions_nb <= 0) then 
    call die (l_here, ' nb of dimensions ='+dimensions_nb)
  end if require_dim 

  walkers_nb = system%walkers_nb
  require_walk: if(walkers_nb <= 0) then 
    write (logf,*)trim(l_here),'-e: '
    write (logf,*)trim(l_here),'-e: nb of walkers = ',walkers_nb,' is not initialized'
    write (logf,*)trim(l_here),'-e: '
    write (logf,*)trim(l_here),'-e: include line " number = [integer]"'
    write (logf,*)trim(l_here),'-e: in walkers command input'
    write (logf,*)trim(l_here),'-e: '
    call die (l_here, ' nb of walkers is not initialized')
  end if require_walk

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
    write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if
 
! end requirements:


 
  unit = unit_from_type ('walkers-in')

! walkers-in not catalogued (unit=0) => generate walkers randomly
! else read walkers

  write (logf,*) trim(l_here), ' Walkers initialization'
  write (res, *) 
  write (res, '(2a)') ' Walkers initialization:'
  write (res, '(2a)') ' -----------------------'

 
! allocations:

  call alloc ('elec_coor_bywlk', elec_coor_bywlk, &
       electrons_nb, dimensions_nb, walkers_nb)


  call alloc ('density_bywlk', density_bywlk, walkers_nb)
! end allocations

  if(unit == 0) then
     write (logf,*) trim (l_here),': No file walkers-in defined in the input'
     write (logf,*) trim (l_here), ': => walkers generated at random '
     write (res, *)  'No file walkers-in defined in the input' &
                   //' => walkers  generated at random '
    

     call elec_coor_bywlk_random_init 
  
     density_bywlk = 1_dp
  else     
     write (logf, *) trim (l_here), ': Walkers  initialized from file'
     write (res,  *) 'Walkers  initialized from file'

     call walkers_bywlk_read (walkers_nb_read)
     walkers_nb = system%walkers_nb
     if (walkers_nb_read < walkers_nb) then

        write (logf,*) trim(l_here),':',walkers_nb-walkers_nb_read, ' walkers not initialized'
        write (logf,*) trim(l_here),': will be initialized by duplication of the walkers already read'
        
        write (res,*)  walkers_nb_read, ' walkers read in file'
        write (res,*) walkers_nb-walkers_nb_read, 'walkers initialized by duplication'

        do walk_i=walkers_nb_read,walkers_nb
           copied=mod (walk_i,walkers_nb_read)+1
           write (logf,*) trim(l_here),': walker #',copied,' is copied on #',  walk_i

           elec_coor_bywlk (:,:,walk_i)=elec_coor_bywlk (:,:,copied)
           density_bywlk (walk_i)    = density_bywlk (copied)
        enddo

     endif
 
  end if

  include 'chksiz_elec_coor_bywlk.inc'

! dimensions:

  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)

  if (.not.associated(elec_valid_bywlk)) then 
   allocate (elec_valid_bywlk (electrons_nb,walkers_nb))
   elec_valid_bywlk = .false.
  endif

  if(debug) then
    write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
    write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
    write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if

! end dimensions

  include 'debug_elec_coor_bywlk.inc'
 
  call exit (l_here)

  end subroutine walkers_bywlk_init

! ========================================================================
  subroutine walkers_bywlk_read (walkers_nb_read)
! ------------------------------------------------------------------------
!
! Description    : Reads walkers from file
!
! -- Creates     : elec_coor_bywlk (elec, dim, walk)
! -- Creates     : drift_bywlk (elec, dim, walk)
! -- Creates     : psi_bywlk (walk)
! -- Creates     : density_bywlk (walk)
!
! Authors:F. Colonna 
! Date  :23 Jan 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
   integer(dpi) :: walkers_nb_read

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
   
  integer(dpi)                               :: dimensions_nb
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: walkers_nb
  integer(dpi)                               :: elec_i, dim_i, walk_i
  integer(dpi)                               :: unit
 
  logical(bool)                              :: eof, done
! begin:
  l_here ='walkers_bywlk_read'
  call enter (l_here)
  
  f_type = 'walkers-in'//blanks
     unit = file_open_and_cat_from_type (f_type)

  message = 'in routine'*l_here*' walkers-file unit not found'
  call require (message, unit /= 0)

  f_name   = files_names(unit)
  f_action = files_actions(unit)

  if(debug) then
     write (logf,*)trim(l_here),'-d: walkers file unit   =',unit
     write (logf,*)trim(l_here),'-d: walkers file name   >',trim(f_name),'<'
     write (logf,*)trim(l_here),'-d: walkers file type   >',trim(f_type),'<'
     write (logf,*)trim(l_here),'-d: walkers file action >',trim(f_action),'<'
  end if

! read header (initialize system%walkers_nb)
  call file_header_read (unit)

! require_walk: 
  walkers_nb = system%walkers_nb 
  if(walkers_nb <= 0) call die &
    (l_here, ' nb of walkers ='+walkers_nb)

! require_elec: 
  electrons_nb = system%electrons_nb
  if(electrons_nb <= 0) call die &
    (l_here, ' nb of electrons ='+electrons_nb)

! require_dim: 
  dimensions_nb = system%dimensions_nb
  if(dimensions_nb <= 0) call die &
    (l_here, ' nb of dimensions ='+dimensions_nb)

  if(debug) then
     write (logf,*)trim(l_here),'-d: electrons_nb =',electrons_nb
     write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
     write (logf,*)trim(l_here),'-d: walkers_nb =',walkers_nb
  end if

  walkers_nb_read=0
  done = .false.

  do while (.not. done) 
    walkers_nb_read = walkers_nb_read + 1
    call one_walker_read (unit, walkers_nb_read, eof)
    if (eof .or. walkers_nb_read == walkers_nb) done = .true.
  enddo
 
  write (logf,*)trim(l_here),': ',walkers_nb_read,' walkers read ',&
                       ' from file >',trim(f_name),'<'

  include 'debug_elec_coor_bywlk.inc'


  call file_close_by_unit(unit)

  call exit (l_here)

  end subroutine walkers_bywlk_read

! ========================================================================
  subroutine walkers_bywlk_print
! ------------------------------------------------------------------------
!
! Description    : prints a set of walkers
!
! Authors:F. Colonna 
! Date  :22 Jan 2001
! Revision: 23 Jan 2002 by F. Colonna 
! ------------------------------------------------------------------------
  implicit none

! local:
  character(len=max_string_len_routine_name)  :: l_here

! begin:
  l_here ='walkers_bywlk_print'
  call enter (l_here)

!  call walkers_bywlk_write (logf)

  call exit (l_here)

  end subroutine walkers_bywlk_print

! ========================================================================
  subroutine one_walker_write (unit, walk_i)
! ------------------------------------------------------------------------
!
! Name           : one_walker_write
!
! Argument (in ) : unit
! Argument (in ) : walk_i

! Description    : writes a walker for a given walker
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : density_bywlk (walk)
!
! Authors:X. Krokidis
! Date  :22 Jan 2001
! Revision: 23 Jan 2002 by F. Colonna 
! Revision : 23 Janv 2003 by R. Assaraf
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                    :: unit
  integer(dpi)                    :: walk_i

! local:
  character(len=max_string_len_routine_name),save :: l_here
  integer(dpi)                               :: dim_i, elec_i
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: dimensions_nb

! header:

  if (header_exe) then

    l_here = routine_cur_name ()

    call object_needed ('elec_coor_bywlk')
    call object_needed ('density_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  require_unit: if(unit == 0) then
    message  = 'in routine'*l_here*' walkers-file unit not found'
    call require (message, unit /= 0)
  end if require_unit

  require_walk: if(walk_i <= 0) then
    message  = 'in routine'*l_here*' walker number <= 0'
    call require (message, unit /= 0)
  end if require_walk

  include 'chksiz_elec_coor_bywlk.inc'
 
! end requirements

! dimensions:
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)

! end dimensions

  do elec_i = 1, electrons_nb
    write(unit,'(a,t15,i10,3e25.15)') &
          'electron',elec_i,          &
          (elec_coor_bywlk (elec_i, dim_i, walk_i), dim_i = 1,dimensions_nb)
 
  enddo ! do elec_i

  write(unit,'(a,t15,e25.15)')'density  ',density_bywlk (walk_i)

  call exit (l_here)

  end subroutine one_walker_write

! ========================================================================
  subroutine one_walker_read (unit, walker, eof)
! ------------------------------------------------------------------------
!
! Name           : one_walker_read
!
! Argument (in ) : unit
! Argument (in ) : walker
! Argument (out) : eof
!
! Description    : reads a walker for a given walker
!
! Authors:X. Krokidis
! Date  :22 Jan 2001
! Revision: 14 Jan 2002 by F. Colonna
! Revision : 17 Jan 2003 by R. Assaraf
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                    :: unit
  integer(dpi)                    :: walker
  logical(bool)                   ::  eof

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len)   :: line  
  character(len= 24)                          :: item

  integer(dpi)                    :: rec_nb
  integer(dpi)                    :: elec_i, elec, i
  integer(dpi)                    :: electrons_nb
  logical(bool)                   :: done

! begin:
  l_here ='one_walker_read'
  call enter (l_here,5)
  if(debug) write (logf,*)trim(l_here),'-d: reading walker # ',walker 
  
! require_elec: 
  electrons_nb = system%electrons_nb
  if(electrons_nb <= 0) call die &
  (l_here, ' nb of electrons ='+electrons_nb)

  line = ' '
  rec_nb = 0
  elec_i = 0

  done = .false.
  eof = .true.
  do while (.not.done)

    rec_nb = rec_nb + 1

    read(unit,'(a)') line

    n_string_words = string_to_words_count (line,' ', n_string_words)
    call split_string (line, ' ', string_words, string_words_length, n_string_words, len(string_words))

    if(n_string_words == 0) cycle

    item = string_words (1)

    if(debug) write (logf,*)trim(l_here),'-d: line >',trim(line),'<'   
    if(debug) write (logf,*)trim(l_here),'-d: item >',item,'<'
    if(debug) write (logf,*)trim(l_here),'-d: string_words:',(trim(string_words(i)),i= 1,size (string_words))
    select case (item)

    case ("electron")
      elec = string_to_integer (trim(string_words (2)))
      elec_i = elec_i + 1

      if(debug) write (logf,*)trim(l_here),': electron # ',elec_i   

      if(elec /= elec_i) then
       message =' electron number read ='+elec+' while current number ='+elec_i
       call ensure (message, elec == elec_i)
      end if

      elec_coor_bywlk (elec_i, 1, walker) = string_to_double (string_words (3))
      elec_coor_bywlk (elec_i, 2, walker) = string_to_double (string_words (4))
      elec_coor_bywlk (elec_i, 3, walker) = string_to_double (string_words (5))

      if(debug) write (logf,*)trim(l_here),': elec_coor_bywlk :', &
            elec_coor_bywlk (elec_i, :, walker)

    case ("density")
      density_bywlk (walker) = string_to_double (trim(string_words(2)))
      if(debug) write (logf,*)trim(l_here),': density_bywlk (',walker,') = ',density_bywlk (walker)

     case ("end_of_walker")
      done = .true.
      eof = .false.

     case ("end_of_file")
      eof = .true.
      done = .true.

    end select

  enddo

!  read (unit,'(a)') line  

!  write (logf,*)trim(l_here),': density_bywlk ',density_bywlk (walker)

   call exit (l_here,5)

  end subroutine one_walker_read

! ========================================================================
  subroutine walkers_bywlk_header_write (unit )
! ------------------------------------------------------------------------
!
! Description    : write header for walkers' file on unit
!
! Argument (in ) : unit
!
! Authors:F. Colonna
! Date  :18 Feb 2002
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                               :: unit

! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name

! begin:
  l_here ='walkers_bywlk_header_write'
  call enter (l_here)

  if(system%dimensions_nb <= 0) call die &
  (l_here, ' nb of dimensions ='+system%dimensions_nb)
  if(system%electrons_nb <= 0) call die &
  (l_here, ' nb of electrons ='+system%electrons_nb)
  if(system%alfa_electrons_nb <= 0) call die &
  (l_here, ' nb of alfa_electrons ='+system%alfa_electrons_nb)
  if(system%beta_electrons_nb <= 0) call die &
  (l_here, ' nb of beta_electrons ='+system%beta_electrons_nb)
  if(system%walkers_nb <= 0) call die &
  (l_here, ' nb of walkers ='+system%walkers_nb)

  message  = 'in routine'*l_here*' walkers-file unit not found'
  call require (message, unit /= 0)

  f_type   = files_types(unit)
  f_name   = files_names(unit)
  f_action = files_actions(unit)  

  if(debug) then
   write (logf,*)trim(l_here),'-d: walkers file unit   =',unit
   write (logf,*)trim(l_here),'-d: walkers file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: walkers file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: walkers file action >',trim(f_action),'<'
  end if

  call file_header_write (unit)
  write(unit,'(a,i5)')'number_of_walkers        =',system%walkers_nb
  write(unit,'(a,i5)')'number_of_dimensions     =',system%dimensions_nb
  write(unit,'(a,i5)')'number_of_electrons      =',system%electrons_nb
  write(unit,'(a,i5)')'number_alfa_of_electrons =',system%alfa_electrons_nb
  write(unit,'(a,i5)')'number_beta_of_electrons =',system%beta_electrons_nb
  write(unit,'(a)')'end_of_header'

  call exit (l_here)

  end subroutine walkers_bywlk_header_write

! ========================================================================
  subroutine walkers_bywlk_write
! ------------------------------------------------------------------------
!
! Description    : write a set of walkers on unit
!
! Argument (in ) : unit
!
! Needs          : elec_coor_bywlk (elec, dim, walk)
! Needs          : density_bywlk (walk)
!
! Authors:X. Krokidis
! Date  :22 Jan 2001
! Revision: 18 Jan 2002 by F. Colonna 
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi), save                         :: unit = 0

! local:
  character(len=max_string_len_routine_name),save :: l_here
  character(len=max_string_len_file_type)    :: f_type
  character(len=max_string_len_action)       :: f_action
  character(len=max_string_len_file_name)    :: f_name
  integer(dpi)                               :: walk_i
  integer(dpi)                               :: electrons_nb
  integer(dpi)                               :: dimensions_nb
  integer(dpi)                               :: walkers_nb
  logical(bool)                              :: first_write = .true.
! header:

  if (header_exe) then

    l_here = routine_cur_name ()

    call object_needed ('elec_coor_bywlk')
    call object_needed ('density_bywlk')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

  include 'chksiz_elec_coor_bywlk.inc'
   
! end requirements

! dimensions:
  electrons_nb  = size (elec_coor_bywlk, 1)
  dimensions_nb = size (elec_coor_bywlk, 2)
  walkers_nb    = size (elec_coor_bywlk, 3)

! end dimensions

! file
  f_type='walkers-out'//blanks
  unit = unit_from_type (trim(f_type))
  f_name   = files_names(unit)
  unit = file_open_and_cat_from_type (f_type)
  f_action = files_actions(unit)  

  if(debug) then
   write (logf,*)trim(l_here),'-d: electrons_nb  =',electrons_nb
   write (logf,*)trim(l_here),'-d: dimensions_nb =',dimensions_nb
   write (logf,*)trim(l_here),'-d: walkers_nb    =',walkers_nb
   write (logf,*)trim(l_here),'-d: walkers file unit   =',unit
   write (logf,*)trim(l_here),'-d: walkers file name   >',trim(f_name),'<'
   write (logf,*)trim(l_here),'-d: walkers file type   >',trim(f_type),'<'
   write (logf,*)trim(l_here),'-d: walkers file action >',trim(f_action),'<'
  end if

  if (first_write) then
     call walkers_bywlk_header_write (unit)
     first_write = .false.
  endif

  do walk_i = 1, walkers_nb
    write (unit,'(a,t15, i10)') 'walker ',  walk_i
    call one_walker_write (unit, walk_i)
    write (unit,'(a)') 'end_of_walker'
  end do

! end of file
 if (f_action /= "append")  write (unit,'(a)') 'end_of_file'
  write (logf,*) trim(l_here),':', walkers_nb,  'walkers written in file>',trim(f_name),'<' 

  if (f_action /= "append")  then
     call file_close_by_unit (unit)
     first_write = .true.
  endif

  call exit (l_here)

end subroutine walkers_bywlk_write

! ========================================================================
  subroutine one_walker_energies_write (unit, walk_i)
! ------------------------------------------------------------------------
!
! Name           : one_walker_energies_write
!
! Argument (in ) : unit
! Argument (in ) : walk_i

! Description    : writes a walker energies
!
! Authors:X. Krokidis
! Date  :22 Jan 2001
! Revision: 23 Jan 2002 by F. Colonna 
! ------------------------------------------------------------------------
  implicit none

! i/o:
  integer(dpi)                    :: unit
  integer(dpi)                    :: walk_i

! local:
  character(len=max_string_len_routine_name) :: l_here

! begin:
  l_here ='one_walker_energies_write'
  call enter (l_here)

! requirements:

!  include 'chksiz_e_loc_tot_bywlk.inc'

! end requirements


! write energies
!  write(unit,'(a,e25.15)')'e_loc_kin_bywlk ',e_loc_kin_bywlk (walk_i)
!  write(unit,'(a,e25.15)')'e_loc_vne_bywlk ',e_loc_vne_bywlk (walk_i)
!  write(unit,'(a,e25.15)')'e_loc_vee_bywlk ',e_loc_vee_bywlk (walk_i)
!  write(unit,'(a,e25.15)')'e_loc_tot_bywlk ',e_loc_tot_bywlk (walk_i)

  call exit (l_here)

  end subroutine one_walker_energies_write

end module walkers_bywlk_mod
