! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!             Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module wavefunction_menu_mod

! ------------------------------------------------------------------------
! 
! Author: X.Krokidis, F.Colonna, P.Reinhardt
! Date  : 18 Oct 2000, 22 Nov 2000, 29 Nov 2000
! Revision: X.Krokidis, F.Colonna, R. Assaraf 11 Jan 01
! Revision: seriously ... P.Reinhardt 02/02/2001
! Revision: completely recast F. Colonna 31 Jan 2002
! ------------------------------------------------------------------------
 
  use configurations_mod
! use jastrow_menu_mod
  use system_data_mod

! In this module the specific wave function is defined
! and generated
 
! NAMELY: 
!                  jastrow_of_conf_c
!  sum_over_confs    |        sum_over_dets  dets_alpha_and_beta
!       |            |            |                  |
!       | weight_of_c|            | weight_of_d      |
!       |     |      |            |   |      .--------------.
!       v     v      v            v   v      v              v
! Psi = Sum_c W(c) * Jastrow(c) * S_d w(d) * det_alpha(d,c)*det_beta(d,c)
 
  contains
 
! ========================================================================
  subroutine wavefunction_menu
! ------------------------------------------------------------------------
! Name           : wavefunction_menu
 
! Description    : input wavefunction information
 
! Author: X. Krokidis, F. Colonna
! Date  : 21 Oct 2000
! Revision: X. Krokidis, F. Colonna 11 Jan 01
! Revision: 31 Mar 2001 F. Colonna n_charge
! Revision: 08 Nov 2002 F. Colonna jastrow_calc
! ------------------------------------------------------------------------
  implicit none
    
! local:
  character(len=max_string_len_routine_name) :: l_here
  character(len=max_string_len) :: what   
  integer(dpi)        :: atom_i
  integer(dpi)        :: n_deter,det_i ,n_alpha,n_beta
  integer(dpi)        :: conf_i, conf_nb
  integer(dpi)        :: electrons_nb
  integer(dpi)        :: obj_i
  logical(bool)       :: done
  logical(bool)       :: jastrow_calc
  real(dp)            :: l_charge, n_charge
    
! begin:
  l_here ='wavefunction_menu'
  call enter(l_here)

! initialize:
  jastrow_calc = .false.
  done = .false.
  what = NOT_INIT_S
  
  call next_command_get ('wave')
  do while (.not.done)
     call next_word_get('wave')
     if(debug) write (logf,*)trim(l_here),'-d: cur_word >',trim(cur_word),'<'
     if(trim(cur_word) .eq. 'help') then
      write (logf,*) ' wave-h: menu'
      write (logf,*) ' wave-h: syntax'
      write (logf,*) ' wave-h: wav{efunction}'
      write (logf,*) ' wave-h:  help'
      write (logf,*) ' wave-h:  con{figuration} [command] (repeat)'
      write (logf,*) ' wave-h:  print = con{figurations}'
      write (logf,*) ' wave-h:  end_wav'
      write (logf,*) ' wave-h: example'
      
  else if(cur_word(1:3) .eq. 'con') then
      call configuration_menu  
      
  else if(token_is ('pri', what)) then
      
  else
      call menu_exit (l_here, done)
  end if
     
  end do
    
! check the charge
  electrons_nb = system%electrons_nb
  message='in routine'*l_here*' system%electrons_nb='+system%electrons_nb
  call require (message, system%electrons_nb > 0)
  
  n_charge = zero
  do atom_i = 1,size(atoms)
      n_charge = n_charge + atoms(atom_i)%charge
  enddo

  l_charge = n_charge - float(electrons_nb)
  if(abs(l_charge - system%charge) > EPSILON(+1.0)) then
      message='system charge from input = '*system%charge &
              +CR*" "*l_here+'-die: sum of nuclei charges '+n_charge &
              +' - electrons charge '+electrons_nb+' = '*l_charge  &
              +CR*" "*l_here+'-die: please check charge in input'
      call warning (l_here, message)
  endif

! all configurations fields are now known:
    
  call check_asso_conf ('configurations', configurations, &
       'configurations_init', configurations_init)

  include 'chksiz_configurations.inc'

  conf_nb = system%configurations_nb
     
  do conf_i = 1, conf_nb

    if (configurations(conf_i)%jastr_type      /= NOT_INIT_S  .and. &
        configurations(conf_i)%jastr_type(1:3) /= "NON") jastrow_calc = .true.

      if(print_level .ge. 1 .or. debug) then
        write (logf,*)trim(l_here),'-d: conf # ',conf_i, &
               ' named >',trim(configurations(conf_i)%name),'<', &
               ' jastr_type >',trim(configurations(conf_i)%jastr_type),'<'
    end if

  end do
     
! This to disable jastrow calculation if not used.
  if(jastrow_calc) then
    write (logf,*)trim(l_here),': jastrow is USED in this calculation'
  else
    write (logf,*)trim(l_here),': NO jastrow used in this calculation'
    do obj_i = 1, objects_nb
      if (objects_names(obj_i)(1:len('jastrow_factor_')) == 'jastrow_factor_') then
          objects_used(obj_i) = .false.
          write (logf,*)trim(l_here),': object >',trim(objects_names(obj_i)),'< disabled'
      end if 
    end do
  end if

  if (debug) then
    do conf_i = 1,conf_nb
      n_deter = size(configurations(conf_i)%conf_deter)
      write (logf,*)trim(l_here),' configuration ',conf_i,': n_deter = ',n_deter
      do det_i = 1,n_deter
       write (logf,*)trim(l_here),' determinant ',det_i 
       n_alpha = size(configurations(conf_i)%conf_deter(det_i )%alfa_orbitals)
       n_beta = size(configurations(conf_i)%conf_deter(det_i )%beta_orbitals)
       write (logf,*)trim(l_here),'-d: ',n_alpha, &
            ' alpha orbitals and ',system%alfa_electrons_nb,' alpha electrons'
       write (logf,*)trim(l_here),'-d: ',n_beta, &
            ' beta  orbitals and ',system%beta_electrons_nb,' beta  electrons'
      end do
    end do
  end if
    
  if(what(1:3) == "con") then
    message ='configurations of wavefunction'
    call conf_jastr_print_res (message)
  end if

  call exit(l_here)
    
  end subroutine wavefunction_menu
 
end module wavefunction_menu_mod
