! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module write_mod

  use parser_mod
  use mos_mod
  use aos_mod
  use configurations_mod
  use density_grid_mod
!  use intracule_mod

  contains

! ========================================================================
  subroutine write_menu
! ------------------------------------------------------------------------
!
! Name           : write_menu
!
! Description    : write command menu
!
! Authors:X. Krokidis, F. Colonna
! Date  :12 Oct 2000
! Revision: 08 Oct 2002 F. Colonna
! ------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name)  :: l_here
  logical(bool)                               :: done
  logical(bool)                               :: write
  character(len=4)                            :: what
  character(len = max_string_len_object_name) :: density_grid_print

! begin:
  l_here ='write_menu'
  call enter (l_here)

! initialization
  what = blanks
  write = .false.

  done = .false.
  call next_command_get ('write')
  do while (.not.done)
   call next_word_get ('write')
   if(cur_word(1:4) .eq. 'help') then
    write (logf,'(a)') ' write-h: write command'
    write (logf,'(a)') ' write-h: syntax:'
    write (logf,'(a)') ' write-h: write'
    write (logf,'(a)') ' write-h:   help'
    write (logf,'(a)') ' write-h:   epf{s}    = [boolean]'
    write (logf,'(a)') ' write-h:   aos       = [string] :cart:sphe:'
    write (logf,'(a)') ' write-h:   mos         [command]'
    write (logf,'(a)') ' write-h:   jas{trow} = [boolean]'
    write (logf,'(a)') ' write-h:   den{sity_grid} = [string] :val:err:'
    write (logf,'(a)') ' write-h: end'
    write (logf,'(a)') ' write-h: example'
    write (logf,'(a)') ' write-h: write aos = cart end '
    write (logf,'(a)') ' write-h: write epf{s} = true end '
    write (logf,'(a)') ' write-h: write mos ... end end'
    write (logf,'(a)') ' write-h: write den = val end'
  else if(token_is ('aos', what)) then

    select case(what)
    case ("cart")
!    call aos_c_write 
     call die (l_here,'aos_c_write not yet implemented' )
    case ("sphe")
     call die (l_here,'aos_s_write not yet implemented' )
    case default
     call die (l_here, & 
     'aos type >'//trim(what)//' should be "cart" or "sphe"')
    end select

  else if(token_is ('epf', write)) then
    if(write) call epfs_write 

  else if(cur_word (1:3) == 'mos') then
    call mos_write_menu

  else if(token_is ('jas', write)) then
   if(write) call conf_jastr_write

  else if(token_is ('den', density_grid_print)) then

     select case (density_grid_print)

       case ('val')
        call density_grid_write (density_grid)
      
       case ('err') 
        call density_grid_write (density_grid_err)

       case ('var') 
        call density_grid_write (density_grid_var)

       case ('alp') 
        call density_grid_write (density_grid_a)

       case ('bet') 
        call density_grid_write (density_grid_b)

       case ('aer') 
        call density_grid_write (density_grid_a_err)

       case ('ber') 
        call density_grid_write (density_grid_b_err)

       case ('ava') 
        call density_grid_write (density_grid_a_var)

       case ('bva') 
        call density_grid_write (density_grid_b_var)

       case default
        call die (l_here, 'density_grid='+density_grid_print+' invalid option')
     end select

!  else if(token_is ('int', write)) then
!   if(write) call intracule_write

  else

   call menu_exit (l_here, done)

  end if

  end do

  if(debug)write (logf,*)trim(l_here),'-d: what >',trim(what),'<'

  call exit (l_here)

  end subroutine write_menu

end module write_mod
