#! /usr/bin/perl

use array;
use atom_parser;
use basis_parser;
use mo_parser;
use energy_parser;

my($base) = @ARGV[0];
my($type) = @ARGV[1];

$type = lc($type);

my($state)    = "ground";
my($charge)   = "neutral";
my($whose)    = "adf";

my($file_dat) = "$base.dat";
my($file_inp) = "$base.inp";
my($file_out) = "$base.out";

my($ENERGY) = energy_parser->ENERGY_DATA($file_out, "gamess");

my($orbital_energy) = $ENERGY->{orbital_energy};

my($MO)   = mo_parser->STO_MO_DATA($file_dat, "gamess");

my($STO)  = basis_parser->STO_BASIS_DATA($file_inp, "slater_gamess_input");

my($symmetry) = $STO->{symmetry};

my($exponent) = $STO->{exponent};

my($level)    = $STO->{r_power};

my($basis_nb) = $STO->{function_number};

$file_inp =~ m/([a-z]+)/i;

my($element) = $1;

my($ATOM) = atom_parser->ATOM_DATA($element,"query");

my($z) = $ATOM->{element_charge};

my($name)   = $ATOM->{element_name};

my($label) = $ATOM->{element_label};

my($multiplicity) = $ATOM->{element_multiplicity};

my($s_orb_nb) = $ATOM->{s_orb_nb};

my($p_orb_nb) = $ATOM->{p_orb_nb};

my($d_orb_nb) = $ATOM->{d_orb_nb};

my($occ) = $ATOM->{element_occupation};

my($mo) = $MO->{mo};

my($orb_nb) = $s_orb_nb + $p_orb_nb + $d_orb_nb;

my($mo_nb)    = $MO->{mo_nb};

my($coeff_nb) = $MO->{coeff_nb};

expand_basis();

$occupation = $ATOM->{element_occupation};
	
if($occupation =~ m/K\(2\)/i)
{
    $s_string = $s_string . "1s(2) ";
}

if($occupation =~ m/L\(8\)/i)
{
    $s_string = $s_string . "2s(2) ";
    $p_string = $p_string . "2p(6) ";
}

if($occupation =~ m/M\(18\)/i)
{
    $s_string = $s_string . "3s(2) ";
    $p_string = $p_string . "3p(6) ";
    $d_string = $d_string . "3d(10) ";
}

while($occupation =~ m/(\d+[spdf]\(\d+\))/ig)
{
    $orb = lc($1);
    
    if($orb =~ m/S/i) { $s_string = $s_string . "$orb "; }
    if($orb =~ m/P/i) { $p_string = $p_string . "$orb "; }
    if($orb =~ m/D/i) { $d_string = $d_string . "$orb "; }
    if($orb =~ m/F/i) { $f_string = $f_string . "$orb "; }
}

my($output) = "$label-$whose-$type-$charge-$state-ref.dat";

open(OUT, ">$output") or die "cannot create output file\n";

$whose = uc($whose);
$state = ucfirst($state);
$type  = uc($type);

print OUT  "Element          : $name\n";
print OUT  "Z                : $z\n";
print OUT  "State            : $state\n";
print OUT  "Charge           : 0\n";
print OUT  "Whose            : $whose\n";
print OUT  "Type             : $type\n";
print OUT  "Multiplicity     : $multiplicity\n";
print OUT  "\n";
print OUT  "Orbital Occupations:\n";
print OUT  "\n";

if($s_string ne "") { print OUT "S : $s_string\n"; }
if($p_string ne "") { print OUT "P : $p_string\n"; }
if($d_string ne "") { print OUT "D : $d_string\n"; }

print OUT "\n";
printf OUT "Total energy     : %17.10f\n", $ENERGY->{total_energy};
printf OUT "Kinetic energy   : %17.10f\n", $ENERGY->{kinetic_energy};
printf OUT "Potential energy : %17.10f\n", $ENERGY->{potential_energy};
printf OUT "Virial ratio     : %17.10f\n", $ENERGY->{virial_ratio};
print OUT "\n";


print_basis();

print OUT "Atomic orbitals: \n";
print OUT "\n";

$rem = $orb_nb%5;
$num = int $orb_nb / 5;

for($l=1; $l<= $num; $l++)
{
    $min = ($l-1) * 5 + 1 ;
    $max = ($l) * 5;

    print OUT "Orbital energy  \t";

    for($t=$min-1; $t<=$max-1; $t++)
    {
	printf OUT "%15.10f", $orbital_energy->[$t];
    }
    
    print OUT "\n";
    print OUT "Basis function\tExponent\n";

    print_mos($min, $max);
    
    print OUT "\n";
}

if($rem > 0)
{
    $max = $orb_nb;
    $min = $orb_nb - $rem +1 ;

    print OUT "Orbital energy  \t";

    for($t=$min-1; $t<=$max-1; $t++)
    {
	printf OUT "%15.10f", $orbital_energy->[$t];
    }
    
    print OUT "\n";
    print OUT "Basis function\tExponent\n";

    print_mos($min, $max);
}

sub print_basis
{
    print OUT "Basis function   Exponent\n";

    for($m=0; $m< size($exponent); $m++)
    {
	print OUT "$level->[$m]$symmetry->[$m]\t\t   $exponent->[$m]\n";
    }

    print OUT "\n";
}


sub expand_basis
{
    for($m=0; $m< size($exponent); $m++)
    {
	if($symmetry->[$m] =~ m/s/i)
	{ 
	    push(@sym, "$symmetry->[$m]   ");
	    push(@exp, $exponent->[$m]);
	    push(@pow, $level->[$m]);
	}
	
	if($symmetry->[$m] =~ m/p/i)
	{ 
	    @p[0] = "x";
	    @p[1] = "y";
	    @p[2] = "z";
	    
	    for($n=0; $n< 3; $n++)
	    {
		push(@sym, "$symmetry->[$m]  @p[$n]");
		push(@exp, $exponent->[$m]);
		push(@pow, $level->[$m]); 
	    }
	}
	
	if($symmetry->[$m] =~ m/d/i)
	{
	    @p[0] = "xx";
	    @p[1] = "yy";
	    @p[2] = "zz";
	    @p[3] = "xy";
	    @p[4] = "xz";
	    @p[5] = "yz";
	    
	    for($n=0; $n< 6; $n++)
	    {
		push(@sym, "$symmetry->[$m] @p[$n]");
		push(@exp, $exponent->[$m]);
		push(@pow, $level->[$m]); 
	    }	
	}
    }
}

sub print_mos
{
    my($min) = $_[0];
    my($max) = $_[1];

    for($j=0; $j< $coeff_nb; $j++)
    {	
	printf OUT "%5s %18.3f","@pow[$j]@sym[$j]", @exp[$j];
	
	for($i=$min-1; $i<= $max-1; $i++)
	{
	    printf OUT "%15.10f", $mo->[$i]{$j};
	}  
	
	print OUT "\n";
    }
}


sub size
{
    my($array) = @_[0];
    my($count) = 0;

    foreach my $item (@$array) {$count++;}

    return $count;
}


sub extract_mo_coefficients
{
    my($orb_min)  = $_[0];
    my($orb_max)  = $_[1];
    my($coeff_nb) = $_[2];
    my($mo)       = @_[3];

    my(@tmp) = ();
    my($l)   = 0;
    my($orb) = 0;

    for($i = $orb_min; $i< $orb_max; $i++)
    {
	for($j=0; $j< $coeff_nb; $j++)
	{
	    $tmp[$orb]{$l} = $mo->[$i]{$j};
	    
	    $l++;
	}  
    
	$l = 0;
	$orb++;
    }

    return \@tmp;
}
