#! /usr/bin/perl

use Env qw(HOME PWD SYS LD_LIBRARY_PATH USER SLATER);
use English;
use locale;
use File::stat;
use File::Find;
use Getopt::Std;
use File::Basename;
use IO::Handle;

getopt("fhpq:",\%args);

@DIRLIST[0]  = "$SLATER";

my($dat_ext) = "dat";
my($aux_ext) = "aux";
my($bas_ext) = "bas";

my($input_file)  = $args{f}; # input file
my($help)        = $args{h}; # help
my($query)       = $args{q}; # query what basis sets are available
my($print_basis) = $args{p}; # print basis set

check_parameters($input_file, $SLATER, $query, $help);

my($file_dat, $file_aux, $base)                         = create_file_names($input_file, $dat_ext, $aux_ext);

check_file_exists($file_dat);

check_file_exists($file_aux);
    
my($mol_charge, $mult)                                  = parse_molecule($input_file);

my($diis, $quad)                                        = parse_scf($input_file);

my($x, $y, $z, $atom, $charge, $basis, $aux, $atoms_nb) = parse_geometry_file($input_file);

my($bas)                                                = create_string($basis, $bas_ext, $atoms_nb, $atom);

my($aux)                                                = create_string($aux, $aux_ext, $atoms_nb, $atom);

print_file_header($file_dat, $base);

process_file($file_aux, $aux, \@DIRLIST);

process_file($file_dat, $bas, \@DIRLIST, $atoms);

print_file_footer($file_dat, $atoms_nb, $diis, $mult);

sub print_to_screen
{

}

sub check_parameters
{
    my($file)   = $_[0];
    my($slater) = $_[1];
    my($query)  = $_[2];
    my($help)   = $_[3];

    help($help);

    if($slater eq "")
    {
	print "\n";
	print "\tneed to export SLATER variable, export SLATER= exiting...\n";
	print "\n";
	
	exit;
    }

    if( $query ne ""       ) { query_available_basis_sets($query,\@DIRLIST);   }
    if( $print_basis ne "" ) { find(\&process_file_to_screen, @DIRLIST); exit; }
    if( $file eq ""        )
    {
	print "\n";
	print "\tinput file needs to be specified, -f [string], exiting...\n";
	print "\n";

	exit;
    }
}

sub check_file_exists
{
    my($file_dat) = $_[0];

    if (-e $file_dat )
    { 
	print "\n";
	print "\t$file_dat exists, delete or rename file, exiting...\n";
	print "\n";

	exit;
    }
}

sub create_file_names
{
    my($input_file) = $_[0];
    my($dat_ext)    = $_[1];
    my($aux_ext)    = $_[2];

    my(@tmp)        = split(/\.geom/,$input_file);
    my($base)       = @tmp[0];
    my($file_dat)   = $base . "." . $dat_ext;
    my($file_aux)   = $base . "." . $aux_ext;

    return $file_dat, $file_aux, $base;
}

sub process_file
{
    my($file_aux)  = $_[0];
    my($aux)       = @_[1];
    my($dirlist)   = @_[2];
    my($atoms)     = @_[3];    

    open(AUX,"+>>$file_aux") or die "cannot create aux file\n";
    
    $count_aux = 0;
    foreach $item (@$aux)
    {

	if($item =~ m/\.aux/i)
	{
	    $count_aux++;
	    find(\&process_aux_file, @DIRLIST);
	}

	if($item =~ m/\.bas/i)
	{
	    find(\&process_basis_file, @DIRLIST);
	    $count_aux++;
	}
    }
    
    close(AUX);

}

sub process_file_to_screen
{
    if($_ eq $print_basis)
    {
	print "\n";
	print "\t\t$print_basis\n";
	print "\n";

	open(FILE,"$print_basis") or die "cannot open $item file\n";

	while(<FILE>)
	{
	    chomp $_;
	    print "\t\t$_\n";
	}

	close(FILE);

	print "\n";
    }
}

sub process_aux_file
{

    if($_ eq $item)
    {
	open(FILE,"$item") or die "cannot open $item file\n";
	line: while(<FILE>)
	{
	    chomp $_;
	    
	    if($_ eq "") { next line; }

	    if($_ =~ m/(\d+)\s+(\d+\.\d+)\s+(\d+)/i) { print AUX "$1\t$2\t$count_aux\n"; }

	    else { print AUX "$_\n"; }
	}
	close(FILE);
    }
}

sub print_file_header
{
    my($file) = $_[0];
    my($base) = $_[1];

    open(DAT,"+>>$file") or die "cannot create aux file\n";
    
    print DAT "TITLE\n";
    print DAT "$base\n";
    print DAT "SLATER\n";
    print DAT "ANGSTROM\n";
    print DAT "ATOMS\n";

    close(DAT);
}

sub print_file_footer
{
    my($file)     = $_[0];
    my($atoms_nb) = $_[1];
    my($diis)     = $_[2];
    my($mult)     = $_[3];

    open(DAT,"+>>$file") or die "cannot create aux file\n";

    if($atoms_nb >1) { print DAT "END\n"; }

    print DAT "END\n";
    print DAT "DFT\n";
    print DAT "KOHNSHAM\n";
    print DAT "EM\n";
    print DAT "LEBEDEV\n";
    print DAT "QUAD $quad\n";
    print DAT "EXCHANGE FITTED\n";
    print DAT "END\n";
    print DAT "ENERGY\n";
    
    if($mult > 1)
    {
	print DAT "UHF\n";
	print DAT "MULTIPLICITY $mult\n";
    }
    
    print DAT "DIIS $diis\n";
    print DAT "START\n";
    print DAT "FINISH\n";
    
    close(DAT);
}

sub create_string
{
    my($basis)     = @_[0];
    my($extension) = $_[1];
    my($atoms_nb)  = $_[2];
    my($atom)      = @_[3];
    my(@array)     = ();
    my($string)    = ();

    for($i=0;$i<$atoms_nb;$i++)
    {
	$string = $atom->[$i] . "-" .$basis->[$i] . ".". $extension;
	push(@array,$string);
    }

    return \@array;
}

sub process_basis_file
{
    if($_ eq $item)
    {
	open(FILE,"$item") or die "cannot open $item file\n";

	print AUX "$atom->[$count_aux] $charge->[$count_aux] $x->[$count_aux] $y->[$count_aux] $z->[$count_aux]\n";

	line: while(<FILE>)
	{
	    chomp $_;
	    
	    if($_ eq "") { next line;        }
	    else         { print AUX "$_\n"; }
	}

	print AUX "END\n";

	close(FILE);
    }
}

sub parse_molecule
{
    my($file) = $_[0];

    my($label)        = ();
    my($charge)       = ();
    my($multiplicity) = ();

    open(FILE,"$file") or die "cannot open input file\n";

    while(<FILE>)
    { if( $_ =~ m/mol/i ) { last; } }

    outer: while(<FILE>)
    {
	$line = $_;
	
	if    ( $_ =~ m/end_mol/i  ) { last;        }
	elsif ( $_ eq ""           ) { next: outer; }

	elsif ( $_ =~ m/(charge)/i )
	{
	    $label = $1;
	    $charge = parse_function($_,$label);
	}

	elsif ( $_ =~ m/(multiplicity)/i    )
	{
	    $label = $1;
	    $mult  = parse_function($_,$label);
	}
    }

    close(FILE);

    return $charge, $mult;
}


sub parse_scf
{
    my($file)       = $_[0];
    my($label)      = ();
    my($diis)       = ();
    my($quad)       = ();

    open(FILE,"$file") or die "cannot open input file\n";

    while(<FILE>)
    { if( $_ =~ m/scf/i ) { last; } }

    outer: while(<FILE>)
    {

	$line = $_;
	
	if    ( $_ =~ m/end_scf/i ) { last;        }
	elsif ( $_ eq ""          ) { next: outer; }

	elsif ( $_ =~ m/(diis)/i  )
	{
	    $label = $1;
	    $diis  = parse_function($_,$label);
	    $diis  = uc($diis);
	}

	elsif ( $_ =~ m/(quad)/i )
	{
	    $label = $1;
	    $quad  = parse_function($_,$label);
	}
    }

    close(FILE);

    return $diis, $quad;
}

sub parse_geometry_file
{
    my($file)   = $_[0];
    my($label)    = ();
    my($term)     = ();
    my(@x)        = ();
    my(@y)        = ();
    my(@z)        = ();
    my(@atom)     = ();
    my(@charge)   = ();
    my(@basis)    = ();
    my(@aux)      = ();
    my($atoms_nb) = ();

    open(FILE,"$file") or die "cannot open input file\n";

    while(<FILE>) { if( $_ =~ m/nuc/i ) { last; } }

    outer: while(<FILE>)
    {
	$line = $_;

	if    ( $_ =~ m/end_nuc/i      )   { last;                     }
	elsif ( $_ eq ""               )   { next: outer;              }
	elsif ( $_ =~ m/atom|nuc|end/i )   { $item eq ""; next: outer; }

	elsif($_ =~ m/(name)/i)
	{
	    $label = $1;
	    $item  = parse_function($_,$label);
	    $item  = ucfirst(lc($item));

	    push(@atom,$item);
	    $atoms_nb = @atom;
	}

	elsif($_ =~ m/(charge)/i)
	{
	    $label = $1;
	    $item  = parse_function($_,$label);

	    if($item =~ m/(\d+)\.(\d+)/i) { $item = $1; } 

	    push(@charge,$item);
	}
	
	elsif($_ =~ m/(basis)/i)
	{
	    $label = $1;
	    $item  = parse_function($_,$label);
	    push(@basis,$item);
	}

	elsif($_ =~ m/(aux)/i)
	{
	    $label = $1;
	    $item  = parse_function($_,$label);
	    push(@aux,$item);
	}

	elsif($_ =~ m/(coordinates)/i)
	{
	    $label = $1;
	    $item  = parse_function($_,$label);

	    $count = 0;
	    while($item =~ m/(-\d+\.\d+|\d+\.\d+)/ig)
	    {
		
		$count++;
		if($count == 1) { push(@x,$1); }
		if($count == 2) { push(@y,$1); }
		if($count == 3) { push(@z,$1); }
	    }
	}
    }
	
    close(FILE);

    return \(@x,@y, @z, @atom, @charge, @basis, @aux), $atoms_nb;
}

sub parse_function
{
    my($line) = $_[0];
    my($name) = $_[1];

    $line =~ m/($name\s+\=\s+|$name=\s+|$name=)([a-z0-9\-\_\+\.\,\(\) ]+)/ig;

    my($item) = $2;

    return $item;
}

sub query_available_basis_sets
{
    my($query)     = $_[0];
    my($dirlist)   = @_[1];
    my(@tmp)       = ();
    my(@tmp2)      = ();
    my($size)      = ();
    my($string1)   = ();
    my($string2)   = ();

    find(\&process_dir, @DIRLIST);

    if( @avail_aux > @avail_bas ) { $size = @avail_aux; }
    else                          { $size = @avail_bas; }
    
    @tmp = @avail_aux;
    
    print "\n";
    printf "%35s %35s\n","Basis", "Auxiliary";
    printf "%35s %35s\n","-----------","-----------";

  loop: for($i=0; $i<$size; $i++)
  {
      @avail_bas[$i] =~ m/($query\-[a-z0-9\-]+)/;
      $string1 = $1;

      for($j=0; $j<$size; $j++)
      {
	  @avail_aux[$j] =~ m/($query\-[a-z0-9\-]+)/;
	  $string2 = $1;
	  
	  if($string1 eq $string2)
	  {
	      if( @avail_aux[$j] eq "" ) { push(@tmp2,@avail_bas[$i]); }

	      if( @avail_aux[$j] ne "" )
	      {
		  printf "%35s %35s\n",@avail_bas[$i], @avail_aux[$j];
		  @tmp[$j] = "found";
	      }

	      next loop;
	  }
      }
  }

    print "\n";
    
    foreach $item (@tmp)
    {
	if($item !~ m/found/i) { printf "%35s %35s\n","none", $item; }
    }

    print "\n";
    
    foreach $item (@tmp2)
    {
	if($item !~ m/found/i) { printf "%35s %35s\n", $item, "none"; }
    }
    
    print "\n";

    exit;
}

sub process_dir
{
    if($_ =~ m/$query\-[a-z0-9\-]+\.bas/) { push(@avail_bas,$_); }
    if($_ =~ m/$query\-[a-z0-9\-]+\.aux/) { push(@avail_aux,$_); }
}

sub help
{
    if($help eq "help") { \&help_file(); exit;}
}

sub help_file
{    
    warn "script: @_\n" if @_;
    die <<EOF;

    Options:
           
           -h help     - prints this menu.

           -f [string] - input file, where string is the name of the file
                         that contains geometry and molecular information.
                         See below for further information.  The file name
                         must have an extension, e.g. C.geom., C.inp, etc.

           -q [string] - Query the basis library for available basis sets.
                         for a particular element, He, Fe, Mg, etc.
		      
    Example:
 
	   cadpac_basis_build -f C.geom

	   This will create the files C.dat and C.aux.  The file C.dat
           contains the CADPAC input for carbon atom, while C.aux contains
           the necessary auxiliary basis functions.

           cadpac_basis_build -q Fe

	   This will find all auxiliary and STO basis files for iron. The output
           will look similar to the following:


                              Basis                           Auxiliary
                        -----------                         -----------
                    Li-tz2p-adf.bas                     Li-tz2p-adf.aux
                     Li-tzp-adf.bas                      Li-tzp-adf.aux
                 Li-et-pvqz-adf.bas                  Li-et-pvqz-adf.aux
                      Li-sz-adf.bas                       Li-sz-adf.aux
                     Li-dzp-adf.bas                      Li-dzp-adf.aux
                      Li-dz-adf.bas                       Li-dz-adf.aux
                 Li-et-qz3p-adf.bas                  Li-et-qz3p-adf.aux
        Li-et-qz3p-1diffuse-adf.bas         Li-et-qz3p-1diffuse-adf.aux
        Li-et-qz3p-2diffuse-adf.bas         Li-et-qz3p-2diffuse-adf.aux
        Li-et-qz3p-3diffuse-adf.bas         Li-et-qz3p-3diffuse-adf.aux

                               none                       Li-cadpac.aux
                               none                   Li-cadpac-big.aux

                    Li-hf-sakai.bas                                none
             Li-hf-cusp-thakkar.bas                                none
                  Li-et-thakkar.bas                                none
                Li-nisz-thakkar.bas                                none
                  Li-sz-thakkar.bas                                none
                 Li-det-thakkar.bas                                none
                  Li-dz-thakkar.bas                                none
                  Li-hf-thakkar.bas                                none

    In addition, element and basis can be queries together, for example,

    cadpac_basis_build.pl -q He-hf

    produces the following:

                              Basis                           Auxiliary
                        -----------                         -----------


                    He-hf-sakai.bas                                none
             He-hf-cusp-thakkar.bas                                none
                  He-hf-thakkar.bas                                none


    Exporting Slater variable:

           The Slater basis sets must be stored somewhere in your home 
           directory, e.g, /home/sasha/lib/.

	   In order for the script to find the requested basis sets, the 
           .bashrc file (or shell equivalent) must be augmented to export 
           the SLATER variable.  This variable points to where the STO and 
           auxiliary basis functions are stored. For example:

                        export SLATER=$HOME/bin/basis/sto/cadpac

    Input file example:

           The following menus need to be included: scf, molecule, and nuclei.
 
           The scf menu information is given the number of quadrature points,
           the whether or not direct-inversion of iterative subspace (DIIS) 
           is used during the SCF calculation. Example:

           scf
	           diis = off
	           quad = 4
           end_scf

           The token value of diis is either "on" or "off".
           The token value of quad is 1, 2, 3, or 4. 
          
           The molecule menu contains molecule spin multiplicity and molecular
           charge. Example:

            molecule
	             charge       = 0
   	             multiplicity = 3
            end_molecule

            The molecular geometry is specified in the nuclei menu where each
            atom is started by "atom" and ended "end_atom".  The following 
            information must be included for each atom. element name (name)
            charge, coordinates (in x,y,z) format, basis and aux; each token
            must be followed by a value.  The basis and auxiliary (aux) designate
            these basis which will used in the calculation. These basis must be
            located in the directory specified by the $SLATER variable.  The
            option -q [element] produces the available basis for the specified
            element.  Example:

            nuclei
                  atom
                      name        = C
                      charge      = 6.0
                      coordinates = ( 0.0, 0.0, 0.0 )
                      basis       = hf-sakai
                      aux         = cadpac-big
                  end_atom
            end_nuc

EOF
}
