#! /usr/bin/perl

my($file) = $ARGV[0];

my($jast_pow_max) = 4;
my($ee_nb)        = $jast_pow_max;
my($en_nb)        = $jast_pow_max;
my($een_nb)       = 2;

my($string_psi)    = "psi";
my($string_mos)    = "mos";
my($string_geom)   = "geom";
my($string_bas)    = "bas";
my($string_dat)    = "dat";
my($string_jas_in) = "jas-in";
my($string_inp)    = "inp";
my($string_out)    = "out";

my($base) = create_base($file);

my($files) = create_output_files($base);

my($doc_nb, $alpha_nb, $beta_nb, $charge, $basis_nb, $atoms_nb, $electrons_nb) = parse_parameters($file);

my($alpha_nb, $beta_nb, $orbital_nb) = create_wave_function($doc_nb, $alpha_nb, $beta_nb);

my($atom_charge, $atom_label, $atom_position_x, $atom_position_y, $atom_position_z) = parse_geometry($file);

my($orbital_array) = parse_orbitals($basis_nb, $files, $string_out);

my($alpha_ee, $alpha_en, $alpha_een, $beta_en) = create_jastrow($atoms_nb, $ee_nb, $en_nb, $een_nb);

print_wave_function($alpha_nb, $beta_nb, $charge, $files, $string_psi);

print_orbitals($orbital_array, $orbital_nb, $basis_nb, $files, $string_mos);

print_geometry($atom_charge, $atom_label, $atom_position_x, $atom_position_y, $atom_position_z, $atoms_nb, $files, $string_geom);

parse_basis($files, $string_bas, $string_dat);

output_molec_param($electrons_nb, $atoms_nb, $charge, $alpha_nb, $beta_nb, $doc_nb, $files);

print_jastrow_file($atoms_nb, $ee_nb, $atom_label, $file_jas, $alpha_ee, $alpha_en, $beta_en, $alpha_een, $alpha_en, $een_nb, $en_nb,$files, $string_jas_in);

print_qmcmol_input_file($files, $string_inp, $string_geom, $string_psi);

sub create_base
{
    my($file) = $_[0];

    $file =~ m/([a-z0-9\_\-\.\+]+)\.out/i;
   
    return $1;
}

sub print_qmcmol_input_file
{
    my($files)       = @_[0];
    my($string_inp)  = $_[1];
    my($string_geom) = $_[2];
    my($string_psi)  = $_[3];

    my($file_inp)    = find_file($files, $string_inp);
    my($file_geom)   = find_file($files, $string_geom);
    my($file_psi)    = find_file($files, $string_psi);
						   
    open(INP, ">$file_inp") or die "cannot create qmcmol input file\n";

    foreach $item (@$files)
    {
	my($header_string, $type, $status, $action) = file_menu_parameters($item);
	
	if($type ne "")
	{
	    print_file_menu($item, $header_string, $type, $status, $action);
	}
    }
    
    print INP "!----------------------------------------------------------!\n";
    print INP "! Molecular geometry                                       !\n";
    print INP "!----------------------------------------------------------!\n"; 
    print INP "\n";
    print INP "include $file_geom\n";
    print INP "\n";
    print INP "!----------------------------------------------------------!\n";
    print INP "! Number of alpha/beta electrons                           !\n";
    print INP "!----------------------------------------------------------!\n";
    print INP "\n";
    print INP "include $file_psi\n";
    print INP "\n";
    print INP "!----------------------------------------------------------!\n";
    print INP "! VMC expressions                                          !\n";
    print INP "!----------------------------------------------------------!\n";
    print INP "\n";
    print INP "objects\n";
    print INP "\tel          = real ( alias ( e_loc_tot_bywlk ) )\n";             
    print INP "\tenergy_av   = real ( av(el ) )\n";              
    print INP "\tenergy_blk  = real ( av ( blk ( el ) ) )\n";            
    print INP "\tenergy_var  = real ( av ( blk ( el*el ) ) - energy_blk^2. )\n";        
    print INP "\tprob_acc    = real ( av ( blk ( prob_accept_bywlk    ) ) )\n";            
    print INP "end_obj\n";
    print INP "\n";
    print INP "!----------------------------------------------------------!\n";
    print INP "! Variational Monte Carlo commands                         !\n";
    print INP "!----------------------------------------------------------!\n";
    print INP "\n";
    print INP "montecarlo\n";
    print INP "\n";
    print INP "\ttime_step   = 0.05\n";
    print INP "\truntype     = VMC\n";
    print INP "\tbloc        = 1000\n";
    print INP "\tsteps       = 200\n";
    print INP "\twalkers     = 1000\n";
    print INP "\treconfigure = false\n";
    print INP "\n";
    print INP "\tout\n";
    print INP "\t\tlist = (\n";
    print INP "\t\tenergy_av,\n";           
    print INP "\t\tenergy_blk,\n";          
    print INP "\t\terr(energy_av),\n";      
    print INP "\t\tenergy_var,\n";          
    print INP "\t\terr(energy_var),\n";     
    print INP "\t\tprob_acc\n";             
    print INP "\t\t)\n";
    print INP "\n";
    print INP "\tend_out\n";
    print INP "\n";
    print INP "end_montecarlo\n";
    print INP "\n";
    print INP "stop\n";

    close(INP);
}

sub print_file_menu
{
    my($file_type)     = $_[0];
    my($header_string) = $_[1];
    my($type)          = $_[2];
    my($status)        = $_[3];
    my($action)        = $_[4];

    print  INP "!----------------------------------------------------------!\n";
    printf INP "! %30s %27s\n",$header_string, "!";
    print  INP "!----------------------------------------------------------!\n";
    print  INP "\n";
    print  INP "file\n";
    print  INP "\tname    = $file_type\n";
    print  INP "\ttype    = $type\n";
    print  INP "\tstatus  = $status\n";
    print  INP "\taction  = $action\n";

    if( ( $file_type =~ m/\.bas/i ) or ( $file_type =~ m/\.mos/i ) )
    {
	print INP "\tprogram = qmcmol\n";
    }

    print INP "end_file\n";
    print INP "\n";
}

sub file_menu_parameters
{
    my($file_type)     = $_[0];

    my($header_string) = ();
    my($type)          = ();
    my($status)        = ();
    my($action)        = ();

    if( $file_type =~ m/\.mos/i ) 
    { 
	$header_string = "MO coefficients input file";
	$type          = "mos";
	$status        = "old";
	$action        = "read";
    }
    
    elsif( $file_type =~ m/\.bas/i ) 
    { 
	$header_string = "Basis set input file"; 
	$type          = "epfs";
	$status        = "old";
	$action        = "read";
    }
    
    elsif( $file_type =~ m/\.wlk-in/i )
    {
	$header_string = "Input walker file";
	$type          = "walkers";
	$status        = "old";
	$action        = "read";
    }

    elsif( $file_type =~ m/\.wlk-out/i )
    {
	$header_string = "Output walker file";
	$type          = "walkers";
	$status        = "unknown";
	$action        = "write";
    }

    elsif( $file_type =~ m/\.par-in/i )
    {
	$header_string = "Input parameter file";
	$type          = "objects";
	$status        = "unknown";
	$action        = "read";
    }

    elsif( $file_type =~ m/\.par-out/i )
    {
	$header_string = "Output parameter file";
	$type          = "objects";
	$status        = "unknown";
	$action        = "write";
    }

    elsif( $file_type =~ m/\.jas-in/i )
    {
	$header_string = "Jastrow parameter input file";
	$type          = "jastrow";
	$status        = "unknown";
	$action        = "read";
    }

    elsif( $file_type =~ m/\.jas-out/i )
    {
	$header_string = "Jastrow parameter output file";
	$type          = "jastrow";
	$status        = "unknown";
	$action        = "write";
    }

    return $header_string, $type, $status, $action;
    
}

sub output_molec_param
{
    my($electrons_nb) = $_[0];
    my($atoms_nb)     = $_[1];
    my($charge)       = $_[2];
    my($alpha_nb)     = $_[3];
    my($beta_nb)      = $_[4];
    my($doc_nb)       = $_[5];
    my($files)        = $_[6];

    print "\n";
    print "\tnumber of electrons                      : $electrons_nb\n";
    print "\tnumber of atoms                          : $atoms_nb\n";
    print "\tmolecular charge                         : $charge\n";
    print "\tnumber of singly occupied alpha orbitals : $alpha_nb\n";
    print "\tnumber of singly occupied beta orbitals  : $beta_nb\n";
    print "\tnumber of doubly occupied orbitals       : $doc_nb\n";

    print "\n";
    print "CADPAC files:\n";
    print "\n";

    foreach $item (@$files)
    {
	if($item =~ m/\.out/i)    { print "\toutput file                              : $item\n"; }
	if($item =~ m/\.dat/i)    { print "\tinput file                               : $item\n"; }
    }

    print "\n";
    print "QMCMOL files:\n";
    print "\n";

    foreach $item (@$files)
    {
	if($item =~ m/\.bas/i)    { print "\tbasis file                               : $item\n"; }
	if($item =~ m/\.inp/i)    { print "\tinput file                               : $item\n"; }
	if($item =~ m/\.psi/i)    { print "\twave function file                       : $item\n"; }
	if($item =~ m/\.mos/i)    { print "\tmolecular orbital file                   : $item\n"; }
	if($item =~ m/\.geom/i)   { print "\tgeometry file                            : $item\n"; }
	if($item =~ m/\.jas-in/i) { print "\tjastrow input file                       : $item\n"; }
    }

    print "\n";
    print "QMCMOL files need to be created:\n";
    print "\n";

    foreach $item (@$files)
    {
	if($item =~ m/\.par-in/i) { print "\tparameter input file                     : $item\n"; }
	if($item =~ m/\.wlk-in/i) { print "\twalker input file                        : $item\n"; }
    }

    print "\n";
}

sub create_jastrow
{
    my($atoms_nb) = $_[0];
    my($ee_nb)    = $_[1];
    my($en_nb)    = $_[2];
    my($een_nb)   = $_[3];

    my(@alpha_ee)  = ();
    my(@alpha_en)  = ();
    my(@alpha_een) = ();
    my(@beta_en)   = ();
    my(@beta_en)   = ();

    my($beta_ee1)   = 1.0;
    my($beta_ee2)   = 1.0;

    for($i=0; $i<$ee_nb; $i++) 
    { 
	@alpha_ee[$i] = 0.0; 
    }
    
    for($atom_i=0; $atom_i<$atoms_nb; $atom_i++)
    {
	for($i=0; $i<$en_nb; $i++) 
	{ 
	    $alpha_en[$atom_i]{$i}  = 0;
	}
    }
    
    for($atom_i=0; $atom_i<$atoms_nb; $atom_i++)
    {
	for($i=0; $i<$een_nb; $i++) 
	{ 
	    $alpha_een[$atom_i]{$i}  = 0;
	}
    }

    for($atom_i=0; $atom_i<$atoms_nb; $atom_i++)
    {
	 @beta_en[$atom_i] = 1.0;
    }

    return \@alpha_ee, \@alpha_en, \@alpha_een, \@beta_en;
}

sub print_jastrow_file
{
    my($atoms_nb)   = $_[0];
    my($num_ee)     = $_[1];
    my($atom_label) = @_[2];
    my($file_jas)   = $_[3];
    my($alpha_ee)   = @_[4];
    my($alpha_en)   = @_[5];
    my($beta_en)    = @_[6];
    my($alpha_een)  = @_[7];
    my($alpha_en)   = @_[8];
    my($num_een)    = @_[9];
    my($num_en)     = @_[10];
    my($files)      = @_[11];
    my($string)     = @_[12];

    my($file_jas)   = find_file($files, $string);

    open(JAS, ">$file_jas") or die "cannot create jastrow file\n";

    printf JAS "title ?\n";
    printf JAS "\n";
    printf JAS "number of nuclei         =     $atoms_nb\n";
    printf JAS "number of configurations =     1\n";
    printf JAS "end_of_header\n";
    printf JAS "CONFIGURATION = CONF_1\n";
    printf JAS "\n";
    printf JAS "BETA_EE     = %18.15e %18.15e\n",$beta, $beta;
    printf JAS "A_EE_LIKE   = ";

    for($i=0; $i<$num_ee; $i++)
    {
	printf JAS "%18.15e",$alpha->[$i];
    }

    print JAS "\n";
    printf JAS "A_EE_UNLIKE = ";

    for($i=0; $i<$num_ee; $i++)
    {
	printf JAS "%18.15e",$alpha->[$i];
    }

    print JAS "\n";

    for($atoms_i=0; $atoms_i<$atoms_nb; $atoms_i++)
    {
	my($j) = $atoms_i+1;

	printf JAS "ATOM_NAME   = $atom_label->[$atoms_i]$j\n"; 
	printf JAS "BETA_EN     = %18.15e\n", $beta_en->[$atoms_i];
	printf JAS "A_EN        = ";

	for($j=0; $j<$num_en; $j++)
	{
	    printf JAS "%18.15e", $alpha_en->[$atoms_i]{$j};
	}
	
	printf JAS "\n";
	printf JAS "A_EEN       = ";

	for($j=0; $j<$num_een; $j++)
	{
	    printf JAS "%18.15e", $alpha_een->[$atoms_i]{$j};
	}
	
	print JAS "\n";

    }

    printf JAS "\n";
    printf JAS "END\n";
    printf JAS "\n";
    printf JAS "eof\n";
    
    close(JAS);
}

sub create_output_files
{
    my($base)      = $_[0];
    my(@files)     = ();
    
    @files[0]  = $base . ".psi"; 
    @files[1]  = $base . ".bas";
    @files[2]  = $base . ".mos";
    @files[3]  = $base . ".geom";
    @files[4]  = $base . ".inp";
    @files[5]  = $base . ".jas-in";
    @files[6]  = $base . ".jas-out";
    @files[7]  = $base . ".par-in";
    @files[8]  = $base . ".par-out";
    @files[9]  = $base . ".wlk-in";
    @files[10] = $base . ".wlk-out";
    @files[11] = $base . ".dat";
    @files[12] = $base . ".out";
    return \@files;
}

sub print_orbitals
{
    my($orbital_array) = @_[0];
    my($orbital_nb)    = $_[1];
    my($basis_nb)      = $_[2];
    my($files)         = @_[3];
    my($string)        = $_[4];

    my($file) = find_file($files, $string);

    open(MOS, ">$file") or die "cannot create mo file\n";
    
    print MOS "mos_on_what = aos_c\n";
    print MOS "end_of_header\n";
    print MOS "\n";

    for($i=0; $i<$orbital_nb; $i++)
    {
	print MOS "mo\n";

	for($j=1; $j<=$basis_nb; $j++)
	{
	    if( ($j%5 == 0) && ($j > 0) )
	    {
		printf MOS "%18.10e\n", $orbital_array->[$i]{$j-1};
	    }
	    else
	    {
		printf MOS "%18.10e", $orbital_array->[$i]{$j-1};
	    }

	    if($j == $basis_nb) { print MOS "\n"; }
	}
	
	print MOS "end_mo\n";
	print MOS "\n";

    }

    print MOS "end_of_mos\n";

    close(MOS);
}

sub parse_orbitals
{
    my($basis_nb) = $_[0];
    my($files)    = @_[1];
    my($string)   = $_[2];

    my($orbital_count) = 0;
    my($increment)     = 0;
    my($orb)           = 0;
    my($num_orb_found) = 0;
    my($bf)            = 0;
    my($count)         = 0;
    my(@orbital_array) = ();

    my($file) = find_file($files, $string);

    open(FILE, "$file") or die "cannot open cadpac output file\n";

    while(<FILE>)
    {
	if( $_ =~ m/eigenvectors/i ) { last; }
    }

    outer: while(<FILE>)
    {
	chomp $_;
	if( $_ =~ m/eigenvalues/i) { last; }

	if( $_ eq "") 
	{ 
	    $_ =<FILE>;
	    $orb           = $orb + $count;
	    $orbital_count = 0;
	    next outer; 
	}

	if( $_ =~ m/([0-9]+)\s+[a-z]+\s+([a-z0-9]+)/i )
	{
	   
	    $bf = $1 - 1;
	    $count  = 0;
	
	  inner: while( $_ =~ m/(\d+\.\d+|-\d+\.\d+)/ig )
	  {
	      chomp $_;

	      $count++;
	      $num_orb_found                      = $count + $orb - 1;
	      $orbital_array[$num_orb_found]{$bf} = $1; 
	      $orbital_count++;
	  }
	}
    }
    
    close(FILE);
    
    return \@orbital_array; 
}

sub find_file
{
    my($files)  = @_[0];
    my($string) = $_[1];
    
    my($file)   = ();
    
    foreach $item (@$files)
    {
	if( $item =~ m/\.$string/i ) 
	{ 
	    $file = $item; 
	    last;
	}
    }
    
    return $file;
}

sub print_wave_function
{
    my($alpha_nb) = $_[0];
    my($beta_nb)  = $_[1];
    my($charge)   = $_[2];
    my($files)    = @_[3];
    my($string)   = $_[4];

    my($file_wfn) = find_file($files, $string);

    open(WFN, ">$file_wfn") or die "cannot create wave function file\n";

    print WFN "!----------------------------------------------------------!\n";
    print WFN "! Number of alpha/beta electrons                           !\n";
    print WFN "!----------------------------------------------------------!\n";
    print WFN "\n";
    print WFN "\telec\n";                                           
    print WFN "\t\tcharge = $charge\n";                                       
    print WFN "\t\talfa   = $alpha_nb\n"; 
    print WFN "\t\tbeta   = $beta_nb\n"; 
    print WFN "\tend_elec\n";                                       
    print WFN "\n";
    print WFN "!----------------------------------------------------------!\n";
    print WFN "! Wave function                                            !\n";
    print WFN "!----------------------------------------------------------!\n";
    print WFN "\n";
    print WFN "\twavefunction\n";
    print WFN "\t\tconfiguration\n";
    print WFN "\t\t\tname      = CONF_1\n";
    print WFN "\t\t\tprint     = false\n";
    print WFN "\t\t\tjastrow   = none\n"; 
    print WFN "\t\t\tweight    = 1.0\n";
    print WFN "\t\t\tpower_max = 4\n"; 
    print WFN "\n";
    print WFN "\t\t\tdeterminant\n";
    print WFN "\t\t\t\talfa_orbitals = (";

    for($i=1; $i<=$alpha_nb; $i++)
    {
	if($i == $alpha_nb) { print WFN "$i";  }
	else                { print WFN "$i,"; }
    }
    
    print WFN ")\n";
    print WFN"\t\t\t\tbeta_orbitals = (";

    for($i=1; $i<=$beta_nb; $i++)
    {
	if($i == $beta_nb) { print WFN "$i";  }
	else               { print WFN "$i,"; }
    }

    print WFN ")\n";
    print WFN "\t\t\t\tweight        = 1\n";
    print WFN "\t\t\tend_determinant\n";
    print WFN "\t\tend_conf\n";
    print WFN "\tend_wavefunction\n";

    close(WFN);
}

sub print_geometry
{
    my($atom_charge)     = @_[0];
    my($atom_label)      = @_[1];
    my($atom_position_x) = @_[2];
    my($atom_position_y) = @_[3];
    my($atom_position_z) = @_[4];
    my($atoms_nb)        = $_[5];
    my($files)           = @_[6];
    my($string    )      = $_[7];

    my($file_geom)       = find_file($files, $string);

    open(GEOM, ">$file_geom") or die "cannot create geometry file\n";
    
    print GEOM "!----------------------------------------------------------!\n";
    print GEOM "! Molecular geometry                                       !\n";
    print GEOM "!----------------------------------------------------------!\n";
    print GEOM "\n";
    print GEOM "nuclei\n";

    for($i=0; $i< $atoms_nb; $i++)
    {
	my($j) = $i+1;

	print GEOM "\tatom\n";
	print GEOM "\t\tname        = $atom_label->[$i]$j\n";
	print GEOM "\t\tsymbol      = $atom_label->[$i]\n";
	print GEOM "\t\tcharge      = $atom_charge->[$i]\n";
	print GEOM "\t\tcoordinates = ( $atom_position_x->[$i], $atom_position_y->[$i], $atom_position_z->[$i] )\n";
	print GEOM "\t\tpseudo      = all\n";
        print GEOM "\tend_atom\n";
	print GEOM "\n";
    }

    print GEOM "end_nuclei\n";

    close(GEOM);
}

sub create_wave_function
{
    my($doc_nb)       = $_[0];
    my($alpha_nb)     = $_[1];
    my($beta_nb)      = $_[2];
   
    my($single)     = $doc_nb;
    my($alpha)      = $single + $alpha_nb;
    my($beta)       = $single + $beta_nb;   
    my($orbital_nb) = $single + $alpha_nb + $beta_nb;

    return $alpha, $beta, $orbital_nb;
}

sub parse_parameters
{
    my($file)         = $_[0];
    my($electrons_nb) = ();
    my($charge)       = ();
    my($atoms_nb)     = ();
    my($doc_nb)       = ();
    my($alpha_nb)     = ();
    my($beta_nb)      = ();
    my($basis_nb)     = ();

    open(FILE, "$file") or die "cannot open input file\n";

    while(<FILE>)
    {
	chomp $_;
	
	if( $_ =~ m/number of electrons\s+\=\s+(\d+)/i                      ) { $electrons_nb = $1; }
	if( $_ =~ m/charge on molecule\s+\=\s+(\d+)/i                       ) { $charge       = $1; }
	if( $_ =~ m/number of atoms\s+\=\s+(\d+)/i                          ) { $atoms_nb     = $1; }
	if( $_ =~ m/number of doubly occupied orbitals\s+\=\s+(\d+)/i       ) { $doc_nb       = $1; }
	if( $_ =~ m/number of singly occupied alpha orbitals\s+\=\s+(\d+)/i ) { $alpha_nb     = $1; }
	if( $_ =~ m/number of singly occupied beta orbitals\s+\=\s+(\d+)/i  ) { $beta_nb      = $1; }
	if( $_ =~ m/number of basis functions\s+\=\s+(\d+)/i                ) { $basis_nb     = $1; }
    }

    close(FILE);

    return $doc_nb, $alpha_nb, $beta_nb, $charge, $basis_nb, $atoms_nb, $electrons_nb;
}

sub parse_geometry
{
    my($file) = $_[0];

    my($atom_tmp)        = ();
    my($atom)            = ();
    my($atom_label)      = ();

    my(@atom_charge)     = ();
    my(@atom_position_x) = ();
    my(@atom_position_y) = ();
    my(@atom_position_z) = ();

    open(FILE, "$file") or die "cannot open input file\n";

    while(<FILE>)
    {
	chomp $_;
	if( $_ =~ m/basis set/i          ) { last; }
	if( $_ =~ m/charge\s+x\s+y\s+z/i ) { last; }
    } 

    while(<FILE>)
    {
	chomp $_;
	if($_ =~ m/basis set/i) { last; }

	if($_ =~ m/(([a-z])+|([a-z]+[0-9]+))/i)
	{
	    $atom_tmp = $1;
	    $atom_tmp =~ m/([a-z]+)/i;

	    push(@atom_label,$1);

	    $_ =~ m/(\d+.\d+)\s+(-\d+.\d+|\d+.\d+)\s+(-\d+.\d+|\d+.\d+)\s+(-\d+.\d+|\d+.\d+)/i;

	    push(@atom_charge,$1);
	    push(@atom_position_x,$2);
	    push(@atom_position_y,$3);
	    push(@atom_position_z,$4);
	}
    } 

    close(FILE);

    return \(@atom_charge, @atom_label, @atom_position_x, @atom_position_y, @atom_position_y);
}

sub parse_basis
{
    my($files)   = @_[0];
    my($string1) = $_[1];
    my($string2) = $_[2];

    my($file_basis) = find_file($files, $string1);
    my($file_dat)   = find_file($files, $string2);

    my($count) = ();
    my($elem)  = ();
    my($basis) = ();

    open(FILE, "$file_dat") or die "cannot open input file\n";
    open(BASIS, ">$file_basis") or die "cannot create basis file\n";

    print BASIS "title :\n";
    print BASIS "end_of_header\n";
    print BASIS "\n";

    while( <FILE> ) { if($_ =~ m/atoms/i) { last; } }

    while( <FILE> )
    {
	if($_ =~ m/([a-z]+)\s+(\d+)\s+(\d+\.\d+|-\d+\.\d+)\s+(\d+\.\d+|-\d+\.\d+)\s+(\d+\.\d+|-\d+\.\d+)/i)
	{
	    $count=0;
	    $elem = ucfirst(lc($1));
	    print BASIS "\telem $elem\n";
	    print BASIS "\n";
	}

	if( ($_ =~ m/end/i) && ($count == 0) ) 
	{
	    $count++;
	    print BASIS "\tend elem\n"; 
	    print BASIS "\n";
	}

	if($_ =~ m/1\s+([SPDF]|(\d+)[SPDF])\s+1/i)
	{
	    $basis   = $2;
	    $angular = $1;
	    $angular =~ m/([SPDF])/i;
	    $angular = $1;

	    if($basis eq "") { $basis = 1; }

	    $line = <FILE>;

	    chomp $line;
	    
	    $line     =~ m/\d+\s+(\d+\.\d+)\s+\d+/i;
	    $exponent = $1;
	    
	    print BASIS "\t\tsym $angular\n";
	    print BASIS "\t\t\tsto $basis $exponent 1.0\n";
	    print BASIS "\t\tend sym\n";
	    print BASIS "\n";
	}
    }
 
    print BASIS "end_of_basis\n";

    close(BASIS);
    close(FILE);
}
