#! /usr/bin/perl

my($param_file) = @ARGV[0];
my($data_file)  = @ARGV[1];

my($values_array, 
   $parameters_array, 
   $expression,
   $variables,
   $distance)  = parse_parameter_file($param_file);

my($hash)      = build_hash($parameters_array, $values_array);

my($string)    = reformat_string($expression);

my($build)     = create_expression($string, $variables, $hash);

my($function_value) = evaluate_expression($build, $distance);

print_output($distance, $function_value);

sub print_output
{
    my($distance)       = @_[0];
    my($function_value) = @_[1];

    for($i=0;$i<@$distance; $i++)
    {
	printf "%15.10f %15.10f\n", $distance->[$i], $function_value->[$i];
    }
}

sub reformat_string
{
    my($string) = $_[0];

    $string =~ s/\-/ \- /ig;
    $string =~ s/\*/ \* /ig;
    $string =~ s/\+/ \+ /ig;
    $string =~ s/\// \/ /ig;
    $string =~ s/\(/ \( /ig;
    $string =~ s/\)/ \) /ig;

    return $string;
}

sub evaluate_expression
{
    my($build)      = $_[0];
    my($distance)   = @_[1];

    my(@function_value) = ();
    my($number)         = ();
    my($expression)     = $build;

    foreach $item (@$distance)
    {
	$expression  =~ s/r/$item/ig;
	$number      = eval $expression;
	$expression  = $build;
	
	push(@function_value, $number);
    }
    
    return \@function_value;
}

sub create_expression
{
    my($string)   = $_[0];
    my($variable) = @_[1];
    my($hash)     = $_[2];

    my($build)    = ();

    while($string =~ m/([a-z0-9\/\+\*\)\(\-\.]+)/ig)
    {
	$item = $1;
    
	foreach $var (@$variable)
	{
	    if ( $item eq $var ) 
	    { 
		$build = $build . $item;
	    }
	}

	if ( $item =~ m/(exp)/i    ) { $build = $build . $item; }
	
	elsif ( $item !~ m/([a-z]+)/i ) { $build = $build . $item; }
	
	else
	{
	    foreach $param (keys %$hash)
	    {
		if($param eq $item) { $build = $build . $hash->{$param}; }
	    }
	}
    }

    return $build;
}

sub parse_parameter_file
{
    my($param_file) = $_[0];

    my($expression)       = ();
    my(@values_array)     = ();
    my(@parameters_array) = ();
    my(@variable)         = ();
    my(@distance)         = ();

    open(PARAM,"$param_file") or die "cannot open input file\n";

    while(<PARAM>)
    {
	chomp $_;

	if($_ =~ m/param/i)
	{
	    while(<PARAM>)
	    {
		chomp $_;

		if($_ =~ m/end_param/i) { last; }

		my($values)     = parse_real_number($_);

		my($parameters) = parse_characters($_);
		
		push(@values_array, $values);

		push(@parameters_array, $parameters);
	    }
	}
	
	if($_ =~ m/expression/i)
	{
	    while(<PARAM>)
	    {
		chomp $_;

		if($_ =~ m/end_expression/i) { last; }

		$expression = parse_string($_);
	    }
	}

	if($_ =~ m/variable/i)
	{
	    while(<PARAM>)
	    {
		chomp $_;
 
		if($_ =~ m/end_variable/i) { last; }

		my($variables) =  parse_string($_);
	        $variables     =~ m/([0-9a-z]+)/i;
		push(@variable, $1);
	    }
	}

	if($_ =~ m/distance/i)
	{
	    while(<PARAM>)
	    {
		chomp $_;
 
		if($_ =~ m/end_distance/i) { last; }

		my($values)     = parse_real_number($_);
		
		push(@distance,$values);
	    }
	}

    }

    close(PARAM);

    return \@values_array, \@parameters_array, $expression, \@variable, \@distance;
}

sub build_hash
{
    my($keys)   = @_[0];
    my($values) = @_[1];

    my(%hash)   = ();

    for($i=0; $i<@$values; $i++) { $hash{"$keys->[$i]"} = $values->[$i]; }
    
    return \%hash;
}

sub parse_string
{
    my($item) = $_[0];

    $item =~ m/([a-z0-9\.\-\+\*\/\(\) ]+)/i;

    return $1;
}

sub parse_characters
{
    my($item) = $_[0];

    $item =~ m/([a-z]+)/i;

    return $1;
}

sub parse_real_number
{

    my($item) = $_[0];

    $item =~ m/(\d+.\d+|-\d+.\d+)/i;
    
    return $1;
}
