#! /usr/bin/perl

use regular_expressions;
use cascade;
use tools;

my($file)        = @ARGV[0];

my($character)   = regular_expressions->string_type(character);

my($integer)     = regular_expressions->string_type(integer);

my($real)        = regular_expressions->string_type(real);

my($element)     = get_element($file);

my($whose)       = get_whose($file);

my($STO_DATA)    = cascade->CASCADE($file);

my($slater_data) = $STO_DATA->{array};

my($hash)        = parse_cascade_to_hash($slater_data);

my($hash)        = adjust_r_power($hash);

print_qmcmol_basis_format($hash);

###########################################################################

sub get_whose
{
    my($file) = $_[0];

    $file =~ m/[a-z]+\-([a-z]+)/i;

    return $1;
}

sub get_element
{
    my($file) = $_[0];

    $file     =~ m/($character)\-/i;
    
    return $1;
}

sub print_qmcmol_basis_format
{
    my($hash) = @_[0];
    
    print "title: $whose basis $element atom\n";
    print "end_of_header\n";
    print "elem $element\n";
    print "\n";

    for my $role (sort keys %$hash)
    {
	my($symmetry) = $hash->{$role}->{symmetry};
	
	my($r_power)  = $hash->{$role}->{"r_power"};
	
	my($exponent) = $hash->{$role}->{exponent};

	print "sym $symmetry\n";
	print "sto $r_power $exponent 1.\n";
	print "end_sym\n";
	print "\n";
    }
    
    print "end_elem\n";
    print "end_of_basis\n";
}


sub adjust_r_power
{
    my($hash) = @_[0];

    for my $role (sort keys %$hash)
    {
	my($symmetry) = $hash->{$role}->{symmetry};
    
	my($r_power)  = $hash->{$role}->{r_power};

	if($symmetry =~ m/s/i) 
	{
	    $hash->{$role}->{"r_power"} = $r_power;
	}
	
	elsif($symmetry =~ m/p/i)
	{
	    $hash->{$role}->{"r_power"} = $r_power - 1;
	}
	
	elsif($symmetry =~ m/d/i)
	{
	    $hash->{$role}->{"r_power"} = $r_power - 2;
	}

	elsif($symmetry =~ m/f/i)
	{
	    $hash->{$role}->{"r_power"} = $r_power - 3;
	}
    }

    return $hash;
}

sub parse_cascade_to_hash
{
    my($slater_data) = @_[0];

    my($line_i)      = 0;
    
    my($count)       = 0;

    my(%hash)        = ();

    foreach my $line (@$slater_data)
    {
	if( $line =~ m/symmetry\s+\=\s+($character)/i )
	{   
	    $line_i++;
	    
	    if($line_i < 10) { $count = tools->cat_string(0, $line_i); }
	    
	    else             { $count = $line_i; }	

	    $hash{$count}{symmetry} = $1;
	}
	
	if( $line =~ m/r_power\s+\=\s+($integer)/i )
	{
	    $hash{$count}{"r_power"} = $1;
	}
	
	if( $line =~ m/exponent\s+\=\s+($real)/i )
	{
	    $hash{$count}{exponent} = $1;
	}   
    }

    return \%hash;
}
