#! /usr/bin/perl

use Getopt::Std;
use atom_parser;
use array;

getopt("befh",\%args);

my($input_file) = $args{f};
my($base)       = $args{b};
my($extension)  = $args{e};
my($help)       = $args{h};

if($help eq "help") { help(); }

check_arguments();

my($DATA)  = array->ARRAY($input_file);

foreach $file_name (@{$DATA->{array}})
{
    $file_name =~ m/([a-z]+)/i;

    $string = $1;

    $ATOM = atom_parser->ATOM_DATA($string,"query");

    $label = $ATOM->{element_label};

    $new_file_name = $label . "-" . $base . ".dat";

    print "$file_name ---> $new_file_name\n";
   
    system("cp $file_name $new_file_name");
}

sub help
{
    warn "change-basis-file-name: @_\n" if @_;
    die <<EOF;

  usage:  change-basis-file-name [options]

Options are:
      -f [string] list file
      -b [string] base file name
      -e [string] extension

      There are no default for these options. The array.pm module is needed
      to use the script. The list file contains a list of slater basis 
      file names where the first is the is the element name. e.g.
      
      Boron-clementi-hf-neutral-ground-adnd.dat
      Carbon-clementi-hf-neutral-ground-adnd.dat
      Fluorine-clementi-hf-neutral-ground-adnd.dat
      Helium-clementi-hf-neutral-ground-adnd.dat
      Lithium-clementi-hf-neutral-ground-adnd.dat
      Neon-clementi-hf-neutral-ground-adnd.dat
      Nitrogen-clementi-hf-neutral-ground-adnd.dat
      Oxygen-clementi-hf-neutral-ground-adnd.dat
      Sodium-clementi-hf-neutral-ground-adnd.dat
      Xenon-clementi-hf-neutral-ground-adnd.dat
      
    Example: 
      
      ./change-basis-file-name.pl -f list -b test -e dat
      copies the files found above in list to
      
      Boron-clementi-hf-neutral-ground-adnd.dat ---> B-test.dat
      Carbon-clementi-hf-neutral-ground-adnd.dat ---> C-test.dat
      Fluorine-clementi-hf-neutral-ground-adnd.dat ---> F-test.dat
      Helium-clementi-hf-neutral-ground-adnd.dat ---> He-test.dat
      Lithium-clementi-hf-neutral-ground-adnd.dat ---> Li-test.dat
      Neon-clementi-hf-neutral-ground-adnd.dat ---> Ne-test.dat
      Nitrogen-clementi-hf-neutral-ground-adnd.dat ---> N-test.dat
      Oxygen-clementi-hf-neutral-ground-adnd.dat ---> O-test.dat
      Sodium-clementi-hf-neutral-ground-adnd.dat ---> Na-test.dat
      Xenon-clementi-hf-neutral-ground-adnd.dat ---> Xe-test.dat
      
EOF

exit;
}

sub check_arguments
{
    if($input_file eq "")
    {
	print "\n";
	print "\t Input list file missing...\n";
	print "\t Specify list file, -f [string] (-f list)\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    if($base eq "")
    {
	print "\n";
	print "\t Base file name missing...\n";
	print "\t Specify base file name, -b [string] (-b clementi)\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    if($extension eq "")
    {
	print "\n";
	print "\t File extension missing...\n";
	print "\t Specify file extension, -e [string] (-e dat)\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }


}
