#! /usr/bin/perl

use atom_parser;
use array;

my($input_file) = @ARGV[0];

my($DATA)  = array->ARRAY($input_file);

foreach $file_name (@{$DATA->{array}})
{
    my($charge, 
       $name, 
       $label) = atom_data($file_name);

    my($total_energy, 
       $kinetic_energy, 
       $potential_energy, 
       $virial ) = parse_energies($file_name);
    
    my($file_out) = $name . ".dat";

    print "\n";
    print "reading $file_name and creating $file_out\n";
    print "\n";
    
    open(OUT,">$file_out") or die "cannot create output file\n";
    
    print OUT "\n";
    print OUT "$name Z = $charge\n";
    print OUT "\n";
    print OUT "Orbital Occupations:\n";
    print OUT "\n";
    
    parse_occupancies($file_name);

    print OUT "\n";
    print OUT "\n";
    printf OUT "Total energy     : %17.10f\n", $total_energy;
    printf OUT "Potential energy : %17.10f\n", $potential_energy;
    printf OUT "Kinetic energy   : %17.10f\n", $kinetic_energy;
    printf OUT "Virial           : %17.10f\n", $virial;

    parse_basis($file_name);

    close(OUT);
}

sub parse_occupancies
{

    my($file) = $_[0];

    open(INP,"$file") or die "cannot open input file\n";

    while(<INP>)
    {
	if($_ =~ m/([spdfklm]\(\d+\))/i)
	{
	    $occ = $_;
	    last;
	}
    }

    close(INP);

    my($s_string) = "";
    my($p_string) = "";
    my($d_string) = "";

    if($occ =~ m/K\(2\)/i)
    {
	$s_string = $s_string . "1s(2) ";
    }

    if($occ =~ m/L\(8\)/i)
    {
	$s_string = $s_string . "2s(2) ";
	$p_string = $p_string . "2p(6) ";
    }

    if($occ =~ m/M\(18\)/i)
    {
	$s_string = $s_string . "3s(2) ";
	$p_string = $p_string . "3p(6) ";
	$d_string = $d_string . "3d(10) ";
    }

    while($occ =~ m/(\d+[spdf]\(\d+\))/ig)
    {
	$orb = lc($1);
	
	if($orb =~ m/S/i) { $s_string = $s_string . "$orb "; }
	if($orb =~ m/P/i) { $p_string = $p_string . "$orb "; }
	if($orb =~ m/D/i) { $d_string = $d_string . "$orb "; }
	if($orb =~ m/F/i) { $f_string = $f_string . "$orb "; }
	
    }

    if($s_string ne "") { print OUT "S : $s_string\n"; }
    if($p_string ne "") { print OUT "P : $p_string\n"; }
    if($d_string ne "") { print OUT "D : $d_string\n"; }
    
}

sub parse_basis
{
    my($file) = $_[0];
    
    open(INP,"$file") or die "cannot open input file\n";

    while(<INP>)
    {
	chomp $_;
	
	if($_ =~ m/([spdf])\s+([0-9]+[spdf])/i)
	{
	    print OUT "\n";
	    print OUT "\t\t\t\t        ";

	    while($_ =~ m/([0-9]+[spdf])/ig)
	    {
		$orb = lc($1);
		print OUT "$orb             ";
	    }

	    print OUT "\n";
	}

	if($_ =~ m/BASIS\/ORB/i)
	{
	    print OUT "Orbital energy";
	    print OUT "                 ";

	    while($_ =~ m/(\d+\.\d+|-\d+\.\d+)/ig)
	    {
		printf OUT "%15.8f", $1;
	    }

	    print OUT "\n";
	    print OUT "Basis function\t       Exponent\n";
	}

	if($_ =~ m/([0-9]+[spdf])\s+(\d+\.\d+)/ig)
	{
	    printf OUT "$1\t\t%15.8f",$2;

	    while($_ =~ m/(\d+\.\d+|\-\d+\.\d+)/ig)
	    {
		printf OUT "%15.8f",$1;
	    }
	    
	    print OUT "\n";
	}
    }

    close(INP);
}

sub atom_data
{
    my($file)   = $_[0];

    my($string) = ucfirst($file);

    my($ATOM) = atom_parser->ATOM_DATA($string,"query");
    
    my($label)  = $ATOM->{element_label};
    my($charge) = $ATOM->{element_charge};
    my($name)   = $ATOM->{element_name};

    return $charge, $name, $label;
}

sub parse_energies
{
    my($file) = $_[0];
    
    my($total_energy)     = ();
    my($kinetic_energy)   = ();
    my($potential_energy) = ();
    my($virial)           = ();

    open(INP,"$file") or die "cannot open input file\n";

    while(<INP>)
    {
	chomp $_;
	
	if($_ =~ m/\=/i)                      { $_ =~ s/\=/ \= /ig; }
	if($_ =~ m/E\s+\=\s+(\-\d+\.\d+)/i)   {$total_energy     = $1;}
	if($_ =~ m/T\s+\=\s+(\d+\.\d+)/i)     {$kinetic_energy   = $1;}
	if($_ =~ m/V\s+\=\s+(\-\d+\.\d+)/i)   {$potential_energy = $1;}
	if($_ =~ m/V\/T\s+\=\s+(-\d+\.\d+)/i) {$virial           = $1;}
    }
	
    close(INP);

    return $total_energy, $kinetic_energy, $potential_energy, $virial;
}
