#! /usr/bin/perl
use English;
use locale;
use File::stat;
use File::Find;
use Getopt::Std;
use File::Basename;
use IO::Handle;

my($list)  = @ARGV[0];
my($type)  = @ARGV[1];

my($list_array) = parse_list_file($list);

foreach $list (@$list_array)
{
    $list =~ m/([a-z]+)\-/i;
    
    $element = $1;

    $basis_output = $element . "-" . $type . "-cadpac.bas";
 
    printf "%35s --> %35s\n",$list,$basis_output;

    open(OUTPUT, ">$basis_output") or die "cannot create $basis_output file\n";   

    my($string_array) = parse_sto_basis($list);

    foreach $item (@$string_array)
    {
	if($item =~ m/(1)\s+(\d+\.\d+)\s+(1.0)/i)
	{
	    printf OUTPUT "$1 %15.10f $3\n",$2;
	}
	else
	{
	    print OUTPUT "$item\n";
	}
    }

    close(OUTPUT);
}

sub parse_list_file
{
    my($list) = $_[0];

    open(LIST,"$list") or die "cannot open list file\n";

    while(<LIST>)
    {
	chomp $_;

	if($_ =~ m/([a-z\-\.]+)/i) { push(@file_array, $1); }
    }

    close(LIST);

    return \@file_array;
}

sub parse_sto_basis
{
    my($basis_file)   = $_[0];

    my(@string_array)   = ();
    my($basis_function) = ();
    my($angular)        = ();
    my($exponent)       = ();

    open(BASIS,"$basis_file") or die "cannot open $basis_file basis file\n";

    while(<BASIS>) { if($_ =~ m/BASIS/i) { last; } }
   
    line: while(<BASIS>)
    {
	chomp $_; 

	if( ($_ =~ m/(([0-9]+)([a-z]+))\s+(\d+\.\d+)/i))               
	{       
	    $basis_function = $1;
	    $angular        = $3;
	    $exponent       = $4;

	    if($basis_function =~ m/1s|2p|3d|4f|5g/i)
	    {
		push(@string_array,"1 $angular  1"   );
		push(@string_array,"1 $exponent 1.0" );
	    }
	    else
	    {
		push(@string_array, "1 $basis_function 1");
		push(@string_array, "1 $exponent 1.0"    );
	    }
	}
    }

    close(BASIS);

    return \@string_array;
}


