#! /usr/bin/perl

use array;
use atom_parser;
use basis_parser;
use mo_parser;

my($file) = @ARGV[0];

$file =~ m/([a-z0-9\-]+)/i;

my($file_out) =  $1 . ".mos";

my($occupation, $element) = determine_occupation($file);

my($s_orb, $p_orb, $d_orb, $f_orb) = parse_occupation($occupation);

$MO = mo_parser->STO_MO_DATA($file, "slater");

my($mo_hash)  = $MO->{mo};

$STO  = basis_parser->STO_BASIS_DATA($file,"slater");

$s_bf_nb = $STO->{s_function_nb};
$p_bf_nb = $STO->{p_function_nb};
$d_bf_nb = $STO->{d_function_nb};

$s_orb_nb = @$s_orb;
$p_orb_nb = @$p_orb;
$d_orb_nb = @$d_orb;

create_array($mo_hash, $s_orb);

create_array($mo_hash, $p_orb);

create_array($mo_hash, $d_orb);

open(OUT,">$file_out") or die "cannot create output file\n";

print_mos_qmcmol_format(\@mo_array, $count_orb, $index);

close(OUT);

print "\n";
print "\t$file --> $file_out\n";
print "\n";

sub print_mos_qmcmol_format
{
    my($mo_array) = @_[0];
    my($mo_nb)    = $_[1]; 
    my($coeff_nb) = $_[2];

    my($mod)      = ();

    print OUT "title: mos for $element in sto basis\n";
    print OUT "mos_on_what = aos_c_nrmd\n";
    print OUT "end_of_header\n";
    print OUT "\n";

    for($mo_i = 1; $mo_i<= $mo_nb; $mo_i++)
    {
	$mo = $mo_i - 1;
 
	print OUT "\tmo !mo \# $mo_i\n";

	for($coeff_i = 1; $coeff_i<= $coeff_nb; $coeff_i++)
	{
	    $mod   = $coeff_i%5;

	    $coeff = $coeff_i - 1;
	    
	    if($mod == 1)  { printf OUT  "\t%15.10f", $mo_array->[$mo]{$coeff}; }
	    
	    else { printf OUT  "%15.10f ", $mo_array->[$mo]{$coeff}; }

	    if($mod == 0) {print OUT "\n";}
	}
	
	if( $mod != 0 ) { print OUT "\n"; }

	print OUT "\tend mo ! mo \# $mo_i\n";
	print OUT "\n";
    }

    print OUT "end_of_mos\n";
}

sub create_array
{
    my($j) = 0;
    my($mo_hash) = @_[0];
    my($orb)     = @_[1];
    my($count)   = 0;

    foreach $item (@$orb)
    {
	$item =~ m/(([0-9]+)([a-z]+))\((\d+)\)/i;
	
	$j++;

	$item   = $1;
	$shell  = $3;
	$number = $4;
	
	if($shell eq "s") { $half_occ = 1; }
	if($shell eq "p") { $half_occ = 3; }
	if($shell eq "d") { $half_occ = 5; }
	
	if( $number <= $half_occ ) { $total = $number; }
	if( $number > $half_occ  ) { $total = $half_occ; }
	
	for($i=1;$i<=$total; $i++)
	{
	    $count_orb ++;
	    
	    for $role ( sort keys %{$mo_hash->{$item}} )
	    {
		if($shell eq "p")
		{
		    $index = $s_bf_nb + ($i-1) * $p_bf_nb + $role + $count;
		}

		if($shell eq "s") { $index = $role; }

		if($shell eq "d")
		{
		    $index = 3 * $p_bf_nb * $p_orb_nb + ($s_bf_nb) + ($i-1) * $d_bf_nb + $role + $count;
		}

		$mo_array[$count_orb - 1]{$index - 1} = $mo_hash->{$item}->{$role};

#		print "$item $count_orb $index $role $mo_hash->{$item}->{$role}\n";
	    }
	}

	if($shell eq "p") { $count = $j * $total * $p_bf_nb; }
	
	if($shell eq "d") { $count = $j * $total * $d_bf_nb; }
    }
}

sub determine_occupation
{
    my($file) = $_[0];
    
    $file =~ m/([a-z]+)\-([a-z]+)\-([a-z]+)\-(\+\d+|\-\d+|neutral)/i;

    my($element) = $1;

    my($charge)  = $4;

    $ATOM = atom_parser->ATOM_DATA($element, "query");

    my($occupation) = $ATOM->{element_occupation};

    return $occupation, $element;
}

sub parse_occupation
{
    my($occupation) = $_[0];

    my(@s_orb) = ();
    my(@p_orb) = ();
    my(@d_orb) = ();
    my(@f_orb) = ();

    if($occupation =~ m/K\(2\)/i)
    {
	push(@s_orb,"1s(2)");
    }
    
    if($occupation =~ m/L\(8\)/i)
    {
	push(@s_orb,"2s(2)");
	push(@p_orb,"2p(6)");
    }
    
    if($occupation =~ m/M\(18\)/i)
    {
	push(@s_orb,"3s(2)");
	push(@p_orb,"3p(6)");
	push(@d_orb,"3d(10)");
    }
    
    while($occupation =~ m/(\d+[spdf]\(\d+\))/ig)
    {
	my($orb) = lc($1);
	
	if($orb =~ m/S/i) { push(@s_orb,$orb); }
	if($orb =~ m/P/i) { push(@p_orb,$orb); }
	if($orb =~ m/D/i) { push(@d_orb,$orb); }
	if($orb =~ m/F/i) { push(@f_orb,$orb); }
    }
    
    return \@s_orb, \@p_orb, \@d_orb, \@f_orb;
}
