#! /usr/bin/perl
use XML::Writer;
use IO::File;

my($file_bas) = @ARGV[0];
my($file_mos) = @ARGV[1];
my($type)     = @ARGV[2];

my($element) = determine_element($file_bas);

my($charge_hash) = charge_hash();

my($charge) = determine_charge($charge_hash, $element);

my($symmetry, $level, $exponent) = parse_qmcmol_basis($file_bas);

my($mo_array, $mo_nb, $vector_nb) = parse_qmcmol_mos($file_mos);

my($output, $writer) = xml_output_file();

print_basis_xml($symmetry, $level, $exponent);

print_mo_xml($mo_array, $mo_nb, $vector_nb, $writer);

sub print_mo_xml
{
    my($mo_array)  = @_[0];
    my($mo_nb)     = $_[1];
    my($vector_nb) = $_[2];
    my($writer)    = $_[3];

    $writer->startTag('MOVectors');

    for($i=0; $i<$mo_nb; $i++)
    {
	$writer->startTag('Vector');
       
	$writer->characters("\n");

	for($j=0;$j<$vector_nb; $j++)
	{
#	    $writer->characters($mo_array->[$i]{$j});
#	    $writer->characters("\n");
	    
	    printf $output "\t%15.10f\n", $mo_array->[$i]{$j};
	}

	$writer->endTag('Vector');
    }

    $writer->endTag('MOVectors');

    $writer->endTag('Orbitals');
}

$writer->end();
$output->close();

sub parse_qmcmol_mos
{
    my($file)          = $_[0];
    my($count_mo)      = -1;
    my($count_vectors) = 0;
    my(@mo_array)      = ();
    my($vector_nb)     = 0;

    open(MOS, "$file") or die "cannot open qmcmol mo file\n";
    while(<MOS>)
    {
	chomp $_;

	if($_ eq "")
	{
	    $count_mo++;
	    $count_vectors = 0;
	}

	elsif($_ =~ m/\d+\.\d+/i)
	{
	    while($_ =~ m/(\d+\.\d+|\-\d+\.\d+)/ig)
	    {
		$mo_array[$count_mo]{$count_vectors} = $1;
		$vector_nb = $count_vectors;
		$count_vectors++;
	    }
	}
	
    }
    close(MOS);

    $count_mo  = $count_mo - 1;
    $vector_nb = $vector_nb + 1;

    return \@mo_array, $count_mo, $vector_nb;
}

sub xml_output_file
{

    my($output) = new IO::File(">output.xml");

    my($writer) = new XML::Writer(NAMESPACES  => 1,
				  DATA_MODE   => 'true',
				  DATA_INDENT => 1,
				  OUTPUT      => $output
				  );
    $writer->xmlDecl();

    return $output, $writer;
}

sub determine_element
{
    $_[0] =~ m/([a-z]+)\-/i;
    return $1;
}


sub print_basis_xml
{
    my($symmetry) = @_[0];
    my($level)    = @_[1];
    my($exponent) = @_[2];


    $writer->startTag('Orbitals');
    $writer->startTag('Basis');
    $writer->startTag('Center', 
		      'Name'   => $element,
		      'Charge' => $charge, 
		      'X'      => "0.0",
		      'Y'      => "0.0",
		      'Z'      => "0.0",
		      );

    for($i=0; $i<@$symmetry;$i++)
    {
	
	$writer->startTag('Contraction', 'Slaterize' => 'False');
	$writer->startTag('Slater', 
			  'Level'       => $level->[$i],
			  'Type'        => $symmetry->[$i],
			  'Exponent'    => $exponent->[$i],
			  'Coefficient' => "1.0"
			  );
	$writer->endTag('Slater');
	$writer->endTag('Contraction');
    }

    $writer->endTag('Center');
    $writer->endTag('Basis');
}

sub determine_charge
{
    my($chage_hash) = @_[0];
    my($element)    = $_[1];
    my($z)          = ();

    foreach $key (keys %{$charge_hash})
    {
	if($key eq $element)
	{
	    $z = $charge_hash->{$key};
	    last;
	}
    }

    return $z;
}

sub parse_qmcmol_basis
{
    my($file) = $_[0];
    
    open(BAS,"$file") or die "cannot open basis file\n";
    while(<BAS>)
    {
	chomp $_;
	
	if($_ =~ m/sym\s+(S|P|D|F|G)/)
	{
	    push(@symmetry, $1);
	}
	
	if($_ =~ m/sto\s+(\d+)\s+(\d+\.\d+)/i)
	{
	    push(@level, $1);
	    push(@exponent, $2);
	}
    }
    close(BAS);

    return \(@symmetry, @level, @exponent);
}

sub charge_hash
{
    $charge{"Helium"} = "2.0";
    $charge{"Lithium"} = "3.0";
    $charge{"Beryllium"} = "4.0";
    $charge{"Boron"} = "5.0";
    $charge{"Carbon"} = "6.0";
    $charge{"Nitrogen"} = "7.0";
    $charge{"Oxygen"} = "8.0";
    $charge{"Fluorine"} = "9.0";
    $charge{"Neon"} = "10.0";
    $charge{"Sodium"} = "11.0";
    $charge{"Magnesium"} = "12.0";
    $charge{"Aluminum"} = "13.0";
    $charge{"Silicon"} = "14.0";
    $charge{"Phosphorus"} = "15.0";
    $charge{"Sulfur"} = "16.0";
    $charge{"Chlorine"} = "17.0";
    $charge{"Argon"} = "18.0";
    $charge{"Potassium"} = "19.0";
    $charge{"Calcium"} = "20.0";
    $charge{"Scandium"} = "21.0";
    $charge{"Titanium"} = "22.0";
    $charge{"Vanadium"} = "23.0";
    $charge{"Chromium"} = "24.0";
    $charge{"Manganese"} = "25.0";
    $charge{"Iron"} = "26.0";
    $charge{"Cobalt"} = "27.0";
    $charge{"Nickel"} = "28.0";
    $charge{"Copper"} = "29.0";
    $charge{"Zinc"} = "30.0";
    $charge{"Gallium"} = "31.0";
    $charge{"Germanium"} = "32.0";
    $charge{"Arsenic"} = "33.0";
    $charge{"Selenium"} = "34.0";
    $charge{"Bromine"} = "35.0";
    $charge{"Krypton"} = "36.0";
    $charge{"Rubidium"} = "37.0";
    $charge{"Strontium"} = "38.0";
    $charge{"Yttrium"} = "39.0";
    $charge{"Zirconium"} = "40.0";
    $charge{"Niobium"} = "41.0";
    $charge{"Molybdenum"} = "42.0";
    $charge{"Technetium"} = "43.0";
    $charge{"Ruthenium"} = "44.0";
    $charge{"Rhodium"} = "45.0";
    $charge{"Palladium"} = "46.0";
    $charge{"Silver"} = "47.0";
    $charge{"Cadmium"} = "48.0";
    $charge{"Indium"} = "49.0";
    $charge{"Tin"} = "50.0";
    $charge{"Antimony"} = "51.0";
    $charge{"Tellurium"} = "52.0";
    $charge{"Iodine"} = "53.0";
    $charge{"Xenon"} = "54.0";

    return \%charge;
}
