#! /usr/bin/perl

use array;
use parse_geom;
use atom_parser;
use tools;

my($input_file)     = @ARGV[0];

my($GEOM)           = parse_geom->GEOM_DATA($input_file, "xyz");

my($elements)       = $GEOM->{atom_label};

my($atom_xyz_coord) = $GEOM->{atom_xyz_coord};
 
my($atom_charges)   = get_atom_charges($elements);

my($atoms_nb)       = tools->size_dim1_array($atom_charges);

my($dimensions_nb)  = tools->size_dim2_array($atom_xyz_coord, 2);

for($atom_i = 0; $atom_i < $atoms_nb; $atom_i++)
{
    my($atom) = $elements->[$atom_i];

    my($charge) = $atom_charges->[$atom_i];

    printf "%2s  %4.1f", $atom, $charge;

    for($dim_i = 0; $dim_i < $dimensions_nb; $dim_i++)
    {
	printf "%15.10f", $atom_xyz_coord->[$atom_i]{$dim_i};
    }
    
    print "\n";
}

#########################################################################

sub get_atom_charges
{
    my($atoms) = @_[0];

    my(@atom_charges) = ();

    foreach my $element (@$atoms)
    {
	my($ATOM) = atom_parser->ATOM_DATA($element,"query");

	my($charge) = $ATOM->{element_charge};

	push(@atom_charges, $charge);
    }

    return \@atom_charges;
}
