#! /usr/bin/perl

use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::Handle;
use Term::ANSIColor;

getopt("blj:",\%args);

$list    = $args{l}; # list file
$base    = $args{b}; # base file name
$job     = $args{j}; # job types: dmc/opt/vmcorr

open(LIST,"$list") or die "cannot open list file\n";

foreach $item (@ARGV) { push(@atom_label, $item); }

while(<LIST>)
{
	chomp $_;
	if($_ =~ m/geom/i)
	{
	    while(<LIST>)
	    {
		if($_ =~ m/end_geom/i)      { last;            }
		elsif($_ =~ m/(\d+\.\d+)/i) { push(@array,$1); }
	    }
	}
}

close(LIST);

foreach $item (@array)
{
    $file_corr = $base . "-" . $item . "-" . $job . ".corr";
    $file_out  = $base . "-" . $item . ".jas-in";
 
    open(CORR,"$file_corr") or die "cannot open $file_corr corr file\n";
    open(OUT,">$file_out") or die "cannot create output file\n";

    print "$file_corr -> $file_out\n";

    while(<CORR>)
    {
	chomp $_;

	if($_ =~ m/number of nuclei/i)
	{
	    $_ = <CORR>;
	    chomp $_;
	    $_ =~ m/(\d+)/i;
	    $atom_nb = $1;
	    last;
	}
    }

    print OUT "number of nuclei         =     $atom_nb\n";
    print OUT "number of configurations =     1\n";
    print OUT "end_of_header\n";
    print OUT "CONFIGURATION = CONF_1\n";
    print OUT "\n";
    
    while(<CORR>)
    {
	chomp $_;
	
	if($_ =~ m/for nucleus\s+(\d+)/i)
	{
	    $atom_i = $1 - 1;
	}

	if($_ =~ m/electron-electron inverse range/i)
	{
	    $_ = <CORR>;

	    chomp $_;

	    $_ =~ m/(\d+\.\d+)/;

	    printf OUT "BETA_EE      = %24.10e %24.10e\n",$1,$1;
	}

	if($_ =~ m/electron-electron coefficients/i)
	{
	    $_ = <CORR>;

	    chomp $_;

	    while($_ =~ m/(-\d+\.\d+|\d+\.\d+)/g) { push(@ee_coeff,$1); }
	    
	    print OUT "A_EE_LIKE    =";

	    for($i=0; $i<@ee_coeff; $i++)
	    {
		if($i==0) { $tmp = @ee_coeff[$i]/2; }
		else      { $tmp = @ee_coeff[$i];   }
		printf OUT "%25.10e",$tmp;
	    }

	    print OUT "\n";
	    print OUT "A_EE_UNLIKE  =";

	    for($i=0; $i<@ee_coeff; $i++)
	    {
		printf OUT "%25.10e",@ee_coeff[$i];
	    }

	    print OUT "\n";

	    @ee_coeff=();
	}

	if($_ =~ m/inverse range \w+\s+nucleus\s+(\d+)/)
	{
	    $atom = $1;

	    $_    = <CORR>;

	    chomp $_;

	    $_    =~ m/(\d+\.\d+)/;

	    $ben  = $1;
	}

	if($_ =~ m/electron-electron-nucleus coefficients/i)
	{
	    $_ = <CORR>;

	    chomp $_;

	    while($_ =~ m/(-\d+\.\d+|\d+\.\d+)/g) { push(@een,$1); }

	    print OUT "A_EEN        =";

	    for($i=0; $i<@een; $i++) { printf OUT "%25.10e",@een[$i]; }

	    print OUT "\n";

	    @een = ();
	}

	elsif($_ =~ m/electron-nucleus coefficients/i)
	{
	    $_ = <CORR>;

	    chomp $_;

	    while($_ =~ m/(-\d+\.\d+|\d+\.\d+)/g) { push(@en,$1); }

	    print  OUT "ATOM_NAME    = @atom_label[$atom_i]\n";
	    printf OUT "BETA_EN      = %24.10e\n",$ben;
	    printf OUT "A_EN         =";

	    for($i=0; $i<@en; $i++) 
	    {
		@en[$i] = -1.0*$en[$i];
		printf OUT "%25.10e",@en[$i]; 
	    }
	    
	    print OUT "\n";

	    @en = ();
	}
	
    }

    print OUT "\n";
    print OUT "END\n";
    print OUT "\n";
    print OUT "eof\n";

    close(CORR);
    close(OUT);

}
