#! /usr/bin/perl
use Env qw(HOME PWD SYS LD_LIBRARY_PATH USER);
use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::Handle;
use Term::ANSIColor;

getopt("blhio:",\%args);

$base            = $args{b}; # base file name
$list            = $args{l}; # list file
$help            = $args{h}; # help
$input_suffix    = $args{i}; # input extension
$output_suffix   = $args{o}; # output extension

if($input_suffix eq "") 
{
    print "\n";
    print "\tneed to include input suffix, -i [string], exiting...\n";
    print "\n";
    exit;
}

if($output_suffix eq "") 
{
    print "\n";
    print "\tneed to include output suffix, -o [string], exiting...\n";
    print "\n";
    exit;
}

if($list eq "")
{
    print "\n";
    print "\tneed to include list file, -l [string], exiting...\n";
    print "\n";
    exit;
}

if($base eq "")
{

    print "\n";
    print "\tneed to include base file name, -b [string], exiting...\n";
    print "\n";
    exit;
}

if( $help eq "help"        ) { help(); exit; }

open(IN,"$list") or die "cannot open input file\n";
line: while(<IN>)
{
    chomp $_;
    if($_ =~ m/geom/i)
    {
	inner: while(<IN>)
	{
	    chomp $_;
	    if($_ =~m/\#/i)           { next inner;      }
	    elsif($_ =~m/end_geom/i)  { last;            }
	    elsif($_ =~ m/(\d+.\d+)/) { push(@array,$1); }
	}
    }
}
close(IN);

print "\n";
print "\t";

foreach $item (@array)
{
    $count++;

    $in   = $base . "-" . $item . "." . $input_suffix;
    $out  = $base . "-" . $item . "." . $output_suffix;

    system("cp $in $out");

    if( $count%2 ==0) { print "$out -> $in\n"; print "\t";}
    else              { print "$out -> $in "; }
}
print "\n";

sub help
{
    warn "script: @_\n" if @_;
    die <<EOF;

This script copies from one extension to another: 
(file.par-in to file.par-initial)

Options are:
            -h help
            -b [string] base file name.
            -i [string] input extension wlk-in, par-out, jas-in, etc.
	    -o [string] output extension par-initial
            -l [string] list file containing all geometries.
	                of the form:
               geom
                       1.40
                       1.30
                       1.50
                       ...
                       3.015
               end_geom
                        
                        The "#" symbol is used to comment out a particular
                        line.

Example:
	copy_files.pl -l list -b lih-sbk -i par-in -o par-initial

EOF
}
