#! /usr/bin/perl

#standard perl module
use Getopt::Std;

# local modules
use array;
use cascade;
use tools;

getopt("iefhqt",\%args);

my($input_file) = $args{i};
my($type)       = $args{t};
my($element)    = $args{e};
my($format)     = $args{f};
my($help)       = $args{h};
my($query)      = $args{q};

if($help ne "") { help(); }

my($program) = determine_program();

my($INPUT_DATA)  = cascade->CASCADE($input_file);

$array = $INPUT_DATA->{array};

if( $query ne "") { query_ecp_data(); }

my($string) = extract_atom_ecp($array, $type, $element, $program);

my($ecp_hash) = extract_parameters($string);

my($potential_array) = create_potential_array();

if($format =~ m/gamess/i)   { print_gamess_format($ecp_hash);   }

if($format =~ m/gaussian/i) { print_gaussian_format($ecp_hash); }

#-------------------------------------------------------------------#

sub create_potential_array
{

    my(@potential_array) = ("u(f)", "u(s-f)", "u(p-f)", "u(d-f)",
			    "u(d)", "u(s-d)", "u(p-d)",
			    "u(p)", "u(s-p)",
			    "u(g)", "u(s-g)", "u(p-g)", "u(d-g)", "u(f-g)" );

    return \@potential_array;
}

sub print_gaussian_format
{
    my($ecp_hash) = @_[0];

    my($tmp1)     = ();
    my($tmp2)     = ();

    my($core) = $hash{$element}{core};
    my($lmax) = $hash{$element}{lmax};

    print "$element 0\n";
    print "OLP $lmax $core\n";

    foreach my $pot_type (@$potential_array)
    {
	for my $family (sort keys %$ecp_hash)
	{
	    $family !~ m/(u\(([a-z\-]+)\))/i;

	    $tmp1    = $1;

	    if($tmp1 eq $pot_type)
	    {
		my($term_nb) = $ecp_hash->{$tmp1}->{term_nb};
	    
		if($tmp1 ne $tmp2) 
		{
		    if($tmp1 =~ m/u\(([a-z]+)\)/i)
		    {
			print "$1 component\n";
		    }

		    if($tmp1 =~ m/u\(([a-z]\-[a-z])\)/i)
		    {
			print "$1 projection\n";
		    }

		    print "$term_nb\n";   
		}
	    
		my($value) = $ecp_hash->{$family}->{exponent};

		if($value ne "")
		{
		    my($exponent)    = $ecp_hash->{$family}->{exponent};
		    my($coefficient) = $ecp_hash->{$family}->{coefficient};
		    my($power)       = $ecp_hash->{$family}->{r_power};
		
		    printf "%1d %15.10f %20.10f\n", $power, $exponent, $coefficient;
		}
	
		$family =~ m/(u\(([a-z\-]+)\))/i;

		$tmp2   = $1;
	    }
	}
    }
}

sub print_gamess_format
{
    my($ecp_hash) = @_[0];

    my($tmp1)     = ();
    my($tmp2)     = ();

    my($core) = $hash{$element}{core};
    my($lmax) = $hash{$element}{lmax};

    print "$element-ECP GEN $core $lmax\n";

    foreach my $pot_type (@$potential_array)
    {
	for my $family (sort keys %$ecp_hash)
	{
	    $family !~ m/(u\(([a-z\-]+)\))/i;

	    $tmp1    = $1;

	    if($tmp1 eq $pot_type)
	    {
		my($term_nb) = $ecp_hash->{$tmp1}->{term_nb};
	    
		if($tmp1 ne $tmp2) 
		{
		    print " $term_nb      -----  $element $tmp1 -----\n";
		}
	    
		my($value) = $ecp_hash->{$family}->{exponent};

		if($value ne "")
		{
		    my($exponent)    = $ecp_hash->{$family}->{exponent};
		    my($coefficient) = $ecp_hash->{$family}->{coefficient};
		    my($power)       = $ecp_hash->{$family}->{r_power};
		
		    printf "%25.10f %5d %15.10f\n", $coefficient, $power, $exponent;
		}
	
		$family =~ m/(u\(([a-z\-]+)\))/i;

		$tmp2   = $1;
	    }
	}
    }
}

sub extract_atom_ecp
{
    my($array)   = @_[0];
    my($type)    = $_[1];
    my($element) = $_[2];
    my($program) = $_[3];

    foreach my $item (@$array)
    {
	if( 
	    ($item =~ m/element\s+\=\s+$element\s+/i) && 
	    ($item =~ m/type\s+\=\s+$type/i)          &&
	    ($item =~ m/program\s+\=\s+$program/i)     
	    )

	{
	    return $item;
	}
    }
}

sub extract_parameters
{
    my($string) = $_[0];
    
    my($hash)   = ();

    while($string =~ m/([a-z0-9\.\(\)\_\-]+)\s+\=\s+([a-z0-9\.\(\)\_\-]+)/ig)
    {
	my($token) = $1;
	my($value) = $2;
	
	if($token =~ m/element/i)         { $element     = $value; }
	if($token =~ m/lmax/i)            { $lmax        = $value; }
	if($token =~ m/core\_electrons/i) { $core        = $value; }
	if($token =~ m/exponent/i)        { $exponent    = $value; }
	if($token =~ m/coefficient/i)     { $coefficient = $value; }
	if($token =~ m/r\_power/i)        { $power       = $value; }

	if( ($power ne "") && ($coefficient ne "") && ($exponent ne "") )
	{
	    $count++;

	    $hash{$potential}{"term_nb"} = $count;
	    
	    my($key) = "$potential\_$count";
	    
	    $hash{$element}{"core"}    = $core;
	    $hash{$element}{"name"}    = $element;
	    $hash{$element}{"lmax"}    = $lmax;
	    $hash{$key}{"exponent"}    = $exponent;
	    $hash{$key}{"coefficient"} = $coefficient;
	    $hash{$key}{"r_power"}     = $power;

	    $exponent    = ();
	    $coefficient = ();
	    $power       = ();
	}

	if( $value =~ m/(u\(([a-z\-]+)\))/i)
	{
	    $projection = uc($2);
	    $count      = 0;
	    $potential  = lc($1);
	}
    }

    return \%hash;
}

sub determine_program
{
    my($program) = ();

    if( ($type =~ m/needs/i) || ($type =~ m/tr/i) )
    {
	if($format =~ m/gamess/i)   { $program = "gamess"; }
	if($format =~ m/gaussian/i) { $program = "gaussian"; }
    }

    else { $program = "any"; }

    return $program;
}

sub query_ecp_data
{
    my($type)    = ();
    my($program) = ();

    print "\n";
    print "\tElement: $query\n";
    print "\n";
    print "     ECP TYPE         Program\n";
    print "  ------------------------------\n";
    
    foreach my $item (@$array)
    {
	if($item =~ m/(element\s+\=\s+$query\s+)/i )
	{
	    if( ($item !~ m/type\s+\=\s+[\(\)]+/i) && ($item =~ m/type\s+\=\s+([a-z0-9]+)/i ) )
	    {
		$type = $1;
	    }
	    
	    if( ($item =~ m/program\s+\=\s+([a-z0-9]+)/i) )
	    {
		$program = $1;
	    } 
	    
	    if( ( $type ne "" ) && ( $program ne "" ) )
	    {
		printf "\t$type\t\t% -20.10s\n", $program;
		
		$program = ();
	    }
	}
    }
    
    print "\n";

    if($type eq "") 
    {
	print "\tElement, $query, not found in file\n";
	print "\texiting...\n";
	print "\n";
    }
    
    exit;
}

sub help
{
    print <<EOF;
	
  Options:

          -e [string] - element, use element abbreviation
          -f [string] - ab initio format, currently, gamess or gaussian.
          -h [string] - help
          -i [string] - input file, contains ecp data in cascade format.
          -q [string] - query ecp data,
          -t [string] - ecp type, SBK, OLA, etc, use query option to determine
                        types of ecps in ecp file.

  Note:

          There are no defaults, and command line input is case insensitive.

  Example:

  To obtain gamess format output

           ./ecp-data-to-ab-initio.pl -i ecp.lib -e C -t SBK -f gamess

  Will procude the following output:

C-ECP GEN 2 1
 1      -----  C u(p) -----
            -0.8937100000     1    8.5646000000
 2      -----  C u(s-p) -----
             1.9292600000     0    2.8149000000
            14.8819900000     2    8.1129000000

  To obtain gaussian format output

           ./ecp-data-to-ab-initio.pl -i ecp.lib -e C -t SBK -f gaussian

  Will procude the following output:

  C 0
  OLP 1 2
  p component
  1
  1    8.5646000000        -0.8937100000
  s-p projection
  2
  0    2.8149000000         1.9292600000
  2    8.1129000000        14.8819900000

  To query ecp data

	   ecp-data-to-ab-initio.pl -q H -i ecp.lib

  Will produce the following:
	   	   
        Element: C

     ECP TYPE         Program
  ------------------------------
        SBK             any
        GLP             any
        LMP             any
        NEEDS           gaussian
        OLA             any
        TR              gaussian
        NEEDS           gamess
        TR              gamess

	   
EOF

    exit;

}
