#! /usr/bin/perl

use array;
use atom_parser;
use tools;

my($file) = @ARGV[0];

my($FILE)  = array->ARRAY($file);

my($array) = $FILE->{array};

my($element_array) = parse_gaussian_charge($array);

my($geometry_array) = parse_gaussian_geometry($array);

my($element_label) = parse_element_abbreviations($element_array);

my($element_charge) = parse_element_charge($element_array);

print_geometry($element_charge, $element_label, $geometry_array);

sub print_geometry
{
    my($element_charge) = @_[0];
    my($element_label)  = @_[1];
    my($geometry_array) = @_[2];

    my($atom_nb) = tools->size_dim1_array($element_label);

    my($dim_nb)  = tools->size_dim2_array($geometry_array,2);

    for($atom_i=0; $atom_i < $atom_nb; $atom_i++)
    {
	print " $element_label->[$atom_i]  $element_charge->[$atom_i] ";

	for($dim_i=0; $dim_i < $dim_nb; $dim_i++)
	{
	    printf "%15.10f", $geometry_array->[$atom_i]{$dim_i};
	}

	print "\n";
    }

}

sub parse_element_charge
{
    my($element_array) = @_[0];

    my(@element_charge) = ();

    foreach my $atom_name (@$element_array)
    {
	my($ATOM) = atom_parser->ATOM_DATA($atom_name,"query");

	my($charge) = $ATOM->{element_charge};

	push(@element_charge, $charge);
    }

    return \@element_charge;
}

sub parse_element_abbreviations
{
    my($element_array) = @_[0];

    my(@element_abbrev) = ();

    foreach my $atom_name (@$element_array)
    {
	my($ATOM) = atom_parser->ATOM_DATA($atom_name,"query");

	my($atom_label) = $ATOM->{element_label};

	push(@element_abbrev, $atom_label);
    }

    return \@element_abbrev;
}

sub parse_gaussian_geometry
{
    my($array) = @_[0];

    my($geom_array) = ();

    my($atom_i) = 0;

    foreach my $item (@$array)
    {
	my($dim_i)  = 0;

	while($item =~ m/(\d+\.\d+|-\d+\.\d+)/ig)
	{
	    $geom_array[$atom_i]{$dim_i} = $1;

	    $dim_i++;
	}

	if( $item =~ m/\d+\.\d+|-\d+\.\d+/i){$atom_i++;}
    }

    return \@geom_array;
}


sub parse_gaussian_charge
{
  

    my($charge_hash) = atom_parser->charge_table();

    my($element) = ();
    my(@element_array) = ();

    foreach my $item (@$array)
    {
	if( $item =~ m/(\d+)\s+(\d+\.\d+|-\d+\.\d+)/i)
	{
	    $charge = "$1\.0";
	    
	    for $element_names (sort keys %$charge_hash)
	    {
		if($element_names eq $charge)
		{
		    $element = $charge_hash->{$element_names};

		    push(@element_array, $element);
		}
	    }
	}
    }

    return \@element_array;
}
