#! /usr/bin/perl

use Env qw(HOME PWD SYS LD_LIBRARY_PATH USER);
use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::Handle;
use Term::ANSIColor;

getopt("blhs:",\%args);

$base      = $args{b}; # base file name
$list      = $args{l}; # list file
$help      = $args{h}; # help
$suffix    = $args{s}; # suffix

if( $help eq "help"        ) { help(); exit; }

if($list eq "")
{
    print "\n";
    print "\tneed to include list file, -l [string], exiting...\n";
    print "\n";
    exit;
}
if($base eq "")
{
    print "\n";
    print "\tneed to include base file name, -b [string], exiting...\n";
    print "\n";
    exit;
}

if($suffix eq "")
{
    print "\n";
    print "\tneed to include suffix, -s [string], exiting...\n";
    print "\n";
    exit;
}

open(IN,"$list") or die "cannot open input file\n";

line: while(<IN>)
{
    chomp $_;

    if($_ =~ m/geom/i)
    {
	outer: while(<IN>)
	{
	    chomp $_;

	    if($_ =~m/\#/i)           { next outer;      }
	    if($_ =~ m/end_geom/i)    { last;            }
	    elsif($_ =~ m/(\d+.\d+)/) { push(@array,$1); }
	}
    }
}

$beta_ee         = 0;
$beta_en         = 0;
$beta_en_atom    = 0;
$alpha_ee_like   = 0;
$alpha_ee_unlike = 0;
$alpha_en_atom   = 0;
$alpha_een_atom  = 0;

print "\n";
print "\t";

foreach $item (@array)
{
    $count++;

    $jas_file        = $base . "-" . $item . ".jas-" . $suffix;

    open(JAS, "$jas_file") or die "cannot open $jas_file\n";

    $par_file        = $base . "-" . $item . ".par-in";

    open(PAR, ">$par_file") or die "cannot create $par_file file\n";

    print PAR "title ?\n";
    print PAR "\n";
    print PAR "end_of_header\n";

    if( $count%2 ==0) { print "$jas_file -> $par_file\n"; print "\t";}
    else              { print "$jas_file -> $par_file "; }


    while(<JAS>)
    {
	chomp $_;

	if($_ =~ m/beta_en/i)
	{
	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		$beta_en++;
		print PAR "conf_jastr_beta_en(1,$beta_en) = $1\n";
	    }
	}

	if( ($_ =~ m/a_en/i) && ($_ !~ m/beta/i) )
	{
	    $alpha_en_atom++;
	    $alpha_en=0;
	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		$alpha_en++;
		print PAR "conf_jastr_a_en(1,$alpha_en,$alpha_en_atom) = $1\n";
	    }
	}

	if( ($_ =~ m/a_een/i) && ($_ !~ m/beta/i) )
	{
	    $alpha_een_atom++;
	    $alpha_een=0;
	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		$alpha_een++;
		print PAR "conf_jastr_a_een(1,$alpha_een,$alpha_een_atom) = $1\n";
	    }
	}

	if($_ =~ m/b_en/i)
	{
	    $beta_en_atom++;
	    $beta_en = 0;
	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		print PAR "conf_jastr_beta_en(1,$beta_en_atom) = $1\n";
	    }
	}

	if($_ =~m/beta_ee/i)
	{

	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		$beta_ee++;
		print PAR "conf_jastr_beta_ee(1,$beta_ee) = $1\n";
	    }
	}

	if($_ =~ m/A_EE_LIKE/i)
	{
	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		$alpha_ee_like++;
		print PAR "conf_jastr_a_ee(1,1,$alpha_ee_like) = $1\n";
	    }
	}

	if($_ =~ m/A_EE_UNLIKE/i)
	{
	    while($_ =~m/((\d+.\d+|-\d+.\d+)e(\+|\-)\d+)/ig)
	    {
		$alpha_ee_unlike++;
		print PAR "conf_jastr_a_ee(1,2,$alpha_ee_unlike) = $1\n";
	    }
	}
	
    }
    
    $beta_en         = 0;
    $beta_en_atom    = 0;
    $beta_ee         = 0;
    $alpha_ee_like   = 0;
    $alpha_ee_unlike = 0;
    $alpha_en_atom   = 0;
    $alpha_een_atom  = 0;
    
    close(PAR);
    close(JAS);
}

print "\n";

sub help
{
    warn "script: @_\n" if @_;
    die <<EOF;

This script copies .out to .in files (jastrow, walker, and parameter files)

Options are:
            -h help
            -b [string] base file name.
            -s [string] suffix e.g. out or in.
            -l [string] list file containing all geometries.
	                of the form:
               geom
                       1.40
                       1.30
                       1.50
                       ...
                       3.015
               end_geom

                        The symbol "#" is used to comment out a 
			particular line.

Example: 
         jas_to_par 

EOF
}
