#! /usr/bin/env python

# Magic2Zori.py
# Python script for converting qmagic walkers into walkers for Zori 
# (C) Alan Aspuru-Guzik 2002
# All code under the GPL License
import sys
from string import *
from os import system
import re, getopt, sys

input = "outxx"
output = "walkers.xml"

options,extra = getopt.getopt(sys.argv[1:],'h:i:o:p:r:f:v')

flags = {}
for o in options:
	flags[o[0]] = o[1:]


if flags.has_key('-v'):
	print "Verbose mode"
	verbose=1
else:
	verbose=0

if flags.has_key('-h') or len(options) == 0:
	print """
magic2zori.py -- Python Script for converting QMagiC walkers into 
              Zori walkers

              Usage:
              magic2zori -p <particles> -i <input file>
                                          -o <output file>
              
options:
 -h: this text
[ Required options ]
 -p: number of particles

[ Configuration ]
 -i: input file [Default: outxx]
 -o: output file [Default: walkers.xml]
 

"""
	system.exit(0)

if flags.has_key('-p'):
	input_list = flags['-p']
	particles = atoi(input_list[0])
        print "Particles: " + str(particles)
else:
	print "You need to specify the number of electrons (particles)"
	print "with -p <particles> next time, buddy!"
        system.exit(0)

if flags.has_key('-i'):
	input_list = flags['-i']
	input = input_list[0]



if flags.has_key('-o'):
	output_list = flags['-o']
	output = output_list[0]

print "magic2zori: Walker convesrion tool"
print "Beginning conversion!"

print "Input file: " + str(input)
print "Output file: " + str(output) 

# regular expressions

#re_title = re.compile("TITLE")
#re_etrial = re.compile("ETRIAL")
#re_rand = re.compile(" RAND")
#re_xx = re.compile(" XX")
#re_end = re.compile(" END")

input_file = open (input, 'r')
#stripping the list, i don't know why it doesn't work with
#012's all around
input_list = input_file.readlines()
input_list = map(strip,input_list)
lines = len(input_list)

output_file = open(output, 'w')

xml_header = """<?xml version="1.0"?>
"""

output_file.write(xml_header)
output_file.write('<Walkers>\n');
output_file.write('    <Ensemble Type="Single" Dim="3" Particles="'
                  + str(particles) + '">\n')

restart = []
restart.append("       <Restart>\n")

#print input_list
#line = filter(re_title.match,input_list)
print 'Title found at line ' + str(input_list.index('TITLE'))
title = input_list[input_list.index('TITLE')+1]
restart.append('     <Simulation Name="' + title + '" />\n')

print 'Etrial found at line ' + str(input_list.index('ETRIAL'))
etrial = input_list[input_list.index('ETRIAL')+1]
restart.append('     <ERef Value="'+ str(atof(etrial)) +'" />\n')

print 'Rand found at line' + str(input_list.index('RAND'))
rand = input_list[input_list.index('RAND')+1]
#restart.append('     <RNG Seed="'+ str(atoi(rand)) +'" />\n')
restart.append('     <RNG Seed="'+ str(rand) +'" />\n')
restart.append("    </Restart>")

line_found = input_list.index('XX')
print 'XX found at line ' + str(line_found)

i=1
particle=0
list_of_walkers = []
while input_list[line_found+i] != 'END':
    line = input_list[line_found+i]
    line_split=split(line)
    #print line_split[0]
    #print line_split[1]
    #print line_split[2]
    # create walker!!!
    walker = []
    if (particle==0):
        walker.append("      <Walker>\n")
    walker.append("       " + str(line_split[0]) + " " +
                  str(line_split[1])+ " " +
                  str(line_split[2])+ "\n")
    particle = particle +1
    if (particle == particles):
        walker.append("      </Walker>\n")
        particle = 0
    list_of_walkers = list_of_walkers + walker
    i=i+1

list_of_walkers.append("    </Ensemble>\n")
list_of_walkers.append("")
                       
#print list_of_walkers
#print restart
print "Number of random walkers converted = " + str((i-1)/particles)
#finalizing

for line in list_of_walkers:
    output_file.write(line)
for line in restart:
    output_file.write(line)
output_file.write("\n  </Walkers>\n")

output_file.close()
input_file.close()









