#! /usr/bin/perl

use Getopt::Std;

getopt("foh:",\%args);

my($file)  = $args{f};
my($value) = $args{o};
my($help)  = $args{h};

if( $help =~ m/help|h/i )
{
    help();
}

check_arguments();

my($hash, $final_block) = parse_file();

my($file_out, $file_plt) = create_output_files();

print_to_screen();

print_last_values();

print_all_values();

print_selected_block();

create_gnuplot_file();

###############################################################################

sub print_to_screen
{
    print "\n";
    print "\t input file   : $file\n";

    if ( $value eq "all" )
    {
	print "\t data file    : $file_out\n";
	print "\t gnuplot file : $file_plt\n";
    }
    
    print "\n";

}

sub create_output_files
{
    @tmp = split(/\.out/,$file);

    my($base) = @tmp[0];

    my($file_out) = $base . ".data";

    my($file_plt) = $base . ".plt";

    return $file_out, $file_plt;
}

sub create_gnuplot_file
{
    my(@value) = ();
    my(@error) = ();

    if($value eq "all")
    {
	open(PLT,">$file_plt") or die "cannot create gnuplot file\n";

	my($count) = 1;
	my($final) = ();

	if( $final_block =~ m/[0]+(\d+)/i ) { $final = $1;           }
	else                                { $final = $final_block; }

	my($lower_bound) = 1;
	my($upper_bound) = $final + 5;

	for my $family ( sort keys %{$hash->{$final_block} } )
	{
	    $count++;

	    print PLT "# quantity: $family\n";
	    print PLT "\n";
	    print PLT "set title \"$file : $family\"\n";
	    print PLT "set xlabel \"blocks\"\n";
	    print PLT "set ylabel \"$family (a.u.)\"\n";
	    print PLT "\n";
	    print PLT "show title\n";
	    print PLT "show ylabel\n";
	    print PLT "show xlabel\n";
	    print PLT "\n";
	    print PLT "a = $lower_bound\n";
	    print PLT "b = $upper_bound\n";
	    print PLT "\n";
	    print PLT "plot [a:b][:] \"$file_out\" using 1:$count t\"$family\" w l\n";
	    print PLT "\n";
	    print PLT "pause -1\n";
	    print PLT "\n";
	    print PLT "\#-----------------------------------------------------------------\n";
	    print PLT "\n";

	    if($family !~ m/err\(/i)
	    {
		push(@value, $family);
	    }

	}

	$c =1 ;

	foreach my $item (@value)
	{
	    $c++;
	    $count = 1;
	    for my $family ( sort keys %{$hash->{$final_block} } )
	    {
		$count++;
		if( ( $family =~ m/err\($item\)/i ) ) 
		{
		    print PLT "# quantity: $item with error bars\n";
		    print PLT "\n";
		    print PLT "set title \"$file : $item\"\n";
		    print PLT "set xlabel \"blocks\"\n";
		    print PLT "set ylabel \"$item (a.u.)\"\n";
		    print PLT "\n";
		    print PLT "show title\n";
		    print PLT "show ylabel\n";
		    print PLT "show xlabel\n";
		    print PLT "\n";
		    print PLT "a = $lower_bound\n";
		    print PLT "b = $upper_bound\n";
		    print PLT "\n";
		    print PLT "plot [a:b][:] \"$file_out\" using 1:$c:$count t\"$item\" w yerr\n";
		    print PLT "\n";
		    print PLT "pause -1\n";
		    print PLT "\n";
		    print PLT "\#-----------------------------------------------------------------\n";
		    print PLT "\n";

		    
		}
	    }
	}
   
	close(PLT);
    }
}

sub check_arguments
{
    if( $file eq "" )
    {
	print "\n";
	print "\t need to include file name\n";
	print "\t -f [string]\n";
	print "\t exiting...\n";
	print "\n";
	
	exit;
    }

    if( $value eq "" )
    {
	print "\n";
	print "\t need to include block parsing options\n";
	print "\t -o [string|integer]\n";
	print "\n";
	print "\t -o all       - get all block values\n";
	print "\t -o last      - get last block value\n";
	print "\t -o [integer] - print specific block value\n";
	print "\t exiting\n";
	print "\n"; 
	
	exit;
    }
}

sub print_selected_block
{
    if( $value =~ m/(\d+)/i )
    {
	if    ( $value < 10  ) { $block = "00" . $value; } 

	elsif ( $value < 100 ) { $block = "0"  . $value; }

	else                   { $block = $value;        }

	print "\t ---------------------------------\n";
	print "\t Block: $value              values\n";
	print "\t ---------------------------------\n";

	for my $family ( sort keys %{$hash->{$block}} )
	{
	    my($value) = $hash->{$block}->{$family};
	
	    printf "\t %-15s %15.10f\n", $family, $value;
	}

	print "\t ---------------------------------\n";
	print "\n";
    }
}

sub print_last_values
{
    if($value eq "last")
    {
	print "\t Block $final_block\n";

	for my $family (keys %{$hash->{$final_block}})
	{
	    my($value) = $hash->{$final_block}->{$family};
	
	    printf "\t %-15s %15.10f\n", $family, $value;
	}

	print "\n";
    }
}

sub print_all_values
{
    open(OUT, ">$file_out") or die "cannot create output file\n";

    if($value eq "all")
    {
	print OUT "\#\t";
	
	for my $family ( sort keys %{$hash->{$final_block} } )
	{
	    printf OUT "%20.15s", $family;
	}
	
	print OUT "\n";

	for my $family (sort keys %$hash )
	{
	    printf OUT "%-10d", $family;
	    
	    for my $role (sort keys %{$hash->{$family}})
	    {
		my($value) = $hash->{$family}->{$role};
		
		printf OUT "%20.10f", $value;
	    }
	
	    print OUT "\n";
	}
    }

    close(OUT);
}


sub parse_file
{
    open(IN, "$file") or die "cannot open input file\n";

    my(%hash) = ();
    my($block)= ();
    my($num)  = ();

    while(<IN>)
    {
	chomp $_;
    
	if( $_ =~ m/object\_value\_byind\_write\:/i)
	{
	    $_ =~ m/([a-z0-9\_\-\(\)]+)\s*=(.*)/i;
	    
	    my($item)  = $1;

	    my($value) = $2;
	    
	    $hash{$num}{$item} = $value;
	}

	if($_ =~ m/sample\_bld\s+block\s+\#\s+(\d+)/i)
	{
	    $block       = $1;

	    if    ( $block < 10  ) { $num = "00" . $block; }
	    elsif ( $block < 100 ) { $num = "0"  . $block; }
	    else                   { $num =        $block; }
	    
	    $final_block = $num;
	}
    
    }

    close(IN);

    return \%hash, $final_block;
}

sub help
{
    print <<EOF;

DESCRIPTION:

This script parses the data found in QMCMOL log file.  This script
automatically looks for the objects used in the Monte Carlo menu and holds
the data in a hash table. In addition a gnuplot file is produced for plotting
the data.

OPTIONS:

        -f [string]         - input file with extension
        -o [string|integer] - specify the punch all the data, a specific 
                              block, or last block of data.
        -o all              - print all the data.

        -o last             - print last block
        -o \#               - print a specific block of data. 

EXAMPLE: 

option: -o all

For all the calculated data:

        ./qmcmol-parse-log.pl -f test-vmc.inp -o all

This will produce the following output:

\#    energy         energy_var          err(energy)         err(energy_var)
1    -40.2075352000  9.2596294550       -0.1797693135       -0.1797693135  
2    -40.2246307300  9.8671489150        0.0170955335        0.6072274448  
3    -40.2001641700  9.2902550230        0.0263824141        0.6758292863  
4    -40.2236661800  8.5184290210        0.0300060051        0.9078074953  
5    -40.2251731300  9.0368631040        0.0232913526        0.8736556457  
6    -40.2011729300  9.1315818560        0.0306213644        0.7193073929  

For this case the serial data is found in test-vmc.data file. In addition
a test-vmc.plt gnuplot file is produced which contains plot commands for
each of the MC objects, including error bars. The pause -1 is used to pause
plotting between gnuplot commands. The x- and y-axis are labeled and each 
graph is titled.


option: -o last

For the last data:

        ./qmcmol-parse-log.pl -f test-vmc.inp -o last

         ---------------------------------
         Block: 250             values
         ---------------------------------
         energy           -40.2096001100
         energy_var        10.2481035100
         prob_acc           0.5676955680
         err(energy)        0.0047763755
         percent_err        5.9399806220
         err(energy_var)    0.6087353628
         ---------------------------------

option: -o 51

For a specific block

        ./qmcmol-parse-log.pl -f test-vmc.inp -o 51

         ---------------------------------
         Block: 51              values
         ---------------------------------
         energy           -40.2096102200
         energy_var        11.4471989700
         err(energy)        0.0147525448
         err(energy_var)    2.1874321390
         percent_err       19.1088854700
         prob_acc           0.5665888205
         ---------------------------------

EOF

exit;

}
