#! /usr/bin/perl

use Env qw(HOME PWD SYS LD_LIBRARY_PATH);
use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::Handle;
use Term::ANSIColor;

getopt("befjlqst:",\%args);

$base       = $args{b}; # base file name
$exe        = $args{e}; # qmcmol executable 
$file       = $args{f}; # file type, gamess, gaussian, qmcmol, or mixed
$job        = $args{j}; # job type for adding mo derivatives
$list       = $args{l}; # list file 
$queue      = $args{q}; # queue type
$template   = $args{t}; # template file
$basis_type = $args{s}; # basis file name, used only with mixed option

if( $args{h}               ) { help();                                                                  exit  }
if( $SYS eq ""             ) { print "\tneed to specify: export SYS=hostname, exiting...\n";            exit; }
if( $LD_LIBRARY_PATH eq "" ) { print "\tneed to specify: export LD_LIBRARY_PATH=path\n";                exit; }
if( $base eq ""            ) { print "\tneed to specify: base file name, (-b string), exiting...\n";    exit; }
if( $list eq ""            ) { print "\tneed to specify: list file name, (-l string), exiting...\n";    exit; }
if( $template eq ""        ) { print "\tneed to specify: template file name, (-t string), exiting..\n"; exit; }
if( $file eq ""            ) { print "\tneed to specify: file type (-f gamess|gaussian|qmcmol), exiting...\n"; exit; }

if($queue eq "")  { $queue = "none";        }
if( $exe  eq "" ) { $exe   = "qmcmol.exe";  }
else              { $exe   = $exe . ".exe"; }

print "\n";
print "\t system         : $SYS\n";
print "\t list file      : $list\n";
print "\t base file name : $base\n";
print "\t qmcmol exe     : $exe\n";
print "\t template file  : $template\n";
print "\t queue          : $queue\n";
print "\n";

open(IN,"$list") or die "cannot open input file\n";
open(RUN,">scr") or die "cannot create scr file\n";

print "\t opening $list file...\n";
print "\t creating scr file...\n";
print "\t parsing list file...\n";

while(<IN>)
{
    chomp $_;
    if($_ =~ m/geom/i)
    {
	print "\t\tparsing geometries...";
	while(<IN>)
	{
	    chomp $_;
	    if($_ =~ m/end_geom/i) 
	    {
		print "finished\n";
		last;
	    }
	    if($_ =~ m/(\d+.\d+)/) { push(@array,$1); }
	}
    }
    
    if($_ =~ m/time/i)
    {
	print "\t\tparsing time steps...";
	while(<IN>)
	{
	    chomp $_;
	    if($_ =~ m/end_time/i)
	    {
		print "finished\n";
		last;
	    }
	    if($_ =~ m/(\d+.\d+)/) { push(@time,$1); }
	}
    }

    if($_ =~ m/force/i)
    {
	print "\t\tparsing forces...";
	while(<IN>)
	{
	    chomp $_;
	    if($_ =~ m/end_force/i)
	    {
		print "finished\n";
		last;
	    }
	    if($_ =~ m/(\d+.\d+|-\d+\.\d+)/) { push(@force,$1); }
	}
    }    

    if($_ =~ m/energy/i)
    {
	print "\t\tparsing energies...";
	while(<IN>)
	{
	    chomp $_;
	    if($_ =~ m/end_energy/i)
	    {
		print "finished\n";
		last;
	    }
	    if($_ =~ m/(\d+.\d+|-\d+\.\d+)/) { push(@energy,$1); }
	}
    }    
}

print "\tfinished\n";
print "\n";

if( @geom eq "" )
{
    print "need to specify geom list, as\n";
    print "geom\n";
    print "\t 1.23\n";
    print "\t 1.32\n";
    print "\t ...\n";
    print "\t 1.45\n";
    print "end_geom\n";
}

$count=0;

foreach $item (@array)
{
    $file_scr = $base . "-" . $item . ".scr";
    $file_inp = $base . "-" . $item . ".inp";
    $file_out = $base . "-" . $item . ".out";
 
    if($file eq "gamess")
    {
	if($job eq "diff") { $file_dat = $base . "-diff-" . $item . ".dat"; }
	else               { $file_dat = $base . "-"      . $item . ".dat"; }
    }

    if( ($file eq "qmcmol") )
    {
	$file_mos = $base . "-" . $item . ".mos"; 
        $file_bas = $base . "-" . $item . ".bas";
    }

    if( $file eq "mixed" )
    {
	$file_mos = $base . "-" . $item . ".dat"; 
        $file_bas = $basis_type;
    }

    $par_in   = $base . "-" . $item . ".par-in";
    $par_out  = $base . "-" . $item . ".par-out";
    $wlk_in   = $base . "-" . $item . ".wlk-in";
    $wlk_out  = $base . "-" . $item . ".wlk-out";
    $jas_in   = $base . "-" . $item . ".jas-in";
    $jas_out  = $base . "-" . $item . ".jas-out";
    $keep     = $template;

    print "$file_inp $item...\n";
    print "\t creating $file_scr file...\n";

    open(SCR,">$file_scr") or die "cannot create script file\n";

    if($SYS eq "hydrogene")
    {
	print SCR "\#!/bin/sh\n";
	print SCR "\#\n";
	print SCR "cd $HOME\n";
	print SCR "cd $PWD\n";
	print SCR "\#\n";
	print SCR "./$exe < $file_inp > $file_out\n";

	print RUN "sub -l MC=1 $file_scr\n";
    }

    if($SYS eq "ganita")
    {
	print SCR "#!/bin/sh\n";
	print SCR "\$ -N $base-$item\n";
	print SCR "setenv LD_LIBRARY_PATH /home/software/lf9560/lib/\n";
	print SCR "setenv QMCMOL_UNAME_a \`uname -a\`\n";
	print SCR "setenv QMCMOL_UNAME_m \`uname -m\`\n";
	print SCR "setenv QMCMOL_UNAME_s \`uname -s\`\n";
	print SCR "setenv PWD \`pwd\`\n";
	print SCR "cd $HOME\n";
	print SCR "cd $PWD\n"; 
	print SCR "./$exe < $file_inp > $file_out\n";

	if($queue ne "") { print RUN "qsub -l $queue=1 $file_scr\n"; }
	else             { print RUN "qsub $file_scr\n";             }
    }

    close(SCR);

    print "\t closing $file_scr file...\n";
    print "\t opening template file...\n";
    print "\t creating $file_inp file...\n";

    open(IN,"$keep") or die "cannot open keep file\n";
    open(OUT,">$file_inp") or die "cannot create input file\n";

    line: while(<IN>)
    {	
	chomp $_;
        if($_ =~ m/\!/)
        {
            print OUT "$_\n";
            next line;
        }
	elsif($_ =~ m/change_geom/i)
	{
	    print OUT "              coordinates = ( 0.0, 0.0, $item )\n";
	    $found_geom = "true";
	}
	elsif($_ =~ m/\.mos/i)   { print OUT "         name    = $file_mos\n"; }
        elsif($_ =~ m/\.bas/i)   { print OUT "         name    = $file_bas\n"; }
	elsif($_ =~ m/\.dat/i)   { print OUT "         name    = $file_dat\n"; }
	elsif($_ =~ m/wlk-out/i) { print OUT "         name    = $wlk_out\n";  }
	elsif($_ =~ m/wlk-in/i)  { print OUT "         name    = $wlk_in\n";   }
	elsif($_ =~ m/jas-out/i) { print OUT "         name    = $jas_out\n";  }
	elsif($_ =~ m/jas-in/i)  { print OUT "         name    = $jas_in\n";   }
	elsif($_ =~ m/par-in/i)  { print OUT "         name    = $par_in\n";   }
	elsif($_ =~ m/par-out/i) { print OUT "         name    = $par_out\n";  }

	elsif( ($_ =~ m/change_force/i) && (@force ne "") )
	{
	    print OUT "         force_hf             = real ( @force[$count] )\n";
	}

	elsif( ($_ =~ m/change_time_step/i) && (@time ne "") )
	{
	    print OUT "   time_step  = @time[$count]\n";
	}

	elsif( ($_ =~ m/change_energy/i) && (@energy ne "") )
	{
	    print OUT "         hf_energy             = real ( @energy[$count] )\n";
	}

	else { print OUT "$_\n"; }
    }

    close(IN);
    close(OUT);

    if($found_geom ne "true")
    {
	print "\t cannot find change_geom keyword in the template file, exiting...\n";
    }

    print "\t closing template file...\n";
    print "\t closing $file_inp file...\n";
    print "\n";

    $count++;

}

close(RUN);

print "\t closing scr file...\n";
print "\t finished!\n";

sub help
{
    warn "script: @_\n" if @_;
    die <<EOF;
Options are:
            -h help
            -b [string] base file name. The final output file will be created as a concatention of the
                        base name and the geom, i.e., base-geom.inp,
            -t [string] template qmcmol file.
            -q [string] queue type. for use on ganita only.
            -j [string] job type for mo derivatives only. 
            -l [string] list file.  
                        This file must contain a list of geometries in the format,

               geom
                       1.40
                       1.30
                       1.50
                       ...
                       3.015
               end_geom

               The script goes through are changes the geometry where change geom_is specified, i.e.,

                        atom
                           name        = H_b
                           symbol      = H
                           charge      = 1
                           coordinates = ( 0.0, 0.0, change_geom )
                           pseudo      = all
                        end_atom

               Other quantities, though not necessary, are the time step, energy, and force specified by:
 
               time_step
                       ...
               end_time_step


               force
                       ...
               end_force

          
               energy
                       ...
               end_energy
  
               These are specified by change_time_step, change_energy, and change_force in the appropriate places.
               The lists must contain as many entries as different geometries.
EOF
}
