#! /usr/bin/perl

use IO::Handle;
use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::Handle;

$| = (1>0);
getopt("bilst",\%args);

$base   = $args{b}; # base file name
$iter   = $args{i}; # iteration to search for
$list   = $args{l}; # list file
$suffix = $args{s}; # suffix .out
$type   = $args{t}; # job type all gets data over all blocks

if( $args{h}         ) { help(); exit; }
if( $base   eq ""    ) { print "\tbase file needs to be specified, (-b [string]), exiting...\n"; exit; }
if( $list   eq ""    ) { print "\tlist file needs to be specified, (-l [string]), exiting...\n"; exit; }
if( $suffix eq ""    ) { print "\tsuffix needs to be specified, (-s [string]), exiting...\n";    exit; }
if( $iter   ne ""    ) { $iter++;              }
if( $type   eq ""    ) { $type = "last value"; } 

print "\n";
print "\tbase file name : $base\n";
print "\tlist file      : $list\n";
print "\tsuffix         : $suffix\n";
print "\tparse type     : $type\n";
print "\n";
print "\topening $list file...\n";

open(LIST,"<$list") or die "cannot open list ($list) file..exiting\n";

while(<LIST>)
{
    chomp $_;
    if($_ =~ m/geom/i)
    {
	print "\t\tparsing geometries...";

	geom: while(<LIST>) 
	{
	    chomp $_;
	    if    ( $_ =~ m/end_geom/i ) { last;      }
            elsif ( $_ eq ""           ) { next geom; }
	    elsif ( $_ =~ m/\#/i       ) { next geom; }
	    else
	    {
		$_ =~ m/(\d+\.\d+|-\d+\.\d+|\d+)/i;
		push(@geom,$1);
	    }
	}
	
	print "\t...finished\n\n";
    }

    if( $_ =~ m/val/i )
    {
	print "\t\tparsing values...";
	
	val: while(<LIST>)
	{
	    chomp $_;
	    if    ( $_ =~ m/end_val/i ) { last;     }
	    elsif ( $_ =~ m/\#/i      ) { next val; }
	    elsif ( $_ eq ""          ) { next val; }
	    else
	    {
		$_ =~ m/([a-zA-Z0-9_\(\)]+)/;

		$val      = $1;
		$val_file = $1;

		$val_file =~s/\(/\_/g;
		$val_file =~s/\)/\_/g;

		$val =~s/\(/\\\(/g;
		$val =~s/\)/\\\)/g;

		push(@files, $val_file);
		push(@values,$val);
	    }
	}

	print "\t...finished\n\n";
    }
}

print "\tfinished parsing $list file\n";
print "\tclosing $list file\n";

close(LIST);

$size_geom = @geom;
$size_val  = @values;

if( $size_geom == 0 )    { print "\n\tno geometies found in $list file, exiting...\n";             exit; }
if( $size_val  == 0 )    { print "\n\tno values found in $list file, exiting...\n";                exit; }
if( @geom[0] eq ""  )
{ 
    @geom[0] = "none";
    print "\n\tno geometries specified, only one file will be parsed...\n"; 
}

$count_file = 0;

foreach $val (@values)
{
    $file_out = @files[$count_file] . ".dat";
    
    open(OUT,">$file_out") or die "cannot create $file_out file\n";

    line: foreach $item (@geom)
    {
	if($item =~ m/none/i)
	{
	    $file_in = $base . "." . $suffix;
	}
	
	if($item !~ m/none/i)
	{
	    $file_in  = $base . "-" . $item . "." . $suffix;
	    $count = 1;
	}

	open(IN,"$file_in") or die "cannot open $file file\n";

        print "$val $file_in\n";

	while(<IN>) 
	{
	    if(($count == $iter) && ($iter ne ""))
	    {
		close(IN);
		print OUT "$item\t$value\t$err\n";
		$value=();
		next line;
	    }

	    if($_ =~ m/\s$val\s*=(.*)/)         
	    { 
		$value = $1; 
		if($value =~ m/\d+\.\d+/i) { push(@ave,$value); }
	    }

            if($_ =~ m/$val\s*=(.*)/)           { $value = $1; }

	    if($_ =~m/\serr\S*\($val\).*=(.*)/) 
	    { 
		$err   = $1;
		push(@error,$err);
	    }

	    if($value ne "") { $count++;    }
	}
 
	if($type eq "all")
	{
	    if(@files[$count_file] =~ m/\_\_/i)
	    {
		$output = @files[$count_file] . $item . ".dat";
	    }
	    else
	    {
		$output = @files[$count_file] . "-" . $item . ".dat";
	    }

	    open(ALL, ">$output") or die "cannot open output file\n";
	    for($i=0;$i<@ave;$i++)
	    {
		$j = $i + 1;
		printf ALL"%5s %15.10f %15.10f\n",$j,@ave[$i],@error[$i];  
	    }
	    close(ALL);

	    @ave   = ();
	    @error = ();
	}
	
	if($iter eq "") 
	{
	    @ave   = ();
	    @error = ();

	    print OUT "$item\t$value\t$err\n"; 
	}

	$value = ();
	$error = ();

	close(IN);
    }

    close(OUT);

    $count_file++;
}

sub help
{
    warn "script: @_\n" if @_;
    die <<EOF;

  Options are:
            -h help
            -b [string] base file name. The final output file will be created as a concatention of the
                        base name and the geom, i.e., base-geom.inp,
            -t [string] template qmcmol file.
            -q [string] queue type. for use on ganita only.
            -j [string] job type for mo derivatives only. 
            -l [string] list file.  
                        This file must contain a list of geometries in the format,

               geom
                       1.40
                       1.30
                       1.50
                       ...
                       3.015
               end_geom
               The script goes through are changes the geometry where change geom_is specified, i.e.,

                        atom
                           name        = H_b
                           symbol      = H
                           charge      = 1
                           coordinates = ( 0.0, 0.0, change_geom )
                           pseudo      = all
                        end_atom

               Other quantities, though not necessary, are the time step, energy, and force specified by:
 
               time_step
                       ...
               end_time_step

               force
                       ...
               end_force
          
               energy
                       ...
               end_energy
  
               These are specified by change_time_step, change_energy, and change_force in the appropriate places.
               The lists must contain as many entries as different geometries.
EOF
}
