#! /usr/bin/perl

use Getopt::Std;
use File::Find;
use Env qw(SLATER_BASIS_REPO PWD);

use basis_parser;
use tools;
use atom_parser;
###############################################################################

@DIRLIST[0]  = "$SLATER_BASIS_REPO";

getopt("fhpq",\%args);

my($help)         = $args{h}; # help
my($library_file) = $args{f}; # library file
my($print)        = $args{p}; # print basis set data from repository
my($query)        = $args{q}; # query basis set repository

###############################################################################

my($path) = check_input();

my($input_file, $output_file) = create_files($path, $library_file);

print_basis_help();   
 
print_help(); 

query_basis_set($query);

print_basis_set($print); 

#############################################################################

my($element, $whose)               = parse_library_file_name($library_file);

my($r_power, $symmetry, $exponent) = parse_slater_basis_file($input_file);

#############################################################################

open(OUT, ">$output_file") or die 
    "\n in print_basis_cascade routine\n \tcannot create output file\n";

print_atom_cascade($element);

print_basis_cascade($r_power, $symmetry, $exponent);

close(OUT);

print_to_screen();

#############################################################################

sub print_atom_cascade
{
    my($element)   = $_[0];

    my($ATOM)      = atom_parser->ATOM_DATA($element,"query", $charge);
    
    my($atom_name) = $ATOM->{element_name};

    my($charge)    = $ATOM->{element_charge};

    print OUT "\n";
    print OUT "element\n";
    print OUT "\t charge  = $charge\n";
    print OUT "\t name    = $atom_name\n";
    print OUT "\t label   = $element\n";
    print OUT "end_element\n";
    print OUT "\n";
}
 
sub create_files
{
    my($path)         = $_[0];

    my($library_file) = $_[1];

    my($input_file)   = "$path/$library_file";

    my(@tmp)          = split(/\.dat/,$library_file);       

    my($base)         = @tmp[0];

    my($output_file)  = "$PWD/$base.bas";

    return $input_file, $output_file;
}

sub parse_slater_basis_file
{
    my($input_file) = $_[0];

    my($STO)      = basis_parser->STO_BASIS_DATA($input_file, slater);

    my($r_power)  = $STO->{r_power};

    my($symmetry) = $STO->{symmetry};

    my($exponent) = $STO->{exponent};

    return $r_power, $symmetry, $exponent;
}

sub parse_library_file_name
{
    my($library_file) = $_[0];

    $library_file =~ m/([a-z]+)\-([a-z0-9]+)\-/i;

    my($element)  = $1;

    my($whose)    = $2;

    return $element, $whose;
}

sub check_input
{
    my($path) = ();

    print "\n";
    print "\t in check_input routine:\n";

    if( $SLATER_BASIS_REPO eq "" )
    {
	print "\t\t SLATER_BASIS_REPO variable is missing\n";
	print "\t\t exiting...\n";
	print "\n";

	exit;
    }
    
    if ( -e "$SLATER_BASIS_REPO/$library_file" ) 
    { 
	print "\t\t File $library_file found in:\n";
	print "\t\t $SLATER_BASIS_REPO directory\n";
	print "\n";

	return $SLATER_BASIS_REPO;
    }

    if( -e "$PWD/$library_file" )
    {
	print "\t\t File $library_file found in:\n";
	print "\t\t $PWD directory\n";
	print "\n";

	return $PWD;
    }

    else
    {
	print "\t\t Slater basis file $library_file cannot be found\n";
	print "\t\t Checking spelling or if desired file is really in\n";
	print "\t\t $SLATER_BASIS_REPO directory\n";
	print "\t\t exiting...\n";
	print "\n";

	exit;
    }
}

sub print_to_screen
{
    print "\t Output file found in $PWD\n";
    print "\n";
    print "\t input file  : $input_file\n";
    print "\t output file : $output_file\n";
    print "\n";
}

sub print_basis_help
{
    if( $help eq "basis"    ) 
    {

	open(README,"@DIRLIST[0]/README") or die "cannot open readme file\n";
    
	print "\n";

	while(<README>)
	{
	    chomp $_;
	
	    print "\t$_\n";
	}

	close(README);

	print "\n";

	exit;
    }
}

sub print_basis_set
{
    my($print) = $_[0];

    if( $print ne ""        ) 
    {

	print "$print\n";
	print "\n";
    
	my($input_file) = @DIRLIST[0] . "/" . $print;
    
	open(INP,"$input_file") or die "cannot open input file\n";
    
	while(<INP>)
	{
	    chomp $_;
	
	    print "$_\n";
	}
    
	close(INP);
    
	print "\n";
    
	exit;        
    }
}

sub query_basis_set
{
    my($query) = $_[0];

    if( $query ne "" ) 
    {
	$atom_query = lc($query);

	print "\n";
	print "\t Querying the Slater basis repository found in\n";
	print "\t\t $SLATER_BASIS_REPO\n";
	print "\n";
	print "\t The following basis sets are available for the $atom_query atom\n";
	print "\n";
	
	find( \&process_basis_file, @DIRLIST);

	print "\n";
	print "\t $basis_set_i Slater basis sets found for $query\n";
	print "\n";

	exit;
    }
}

sub process_basis_file
{
    $_ =~ m/([a-z]+)\-/i;
    
    my($element) = lc($1);
    
    if($element eq $atom_query)
    {
	$basis_set_i++;

	print "\t\t$_\n";
    }

}

sub determine_r_power
{
    my($symmetry) = @_[0];
    my($r_power)  = @_[1];
    my($pow_i)    = $_[2];

    my($power)    = ();

    if($symmetry->[$pow_i] =~ m/s/i)
    {
	$power = $r_power->[$pow_i];
    }
    
    elsif($symmetry->[$pow_i] =~ m/p/i)
    {
	$power = $r_power->[$pow_i];
    }
    
    elsif($symmetry->[$pow_i] =~ m/d/i)
    {
	$power = $r_power->[$pow_i];
    }
    
    elsif($symmetry->[$pow_i] =~ m/f/i)
    {
	$power = $r_power->[$pow_i];
    }
    
    elsif($symmetry->[$pow_i] =~ m/g/i)
    {
	$power = $r_power->[$pow_i];
    }
    
    else
    {
	print "\n";
	print "\t in determine_r_power routine:\n";
	print "\t\t incorrect basis function symmetry specified\n";
	print "\t\t current symmetry is: $symmetry->[$pow_i]\n";
	print "\t\t exiting...\n";
	print "\n";
	
	exit;
    }
    
    return $power;
}

sub print_basis_cascade
{
    my($r_power)  = @_[0];
    my($symmetry) = @_[1];
    my($exponent) = @_[2];

    my($power_nb) = tools->size_dim1_array($r_power);

    for($pow_i=0; $pow_i< $power_nb; $pow_i++)
    {
	my($power) = determine_r_power($symmetry, $r_power, $pow_i);

	print OUT "primitive\n";
	print OUT "\tsymmetry    = $symmetry->[$pow_i]\n";
	print OUT "\tr_power     = $power\n";
	print OUT "\texponent    = $exponent->[$pow_i]\n";
	print OUT "\tcoefficient = 1.0\n";
	print OUT "end_primitive\n";
	print OUT "\n";
    }
}


sub print_help
{
    if( $help eq "help"     ) 
    {

    print <<EOF;

Module Dependence:
   
    This script depends on the following perl modules:

        basis_parser
    
    To have these modules accessible export the following variable in the
    .bashrc file:

        export PERL5LIB="path to modules".

    Example:

	To point to the modules in the QMCMOL CVS:

	export PERL5LIB=$HOME/QMCMOL/TOOLS/USER:.

Variables:

    The following \$PATH variable needs to be set in the .bashrc or equivalent 
    file specifying the repository of the Slater basis set files:

        export SLATER_BASIS_REPO="path to repository"
	
    Example:

	Using the repository found in the QMCMOL CVS:

	export SLATER_BASIS_REPO=\$HOME/QMCMOL/BASIS

Options:

    The following are options used with this scripts. See Usage Examples section
    for use details.
    
        -f [string] - convert a file in the Slater repository to cascade format.
                      Location of the library is given by the SLATER_BASIS_REPO
		      variable.

	-p [string] - print the desired Slater basis.

	-q [string] - query atom to see list of available basis sets:

Usage Examples:

    -q Query by atom abbreviation or name:

        To obtain a list of all the Slater sets by atom (-q atom label). 
        For example to obtain a list of all available Slater basis sets for
        the He atom:

        Specify the atom abbreviation:

	     slater-basis-to-cascade.pl -q He 

	Alternatively the name of the atom can be used:

	     slater-basis-to-cascade.pl -q Helium

        This will produce the following list of files:

        He-clementi-hf-neutral-ground-IBM.dat
        He-thakkar-hfcusp-neutral-ground-IJQC.dat
        He-adf-dzp-neutral-ground-ref.dat
        He-bunge-hf-neutral-ground-ADND.dat
        He-thakkar-dz-neutral-ground-BCSJ.dat
        He-adf-dz-neutral-ground-ref.dat
        He-clementi-dz-neutral-ground-ADND.dat
        He-thakkar-sz-neutral-ground-TCA.dat
        He-adf-sz-neutral-ground-ref.dat
        He-thakkar-det-neutral-ground-TCA.dat
        He-thakkar-clemreopt-neutral-ground-PRA.dat
        He-clementi-hf-neutral-ground-ADND.dat
        He-thakkar-hf-neutral-ground-JCP.dat
        He-clementi-sz-neutral-ground-ADND.dat
        He-thakkar-et-neutral-ground-TCA.dat
        He-sarsa-hf-neutral-ground-ADND.dat


    -p Print requested atom basis information

    	     slater-basis-to-cascade.pl -p He-clementi-dz-neutral-ground-ADND.dat

        This will print the data of the file to screen


    -f convert requested file to cascade format. 

    	     slater-basis-to-cascade.pl -f He-clementi-dz-neutral-ground-ADND.dat

       This will convert   

       He-clementi-dz-neutral-ground-ADND.dat to 
       He-clementi-dz-neutral-ground-ADND.bas

OUTPUT:
       
       The output file is contains data in the cascading menu format
       consisting of command and token=value data.

       primitive                     <- being primitive command
	    symmetry    = S            <- symmetry of the basis function
	    r_power     = 1            <- power of r used in the expansion
	    exponent    = 2.9109300    <- exponent value
	    coefficient = 1.0          <- coefficient value
       end_primitive                 <- end primitive command

Notes:

    This script does not currently work with the adf basis sets found
    in the Slater basis repository.

EOF
exit;
}
}
