#! /usr/bin/perl

use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::File;

use atom_parser;
use basis_parser;

getopt("fw",\%args);

my($input_file) = $args{f};
my($whose)      = $args{w};

$ATOM = atom_parser->ATOM_DATA($input_file,"slater");

$STO  = basis_parser->STO_BASIS_DATA($input_file,"slater");

print_header(
	     $ATOM->{element_label},
	     $whose
	     );

print_qmcmol_basis_format( 
			   $STO->{r_power},
			   $STO->{symmetry},
			   $STO->{exponent},
			   $STO->{coefficient},
			   $STO->{function_number}
			   );

print_footer();

sub print_footer
{
    print "end_elem\n";
    print "end_of_basis\n";
}

sub print_qmcmol_basis_format
{
    my($r_power)     = @_[0];
    my($symmetry)    = @_[1];
    my($exponent)    = @_[2];
    my($coefficient) = @_[3];
    my($function_nb) = $_[4];

    for($pow_i = 0; $pow_i< $function_nb; $pow_i++)
    {
	if($symmetry->[$pow_i] =~ m/s/i)
	{
	    $power = $r_power->[$pow_i];
	}

	elsif($symmetry->[$pow_i] =~ m/p/i)
	{
	    $power = $r_power->[$pow_i]-1;
	}

	elsif($symmetry->[$pow_i] =~ m/d/i)
	{
	    $power = $r_power->[$pow_i]-2;
	}

	print "\tsymmetry $symmetry->[$pow_i]\n";
	print "\t\tsto $power $exponent->[$pow_i] $coefficient->[$pow_i]\n";
	print "\tend_sym\n";
	print "\n";
    }
}

sub print_header
{
    my($label) = $_[0];
    my($whose) = $_[1];

    print "title: $whose STF basis set for $label atom\n";
    print "end_of_header\n";
    print "\n";
    print "elem $label\n";
}
