#! /usr/bin/perl

use Math::Trig;

use basis_parser;
use mo_parser;
use atom_parser;

my($input_file) = @ARGV[0];

$input_file =~ m/([a-z]+)/i;

$element = $1;

$ATOM = atom_parser->ATOM_DATA($element, "query");
$STO  = basis_parser->STO_BASIS_DATA($input_file,"slater");
$MO   = mo_parser->STO_MO_DATA($input_file, "slater");

$charge = $ATOM->{element_charge};

$exponent = $STO->{exponent};
$symmetry = $STO->{symmetry};
$r_power  = $STO->{r_power};
$mo_hash  = $MO->{mo};

for $family ( sort keys %$mo_hash )
{
    if($family =~ m/s/i)
    {
	push(@s_orbital, $family)
    }
}

printf "%5s %9s %17s\n","Orbital","Cusp","Deviation";

foreach $item (@s_orbital)
{   
    $phi     = 0.0;
    $phi_der = 0.0;
    $count   = 0;
    
    for $role ( sort keys %{$mo_hash->{$item}} )
    {
	$c    = $mo_hash->{$item}->{$role};
	$zeta = $exponent->[$count];
	$n    = $r_power->[$count];

	my($norm)       = slater_spherical_normalization($zeta, $n);
	my($radial)     = slater_radial($n);
	my($radial_der) = slater_radial_derivative($n, $zeta);
	
#	print "TEST   : $exponent->[0]\n";
#	print "zeta   : $zeta\n";
#	print "c      : $c\n";
#	print "norm   : $norm\n";
#	print "radial : $radial\n";

	$phi     = $phi + $c * $norm * $radial;
	$phi_der = $phi_der + $c * $norm * $radial_der;

	$count++;
    }

#    print "PHI: $phi\n";

#    exit;

    $cusp = $phi_der / $phi;
    $deviation = $cusp + $charge;

    printf "%0.15s %15.5f %15.5f\n", $item, $cusp, $deviation
}

sub slater_spherical_normalization
{
    my($zeta) = $_[0];
    my($n)    = $_[1];

    my($norm) = (( 2.0 * $zeta )**(2 * $n + 1.0)/factorial(2.0*$n))**(0.5);

    return $norm;
}

sub slater_radial_derivative
{
    my($n)      = $_[0];
    my($zeta)   = $_[1];
    my($radial) = ();

    if($n == 1) { $radial = -$zeta; }
    if($n == 2) { $radial = 1.0;    }
    if($n > 2 ) { $radial = 0.0;    }

    return $radial;
}

sub slater_radial
{
    my($n)    = $_[0];
    my($radial) = ();
    
    if($n == 1) { $radial = 1.0; }
    if($n > 1)  { $radial = 0.0; }

    return $radial;
}

sub factorial
{
    my ($n) = @_;

    if    ( $n == 0 ) { return 1;                    }
    elsif ( $n < 2  ) { return $n;                   }
    else              { return $n * factorial($n-1); }
}
