#! /usr/bin/perl
use Env qw(HOME PWD SYS LD_LIBRARY_PATH USER);
use English;
use FileHandle;
use File::Copy;
use Getopt::Std;
use File::Basename;
use IO::Handle;
use Term::ANSIColor;

getopt("bhlst:",\%args);

$base      = $args{b}; # base file name
$help      = $args{h}; # help
$suffix    = $args{s}; # file suffix
$list      = $args{l}; # list file
$template  = $args{t}; # template file

if( $help eq "help"        ) { help(); exit; }

if($base eq "") 
{
    print "\n";
    print "\tneed to include base file name, -b [string], exiting...\n";
    print "\n";
    exit;
}

if($template eq "") 
{
    print "\n";
    print "\tneed to include template file name, -t [string], exiting...\n";
    print "\n";
    exit;
}

if($suffix eq "")
{
    print "\n";
    print "\tneed to include suffix, -s [string], exiting...\n";
    print "\n";
    exit;
}

if($list eq "")
{

    print "\n";
    print "\tneed to include list file name, -l [string], exiting...\n";
    print "\n";
    exit;
}

open(IN,"$list") or die "cannot open input file\n";
while(<IN>)
{
    chomp $_;
    if($_ =~ m/geom/)
    {
	inner: while(<IN>)
	{
	    chomp $_;
	    if($_ =~ m/end_geom/i) { last;            }
	    if($_ =~ m/\#/i)       { next inner;      }
	    if($_ =~ m/(\d+.\d+)/) { push(@array,$1); }
	}
    }
}
close(IN);

print "\n";
print "\t $template file ->\n";
print "\t\t\t\t";

foreach $item (@array)
{
    $count++;
    $out  = $base . "-" . $item . "." . $suffix;
    if( $count%5 ==0) { print "$out\n"; print "\t\t\t\t";}
    else              { print "$out "; }

    system("cp $template $out");
}
print "\n";
print "\n";

sub help
{
    warn "script: @_\n" if @_;
    die <<EOF;

This script copies .out to .in files (jastrow, walker, and parameter files)

Options are:
            -h help
            -b [string] base file name.
            -s [string] suffix, i.e., jas-in, par-in etc.
            -t [string] template file containing quantities to be copied over.
            -l [string] list file containing all geometries.
	                of the form:
               geom
                       1.40
                       1.30
                      #1.50
                       ...
                       3.015
               end_geom

                        The "#" symbol is used to comment out the particular
                        line.

Example:

         template_copy.pl -b lih-sbk -l list -s par-in -t keep.par-in

EOF
}
