#! /usr/bin/perl

my($file_in) = @ARGV[0];

if(@ARGV[0] eq "-h"){ help ();}

my(@tmp) = split(/\./,$file_in);

my($file_out) = @tmp[0] . ".wiki";

my($latex_data) = create_latex_array($file_in);

my($hash) = find_replace_hash();

open(OUT,">$file_out") or die "cannot create $file_out as output file\n";

output();

foreach $line (@$latex_data)
{
    chomp $line;

    ($line) = remove_markup();
    ($line) = replace_inline_math();
    ($line) = replace_begin();
    ($line) = replace_text_color();

    print OUT "$line\n";
}

close(OUT);

sub replace_begin
{
    my(@array) = ();
    my($mark)  = ();

    @array[0] = "\\\\begin\\{slide\\}\\{(\.\*\?)\\}";
    @array[1] = "\\\\chapter\\{(\.\*\?)\\}";
    @array[2] = "\\\\subsection\\{(\.\*\?)\\}";
    @array[3] = "\\\\section\\{(\.\*\?)\\}";

    foreach my $item (@array)
    {
	if($item =~ m/\\\\subsection/i)       { $mark = "=="; } 
	if($item =~ m/\\\\section/i)          { $mark = "=";  }
	if($item =~ m/\\\\chapter/i)          { $mark = "=";  }
	if($item =~ m/\\\\begin\{{slide\\}/i) { $mark = "=";  }

	if($line =~ m/$item/i)
	{
	    $line =~ m/$item/i;
	    $line = $mark . $1 . $mark;
	}
    }

    return $line;
}

sub replace_inline_math
{
    my($item) = "\\\$(\.\*\?)\\\$";

    if($line =~ m/$item/ig)
    {
	my($type) = $1;
	$line =~ s/$item/\<math\>$type\<\/math\>/ig;
    }
    
    return $line;
}


sub replace_text_color
{
    my($item) = "\\\\textcolor\\{[a-z]+\\}\\{(\.\*\?)\\}";
    
    if($line =~ m/$item/i)
    {
	my($data) = $1;
	$line =~ s/$item/$data/i;
    }

    return $line;
}

sub create_latex_array
{
    my($file)       = $_[0];
    my(@latex_data) = ();

    open(TEX,"$file") or die "cannot open $file latex file\n";

    @latex_data = <TEX>;

    close(TEX);

    return \@latex_data;
}

sub remove_markup
{
    my($string)      = ();
    my($replace)     = ();

    foreach $key (keys %$hash)
    {
	$string  = $key;
	$replace = $hash->{$key};
	
	if($line =~ m/$string/i)
	{
	    $line = substitute($line, $string, $replace);
	}
    }

    return $line;
}

sub find_replace_hash
{
    my(%replace_hash) = ();
    
    my($open) = "\\\\";

    $string        = "\\\\begin\{equation\}";
    $replace       = "<math><center>";
    $hash{$string} = $replace;

    $string        = "\\\\begin\{eqnarray\}";
    $replace       = "<math><center>";
    $hash{$string} = $replace;

    $string        = "\\\\widetilde";
    $replace       = "\\tilde";
    $hash{$string} = $replace;

    $string        = "\\\\label\\{\[a\-z0\-9\\:\\_\]+\\}";
    $replace       = " ";
    $hash{$string} = $replace;

    $string        = "\\\\end\{equation\}";
    $replace       = "<\/center><\/math>";
    $hash{$string} = $replace;

    $string        = "\\\\end\{eqnarray\}";
    $replace       = "<\/center><\/math>";
    $hash{$string} = $replace;

    $string        = "\\\\hspace\\{\[a\-z0\-9\.\]+\\}\\.";
    $replace       = " ";
    $hash{$string} = $replace;

    $string        = "\\%\\%";
    $replace       = " ";
    $hash{$string} = $replace;

    $string        = "\\\\begin\{Verbatim\}\\[\.\*";
    $replace       = "<pre>";
    $hash{$string} = $replace;

    $string        = "\\\\end\{Verbatim\}";
    $replace       = "</pre>";
    $hash{$string} = $replace;

    $string        = "\\\\end\{slide\}";
    $replace       = "";
    $hash{$string} = $replace;

    $string        = "\\\\clearpage";
    $replace       = "";
    $hash{$string} = $replace;

    $string        = "\\\\newpage";
    $replace       = "";
    $hash{$string} = $replace;

    return \%hash
}

sub substitute
{
    my($line)    = $_[0];
    my($string)  = $_[1];
    my($replace) = $_[2];

    if($line =~ m/$string/i)
    {
	$line =~ s/$string/$replace/ig;
    }
   
    return $line;
}

sub output
{
    print "\n";
    print "\tinput file  : $file_in\n";
    print "\toutput file : $file_out\n";
    print "\n";
}

sub help
{
    warn "tex-to-wiki: @_\n" if @_;
    die <<EOF;

  usage: tex-to-wiki.pl file.tex

  The parsed/reformatted data is found in file.wiki.  

 Limitations:
 
 *Tables cannot be parsed yet.
 *Itemized lists.
 *Limited latex syntax is currently parsed.

EOF
}
