! -*- f90 -*-
!-----------------------------------------------------------------------------
!          This file is part of QmcMol A GNU QMC program for molecules.
!       Copyright (C) 2002 CNRS - U.P.M.C. Paris 6 - FRANCE
!          R. Assaraf, M. Caffarel, F. Colonna, X. Krokidis, B. Levy, 
!          P. Pernod, and P. Reinhardt - qmcmol@lct.jussieu.fr
!                       http://www.lct.jussieu.fr/QmcMol
! QmcMol is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the Free
! Software  Foundation;  either version 2, or (at your option) any later
! version. QmcMol is distributed in the hope that it will be useful,but
! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
! or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
! for  more  details.  You  should  have  received a copy of the GNU General
! Public  License  along  with  QmcMol;  see the file COPYING.  If not,
! write to the  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
! Boston, MA 02111-1307, USA.
!-----------------------------------------------------------------------------

module jastrow_u_een_bywlk_mod

  use basic_types_tools_mod
  use ee_r_bar_bywlk_mod
  use ea_r_bar_bywlk_mod
  use jastrow_u_ee_bywlk_mod
  use jastrow_u_en_bywlk_mod

! Creates     : u_een_bywlk                       (elec, elec, atom, conf, walk)
! Creates     : u_een_grad_bywlk                  (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_bywlk                  (elec, elec, atom, conf, walk)
! Creates     : u_een_a_der_bywlk               (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_bee_der_bywlk            (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_bee_der2_bywlk           (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_ben_der_bywlk            (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_ben_der2_bywlk           (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_bee_ben_der2_bywlk      (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_a_ben_der2_bywlk         (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_a_bee_der2_bywlk         (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_a_der_bywlk          (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_ben_der_bywlk       (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_bee_der_bywlk       (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_ben_der2_bywlk      (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_bee_der2_bywlk      (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_bee_ben_der2_bywlk (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_a_ben_der2_bywlk    (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_grad_a_bee_der2_bywlk    (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_a_der_bywlk          (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_ben_der_bywlk       (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_ben_der2_bywlk      (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_bee_der_bywlk       (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_bee_der2_bywlk      (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_a_ben_der2_bywlk    (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_a_bee_der2_bywlk    (elec, elec, atom, dim, conf, walk)
! Creates     : u_een_lapl_bee_ben_der2_bywlk (elec, elec, atom, dim, conf, walk)

  real(dp), pointer :: u_een_bywlk                       (:,:,:,:,:)      => NULL()
  real(dp), pointer :: u_een_a_der_bywlk               (:,:,:,:,:,:)    => NULL()
  real(dp), pointer :: u_een_bee_der_bywlk            (:,:,:,:,:)      => NULL()  
  real(dp), pointer :: u_een_bee_der2_bywlk           (:,:,:,:,:)      => NULL()  
  real(dp), pointer :: u_een_ben_der_bywlk            (:,:,:,:,:)      => NULL()  
  real(dp), pointer :: u_een_ben_der2_bywlk           (:,:,:,:,:)      => NULL()  
  real(dp), pointer :: u_een_bee_ben_der2_bywlk      (:,:,:,:,:)      => NULL()  
  real(dp), pointer :: u_een_a_ben_der2_bywlk         (:,:,:,:,:,:)    => NULL() 
  real(dp), pointer :: u_een_a_bee_der2_bywlk         (:,:,:,:,:,:)    => NULL() 

  real(dp), pointer :: u_een_grad_bywlk                  (:,:,:,:,:,:)    => NULL()
  real(dp), pointer :: u_een_grad_a_der_bywlk          (:,:,:,:,:,:,:)  => NULL()
  real(dp), pointer :: u_een_grad_ben_der_bywlk       (:,:,:,:,:,:)    => NULL()  
  real(dp), pointer :: u_een_grad_bee_der_bywlk       (:,:,:,:,:,:)    => NULL()  
  real(dp), pointer :: u_een_grad_ben_der2_bywlk      (:,:,:,:,:,:)    => NULL()  
  real(dp), pointer :: u_een_grad_bee_der2_bywlk      (:,:,:,:,:,:)    => NULL()  
  real(dp), pointer :: u_een_grad_bee_ben_der2_bywlk (:,:,:,:,:,:)    => NULL() 
  real(dp), pointer :: u_een_grad_a_ben_der2_bywlk    (:,:,:,:,:,:,:)  => NULL() 
  real(dp), pointer :: u_een_grad_a_bee_der2_bywlk    (:,:,:,:,:,:,:)  => NULL() 

  real(dp), pointer :: u_een_lapl_bywlk                  (:,:,:,:,:)      => NULL()
  real(dp), pointer :: u_een_lapl_a_der_bywlk          (:,:,:,:,:,:)    => NULL()
  real(dp), pointer :: u_een_lapl_ben_der_bywlk       (:,:,:,:,:)      => NULL()
  real(dp), pointer :: u_een_lapl_ben_der2_bywlk      (:,:,:,:,:)      => NULL()
  real(dp), pointer :: u_een_lapl_bee_der_bywlk       (:,:,:,:,:)      => NULL()
  real(dp), pointer :: u_een_lapl_bee_der2_bywlk      (:,:,:,:,:)      => NULL()
  real(dp), pointer :: u_een_lapl_a_ben_der2_bywlk    (:,:,:,:,:,:)    => NULL() 
  real(dp), pointer :: u_een_lapl_a_bee_der2_bywlk    (:,:,:,:,:,:)    => NULL() 
  real(dp), pointer :: u_een_lapl_bee_ben_der2_bywlk (:,:,:,:,:)      => NULL() 

  contains

! ===================================================================================
  subroutine u_een_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_bywlk_bld
!
! Creates        : u_een_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : conf_jastr_a_een (conf,lik, atom)    
!
! Descriptions   : u_een_bywlk = a_een1*ria^2 rja^2 + a_een2*rij^2 (ria^2 + rja^2)
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! Revision       : R. Assaraf 04 Apr 2002 conf_jastr_a_een
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_2
  real(dp)                      :: r_ja_bar 
  real(dp)                      :: r_ja_bar_2
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_2
  real(dp)                      :: a_een_1   
  real(dp)                      :: a_een_2 
  real(dp)                      :: u_een

! header:

  if (header_exe) then

    l_here = "u_een_bywlk_bld"

    call object_create ('u_een_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	
!  include 'chksiz_configurations%conf_jastr%a_een.inc'
  include 'chksiz_ea_r_bar_bywlk.inc'
  include 'chksiz_ee_r_bar_bywlk.inc'
! end requirements	

! dimensions:
  electrons_nb      = size (ee_r_bar_bywlk, 1)
  atoms_nb          = size (ea_r_bar_bywlk, 2)
  configurations_nb = size (ee_r_bar_bywlk, 3)
  walkers_nb        = size (ee_r_bar_bywlk, 4)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_bywlk', u_een_bywlk, & 
         electrons_nb, electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb
        
        do atom_i = 1, atoms_nb

           a_een_1  = conf_jastr_a_een (conf_i, 1, atom_i)
           a_een_2  = conf_jastr_a_een (conf_i, 2, atom_i)

           do elec_j = 1, electrons_nb

              r_ja_bar = ea_r_bar_bywlk (elec_j, atom_i, conf_i, walk_i) 
              r_ja_bar_2 = r_ja_bar * r_ja_bar

              ! Improve store squares in temporary arrays

              do elec_i = 1, electrons_nb

                 r_ia_bar   = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
                 r_ia_bar_2 = r_ia_bar * r_ia_bar

                 r_ij_bar   = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)
                 r_ij_bar_2 = r_ij_bar * r_ij_bar

                 ! verify formula
                 u_een      = a_een_1 * r_ia_bar_2 * r_ja_bar_2                 &
                      + a_een_2 * r_ij_bar_2 * ( r_ia_bar_2 + r_ja_bar_2)

                 u_een_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = u_een 

              end do ! do elec_i

           end do ! do elec_j

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_u_een_bywlk.inc'
!   include 'ensnb_u_een_bywlk.inc'

  call exit (l_here)

  end subroutine u_een_bywlk_bld

! ===================================================================================
  subroutine u_een_a_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_a_der_bywlk_bld
!
! Creates        : u_een_a_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Descriptions   : 
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb

  real(dp)                      :: u_ee_ij
  real(dp)                      :: u_en_ja
  real(dp)                      :: u_en_ia
  real(dp)                      :: term1 
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "u_een_a_der_bywlk_bld"

    call object_create ('u_een_a_der_bywlk')

    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (u_ee_a_der_bywlk, 1)
  aeen_jastrow_nb   = size (conf_jastr_a_een,   2)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)

! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_a_der_bywlk', u_een_a_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           u_en_ja = u_en_a_der_bywlk(elec_j, atom_i, 2, conf_i, walk_i)
           
           do elec_i = 1, electrons_nb

              u_en_ia = u_en_a_der_bywlk(elec_i, atom_i, 2, conf_i, walk_i)

              u_ee_ij = u_ee_a_der_bywlk(elec_i, elec_j, 2, conf_i, walk_i)

              term1 = u_en_ia * u_en_ja
              
              term2 = u_ee_ij * ( u_en_ia + u_en_ja )

              u_een_a_der_bywlk (elec_i, elec_j, atom_i, 1, conf_i, walk_i) = term1

              u_een_a_der_bywlk (elec_i, elec_j, atom_i, 2, conf_i, walk_i) = term2

           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i
      
    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_a_der_bywlk_bld

! ===================================================================================
  subroutine u_een_bee_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_bee_der_bywlk_bld
!
! Creates        : u_een_bee_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Descriptions   : 
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: u_en_ia_2
  real(dp)                      :: u_en_ja_2
  real(dp)                      :: u_ee_ij_1
  real(dp)                      :: u_ee_beta_deriv
  real(dp)                      :: term
  real(dp)                      :: a_een_2

! header:

  if (header_exe) then

    l_here = "u_een_bee_der_bywlk_bld"

    call object_create ('u_een_bee_der_bywlk')

    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (u_ee_a_der_bywlk, 1)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)

! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_bee_der_bywlk', u_een_bee_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_bee_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           u_en_ja_2 = u_en_a_der_bywlk(elec_j, atom_i, 2, conf_i, walk_i)

           do elec_i = 1, electrons_nb

              a_een_2         = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_ia_2       = u_en_a_der_bywlk(elec_i, atom_i, 2, conf_i, walk_i)

              u_ee_ij_1       = u_ee_a_der_bywlk(elec_i, elec_j, 1, conf_i, walk_i)

              u_ee_beta_deriv = ee_r_bar_beta_der_bywlk(elec_i, elec_j, 1, conf_i, walk_i)
              
              term = 2.d0 * a_een_2 * u_ee_ij_1 * u_ee_beta_deriv * ( u_en_ia_2 + u_en_ja_2  )

              u_een_bee_der_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = term

           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_bee_der_bywlk_bld

! ===================================================================================
  subroutine u_een_ben_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_ben_der_bywlk_bld
!
! Creates        : u_een_ben_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : 
! Needs          : 
! Needs          : 
!
! Descriptions   : 
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2

  real(dp)                      :: u_en_b_der_ia
  real(dp)                      :: u_en_b_der_ja

  real(dp)                      :: u_en_ia
  real(dp)                      :: u_en_ja
  real(dp)                      :: u_ee_ij

! header:

  if (header_exe) then

    l_here = "u_een_ben_der_bywlk_bld"

    call object_create ('u_een_ben_der_bywlk')

    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_b_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_ee_a_der_bywlk, 1)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_ben_der_bywlk', u_een_ben_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_ben_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1            = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2            = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_b_der_ia    = ea_r_bar_beta_der_bywlk(elec_i, atom_i, 2, conf_i, walk_i)

              u_en_b_der_ja    = ea_r_bar_beta_der_bywlk(elec_j, atom_i, 2, conf_i, walk_i)

              u_en_ia            = u_en_a_der_bywlk(elec_i, atom_i,2,conf_i,walk_i)

              u_en_ja            = u_en_a_der_bywlk(elec_j, atom_i, 2, conf_i, walk_i)

              u_ee_ij            = u_ee_a_der_bywlk(elec_i, elec_j,2,conf_i,walk_i)

              term1              = a_een_1 * u_en_ja * u_en_b_der_ia

              term2              = a_een_1 * u_en_ia * u_en_b_der_ja

              term3              = a_een_2 * u_ee_ij * u_en_b_der_ia

              term4              = a_een_2 * u_ee_ij * u_en_b_der_ja

              u_een_ben_der_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
              term1 + term2 + term3 + term4
                   
           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_ben_der_bywlk_bld

! ===================================================================================
  subroutine u_een_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_ben_der2_bywlk_bld
!
! Creates        : u_een_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                  (conf, 1,    atom           )
! Needs          : ea_r_bar_beta_der2_bywlk (elec, atom, pow, conf, walk)
! Needs          : ea_r_bar_beta_der_bywlk  (elec, atom, pow, conf, walk)
! Needs          : u_en_a_der_bywlk                (elec, atom, pow, conf, walk)
! Needs          : u_ee_a_der_bywlk                (elec, elec, pow, conf, walk)
!
! Descriptions   : u_een_ben_deriv2 = 
! Descriptions   : a_een_1 * u_en_b_der_ia * u_en_b_der_ja * 2.d0 +
! Descriptions   : a_een_1 * u_en_ja         * u_en_b_der2_ia       +
! Descriptions   : a_een_1 * u_en_ia         * u_en_b_der2_ja       +
! Descriptions   : a_een_2 * u_ee_ij         * u_en_b_der2_ia       +
! Descriptions   : a_een_2 * u_ee_ij         * u_en_b_der2_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2

  real(dp)                      :: u_en_b_der_ia
  real(dp)                      :: u_en_b_der2_ia
  real(dp)                      :: u_en_b_der_ja
  real(dp)                      :: u_en_b_der2_ja

  real(dp)                      :: u_en_ia
  real(dp)                      :: u_en_ja
  real(dp)                      :: u_ee_ij

  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4
  real(dp)                      :: term5

! header:

  if (header_exe) then

    l_here = "u_een_ben_der2_bywlk_bld"

    call object_create ('u_een_ben_der2_bywlk')

    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_b_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der2_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_ee_a_der_bywlk, 1)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_ben_der2_bywlk', u_een_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1            = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2            = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_b_der2_ia   = ea_r_bar_beta_der2_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der_ia    = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der2_ja   = ea_r_bar_beta_der2_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_b_der_ja    = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_ia            = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja            = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_ee_ij            = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              term1              = a_een_1 * u_en_ja * u_en_b_der2_ia

              term2              = a_een_1 * u_en_b_der_ia * u_en_b_der_ja * 2.d0

              term3              = a_een_1 * u_en_ia*u_en_b_der2_ja

              term4              = a_een_2 * u_ee_ij*u_en_b_der2_ia

              term5              = a_een_2 * u_ee_ij*u_en_b_der2_ja

              u_een_ben_der2_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
              term1 + term2 + term3 + term4 + term5 
                   

           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_bee_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_bee_der2_bywlk_bld
!
! Creates        : u_een_bee_der2_bywlk (elec, elec, atom, conf, walk)
! 
! Needs          : conf_jastr_a_een                  (conf, 1,    atom           )
! Needs          : u_en_a_der_bywlk                (elec, atom, pow, conf, walk)
! Needs          : ee_r_bar_beta_der2_bywlk (elec, elec, pow, conf, walk)
!             
! Descriptions   : u_een_bee_deriv2 =  
! Descriptions   : a_een_2 * u_ee_b_der2_ij * u_en_ia
! Descriptions   : a_een_2 * u_ee_b_der2_ij * u_en_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_ee_b_der2_ij
  real(dp)                      :: u_en_ia
  real(dp)                      :: u_en_ja
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "u_een_bee_der2_bywlk_bld"

    call object_create ('u_een_bee_der2_bywlk')

    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_b_der_bywlk')
    call object_needed ('ee_r_bar_beta_der2_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_ee_a_der_bywlk, 1)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_bee_der2_bywlk', u_een_bee_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_bee_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1            = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2            = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_ia            = u_en_a_der_bywlk(elec_i, atom_i, 2, conf_i, walk_i)

              u_en_ja            = u_en_a_der_bywlk(elec_j, atom_i, 2, conf_i, walk_i)

              u_ee_b_der2_ij   = ee_r_bar_beta_der2_bywlk(elec_i, elec_j, 2, conf_i, walk_i)
              
              term1              = a_een_2 * u_ee_b_der2_ij * u_en_ia

              term2              = a_een_2 * u_ee_b_der2_ij * u_en_ja

              u_een_bee_der2_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
              term1 + term2
                   
           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_bee_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_bee_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_bee_ben_der2_bywlk_bld
!
! Creates        : u_een_bee_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                 (conf, 1,    atom           )
! Needs          : ea_r_bar_beta_der_bywlk (elec, atom, pow, conf, walk)
! Needs          : ee_r_bar_beta_der_bywlk (elec, elec, pow, conf, walk)
!
! Descriptions   : u_een_bee_ben_deriv2 = 
! Descriptions   : a_een_2 * u_ee_b_der_ij * u_en_b_der_ia +
! Descriptions   : a_een_2 * u_ee_b_der_ij * u_en_b_der_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_en_b_der_ja
  real(dp)                      :: u_en_b_der_ia
  real(dp)                      :: u_ee_b_der_ij
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "u_een_bee_ben_der2_bywlk_bld"

    call object_create ('u_een_bee_ben_der2_bywlk')

    call object_needed ('ee_r_bar_beta_der2_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_beta_der_bywlk, 1)
  atoms_nb          = size (ea_r_bar_beta_der_bywlk, 2)
  configurations_nb = size (ea_r_bar_beta_der_bywlk, 4)
  walkers_nb        = size (ea_r_bar_beta_der_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_bee_ben_der2_bywlk', u_een_bee_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_bee_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1         = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2         = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_b_der_ja = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_b_der_ia = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_ee_b_der_ij = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              term1 = a_een_2*u_ee_b_der_ij*u_en_b_der_ia

              term2 = a_een_2*u_ee_b_der_ij*u_en_b_der_ja

              u_een_bee_ben_der2_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
              term1 + term2
                   
           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_bee_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_a_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_a_ben_der2_bywlk_bld
!
! Creates        : u_een_a_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : u_en_a_der_bywlk               (elec, atom, pow, conf, walk)
! Needs          : ea_r_bar_beta_der_bywlk (elec, atom, pow, conf, walk)
! Needs          : u_ee_a_der_bywlk               (elec, elec, pow, conf, walk)
!
! Descriptions   : u_een_a_ben_der2_bywlk (elec, elec, atom, 1, conf, walk) =
! Descriptions   : u_en_b_der_ia * u_en_ja +
! Descriptions   : u_en_b_der_ja * u_en_ia
!
! Descriptions   : u_een_a_ben_der2_bywlk (elec, elec, atom, 2, conf, walk) =
! Descriptions   : u_en_b_der_ia * u_ee_ij +
! Descriptions   : u_en_b_der_ja * u_ee_ij
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb 

  real(dp)                      :: u_en_b_der_ia 
  real(dp)                      :: u_en_b_der_ja 
  real(dp)                      :: u_en_ia            
  real(dp)                      :: u_en_ja
  real(dp)                      :: u_ee_ij
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4

! header:

  if (header_exe) then

    l_here = "u_een_a_ben_der2_bywlk_bld"

    call object_create ('u_een_a_ben_der2_bywlk')

    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')
    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (u_en_a_der_bywlk, 1)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,   2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)

! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_a_ben_der2_bywlk', u_een_a_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_a_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              u_en_ia         = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja         = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_b_der_ia = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der_ja = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_ee_ij         = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              term1           = u_en_b_der_ia * u_en_ja

              term2           = u_en_b_der_ja * u_en_ia

              term3           = u_en_b_der_ia * u_ee_ij

              term4           = u_en_b_der_ja * u_ee_ij

              u_een_a_ben_der2_bywlk (elec_i, elec_j, atom_i, 1, conf_i, walk_i) = &
                   term1 + term2

              u_een_a_ben_der2_bywlk (elec_i, elec_j, atom_i, 2, conf_i, walk_i) = &
                   term3 + term4

           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_a_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_a_bee_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_a_bee_der2_bywlk_bld
!
! Creates        : u_een_a_bee_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ee_r_bar_beta_der_bywlk (elec, elec, pow, conf, walk)
! Needs          : u_en_a_der_bywlk               (elec, atom, pow, conf, walk)
!
! Descriptions   : u_een_a_bee_deriv2 = 
! Descriptions   : u_ee_b_der_ij * u_en_ia
! Descriptions   : u_ee_b_der_ij * u_en_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb 
         
  real(dp)                      :: u_ee_b_der_ij  
  real(dp)                      :: u_en_ia            
  real(dp)                      :: u_en_ja
  real(dp)                      :: term1
  real(dp)                      :: term2

! header:

  if (header_exe) then

    l_here = "u_een_a_bee_der2_bywlk_bld"

    call object_create ('u_een_a_bee_der2_bywlk')

    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')
    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (u_en_a_der_bywlk, 1)
  atoms_nb          = size (u_en_a_der_bywlk, 2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,   2)
  configurations_nb = size (u_en_a_der_bywlk, 4)
  walkers_nb        = size (u_en_a_der_bywlk, 5)

! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_a_bee_der2_bywlk', u_een_a_bee_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_a_bee_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              u_ee_b_der_ij = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_ia         = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja         = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              term1           = u_ee_b_der_ij * u_en_ia

              term2           = u_ee_b_der_ij * u_en_ja

              u_een_a_bee_der2_bywlk (elec_i, elec_j, atom_i, 1, conf_i, walk_i) = &
                   0.d0

              u_een_a_bee_der2_bywlk (elec_i, elec_j, atom_i, 2, conf_i, walk_i) = &
                   term1 + term2

           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_a_bee_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_bywlk_bld
!
! Creates        : u_een_grad_bywlk (elec, elec, atom, dim, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : ea_r_bar_grad_bywlk (elec, elec, dim, conf, walk)
! Needs          : ee_r_bar_grad_bywlk (elec, elec, dim, conf, walk)
!
! Needs          : conf_jastr%a_een (conf,lik, atom)      
!
! Descriptions   : gardient over elec_i coordinates only
! Descriptions   : u_een_grad_bywlk 
! Descriptions   :      = 2 * a_een1 *ria rja^2 * grad_i ria
! Descriptions   :      + 2 * a_een2 *rij (ria^2 + rja^2) * grad_i rij
! Descriptions   :      + 2 * a_een2 *rij^2 *ria * grad_i ria
! Descriptions   : u_een_grad_bywlk 
! Descriptions   :      = 2 (( a_een1 * rja^2 + a_een2 * rij^2 ) * ria * grad_i ria
! Descriptions   :      + a_een2 * (ria^2 + rja^2) * rij * grad_i rij )
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_2
  real(dp)                      :: r_ja_bar
  real(dp)                      :: r_ja_bar_2
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_2
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp)                      :: r_ij_bar_grad(3)
  real(dp)                      :: a_een_1   
  real(dp)                      :: a_een_2 
  real(dp)                      :: d_u_een(3)

! header:

  if (header_exe) then

    l_here = "u_een_grad_bywlk_bld"

    call object_create ('u_een_grad_bywlk')

    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! requirements:	
!  include 'chksiz_configurations%conf_jastr%a_een.inc'
  include 'chksiz_ea_r_bar_bywlk.inc'
  include 'chksiz_ee_r_bar_bywlk.inc'
  include 'chksiz_ee_r_bar_grad_bywlk.inc'
  include 'chksiz_ea_r_bar_grad_bywlk.inc'

! end requirements
! dimensions:

  electrons_nb      = size(ea_r_bar_grad_bywlk, 1)
  atoms_nb          = size(ea_r_bar_grad_bywlk, 2)
  dimensions_nb     = size(ea_r_bar_grad_bywlk, 3)
  configurations_nb = size(ea_r_bar_grad_bywlk, 4)
  walkers_nb        = size(ea_r_bar_grad_bywlk, 5)

! end dimensions

  if(debug) then
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
    write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
    write (logf,*)trim(l_here),'-d: dimensions_nb     = ',dimensions_nb
    write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! allocations
 
  call alloc ('u_een_grad_bywlk', u_een_grad_bywlk, electrons_nb, electrons_nb, atoms_nb,  &
       dimensions_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           a_een_1  = conf_jastr_a_een (conf_i,1, atom_i)
           a_een_2  = conf_jastr_a_een (conf_i,2, atom_i)

           do elec_j = 1, electrons_nb

              r_ja_bar = ea_r_bar_bywlk (elec_j, atom_i, conf_i, walk_i) 
              r_ja_bar_2 = r_ja_bar * r_ja_bar

              ! Improve store squares in temporary arrays

              do elec_i = 1, electrons_nb

                 r_ia_bar   = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)
                 r_ia_bar_2 = r_ia_bar * r_ia_bar
                 
                 r_ia_bar_grad &
                      = ea_r_bar_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)
                 
                 r_ij_bar   = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)
                 r_ij_bar_2 = r_ij_bar * r_ij_bar

                 r_ij_bar_grad &
                      = ee_r_bar_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

                 d_u_een(:) = two * (                                  &
                      (a_een_1 * r_ja_bar_2 + a_een_2 * r_ij_bar_2 )   &
                      * r_ia_bar * r_ia_bar_grad (:)                   & 
                      + a_een_2 * (r_ia_bar_2 + r_ja_bar_2 )           &
                      * r_ij_bar * r_ij_bar_grad (:)  )

                 u_een_grad_bywlk (elec_i, elec_j, atom_i, :, conf_i, walk_i) = d_u_een (:)

              end do ! do elec_i

           end do ! do elec_j

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_u_een_grad_bywlk.inc'
!   include 'ensnb_u_een_grad_bywlk.inc'

  call exit (l_here)

  end subroutine u_een_grad_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_ben_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_ben_der_bywlk_bld
!
! Creates        : u_een_grad_ben_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                      (conf, 1,    atom                )
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk     )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk     )
! Needs          : u_ee_a_der_bywlk                    (elec, elec, pow, conf, walk     )
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow, conf, walk)
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow, conf, walk)
! Needs          : u_ee_grad_a_der_bywlk               (elec, elec, dim, pow, conf, walk)
!
! Descriptions   : u_een_grad_ben_deriv = 
! Descriptions   : a_een_2 * u_ee_ij              * u_en_grad_b_der_ia +
! Descriptions   : a_een_1 * u_en_grad_ia         * u_en_b_der_ja      +
! Descriptions   : a_een_2 * u_ee_grad_ij         * u_en_b_der_ia      +
! Descriptions   : a_een_2 * u_ee_grad_ij         * u_en_b_der_ja      +
! Descriptions   : a_een_1 * u_en_grad_b_der_ia * u_en_ja      
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_en_grad_b_der_ia 
  real(dp)                      :: u_en_b_der_ja      
  real(dp)                      :: u_en_b_der_ia      
  real(dp)                      :: u_en_ja              
  real(dp)                      :: u_ee_ij              
  real(dp)                      :: u_en_grad_ia         
  real(dp)                      :: u_ee_grad_ij         
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4
  real(dp)                      :: term5  

! header:

  if (header_exe) then

    l_here = "u_een_grad_ben_der_bywlk_bld"

    call object_create ('u_een_grad_ben_der_bywlk')

    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_grad_bywlk, 1)
  atoms_nb          = size (ea_r_bar_grad_bywlk, 2)
  dimensions_nb     = size (ea_r_bar_grad_bywlk, 3)
  configurations_nb = size (ea_r_bar_grad_bywlk, 4)
  walkers_nb        = size (ea_r_bar_grad_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_ben_der_bywlk', u_een_grad_ben_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_ben_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 a_een_1              = conf_jastr_a_een (conf_i, 1, atom_i)
                 
                 a_een_2              = conf_jastr_a_een (conf_i, 2, atom_i)
                 
                 u_en_grad_b_der_ia = ea_r_bar_grad_beta_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_b_der_ja      = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 u_en_b_der_ia      = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

                 u_en_ja              = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 u_ee_ij              = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 u_en_grad_ia         = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_ee_grad_ij         = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 term1 = a_een_1 * u_en_grad_b_der_ia * u_en_ja
                 
                 term2 = a_een_1 * u_en_grad_ia * u_en_b_der_ja

                 term3 = a_een_2 * u_ee_grad_ij * u_en_b_der_ia

                 term4 = a_een_2 * u_ee_grad_ij * u_en_b_der_ja

                 term5 = a_een_2 * u_ee_ij * u_en_grad_b_der_ia

                 u_een_grad_ben_der_bywlk (elec_i, elec_j, atom_i, dim_i, conf_i, walk_i) = &
                      term1 + term2 + term3 + term4 + term5
                 
              enddo ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_ben_der_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_ben_der2_bywlk_bld
!
! Creates        : u_een_grad_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                       (conf, 2,    atom)
! Needs          : u_en_a_der_bywlk                     (elec, atom, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk                     (elec, elec, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der_bywlk       (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der2_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_ee_grad_a_der_bywlk                (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk                (elec, atom, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der_bywlk  (elec, atom, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der2_bywlk (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_grad_ben_deriv2 =  
! Descriptions   : a_een_1 * u_en_grad_b_der2_ia * u_en_ja                +
! Descriptions   : a_een_1 * u_en_grad_b_der_ia  * u_en_b_der_ja * 2    +
! Descriptions   : a_een_1 * u_en_grad_ia          * u_en_b_der2_ja       +
! Descriptions   : a_een_2 * u_ee_grad_ij          * u_en_b_der2_ia       +
! Descriptions   : a_een_2 * u_ee_grad_ij          * u_en_b_der2_ja       +
! Descriptions   : a_een_2 * u_ee_ij               * u_en_grad_b_der2_ia  
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_en_ja
  real(dp)                      :: u_ee_ij 
  real(dp)                      :: u_ee_grad_ij         
  real(dp)                      :: u_en_grad_ia
  real(dp)                      :: u_en_b_der_ja 
  real(dp)                      :: u_en_b_der2_ia     
  real(dp)                      :: u_en_b_der2_ja     
  real(dp)                      :: u_en_grad_b_der2_ia
  real(dp)                      :: u_en_grad_b_der_ia      
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4
  real(dp)                      :: term5  
  real(dp)                      :: term6

! header:

  if (header_exe) then

    l_here = "u_een_grad_ben_der2_bywlk_bld"

    call object_create ('u_een_grad_ben_der2_bywlk')

    call object_needed ('ea_r_bar_grad_beta_der2_bywlk')
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der2_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_grad_bywlk, 1)
  atoms_nb          = size (ea_r_bar_grad_bywlk, 2)
  dimensions_nb     = size (ea_r_bar_grad_bywlk, 3)
  configurations_nb = size (ea_r_bar_grad_bywlk, 4)
  walkers_nb        = size (ea_r_bar_grad_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_ben_der2_bywlk', u_een_grad_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
                 
                 a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

                 u_en_ja               = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)
                 u_ee_ij               = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 u_ee_grad_ij          = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)
                 u_en_grad_ia          = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_b_der_ja       = ea_r_bar_beta_der_bywlk( elec_j,atom_i,2,conf_i,walk_i)
                 u_en_b_der2_ia      = ea_r_bar_beta_der2_bywlk(elec_i,atom_i,2,conf_i,walk_i)
                 u_en_b_der2_ja      = ea_r_bar_beta_der2_bywlk(elec_j,atom_i,2,conf_i,walk_i)                 

                 u_en_grad_b_der_ia  = ea_r_bar_grad_beta_der_bywlk( elec_i,atom_i,dim_i,2,conf_i,walk_i)
                 u_en_grad_b_der2_ia = ea_r_bar_grad_beta_der2_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term1 = a_een_1 * u_en_grad_b_der2_ia * u_en_ja
                 term2 = a_een_1 * u_en_grad_b_der_ia  * u_en_b_der_ja * 2.d0
                 term3 = a_een_1 * u_en_grad_ia          * u_en_b_der2_ja

                 term4 = a_een_2 * u_ee_grad_ij          * u_en_b_der2_ia
                 term5 = a_een_2 * u_ee_grad_ij          * u_en_b_der2_ja
                 term6 = a_een_2 * u_ee_ij               * u_en_grad_b_der2_ia

                 u_een_grad_ben_der2_bywlk (elec_i, elec_j, atom_i, dim_i, conf_i, walk_i) = &
                      term1 + term2 + term3 + term4 + term5 + term6
                 
              end do ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_bee_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_bee_der_bywlk_bld
!
! Creates        : u_een_grad_bee_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                      (conf, 1,    atom                 )
! Needs          : ee_r_bar_beta_der_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk      )
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow,  conf, walk)
!
! Discriptions   : u_een_grad_bee_deriv = 
! Discriptions   : a_een_2 * u_ee_b_der_ij      * u_en_grad_ia +
! Discriptions   : a_een_2 * u_ee_grad_b_der_ij * u_en_ia      +
! Discriptions   : a_een_2 * u_ee_grad_b_der_ij * u_en_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_ee_grad_b_der_ij 
  real(dp)                      :: u_ee_b_der_ij      
  real(dp)                      :: u_en_grad_ia         
  real(dp)                      :: u_en_ia              
  real(dp)                      :: u_en_ja              
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3

! header:

  if (header_exe) then

    l_here = "u_een_grad_bee_der_bywlk_bld"

    call object_create ('u_een_grad_bee_der_bywlk')

    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  atoms_nb          = size (u_en_grad_a_der_bywlk,               2)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_bee_der_bywlk', u_een_grad_bee_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_bee_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
                 
                 a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

                 u_ee_grad_b_der_ij = ee_r_bar_grad_beta_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_ee_b_der_ij      = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 u_en_grad_ia         = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_ia              = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

                 u_en_ja              = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 term1 = a_een_2*u_ee_grad_b_der_ij*u_en_ia

                 term2 = a_een_2*u_ee_grad_b_der_ij*u_en_ja

                 term3 = a_een_2*u_ee_b_der_ij*u_en_grad_ia

                 u_een_grad_bee_der_bywlk (elec_i, elec_j, atom_i, dim_i, conf_i, walk_i) = &
                      term1 + term2 + term3 
                 
              end do ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i
     
  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_bee_der_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_bee_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_bee_der2_bywlk_bld
!
! Creates        : u_een_grad_bee_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                       (conf, pow,  atom                 )
! Needs          : u_en_a_der_bywlk                     (elec, atom, pow, conf, walk      )
! Needs          : ee_r_bar_beta_der2_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : u_en_grad_a_der_bywlk                (elec, atom, dim, pow,  conf, walk)
! Needs          : ee_r_bar_grad_beta_der2_bywlk (elec, elec, dim, pow,  conf, walk)
!
! Descriptions   : u_een_grad_bee_deriv2 = 
! Descriptions   : a_een_2 * u_ee_b_der2_ij      * u_en_grad_ia +
! Descriptions   : a_een_2 * u_ee_grad_b_der2_ij * u_en_ia      +
! Descriptions   : a_een_2 * u_ee_grad_b_der2_ij * u_en_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_ee_grad_b_der2_ij 
  real(dp)                      :: u_ee_b_der2_ij      
  real(dp)                      :: u_en_grad_ia          
  real(dp)                      :: u_en_ja               
  real(dp)                      :: u_en_ia               
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3

! header:

  if (header_exe) then

    l_here = "u_een_grad_bee_der2_bywlk_bld"

    call object_create ('u_een_grad_bee_der2_bywlk')

    call object_needed ('ee_r_bar_grad_beta_der2_bywlk')
    call object_needed ('ee_r_bar_beta_der2_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  atoms_nb          = size (u_en_grad_a_der_bywlk,               2)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_bee_der2_bywlk', u_een_grad_bee_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_bee_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
                 
                 a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

                 u_ee_grad_b_der2_ij = ee_r_bar_grad_beta_der2_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_ee_b_der2_ij      = ee_r_bar_beta_der2_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 u_en_grad_ia          = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_ja               = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 u_en_ia               = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

                 term1 = a_een_2 * u_ee_grad_b_der2_ij * u_en_ia
                 
                 term2 = a_een_2 * u_ee_grad_b_der2_ij * u_en_ja

                 term3 = a_een_2 * u_ee_b_der2_ij * u_en_grad_ia

                 u_een_grad_bee_der2_bywlk (elec_i, elec_j, atom_i, dim_i, conf_i, walk_i) = &
                      term1 + term2 + term3
                 
              end do ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_bee_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_bee_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_bee_ben_der2_bywlk_bld
!
! Creates        : u_een_grad_bee_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                      (conf, 1,    atom                 )
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : ee_r_bar_beta_der_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_grad_bee_ben_deriv2 = 
! Descriptions   : a_een_2 * u_ee_b_der_ij      * u_en_grad_b_der_ia +
! Descriptions   : a_een_2 * u_ee_grad_b_der_ij * u_en_b_der_ia      + 
! Descriptions   : a_een_2 * u_ee_grad_b_der_ij * u_en_b_der_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1
  real(dp)                      :: a_een_2
  real(dp)                      :: u_ee_grad_b_der_ij
  real(dp)                      :: u_en_grad_b_der_ia
  real(dp)                      :: u_en_b_der_ia
  real(dp)                      :: u_en_b_der_ja
  real(dp)                      :: u_ee_b_der_ij
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3

! header:

  if (header_exe) then

    l_here = "u_een_grad_bee_ben_der2_bywlk_bld"

    call object_create ('u_een_grad_bee_ben_der2_bywlk')

    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_grad_beta_der_bywlk, 1)
  atoms_nb          = size (ea_r_bar_grad_beta_der_bywlk, 2)
  dimensions_nb     = size (ea_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ea_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ea_r_bar_grad_beta_der_bywlk, 6)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_bee_ben_der2_bywlk', u_een_grad_bee_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_bee_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
                 
                 a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

                 u_ee_grad_b_der_ij = ee_r_bar_grad_beta_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_en_grad_b_der_ia = ea_r_bar_grad_beta_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_b_der_ia = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

                 u_en_b_der_ja = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 u_ee_b_der_ij = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 term1 = a_een_2*u_ee_grad_b_der_ij*u_en_b_der_ia

                 term2 = a_een_2*u_ee_grad_b_der_ij*u_en_b_der_ja

                 term3 = a_een_2*u_ee_b_der_ij*u_en_grad_b_der_ia

                 u_een_grad_bee_ben_der2_bywlk (elec_i, elec_j, atom_i, dim_i, conf_i, walk_i) = &
                      term1 + term2 + term3 
                 
              end do ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j
        
     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_bee_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_a_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_a_der_bywlk_bld
!
! Creates        : u_een_grad_a_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : u_en_grad_a_der_bywlk (elec, atom, dim, pow,  conf, walk)
! Needs          : u_ee_grad_a_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_a_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk      (elec, elec, pow, conf, walk      )
!
! Descriptions   : u_een_grad_a_der_bywlk (elec, elec, atom, dim, 1, conf, walk) =
! Descriptions   : u_en_grad_ia * u_en_ja
!
! Descriptions   : u_een_grad_a_der_bywlk (elec, elec, atom, dim, 2, conf, walk) =
! Descriptions   : u_ee_ij      * u_en_grad_ia +
! Descriptions   : u_ee_grad_ij * u_en_ia      + 
! Descriptions   : u_ee_grad_ij * u_en_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: aeen_jastrow_nb   

  real(dp)                      :: u_en_grad_ia
  real(dp)                      :: u_ee_grad_ij
  real(dp)                      :: u_ee_ij
  real(dp)                      :: u_en_ia
  real(dp)                      :: u_en_ja
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4

! header:

  if (header_exe) then

    l_here = "u_een_grad_a_der_bywlk_bld"

    call object_create ('u_een_grad_a_der_bywlk')

    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:

  electrons_nb      = size (u_en_grad_a_der_bywlk, 1)
  atoms_nb          = size (u_en_grad_a_der_bywlk, 2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,        2)
  dimensions_nb     = size (u_en_grad_a_der_bywlk, 3)
  configurations_nb = size (u_en_grad_a_der_bywlk, 5)
  walkers_nb        = size (u_en_grad_a_der_bywlk, 6)

! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_a_der_bywlk', u_een_grad_a_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb
       
      do atom_i = 1, atoms_nb
         
        do elec_i = 1, electrons_nb

           do elec_j = 1, electrons_nb

              do dim_i = 1, dimensions_nb
                 
                 u_en_grad_ia = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)
                 
                 u_ee_grad_ij = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)
                 
                 u_en_ja = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)
                 
                 u_en_ia = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)
                 
                 u_ee_ij = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 term1 = u_en_grad_ia*u_en_ja

                 term2 = u_ee_grad_ij*u_en_ia

                 term3 = u_ee_grad_ij*u_en_ja

                 term4 = u_ee_ij*u_en_grad_ia

                 u_een_grad_a_der_bywlk (elec_i, elec_j, atom_i, dim_i, 1, conf_i, walk_i) = term1

                 u_een_grad_a_der_bywlk (elec_i, elec_j, atom_i, dim_i, 2, conf_i, walk_i) = &
                      term2 + term3 + term4
                 
              end do ! do dim_i
                   
           end do ! do elec_j
           
        end do ! do elec_i

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_a_der_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_a_bee_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_a_bee_der2_bywlk_bld
!
! Creates        : u_een_grad_a_bee_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow,  conf, walk)
! Needs          : ee_r_bar_beta_der_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk      )
!
! Descriptions   : u_een_grad_a_bee_deriv2 =  
! Descriptions   : u_ee_b_der_ij      * u_en_grad_ia +
! Descriptions   : u_ee_grad_b_der_ij * u_en_ia      +
! Descriptions   : u_ee_grad_b_der_ij * u_en_ja 

!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  :: aeen_jastrow_nb 

  real(dp)                      :: u_ee_grad_b_der_ij 
  real(dp)                      :: u_ee_b_der_ij      
  real(dp)                      :: u_en_grad_ia         
  real(dp)                      :: u_en_ia              
  real(dp)                      :: u_en_ja              
  
  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3

! header:

  if (header_exe) then

    l_here = "u_een_grad_a_bee_der2_bywlk_bld"

    call object_create ('u_een_grad_a_bee_der2_bywlk')

    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  atoms_nb          = size (u_en_grad_a_der_bywlk,               2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,                      2)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_a_bee_der2_bywlk', u_een_grad_a_bee_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb,  aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_a_bee_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 u_ee_grad_b_der_ij = ee_r_bar_grad_beta_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_ee_b_der_ij      = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 u_en_grad_ia         = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_ia              = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

                 u_en_ja              = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 term1 = u_ee_grad_b_der_ij*u_en_ia

                 term2 = u_ee_grad_b_der_ij*u_en_ja

                 term3 = u_ee_b_der_ij*u_en_grad_ia

                 u_een_grad_a_bee_der2_bywlk (elec_i, elec_j, atom_i, dim_i, 1, conf_i, walk_i) = 0.d0
                 
                 u_een_grad_a_bee_der2_bywlk (elec_i, elec_j, atom_i, dim_i, 2, conf_i, walk_i) = &
                      term1 + term2 + term3
                 
              end do ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_a_bee_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_grad_a_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_grad_a_ben_der2_bywlk_bld
!
! Creates        : u_een_grad_a_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk                    (elec, elec, pow, conf, walk      )
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow,  conf, walk)
! Needs          : u_ee_grad_a_der_bywlk               (elec, elec, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_grad_a_ben_der2_bywlk (elec, elec, atom, dim, 1, conf, walk) = 
! Descriptions   : u_en_grad_b_der_ia * u_en_ja + u_en_grad_ia * u_en_b_der_ja
!
! Descriptions   : u_een_grad_a_ben_der2_bywlk (elec, elec, atom, dim, 2, conf, walk) = 
! Descriptions   : u_ee_ij      * u_en_grad_b_der_ia +
! Descriptions   : u_ee_grad_ij * u_en_b_der_ia      + 
! Descriptions   : u_ee_grad_ij * u_en_b_der_ja
!

!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i  

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: dimensions_nb
  integer(dpi)                  ::  aeen_jastrow_nb 

  real(dp)                      :: u_en_grad_b_der_ia 
  real(dp)                      :: u_en_b_der_ja      
  real(dp)                      :: u_en_b_der_ia      
  real(dp)                      :: u_en_ja              
  real(dp)                      :: u_ee_ij              
  real(dp)                      :: u_en_grad_ia         
  real(dp)                      :: u_ee_grad_ij         

  real(dp)                      :: term1
  real(dp)                      :: term2
  real(dp)                      :: term3
  real(dp)                      :: term4
  real(dp)                      :: term5  

! header:

  if (header_exe) then

    l_here = "u_een_grad_a_ben_der2_bywlk_bld"

    call object_create ('u_een_grad_a_ben_der2_bywlk')

    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ea_r_bar_grad_bywlk, 1)
  atoms_nb          = size (ea_r_bar_grad_bywlk, 2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,           2)
  dimensions_nb     = size (ea_r_bar_grad_bywlk, 3)
  configurations_nb = size (ea_r_bar_grad_bywlk, 4)
  walkers_nb        = size (ea_r_bar_grad_bywlk, 5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_grad_a_ben_der2_bywlk', u_een_grad_a_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, dimensions_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_grad_a_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb
                 
                 u_en_grad_b_der_ia = ea_r_bar_grad_beta_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_en_b_der_ja      = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 u_en_b_der_ia      = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

                 u_en_ja              = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

                 u_ee_ij              = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

                 u_en_grad_ia         = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 u_ee_grad_ij         = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 term1 = u_en_grad_b_der_ia * u_en_ja
                 
                 term2 = u_en_grad_ia * u_en_b_der_ja

                 term3 = u_ee_grad_ij * u_en_b_der_ia

                 term4 = u_ee_grad_ij * u_en_b_der_ja

                 term5 = u_ee_ij * u_en_grad_b_der_ia

                 u_een_grad_a_ben_der2_bywlk (elec_i, elec_j, atom_i, dim_i, 1, conf_i, walk_i) = &
                      term1 + term2

                 u_een_grad_a_ben_der2_bywlk (elec_i, elec_j, atom_i, dim_i, 2, conf_i, walk_i) = &
                      term3 + term4 + term5
                 
              end do ! do dim_i
                   
           end do ! do elec_i

        end do ! do elec_j
        
     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_grad_a_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_bywlk_bld
!
! Creates        : u_een_lapl_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ea_r_bar_bywlk (elec, atom, conf, walk)     
! Needs          : ee_r_bar_bywlk (elec, elec, conf, walk)
! Needs          : conf_jastr_a_een (conf, lik, atom)      
!
! Descriptions   : laplacian over elec_i coordinates
! Descriptions   : lapl u_een_grad_bywlk 
! Descriptions   :  = 2 * (a_een1 * rja^2 + a_een2 *rij^2 ) ria * lapl_i ria
! Descriptions   :  + 2 * a_een2 * (ria^2 + rja^2) * rij * lapl_i rij
! Descriptions   :  + 2 * (a_een1 * rja^2 + a_een2 *rij^2 ) grad_i ria . grad_i ria
! Descriptions   :  + 2 * a_een2 * (ria^2 + rja^2) * grad_i rij . grad_i rij
! Descriptions   :  + 8 * a_een2 *rij *ria * grad_i ria . grad_i rij
!
! Author         : P. Reinhardt
! Date           : 23/01/2001
! Revision       : X. Krokidis and R. Assaraf 27 June 2001
! Revision       : F. Colonna: 02 Mar 2002 by walkers
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb

  real(dp)                      :: r_ia_bar
  real(dp)                      :: r_ia_bar_2
  real(dp)                      :: r_ja_bar
  real(dp)                      :: r_ja_bar_2
  real(dp)                      :: r_ij_bar
  real(dp)                      :: r_ij_bar_2
  real(dp)                      :: r_ia_bar_lapl
  real(dp)                      :: r_ia_bar_grad(3)
  real(dp)                      :: r_ij_bar_lapl
  real(dp)                      :: r_ij_bar_grad(3)
  real(dp)                      :: a_een_1   
  real(dp)                      :: a_een_2 
  real(dp)                      :: dot_prod_ia
  real(dp)                      :: dot_prod_ij
  real(dp)                      :: dot_prod_ijia
  real(dp)                      :: lapl

! header:

  if (header_exe) then

    l_here = "u_een_lapl_bywlk_bld"

    call object_create ('u_een_lapl_bywlk')

    call object_needed ('ee_r_bar_bywlk')
    call object_needed ('ee_r_bar_grad_bywlk')
    call object_needed ('ee_r_bar_lapl_bywlk')
    call object_needed ('ea_r_bar_bywlk')
    call object_needed ('ea_r_bar_grad_bywlk')
    call object_needed ('ea_r_bar_lapl_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! requirements:

!  include 'chksiz_configurations%conf_jastr%a_een.inc'
  include 'chksiz_ea_r_bar_bywlk.inc'
  include 'chksiz_ee_r_bar_bywlk.inc'

! end requirements

! dimensions:

  electrons_nb      = size(ea_r_bar_grad_bywlk, 1)
  atoms_nb          = size(ea_r_bar_grad_bywlk, 2)
  configurations_nb = size(ea_r_bar_grad_bywlk, 4)
  walkers_nb        = size(ea_r_bar_grad_bywlk, 5)

  if(debug) then
     write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
     write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb 
     write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
     write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  end if 

! end dimensions

! allocation 
 
  call alloc ('u_een_lapl_bywlk', u_een_lapl_bywlk, electrons_nb, &
       electrons_nb, atoms_nb, configurations_nb, walkers_nb)
 
  u_een_lapl_bywlk = zero

  do walk_i = 1, walkers_nb

     do conf_i = 1, configurations_nb

        do atom_i = 1, atoms_nb

           a_een_1  = conf_jastr_a_een (conf_i,1, atom_i)
           a_een_2  = conf_jastr_a_een (conf_i,2, atom_i)

           do elec_j = 1, electrons_nb

              r_ja_bar   = ea_r_bar_bywlk (elec_j, atom_i, conf_i, walk_i) 

              r_ja_bar_2 = r_ja_bar * r_ja_bar

              ! Improve: verify formula
              ! Improve store squares in temporary arrays

              do elec_i = 1, electrons_nb

                 r_ia_bar   = ea_r_bar_bywlk (elec_i, atom_i, conf_i, walk_i)

                 r_ia_bar_2 = r_ia_bar * r_ia_bar

                 r_ia_bar_grad &
                      = ea_r_bar_grad_bywlk (elec_i, atom_i, :, conf_i, walk_i)
                 
                 r_ia_bar_lapl &
                      = ea_r_bar_lapl_bywlk (elec_i, atom_i, conf_i, walk_i)

                 r_ij_bar   = ee_r_bar_bywlk (elec_i, elec_j, conf_i, walk_i)

                 r_ij_bar_2 = r_ij_bar * r_ij_bar

                 r_ij_bar_grad &
                      = ee_r_bar_grad_bywlk (elec_i, elec_j, :, conf_i, walk_i)

                 r_ij_bar_lapl &
                      = ee_r_bar_lapl_bywlk (elec_i, elec_j, conf_i, walk_i)
   
                 dot_prod_ia   = dot_product (r_ia_bar_grad, r_ia_bar_grad)
                 dot_prod_ij   = dot_product (r_ij_bar_grad, r_ij_bar_grad)
                 dot_prod_ijia = dot_product (r_ij_bar_grad, r_ia_bar_grad)

                 lapl       = two * ( a_een_1 * r_ja_bar_2 + a_een_2 * r_ij_bar_2 ) &
                      *  r_ia_bar * r_ia_bar_lapl                                   &

                      + two * a_een_2 * (r_ia_bar_2 + r_ja_bar_2)                   &
                      *  r_ij_bar * r_ij_bar_lapl                                   &
                      
                      + two * ( a_een_1 * r_ja_bar_2 + a_een_2 * r_ij_bar_2 )       &
                      *  dot_prod_ia                                                &
                      
                      + two * a_een_2 * (r_ia_bar_2 + r_ja_bar_2)                   &
                      *  dot_prod_ij                                                &
                      
                      + eight * a_een_2 * r_ij_bar * r_ia_bar * dot_prod_ijia 
                 
                 u_een_lapl_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = lapl

              end do ! do elec_i

           end do ! do elec_j

        end do ! do atom_i

     end do ! do conf_i

  end do ! do walk_i

  include 'debug_u_een_lapl_bywlk.inc'
!   include 'ensnb_u_een_lapl_bywlk.inc'

  call exit (l_here)

  end subroutine u_een_lapl_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_a_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_a_der_bywlk_bld
!
! Creates        : u_een_lapl_a_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : u_ee_grad_a_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk (elec, atom, dim, pow,  conf, walk)
! Needs          : u_en_a_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_en_lapl_a_der_bywlk (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_ee_lapl_a_der_bywlk (elec, elec, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk      (elec, elec, pow, conf, walk      )
!
! Descriptions   : u_een_lapl_a_deriv = 
! Descriptions   : 2 * u_ee_grad_ij * u_en_grad_ia +
! Descriptions   :     u_ee_ij      * u_en_lapl_ia +
! Descriptions   :     u_en_lapl_ia * u_en_ja      + 
! Descriptions   :     u_ee_lapl_ij * u_en_ia      +
! Descriptions   :     u_ee_lapl_ij * u_en_ja     

!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: u_en_ja      
  real(dp)                      :: u_en_lapl_ia 
  real(dp)                      :: u_en_ia      
  real(dp)                      :: u_ee_lapl_ij 
  real(dp)                      :: u_ee_grad_ij 
  real(dp)                      :: u_en_grad_ia 
  real(dp)                      :: u_ee_ij 
     
  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3  
  real(dp)                      :: term4(3) 
  real(dp)                      :: term5
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_a_der_bywlk_bld"

    call object_create ('u_een_lapl_a_der_bywlk')

    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('u_ee_lapl_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_en_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_a_der_bywlk', u_een_lapl_a_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_a_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              do dim_i = 1, dimensions_nb

                 u_ee_grad_ij = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)
                 
                 u_en_grad_ia = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term4(dim_i) = 2.d0 * u_ee_grad_ij * u_en_grad_ia

              end do ! do dim_i

              u_en_ja      = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_lapl_ia = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ia      = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_ee_lapl_ij = u_ee_lapl_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_ee_ij      = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              term1 = u_en_lapl_ia * u_en_ja

              term2 = u_ee_lapl_ij * u_en_ia

              term3 = u_ee_lapl_ij * u_en_ja

              term5 = u_ee_ij*u_en_lapl_ia

              u_een_lapl_a_der_bywlk (elec_i, elec_j, atom_i, 1, conf_i, walk_i) = 0 !term1

              u_een_lapl_a_der_bywlk (elec_i, elec_j, atom_i, 2, conf_i, walk_i) = &
                   term2 + term3 + term4(1) + term4(2) + term4(3) + term5

           end do ! do elec_i

        end do ! do elec_j
        
     end do ! do atom_i
     
  end do ! do conf_i
  
end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_a_der_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_ben_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_ben_der_bywlk_bld
!
! Creates        : u_een_lapl_ben_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                      (conf, 1,    atom                 )
! Needs          : ea_r_bar_lapl_beta_der_bywlk (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk                    (elec, elec, pow, conf, walk      )
! Needs          : u_en_lapl_a_der_bywlk               (elec, atom, pow, conf, walk      )
! Needs          : u_ee_lapl_a_der_bywlk               (elec, elec, pow, conf, walk      )
! Needs          : u_ee_grad_a_der_bywlk               (elec, elec, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_lapl_ben_deriv = 
! Descriptions   : 2 * a_een_2 * u_ee_grad_ij         * u_en_grad_b_der_ia +
! Descriptions   :     a_een_2 * u_ee_ij              * u_en_lapl_b_der_ia +
! Descriptions   :     a_een_1 * u_en_lapl_ia         * u_en_b_der_ja      +
! Descriptions   :     a_een_2 * u_ee_lapl_ij         * u_en_b_der_ia      +
! Descriptions   :     a_een_2 * u_ee_lapl_ij         * u_en_b_der_ja      +
! Descriptions   :     a_een_1 * u_en_lapl_b_der_ia * u_en_ja     

!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: u_en_ja
  real(dp)                      :: u_ee_ij
  real(dp)                      :: u_en_b_der_ja
  real(dp)                      :: u_en_b_der_ia
  real(dp)                      :: u_en_lapl_ia
  real(dp)                      :: u_ee_lapl_ij
  real(dp)                      :: u_ee_grad_ij
  real(dp)                      :: u_en_lapl_b_der_ia
  real(dp)                      :: u_en_grad_b_der_ia

  real(dp)                      :: a_een_1 
  real(dp)                      :: a_een_2 

  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3  
  real(dp)                      :: term4 
  real(dp)                      :: term5(3)
  real(dp)                      :: term6
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_ben_der_bywlk_bld"

    call object_create ('u_een_lapl_ben_der_bywlk')

    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_ee_lapl_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_ee_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_ben_der_bywlk', u_een_lapl_ben_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_ben_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1              = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2              = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_lapl_b_der_ia = ea_r_bar_lapl_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der_ja      = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_b_der_ia      = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja              = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_ee_ij              = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_ia         = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_ee_lapl_ij         = u_ee_lapl_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              do dim_i = 1, dimensions_nb
                 
                 u_ee_grad_ij         = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_en_grad_b_der_ia = ea_r_bar_grad_beta_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term5(dim_i)         = 2.d0*a_een_2*u_ee_grad_ij*u_en_grad_b_der_ia

              end do ! do dim_i

              term1 = a_een_1 * u_en_lapl_b_der_ia * u_en_ja
              
              term2 = a_een_1 * u_en_lapl_ia * u_en_b_der_ja

              term3 = a_een_2 * u_ee_lapl_ij * u_en_b_der_ia

              term4 = a_een_2 * u_ee_lapl_ij * u_en_b_der_ja

              term6 = a_een_2 * u_ee_ij*u_en_lapl_b_der_ia

              u_een_lapl_ben_der_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
                   term1 + term2 + term3 + term4 + term5(1) + term5(2) + term5(3) + term6

           end do ! do elec_i

        end do ! do elec_j

      end do ! do atom_i

    end do ! do conf_i

  end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_ben_der_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_ben_der2_bywlk_bld
!
! Creates        : u_een_lapl_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                       (conf, 1,    atom                 )
! Needs          : ea_r_bar_lapl_beta_der2_bywlk (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_lapl_beta_der_bywlk  (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der2_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der_bywlk       (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                     (elec, atom, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk                     (elec, elec, pow, conf, walk      )
! Needs          : u_en_lapl_a_der_bywlk                (elec, atom, pow, conf, walk      )
! Needs          : u_ee_lapl_a_der_bywlk                (elec, elec, pow, conf, walk      )
! Needs          : u_ee_grad_a_der_bywlk                (elec, elec, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der2_bywlk (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_lapl_ben_deriv2 = 
! Descriptions   : 2 * a_een_2 * u_ee_grad_ij          * u_en_grad_b_der2_ia    +
! Descriptions   :     a_een_2 * u_ee_ij               * u_en_lapl_b_der2_ia    +
! Descriptions   :     a_een_1 * u_en_lapl_ia          * u_en_b_der2_ja         +
! Descriptions   :     a_een_2 * u_ee_lapl_ij          * u_en_b_der2_ia         +
! Descriptions   :     a_een_2 * u_ee_lapl_ij          * u_en_b_der2_ja         +
! Descriptions   : 2 * a_een_1 * u_en_lapl_b_der_ia  * u_en_b_der_ja          +
! Descriptions   :     a_een_1 * u_en_lapl_b_der2_ia * u_en_ja                  
! Descriptions   : pow = 2
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1 
  real(dp)                      :: a_een_2 

  real(dp)                      :: u_en_lapl_b_der2_ia 
  real(dp)                      :: u_en_lapl_b_der_ia  
  real(dp)                      :: u_en_b_der2_ja      
  real(dp)                      :: u_en_b_der2_ia      
  real(dp)                      :: u_en_b_der_ja       
  real(dp)                      :: u_en_ja               
  real(dp)                      :: u_ee_ij               
  real(dp)                      :: u_en_lapl_ia          
  real(dp)                      :: u_ee_lapl_ij          
  real(dp)                      :: u_ee_grad_ij          
  real(dp)                      :: u_en_grad_b_der2_ia 

  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3  
  real(dp)                      :: term4 
  real(dp)                      :: term5
  real(dp)                      :: term6(3)
  real(dp)                      :: term7
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_ben_der2_bywlk_bld"

    call object_create ('u_een_lapl_ben_der2_bywlk')

    call object_needed ('ea_r_bar_lapl_beta_der2_bywlk')
    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der2_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('ea_r_bar_grad_beta_der2_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_ee_lapl_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('u_en_lapl_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_en_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_ben_der2_bywlk', u_een_lapl_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

              u_en_lapl_b_der2_ia = ea_r_bar_lapl_beta_der2_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_lapl_b_der_ia  = ea_r_bar_lapl_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der2_ja      = ea_r_bar_beta_der2_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_b_der2_ia      = ea_r_bar_beta_der2_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der_ja       = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_ja               = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_ee_ij               = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_ia          = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_ee_lapl_ij          = u_ee_lapl_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              term1 = a_een_1*u_en_lapl_b_der2_ia*u_en_ja

              term2 = 2.d0*a_een_1*u_en_lapl_b_der_ia*u_en_b_der_ja

              term3 = a_een_1*u_en_lapl_ia*u_en_b_der2_ja

              term4 = u_ee_lapl_ij * u_en_b_der2_ia * a_een_2

              term5 = a_een_2*u_ee_lapl_ij*u_en_b_der2_ja

              term7 = a_een_2*u_ee_ij * u_en_lapl_b_der2_ia

              do dim_i = 1, dimensions_nb
                 
                 u_ee_grad_ij          = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i, 2,conf_i,walk_i)

                 u_en_grad_b_der2_ia = ea_r_bar_grad_beta_der2_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term6(dim_i) =  u_ee_grad_ij * 2.d0 * a_een_2 * u_en_grad_b_der2_ia

              end do ! do dim_i

              u_een_lapl_ben_der2_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
                   term1 + term2 + term3 + term4 + term5 + term6(1) + term6(2) + term6(3) + term7

           end do ! do elec_i
           
        end do ! do elec_j
        
     end do ! do atom_i
     
  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_bee_der_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_bee_der_bywlk_bld
!
! Creates        : u_een_lapl_bee_der_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                      (conf, 1,    atom                 ) 
! Needs          : ee_r_bar_lapl_beta_der_bywlk (elec, elec, pow, conf, walk      )
! Needs          : ee_r_bar_beta_der_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : u_en_lapl_a_der_bywlk               (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk      )
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow,  conf, walk)
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_lapl_bee_deriv = 
! Descriptions   : 2_een_2 * u_ee_grad_b_der_ij * u_en_grad_ia * 2 +
! Descriptions   : 2_een_2 * u_ee_b_der_ij      * u_en_lapl_ia     +
! Descriptions   : 2_een_2 * u_ee_lapl_b_der_ij * u_en_ia          +
! Descriptions   : 2_een_2 * u_ee_lapl_b_der_ij * u_en_ja
! Descriptions   : pow = 2
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1 
  real(dp)                      :: a_een_2 

  real(dp)                      :: u_ee_lapl_b_der_ij  
  real(dp)                      :: u_ee_b_der_ij       
  real(dp)                      :: u_en_lapl_ia          
  real(dp)                      :: u_en_ia               
  real(dp)                      :: u_en_ja               
  real(dp)                      :: u_ee_grad_b_der_ij 
  real(dp)                      :: u_en_grad_ia         
  
  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3(3)  
  real(dp)                      :: term4 
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_bee_der_bywlk_bld"

    call object_create ('u_een_lapl_bee_der_bywlk')

    call object_needed ('ee_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_en_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_bee_der_bywlk', u_een_lapl_bee_der_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_bee_der_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

              u_ee_lapl_b_der_ij  = ee_r_bar_lapl_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_ee_b_der_ij       = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_ia          = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ia               = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja               = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              term1 = a_een_2*u_ee_lapl_b_der_ij*u_en_ia
              
              term2 = a_een_2*u_ee_lapl_b_der_ij*u_en_ja

              term4 = a_een_2*u_ee_b_der_ij*u_en_lapl_ia

              do dim_i = 1, dimensions_nb

                 u_ee_grad_b_der_ij = ee_r_bar_grad_beta_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_en_grad_ia         = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term3(dim_i)         = 2.d0*a_een_2*u_ee_grad_b_der_ij*u_en_grad_ia

              end do ! do dim_i

              u_een_lapl_bee_der_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
                   term1 + term2 + term3(1) + term3(2) + term3(3) + term4

           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_bee_der_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_bee_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_bee_der2_bywlk_bld
!
! Creates        : u_een_lapl_bee_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : conf_jastr_a_een                       (conf, 1,    atom                 )
! Needs          : ee_r_bar_lapl_beta_der2_bywlk (elec, elec, pow, conf, walk      )
! Needs          : ee_r_bar_beta_der2_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : u_en_lapl_a_der_bywlk                (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                     (elec, atom, pow, conf, walk      )
! Needs          : ee_r_bar_grad_beta_der2_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : u_en_grad_a_der_bywlk                (elec, atom, dim, pow,  conf, walk)
!
! Descriptions   : u_een_lapl_bee_deriv2 = 
! Descriptions   : a_een_2 * u_ee_b_der2_ij      * u_en_lapl_ia * 2 +
! Descriptions   : a_een_2 * u_ee_grad_b_der2_ij * u_en_grad_ia     +
! Descriptions   : a_een_2 * u_ee_lapl_b_der2_ij * u_en_ia          +
! Descriptions   : a_een_2 * u_ee_lapl_b_der2_ij * u_en_ja
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1 
  real(dp)                      :: a_een_2 

  real(dp)                      :: u_ee_lapl_b_der2_ij 
  real(dp)                      :: u_ee_b_der2_ij 
  real(dp)                      :: u_en_lapl_ia 
  real(dp)                      :: u_en_ia 
  real(dp)                      :: u_en_ja 
  real(dp)                      :: u_ee_grad_b_der2_ij 
  real(dp)                      :: u_en_grad_ia 
  
  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3(3)  
  real(dp)                      :: term4 
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_bee_der2_bywlk_bld"

    call object_create ('u_een_lapl_bee_der2_bywlk')

    call object_needed ('ee_r_bar_lapl_beta_der2_bywlk')
    call object_needed ('ee_r_bar_beta_der2_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('ee_r_bar_grad_beta_der2_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_en_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_bee_der2_bywlk', u_een_lapl_bee_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_bee_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

              u_ee_lapl_b_der2_ij = ee_r_bar_lapl_beta_der2_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_ee_b_der2_ij = ee_r_bar_beta_der2_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_ia = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ia = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              term1 = a_een_2*u_ee_lapl_b_der2_ij*u_en_ia

              term2 = a_een_2*u_ee_lapl_b_der2_ij*u_en_ja

              term4 = a_een_2*u_ee_b_der2_ij*u_en_lapl_ia

              do dim_i = 1, dimensions_nb

                 u_ee_grad_b_der2_ij = ee_r_bar_grad_beta_der2_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)
                 
                 u_en_grad_ia = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term3(dim_i) = 2.d0 * a_een_2 * u_ee_grad_b_der2_ij * u_en_grad_ia

              end do ! do dim_i

              u_een_lapl_bee_der2_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
                   term1 + term2 + term3(1) + term3(2) + term3(3) + term4 

           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_bee_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_a_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_a_ben_der2_bywlk_bld
!
! Creates        : u_een_lapl_a_ben_der2_bywlk (elec, elec, atom, pow, conf, walk)
!
! Needs          : ea_r_bar_lapl_beta_der_bywlk (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk      )
! Needs          : u_ee_a_der_bywlk                    (elec, elec, pow, conf, walk      )
! Needs          : u_en_lapl_a_der_bywlk               (elec, atom, pow, conf, walk      )
! Needs          : u_ee_lapl_a_der_bywlk               (elec, elec, pow, conf, walk      )
! Needs          : u_ee_grad_a_der_bywlk               (elec, elec, dim, pow , conf, walk)
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow , conf, walk)
!
! Descriptions   : u_een_lapl_a_ben_deriv2 (elec, elec, atom, 1, conf, walk) = 
! Descriptions   : u_en_lapl_b_der_ia * u_en_ja + term2 = u_en_lapl_ia * u_en_b_der_ja
!
! Descriptions   : u_een_lapl_a_ben_der2_bywlk (elec, elec, atom, 2, conf, walk) =
! Descriptions   : u_ee_grad_ij * u_en_grad_b_der_ia * 2 +
! Descriptions   : u_ee_ij      * u_en_lapl_b_der_ia     +
! Descriptions   : u_ee_lapl_ij * u_en_b_der_ia          +
! Descriptions   : u_ee_lapl_ij * u_en_b_der_ja          

! Descriptions   : pow = 1,2
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: u_en_ja
  real(dp)                      :: u_ee_ij
  real(dp)                      :: u_en_b_der_ja
  real(dp)                      :: u_en_b_der_ia
  real(dp)                      :: u_en_lapl_ia
  real(dp)                      :: u_ee_lapl_ij
  real(dp)                      :: u_ee_grad_ij
  real(dp)                      :: u_en_lapl_b_der_ia
  real(dp)                      :: u_en_grad_b_der_ia

  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3  
  real(dp)                      :: term4 
  real(dp)                      :: term5(3)
  real(dp)                      :: term6
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_a_ben_der2_bywlk_bld"

    call object_create ('u_een_lapl_a_ben_der2_bywlk')

    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_ee_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_ee_lapl_a_der_bywlk')
    call object_needed ('u_ee_grad_a_der_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_ee_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_a_ben_der2_bywlk', u_een_lapl_a_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb,   aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_a_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              u_en_lapl_b_der_ia = ea_r_bar_lapl_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der_ja      = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_en_b_der_ia      = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja              = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              u_ee_ij              = u_ee_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_ia         = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_ee_lapl_ij         = u_ee_lapl_a_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              do dim_i = 1, dimensions_nb
                 
                 u_ee_grad_ij         = u_ee_grad_a_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_en_grad_b_der_ia = ea_r_bar_grad_beta_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term5(dim_i)         = 2.d0*u_ee_grad_ij*u_en_grad_b_der_ia

              end do ! do dim_i

              term1 = u_en_lapl_b_der_ia * u_en_ja
              
              term2 = u_en_lapl_ia * u_en_b_der_ja

              term3 = u_ee_lapl_ij * u_en_b_der_ia

              term4 = u_ee_lapl_ij * u_en_b_der_ja

              term6 = u_ee_ij*u_en_lapl_b_der_ia

              u_een_lapl_a_ben_der2_bywlk (elec_i, elec_j, atom_i, 1, conf_i, walk_i) = &
                   term1 + term2
              
              u_een_lapl_a_ben_der2_bywlk (elec_i, elec_j, atom_i, 2, conf_i, walk_i) = &
                   term3 + term4 + term5(1) + term5(2) + term5(3) + term6

           end do ! do elec_i
           
        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_a_ben_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_a_bee_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_a_bee_der2_bywlk_bld
!
! Creates        : u_een_lapl_a_bee_der2_bywlk (elec, elec, atom, num, conf, walk)
!
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow , conf, walk )
! Needs          : u_en_grad_a_der_bywlk               (elec, atom, dim, pow , conf, walk )
! Needs          : ee_r_bar_lapl_beta_der_bywlk (elec, elec, pow, conf, walk       )
! Needs          : ee_r_bar_beta_der_bywlk      (elec, elec, pow, conf, walk       )
! Needs          : u_en_lapl_a_der_bywlk               (elec, atom, pow, conf, walk       )
! Needs          : u_en_a_der_bywlk                    (elec, atom, pow, conf, walk       )
! Needs          : conf_jastr_a_een                      (conf, pow , atom                  )
!
! Descriptions   : u_een_lapl_a_bee_deriv2 = 
! Descriptions   : u_ee_b_der_ij      * u_en_lapl_ia + 
! Descriptions   : u_ee_grad_b_der_ij * u_en_grad_ia +
! Descriptions   : u_ee_lapl_b_der_ij * u_en_ia      + 
! Descriptions   : u_ee_lapl_b_der_ij * u_en_ja

! Descriptions   : pow = 2
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: u_ee_lapl_b_der_ij  
  real(dp)                      :: u_ee_b_der_ij       
  real(dp)                      :: u_en_lapl_ia          
  real(dp)                      :: u_en_ia               
  real(dp)                      :: u_en_ja               
  real(dp)                      :: u_ee_grad_b_der_ij 
  real(dp)                      :: u_en_grad_ia         
  
  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3(3)  
  real(dp)                      :: term4 
  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_a_bee_der2_bywlk_bld"

    call object_create ('u_een_lapl_a_bee_der2_bywlk')

    call object_needed ('ee_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('u_en_lapl_a_der_bywlk')
    call object_needed ('u_en_a_der_bywlk')
    call object_needed ('u_en_grad_a_der_bywlk')
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (u_en_a_der_bywlk,     1)
  atoms_nb          = size (u_en_a_der_bywlk,     2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,       2)
  dimensions_nb     = size (u_en_grad_a_der_bywlk,3)
  configurations_nb = size (u_en_a_der_bywlk,     4)
  walkers_nb        = size (u_en_a_der_bywlk,     5)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_a_bee_der2_bywlk', u_een_lapl_a_bee_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, aeen_jastrow_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_a_bee_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              u_ee_lapl_b_der_ij  = ee_r_bar_lapl_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_ee_b_der_ij       = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_ia          = u_en_lapl_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ia               = u_en_a_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_ja               = u_en_a_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              term1 = u_ee_lapl_b_der_ij*u_en_ia
              
              term2 = u_ee_lapl_b_der_ij*u_en_ja

              term4 = u_ee_b_der_ij*u_en_lapl_ia

              do dim_i = 1, dimensions_nb

                 u_ee_grad_b_der_ij = ee_r_bar_grad_beta_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_en_grad_ia         = u_en_grad_a_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term3(dim_i)         = 2.d0*u_ee_grad_b_der_ij*u_en_grad_ia

              end do ! do dim_i

              u_een_lapl_a_bee_der2_bywlk (elec_i, elec_j, atom_i, 1, conf_i, walk_i) = 0.d0

              u_een_lapl_a_bee_der2_bywlk (elec_i, elec_j, atom_i, 2, conf_i, walk_i) = &
                term1+ term2 + term3(1) + term3(2) + term3(3) + term4

           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i
     
  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_a_bee_der2_bywlk_bld

! ===================================================================================
  subroutine u_een_lapl_bee_ben_der2_bywlk_bld 
! -----------------------------------------------------------------------------------
!
! Name           : u_een_lapl_bee_ben_der2_bywlk_bld
!
! Creates        : u_een_lapl_bee_ben_der2_bywlk (elec, elec, atom, conf, walk)
!
! Needs          : ee_r_bar_grad_beta_der_bywlk (elec, elec, dim, pow,  conf, walk)
! Needs          : ea_r_bar_grad_beta_der_bywlk (elec, atom, dim, pow,  conf, walk)
! Needs          : ee_r_bar_lapl_beta_der_bywlk (elec, elec, pow, conf, walk      )
! Needs          : ea_r_bar_lapl_beta_der_bywlk (elec, atom, pow, conf, walk      )
! Needs          : ee_r_bar_beta_der_bywlk      (elec, elec, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : ea_r_bar_beta_der_bywlk      (elec, atom, pow, conf, walk      )
! Needs          : conf_jastr_a_een                      (conf, pow,  atom                 )
!
! Descriptions   : b_ee and b_en second derivative of the laplacian over elec_i coordinates
! Descriptions   : a_een_2 * u_een_lapl_bee_ben_deriv2 = 
! Descriptions   : a_een_2 * u_ee_grad_b_der_ij . u_en_grad_b_der_ia * 2 +
! Descriptions   : a_een_2 * u_ee_lapl_b_der_ij * u_en_b_der_ia          + 
! Descriptions   : a_een_2 * u_ee_lapl_b_der_ij * u_en_b_der_ja          +
! Descriptions   : a_een_2 * u_en_lapl_b_der_ia * u_ee_b_der_ij 
! Descriptions   : pow = 2
!
! Author         : A.C. Kollias
! Date           : 08/09/2005
! -----------------------------------------------------------------------------------
  implicit none

! i/o:

! local:
  character(len=max_string_len_routine_name),save :: l_here

  integer(dpi)                  :: elec_i
  integer(dpi)                  :: elec_j
  integer(dpi)                  :: walk_i
  integer(dpi)                  :: conf_i
  integer(dpi)                  :: atom_i
  integer(dpi)                  :: dim_i

  integer(dpi)                  :: atoms_nb
  integer(dpi)                  :: configurations_nb
  integer(dpi)                  :: electrons_nb
  integer(dpi)                  :: walkers_nb
  integer(dpi)                  :: aeen_jastrow_nb
  integer(dpi)                  :: dimensions_nb

  real(dp)                      :: a_een_1 
  real(dp)                      :: a_een_2 

  real(dp)                      :: u_ee_lapl_b_der_ij 
  real(dp)                      :: u_en_lapl_b_der_ia 
  real(dp)                      :: u_ee_b_der_ij 
  real(dp)                      :: u_en_b_der_ia 
  real(dp)                      :: u_en_b_der_ja 
  real(dp)                      :: u_ee_grad_b_der_ij 
  real(dp)                      :: u_en_grad_b_der_ia 

  real(dp)                      :: term1        
  real(dp)                      :: term2  
  real(dp)                      :: term3(3)  
  real(dp)                      :: term4 

  
! header:

  if (header_exe) then

    l_here = "u_een_lapl_bee_ben_der2_bywlk_bld"

    call object_create ('u_een_lapl_bee_ben_der2_bywlk')

    call object_needed ('ee_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ea_r_bar_lapl_beta_der_bywlk')
    call object_needed ('ee_r_bar_grad_beta_der_bywlk')
    call object_needed ('ea_r_bar_grad_beta_der_bywlk')
    call object_needed ('ee_r_bar_beta_der_bywlk')
    call object_needed ('ea_r_bar_beta_der_bywlk') 
    call object_needed ('u_ee_lapl_bywlk')
    call object_needed ('conf_jastr_a_een')

    return

  end if

! begin:
  call enter (l_here)

! dimensions:
  electrons_nb      = size (ee_r_bar_grad_beta_der_bywlk, 1)
  atoms_nb          = size (ee_r_bar_grad_beta_der_bywlk, 2)
  aeen_jastrow_nb   = size (conf_jastr_a_een,                      2)
  dimensions_nb     = size (ee_r_bar_grad_beta_der_bywlk, 3)
  configurations_nb = size (ee_r_bar_grad_beta_der_bywlk, 5)
  walkers_nb        = size (ee_r_bar_grad_beta_der_bywlk, 6)
! end dimensions

  if(debug) then
   write (logf,*)trim(l_here),'-d: atoms_nb          = ',atoms_nb
   write (logf,*)trim(l_here),'-d: electrons_nb      = ',electrons_nb
   write (logf,*)trim(l_here),'-d: configurations_nb = ',configurations_nb
   write (logf,*)trim(l_here),'-d: walkers_nb        = ',walkers_nb
  
   do conf_i = 1, configurations_nb
     do atom_i = 1, atoms_nb
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 1, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',1,',atom_i,') =', &
                conf_jastr_a_een (conf_i,1, atom_i)

        write (logf,*)trim(l_here),'-d: conf_jastr_a_een (conf, 2, atom)'
        write (logf,*)trim(l_here),'-d: conf_jastr_a_een(',conf_i,',2,',atom_i,') =', &
                conf_jastr_a_een (conf_i,2, atom_i)
     end do 
   end do 

  end if 

! allocations:
 
    call alloc ('u_een_lapl_bee_ben_der2_bywlk', u_een_lapl_bee_ben_der2_bywlk, electrons_nb, &
         electrons_nb, atoms_nb, configurations_nb, walkers_nb)

! end allocations

  u_een_lapl_bee_ben_der2_bywlk = zero

  do walk_i = 1, walkers_nb

    do conf_i = 1, configurations_nb

      do atom_i = 1, atoms_nb

        do elec_j = 1, electrons_nb

           do elec_i = 1, electrons_nb

              a_een_1               = conf_jastr_a_een (conf_i, 1, atom_i)
              
              a_een_2               = conf_jastr_a_een (conf_i, 2, atom_i)

              u_ee_lapl_b_der_ij = ee_r_bar_lapl_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_lapl_b_der_ia = ea_r_bar_lapl_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_ee_b_der_ij = ee_r_bar_beta_der_bywlk(elec_i,elec_j,2,conf_i,walk_i)

              u_en_b_der_ia = ea_r_bar_beta_der_bywlk(elec_i,atom_i,2,conf_i,walk_i)

              u_en_b_der_ja = ea_r_bar_beta_der_bywlk(elec_j,atom_i,2,conf_i,walk_i)

              term1 = a_een_2*u_ee_lapl_b_der_ij*u_en_b_der_ia

              term2 = a_een_2*u_ee_lapl_b_der_ij*u_en_b_der_ja

              term4 = a_een_2*u_ee_b_der_ij*u_en_lapl_b_der_ia

              do dim_i = 1, dimensions_nb

                 u_ee_grad_b_der_ij = ee_r_bar_grad_beta_der_bywlk(elec_i,elec_j,dim_i,2,conf_i,walk_i)

                 u_en_grad_b_der_ia = ea_r_bar_grad_beta_der_bywlk(elec_i,atom_i,dim_i,2,conf_i,walk_i)

                 term3(dim_i) = 2.d0 * a_een_2 * u_ee_grad_b_der_ij*u_en_grad_b_der_ia

              end do ! do dim_i

              u_een_lapl_bee_ben_der2_bywlk (elec_i, elec_j, atom_i, conf_i, walk_i) = &
                  term1 + term2 + term3(1) + term3(2) + term3(3) + term4

           end do ! do elec_i

        end do ! do elec_j

     end do ! do atom_i

  end do ! do conf_i

end do ! do walk_i

  call exit (l_here)

  end subroutine u_een_lapl_bee_ben_der2_bywlk_bld

end module jastrow_u_een_bywlk_mod
